#[allow(clippy::unreadable_literal)]
static SERVICES: phf::Map<ServiceQuery, Service> = ::phf::Map {
    key: 16287231350648472473,
    disps: &[
        (0, 31),
        (0, 2),
        (0, 8),
        (0, 6),
        (0, 0),
        (0, 0),
        (0, 8),
        (0, 60),
        (0, 0),
        (0, 4),
        (0, 6),
        (0, 0),
        (0, 132),
        (0, 2),
        (0, 12),
        (0, 1),
        (0, 12),
        (0, 45),
        (0, 1),
        (0, 2),
        (0, 11),
        (0, 0),
        (0, 1),
        (0, 90),
        (0, 95),
        (0, 2),
        (0, 26),
        (0, 1),
        (0, 12),
        (0, 2),
        (0, 1),
        (0, 3),
        (0, 6),
        (0, 2),
        (0, 5),
        (0, 2),
        (0, 9),
        (0, 2),
        (0, 8),
        (0, 0),
        (0, 1),
        (0, 5),
        (0, 2),
        (0, 30),
        (0, 4),
        (0, 25),
        (0, 4),
        (0, 4),
        (0, 0),
        (0, 4),
        (0, 13),
        (0, 0),
        (0, 103),
        (0, 264),
        (0, 26),
        (0, 3),
        (0, 33),
        (0, 6),
        (0, 12),
        (0, 2),
        (0, 5),
        (0, 7),
        (0, 0),
        (0, 29),
        (0, 3),
        (0, 12),
        (0, 0),
        (0, 1),
        (0, 0),
        (0, 1),
        (0, 10),
        (0, 18),
        (0, 27),
        (0, 245),
        (0, 3),
        (0, 2),
        (0, 2),
        (0, 6),
        (0, 2),
        (0, 27),
        (0, 3),
        (0, 8),
        (0, 81),
        (0, 0),
        (0, 1),
        (0, 3),
        (0, 10),
        (0, 30),
        (0, 1),
        (0, 9),
        (0, 14),
        (0, 3),
        (0, 1),
        (0, 27),
        (0, 176),
        (0, 28),
        (0, 8),
        (0, 26),
        (0, 0),
        (0, 12),
        (0, 9),
        (0, 15),
        (0, 1),
        (0, 2),
        (0, 0),
        (0, 0),
        (0, 11),
        (0, 95),
        (0, 1),
        (0, 47),
        (0, 0),
        (0, 0),
        (0, 15),
        (0, 77),
        (0, 0),
        (0, 0),
        (0, 2),
        (0, 16),
        (0, 2),
        (0, 34),
        (0, 0),
        (0, 22),
        (0, 114),
        (0, 6),
        (0, 131),
        (0, 1),
        (0, 24),
        (0, 0),
        (0, 0),
        (0, 1),
        (0, 7),
        (0, 1),
        (0, 207),
        (0, 77),
        (0, 48),
        (0, 0),
        (0, 5),
        (0, 8),
        (0, 0),
        (0, 25),
        (0, 20),
        (0, 0),
        (0, 4),
        (0, 3),
        (0, 0),
        (0, 1),
        (0, 3),
        (0, 69),
        (0, 1),
        (0, 3),
        (0, 3),
        (0, 13),
        (0, 4),
        (0, 15),
        (0, 51),
        (0, 8),
        (0, 6),
        (0, 17),
        (0, 6),
        (0, 4),
        (0, 1),
        (0, 69),
        (0, 22),
        (0, 21),
        (0, 29),
        (0, 3),
        (0, 8),
        (0, 5),
        (0, 3),
        (0, 23),
        (0, 105),
        (0, 3),
        (0, 1),
        (0, 17),
        (0, 95),
        (0, 21),
        (0, 1),
        (0, 52),
        (0, 2),
        (0, 0),
        (0, 0),
        (0, 5),
        (0, 25),
        (0, 80),
        (0, 4),
        (0, 0),
        (0, 22),
        (0, 10),
        (0, 0),
        (0, 0),
        (0, 9),
        (0, 16),
        (0, 42),
        (0, 9),
        (0, 7),
        (0, 0),
        (0, 10),
        (0, 0),
        (0, 0),
        (0, 1),
        (0, 54),
        (0, 28),
        (0, 29),
        (0, 4),
        (0, 0),
        (0, 0),
        (0, 8),
        (0, 0),
        (0, 5),
        (0, 34),
        (0, 5),
        (0, 1),
        (0, 0),
        (0, 15),
        (0, 0),
        (0, 0),
        (0, 36),
        (0, 0),
        (0, 0),
        (0, 4),
        (0, 0),
        (0, 0),
        (0, 19),
        (0, 157),
        (0, 89),
        (0, 0),
        (0, 0),
        (0, 89),
        (0, 0),
        (0, 5),
        (0, 3),
        (0, 2),
        (0, 5),
        (0, 16),
        (0, 2),
        (0, 23),
        (0, 1),
        (0, 4),
        (0, 2),
        (0, 35),
        (0, 4),
        (0, 0),
        (0, 6),
        (0, 9),
        (0, 7),
        (0, 8),
        (0, 1),
        (0, 108),
        (0, 6),
        (0, 47),
        (0, 0),
        (0, 22),
        (0, 17),
        (0, 4),
        (0, 0),
        (0, 12),
        (0, 0),
        (0, 1),
        (0, 1),
        (0, 22),
        (0, 53),
        (0, 11),
        (0, 21),
        (0, 1),
        (0, 5),
        (0, 2),
        (0, 0),
        (0, 29),
        (0, 21),
        (0, 15),
        (0, 1),
        (0, 8),
        (0, 39),
        (0, 28),
        (0, 0),
        (0, 1),
        (0, 108),
        (0, 10),
        (0, 4),
        (0, 49),
        (0, 2),
        (0, 4),
        (0, 9),
        (0, 2),
        (0, 0),
        (0, 286),
        (0, 5),
        (0, 77),
        (0, 2),
        (0, 0),
        (0, 15),
        (0, 5),
        (0, 57),
        (0, 0),
        (0, 1),
        (0, 0),
        (0, 5),
        (0, 5),
        (0, 8),
        (0, 1),
        (0, 8),
        (0, 51),
        (0, 23),
        (0, 25),
        (0, 71),
        (0, 6),
        (0, 12),
        (0, 0),
        (0, 503),
        (0, 6),
        (0, 29),
        (0, 29),
        (0, 4),
        (0, 150),
        (0, 4),
        (0, 145),
        (0, 2),
        (0, 19),
        (0, 0),
        (0, 1),
        (0, 29),
        (0, 2),
        (0, 12),
        (0, 5),
        (0, 6),
        (0, 60),
        (0, 78),
        (0, 27),
        (0, 0),
        (0, 15),
        (0, 1),
        (0, 7),
        (0, 0),
        (0, 5),
        (0, 1),
        (0, 9),
        (0, 1),
        (0, 12),
        (0, 0),
        (0, 37),
        (0, 6),
        (0, 32),
        (0, 6),
        (0, 7),
        (0, 3),
        (0, 63),
        (0, 13),
        (0, 19),
        (0, 0),
        (0, 12),
        (0, 10),
        (0, 46),
        (0, 0),
        (0, 1),
        (0, 6),
        (0, 0),
        (0, 21),
        (0, 5),
        (0, 90),
        (0, 7),
        (0, 0),
        (0, 11),
        (0, 1),
        (0, 4),
        (0, 1),
        (0, 14),
        (0, 2),
        (0, 0),
        (0, 0),
        (0, 5),
        (0, 0),
        (0, 4),
        (0, 20),
        (0, 0),
        (0, 0),
        (0, 9),
        (0, 110),
        (0, 0),
        (0, 34),
        (0, 29),
        (0, 0),
        (0, 55),
        (0, 93),
        (0, 3),
        (0, 0),
        (0, 19),
        (0, 4),
        (0, 18),
        (0, 67),
        (0, 0),
        (0, 7),
        (0, 9),
        (0, 7),
        (0, 2),
        (0, 7),
        (0, 51),
        (0, 2),
        (0, 2),
        (0, 0),
        (0, 22),
        (0, 141),
        (0, 0),
        (0, 61),
        (0, 0),
        (0, 1),
        (0, 3),
        (0, 0),
        (0, 0),
        (0, 17),
        (0, 5),
        (0, 109),
        (0, 19),
        (0, 3),
        (0, 5),
        (0, 7),
        (0, 50),
        (0, 2),
        (0, 14),
        (0, 1),
        (0, 1),
        (0, 1),
        (0, 10),
        (0, 0),
        (0, 1),
        (0, 11),
        (0, 3),
        (0, 0),
        (0, 38),
        (0, 0),
        (0, 3),
        (0, 0),
        (0, 12),
        (0, 35),
        (0, 0),
        (0, 39),
        (0, 159),
        (0, 0),
        (0, 1),
        (0, 12),
        (0, 105),
        (0, 0),
        (0, 2),
        (0, 13),
        (0, 6),
        (0, 65),
        (0, 9),
        (0, 2),
        (0, 27),
        (0, 16),
        (0, 3),
        (0, 19),
        (0, 2),
        (0, 8),
        (0, 6),
        (0, 4),
        (0, 33),
        (0, 11),
        (0, 5),
        (0, 7),
        (0, 19),
        (0, 11),
        (0, 0),
        (0, 50),
        (0, 0),
        (0, 3),
        (0, 5),
        (0, 0),
        (0, 14),
        (0, 2),
        (0, 24),
        (0, 6),
        (0, 14),
        (0, 0),
        (0, 13),
        (0, 29),
        (0, 14),
        (0, 125),
        (0, 27),
        (0, 13),
        (0, 14),
        (0, 30),
        (0, 7),
        (0, 6),
        (0, 0),
        (0, 85),
        (0, 69),
        (0, 15),
        (0, 15),
        (0, 25),
        (0, 60),
        (0, 1),
        (0, 19),
        (0, 0),
        (0, 4),
        (0, 4),
        (0, 9),
        (0, 9),
        (0, 5),
        (0, 4),
        (0, 19),
        (0, 2),
        (0, 20),
        (0, 0),
        (0, 0),
        (0, 8),
        (0, 8),
        (0, 74),
        (0, 15),
        (0, 23),
        (0, 1),
        (0, 19),
        (0, 16),
        (0, 0),
        (0, 15),
        (0, 1),
        (0, 0),
        (0, 1),
        (0, 9),
        (0, 24),
        (0, 54),
        (0, 1),
        (0, 6),
        (0, 4),
        (0, 6),
        (0, 0),
        (0, 15),
        (0, 2),
        (0, 92),
        (0, 18),
        (0, 9),
        (0, 25),
        (0, 0),
        (0, 2),
        (0, 334),
        (0, 8),
        (0, 7),
        (0, 13),
        (0, 7),
        (0, 0),
        (0, 16),
        (0, 115),
        (0, 43),
        (0, 11),
        (0, 7),
        (0, 0),
        (0, 3),
        (0, 40),
        (0, 19),
        (0, 60),
        (0, 16),
        (0, 28),
        (0, 41),
        (0, 17),
        (0, 38),
        (0, 11),
        (0, 77),
        (0, 51),
        (0, 23),
        (0, 60),
        (0, 2),
        (0, 22),
        (0, 1),
        (0, 2),
        (0, 5),
        (0, 10),
        (0, 3),
        (0, 15),
        (0, 8),
        (0, 0),
        (0, 22),
        (0, 7),
        (0, 40),
        (0, 1),
        (0, 139),
        (0, 0),
        (0, 55),
        (0, 15),
        (0, 2),
        (0, 8),
        (0, 0),
        (0, 0),
        (0, 1),
        (0, 0),
        (0, 14),
        (0, 129),
        (0, 52),
        (0, 92),
        (0, 1),
        (0, 7),
        (0, 53),
        (0, 30),
        (0, 1),
        (0, 103),
        (0, 0),
        (0, 0),
        (0, 0),
        (0, 2),
        (0, 1),
        (0, 0),
        (0, 2),
        (0, 2),
        (0, 5),
        (0, 0),
        (0, 24),
        (0, 3),
        (0, 0),
        (0, 26),
        (0, 15),
        (0, 4),
        (0, 4),
        (0, 0),
        (0, 0),
        (0, 60),
        (0, 34),
        (0, 0),
        (0, 3),
        (0, 79),
        (0, 8),
        (0, 9),
        (0, 0),
        (0, 18),
        (0, 38),
        (0, 4),
        (0, 15),
        (0, 1),
        (0, 4),
        (0, 91),
        (0, 5),
        (0, 0),
        (0, 12),
        (0, 25),
        (0, 11),
        (0, 0),
        (0, 2),
        (0, 213),
        (0, 19),
        (0, 19),
        (0, 0),
        (0, 0),
        (0, 1),
        (0, 4),
        (0, 19),
        (0, 3),
        (0, 0),
        (0, 6),
        (0, 14),
        (0, 2),
        (0, 292),
        (0, 0),
        (0, 4),
        (0, 0),
        (0, 4),
        (0, 2),
        (0, 9),
        (0, 72),
        (0, 33),
        (0, 0),
        (0, 5),
        (0, 0),
        (0, 7),
        (0, 14),
        (0, 56),
        (0, 0),
        (0, 9),
        (0, 6),
        (0, 12),
        (0, 2),
        (0, 2),
        (0, 533),
        (0, 52),
        (0, 16),
        (0, 1),
        (0, 44),
        (0, 2),
        (0, 3),
        (0, 18),
        (0, 53),
        (0, 2),
        (0, 22),
        (0, 24),
        (0, 24),
        (0, 42),
        (0, 47),
        (0, 1),
        (0, 0),
        (0, 1),
        (0, 119),
        (0, 14),
        (0, 3),
        (0, 31),
        (0, 15),
        (0, 0),
        (0, 2),
        (0, 31),
        (0, 22),
        (0, 90),
        (0, 25),
        (0, 39),
        (0, 20),
        (0, 12),
        (0, 200),
        (0, 1),
        (0, 42),
        (0, 3),
        (0, 2),
        (0, 8),
        (0, 2),
        (0, 0),
        (0, 36),
        (0, 23),
        (0, 1),
        (0, 0),
        (0, 45),
        (0, 8),
        (0, 25),
        (0, 348),
        (0, 0),
        (0, 22),
        (0, 2),
        (0, 1),
        (0, 0),
        (0, 0),
        (0, 0),
        (0, 10),
        (0, 87),
        (0, 0),
        (0, 41),
        (0, 17),
        (0, 7),
        (0, 0),
        (0, 10),
        (0, 16),
        (0, 2),
        (0, 27),
        (0, 5),
        (0, 0),
        (0, 4),
        (0, 5),
        (0, 0),
        (0, 2),
        (0, 0),
        (0, 5),
        (0, 28),
        (0, 1),
        (0, 0),
        (0, 15),
        (0, 0),
        (0, 21),
        (0, 15),
        (0, 10),
        (0, 0),
        (0, 2),
        (0, 0),
        (0, 12),
        (0, 15),
        (0, 12),
        (0, 10),
        (0, 417),
        (0, 5),
        (0, 23),
        (0, 54),
        (0, 0),
        (0, 1),
        (0, 20),
        (0, 12),
        (0, 615),
        (0, 9),
        (0, 14),
        (0, 0),
        (0, 0),
        (0, 48),
        (0, 4),
        (0, 2),
        (0, 181),
        (0, 4),
        (0, 7),
        (0, 0),
        (0, 82),
        (0, 3),
        (0, 0),
        (0, 0),
        (0, 8),
        (0, 33),
        (0, 0),
        (0, 2),
        (0, 0),
        (0, 32),
        (0, 4),
        (0, 13),
        (0, 9),
        (0, 602),
        (0, 3),
        (0, 1),
        (0, 63),
        (0, 7),
        (0, 172),
        (0, 128),
        (0, 2),
        (0, 158),
        (0, 45),
        (0, 2),
        (0, 642),
        (0, 3),
        (0, 10),
        (0, 23),
        (0, 454),
        (0, 0),
        (0, 0),
        (0, 98),
        (0, 9),
        (0, 12),
        (0, 0),
        (0, 0),
        (0, 6),
        (0, 0),
        (0, 7),
        (0, 2),
        (0, 0),
        (0, 34),
        (0, 63),
        (0, 35),
        (0, 0),
        (0, 25),
        (0, 56),
        (0, 0),
        (0, 0),
        (0, 2),
        (0, 3),
        (0, 2),
        (0, 9),
        (0, 3),
        (0, 24),
        (0, 16),
        (0, 10),
        (0, 32),
        (0, 122),
        (0, 7),
        (0, 8),
        (0, 6),
        (0, 1),
        (0, 3),
        (0, 47),
        (0, 31),
        (0, 209),
        (0, 0),
        (0, 27),
        (0, 7),
        (0, 14),
        (0, 4),
        (0, 210),
        (0, 6),
        (0, 7),
        (0, 7),
        (0, 59),
        (0, 10),
        (0, 32),
        (0, 1),
        (0, 110),
        (0, 252),
        (0, 0),
        (0, 38),
        (0, 4),
        (0, 2),
        (0, 23),
        (0, 3),
        (0, 0),
        (0, 74),
        (0, 7),
        (0, 73),
        (0, 1),
        (0, 1),
        (0, 9),
        (0, 5),
        (0, 162),
        (0, 11),
        (0, 0),
        (0, 0),
        (0, 0),
        (0, 7),
        (0, 185),
        (0, 32),
        (0, 272),
        (1, 7),
        (0, 4),
        (0, 0),
        (0, 167),
        (0, 1),
        (0, 36),
        (0, 10),
        (0, 7),
        (0, 42),
        (0, 3),
        (0, 1),
        (0, 10),
        (0, 32),
        (0, 26),
        (0, 3),
        (0, 10),
        (0, 12),
        (0, 7),
        (0, 28),
        (0, 47),
        (0, 4),
        (0, 37),
        (0, 9),
        (0, 5),
        (0, 15),
        (0, 0),
        (0, 7),
        (0, 9),
        (0, 61),
        (0, 0),
        (0, 2),
        (0, 8),
        (0, 10),
        (0, 2),
        (0, 9),
        (0, 8),
        (0, 0),
        (0, 92),
        (0, 1),
        (0, 9),
        (0, 18),
        (0, 2),
        (0, 11),
        (0, 0),
        (0, 16),
        (0, 3),
        (0, 33),
        (0, 20),
        (0, 39),
        (0, 34),
        (0, 41),
        (0, 0),
        (0, 11),
        (0, 4),
        (0, 1),
        (0, 9),
        (0, 2),
        (0, 5),
        (0, 35),
        (0, 25),
        (0, 0),
        (0, 0),
        (0, 0),
        (0, 1),
        (0, 578),
        (0, 13),
        (0, 8),
        (0, 19),
        (0, 11),
        (0, 11),
        (0, 203),
        (0, 0),
        (0, 17),
        (0, 142),
        (0, 25),
        (0, 81),
        (0, 8),
        (0, 5),
        (0, 3),
        (0, 0),
        (0, 18),
        (0, 6),
        (0, 26),
        (0, 28),
        (0, 0),
        (0, 73),
        (0, 22),
        (0, 5),
        (0, 142),
        (0, 0),
        (0, 2),
        (0, 0),
        (0, 15),
        (0, 2),
        (0, 0),
        (0, 0),
        (0, 4),
        (0, 2),
        (0, 13),
        (0, 7),
        (0, 28),
        (0, 1),
        (0, 0),
        (0, 12),
        (0, 4),
        (0, 0),
        (0, 285),
        (0, 1),
        (0, 9),
        (0, 3),
        (0, 10),
        (0, 1),
        (0, 1),
        (0, 19),
        (0, 7),
        (0, 3),
        (0, 17),
        (0, 48),
        (0, 62),
        (0, 11),
        (0, 10),
        (0, 0),
        (0, 39),
        (0, 14),
        (0, 5),
        (0, 0),
        (0, 16),
        (0, 3),
        (0, 2),
        (0, 55),
        (0, 2),
        (0, 5),
        (0, 0),
        (0, 8),
        (0, 64),
        (0, 12),
        (0, 0),
        (0, 3),
        (0, 0),
        (0, 0),
        (0, 0),
        (0, 99),
        (0, 55),
        (0, 354),
        (0, 5),
        (0, 13),
        (0, 1),
        (0, 3),
        (0, 140),
        (0, 0),
        (0, 3),
        (0, 12),
        (0, 0),
        (0, 0),
        (0, 1),
        (0, 4),
        (0, 29),
        (0, 21),
        (0, 51),
        (0, 0),
        (0, 21),
        (0, 0),
        (0, 2),
        (0, 1),
        (0, 61),
        (0, 23),
        (0, 93),
        (0, 0),
        (0, 0),
        (0, 46),
        (0, 12),
        (0, 172),
        (0, 15),
        (0, 0),
        (0, 87),
        (0, 32),
        (0, 1),
        (0, 2),
        (0, 16),
        (0, 0),
        (0, 0),
        (0, 10),
        (0, 8),
        (0, 9),
        (0, 7),
        (0, 1),
        (0, 45),
        (0, 0),
        (0, 23),
        (0, 6),
        (0, 0),
        (0, 4),
        (0, 0),
        (0, 23),
        (0, 23),
        (0, 2),
        (0, 9),
        (0, 32),
        (0, 6),
        (0, 5),
        (0, 7),
        (0, 4),
        (0, 1),
        (0, 42),
        (0, 5),
        (0, 13),
        (0, 0),
        (0, 1),
        (0, 40),
        (0, 6),
        (0, 4),
        (0, 40),
        (0, 19),
        (0, 41),
        (0, 0),
        (0, 41),
        (0, 22),
        (0, 34),
        (0, 0),
        (0, 1),
        (0, 1),
        (0, 4),
        (0, 6),
        (0, 42),
        (0, 4),
        (0, 19),
        (0, 703),
        (0, 6),
        (0, 10),
        (0, 4),
        (0, 0),
        (0, 0),
        (0, 928),
        (0, 0),
        (0, 1),
        (0, 27),
        (0, 130),
        (0, 30),
        (0, 17),
        (0, 28),
        (0, 12),
        (0, 0),
        (0, 28),
        (0, 6),
        (0, 271),
        (0, 39),
        (0, 16),
        (0, 42),
        (0, 7),
        (0, 3),
        (0, 12),
        (0, 0),
        (0, 60),
        (0, 160),
        (0, 13),
        (0, 54),
        (0, 66),
        (0, 791),
        (0, 0),
        (0, 46),
        (0, 1),
        (0, 0),
        (0, 0),
        (0, 17),
        (0, 89),
        (0, 13),
        (0, 14),
        (0, 0),
        (0, 140),
        (0, 246),
        (0, 1),
        (0, 1),
        (0, 62),
        (0, 33),
        (0, 15),
        (0, 0),
        (0, 44),
        (0, 9),
        (0, 43),
        (0, 45),
        (0, 0),
        (0, 3),
        (0, 797),
        (0, 43),
        (0, 41),
        (0, 12),
        (0, 52),
        (0, 28),
        (0, 6),
        (0, 2),
        (0, 0),
        (0, 5),
        (0, 1),
        (0, 3),
        (0, 463),
        (0, 0),
        (0, 134),
        (0, 1),
        (0, 3),
        (0, 5),
        (0, 107),
        (0, 15),
        (0, 25),
        (0, 7),
        (0, 85),
        (0, 1),
        (0, 5),
        (0, 4),
        (0, 1),
        (0, 131),
        (0, 50),
        (0, 467),
        (0, 2),
        (0, 2),
        (0, 0),
        (0, 0),
        (0, 2),
        (0, 3),
        (0, 2),
        (0, 8),
        (0, 31),
        (0, 15),
        (0, 12),
        (0, 3),
        (0, 44),
        (0, 97),
        (0, 19),
        (0, 7),
        (0, 4),
        (0, 6),
        (0, 5),
        (0, 36),
        (0, 98),
        (0, 127),
        (0, 115),
        (0, 1),
        (0, 0),
        (0, 36),
        (0, 16),
        (0, 52),
        (0, 7),
        (0, 3),
        (0, 73),
        (0, 0),
        (0, 4),
        (0, 4),
        (0, 84),
        (0, 13),
        (0, 42),
        (0, 0),
        (0, 611),
        (0, 3),
        (0, 5),
        (0, 1),
        (0, 20),
        (0, 24),
        (0, 20),
        (0, 2),
        (0, 32),
        (0, 0),
        (0, 6),
        (0, 1),
        (0, 28),
        (0, 53),
        (0, 4),
        (0, 8),
        (0, 16),
        (0, 0),
        (0, 9),
        (0, 6),
        (0, 0),
        (0, 5),
        (0, 99),
        (0, 112),
        (0, 127),
        (0, 2),
        (0, 4),
        (0, 62),
        (0, 2),
        (0, 0),
        (0, 206),
        (0, 29),
        (0, 0),
        (0, 0),
        (0, 1),
        (0, 7),
        (0, 7),
        (0, 3),
        (0, 49),
        (0, 7),
        (0, 8),
        (0, 60),
        (0, 2),
        (0, 5),
        (0, 106),
        (0, 2),
        (0, 115),
        (0, 11),
        (0, 0),
        (0, 7),
        (0, 52),
        (0, 194),
        (0, 0),
        (0, 11),
        (0, 1),
        (0, 114),
        (0, 14),
        (0, 180),
        (0, 56),
        (0, 117),
        (0, 132),
        (0, 1),
        (0, 4),
        (0, 3),
        (0, 14),
        (0, 0),
        (0, 184),
        (0, 46),
        (0, 3),
        (0, 5),
        (0, 8),
        (0, 94),
        (0, 2),
        (0, 46),
        (0, 18),
        (0, 14),
        (0, 11),
        (0, 21),
        (0, 7),
        (0, 4),
        (0, 2),
        (0, 6),
        (0, 3),
        (0, 14),
        (0, 0),
        (0, 4),
        (0, 26),
        (0, 70),
        (0, 16),
        (0, 12),
        (0, 20),
        (0, 6),
        (0, 1),
        (0, 4),
        (0, 72),
        (0, 1),
        (0, 1),
        (0, 12),
        (0, 1),
        (0, 2),
        (0, 2),
        (0, 0),
        (0, 3),
        (0, 0),
        (0, 11),
        (0, 39),
        (0, 29),
        (0, 0),
        (0, 0),
        (0, 1),
        (0, 4),
        (0, 10),
        (0, 13),
        (0, 255),
        (0, 17),
        (0, 11),
        (0, 0),
        (0, 0),
        (0, 5),
        (0, 76),
        (0, 2),
        (0, 0),
        (0, 9),
        (0, 0),
        (0, 988),
        (0, 0),
        (0, 5),
        (0, 22),
        (0, 86),
        (0, 0),
        (0, 14),
        (0, 29),
        (0, 44),
        (0, 153),
        (0, 16),
        (0, 0),
        (0, 2),
        (0, 0),
        (0, 33),
        (0, 108),
        (0, 6),
        (0, 10),
        (0, 0),
        (0, 23),
        (0, 31),
        (0, 106),
        (0, 0),
        (0, 0),
        (0, 1),
        (0, 712),
        (0, 53),
        (0, 7),
        (0, 962),
        (0, 69),
        (0, 11),
        (0, 28),
        (0, 0),
        (0, 29),
        (0, 1),
        (0, 43),
        (0, 0),
        (0, 2),
        (0, 10),
        (0, 11),
        (0, 0),
        (0, 0),
        (0, 0),
        (0, 0),
        (0, 19),
        (0, 1),
        (0, 75),
        (0, 85),
        (0, 221),
        (0, 4),
        (0, 0),
        (0, 124),
        (0, 10),
        (0, 13),
        (0, 3),
        (0, 22),
        (0, 62),
        (0, 0),
        (0, 0),
        (0, 0),
        (0, 5),
        (0, 27),
        (0, 11),
        (0, 14),
        (0, 97),
        (0, 17),
        (0, 1069),
        (0, 2),
        (0, 1),
        (0, 2),
        (0, 24),
        (0, 82),
        (0, 4),
        (0, 69),
        (0, 0),
        (0, 0),
        (0, 64),
        (0, 0),
        (0, 130),
        (0, 4),
        (0, 0),
        (0, 38),
        (0, 10),
        (0, 0),
        (0, 18),
        (0, 111),
        (0, 15),
        (0, 16),
        (0, 13),
        (0, 185),
        (0, 3),
        (0, 27),
        (0, 9),
        (0, 250),
        (0, 110),
        (0, 16),
        (0, 0),
        (0, 21),
        (0, 4),
        (0, 220),
        (0, 6),
        (0, 285),
        (0, 36),
        (0, 7),
        (0, 0),
        (0, 24),
        (0, 0),
        (0, 5),
        (0, 25),
        (0, 27),
        (0, 1),
        (0, 0),
        (0, 17),
        (0, 4),
        (0, 71),
        (0, 103),
        (0, 84),
        (0, 3),
        (0, 3),
        (0, 0),
        (0, 98),
        (0, 50),
        (0, 0),
        (0, 0),
        (0, 85),
        (0, 8),
        (0, 53),
        (0, 9),
        (0, 142),
        (0, 0),
        (0, 173),
        (0, 130),
        (0, 31),
        (0, 8),
        (0, 0),
        (0, 15),
        (0, 3),
        (0, 15),
        (0, 17),
        (0, 0),
        (0, 1),
        (0, 8),
        (0, 3),
        (0, 10),
        (0, 17),
        (0, 2),
        (0, 12),
        (0, 10),
        (0, 8),
        (0, 0),
        (0, 46),
        (0, 17),
        (0, 50),
        (0, 0),
        (0, 2),
        (0, 1),
        (0, 0),
        (0, 9),
        (0, 23),
        (0, 0),
        (0, 1),
        (0, 0),
        (0, 42),
        (0, 0),
        (0, 15),
        (0, 5),
        (0, 30),
        (0, 13),
        (0, 21),
        (0, 2),
        (0, 28),
        (0, 12),
        (0, 27),
        (0, 28),
        (0, 19),
        (0, 41),
        (0, 5),
        (0, 6),
        (0, 17),
        (0, 53),
        (0, 17),
        (0, 4),
        (0, 270),
        (0, 0),
        (0, 24),
        (0, 0),
        (0, 12),
        (0, 14),
        (0, 0),
        (0, 23),
        (0, 48),
        (0, 19),
        (0, 0),
        (0, 170),
        (0, 5),
        (0, 15),
        (0, 31),
        (0, 53),
        (0, 4),
        (0, 9),
        (0, 32),
        (0, 21),
        (0, 2),
        (0, 28),
        (0, 0),
        (0, 18),
        (0, 8),
        (0, 2),
        (0, 11),
        (0, 26),
        (0, 2),
        (0, 5),
        (0, 1),
        (0, 36),
        (0, 3),
        (0, 405),
        (0, 6),
        (0, 3),
        (0, 69),
        (0, 7),
        (0, 0),
        (0, 3),
        (0, 7),
        (0, 7),
        (0, 67),
        (0, 156),
        (0, 22),
        (0, 35),
        (0, 0),
        (0, 18),
        (0, 0),
        (0, 2),
        (0, 52),
        (0, 33),
        (0, 12),
        (0, 23),
        (0, 17),
        (0, 30),
        (0, 226),
        (0, 4),
        (0, 94),
        (0, 26),
        (0, 1),
        (0, 51),
        (0, 331),
        (0, 30),
        (0, 1),
        (0, 0),
        (0, 0),
        (0, 17),
        (0, 0),
        (0, 10),
        (0, 1),
        (0, 25),
        (0, 2),
        (0, 23),
        (0, 39),
        (0, 2),
        (0, 6),
        (0, 55),
        (0, 105),
        (0, 29),
        (0, 1),
        (0, 3),
        (0, 25),
        (0, 42),
        (0, 17),
        (0, 478),
        (0, 6),
        (0, 0),
        (0, 0),
        (0, 9),
        (0, 103),
        (0, 964),
        (0, 0),
        (0, 5),
        (0, 1),
        (0, 28),
        (0, 21),
        (0, 0),
        (0, 26),
        (0, 6),
        (0, 1),
        (0, 27),
        (0, 1),
        (0, 54),
        (0, 0),
        (0, 7),
        (0, 0),
        (0, 0),
        (0, 14),
        (0, 0),
        (0, 2),
        (0, 2),
        (0, 1),
        (0, 21),
        (0, 1),
        (0, 2),
        (0, 0),
        (0, 2),
        (0, 1),
        (0, 134),
        (0, 10),
        (0, 379),
        (0, 0),
        (0, 1),
        (0, 36),
        (0, 25),
        (0, 643),
        (0, 1),
        (0, 22),
        (0, 50),
        (0, 8),
        (0, 55),
        (0, 2),
        (0, 60),
        (0, 0),
        (0, 191),
        (0, 31),
        (0, 1),
        (0, 6),
        (0, 3),
        (0, 28),
        (0, 11),
        (0, 144),
        (0, 18),
        (0, 0),
        (0, 55),
        (0, 137),
        (0, 2),
        (0, 16),
        (0, 2),
        (0, 2),
        (0, 194),
        (0, 0),
        (0, 16),
        (0, 142),
        (0, 0),
        (0, 3),
        (0, 0),
        (0, 30),
        (0, 122),
        (0, 7),
        (0, 16),
        (0, 15),
        (0, 10),
        (0, 3),
        (0, 0),
        (0, 41),
        (0, 9),
        (0, 906),
        (0, 0),
        (0, 85),
        (0, 100),
        (0, 12),
        (0, 24),
        (0, 223),
        (0, 7),
        (0, 1),
        (0, 21),
        (0, 0),
        (0, 10),
        (0, 15),
        (0, 0),
        (0, 8),
        (0, 69),
        (0, 1),
        (0, 87),
        (0, 48),
        (0, 0),
        (0, 156),
        (0, 4),
        (0, 3),
        (0, 40),
        (0, 127),
        (0, 2),
        (0, 10),
        (0, 2),
        (0, 5),
        (0, 0),
        (0, 3),
        (0, 121),
        (0, 0),
        (0, 74),
        (0, 12),
        (0, 66),
        (0, 58),
        (0, 57),
        (0, 3),
        (0, 3),
        (0, 12),
        (0, 0),
        (0, 6),
        (0, 96),
        (0, 261),
        (0, 2),
        (0, 20),
        (0, 54),
        (0, 2),
        (0, 2),
        (0, 36),
        (0, 13),
        (0, 22),
        (0, 60),
        (0, 57),
        (0, 13),
        (0, 534),
        (0, 18),
        (0, 0),
        (0, 75),
        (0, 214),
        (0, 1),
        (0, 18),
        (0, 32),
        (0, 0),
        (0, 275),
        (0, 17),
        (0, 4),
        (0, 11),
        (0, 5),
        (0, 34),
        (0, 86),
        (0, 16),
        (0, 66),
        (0, 9),
        (0, 72),
        (0, 30),
        (0, 212),
        (0, 8),
        (0, 5),
        (0, 1),
        (0, 15),
        (0, 6),
        (0, 52),
        (0, 9),
        (0, 32),
        (0, 47),
        (0, 26),
        (0, 8),
        (0, 0),
        (0, 1),
        (0, 13),
        (0, 98),
        (0, 11),
        (0, 0),
        (0, 129),
        (0, 65),
        (0, 24),
        (0, 13),
        (0, 73),
        (0, 48),
        (0, 9),
        (0, 0),
        (0, 25),
        (0, 81),
        (0, 0),
        (0, 0),
        (0, 0),
        (0, 2),
        (0, 2),
        (0, 5),
        (0, 3),
        (0, 17),
        (0, 156),
        (0, 0),
        (0, 45),
        (0, 20),
        (0, 0),
        (0, 5),
        (0, 47),
        (0, 7),
        (0, 126),
        (0, 0),
        (0, 0),
        (0, 44),
        (0, 37),
        (0, 69),
        (0, 81),
        (0, 588),
        (0, 0),
        (0, 17),
        (0, 12),
        (0, 375),
        (0, 87),
        (0, 58),
        (0, 0),
        (0, 27),
        (0, 189),
        (0, 3),
        (0, 46),
        (0, 0),
        (0, 349),
        (0, 0),
        (0, 60),
        (0, 6),
        (0, 4),
        (0, 1),
        (0, 88),
        (0, 80),
        (0, 33),
        (0, 17),
        (0, 173),
        (0, 17),
        (0, 196),
        (0, 28),
        (0, 137),
        (0, 89),
        (0, 0),
        (0, 17),
        (0, 1161),
        (0, 1396),
        (0, 1),
        (0, 39),
        (0, 115),
        (0, 5),
        (0, 11),
        (0, 304),
        (0, 28),
        (0, 10),
        (0, 1),
        (0, 23),
        (0, 1),
        (0, 10),
        (0, 39),
        (0, 48),
        (0, 9),
        (0, 134),
        (0, 131),
        (0, 4),
        (0, 21),
        (0, 37),
        (0, 5),
        (0, 22),
        (0, 0),
        (0, 2),
        (0, 32),
        (0, 0),
        (0, 6),
        (0, 4),
        (0, 18),
        (0, 30),
        (0, 15),
        (0, 40),
        (0, 5),
        (0, 26),
        (0, 11),
        (0, 2),
        (0, 15),
        (0, 2),
        (0, 4),
        (0, 33),
        (0, 23),
        (0, 26),
        (0, 1),
        (0, 126),
        (0, 0),
        (0, 69),
        (0, 33),
        (0, 1),
        (0, 1),
        (0, 117),
        (0, 782),
        (0, 115),
        (0, 10),
        (0, 6),
        (0, 2),
        (0, 3),
        (0, 0),
        (0, 0),
        (0, 21),
        (0, 0),
        (0, 0),
        (0, 45),
        (0, 3),
        (0, 7),
        (0, 199),
        (0, 0),
        (0, 642),
        (0, 41),
        (0, 66),
        (0, 25),
        (0, 0),
        (0, 8),
        (0, 0),
        (0, 135),
        (0, 1),
        (0, 31),
        (0, 0),
        (0, 31),
        (0, 2),
        (0, 88),
        (0, 36),
        (0, 5),
        (0, 14),
        (0, 193),
        (0, 15),
        (0, 3),
        (0, 13),
        (0, 0),
        (0, 0),
        (0, 29),
        (0, 15),
        (0, 35),
        (0, 0),
        (0, 36),
        (0, 1),
        (0, 13),
        (0, 1),
        (0, 47),
        (0, 47),
        (0, 0),
        (0, 33),
        (0, 41),
        (0, 353),
        (0, 102),
        (0, 11),
        (0, 112),
        (0, 7),
        (0, 1),
        (0, 8),
        (0, 236),
        (0, 12),
        (0, 54),
        (0, 0),
        (0, 18),
        (0, 4),
        (0, 0),
        (0, 11),
        (0, 109),
        (0, 2),
        (0, 18),
        (0, 0),
        (0, 21),
        (0, 2),
        (0, 8),
        (0, 2),
        (0, 51),
        (0, 132),
        (0, 4),
        (0, 1519),
        (0, 1),
        (0, 7),
        (0, 334),
        (0, 46),
        (0, 16),
        (0, 68),
        (0, 0),
        (0, 597),
        (0, 0),
        (0, 423),
        (0, 32),
        (0, 156),
        (0, 93),
        (0, 10),
        (0, 44),
        (0, 296),
        (0, 39),
        (0, 32),
        (0, 15),
        (0, 1),
        (0, 29),
        (0, 8),
        (0, 60),
        (0, 6),
        (0, 0),
        (0, 1),
        (0, 124),
        (0, 0),
        (0, 17),
        (0, 20),
        (0, 187),
        (0, 0),
        (0, 3),
        (0, 78),
        (0, 19),
        (0, 1372),
        (0, 105),
        (0, 50),
        (0, 55),
        (0, 11),
        (0, 7),
        (0, 18),
        (0, 21),
        (0, 55),
        (0, 49),
        (0, 96),
        (0, 60),
        (0, 1),
        (0, 0),
        (0, 0),
        (0, 32),
        (0, 4),
        (0, 5),
        (0, 10),
        (0, 80),
        (0, 9),
        (0, 12),
        (0, 200),
        (0, 77),
        (0, 31),
        (0, 33),
        (0, 2),
        (0, 83),
        (0, 3),
        (0, 50),
        (0, 112),
        (0, 0),
        (0, 67),
        (0, 0),
        (0, 2),
        (0, 118),
        (0, 18),
        (0, 2),
        (0, 59),
        (0, 0),
        (0, 3),
        (0, 81),
        (0, 41),
        (0, 406),
        (0, 3),
        (0, 7),
        (0, 46),
        (0, 8),
        (0, 39),
        (0, 0),
        (0, 215),
        (0, 0),
        (0, 130),
        (0, 23),
        (0, 8),
        (0, 103),
        (0, 623),
        (0, 12),
        (0, 35),
        (0, 2),
        (0, 73),
        (0, 3),
        (0, 0),
        (0, 3),
        (0, 2),
        (0, 130),
        (0, 57),
        (0, 0),
        (0, 111),
        (0, 290),
        (0, 2),
        (0, 104),
        (0, 362),
        (0, 114),
        (0, 0),
        (0, 3),
        (0, 46),
        (0, 6),
        (0, 8),
        (0, 9),
        (0, 27),
        (0, 10),
        (0, 0),
        (0, 4),
        (0, 0),
        (0, 29),
        (0, 0),
        (0, 0),
        (0, 186),
        (0, 109),
        (0, 5),
        (0, 0),
        (0, 26),
        (0, 194),
        (0, 0),
        (0, 2),
        (0, 21),
        (0, 22),
        (0, 215),
        (0, 27),
        (0, 38),
        (0, 16),
        (0, 4),
        (0, 0),
        (0, 14),
        (0, 34),
        (0, 37),
        (0, 202),
        (0, 43),
        (0, 11),
        (0, 4),
        (0, 0),
        (0, 8),
        (0, 97),
        (0, 91),
        (0, 12),
        (0, 40),
        (0, 76),
        (0, 145),
        (0, 12),
        (0, 7),
        (0, 180),
        (0, 2),
        (0, 356),
        (0, 101),
        (0, 64),
        (0, 0),
        (0, 1),
        (0, 51),
        (0, 0),
        (0, 0),
        (0, 0),
        (0, 121),
        (0, 20),
        (0, 122),
        (0, 18),
        (0, 17),
        (0, 280),
        (0, 98),
        (0, 22),
        (0, 10),
        (0, 6),
        (0, 82),
        (0, 114),
        (0, 7),
        (0, 37),
        (0, 8),
        (0, 0),
        (0, 0),
        (0, 487),
        (0, 1),
        (0, 6),
        (0, 90),
        (0, 26),
        (0, 1),
        (0, 3),
        (0, 14),
        (0, 36),
        (0, 64),
        (0, 14),
        (0, 109),
        (0, 177),
        (0, 211),
        (0, 14),
        (0, 66),
        (0, 0),
        (0, 220),
        (0, 11),
        (0, 1375),
        (0, 5),
        (0, 15),
        (0, 2),
        (0, 13),
        (0, 2),
        (0, 27),
        (0, 5),
        (0, 0),
        (0, 0),
        (0, 8),
        (0, 96),
        (0, 25),
        (0, 1),
        (0, 3),
        (0, 41),
        (0, 1),
        (0, 5),
        (0, 29),
        (0, 2),
        (0, 14),
        (0, 0),
        (0, 5),
        (0, 14),
        (0, 300),
        (0, 4),
        (0, 0),
        (0, 975),
        (0, 112),
        (0, 711),
        (0, 7),
        (0, 0),
        (0, 901),
        (0, 15),
        (0, 19),
        (0, 1),
        (0, 25),
        (0, 4),
        (0, 19),
        (0, 1),
        (0, 206),
        (0, 95),
        (0, 0),
        (0, 0),
        (0, 0),
        (0, 0),
        (0, 11),
        (0, 3),
        (0, 61),
        (0, 149),
        (0, 43),
        (0, 3),
        (0, 91),
        (0, 12),
        (0, 46),
        (0, 0),
        (0, 38),
        (0, 2),
        (0, 20),
        (0, 11),
        (0, 167),
        (0, 77),
        (0, 15),
        (0, 66),
        (0, 2),
        (0, 0),
        (0, 113),
        (0, 330),
        (0, 241),
        (0, 9),
        (0, 12),
        (0, 3),
        (0, 52),
        (0, 278),
        (0, 38),
        (0, 47),
        (0, 14),
        (0, 46),
        (0, 110),
        (0, 6),
        (0, 99),
        (0, 28),
        (0, 14),
        (0, 38),
        (0, 22),
        (0, 6),
        (0, 16),
        (0, 1177),
        (0, 0),
        (0, 50),
        (0, 4),
        (0, 83),
        (0, 29),
        (0, 0),
        (0, 4),
        (0, 0),
        (0, 6),
        (0, 1),
        (0, 0),
        (0, 417),
        (0, 98),
        (0, 4),
        (0, 0),
        (0, 225),
        (0, 0),
        (0, 4),
        (0, 2),
        (0, 12),
        (0, 2),
        (0, 1),
        (0, 23),
        (0, 8),
        (0, 0),
        (0, 0),
        (0, 0),
        (0, 27),
        (0, 17),
        (0, 0),
        (0, 51),
        (0, 4),
        (0, 634),
        (0, 0),
        (0, 1),
        (0, 44),
        (0, 225),
        (0, 6),
        (0, 928),
        (0, 2),
        (0, 209),
        (0, 42),
        (0, 63),
        (0, 17),
        (0, 2),
        (0, 3),
        (0, 445),
        (0, 0),
        (0, 7),
        (0, 30),
        (0, 81),
        (0, 0),
        (0, 0),
        (0, 3),
        (0, 14),
        (0, 85),
        (0, 0),
        (0, 0),
        (0, 48),
        (0, 1),
        (0, 364),
        (0, 79),
        (0, 0),
        (0, 9),
        (0, 35),
        (0, 20),
        (0, 1),
        (0, 75),
        (0, 0),
        (0, 1),
        (0, 485),
        (0, 51),
        (0, 5),
        (0, 0),
        (0, 0),
        (0, 2),
        (0, 137),
        (0, 0),
        (0, 2),
        (0, 967),
        (0, 170),
        (0, 74),
        (0, 76),
        (0, 12),
        (0, 36),
        (0, 15),
        (0, 0),
        (0, 42),
        (0, 26),
        (0, 0),
        (0, 84),
        (0, 80),
        (0, 20),
        (0, 10),
        (0, 406),
        (0, 251),
        (0, 11),
        (0, 6),
        (0, 1),
        (0, 42),
        (0, 2),
        (0, 80),
        (0, 60),
        (0, 51),
        (0, 48),
        (0, 3),
        (0, 21),
        (0, 58),
        (0, 49),
        (0, 1),
        (0, 121),
        (0, 8),
        (0, 4),
        (0, 9),
        (0, 112),
        (0, 29),
        (0, 19),
        (0, 23),
        (0, 0),
        (0, 5),
        (0, 327),
        (0, 1691),
        (0, 0),
        (0, 10),
        (0, 43),
        (0, 0),
        (0, 88),
        (0, 8),
        (0, 12),
        (0, 51),
        (0, 0),
        (0, 85),
        (0, 135),
        (0, 367),
        (0, 5),
        (0, 257),
        (0, 8),
        (0, 399),
        (0, 123),
        (0, 14),
        (0, 129),
        (0, 104),
        (0, 263),
        (0, 1),
        (0, 20),
        (0, 26),
        (0, 156),
        (0, 54),
        (0, 17),
        (0, 2),
        (0, 7),
        (0, 0),
        (0, 25),
        (0, 1),
        (0, 15),
        (0, 66),
        (0, 19),
        (0, 5),
        (0, 3),
        (0, 2),
        (0, 0),
        (0, 0),
        (0, 2),
        (0, 27),
        (0, 62),
        (0, 34),
        (0, 3),
        (0, 17),
        (0, 9),
        (0, 100),
        (0, 0),
        (0, 98),
        (0, 144),
        (0, 65),
        (0, 0),
        (0, 10),
        (0, 4),
        (0, 258),
        (0, 65),
        (0, 5),
        (0, 27),
        (0, 7),
        (0, 89),
        (0, 1),
        (0, 2),
        (0, 0),
        (0, 117),
        (0, 16),
        (0, 29),
        (0, 125),
        (0, 12),
        (0, 1468),
        (0, 481),
        (0, 165),
        (0, 0),
        (0, 27),
        (0, 0),
        (0, 17),
        (0, 2182),
        (0, 30),
        (0, 143),
        (0, 0),
        (0, 131),
        (0, 31),
        (0, 0),
        (0, 39),
        (0, 0),
        (0, 1),
        (0, 2),
        (0, 18),
        (0, 42),
        (0, 181),
        (0, 47),
        (0, 4),
        (0, 29),
        (0, 16),
        (0, 4),
        (0, 239),
        (0, 29),
        (0, 634),
        (0, 0),
        (0, 15),
        (0, 4),
        (0, 1),
        (0, 11),
        (0, 69),
        (0, 90),
        (0, 78),
        (0, 4),
        (0, 7),
        (0, 21),
        (0, 104),
        (0, 97),
        (0, 0),
        (0, 4),
        (0, 6),
        (0, 13),
        (0, 4),
        (0, 2344),
        (0, 188),
        (0, 7),
        (0, 31),
        (0, 21),
        (0, 2),
        (0, 694),
        (0, 123),
        (0, 56),
        (0, 1880),
        (0, 0),
        (0, 15),
        (0, 198),
        (0, 0),
        (0, 13),
        (0, 64),
        (0, 0),
        (0, 1),
        (0, 0),
        (0, 114),
        (0, 24),
        (0, 261),
        (0, 0),
        (0, 0),
        (0, 69),
        (0, 4),
        (0, 319),
        (0, 12),
        (0, 65),
        (0, 42),
        (0, 107),
        (0, 48),
        (0, 15),
        (0, 6),
        (0, 178),
        (0, 69),
        (0, 204),
        (0, 13),
        (0, 247),
        (0, 0),
        (0, 2),
        (0, 94),
        (0, 9),
        (0, 95),
        (0, 45),
        (0, 29),
        (0, 2),
        (0, 63),
        (0, 0),
        (0, 7),
        (0, 34),
        (0, 8),
        (0, 3),
        (0, 96),
        (0, 302),
        (0, 155),
        (0, 310),
        (0, 64),
        (0, 5),
        (0, 31),
        (0, 0),
        (0, 2),
        (0, 5),
        (0, 0),
        (0, 123),
        (0, 1),
        (0, 21),
        (0, 1),
        (0, 58),
        (0, 82),
        (0, 6),
        (0, 0),
        (0, 13),
        (0, 75),
        (0, 14),
        (0, 27),
        (0, 1),
        (0, 1),
        (0, 0),
        (0, 47),
        (0, 39),
        (0, 2),
        (0, 97),
        (0, 201),
        (0, 0),
        (0, 4),
        (0, 35),
        (0, 1165),
        (0, 4),
        (0, 33),
        (0, 11),
        (0, 2),
        (0, 16),
        (0, 1),
        (0, 507),
        (0, 16),
        (0, 72),
        (0, 19),
        (0, 24),
        (0, 419),
        (0, 133),
        (0, 158),
        (0, 46),
        (0, 1),
        (0, 8),
        (0, 80),
        (0, 46),
        (0, 0),
        (0, 0),
        (0, 3),
        (0, 11),
        (0, 49),
        (0, 7),
        (0, 0),
        (0, 20),
        (0, 0),
        (0, 102),
        (0, 1),
        (0, 1),
        (0, 44),
        (0, 157),
        (0, 0),
        (0, 10),
        (0, 97),
        (0, 0),
        (0, 33),
        (0, 33),
        (0, 4),
        (0, 1187),
        (0, 619),
        (0, 38),
        (0, 21),
        (0, 0),
        (0, 379),
        (0, 99),
        (0, 28),
        (0, 10),
        (0, 3),
        (0, 56),
        (0, 1),
        (0, 19),
        (0, 0),
        (0, 44),
        (0, 20),
        (0, 280),
        (0, 7),
        (0, 16),
        (0, 33),
        (0, 5),
        (0, 105),
        (0, 9),
        (0, 374),
        (0, 45),
        (0, 1),
        (0, 44),
        (0, 2092),
        (0, 138),
        (0, 0),
        (0, 0),
        (0, 906),
        (0, 53),
        (0, 2),
        (0, 59),
        (0, 173),
        (0, 53),
        (0, 59),
        (0, 96),
        (0, 2),
        (0, 150),
        (0, 85),
        (0, 160),
        (0, 12),
        (0, 0),
        (0, 1312),
        (0, 98),
        (0, 0),
        (0, 247),
        (0, 143),
        (0, 0),
        (0, 1),
        (0, 3),
        (0, 128),
        (0, 13),
        (0, 82),
        (0, 477),
        (0, 3),
        (0, 58),
        (0, 698),
        (0, 51),
        (0, 31),
        (0, 2608),
        (0, 1912),
        (0, 162),
        (0, 0),
        (0, 418),
        (0, 2),
        (0, 0),
        (0, 2397),
        (0, 44),
        (0, 5),
        (0, 63),
        (0, 0),
        (0, 21),
        (0, 1),
        (0, 451),
        (0, 119),
        (0, 60),
        (0, 3),
        (0, 1),
        (0, 0),
        (0, 75),
        (0, 18),
        (0, 85),
        (0, 111),
        (0, 20),
        (0, 106),
        (0, 173),
        (0, 6),
        (0, 106),
        (0, 0),
        (0, 243),
        (0, 493),
        (0, 69),
        (0, 0),
        (0, 101),
        (0, 522),
        (0, 110),
        (0, 41),
        (0, 86),
        (0, 5),
        (0, 19),
        (0, 3),
        (0, 67),
        (0, 108),
        (0, 59),
        (0, 38),
        (0, 0),
        (0, 0),
        (0, 7),
        (0, 16),
        (0, 2213),
        (0, 9),
        (0, 87),
        (0, 84),
        (0, 17),
        (0, 8),
        (0, 24),
        (0, 59),
        (0, 198),
        (0, 229),
        (0, 27),
        (0, 2),
        (0, 39),
        (0, 2),
        (0, 0),
        (0, 0),
        (0, 33),
        (0, 17),
        (0, 1),
        (0, 7),
        (0, 11),
        (0, 73),
        (0, 173),
        (0, 11),
        (0, 1294),
        (0, 207),
        (0, 12),
        (0, 480),
        (0, 408),
        (0, 16),
        (0, 68),
        (0, 53),
        (0, 18),
        (0, 135),
        (0, 913),
        (0, 324),
        (0, 167),
        (0, 1),
        (0, 8),
        (0, 0),
        (0, 228),
        (0, 110),
        (0, 3),
        (0, 67),
        (0, 136),
        (0, 36),
        (0, 1),
        (0, 17),
        (0, 336),
        (0, 56),
        (0, 7),
        (0, 0),
        (0, 70),
        (0, 0),
        (0, 6),
        (0, 0),
        (0, 0),
        (0, 3),
        (0, 63),
        (0, 40),
        (0, 13),
        (0, 190),
        (0, 4),
        (0, 1),
        (0, 75),
        (0, 11),
        (0, 3),
        (0, 8),
        (0, 148),
        (0, 780),
        (0, 17),
        (0, 28),
        (0, 42),
        (0, 19),
        (0, 25),
        (0, 34),
        (0, 0),
        (0, 116),
        (0, 16),
        (0, 21),
        (0, 318),
        (0, 2007),
        (0, 128),
        (0, 3),
        (0, 72),
        (0, 38),
        (0, 261),
        (0, 2),
        (0, 15),
        (0, 195),
        (0, 542),
        (0, 6),
        (0, 26),
        (0, 0),
        (0, 0),
        (0, 28),
        (0, 1),
        (0, 44),
        (0, 9),
        (0, 107),
        (0, 8),
        (0, 116),
        (0, 17),
        (0, 75),
        (0, 316),
        (0, 72),
        (0, 51),
        (0, 327),
        (0, 531),
        (0, 359),
        (0, 159),
        (0, 454),
        (0, 86),
        (0, 743),
        (0, 10),
        (0, 122),
        (0, 0),
        (0, 85),
        (0, 5),
        (0, 6),
        (0, 36),
        (0, 0),
        (0, 2),
        (0, 2206),
        (0, 143),
        (0, 4),
        (0, 10),
        (0, 2388),
        (0, 1),
        (0, 47),
        (0, 58),
        (0, 111),
        (0, 2),
        (0, 1682),
        (0, 0),
        (0, 10),
        (0, 190),
        (0, 573),
        (0, 9),
        (0, 114),
        (0, 16),
        (0, 0),
        (0, 25),
        (0, 23),
        (0, 1),
        (0, 7),
        (0, 0),
        (0, 0),
        (0, 0),
        (0, 516),
        (0, 10),
        (0, 122),
        (0, 25),
        (0, 2),
        (0, 1),
        (0, 5),
        (0, 267),
        (0, 0),
        (0, 13),
        (0, 507),
        (0, 537),
        (0, 0),
        (0, 0),
        (0, 10),
        (0, 2661),
        (0, 8),
        (0, 32),
        (0, 8),
        (0, 0),
        (0, 0),
        (0, 2372),
        (0, 10),
        (0, 2),
        (0, 5),
        (0, 14),
        (0, 1),
        (0, 0),
        (0, 132),
        (0, 97),
        (0, 56),
        (0, 54),
        (0, 137),
        (0, 45),
        (0, 190),
        (0, 42),
        (0, 8),
        (0, 0),
        (0, 5),
        (0, 2220),
        (0, 32),
        (0, 1),
        (0, 14),
        (0, 215),
        (0, 6),
        (0, 38),
        (0, 0),
        (0, 0),
        (0, 2),
        (0, 215),
        (0, 7),
        (0, 14),
        (0, 638),
        (0, 735),
        (0, 23),
        (0, 21),
        (0, 34),
        (0, 0),
        (0, 43),
        (0, 560),
        (0, 0),
        (0, 0),
        (0, 184),
        (0, 2),
        (0, 12),
        (0, 152),
        (0, 37),
        (0, 3),
        (0, 19),
        (0, 59),
        (0, 1),
        (0, 7),
        (0, 0),
        (0, 63),
        (0, 20),
        (0, 0),
        (0, 0),
        (0, 221),
        (0, 0),
        (0, 11),
        (0, 2),
        (0, 0),
        (0, 0),
        (0, 2),
        (0, 267),
        (0, 66),
        (0, 22),
        (0, 17),
        (0, 10),
        (0, 46),
        (0, 944),
        (0, 27),
        (0, 0),
        (0, 2294),
        (0, 0),
        (0, 923),
        (0, 26),
        (0, 0),
        (0, 24),
        (0, 4),
        (0, 184),
        (0, 46),
        (0, 1),
        (0, 28),
        (0, 133),
        (0, 3),
        (0, 61),
        (0, 87),
        (0, 228),
        (0, 12),
        (0, 16),
        (0, 6),
        (0, 122),
        (0, 0),
        (0, 26),
        (0, 0),
        (0, 83),
        (0, 27),
        (0, 15),
        (0, 5),
        (0, 1),
        (0, 1),
        (0, 301),
        (0, 13),
        (0, 11),
        (0, 90),
        (0, 6),
        (0, 76),
        (0, 11),
        (0, 731),
        (0, 47),
        (0, 1),
        (0, 25),
        (0, 11),
        (0, 630),
        (0, 0),
        (0, 179),
        (0, 0),
        (0, 167),
        (0, 17),
        (0, 1),
        (0, 84),
        (0, 14),
        (0, 48),
        (0, 476),
        (0, 229),
        (0, 7),
        (0, 37),
        (0, 1),
        (0, 0),
        (0, 0),
        (0, 2),
        (0, 370),
        (0, 629),
        (0, 36),
        (0, 56),
        (0, 2076),
        (0, 3),
        (0, 2),
        (0, 276),
        (0, 859),
        (0, 242),
        (0, 91),
        (0, 33),
        (0, 1),
        (0, 1136),
        (0, 144),
        (0, 3),
        (0, 3),
        (0, 13),
        (0, 1),
        (0, 34),
        (0, 145),
        (0, 171),
        (0, 98),
        (0, 152),
        (0, 177),
        (0, 1),
        (0, 0),
        (0, 412),
        (0, 30),
        (0, 49),
        (0, 0),
        (0, 5),
        (0, 311),
        (0, 20),
        (0, 2748),
        (0, 91),
        (0, 167),
        (0, 33),
        (0, 99),
        (0, 16),
        (0, 21),
        (0, 157),
        (0, 0),
        (0, 13),
        (0, 1),
        (0, 588),
        (0, 114),
        (0, 6),
        (0, 0),
        (0, 0),
        (0, 36),
        (0, 189),
        (0, 23),
        (0, 7),
        (0, 23),
        (0, 12),
        (0, 69),
        (0, 0),
        (0, 46),
        (0, 0),
        (0, 92),
        (0, 127),
        (0, 3),
        (0, 128),
        (0, 102),
        (0, 253),
        (0, 7),
        (0, 44),
        (0, 1759),
        (0, 714),
        (0, 26),
        (0, 0),
        (0, 632),
        (0, 1),
        (0, 86),
        (0, 110),
        (0, 2),
        (0, 47),
        (0, 581),
        (0, 2537),
        (0, 179),
        (0, 1),
        (0, 0),
        (0, 244),
        (0, 68),
        (0, 2),
        (0, 174),
        (0, 0),
        (0, 311),
        (0, 52),
        (0, 295),
        (0, 1),
        (0, 528),
        (0, 5),
        (0, 4),
        (0, 77),
        (0, 17),
        (0, 323),
        (0, 0),
        (0, 6),
        (0, 0),
        (0, 4),
        (0, 57),
        (0, 226),
        (0, 463),
        (0, 171),
        (0, 864),
        (0, 7),
        (0, 2),
        (0, 36),
        (0, 1),
        (0, 0),
        (0, 2),
        (0, 12),
        (0, 253),
        (0, 8),
        (0, 198),
        (0, 5),
        (0, 6),
        (0, 73),
        (0, 76),
        (0, 25),
        (0, 0),
        (0, 38),
        (0, 55),
        (0, 15),
        (0, 168),
        (0, 18),
        (0, 1369),
        (0, 97),
        (0, 0),
        (0, 382),
        (0, 3),
        (0, 312),
        (0, 44),
        (0, 61),
        (0, 82),
        (0, 4),
        (0, 332),
        (0, 100),
        (0, 0),
        (0, 17),
        (0, 5),
        (0, 423),
        (0, 4),
        (0, 97),
        (0, 2),
        (0, 94),
        (0, 7),
        (0, 0),
        (0, 464),
        (0, 10),
        (0, 39),
        (0, 15),
        (0, 59),
        (0, 5),
        (0, 18),
        (0, 146),
        (0, 2),
        (0, 153),
        (0, 35),
        (0, 0),
        (0, 11),
        (0, 777),
        (0, 45),
        (0, 4),
        (0, 8),
        (0, 1071),
        (0, 4),
        (0, 2),
        (0, 403),
        (0, 372),
        (0, 6),
        (0, 1146),
        (0, 4),
        (0, 1),
        (0, 10),
        (0, 0),
        (0, 0),
        (0, 134),
        (0, 0),
        (0, 115),
        (0, 0),
        (0, 67),
        (0, 8),
        (0, 3),
        (0, 19),
        (0, 9),
        (0, 42),
        (0, 15),
        (0, 15),
        (0, 15),
        (0, 0),
        (0, 7),
        (0, 84),
        (0, 101),
        (0, 214),
        (0, 207),
        (0, 105),
        (0, 160),
        (0, 50),
        (0, 60),
        (0, 2162),
        (0, 395),
        (0, 281),
        (0, 0),
        (0, 117),
        (0, 18),
        (0, 130),
        (0, 567),
        (0, 3),
        (0, 426),
        (0, 55),
        (0, 63),
        (0, 22),
        (0, 0),
        (0, 270),
        (0, 131),
        (0, 56),
        (0, 2041),
        (0, 384),
        (0, 20),
        (0, 6),
        (0, 0),
        (0, 22),
        (0, 152),
        (0, 7),
        (0, 30),
        (0, 7),
        (0, 33),
        (0, 76),
        (0, 77),
        (0, 3),
        (0, 183),
        (0, 141),
        (0, 467),
        (0, 102),
        (0, 37),
        (0, 98),
        (0, 5),
        (0, 38),
        (0, 52),
        (0, 426),
        (0, 66),
        (0, 457),
        (0, 25),
        (0, 131),
        (0, 4),
        (0, 1),
        (0, 42),
        (0, 3),
        (0, 0),
        (0, 29),
        (0, 0),
        (0, 26),
        (0, 126),
        (0, 2),
        (0, 3),
        (0, 31),
        (0, 10),
        (0, 14),
        (0, 11),
        (0, 42),
        (0, 8),
        (0, 0),
        (0, 34),
        (0, 403),
        (0, 26),
        (0, 6),
        (0, 272),
        (0, 1510),
        (0, 143),
        (0, 72),
        (0, 35),
        (0, 10),
        (0, 321),
        (0, 2355),
        (0, 57),
        (0, 34),
        (0, 49),
        (0, 77),
        (0, 2),
        (0, 90),
        (0, 9),
        (0, 36),
        (0, 150),
        (0, 8),
        (0, 855),
        (0, 1),
        (0, 3),
        (0, 4),
        (0, 24),
        (0, 1),
        (0, 176),
        (0, 82),
        (0, 36),
        (0, 20),
        (0, 88),
        (0, 151),
        (0, 170),
        (0, 204),
        (0, 337),
        (0, 1),
        (0, 13),
        (0, 27),
        (0, 461),
        (0, 342),
        (0, 0),
        (0, 203),
        (0, 343),
        (0, 87),
        (0, 24),
        (0, 92),
        (0, 1),
        (0, 11),
        (0, 351),
        (0, 502),
        (0, 0),
        (0, 6),
        (0, 34),
        (0, 19),
        (0, 7),
        (0, 102),
        (0, 62),
        (0, 18),
        (0, 19),
        (0, 7),
        (0, 171),
        (0, 251),
        (0, 178),
        (0, 1595),
        (0, 234),
        (0, 88),
        (0, 0),
        (0, 203),
        (0, 1108),
        (0, 0),
        (0, 197),
        (0, 195),
        (0, 0),
        (0, 65),
        (0, 225),
        (0, 1),
        (0, 1273),
        (0, 1),
        (0, 10),
        (0, 3),
        (0, 387),
        (0, 101),
        (0, 449),
        (0, 2697),
        (0, 9),
        (0, 85),
        (0, 364),
        (0, 55),
        (0, 105),
        (0, 1),
        (0, 0),
        (0, 0),
        (0, 2351),
        (0, 0),
        (0, 32),
        (0, 0),
        (0, 0),
        (0, 10),
        (0, 102),
        (0, 86),
        (0, 15),
        (0, 6),
        (0, 3),
        (0, 191),
        (0, 458),
        (0, 302),
        (0, 0),
        (0, 2),
        (0, 65),
        (0, 4),
        (0, 0),
        (0, 106),
        (0, 190),
        (0, 94),
        (0, 527),
        (0, 122),
        (0, 18),
        (0, 42),
        (0, 61),
        (0, 0),
        (0, 0),
        (0, 333),
        (0, 11),
        (0, 2),
        (0, 183),
        (0, 38),
        (0, 119),
        (0, 32),
        (0, 1144),
        (0, 99),
        (0, 41),
        (0, 1211),
        (0, 31),
        (0, 197),
        (0, 923),
        (0, 4),
        (0, 120),
        (0, 0),
        (0, 13),
        (0, 68),
        (0, 944),
        (0, 0),
        (0, 249),
        (0, 6),
        (0, 1237),
        (0, 4),
        (0, 0),
        (0, 91),
        (0, 4),
        (0, 676),
        (0, 567),
        (0, 6),
        (0, 296),
        (0, 0),
        (0, 95),
        (0, 12),
        (0, 11),
        (0, 29),
        (0, 13),
        (0, 38),
        (0, 161),
        (0, 224),
        (0, 361),
        (0, 34),
        (0, 129),
        (0, 0),
        (0, 0),
        (0, 4),
        (0, 16),
        (0, 96),
        (0, 2871),
        (0, 3051),
        (0, 94),
        (0, 6),
        (0, 916),
        (0, 1),
        (0, 12),
        (0, 0),
        (0, 62),
        (0, 170),
        (0, 13),
        (0, 15),
        (0, 2),
        (0, 29),
        (0, 657),
        (0, 470),
        (0, 11),
        (0, 1761),
        (0, 366),
        (0, 108),
        (0, 178),
        (0, 119),
        (0, 289),
        (0, 0),
        (0, 0),
        (0, 82),
        (0, 408),
        (0, 203),
        (0, 86),
        (0, 1159),
        (0, 34),
        (0, 248),
        (0, 19),
        (0, 1),
        (0, 3),
        (0, 204),
        (0, 0),
        (0, 624),
        (0, 17),
        (0, 45),
        (0, 101),
        (0, 685),
        (0, 41),
        (0, 101),
        (0, 18),
        (0, 91),
        (0, 1773),
        (0, 505),
        (0, 37),
        (0, 120),
        (0, 21),
        (0, 150),
        (0, 212),
        (0, 43),
        (0, 21),
        (0, 2),
        (0, 203),
        (0, 0),
        (0, 28),
        (0, 33),
        (0, 38),
        (0, 291),
        (0, 0),
        (0, 21),
        (0, 0),
        (0, 1256),
        (0, 294),
        (0, 438),
        (0, 7),
        (0, 1),
        (0, 0),
        (0, 23),
        (0, 14),
        (0, 33),
        (0, 1432),
        (0, 0),
        (0, 1),
        (0, 215),
        (0, 54),
        (0, 413),
        (0, 86),
        (0, 0),
        (0, 1953),
        (0, 173),
        (0, 7),
        (0, 0),
        (0, 84),
        (0, 3),
        (0, 105),
        (0, 53),
        (0, 2331),
        (0, 1759),
        (0, 19),
        (0, 2),
        (0, 45),
        (0, 54),
        (0, 2165),
        (0, 172),
        (0, 376),
        (0, 270),
        (0, 97),
        (0, 809),
        (0, 21),
        (0, 0),
        (0, 466),
        (0, 466),
        (0, 78),
        (0, 1712),
        (0, 638),
        (0, 11),
        (0, 4),
        (0, 2119),
        (0, 0),
        (0, 14),
        (0, 1),
        (0, 722),
        (0, 69),
        (0, 1449),
        (0, 122),
        (0, 54),
        (0, 49),
        (0, 3),
        (0, 90),
        (0, 49),
        (0, 15),
        (0, 29),
        (0, 122),
        (0, 779),
        (0, 1116),
        (0, 6),
        (0, 147),
        (0, 22),
        (0, 3),
        (0, 1434),
        (0, 0),
        (0, 0),
        (0, 6),
        (0, 24),
        (0, 309),
        (0, 28),
        (0, 0),
        (0, 0),
        (0, 355),
        (0, 4111),
        (0, 221),
        (0, 28),
        (0, 56),
        (0, 30),
        (0, 0),
        (0, 69),
        (0, 2867),
        (0, 21),
        (0, 50),
        (0, 146),
        (0, 12),
        (0, 17),
        (0, 0),
        (0, 5198),
        (0, 48),
        (0, 440),
        (0, 382),
        (0, 79),
        (0, 0),
        (0, 70),
        (0, 2217),
        (0, 388),
        (0, 141),
        (0, 263),
        (0, 50),
        (0, 4),
        (0, 28),
        (0, 2),
        (0, 229),
        (0, 26),
        (0, 1),
        (0, 371),
        (0, 4),
        (0, 6),
        (0, 0),
        (0, 158),
        (0, 11),
        (0, 190),
        (0, 26),
        (0, 2316),
        (0, 19),
        (0, 6),
        (0, 942),
        (0, 5),
        (0, 4065),
        (0, 9),
        (0, 367),
        (0, 8),
        (0, 1715),
        (0, 55),
        (0, 3535),
        (0, 135),
        (0, 1868),
        (0, 0),
        (0, 28),
        (0, 6),
        (0, 1423),
        (0, 14),
        (0, 3717),
        (0, 541),
        (0, 0),
        (0, 27),
        (0, 585),
        (0, 85),
        (0, 77),
        (0, 41),
        (0, 290),
        (0, 5),
        (0, 1140),
        (0, 0),
        (0, 278),
        (0, 4653),
        (0, 114),
        (0, 2782),
        (0, 7),
        (0, 2367),
        (0, 519),
        (0, 741),
        (0, 1),
        (0, 447),
        (0, 0),
        (0, 3),
        (0, 51),
        (0, 2),
        (0, 3194),
        (0, 41),
        (0, 1),
        (0, 4),
        (0, 45),
        (0, 0),
        (0, 0),
        (0, 16),
        (0, 19),
        (0, 0),
        (0, 11),
        (0, 184),
        (0, 38),
        (0, 2327),
        (0, 269),
        (0, 20),
        (0, 19),
        (0, 335),
        (0, 2228),
        (0, 264),
        (0, 607),
        (0, 11),
        (0, 0),
        (0, 79),
        (0, 0),
        (0, 26),
        (0, 65),
        (0, 1777),
        (0, 447),
        (0, 1900),
        (0, 261),
        (0, 179),
        (0, 5),
        (0, 36),
        (0, 91),
        (0, 210),
        (0, 7),
        (0, 8),
        (0, 0),
        (0, 55),
        (0, 60),
        (0, 50),
        (0, 239),
        (0, 8),
        (0, 1189),
        (0, 213),
        (0, 549),
        (0, 287),
        (0, 238),
        (0, 13),
        (0, 84),
        (0, 611),
        (0, 411),
        (0, 0),
        (0, 4187),
        (0, 22),
        (0, 5237),
        (0, 46),
        (0, 0),
        (0, 0),
        (0, 20),
        (0, 17),
        (0, 3),
        (0, 92),
        (0, 13),
        (0, 277),
        (0, 5),
        (0, 93),
        (0, 1533),
        (0, 0),
        (0, 25),
        (0, 252),
        (0, 7),
        (0, 310),
        (0, 59),
        (0, 19),
        (0, 65),
        (0, 5427),
        (0, 14),
        (0, 54),
        (0, 700),
        (0, 18),
        (0, 99),
        (0, 18),
        (0, 30),
        (0, 43),
        (0, 18),
        (0, 218),
        (0, 203),
        (0, 72),
        (0, 337),
        (0, 270),
        (0, 1421),
        (0, 63),
        (0, 64),
        (0, 6),
        (0, 293),
        (0, 448),
        (0, 0),
        (0, 96),
        (0, 21),
        (0, 2),
        (0, 12),
        (0, 32),
        (0, 1229),
        (0, 4),
        (0, 7),
        (0, 4044),
        (0, 5377),
        (0, 1),
        (0, 132),
        (0, 0),
        (0, 5854),
        (0, 0),
        (0, 1098),
        (0, 42),
        (0, 6),
        (0, 22),
        (0, 335),
        (0, 29),
        (0, 178),
        (0, 15),
        (0, 28),
        (0, 28),
        (0, 0),
        (0, 0),
        (0, 262),
        (0, 42),
        (0, 420),
        (0, 3),
        (0, 7145),
        (0, 67),
        (0, 14),
        (0, 3),
        (0, 41),
        (0, 21),
        (0, 0),
        (0, 4582),
        (0, 21),
        (0, 6993),
        (0, 358),
        (0, 147),
        (0, 26),
        (0, 0),
        (0, 3631),
        (0, 4004),
        (0, 14),
        (0, 83),
        (0, 6133),
        (0, 0),
        (0, 3),
        (0, 113),
        (0, 1),
        (0, 54),
        (0, 7180),
        (0, 7053),
        (0, 108),
        (0, 0),
        (0, 28),
        (0, 86),
        (0, 32),
        (0, 2346),
        (0, 0),
        (0, 949),
        (0, 87),
        (0, 8),
        (0, 0),
        (0, 867),
        (0, 0),
        (0, 88),
        (0, 905),
        (0, 69),
        (0, 881),
        (0, 102),
        (0, 28),
        (0, 0),
        (0, 8141),
        (0, 189),
        (0, 39),
        (0, 4512),
        (0, 37),
        (0, 10313),
        (0, 3),
        (0, 1272),
        (0, 4),
        (0, 10734),
        (0, 163),
        (0, 2461),
        (0, 68),
        (0, 7),
        (0, 1),
        (0, 1),
        (0, 4392),
        (0, 0),
        (0, 20),
        (0, 4537),
        (0, 27),
        (0, 95),
        (0, 10122),
        (0, 0),
        (0, 1034),
        (0, 75),
        (0, 7),
        (0, 7),
        (0, 0),
        (0, 74),
        (0, 4209),
        (0, 311),
        (0, 21),
        (0, 217),
        (0, 1),
        (0, 1046),
        (0, 1239),
        (0, 2),
        (0, 531),
        (0, 7),
        (0, 0),
        (0, 9),
        (0, 11493),
        (0, 4),
        (0, 8540),
        (0, 12),
        (0, 0),
        (0, 363),
        (0, 116),
        (0, 17),
        (0, 41),
        (0, 3369),
        (0, 74),
        (0, 12),
        (0, 51),
        (0, 11552),
        (0, 2652),
        (0, 42),
        (0, 10918),
        (0, 35),
        (0, 290),
        (0, 6941),
        (0, 8125),
        (0, 294),
        (0, 952),
        (0, 170),
        (0, 2),
        (0, 457),
        (0, 4),
        (0, 43),
        (0, 7),
        (0, 0),
        (0, 3959),
        (0, 9),
        (0, 55),
        (0, 450),
        (0, 22),
        (0, 112),
        (0, 795),
        (0, 10),
        (0, 3),
        (0, 3881),
        (0, 30),
        (0, 0),
        (0, 14),
        (0, 64),
        (0, 681),
        (0, 47),
        (0, 822),
        (0, 91),
        (0, 28),
        (0, 62),
        (0, 17),
        (0, 41),
        (0, 188),
        (0, 438),
        (0, 9442),
        (0, 498),
        (0, 17),
        (0, 62),
        (0, 9),
        (0, 153),
        (0, 10699),
        (0, 26),
        (0, 8),
        (0, 848),
        (0, 10926),
        (0, 89),
        (0, 0),
        (0, 324),
        (0, 53),
        (0, 532),
        (0, 7974),
        (0, 16),
        (0, 638),
        (0, 1584),
        (0, 3),
        (0, 0),
        (0, 155),
        (0, 23),
        (0, 2),
        (0, 95),
        (0, 19),
        (0, 84),
    ],
    entries: &[
        (ServiceQuery(6622, Protocol::TCP), Service::Name("mcftp")),
        (ServiceQuery(1493, Protocol::TCP), Service::Name("netmap_lm")),
        (ServiceQuery(4274, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(9996, Protocol::UDP), Service::Name("palace-5")),
        (ServiceQuery(2550, Protocol::TCP), Service::Name("ads")),
        (ServiceQuery(2261, Protocol::UDP), Service::Name("comotionmaster")),
        (ServiceQuery(5432, Protocol::TCP), Service::Name("postgresql")),
        (ServiceQuery(2104, Protocol::TCP), Service::Name("zephyr-hm")),
        (ServiceQuery(873, Protocol::UDP), Service::Name("rsync")),
        (ServiceQuery(600, Protocol::TCP), Service::Name("ipcserver")),
        (ServiceQuery(2276, Protocol::UDP), Service::Name("ibridge-mgmt")),
        (ServiceQuery(9991, Protocol::UDP), Service::Name("osm-oev")),
        (ServiceQuery(5093, Protocol::UDP), Service::Name("sentinel-lm")),
        (ServiceQuery(2027, Protocol::UDP), Service::Name("shadowserver")),
        (ServiceQuery(7734, Protocol::TCP), Service::Name("smip")),
        (ServiceQuery(2901, Protocol::TCP), Service::Name("allstorcns")),
        (ServiceQuery(4838, Protocol::UDP), Service::Name("varadero-1")),
        (ServiceQuery(2705, Protocol::UDP), Service::Name("sds-admin")),
        (ServiceQuery(1662, Protocol::UDP), Service::Name("netview-aix-2")),
        (ServiceQuery(5354, Protocol::TCP), Service::Name("mdnsresponder")),
        (ServiceQuery(221, Protocol::TCP), Service::Name("fln-spx")),
        (ServiceQuery(4136, Protocol::UDP), Service::Name("cl-db-request")),
        (ServiceQuery(771, Protocol::UDP), Service::Name("rtip")),
        (ServiceQuery(4307, Protocol::TCP), Service::Name("visicron-vs")),
        (ServiceQuery(8668, Protocol::TCP), Service::Name("spartan")),
        (ServiceQuery(4837, Protocol::TCP), Service::Name("varadero-0")),
        (ServiceQuery(6007, Protocol::TCP), Service::Name("X11:7")),
        (ServiceQuery(4659, Protocol::UDP), Service::Name("playsta2-lob")),
        (ServiceQuery(3692, Protocol::TCP), Service::Name("intelsync")),
        (ServiceQuery(1478, Protocol::TCP), Service::Name("ms-sna-base")),
        (ServiceQuery(8100, Protocol::TCP), Service::Name("xprint-server")),
        (ServiceQuery(3193, Protocol::TCP), Service::Name("spandataport")),
        (ServiceQuery(2093, Protocol::UDP), Service::Name("nbx-cc")),
        (ServiceQuery(5021, Protocol::TCP), Service::Name("zenginkyo-2")),
        (ServiceQuery(32636, Protocol::UDP), Service::Name("DMExpress")),
        (ServiceQuery(3672, Protocol::UDP), Service::Name("lispworks-orb")),
        (ServiceQuery(831, Protocol::UDP), Service::Name("netconf-beep")),
        (ServiceQuery(2416, Protocol::UDP), Service::Name("rmtserver")),
        (ServiceQuery(1397, Protocol::UDP), Service::Name("audio-activmail")),
        (ServiceQuery(45966, Protocol::TCP), Service::Name("ssr-servermgr")),
        (ServiceQuery(3989, Protocol::TCP), Service::Name("bv-queryengine")),
        (ServiceQuery(1230, Protocol::TCP), Service::Name("periscope")),
        (ServiceQuery(5020, Protocol::UDP), Service::Name("zenginkyo-1")),
        (ServiceQuery(2531, Protocol::UDP), Service::Name("ito-e-gui")),
        (ServiceQuery(6242, Protocol::UDP), Service::Name("jeol-nsddp-2")),
        (ServiceQuery(1349, Protocol::UDP), Service::Name("sbook")),
        (ServiceQuery(1991, Protocol::UDP), Service::Name("stun-p2")),
        (ServiceQuery(11971, Protocol::TCP), Service::Name("tibsd")),
        (ServiceQuery(4899, Protocol::UDP), Service::Name("radmin-port")),
        (ServiceQuery(2335, Protocol::TCP), Service::Name("ace-proxy")),
        (ServiceQuery(3427, Protocol::TCP), Service::Name("websphere-snmp")),
        (ServiceQuery(10201, Protocol::TCP), Service::Name("rsms")),
        (ServiceQuery(2922, Protocol::TCP), Service::Name("cesdcdtrn")),
        (ServiceQuery(6201, Protocol::TCP), Service::Name("thermo-calc")),
        (ServiceQuery(9747, Protocol::UDP), Service::Name("l5nas-parchan")),
        (ServiceQuery(3489, Protocol::TCP), Service::Name("dtp-dia")),
        (ServiceQuery(3407, Protocol::UDP), Service::Name("ldap-admin")),
        (ServiceQuery(1649, Protocol::UDP), Service::Name("kermit")),
        (ServiceQuery(664, Protocol::TCP), Service::Name("secure-aux-bus")),
        (ServiceQuery(1699, Protocol::UDP), Service::Name("rsvp-encap-2")),
        (ServiceQuery(32770, Protocol::UDP), Service::Name("sometimes-rpc4")),
        (ServiceQuery(4661, Protocol::TCP), Service::Name("kar2ouche")),
        (ServiceQuery(3929, Protocol::TCP), Service::Name("smauth-port")),
        (ServiceQuery(3500, Protocol::TCP), Service::Name("rtmp-port")),
        (ServiceQuery(308, Protocol::UDP), Service::Name("novastorbakcup")),
        (ServiceQuery(3456, Protocol::UDP), Service::Name("IISrpc-or-vat")),
        (ServiceQuery(2451, Protocol::TCP), Service::Name("netchat")),
        (ServiceQuery(2533, Protocol::UDP), Service::Name("snifferserver")),
        (ServiceQuery(3161, Protocol::TCP), Service::Name("doc1lm")),
        (ServiceQuery(3789, Protocol::TCP), Service::Name("remotedeploy")),
        (ServiceQuery(1656, Protocol::TCP), Service::Name("dec-mbadmin-h")),
        (ServiceQuery(6602, Protocol::TCP), Service::Name("wsscomfrmwk")),
        (ServiceQuery(1358, Protocol::TCP), Service::Name("connlcli")),
        (ServiceQuery(2038, Protocol::TCP), Service::Name("objectmanager")),
        (ServiceQuery(224, Protocol::UDP), Service::Name("masqdialer")),
        (ServiceQuery(2401, Protocol::TCP), Service::Name("cvspserver")),
        (ServiceQuery(19412, Protocol::UDP), Service::Name("hp-sessmon")),
        (ServiceQuery(12007, Protocol::UDP), Service::Name("accuracer")),
        (ServiceQuery(3664, Protocol::TCP), Service::Name("ups-engine")),
        (ServiceQuery(178, Protocol::UDP), Service::Name("nextstep")),
        (ServiceQuery(5502, Protocol::TCP), Service::Name("fcp-srvr-inst1")),
        (ServiceQuery(412, Protocol::TCP), Service::Name("synoptics-trap")),
        (ServiceQuery(37475, Protocol::TCP), Service::Name("neckar")),
        (ServiceQuery(2262, Protocol::UDP), Service::Name("comotionback")),
        (ServiceQuery(23000, Protocol::UDP), Service::Name("inovaport1")),
        (ServiceQuery(8121, Protocol::UDP), Service::Name("apollo-data")),
        (ServiceQuery(32483, Protocol::UDP), Service::Name("apm-link")),
        (ServiceQuery(3495, Protocol::UDP), Service::Name("seclayer-tcp")),
        (ServiceQuery(1782, Protocol::UDP), Service::Name("hp-hcip")),
        (ServiceQuery(3221, Protocol::UDP), Service::Name("xnm-clear-text")),
        (ServiceQuery(16311, Protocol::UDP), Service::Name("pdefmns")),
        (ServiceQuery(8040, Protocol::UDP), Service::Name("ampify")),
        (ServiceQuery(2555, Protocol::TCP), Service::Name("compaq-wcp")),
        (ServiceQuery(1573, Protocol::UDP), Service::Name("itscomm-ns")),
        (ServiceQuery(366, Protocol::TCP), Service::Name("odmr")),
        (ServiceQuery(4730, Protocol::UDP), Service::Name("gearman")),
        (ServiceQuery(187, Protocol::UDP), Service::Name("aci")),
        (ServiceQuery(29015, Protocol::TCP), Service::Name("rethinkdb")),
        (ServiceQuery(3022, Protocol::TCP), Service::Name("csregagent")),
        (ServiceQuery(14141, Protocol::TCP), Service::Name("bo2k")),
        (ServiceQuery(1524, Protocol::TCP), Service::Name("ingreslock")),
        (ServiceQuery(2481, Protocol::TCP), Service::Name("giop")),
        (ServiceQuery(6405, Protocol::TCP), Service::Name("boe-pagesvr")),
        (ServiceQuery(3125, Protocol::TCP), Service::Name("a13-an")),
        (ServiceQuery(3555, Protocol::TCP), Service::Name("razor")),
        (ServiceQuery(9346, Protocol::UDP), Service::Name("ctechlicensing")),
        (ServiceQuery(13722, Protocol::TCP), Service::Name("netbackup")),
        (ServiceQuery(2755, Protocol::UDP), Service::Name("expresspay")),
        (ServiceQuery(4248, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(5318, Protocol::TCP), Service::Name("pkix-cmc")),
        (ServiceQuery(7913, Protocol::UDP), Service::Name("qo-secure")),
        (ServiceQuery(9888, Protocol::UDP), Service::Name("cyborg-systems")),
        (ServiceQuery(3024, Protocol::TCP), Service::Name("nds_sso")),
        (ServiceQuery(7281, Protocol::TCP), Service::Name("itactionserver2")),
        (ServiceQuery(3686, Protocol::UDP), Service::Name("tnmpv2")),
        (ServiceQuery(3511, Protocol::UDP), Service::Name("webmail-2")),
        (ServiceQuery(7743, Protocol::UDP), Service::Name("sstp-1")),
        (ServiceQuery(6111, Protocol::UDP), Service::Name("spc")),
        (ServiceQuery(3324, Protocol::TCP), Service::Name("active-net")),
        (ServiceQuery(2030, Protocol::TCP), Service::Name("device2")),
        (ServiceQuery(3031, Protocol::TCP), Service::Name("eppc")),
        (ServiceQuery(5504, Protocol::TCP), Service::Name("fcp-cics-gw1")),
        (ServiceQuery(1537, Protocol::TCP), Service::Name("sdsc-lm")),
        (ServiceQuery(25900, Protocol::TCP), Service::Name("tasp-net")),
        (ServiceQuery(3989, Protocol::UDP), Service::Name("bv-queryengine")),
        (ServiceQuery(73, Protocol::UDP), Service::Name("netrjs-3")),
        (ServiceQuery(2834, Protocol::TCP), Service::Name("evtp")),
        (ServiceQuery(2298, Protocol::UDP), Service::Name("d2k-datamover2")),
        (ServiceQuery(1501, Protocol::TCP), Service::Name("sas-3")),
        (ServiceQuery(10541, Protocol::TCP), Service::Name("MOS-upper")),
        (ServiceQuery(4284, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1657, Protocol::UDP), Service::Name("fujitsu-mmpdc")),
        (ServiceQuery(990, Protocol::TCP), Service::Name("ftps")),
        (ServiceQuery(3864, Protocol::TCP), Service::Name("asap-tcp-tls")),
        (ServiceQuery(641, Protocol::UDP), Service::Name("repcmd")),
        (ServiceQuery(1044, Protocol::TCP), Service::Name("dcutility")),
        (ServiceQuery(221, Protocol::UDP), Service::Name("fln-spx")),
        (ServiceQuery(3716, Protocol::TCP), Service::Name("wv-csp-sms-cir")),
        (ServiceQuery(3675, Protocol::TCP), Service::Name("calltrax")),
        (ServiceQuery(3188, Protocol::UDP), Service::Name("brcm-comm-port")),
        (ServiceQuery(13224, Protocol::TCP), Service::Name("powwow-server")),
        (ServiceQuery(16385, Protocol::TCP), Service::Name("rdgs")),
        (ServiceQuery(2739, Protocol::TCP), Service::Name("tn-timing")),
        (ServiceQuery(5103, Protocol::TCP), Service::Name("actifio-c2c")),
        (ServiceQuery(13160, Protocol::UDP), Service::Name("i-zipqd")),
        (ServiceQuery(580, Protocol::TCP), Service::Name("sntp-heartbeat")),
        (ServiceQuery(113, Protocol::TCP), Service::Name("ident")),
        (ServiceQuery(8276, Protocol::TCP), Service::Name("pando-sec")),
        (ServiceQuery(1992, Protocol::TCP), Service::Name("stun-p3")),
        (ServiceQuery(634, Protocol::UDP), Service::Name("ginad")),
        (ServiceQuery(5730, Protocol::TCP), Service::Name("unieng")),
        (ServiceQuery(34964, Protocol::UDP), Service::Name("profinet-cm")),
        (ServiceQuery(116, Protocol::TCP), Service::Name("ansanotify")),
        (ServiceQuery(1847, Protocol::TCP), Service::Name("slp-notify")),
        (ServiceQuery(7392, Protocol::TCP), Service::Name("mrssrendezvous")),
        (ServiceQuery(8351, Protocol::UDP), Service::Name("server-find")),
        (ServiceQuery(8087, Protocol::TCP), Service::Name("simplifymedia")),
        (ServiceQuery(3350, Protocol::UDP), Service::Name("findviatv")),
        (ServiceQuery(18186, Protocol::TCP), Service::Name("ohsc")),
        (ServiceQuery(1590, Protocol::TCP), Service::Name("gemini-lm")),
        (ServiceQuery(3786, Protocol::TCP), Service::Name("upstriggervsw")),
        (ServiceQuery(3813, Protocol::TCP), Service::Name("rap-ip")),
        (ServiceQuery(2166, Protocol::TCP), Service::Name("iwserver")),
        (ServiceQuery(1655, Protocol::UDP), Service::Name("dec-mbadmin")),
        (ServiceQuery(5054, Protocol::TCP), Service::Name("rlm-admin")),
        (ServiceQuery(1876, Protocol::UDP), Service::Name("ewcappsrv")),
        (ServiceQuery(5228, Protocol::TCP), Service::Name("hpvroom")),
        (ServiceQuery(3029, Protocol::TCP), Service::Name("LiebDevMgmt_A")),
        (ServiceQuery(3061, Protocol::UDP), Service::Name("cautcpd")),
        (ServiceQuery(5116, Protocol::TCP), Service::Name("emb-proj-cmd")),
        (ServiceQuery(4991, Protocol::TCP), Service::Name("vrt")),
        (ServiceQuery(5743, Protocol::UDP), Service::Name("watchdoc-pod")),
        (ServiceQuery(2517, Protocol::UDP), Service::Name("call-sig-trans")),
        (ServiceQuery(11110, Protocol::TCP), Service::Name("sgi-soap")),
        (ServiceQuery(5455, Protocol::UDP), Service::Name("apc-5455")),
        (ServiceQuery(1504, Protocol::UDP), Service::Name("evb-elm")),
        (ServiceQuery(401, Protocol::TCP), Service::Name("ups")),
        (ServiceQuery(2441, Protocol::TCP), Service::Name("pvsw-inet")),
        (ServiceQuery(1910, Protocol::UDP), Service::Name("ultrabac")),
        (ServiceQuery(9992, Protocol::TCP), Service::Name("issc")),
        (ServiceQuery(478, Protocol::UDP), Service::Name("spsc")),
        (ServiceQuery(12013, Protocol::UDP), Service::Name("vipera-ssl")),
        (ServiceQuery(2398, Protocol::TCP), Service::Name("orbiter")),
        (ServiceQuery(5299, Protocol::TCP), Service::Name("nlg-data")),
        (ServiceQuery(32777, Protocol::TCP), Service::Name("sometimes-rpc17")),
        (ServiceQuery(3544, Protocol::TCP), Service::Name("teredo")),
        (ServiceQuery(704, Protocol::UDP), Service::Name("elcsd")),
        (ServiceQuery(16309, Protocol::UDP), Service::Name("etb4j")),
        (ServiceQuery(6665, Protocol::UDP), Service::Name("ircu")),
        (ServiceQuery(9535, Protocol::UDP), Service::Name("man")),
        (ServiceQuery(1599, Protocol::TCP), Service::Name("simbaservices")),
        (ServiceQuery(1954, Protocol::TCP), Service::Name("abr-api")),
        (ServiceQuery(541, Protocol::TCP), Service::Name("uucp-rlogin")),
        (ServiceQuery(2264, Protocol::UDP), Service::Name("apx500api-1")),
        (ServiceQuery(2941, Protocol::TCP), Service::Name("sm-pas-4")),
        (ServiceQuery(4038, Protocol::TCP), Service::Name("fazzt-ptp")),
        (ServiceQuery(2585, Protocol::UDP), Service::Name("netx-server")),
        (ServiceQuery(1796, Protocol::UDP), Service::Name("vocaltec-admin")),
        (ServiceQuery(1218, Protocol::UDP), Service::Name("aeroflight-ads")),
        (ServiceQuery(586, Protocol::UDP), Service::Name("password-chg")),
        (ServiceQuery(8084, Protocol::TCP), Service::Name("websnp")),
        (ServiceQuery(7426, Protocol::UDP), Service::Name("pmdmgr")),
        (ServiceQuery(713, Protocol::TCP), Service::Name("iris-xpc")),
        (ServiceQuery(832, Protocol::UDP), Service::Name("netconfsoaphttp")),
        (ServiceQuery(1339, Protocol::TCP), Service::Name("kjtsiteserver")),
        (ServiceQuery(5056, Protocol::TCP), Service::Name("intecom-ps1")),
        (ServiceQuery(1597, Protocol::TCP), Service::Name("orbplus-iiop")),
        (ServiceQuery(18634, Protocol::TCP), Service::Name("rds-ib")),
        (ServiceQuery(3106, Protocol::UDP), Service::Name("cardbox-http")),
        (ServiceQuery(862, Protocol::TCP), Service::Name("twamp-control")),
        (ServiceQuery(2411, Protocol::TCP), Service::Name("netwave-ap-mgmt")),
        (ServiceQuery(8017, Protocol::TCP), Service::Name("cisco-cloudsec")),
        (ServiceQuery(1374, Protocol::TCP), Service::Name("molly")),
        (ServiceQuery(6420, Protocol::TCP), Service::Name("nim-vdrshell")),
        (ServiceQuery(471, Protocol::UDP), Service::Name("mondex")),
        (ServiceQuery(3083, Protocol::UDP), Service::Name("tl1-telnet")),
        (ServiceQuery(679, Protocol::TCP), Service::Name("mrm")),
        (ServiceQuery(8883, Protocol::UDP), Service::Name("secure-mqtt")),
        (ServiceQuery(3838, Protocol::TCP), Service::Name("sos")),
        (ServiceQuery(3654, Protocol::UDP), Service::Name("vaprtm")),
        (ServiceQuery(2289, Protocol::UDP), Service::Name("dict-lookup")),
        (ServiceQuery(216, Protocol::UDP), Service::Name("atls")),
        (ServiceQuery(7887, Protocol::UDP), Service::Name("ubroker")),
        (ServiceQuery(1214, Protocol::TCP), Service::Name("fasttrack")),
        (ServiceQuery(11001, Protocol::TCP), Service::Name("metasys")),
        (ServiceQuery(1267, Protocol::UDP), Service::Name("epc")),
        (ServiceQuery(2680, Protocol::UDP), Service::Name("pxc-sapxom")),
        (ServiceQuery(3647, Protocol::TCP), Service::Name("splitlock-gw")),
        (ServiceQuery(3894, Protocol::UDP), Service::Name("syam-agent")),
        (ServiceQuery(3863, Protocol::TCP), Service::Name("asap-tcp")),
        (ServiceQuery(4455, Protocol::UDP), Service::Name("prchat-user")),
        (ServiceQuery(7683, Protocol::TCP), Service::Name("dmt")),
        (ServiceQuery(8102, Protocol::TCP), Service::Name("kz-migr")),
        (ServiceQuery(1852, Protocol::TCP), Service::Name("virtual-time")),
        (ServiceQuery(2998, Protocol::UDP), Service::Name("realsecure")),
        (ServiceQuery(6110, Protocol::UDP), Service::Name("softcm")),
        (ServiceQuery(1588, Protocol::TCP), Service::Name("triquest-lm")),
        (ServiceQuery(609, Protocol::TCP), Service::Name("npmp-trap")),
        (ServiceQuery(1971, Protocol::UDP), Service::Name("netop-school")),
        (ServiceQuery(2197, Protocol::TCP), Service::Name("mnp-exchange")),
        (ServiceQuery(3346, Protocol::UDP), Service::Name("trnsprntproxy")),
        (ServiceQuery(637, Protocol::TCP), Service::Name("lanserver")),
        (ServiceQuery(23456, Protocol::TCP), Service::Name("aequus")),
        (ServiceQuery(3554, Protocol::TCP), Service::Name("questnotify")),
        (ServiceQuery(4142, Protocol::TCP), Service::Name("oidocsvc")),
        (ServiceQuery(5397, Protocol::TCP), Service::Name("stresstester")),
        (ServiceQuery(8472, Protocol::UDP), Service::Name("otv")),
        (ServiceQuery(2468, Protocol::UDP), Service::Name("qip-msgd")),
        (ServiceQuery(1578, Protocol::UDP), Service::Name("jacobus-lm")),
        (ServiceQuery(7794, Protocol::TCP), Service::Name("q3ade")),
        (ServiceQuery(1623, Protocol::UDP), Service::Name("jaleosnd")),
        (ServiceQuery(3802, Protocol::TCP), Service::Name("vhd")),
        (ServiceQuery(9389, Protocol::TCP), Service::Name("adws")),
        (ServiceQuery(48556, Protocol::TCP), Service::Name("com-bardac-dw")),
        (ServiceQuery(98, Protocol::UDP), Service::Name("tacnews")),
        (ServiceQuery(6024, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(1260, Protocol::TCP), Service::Name("ibm-ssd")),
        (ServiceQuery(552, Protocol::TCP), Service::Name("deviceshare")),
        (ServiceQuery(2995, Protocol::TCP), Service::Name("idrs")),
        (ServiceQuery(3999, Protocol::TCP), Service::Name("remoteanything")),
        (ServiceQuery(3062, Protocol::UDP), Service::Name("ncacn-ip-tcp")),
        (ServiceQuery(3308, Protocol::UDP), Service::Name("tns-server")),
        (ServiceQuery(8007, Protocol::TCP), Service::Name("ajp12")),
        (ServiceQuery(1526, Protocol::TCP), Service::Name("pdap-np")),
        (ServiceQuery(1104, Protocol::TCP), Service::Name("xrl")),
        (ServiceQuery(8881, Protocol::TCP), Service::Name("galaxy4d")),
        (ServiceQuery(21, Protocol::TCP), Service::Name("ftp")),
        (ServiceQuery(2627, Protocol::UDP), Service::Name("webster")),
        (ServiceQuery(482, Protocol::UDP), Service::Name("xlog")),
        (ServiceQuery(5813, Protocol::UDP), Service::Name("icmpd")),
        (ServiceQuery(460, Protocol::TCP), Service::Name("skronk")),
        (ServiceQuery(6001, Protocol::UDP), Service::Name("X11:1")),
        (ServiceQuery(1562, Protocol::UDP), Service::Name("pconnectmgr")),
        (ServiceQuery(259, Protocol::UDP), Service::Name("firewall1-rdp")),
        (ServiceQuery(3215, Protocol::TCP), Service::Name("jmq-daemon-2")),
        (ServiceQuery(56, Protocol::UDP), Service::Name("xns-auth")),
        (ServiceQuery(4114, Protocol::TCP), Service::Name("jomamqmonitor")),
        (ServiceQuery(1389, Protocol::UDP), Service::Name("iclpv-dm")),
        (ServiceQuery(2367, Protocol::TCP), Service::Name("service-ctrl")),
        (ServiceQuery(6508, Protocol::UDP), Service::Name("bdir_pub")),
        (ServiceQuery(8115, Protocol::TCP), Service::Name("mtl8000-matrix")),
        (ServiceQuery(4377, Protocol::UDP), Service::Name("cp-spxsvr")),
        (ServiceQuery(1388, Protocol::UDP), Service::Name("objective-dbc")),
        (ServiceQuery(574, Protocol::TCP), Service::Name("ftp-agent")),
        (ServiceQuery(388, Protocol::TCP), Service::Name("unidata-ldm")),
        (ServiceQuery(9998, Protocol::TCP), Service::Name("distinct32")),
        (ServiceQuery(9667, Protocol::UDP), Service::Name("xmms2")),
        (ServiceQuery(646, Protocol::UDP), Service::Name("ldp")),
        (ServiceQuery(3153, Protocol::UDP), Service::Name("s8-client-port")),
        (ServiceQuery(1216, Protocol::TCP), Service::Name("etebac5")),
        (ServiceQuery(2954, Protocol::TCP), Service::Name("ovalarmsrv-cmd")),
        (ServiceQuery(22001, Protocol::TCP), Service::Name("optocontrol")),
        (ServiceQuery(53, Protocol::TCP), Service::Name("domain")),
        (ServiceQuery(3945, Protocol::UDP), Service::Name("emcads")),
        (ServiceQuery(20002, Protocol::TCP), Service::Name("commtact-http")),
        (ServiceQuery(5026, Protocol::UDP), Service::Name("strexec-d")),
        (ServiceQuery(374, Protocol::TCP), Service::Name("legent-2")),
        (ServiceQuery(3829, Protocol::TCP), Service::Name("neteh-ext")),
        (ServiceQuery(9388, Protocol::TCP), Service::Name("d2ddatatrans")),
        (ServiceQuery(11162, Protocol::UDP), Service::Name("suncacao-jmxmp")),
        (ServiceQuery(6791, Protocol::UDP), Service::Name("hnm")),
        (ServiceQuery(2080, Protocol::UDP), Service::Name("autodesk-nlm")),
        (ServiceQuery(4280, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1242, Protocol::TCP), Service::Name("nmasoverip")),
        (ServiceQuery(3271, Protocol::TCP), Service::Name("csoft-prev")),
        (ServiceQuery(4431, Protocol::TCP), Service::Name("wspipe")),
        (ServiceQuery(16900, Protocol::TCP), Service::Name("newbay-snc-mc")),
        (ServiceQuery(2690, Protocol::UDP), Service::Name("hp-nnm-data")),
        (ServiceQuery(25902, Protocol::UDP), Service::Name("nilinkanalyst")),
        (ServiceQuery(5027, Protocol::TCP), Service::Name("strexec-s")),
        (ServiceQuery(7574, Protocol::TCP), Service::Name("coherence")),
        (ServiceQuery(1160, Protocol::TCP), Service::Name("olsv")),
        (ServiceQuery(3744, Protocol::TCP), Service::Name("sasg")),
        (ServiceQuery(5997, Protocol::TCP), Service::Name("ncd-pref")),
        (ServiceQuery(2046, Protocol::TCP), Service::Name("sdfunc")),
        (ServiceQuery(1778, Protocol::TCP), Service::Name("prodigy-intrnet")),
        (ServiceQuery(1275, Protocol::TCP), Service::Name("ivcollector")),
        (ServiceQuery(782, Protocol::TCP), Service::Name("hp-managed-node")),
        (ServiceQuery(9444, Protocol::TCP), Service::Name("wso2esb-console")),
        (ServiceQuery(6015, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(3818, Protocol::TCP), Service::Name("crinis-hb")),
        (ServiceQuery(5321, Protocol::TCP), Service::Name("bsfsvr-zn-ssl")),
        (ServiceQuery(1799, Protocol::UDP), Service::Name("netrisk")),
        (ServiceQuery(500, Protocol::UDP), Service::Name("isakmp")),
        (ServiceQuery(8019, Protocol::TCP), Service::Name("qbdb")),
        (ServiceQuery(33, Protocol::TCP), Service::Name("dsp")),
        (ServiceQuery(3367, Protocol::TCP), Service::Name("satvid-datalnk")),
        (ServiceQuery(32034, Protocol::UDP), Service::Name("iracinghelper")),
        (ServiceQuery(46998, Protocol::TCP), Service::Name("spremotetablet")),
        (ServiceQuery(4122, Protocol::UDP), Service::Name("fprams")),
        (ServiceQuery(3338, Protocol::TCP), Service::Name("anet-b")),
        (ServiceQuery(1681, Protocol::TCP), Service::Name("sd-elmd")),
        (ServiceQuery(1090, Protocol::TCP), Service::Name("ff-fms")),
        (ServiceQuery(376, Protocol::UDP), Service::Name("nip")),
        (ServiceQuery(3933, Protocol::TCP), Service::Name("plbserve-port")),
        (ServiceQuery(3616, Protocol::TCP), Service::Name("cd3o-protocol")),
        (ServiceQuery(2319, Protocol::UDP), Service::Name("infolibria")),
        (ServiceQuery(2754, Protocol::UDP), Service::Name("apollo-cc")),
        (ServiceQuery(3069, Protocol::UDP), Service::Name("ls3")),
        (ServiceQuery(6442, Protocol::TCP), Service::Name("tarp")),
        (ServiceQuery(2209, Protocol::TCP), Service::Name("rimf-ps")),
        (ServiceQuery(1668, Protocol::TCP), Service::Name("netview-aix-8")),
        (ServiceQuery(578, Protocol::TCP), Service::Name("ipdd")),
        (ServiceQuery(5605, Protocol::UDP), Service::Name("a4-sdunode")),
        (ServiceQuery(3530, Protocol::TCP), Service::Name("gf")),
        (ServiceQuery(1646, Protocol::UDP), Service::Name("radacct")),
        (ServiceQuery(3628, Protocol::TCP), Service::Name("ept-machine")),
        (ServiceQuery(4302, Protocol::UDP), Service::Name("d-data-control")),
        (ServiceQuery(6831, Protocol::UDP), Service::Name("ambit-lm")),
        (ServiceQuery(1323, Protocol::UDP), Service::Name("brcd")),
        (ServiceQuery(497, Protocol::TCP), Service::Name("retrospect")),
        (ServiceQuery(3246, Protocol::UDP), Service::Name("kademlia")),
        (ServiceQuery(730, Protocol::TCP), Service::Name("netviewdm2")),
        (ServiceQuery(2328, Protocol::UDP), Service::Name("netrix-sftm")),
        (ServiceQuery(8101, Protocol::TCP), Service::Name("ldoms-migr")),
        (ServiceQuery(22000, Protocol::TCP), Service::Name("snapenetio")),
        (ServiceQuery(4140, Protocol::UDP), Service::Name("cedros_fds")),
        (ServiceQuery(188, Protocol::TCP), Service::Name("mumps")),
        (ServiceQuery(2951, Protocol::TCP), Service::Name("ottp")),
        (ServiceQuery(2674, Protocol::TCP), Service::Name("ewnn")),
        (ServiceQuery(4265, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(24554, Protocol::TCP), Service::Name("binkp")),
        (ServiceQuery(5705, Protocol::TCP), Service::Name("storageos")),
        (ServiceQuery(11967, Protocol::UDP), Service::Name("sysinfo-sp")),
        (ServiceQuery(3093, Protocol::TCP), Service::Name("rapidmq-center")),
        (ServiceQuery(21213, Protocol::TCP), Service::Name("cohesity-agent")),
        (ServiceQuery(17500, Protocol::TCP), Service::Name("db-lsp")),
        (ServiceQuery(5081, Protocol::TCP), Service::Name("sdl-ets")),
        (ServiceQuery(270, Protocol::TCP), Service::Name("gist")),
        (ServiceQuery(1377, Protocol::TCP), Service::Name("cichlid")),
        (ServiceQuery(3983, Protocol::TCP), Service::Name("eisp")),
        (ServiceQuery(11001, Protocol::UDP), Service::Name("metasys")),
        (ServiceQuery(5074, Protocol::TCP), Service::Name("alesquery")),
        (ServiceQuery(1570, Protocol::UDP), Service::Name("orbixd")),
        (ServiceQuery(2160, Protocol::UDP), Service::Name("apc-2160")),
        (ServiceQuery(8787, Protocol::TCP), Service::Name("msgsrvr")),
        (ServiceQuery(24666, Protocol::TCP), Service::Name("sdtvwcam")),
        (ServiceQuery(7099, Protocol::TCP), Service::Name("lazy-ptop")),
        (ServiceQuery(4227, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1246, Protocol::UDP), Service::Name("payrouter")),
        (ServiceQuery(3808, Protocol::TCP), Service::Name("sun-as-iiops-ca")),
        (ServiceQuery(16368, Protocol::TCP), Service::Name("netserialext4")),
        (ServiceQuery(4425, Protocol::UDP), Service::Name("netrockey6")),
        (ServiceQuery(1285, Protocol::TCP), Service::Name("neoiface")),
        (ServiceQuery(2979, Protocol::TCP), Service::Name("h263-video")),
        (ServiceQuery(2051, Protocol::UDP), Service::Name("epnsdp")),
        (ServiceQuery(6626, Protocol::TCP), Service::Name("wago-service")),
        (ServiceQuery(1789, Protocol::TCP), Service::Name("hello")),
        (ServiceQuery(4349, Protocol::TCP), Service::Name("fsportmap")),
        (ServiceQuery(47806, Protocol::UDP), Service::Name("ap")),
        (ServiceQuery(136, Protocol::TCP), Service::Name("profile")),
        (ServiceQuery(3494, Protocol::UDP), Service::Name("ibm3494")),
        (ServiceQuery(2354, Protocol::TCP), Service::Name("psprserver")),
        (ServiceQuery(22370, Protocol::UDP), Service::Name("hpnpd")),
        (ServiceQuery(517, Protocol::UDP), Service::Name("talk")),
        (ServiceQuery(1240, Protocol::TCP), Service::Name("instantia")),
        (ServiceQuery(23002, Protocol::UDP), Service::Name("inovaport3")),
        (ServiceQuery(3467, Protocol::UDP), Service::Name("rcst")),
        (ServiceQuery(6697, Protocol::TCP), Service::Name("ircs-u")),
        (ServiceQuery(1034, Protocol::TCP), Service::Name("zincite-a")),
        (ServiceQuery(3286, Protocol::TCP), Service::Name("e-net")),
        (ServiceQuery(5566, Protocol::TCP), Service::Name("westec-connect")),
        (ServiceQuery(18333, Protocol::TCP), Service::Name("bitcoin")),
        (ServiceQuery(6432, Protocol::TCP), Service::Name("pgbouncer")),
        (ServiceQuery(2174, Protocol::UDP), Service::Name("msfw-array")),
        (ServiceQuery(3375, Protocol::TCP), Service::Name("vsnm-agent")),
        (ServiceQuery(3859, Protocol::TCP), Service::Name("nav-port")),
        (ServiceQuery(2392, Protocol::TCP), Service::Name("tacticalauth")),
        (ServiceQuery(6703, Protocol::TCP), Service::Name("e-design-web")),
        (ServiceQuery(26208, Protocol::TCP), Service::Name("wnn6_DS")),
        (ServiceQuery(3585, Protocol::UDP), Service::Name("emprise-lls")),
        (ServiceQuery(23294, Protocol::TCP), Service::Name("5afe-dir")),
        (ServiceQuery(2073, Protocol::UDP), Service::Name("gxs-data-port")),
        (ServiceQuery(20222, Protocol::UDP), Service::Name("ipulse-ics")),
        (ServiceQuery(85, Protocol::TCP), Service::Name("mit-ml-dev")),
        (ServiceQuery(1889, Protocol::UDP), Service::Name("unify-adapter")),
        (ServiceQuery(2676, Protocol::TCP), Service::Name("simslink")),
        (ServiceQuery(7491, Protocol::UDP), Service::Name("telops-lmd")),
        (ServiceQuery(2748, Protocol::TCP), Service::Name("fjippol-polsvr")),
        (ServiceQuery(10103, Protocol::TCP), Service::Name("ezrelay")),
        (ServiceQuery(5154, Protocol::UDP), Service::Name("bzflag")),
        (ServiceQuery(5684, Protocol::UDP), Service::Name("coaps")),
        (ServiceQuery(2528, Protocol::TCP), Service::Name("ncr_ccl")),
        (ServiceQuery(4254, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(5358, Protocol::UDP), Service::Name("wsdapi-s")),
        (ServiceQuery(7129, Protocol::UDP), Service::Name("scenccs")),
        (ServiceQuery(3950, Protocol::UDP), Service::Name("namemunge")),
        (ServiceQuery(1935, Protocol::TCP), Service::Name("rtmp")),
        (ServiceQuery(7391, Protocol::UDP), Service::Name("mindfilesys")),
        (ServiceQuery(2771, Protocol::UDP), Service::Name("vergencecm")),
        (ServiceQuery(4737, Protocol::UDP), Service::Name("ipdr-sp")),
        (ServiceQuery(7088, Protocol::TCP), Service::Name("zixi-transport")),
        (ServiceQuery(6785, Protocol::TCP), Service::Name("dgpf-exchg")),
        (ServiceQuery(1525, Protocol::UDP), Service::Name("oracle")),
        (ServiceQuery(3258, Protocol::TCP), Service::Name("ivecon-port")),
        (ServiceQuery(1692, Protocol::UDP), Service::Name("sstsys-lm")),
        (ServiceQuery(2956, Protocol::UDP), Service::Name("ovrimosdbman")),
        (ServiceQuery(7395, Protocol::UDP), Service::Name("winqedit")),
        (ServiceQuery(8383, Protocol::UDP), Service::Name("m2mservices")),
        (ServiceQuery(3730, Protocol::TCP), Service::Name("client-ctrl")),
        (ServiceQuery(1337, Protocol::UDP), Service::Name("menandmice-dns")),
        (ServiceQuery(4130, Protocol::UDP), Service::Name("fronet")),
        (ServiceQuery(6842, Protocol::TCP), Service::Name("netmo-http")),
        (ServiceQuery(3308, Protocol::TCP), Service::Name("tns-server")),
        (ServiceQuery(3397, Protocol::UDP), Service::Name("cloanto-lm")),
        (ServiceQuery(2083, Protocol::UDP), Service::Name("radsec")),
        (ServiceQuery(8313, Protocol::TCP), Service::Name("hub-open-net")),
        (ServiceQuery(3142, Protocol::TCP), Service::Name("apt-cacher")),
        (ServiceQuery(2331, Protocol::UDP), Service::Name("agentview")),
        (ServiceQuery(19788, Protocol::TCP), Service::Name("mle")),
        (ServiceQuery(3335, Protocol::TCP), Service::Name("directv-soft")),
        (ServiceQuery(1926, Protocol::UDP), Service::Name("egs")),
        (ServiceQuery(10001, Protocol::UDP), Service::Name("scp-config")),
        (ServiceQuery(6042, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(10004, Protocol::TCP), Service::Name("emcrmirccd")),
        (ServiceQuery(4348, Protocol::UDP), Service::Name("itose")),
        (ServiceQuery(13400, Protocol::TCP), Service::Name("doip-data")),
        (ServiceQuery(311, Protocol::UDP), Service::Name("asip-webadmin")),
        (ServiceQuery(2108, Protocol::TCP), Service::Name("rkinit")),
        (ServiceQuery(1425, Protocol::UDP), Service::Name("zion-lm")),
        (ServiceQuery(8056, Protocol::UDP), Service::Name("senomix05")),
        (ServiceQuery(8118, Protocol::UDP), Service::Name("privoxy")),
        (ServiceQuery(2681, Protocol::UDP), Service::Name("mpnjsomb")),
        (ServiceQuery(1460, Protocol::UDP), Service::Name("proshare2")),
        (ServiceQuery(5235, Protocol::UDP), Service::Name("galaxy-network")),
        (ServiceQuery(1133, Protocol::UDP), Service::Name("dfn")),
        (ServiceQuery(6500, Protocol::UDP), Service::Name("boks")),
        (ServiceQuery(3036, Protocol::TCP), Service::Name("hagel-dump")),
        (ServiceQuery(1920, Protocol::TCP), Service::Name("can-ferret")),
        (ServiceQuery(7847, Protocol::TCP), Service::Name("csoauth")),
        (ServiceQuery(8733, Protocol::TCP), Service::Name("ibus")),
        (ServiceQuery(7799, Protocol::TCP), Service::Name("altbsdp")),
        (ServiceQuery(3289, Protocol::UDP), Service::Name("enpc")),
        (ServiceQuery(7798, Protocol::UDP), Service::Name("pnet-enc")),
        (ServiceQuery(7279, Protocol::TCP), Service::Name("ctxlic")),
        (ServiceQuery(6322, Protocol::UDP), Service::Name("emp-server2")),
        (ServiceQuery(6672, Protocol::UDP), Service::Name("vision_server")),
        (ServiceQuery(3006, Protocol::UDP), Service::Name("ii-admin")),
        (ServiceQuery(5407, Protocol::TCP), Service::Name("foresyte-clear")),
        (ServiceQuery(4237, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(7631, Protocol::TCP), Service::Name("tesla-sys-msg")),
        (ServiceQuery(3838, Protocol::UDP), Service::Name("sos")),
        (ServiceQuery(2210, Protocol::UDP), Service::Name("noaaport")),
        (ServiceQuery(5086, Protocol::TCP), Service::Name("aprigo-cs")),
        (ServiceQuery(576, Protocol::UDP), Service::Name("ipcd")),
        (ServiceQuery(2843, Protocol::TCP), Service::Name("pdnet")),
        (ServiceQuery(1587, Protocol::UDP), Service::Name("pra_elmd")),
        (ServiceQuery(1846, Protocol::TCP), Service::Name("tunstall-pnc")),
        (ServiceQuery(3588, Protocol::TCP), Service::Name("sentinel")),
        (ServiceQuery(3578, Protocol::UDP), Service::Name("data-port")),
        (ServiceQuery(6788, Protocol::TCP), Service::Name("smc-http")),
        (ServiceQuery(3412, Protocol::UDP), Service::Name("xmlblaster")),
        (ServiceQuery(6060, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(5101, Protocol::UDP), Service::Name("talarian-udp")),
        (ServiceQuery(1814, Protocol::TCP), Service::Name("tdp-suite")),
        (ServiceQuery(4396, Protocol::TCP), Service::Name("fly")),
        (ServiceQuery(32776, Protocol::TCP), Service::Name("sometimes-rpc15")),
        (ServiceQuery(3663, Protocol::TCP), Service::Name("dtp")),
        (ServiceQuery(2017, Protocol::TCP), Service::Name("cypress-stat")),
        (ServiceQuery(3748, Protocol::UDP), Service::Name("webdata")),
        (ServiceQuery(6382, Protocol::TCP), Service::Name("metatude-mds")),
        (ServiceQuery(2736, Protocol::UDP), Service::Name("radwiz-nms-srv")),
        (ServiceQuery(538, Protocol::TCP), Service::Name("gdomap")),
        (ServiceQuery(5309, Protocol::TCP), Service::Name("jprinter")),
        (ServiceQuery(172, Protocol::UDP), Service::Name("cl-1")),
        (ServiceQuery(25002, Protocol::UDP), Service::Name("icl-twobase3")),
        (ServiceQuery(131, Protocol::TCP), Service::Name("cisco-tna")),
        (ServiceQuery(6389, Protocol::UDP), Service::Name("clariion-evr01")),
        (ServiceQuery(2120, Protocol::UDP), Service::Name("qencp")),
        (ServiceQuery(435, Protocol::TCP), Service::Name("mobilip-mn")),
        (ServiceQuery(91, Protocol::TCP), Service::Name("mit-dov")),
        (ServiceQuery(3667, Protocol::UDP), Service::Name("infoexch")),
        (ServiceQuery(2860, Protocol::TCP), Service::Name("dialpad-voice1")),
        (ServiceQuery(22763, Protocol::UDP), Service::Name("talikaserver")),
        (ServiceQuery(2657, Protocol::TCP), Service::Name("sns-dispatcher")),
        (ServiceQuery(3676, Protocol::TCP), Service::Name("va-pacbase")),
        (ServiceQuery(7276, Protocol::TCP), Service::Name("oma-ilp")),
        (ServiceQuery(655, Protocol::TCP), Service::Name("tinc")),
        (ServiceQuery(3072, Protocol::TCP), Service::Name("csd-monitor")),
        (ServiceQuery(4168, Protocol::TCP), Service::Name("pslicser")),
        (ServiceQuery(3685, Protocol::TCP), Service::Name("dsx-agent")),
        (ServiceQuery(1725, Protocol::UDP), Service::Name("iden-ralp")),
        (ServiceQuery(13709, Protocol::TCP), Service::Name("netbackup")),
        (ServiceQuery(463, Protocol::UDP), Service::Name("alpes")),
        (ServiceQuery(3022, Protocol::UDP), Service::Name("csregagent")),
        (ServiceQuery(3954, Protocol::TCP), Service::Name("adrep")),
        (ServiceQuery(4953, Protocol::TCP), Service::Name("dbsyncarbiter")),
        (ServiceQuery(517, Protocol::TCP), Service::Name("talk")),
        (ServiceQuery(4731, Protocol::TCP), Service::Name("remcap")),
        (ServiceQuery(527, Protocol::TCP), Service::Name("stx")),
        (ServiceQuery(4728, Protocol::UDP), Service::Name("capmux")),
        (ServiceQuery(2771, Protocol::TCP), Service::Name("vergencecm")),
        (ServiceQuery(582, Protocol::TCP), Service::Name("scc-security")),
        (ServiceQuery(3195, Protocol::TCP), Service::Name("ncu-1")),
        (ServiceQuery(6066, Protocol::TCP), Service::Name("ewctsp")),
        (ServiceQuery(4702, Protocol::TCP), Service::Name("netxms-sync")),
        (ServiceQuery(8732, Protocol::TCP), Service::Name("dtp-net")),
        (ServiceQuery(747, Protocol::TCP), Service::Name("fujitsu-dev")),
        (ServiceQuery(3976, Protocol::TCP), Service::Name("opswagent")),
        (ServiceQuery(7100, Protocol::TCP), Service::Name("font-service")),
        (ServiceQuery(20005, Protocol::TCP), Service::Name("btx")),
        (ServiceQuery(528, Protocol::TCP), Service::Name("custix")),
        (ServiceQuery(3828, Protocol::UDP), Service::Name("neteh")),
        (ServiceQuery(6104, Protocol::UDP), Service::Name("dbdb")),
        (ServiceQuery(8998, Protocol::TCP), Service::Name("canto-roboflow")),
        (ServiceQuery(3759, Protocol::TCP), Service::Name("exapt-lmgr")),
        (ServiceQuery(2875, Protocol::TCP), Service::Name("dxmessagebase2")),
        (ServiceQuery(4134, Protocol::UDP), Service::Name("nifty-hmi")),
        (ServiceQuery(9950, Protocol::TCP), Service::Name("apc-9950")),
        (ServiceQuery(2025, Protocol::UDP), Service::Name("xribs")),
        (ServiceQuery(1966, Protocol::UDP), Service::Name("slush")),
        (ServiceQuery(7317, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(5003, Protocol::UDP), Service::Name("filemaker")),
        (ServiceQuery(3312, Protocol::UDP), Service::Name("appman-server")),
        (ServiceQuery(1558, Protocol::UDP), Service::Name("xingmpeg")),
        (ServiceQuery(19150, Protocol::TCP), Service::Name("gkrellm")),
        (ServiceQuery(1786, Protocol::TCP), Service::Name("funk-logger")),
        (ServiceQuery(1672, Protocol::TCP), Service::Name("netview-aix-12")),
        (ServiceQuery(668, Protocol::UDP), Service::Name("mecomm")),
        (ServiceQuery(2911, Protocol::UDP), Service::Name("blockade")),
        (ServiceQuery(7341, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(707, Protocol::TCP), Service::Name("borland-dsj")),
        (ServiceQuery(4880, Protocol::TCP), Service::Name("hislip")),
        (ServiceQuery(776, Protocol::UDP), Service::Name("wpages")),
        (ServiceQuery(11173, Protocol::TCP), Service::Name("t5-straton")),
        (ServiceQuery(3888, Protocol::UDP), Service::Name("ciphire-serv")),
        (ServiceQuery(2715, Protocol::TCP), Service::Name("hpstgmgr2")),
        (ServiceQuery(6965, Protocol::UDP), Service::Name("swistrap")),
        (ServiceQuery(3647, Protocol::UDP), Service::Name("splitlock-gw")),
        (ServiceQuery(1257, Protocol::TCP), Service::Name("shockwave2")),
        (ServiceQuery(1054, Protocol::TCP), Service::Name("brvread")),
        (ServiceQuery(3839, Protocol::TCP), Service::Name("amx-rms")),
        (ServiceQuery(4899, Protocol::TCP), Service::Name("radmin")),
        (ServiceQuery(519, Protocol::UDP), Service::Name("utime")),
        (ServiceQuery(3361, Protocol::TCP), Service::Name("kv-agent")),
        (ServiceQuery(2164, Protocol::TCP), Service::Name("ddns-v3")),
        (ServiceQuery(4681, Protocol::UDP), Service::Name("parliant")),
        (ServiceQuery(7547, Protocol::UDP), Service::Name("cwmp")),
        (ServiceQuery(3082, Protocol::UDP), Service::Name("tl1-raw")),
        (ServiceQuery(7510, Protocol::TCP), Service::Name("ovhpas")),
        (ServiceQuery(7021, Protocol::TCP), Service::Name("dpserveadmin")),
        (ServiceQuery(7734, Protocol::UDP), Service::Name("smip")),
        (ServiceQuery(5078, Protocol::TCP), Service::Name("pixelpusher")),
        (ServiceQuery(2549, Protocol::UDP), Service::Name("ipass")),
        (ServiceQuery(1076, Protocol::UDP), Service::Name("dab-sti-c")),
        (ServiceQuery(3815, Protocol::UDP), Service::Name("lansurveyorxml")),
        (ServiceQuery(2585, Protocol::TCP), Service::Name("netx-server")),
        (ServiceQuery(3825, Protocol::UDP), Service::Name("ffserver")),
        (ServiceQuery(13715, Protocol::TCP), Service::Name("netbackup")),
        (ServiceQuery(3969, Protocol::TCP), Service::Name("landmarks")),
        (ServiceQuery(2834, Protocol::UDP), Service::Name("evtp")),
        (ServiceQuery(32801, Protocol::TCP), Service::Name("mlsn")),
        (ServiceQuery(5784, Protocol::UDP), Service::Name("ibar")),
        (ServiceQuery(2150, Protocol::UDP), Service::Name("dynamic3d")),
        (ServiceQuery(575, Protocol::UDP), Service::Name("vemmi")),
        (ServiceQuery(2568, Protocol::TCP), Service::Name("spamtrap")),
        (ServiceQuery(439, Protocol::UDP), Service::Name("dasp")),
        (ServiceQuery(2248, Protocol::TCP), Service::Name("ums")),
        (ServiceQuery(3856, Protocol::UDP), Service::Name("informer")),
        (ServiceQuery(9040, Protocol::TCP), Service::Name("tor-trans")),
        (ServiceQuery(424, Protocol::UDP), Service::Name("opc-job-track")),
        (ServiceQuery(4841, Protocol::UDP), Service::Name("quosa")),
        (ServiceQuery(2565, Protocol::TCP), Service::Name("coord-svr")),
        (ServiceQuery(691, Protocol::TCP), Service::Name("resvc")),
        (ServiceQuery(18185, Protocol::TCP), Service::Name("opsec-omi")),
        (ServiceQuery(23272, Protocol::TCP), Service::Name("s102")),
        (ServiceQuery(2222, Protocol::UDP), Service::Name("msantipiracy")),
        (ServiceQuery(520, Protocol::UDP), Service::Name("route")),
        (ServiceQuery(433, Protocol::UDP), Service::Name("nnsp")),
        (ServiceQuery(3222, Protocol::TCP), Service::Name("glbp")),
        (ServiceQuery(1639, Protocol::UDP), Service::Name("cert-initiator")),
        (ServiceQuery(1583, Protocol::TCP), Service::Name("simbaexpress")),
        (ServiceQuery(9163, Protocol::TCP), Service::Name("apani4")),
        (ServiceQuery(4034, Protocol::TCP), Service::Name("ubxd")),
        (ServiceQuery(1166, Protocol::UDP), Service::Name("qsm-remote")),
        (ServiceQuery(2427, Protocol::UDP), Service::Name("mgcp-gateway")),
        (ServiceQuery(3367, Protocol::UDP), Service::Name("satvid-datalnk")),
        (ServiceQuery(3274, Protocol::UDP), Service::Name("ordinox-server")),
        (ServiceQuery(618, Protocol::UDP), Service::Name("dei-icda")),
        (ServiceQuery(1270, Protocol::UDP), Service::Name("opsmgr")),
        (ServiceQuery(2837, Protocol::UDP), Service::Name("repliweb")),
        (ServiceQuery(2757, Protocol::UDP), Service::Name("cnrp")),
        (ServiceQuery(4061, Protocol::TCP), Service::Name("ice-location")),
        (ServiceQuery(5724, Protocol::UDP), Service::Name("omsdk")),
        (ServiceQuery(2495, Protocol::TCP), Service::Name("fast-rem-serv")),
        (ServiceQuery(20670, Protocol::UDP), Service::Name("track")),
        (ServiceQuery(2720, Protocol::TCP), Service::Name("wkars")),
        (ServiceQuery(20202, Protocol::UDP), Service::Name("ipdtp-port")),
        (ServiceQuery(3244, Protocol::TCP), Service::Name("onesaf")),
        (ServiceQuery(3086, Protocol::TCP), Service::Name("sj3")),
        (ServiceQuery(11623, Protocol::TCP), Service::Name("emc-xsw-dconfig")),
        (ServiceQuery(5720, Protocol::TCP), Service::Name("ms-licensing")),
        (ServiceQuery(162, Protocol::TCP), Service::Name("snmptrap")),
        (ServiceQuery(848, Protocol::UDP), Service::Name("gdoi")),
        (ServiceQuery(2187, Protocol::TCP), Service::Name("ssmc")),
        (ServiceQuery(11112, Protocol::UDP), Service::Name("dicom")),
        (ServiceQuery(3311, Protocol::UDP), Service::Name("mcns-tel-ret")),
        (ServiceQuery(4012, Protocol::TCP), Service::Name("pda-gate")),
        (ServiceQuery(8600, Protocol::UDP), Service::Name("asterix")),
        (ServiceQuery(6850, Protocol::TCP), Service::Name("iccrushmore")),
        (ServiceQuery(2297, Protocol::UDP), Service::Name("d2k-datamover1")),
        (ServiceQuery(248, Protocol::TCP), Service::Name("bhfhs")),
        (ServiceQuery(6043, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(3037, Protocol::TCP), Service::Name("hp-san-mgmt")),
        (ServiceQuery(2820, Protocol::UDP), Service::Name("univision")),
        (ServiceQuery(1568, Protocol::UDP), Service::Name("tsspmap")),
        (ServiceQuery(3066, Protocol::UDP), Service::Name("netattachsdmp")),
        (ServiceQuery(3012, Protocol::TCP), Service::Name("twsdss")),
        (ServiceQuery(4123, Protocol::UDP), Service::Name("z-wave")),
        (ServiceQuery(8899, Protocol::TCP), Service::Name("ospf-lite")),
        (ServiceQuery(434, Protocol::UDP), Service::Name("mobileip-agent")),
        (ServiceQuery(28010, Protocol::TCP), Service::Name("gruber-cashreg")),
        (ServiceQuery(5985, Protocol::TCP), Service::Name("wsman")),
        (ServiceQuery(8067, Protocol::TCP), Service::Name("infi-async")),
        (ServiceQuery(4300, Protocol::UDP), Service::Name("corelccam")),
        (ServiceQuery(8416, Protocol::UDP), Service::Name("espeech")),
        (ServiceQuery(3149, Protocol::TCP), Service::Name("nm-game-server")),
        (ServiceQuery(1572, Protocol::TCP), Service::Name("chip-lm")),
        (ServiceQuery(2914, Protocol::TCP), Service::Name("gamelobby")),
        (ServiceQuery(1512, Protocol::UDP), Service::Name("wins")),
        (ServiceQuery(3114, Protocol::UDP), Service::Name("ccmad")),
        (ServiceQuery(3861, Protocol::TCP), Service::Name("winshadow-hd")),
        (ServiceQuery(4802, Protocol::UDP), Service::Name("ilss")),
        (ServiceQuery(92, Protocol::TCP), Service::Name("npp")),
        (ServiceQuery(166, Protocol::UDP), Service::Name("s-net")),
        (ServiceQuery(4686, Protocol::UDP), Service::Name("msp-os")),
        (ServiceQuery(44445, Protocol::TCP), Service::Name("acronis-backup")),
        (ServiceQuery(2233, Protocol::TCP), Service::Name("infocrypt")),
        (ServiceQuery(1704, Protocol::TCP), Service::Name("bcs-broker")),
        (ServiceQuery(2121, Protocol::UDP), Service::Name("scientia-ssdb")),
        (ServiceQuery(3619, Protocol::UDP), Service::Name("aairnet-2")),
        (ServiceQuery(5012, Protocol::TCP), Service::Name("nsp")),
        (ServiceQuery(9986, Protocol::TCP), Service::Name("kaostransport")),
        (ServiceQuery(3192, Protocol::TCP), Service::Name("firemonrcc")),
        (ServiceQuery(333, Protocol::TCP), Service::Name("texar")),
        (ServiceQuery(2304, Protocol::TCP), Service::Name("attachmate-uts")),
        (ServiceQuery(3393, Protocol::TCP), Service::Name("d2k-tapestry1")),
        (ServiceQuery(1732, Protocol::UDP), Service::Name("proxim")),
        (ServiceQuery(991, Protocol::TCP), Service::Name("nas")),
        (ServiceQuery(138, Protocol::TCP), Service::Name("netbios-dgm")),
        (ServiceQuery(3354, Protocol::TCP), Service::Name("suitjd")),
        (ServiceQuery(1771, Protocol::TCP), Service::Name("vaultbase")),
        (ServiceQuery(2881, Protocol::TCP), Service::Name("ndsp")),
        (ServiceQuery(4128, Protocol::TCP), Service::Name("nufw")),
        (ServiceQuery(6714, Protocol::TCP), Service::Name("ibprotocol")),
        (ServiceQuery(6790, Protocol::TCP), Service::Name("hnmp")),
        (ServiceQuery(3548, Protocol::TCP), Service::Name("interworld")),
        (ServiceQuery(20048, Protocol::UDP), Service::Name("mountd")),
        (ServiceQuery(5490, Protocol::TCP), Service::Name("connect-proxy")),
        (ServiceQuery(447, Protocol::TCP), Service::Name("ddm-dfm")),
        (ServiceQuery(3239, Protocol::UDP), Service::Name("apparenet-ui")),
        (ServiceQuery(9987, Protocol::UDP), Service::Name("dsm-scm-target")),
        (ServiceQuery(5744, Protocol::UDP), Service::Name("watchdoc")),
        (ServiceQuery(3205, Protocol::TCP), Service::Name("isns")),
        (ServiceQuery(3640, Protocol::TCP), Service::Name("netplay-port1")),
        (ServiceQuery(561, Protocol::TCP), Service::Name("monitor")),
        (ServiceQuery(2496, Protocol::TCP), Service::Name("dirgis")),
        (ServiceQuery(1994, Protocol::TCP), Service::Name("stun-port")),
        (ServiceQuery(6163, Protocol::TCP), Service::Name("pscribe")),
        (ServiceQuery(3698, Protocol::UDP), Service::Name("sagectlpanel")),
        (ServiceQuery(8401, Protocol::UDP), Service::Name("sabarsd")),
        (ServiceQuery(1979, Protocol::TCP), Service::Name("unisql-java")),
        (ServiceQuery(4268, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3170, Protocol::TCP), Service::Name("serverview-asn")),
        (ServiceQuery(3776, Protocol::TCP), Service::Name("dvcprov-port")),
        (ServiceQuery(2120, Protocol::TCP), Service::Name("kauth")),
        (ServiceQuery(445, Protocol::TCP), Service::Name("microsoft-ds")),
        (ServiceQuery(671, Protocol::TCP), Service::Name("vacdsm-app")),
        (ServiceQuery(10008, Protocol::UDP), Service::Name("octopus")),
        (ServiceQuery(428, Protocol::UDP), Service::Name("ocs_cmu")),
        (ServiceQuery(1650, Protocol::TCP), Service::Name("nkd")),
        (ServiceQuery(11201, Protocol::UDP), Service::Name("smsqp")),
        (ServiceQuery(2889, Protocol::TCP), Service::Name("rsom")),
        (ServiceQuery(3414, Protocol::TCP), Service::Name("wip-port")),
        (ServiceQuery(1655, Protocol::TCP), Service::Name("dec-mbadmin")),
        (ServiceQuery(1817, Protocol::UDP), Service::Name("rkb-oscs")),
        (ServiceQuery(3000, Protocol::TCP), Service::Name("ppp")),
        (ServiceQuery(1499, Protocol::UDP), Service::Name("fhc")),
        (ServiceQuery(6543, Protocol::TCP), Service::Name("mythtv")),
        (ServiceQuery(5405, Protocol::UDP), Service::Name("netsupport")),
        (ServiceQuery(5768, Protocol::UDP), Service::Name("openmailpxy")),
        (ServiceQuery(681, Protocol::TCP), Service::Name("entrust-aams")),
        (ServiceQuery(3578, Protocol::TCP), Service::Name("data-port")),
        (ServiceQuery(1347, Protocol::TCP), Service::Name("bbn-mmc")),
        (ServiceQuery(2729, Protocol::TCP), Service::Name("tcim-control")),
        (ServiceQuery(1069, Protocol::TCP), Service::Name("cognex-insight")),
        (ServiceQuery(1766, Protocol::TCP), Service::Name("cft-5")),
        (ServiceQuery(2140, Protocol::TCP), Service::Name("ias-reg")),
        (ServiceQuery(5354, Protocol::UDP), Service::Name("mdnsresponder")),
        (ServiceQuery(7628, Protocol::UDP), Service::Name("zen-pawn")),
        (ServiceQuery(3457, Protocol::UDP), Service::Name("vat-control")),
        (ServiceQuery(1373, Protocol::TCP), Service::Name("chromagrafx")),
        (ServiceQuery(3959, Protocol::UDP), Service::Name("treehopper")),
        (ServiceQuery(1470, Protocol::UDP), Service::Name("uaiact")),
        (ServiceQuery(1239, Protocol::UDP), Service::Name("nmsd")),
        (ServiceQuery(4060, Protocol::TCP), Service::Name("dsmeter_iatc")),
        (ServiceQuery(4259, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(6999, Protocol::UDP), Service::Name("iatp-normalpri")),
        (ServiceQuery(47808, Protocol::UDP), Service::Name("bacnet")),
        (ServiceQuery(12121, Protocol::UDP), Service::Name("nupaper-ss")),
        (ServiceQuery(218, Protocol::TCP), Service::Name("mpp")),
        (ServiceQuery(345, Protocol::TCP), Service::Name("pawserv")),
        (ServiceQuery(22555, Protocol::UDP), Service::Name("vocaltec-phone")),
        (ServiceQuery(3676, Protocol::UDP), Service::Name("va-pacbase")),
        (ServiceQuery(2559, Protocol::TCP), Service::Name("lstp")),
        (ServiceQuery(5412, Protocol::UDP), Service::Name("continuus")),
        (ServiceQuery(2432, Protocol::TCP), Service::Name("codasrv")),
        (ServiceQuery(7010, Protocol::TCP), Service::Name("ups-onlinet")),
        (ServiceQuery(7786, Protocol::UDP), Service::Name("minivend")),
        (ServiceQuery(24465, Protocol::TCP), Service::Name("tonidods")),
        (ServiceQuery(16994, Protocol::UDP), Service::Name("amt-redir-tcp")),
        (ServiceQuery(11161, Protocol::UDP), Service::Name("suncacao-snmp")),
        (ServiceQuery(2084, Protocol::TCP), Service::Name("sunclustergeo")),
        (ServiceQuery(12546, Protocol::TCP), Service::Name("carb-repl-ctrl")),
        (ServiceQuery(1913, Protocol::UDP), Service::Name("armadp")),
        (ServiceQuery(8912, Protocol::UDP), Service::Name("wcbackup")),
        (ServiceQuery(4045, Protocol::UDP), Service::Name("lockd")),
        (ServiceQuery(729, Protocol::UDP), Service::Name("netviewdm1")),
        (ServiceQuery(1718, Protocol::TCP), Service::Name("h323gatedisc")),
        (ServiceQuery(6418, Protocol::TCP), Service::Name("syserverremote")),
        (ServiceQuery(4164, Protocol::TCP), Service::Name("silverpeakcomm")),
        (ServiceQuery(9629, Protocol::UDP), Service::Name("uniport")),
        (ServiceQuery(8161, Protocol::UDP), Service::Name("patrol-snmp")),
        (ServiceQuery(3860, Protocol::TCP), Service::Name("sasp")),
        (ServiceQuery(1370, Protocol::UDP), Service::Name("us-gv")),
        (ServiceQuery(14000, Protocol::TCP), Service::Name("scotty-ft")),
        (ServiceQuery(598, Protocol::UDP), Service::Name("sco-websrvrmg3")),
        (ServiceQuery(3861, Protocol::UDP), Service::Name("winshadow-hd")),
        (ServiceQuery(5227, Protocol::TCP), Service::Name("perfd")),
        (ServiceQuery(17754, Protocol::UDP), Service::Name("zep")),
        (ServiceQuery(75, Protocol::UDP), Service::Name("priv-dial")),
        (ServiceQuery(4242, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1486, Protocol::TCP), Service::Name("nms_topo_serv")),
        (ServiceQuery(212, Protocol::UDP), Service::Name("anet")),
        (ServiceQuery(4293, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(4114, Protocol::UDP), Service::Name("jomamqmonitor")),
        (ServiceQuery(3808, Protocol::UDP), Service::Name("sun-as-iiops-ca")),
        (ServiceQuery(3146, Protocol::TCP), Service::Name("bears-02")),
        (ServiceQuery(397, Protocol::TCP), Service::Name("mptn")),
        (ServiceQuery(2963, Protocol::UDP), Service::Name("iph-policy-adm")),
        (ServiceQuery(1888, Protocol::UDP), Service::Name("ncconfig")),
        (ServiceQuery(6033, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(220, Protocol::UDP), Service::Name("imap3")),
        (ServiceQuery(9089, Protocol::UDP), Service::Name("sqlexec-ssl")),
        (ServiceQuery(555, Protocol::TCP), Service::Name("dsf")),
        (ServiceQuery(574, Protocol::UDP), Service::Name("ftp-agent")),
        (ServiceQuery(8613, Protocol::TCP), Service::Name("canon-bjnp3")),
        (ServiceQuery(1728, Protocol::TCP), Service::Name("telindus")),
        (ServiceQuery(5009, Protocol::TCP), Service::Name("airport-admin")),
        (ServiceQuery(647, Protocol::UDP), Service::Name("dhcp-failover")),
        (ServiceQuery(2851, Protocol::TCP), Service::Name("webemshttp")),
        (ServiceQuery(1111, Protocol::UDP), Service::Name("lmsocialserver")),
        (ServiceQuery(12321, Protocol::TCP), Service::Name("warehouse-sss")),
        (ServiceQuery(7741, Protocol::TCP), Service::Name("scriptview")),
        (ServiceQuery(3842, Protocol::TCP), Service::Name("nhci")),
        (ServiceQuery(3110, Protocol::UDP), Service::Name("sim-control")),
        (ServiceQuery(5859, Protocol::TCP), Service::Name("wherehoo")),
        (ServiceQuery(4036, Protocol::UDP), Service::Name("wap-push-https")),
        (ServiceQuery(6588, Protocol::TCP), Service::Name("analogx")),
        (ServiceQuery(2610, Protocol::TCP), Service::Name("versa-tek")),
        (ServiceQuery(1891, Protocol::TCP), Service::Name("childkey-notif")),
        (ServiceQuery(2966, Protocol::TCP), Service::Name("idp-infotrieve")),
        (ServiceQuery(2222, Protocol::TCP), Service::Name("EtherNetIP-1")),
        (ServiceQuery(7606, Protocol::TCP), Service::Name("mipi-debug")),
        (ServiceQuery(11161, Protocol::TCP), Service::Name("suncacao-snmp")),
        (ServiceQuery(8074, Protocol::TCP), Service::Name("gadugadu")),
        (ServiceQuery(3768, Protocol::TCP), Service::Name("rblcheckd")),
        (ServiceQuery(2723, Protocol::UDP), Service::Name("watchdog-nt")),
        (ServiceQuery(2361, Protocol::UDP), Service::Name("tl1")),
        (ServiceQuery(1333, Protocol::TCP), Service::Name("passwrd-policy")),
        (ServiceQuery(1092, Protocol::TCP), Service::Name("obrpd")),
        (ServiceQuery(7400, Protocol::UDP), Service::Name("rtps-discovery")),
        (ServiceQuery(258, Protocol::UDP), Service::Name("yak-chat")),
        (ServiceQuery(8161, Protocol::TCP), Service::Name("patrol-snmp")),
        (ServiceQuery(2937, Protocol::TCP), Service::Name("pnaconsult-lm")),
        (ServiceQuery(1676, Protocol::UDP), Service::Name("netcomm2")),
        (ServiceQuery(2553, Protocol::TCP), Service::Name("efidiningport")),
        (ServiceQuery(6006, Protocol::UDP), Service::Name("X11:6")),
        (ServiceQuery(2051, Protocol::TCP), Service::Name("epnsdp")),
        (ServiceQuery(2022, Protocol::TCP), Service::Name("down")),
        (ServiceQuery(3995, Protocol::UDP), Service::Name("iss-mgmt-ssl")),
        (ServiceQuery(3550, Protocol::TCP), Service::Name("ssmpp")),
        (ServiceQuery(1320, Protocol::UDP), Service::Name("amx-axbnet")),
        (ServiceQuery(1612, Protocol::UDP), Service::Name("netbill-trans")),
        (ServiceQuery(3982, Protocol::TCP), Service::Name("eis")),
        (ServiceQuery(3335, Protocol::UDP), Service::Name("directv-soft")),
        (ServiceQuery(4375, Protocol::TCP), Service::Name("tolteces")),
        (ServiceQuery(8000, Protocol::TCP), Service::Name("http-alt")),
        (ServiceQuery(1768, Protocol::UDP), Service::Name("cft-7")),
        (ServiceQuery(17235, Protocol::UDP), Service::Name("ssh-mgmt")),
        (ServiceQuery(2368, Protocol::TCP), Service::Name("opentable")),
        (ServiceQuery(3035, Protocol::TCP), Service::Name("fjsv-gssagt")),
        (ServiceQuery(3382, Protocol::TCP), Service::Name("fujitsu-neat")),
        (ServiceQuery(518, Protocol::TCP), Service::Name("ntalk")),
        (ServiceQuery(550, Protocol::UDP), Service::Name("new-rwho")),
        (ServiceQuery(4877, Protocol::TCP), Service::Name("lmcs")),
        (ServiceQuery(2552, Protocol::UDP), Service::Name("call-logging")),
        (ServiceQuery(4298, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3079, Protocol::TCP), Service::Name("lv-frontpanel")),
        (ServiceQuery(377, Protocol::UDP), Service::Name("tnETOS")),
        (ServiceQuery(3078, Protocol::TCP), Service::Name("orbix-cfg-ssl")),
        (ServiceQuery(423, Protocol::TCP), Service::Name("opc-job-start")),
        (ServiceQuery(10542, Protocol::UDP), Service::Name("MOS-aux")),
        (ServiceQuery(3609, Protocol::TCP), Service::Name("cpdi-pidas-cm")),
        (ServiceQuery(3064, Protocol::UDP), Service::Name("rprt")),
        (ServiceQuery(4277, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2284, Protocol::UDP), Service::Name("lnvmaps")),
        (ServiceQuery(1239, Protocol::TCP), Service::Name("nmsd")),
        (ServiceQuery(5344, Protocol::TCP), Service::Name("xkotodrcp")),
        (ServiceQuery(1866, Protocol::UDP), Service::Name("swrmi")),
        (ServiceQuery(3334, Protocol::UDP), Service::Name("directv-web")),
        (ServiceQuery(6102, Protocol::UDP), Service::Name("synchronet-upd")),
        (ServiceQuery(4217, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1656, Protocol::UDP), Service::Name("dec-mbadmin-h")),
        (ServiceQuery(1256, Protocol::UDP), Service::Name("de-server")),
        (ServiceQuery(2710, Protocol::UDP), Service::Name("sso-service")),
        (ServiceQuery(201, Protocol::UDP), Service::Name("at-rtmp")),
        (ServiceQuery(1068, Protocol::UDP), Service::Name("instl_bootc")),
        (ServiceQuery(7365, Protocol::TCP), Service::Name("lcm-server")),
        (ServiceQuery(4502, Protocol::TCP), Service::Name("a25-fap-fgw")),
        (ServiceQuery(4230, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(5066, Protocol::UDP), Service::Name("stanag-5066")),
        (ServiceQuery(4774, Protocol::TCP), Service::Name("converge")),
        (ServiceQuery(3986, Protocol::UDP), Service::Name("mapper-ws_ethd")),
        (ServiceQuery(7359, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(2908, Protocol::UDP), Service::Name("mao")),
        (ServiceQuery(1065, Protocol::TCP), Service::Name("syscomlan")),
        (ServiceQuery(9999, Protocol::TCP), Service::Name("abyss")),
        (ServiceQuery(2563, Protocol::UDP), Service::Name("cti-redwood")),
        (ServiceQuery(5193, Protocol::TCP), Service::Name("aol-3")),
        (ServiceQuery(44553, Protocol::UDP), Service::Name("rbr-debug")),
        (ServiceQuery(2478, Protocol::UDP), Service::Name("ssm-cssps")),
        (ServiceQuery(7727, Protocol::TCP), Service::Name("trident-data")),
        (ServiceQuery(4969, Protocol::UDP), Service::Name("ccss-qmm")),
        (ServiceQuery(9753, Protocol::UDP), Service::Name("rasadv")),
        (ServiceQuery(9284, Protocol::TCP), Service::Name("visd")),
        (ServiceQuery(3926, Protocol::TCP), Service::Name("winport")),
        (ServiceQuery(2910, Protocol::UDP), Service::Name("tdaccess")),
        (ServiceQuery(2083, Protocol::TCP), Service::Name("radsec")),
        (ServiceQuery(2942, Protocol::UDP), Service::Name("sm-pas-5")),
        (ServiceQuery(2211, Protocol::UDP), Service::Name("emwin")),
        (ServiceQuery(50000, Protocol::TCP), Service::Name("ibm-db2")),
        (ServiceQuery(4245, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(7322, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(5464, Protocol::UDP), Service::Name("quailnet")),
        (ServiceQuery(4172, Protocol::UDP), Service::Name("pcoip")),
        (ServiceQuery(414, Protocol::TCP), Service::Name("infoseek")),
        (ServiceQuery(40000, Protocol::TCP), Service::Name("safetynetp")),
        (ServiceQuery(3305, Protocol::TCP), Service::Name("odette-ftp")),
        (ServiceQuery(6306, Protocol::TCP), Service::Name("ufmp")),
        (ServiceQuery(57, Protocol::UDP), Service::Name("priv-term")),
        (ServiceQuery(1713, Protocol::TCP), Service::Name("conferencetalk")),
        (ServiceQuery(3254, Protocol::TCP), Service::Name("pda-sys")),
        (ServiceQuery(2488, Protocol::TCP), Service::Name("moy-corp")),
        (ServiceQuery(3517, Protocol::TCP), Service::Name("802-11-iapp")),
        (ServiceQuery(3730, Protocol::UDP), Service::Name("client-ctrl")),
        (ServiceQuery(1114, Protocol::UDP), Service::Name("mini-sql")),
        (ServiceQuery(4390, Protocol::TCP), Service::Name("wiegand")),
        (ServiceQuery(7543, Protocol::TCP), Service::Name("atul")),
        (ServiceQuery(4056, Protocol::UDP), Service::Name("lms")),
        (ServiceQuery(5910, Protocol::UDP), Service::Name("cm")),
        (ServiceQuery(9002, Protocol::TCP), Service::Name("dynamid")),
        (ServiceQuery(22800, Protocol::UDP), Service::Name("aws-brf")),
        (ServiceQuery(2536, Protocol::TCP), Service::Name("btpp2audctr1")),
        (ServiceQuery(5421, Protocol::TCP), Service::Name("netsupport2")),
        (ServiceQuery(3772, Protocol::UDP), Service::Name("ctp")),
        (ServiceQuery(6419, Protocol::TCP), Service::Name("svdrp")),
        (ServiceQuery(1543, Protocol::UDP), Service::Name("simba-cs")),
        (ServiceQuery(12010, Protocol::TCP), Service::Name("edbsrvr")),
        (ServiceQuery(2252, Protocol::TCP), Service::Name("njenet-ssl")),
        (ServiceQuery(4986, Protocol::UDP), Service::Name("mrip")),
        (ServiceQuery(1457, Protocol::UDP), Service::Name("valisys-lm")),
        (ServiceQuery(1641, Protocol::UDP), Service::Name("invision")),
        (ServiceQuery(1230, Protocol::UDP), Service::Name("periscope")),
        (ServiceQuery(1480, Protocol::TCP), Service::Name("pacerforum")),
        (ServiceQuery(33434, Protocol::TCP), Service::Name("traceroute")),
        (ServiceQuery(6301, Protocol::TCP), Service::Name("bmc_ctd_ldap")),
        (ServiceQuery(1431, Protocol::TCP), Service::Name("rgtp")),
        (ServiceQuery(18241, Protocol::UDP), Service::Name("checkpoint-rtm")),
        (ServiceQuery(29000, Protocol::TCP), Service::Name("saltd-licensing")),
        (ServiceQuery(13783, Protocol::TCP), Service::Name("netbackup")),
        (ServiceQuery(5401, Protocol::UDP), Service::Name("excerpts")),
        (ServiceQuery(3720, Protocol::UDP), Service::Name("ufastro-instr")),
        (ServiceQuery(3765, Protocol::TCP), Service::Name("rtraceroute")),
        (ServiceQuery(1611, Protocol::TCP), Service::Name("ill")),
        (ServiceQuery(3644, Protocol::UDP), Service::Name("ssowatch")),
        (ServiceQuery(283, Protocol::UDP), Service::Name("rescap")),
        (ServiceQuery(13822, Protocol::TCP), Service::Name("dsmcc-ccp")),
        (ServiceQuery(535, Protocol::UDP), Service::Name("iiop")),
        (ServiceQuery(213, Protocol::TCP), Service::Name("ipx")),
        (ServiceQuery(7634, Protocol::TCP), Service::Name("hddtemp")),
        (ServiceQuery(61, Protocol::TCP), Service::Name("ni-mail")),
        (ServiceQuery(7500, Protocol::UDP), Service::Name("silhouette")),
        (ServiceQuery(2747, Protocol::UDP), Service::Name("fjippol-swrly")),
        (ServiceQuery(9090, Protocol::UDP), Service::Name("websm")),
        (ServiceQuery(3196, Protocol::TCP), Service::Name("ncu-2")),
        (ServiceQuery(10543, Protocol::UDP), Service::Name("MOS-soap")),
        (ServiceQuery(479, Protocol::UDP), Service::Name("iafserver")),
        (ServiceQuery(3339, Protocol::UDP), Service::Name("anet-l")),
        (ServiceQuery(8060, Protocol::TCP), Service::Name("aero")),
        (ServiceQuery(3302, Protocol::TCP), Service::Name("mcs-fastmail")),
        (ServiceQuery(2826, Protocol::UDP), Service::Name("slc-systemlog")),
        (ServiceQuery(4426, Protocol::TCP), Service::Name("beacon-port-2")),
        (ServiceQuery(3429, Protocol::UDP), Service::Name("gcsp")),
        (ServiceQuery(45054, Protocol::UDP), Service::Name("invision-ag")),
        (ServiceQuery(2761, Protocol::TCP), Service::Name("dicom-iscl")),
        (ServiceQuery(1012, Protocol::UDP), Service::Name("sometimes-rpc1")),
        (ServiceQuery(637, Protocol::UDP), Service::Name("lanserver")),
        (ServiceQuery(1918, Protocol::TCP), Service::Name("can-nds")),
        (ServiceQuery(1862, Protocol::UDP), Service::Name("mysql-cm-agent")),
        (ServiceQuery(16162, Protocol::TCP), Service::Name("solaris-audit")),
        (ServiceQuery(411, Protocol::TCP), Service::Name("rmt")),
        (ServiceQuery(18181, Protocol::UDP), Service::Name("opsec-cvp")),
        (ServiceQuery(2312, Protocol::TCP), Service::Name("wanscaler")),
        (ServiceQuery(4843, Protocol::UDP), Service::Name("opcua-tls")),
        (ServiceQuery(4073, Protocol::TCP), Service::Name("iRAPP")),
        (ServiceQuery(644, Protocol::UDP), Service::Name("dwr")),
        (ServiceQuery(3032, Protocol::UDP), Service::Name("redwood-chat")),
        (ServiceQuery(11430, Protocol::TCP), Service::Name("lsdp")),
        (ServiceQuery(2309, Protocol::UDP), Service::Name("sdserver")),
        (ServiceQuery(8020, Protocol::TCP), Service::Name("intu-ec-svcdisc")),
        (ServiceQuery(4428, Protocol::TCP), Service::Name("omviserver")),
        (ServiceQuery(3715, Protocol::UDP), Service::Name("anoto-rendezv")),
        (ServiceQuery(2236, Protocol::TCP), Service::Name("nani")),
        (ServiceQuery(3801, Protocol::TCP), Service::Name("ibm-mgr")),
        (ServiceQuery(9401, Protocol::TCP), Service::Name("sec-t4net-clt")),
        (ServiceQuery(5910, Protocol::TCP), Service::Name("cm")),
        (ServiceQuery(3137, Protocol::UDP), Service::Name("rtnt-1")),
        (ServiceQuery(2321, Protocol::TCP), Service::Name("rdlap")),
        (ServiceQuery(3288, Protocol::TCP), Service::Name("cops")),
        (ServiceQuery(3947, Protocol::UDP), Service::Name("ccp")),
        (ServiceQuery(8894, Protocol::TCP), Service::Name("ddi-tcp-7")),
        (ServiceQuery(5741, Protocol::TCP), Service::Name("ida-discover1")),
        (ServiceQuery(2137, Protocol::TCP), Service::Name("connect")),
        (ServiceQuery(8206, Protocol::UDP), Service::Name("lm-dta")),
        (ServiceQuery(4158, Protocol::UDP), Service::Name("stat-cc")),
        (ServiceQuery(5766, Protocol::TCP), Service::Name("openmailns")),
        (ServiceQuery(1985, Protocol::UDP), Service::Name("hsrp")),
        (ServiceQuery(8300, Protocol::TCP), Service::Name("tmi")),
        (ServiceQuery(2159, Protocol::TCP), Service::Name("gdbremote")),
        (ServiceQuery(2627, Protocol::TCP), Service::Name("webster")),
        (ServiceQuery(3422, Protocol::UDP), Service::Name("rusb-sys-port")),
        (ServiceQuery(6568, Protocol::UDP), Service::Name("rp-reputation")),
        (ServiceQuery(1983, Protocol::UDP), Service::Name("lhtp")),
        (ServiceQuery(7163, Protocol::UDP), Service::Name("cacsambroker")),
        (ServiceQuery(3559, Protocol::TCP), Service::Name("cctv-port")),
        (ServiceQuery(3360, Protocol::TCP), Service::Name("kv-server")),
        (ServiceQuery(4226, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3964, Protocol::TCP), Service::Name("sasggprs")),
        (ServiceQuery(652, Protocol::UDP), Service::Name("hello-port")),
        (ServiceQuery(12004, Protocol::UDP), Service::Name("entextlow")),
        (ServiceQuery(1199, Protocol::TCP), Service::Name("dmidi")),
        (ServiceQuery(189, Protocol::UDP), Service::Name("qft")),
        (ServiceQuery(1787, Protocol::TCP), Service::Name("funk-license")),
        (ServiceQuery(2556, Protocol::UDP), Service::Name("nicetec-nmsvc")),
        (ServiceQuery(2175, Protocol::UDP), Service::Name("airsync")),
        (ServiceQuery(4597, Protocol::UDP), Service::Name("a21-an-1xbs")),
        (ServiceQuery(4621, Protocol::TCP), Service::Name("ventoso")),
        (ServiceQuery(2907, Protocol::UDP), Service::Name("webmethods-b2b")),
        (ServiceQuery(1743, Protocol::TCP), Service::Name("cinegrfx-lm")),
        (ServiceQuery(2018, Protocol::TCP), Service::Name("terminaldb")),
        (ServiceQuery(3459, Protocol::UDP), Service::Name("integral")),
        (ServiceQuery(9911, Protocol::TCP), Service::Name("sype-transport")),
        (ServiceQuery(4107, Protocol::UDP), Service::Name("j-ac")),
        (ServiceQuery(657, Protocol::UDP), Service::Name("rmc")),
        (ServiceQuery(2612, Protocol::TCP), Service::Name("qpasa-agent")),
        (ServiceQuery(6161, Protocol::TCP), Service::Name("patrol-ism")),
        (ServiceQuery(5750, Protocol::UDP), Service::Name("rscd")),
        (ServiceQuery(618, Protocol::TCP), Service::Name("dei-icda")),
        (ServiceQuery(2823, Protocol::TCP), Service::Name("cqg-netlan")),
        (ServiceQuery(1355, Protocol::TCP), Service::Name("intuitive-edge")),
        (ServiceQuery(4912, Protocol::TCP), Service::Name("lutap")),
        (ServiceQuery(20, Protocol::TCP), Service::Name("ftp-data")),
        (ServiceQuery(2929, Protocol::TCP), Service::Name("amx-webadmin")),
        (ServiceQuery(15126, Protocol::TCP), Service::Name("swgps")),
        (ServiceQuery(2524, Protocol::TCP), Service::Name("optiwave-lm")),
        (ServiceQuery(3968, Protocol::TCP), Service::Name("ianywhere-dbns")),
        (ServiceQuery(5166, Protocol::TCP), Service::Name("winpcs")),
        (ServiceQuery(2558, Protocol::TCP), Service::Name("pclemultimedia")),
        (ServiceQuery(6027, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(1521, Protocol::UDP), Service::Name("ncube-lm")),
        (ServiceQuery(3755, Protocol::TCP), Service::Name("sas-remote-hlp")),
        (ServiceQuery(1326, Protocol::UDP), Service::Name("wimsic")),
        (ServiceQuery(38, Protocol::TCP), Service::Name("rap")),
        (ServiceQuery(7410, Protocol::TCP), Service::Name("ionixnetmon")),
        (ServiceQuery(6146, Protocol::UDP), Service::Name("lonewolf-lm")),
        (ServiceQuery(776, Protocol::TCP), Service::Name("wpages")),
        (ServiceQuery(104, Protocol::TCP), Service::Name("acr-nema")),
        (ServiceQuery(8293, Protocol::TCP), Service::Name("hiperscan-id")),
        (ServiceQuery(3907, Protocol::UDP), Service::Name("imoguia-port")),
        (ServiceQuery(11106, Protocol::TCP), Service::Name("sgi-lk")),
        (ServiceQuery(4032, Protocol::TCP), Service::Name("vrts-auth-port")),
        (ServiceQuery(7336, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(2047, Protocol::UDP), Service::Name("dls")),
        (ServiceQuery(1244, Protocol::UDP), Service::Name("isbconference1")),
        (ServiceQuery(2847, Protocol::TCP), Service::Name("aimpp-port-req")),
        (ServiceQuery(5011, Protocol::TCP), Service::Name("telelpathattack")),
        (ServiceQuery(3964, Protocol::UDP), Service::Name("sasggprs")),
        (ServiceQuery(8320, Protocol::UDP), Service::Name("tnp-discover")),
        (ServiceQuery(8115, Protocol::UDP), Service::Name("mtl8000-matrix")),
        (ServiceQuery(2727, Protocol::UDP), Service::Name("mgcp-callagent")),
        (ServiceQuery(600, Protocol::UDP), Service::Name("ipcserver")),
        (ServiceQuery(6070, Protocol::UDP), Service::Name("messageasap")),
        (ServiceQuery(5222, Protocol::TCP), Service::Name("xmpp-client")),
        (ServiceQuery(7200, Protocol::TCP), Service::Name("fodms")),
        (ServiceQuery(3947, Protocol::TCP), Service::Name("ccp")),
        (ServiceQuery(1247, Protocol::UDP), Service::Name("visionpyramid")),
        (ServiceQuery(7321, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(5968, Protocol::UDP), Service::Name("mppolicy-v5")),
        (ServiceQuery(4041, Protocol::TCP), Service::Name("houston")),
        (ServiceQuery(10102, Protocol::UDP), Service::Name("ezproxy-2")),
        (ServiceQuery(11751, Protocol::UDP), Service::Name("intrepid-ssl")),
        (ServiceQuery(1024, Protocol::TCP), Service::Name("kdm")),
        (ServiceQuery(2732, Protocol::TCP), Service::Name("g5m")),
        (ServiceQuery(2880, Protocol::UDP), Service::Name("synapse")),
        (ServiceQuery(2371, Protocol::TCP), Service::Name("worldwire")),
        (ServiceQuery(4442, Protocol::UDP), Service::Name("saris")),
        (ServiceQuery(2884, Protocol::TCP), Service::Name("flashmsg")),
        (ServiceQuery(2465, Protocol::TCP), Service::Name("lbm")),
        (ServiceQuery(3034, Protocol::TCP), Service::Name("osmosis-aeea")),
        (ServiceQuery(5168, Protocol::TCP), Service::Name("scte30")),
        (ServiceQuery(5912, Protocol::TCP), Service::Name("fis")),
        (ServiceQuery(6014, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(3157, Protocol::TCP), Service::Name("e3consultants")),
        (ServiceQuery(2141, Protocol::UDP), Service::Name("ias-admind")),
        (ServiceQuery(3892, Protocol::UDP), Service::Name("pcc-image-port")),
        (ServiceQuery(3376, Protocol::TCP), Service::Name("cdbroker")),
        (ServiceQuery(107, Protocol::TCP), Service::Name("rtelnet")),
        (ServiceQuery(14414, Protocol::UDP), Service::Name("ca-web-update")),
        (ServiceQuery(3291, Protocol::TCP), Service::Name("sah-lm")),
        (ServiceQuery(2578, Protocol::TCP), Service::Name("rvs-isdn-dcp")),
        (ServiceQuery(7262, Protocol::TCP), Service::Name("cnap")),
        (ServiceQuery(2560, Protocol::TCP), Service::Name("labrat")),
        (ServiceQuery(9191, Protocol::TCP), Service::Name("sun-as-jpda")),
        (ServiceQuery(3477, Protocol::TCP), Service::Name("ecomm")),
        (ServiceQuery(14000, Protocol::UDP), Service::Name("scotty-ft")),
        (ServiceQuery(160, Protocol::TCP), Service::Name("sgmp-traps")),
        (ServiceQuery(5301, Protocol::UDP), Service::Name("hacl-gs")),
        (ServiceQuery(3878, Protocol::TCP), Service::Name("fotogcad")),
        (ServiceQuery(4349, Protocol::UDP), Service::Name("fsportmap")),
        (ServiceQuery(7428, Protocol::TCP), Service::Name("ovladmgr")),
        (ServiceQuery(6122, Protocol::UDP), Service::Name("bex-webadmin")),
        (ServiceQuery(7784, Protocol::TCP), Service::Name("s-bfd")),
        (ServiceQuery(2803, Protocol::UDP), Service::Name("btprjctrl")),
        (ServiceQuery(14936, Protocol::TCP), Service::Name("hde-lcesrvr-1")),
        (ServiceQuery(4093, Protocol::UDP), Service::Name("pvxpluscs")),
        (ServiceQuery(7013, Protocol::UDP), Service::Name("microtalon-dis")),
        (ServiceQuery(6315, Protocol::TCP), Service::Name("scup")),
        (ServiceQuery(1663, Protocol::TCP), Service::Name("netview-aix-3")),
        (ServiceQuery(2037, Protocol::TCP), Service::Name("applus")),
        (ServiceQuery(2191, Protocol::TCP), Service::Name("tvbus")),
        (ServiceQuery(3108, Protocol::UDP), Service::Name("geolocate")),
        (ServiceQuery(3132, Protocol::TCP), Service::Name("ms-rule-engine")),
        (ServiceQuery(35356, Protocol::TCP), Service::Name("guttersnex")),
        (ServiceQuery(12008, Protocol::UDP), Service::Name("accuracer-dbms")),
        (ServiceQuery(392, Protocol::UDP), Service::Name("synotics-broker")),
        (ServiceQuery(3579, Protocol::UDP), Service::Name("ttat3lb")),
        (ServiceQuery(1513, Protocol::TCP), Service::Name("fujitsu-dtc")),
        (ServiceQuery(415, Protocol::UDP), Service::Name("bnet")),
        (ServiceQuery(2551, Protocol::UDP), Service::Name("isg-uda-server")),
        (ServiceQuery(8043, Protocol::TCP), Service::Name("fs-server")),
        (ServiceQuery(20810, Protocol::TCP), Service::Name("crtech-nlm")),
        (ServiceQuery(6768, Protocol::TCP), Service::Name("bmc-perf-mgrd")),
        (ServiceQuery(3038, Protocol::TCP), Service::Name("santak-ups")),
        (ServiceQuery(211, Protocol::TCP), Service::Name("914c-g")),
        (ServiceQuery(1021, Protocol::TCP), Service::Name("exp1")),
        (ServiceQuery(2473, Protocol::TCP), Service::Name("aker-cdp")),
        (ServiceQuery(4204, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(632, Protocol::TCP), Service::Name("bmpp")),
        (ServiceQuery(9993, Protocol::TCP), Service::Name("palace-2")),
        (ServiceQuery(2620, Protocol::UDP), Service::Name("lpsrecommender")),
        (ServiceQuery(3264, Protocol::TCP), Service::Name("ccmail")),
        (ServiceQuery(4180, Protocol::TCP), Service::Name("httpx")),
        (ServiceQuery(3234, Protocol::UDP), Service::Name("alchemy")),
        (ServiceQuery(210, Protocol::UDP), Service::Name("z39.50")),
        (ServiceQuery(1632, Protocol::TCP), Service::Name("pammratc")),
        (ServiceQuery(3901, Protocol::TCP), Service::Name("nimsh")),
        (ServiceQuery(4055, Protocol::TCP), Service::Name("ccu-comm-3")),
        (ServiceQuery(9282, Protocol::UDP), Service::Name("swtp-port2")),
        (ServiceQuery(493, Protocol::UDP), Service::Name("ticf-2")),
        (ServiceQuery(8016, Protocol::TCP), Service::Name("ads-s")),
        (ServiceQuery(2662, Protocol::UDP), Service::Name("bintec-capi")),
        (ServiceQuery(2087, Protocol::UDP), Service::Name("eli")),
        (ServiceQuery(4569, Protocol::TCP), Service::Name("iax")),
        (ServiceQuery(3753, Protocol::TCP), Service::Name("nattyserver")),
        (ServiceQuery(3999, Protocol::UDP), Service::Name("nvcnet")),
        (ServiceQuery(5060, Protocol::UDP), Service::Name("sip")),
        (ServiceQuery(9400, Protocol::TCP), Service::Name("sec-t4net-srv")),
        (ServiceQuery(993, Protocol::UDP), Service::Name("imaps")),
        (ServiceQuery(25604, Protocol::TCP), Service::Name("idtp")),
        (ServiceQuery(2824, Protocol::TCP), Service::Name("cqg-netlan-1")),
        (ServiceQuery(8383, Protocol::TCP), Service::Name("m2mservices")),
        (ServiceQuery(26487, Protocol::TCP), Service::Name("exoconfig")),
        (ServiceQuery(3087, Protocol::TCP), Service::Name("asoki-sma")),
        (ServiceQuery(6566, Protocol::TCP), Service::Name("sane-port")),
        (ServiceQuery(8402, Protocol::UDP), Service::Name("abarsd")),
        (ServiceQuery(3823, Protocol::UDP), Service::Name("acp-conduit")),
        (ServiceQuery(2450, Protocol::TCP), Service::Name("netadmin")),
        (ServiceQuery(2900, Protocol::TCP), Service::Name("quicksuite")),
        (ServiceQuery(2885, Protocol::UDP), Service::Name("topflow")),
        (ServiceQuery(13702, Protocol::TCP), Service::Name("netbackup")),
        (ServiceQuery(678, Protocol::UDP), Service::Name("ggf-ncp")),
        (ServiceQuery(2008, Protocol::TCP), Service::Name("conf")),
        (ServiceQuery(1934, Protocol::UDP), Service::Name("xaapi")),
        (ServiceQuery(9086, Protocol::UDP), Service::Name("net2display")),
        (ServiceQuery(5600, Protocol::TCP), Service::Name("esmmanager")),
        (ServiceQuery(3912, Protocol::UDP), Service::Name("gbmt-stars")),
        (ServiceQuery(1369, Protocol::UDP), Service::Name("gv-us")),
        (ServiceQuery(2516, Protocol::UDP), Service::Name("maincontrol")),
        (ServiceQuery(4125, Protocol::UDP), Service::Name("opsview-envoy")),
        (ServiceQuery(3072, Protocol::UDP), Service::Name("csd-monitor")),
        (ServiceQuery(1000, Protocol::UDP), Service::Name("ock")),
        (ServiceQuery(5280, Protocol::TCP), Service::Name("xmpp-bosh")),
        (ServiceQuery(4726, Protocol::UDP), Service::Name("a26-fap-fgw")),
        (ServiceQuery(474, Protocol::TCP), Service::Name("tn-tl-w1")),
        (ServiceQuery(7276, Protocol::UDP), Service::Name("oma-ilp")),
        (ServiceQuery(2919, Protocol::TCP), Service::Name("roboer")),
        (ServiceQuery(6058, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(3580, Protocol::TCP), Service::Name("nati-svrloc")),
        (ServiceQuery(3764, Protocol::UDP), Service::Name("mni-prot-rout")),
        (ServiceQuery(15660, Protocol::TCP), Service::Name("bex-xr")),
        (ServiceQuery(4455, Protocol::TCP), Service::Name("prchat-user")),
        (ServiceQuery(7708, Protocol::UDP), Service::Name("scinet")),
        (ServiceQuery(2013, Protocol::TCP), Service::Name("raid-am")),
        (ServiceQuery(3869, Protocol::UDP), Service::Name("ovsam-mgmt")),
        (ServiceQuery(2428, Protocol::TCP), Service::Name("ott")),
        (ServiceQuery(195, Protocol::UDP), Service::Name("dn6-nlm-aud")),
        (ServiceQuery(4240, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(6409, Protocol::UDP), Service::Name("boe-resssvr3")),
        (ServiceQuery(6786, Protocol::TCP), Service::Name("smc-jmx")),
        (ServiceQuery(4083, Protocol::TCP), Service::Name("lorica-out-sec")),
        (ServiceQuery(3745, Protocol::UDP), Service::Name("gw-call-port")),
        (ServiceQuery(667, Protocol::UDP), Service::Name("disclose")),
        (ServiceQuery(3357, Protocol::TCP), Service::Name("adtech-test")),
        (ServiceQuery(5306, Protocol::TCP), Service::Name("sun-mc-grp")),
        (ServiceQuery(4690, Protocol::UDP), Service::Name("prelude")),
        (ServiceQuery(1772, Protocol::UDP), Service::Name("essweb-gw")),
        (ServiceQuery(5938, Protocol::TCP), Service::Name("teamviewer")),
        (ServiceQuery(5414, Protocol::TCP), Service::Name("statusd")),
        (ServiceQuery(8787, Protocol::UDP), Service::Name("msgsrvr")),
        (ServiceQuery(4277, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3359, Protocol::TCP), Service::Name("wg-netforce")),
        (ServiceQuery(1169, Protocol::TCP), Service::Name("tripwire")),
        (ServiceQuery(1818, Protocol::TCP), Service::Name("etftp")),
        (ServiceQuery(1169, Protocol::UDP), Service::Name("tripwire")),
        (ServiceQuery(2161, Protocol::TCP), Service::Name("apc-agent")),
        (ServiceQuery(4091, Protocol::TCP), Service::Name("ewinstaller")),
        (ServiceQuery(2648, Protocol::UDP), Service::Name("upsnotifyprot")),
        (ServiceQuery(3159, Protocol::TCP), Service::Name("navegaweb-port")),
        (ServiceQuery(1295, Protocol::TCP), Service::Name("ehtp")),
        (ServiceQuery(3904, Protocol::TCP), Service::Name("omnilink-port")),
        (ServiceQuery(3899, Protocol::TCP), Service::Name("itv-control")),
        (ServiceQuery(1599, Protocol::UDP), Service::Name("simbaservices")),
        (ServiceQuery(5063, Protocol::TCP), Service::Name("csrpc")),
        (ServiceQuery(3445, Protocol::UDP), Service::Name("monp")),
        (ServiceQuery(2437, Protocol::TCP), Service::Name("unicontrol")),
        (ServiceQuery(994, Protocol::TCP), Service::Name("ircs")),
        (ServiceQuery(3122, Protocol::TCP), Service::Name("vtr-emulator")),
        (ServiceQuery(8913, Protocol::UDP), Service::Name("dragonfly")),
        (ServiceQuery(3116, Protocol::TCP), Service::Name("mctet-gateway")),
        (ServiceQuery(1264, Protocol::TCP), Service::Name("prat")),
        (ServiceQuery(2266, Protocol::TCP), Service::Name("mfserver")),
        (ServiceQuery(1697, Protocol::TCP), Service::Name("rrisat")),
        (ServiceQuery(6055, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(1771, Protocol::UDP), Service::Name("vaultbase")),
        (ServiceQuery(7216, Protocol::TCP), Service::Name("PS-Capture-Pro")),
        (ServiceQuery(6482, Protocol::TCP), Service::Name("ldoms-mgmt")),
        (ServiceQuery(3054, Protocol::UDP), Service::Name("amt-cnf-prot")),
        (ServiceQuery(1798, Protocol::TCP), Service::Name("etp")),
        (ServiceQuery(663, Protocol::TCP), Service::Name("purenoise")),
        (ServiceQuery(6408, Protocol::TCP), Service::Name("boe-resssvr2")),
        (ServiceQuery(3524, Protocol::UDP), Service::Name("ecmport")),
        (ServiceQuery(4573, Protocol::TCP), Service::Name("cardifftec-back")),
        (ServiceQuery(6118, Protocol::TCP), Service::Name("tipc")),
        (ServiceQuery(2838, Protocol::UDP), Service::Name("starbot")),
        (ServiceQuery(4559, Protocol::UDP), Service::Name("hylafax")),
        (ServiceQuery(2913, Protocol::UDP), Service::Name("boosterware")),
        (ServiceQuery(3174, Protocol::UDP), Service::Name("armi-server")),
        (ServiceQuery(6068, Protocol::UDP), Service::Name("gsmp")),
        (ServiceQuery(2284, Protocol::TCP), Service::Name("lnvmaps")),
        (ServiceQuery(9101, Protocol::TCP), Service::Name("jetdirect")),
        (ServiceQuery(4252, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2890, Protocol::UDP), Service::Name("cspclmulti")),
        (ServiceQuery(119, Protocol::UDP), Service::Name("nntp")),
        (ServiceQuery(4127, Protocol::TCP), Service::Name("unikeypro")),
        (ServiceQuery(2994, Protocol::UDP), Service::Name("veritas-vis2")),
        (ServiceQuery(5787, Protocol::TCP), Service::Name("waascluster")),
        (ServiceQuery(2282, Protocol::TCP), Service::Name("lnvalarm")),
        (ServiceQuery(5911, Protocol::TCP), Service::Name("cpdlc")),
        (ServiceQuery(4279, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3970, Protocol::UDP), Service::Name("lanrevagent")),
        (ServiceQuery(2929, Protocol::UDP), Service::Name("amx-webadmin")),
        (ServiceQuery(19283, Protocol::UDP), Service::Name("keysrvr")),
        (ServiceQuery(32770, Protocol::TCP), Service::Name("sometimes-rpc3")),
        (ServiceQuery(10116, Protocol::UDP), Service::Name("netiq-voipa")),
        (ServiceQuery(2871, Protocol::TCP), Service::Name("msi-selectplay")),
        (ServiceQuery(1040, Protocol::TCP), Service::Name("netsaint")),
        (ServiceQuery(2030, Protocol::UDP), Service::Name("device2")),
        (ServiceQuery(3239, Protocol::TCP), Service::Name("apparenet-ui")),
        (ServiceQuery(256, Protocol::UDP), Service::Name("rap")),
        (ServiceQuery(1738, Protocol::TCP), Service::Name("gamegen1")),
        (ServiceQuery(3408, Protocol::UDP), Service::Name("BESApi")),
        (ServiceQuery(3394, Protocol::UDP), Service::Name("d2k-tapestry2")),
        (ServiceQuery(5028, Protocol::TCP), Service::Name("qvr")),
        (ServiceQuery(4372, Protocol::TCP), Service::Name("l2c-data")),
        (ServiceQuery(693, Protocol::UDP), Service::Name("connendp")),
        (ServiceQuery(3057, Protocol::UDP), Service::Name("goahead-fldup")),
        (ServiceQuery(4008, Protocol::UDP), Service::Name("netcheque")),
        (ServiceQuery(1277, Protocol::UDP), Service::Name("miva-mqs")),
        (ServiceQuery(3328, Protocol::UDP), Service::Name("egptlm")),
        (ServiceQuery(1282, Protocol::TCP), Service::Name("emperion")),
        (ServiceQuery(406, Protocol::TCP), Service::Name("imsp")),
        (ServiceQuery(4098, Protocol::TCP), Service::Name("drmsfsd")),
        (ServiceQuery(12346, Protocol::TCP), Service::Name("netbus")),
        (ServiceQuery(7015, Protocol::UDP), Service::Name("talon-webserver")),
        (ServiceQuery(2572, Protocol::TCP), Service::Name("ibp")),
        (ServiceQuery(2623, Protocol::UDP), Service::Name("lmdp")),
        (ServiceQuery(4240, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3632, Protocol::UDP), Service::Name("distcc")),
        (ServiceQuery(4851, Protocol::UDP), Service::Name("derby-repli")),
        (ServiceQuery(714, Protocol::TCP), Service::Name("iris-xpcs")),
        (ServiceQuery(2958, Protocol::TCP), Service::Name("jmact6")),
        (ServiceQuery(5001, Protocol::TCP), Service::Name("commplex-link")),
        (ServiceQuery(4177, Protocol::UDP), Service::Name("wello")),
        (ServiceQuery(4550, Protocol::UDP), Service::Name("gds-adppiw-db")),
        (ServiceQuery(1987, Protocol::UDP), Service::Name("tr-rsrb-p1")),
        (ServiceQuery(4802, Protocol::TCP), Service::Name("ilss")),
        (ServiceQuery(4202, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2571, Protocol::TCP), Service::Name("cecsvc")),
        (ServiceQuery(5343, Protocol::UDP), Service::Name("kfserver")),
        (ServiceQuery(1875, Protocol::TCP), Service::Name("westell-stats")),
        (ServiceQuery(28080, Protocol::TCP), Service::Name("thor-engine")),
        (ServiceQuery(1652, Protocol::TCP), Service::Name("xnmp")),
        (ServiceQuery(2336, Protocol::TCP), Service::Name("appleugcontrol")),
        (ServiceQuery(2845, Protocol::UDP), Service::Name("bpcp-trap")),
        (ServiceQuery(2625, Protocol::TCP), Service::Name("blwnkl-port")),
        (ServiceQuery(3516, Protocol::TCP), Service::Name("smartcard-port")),
        (ServiceQuery(171, Protocol::UDP), Service::Name("multiplex")),
        (ServiceQuery(1234, Protocol::UDP), Service::Name("search-agent")),
        (ServiceQuery(5722, Protocol::TCP), Service::Name("msdfsr")),
        (ServiceQuery(2344, Protocol::TCP), Service::Name("fcmsys")),
        (ServiceQuery(2654, Protocol::UDP), Service::Name("corel_vncadmin")),
        (ServiceQuery(9293, Protocol::TCP), Service::Name("storview")),
        (ServiceQuery(1688, Protocol::UDP), Service::Name("nsjtp-data")),
        (ServiceQuery(3118, Protocol::TCP), Service::Name("pkagent")),
        (ServiceQuery(1785, Protocol::UDP), Service::Name("windlm")),
        (ServiceQuery(3350, Protocol::TCP), Service::Name("findviatv")),
        (ServiceQuery(1029, Protocol::UDP), Service::Name("solid-mux")),
        (ServiceQuery(6209, Protocol::TCP), Service::Name("qmtps")),
        (ServiceQuery(1329, Protocol::TCP), Service::Name("netdb-export")),
        (ServiceQuery(6008, Protocol::UDP), Service::Name("X11:8")),
        (ServiceQuery(13223, Protocol::TCP), Service::Name("powwow-client")),
        (ServiceQuery(3556, Protocol::TCP), Service::Name("sky-transport")),
        (ServiceQuery(1146, Protocol::UDP), Service::Name("audit-transfer")),
        (ServiceQuery(9535, Protocol::TCP), Service::Name("man")),
        (ServiceQuery(1518, Protocol::TCP), Service::Name("vpvd")),
        (ServiceQuery(2973, Protocol::UDP), Service::Name("svnetworks")),
        (ServiceQuery(4299, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(998, Protocol::TCP), Service::Name("busboy")),
        (ServiceQuery(32769, Protocol::UDP), Service::Name("filenet-rpc")),
        (ServiceQuery(3156, Protocol::TCP), Service::Name("indura")),
        (ServiceQuery(45054, Protocol::TCP), Service::Name("invision-ag")),
        (ServiceQuery(179, Protocol::UDP), Service::Name("bgp")),
        (ServiceQuery(2621, Protocol::UDP), Service::Name("miles-apart")),
        (ServiceQuery(4212, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(34249, Protocol::UDP), Service::Name("turbonote-2")),
        (ServiceQuery(1547, Protocol::TCP), Service::Name("laplink")),
        (ServiceQuery(191, Protocol::UDP), Service::Name("prospero")),
        (ServiceQuery(545, Protocol::UDP), Service::Name("appleqtcsrvr")),
        (ServiceQuery(786, Protocol::UDP), Service::Name("concert")),
        (ServiceQuery(349, Protocol::TCP), Service::Name("mftp")),
        (ServiceQuery(2751, Protocol::UDP), Service::Name("fjippol-port2")),
        (ServiceQuery(4441, Protocol::UDP), Service::Name("netblox")),
        (ServiceQuery(7278, Protocol::UDP), Service::Name("oma-dcdocbs")),
        (ServiceQuery(2544, Protocol::TCP), Service::Name("novell-zen")),
        (ServiceQuery(2218, Protocol::UDP), Service::Name("bounzza")),
        (ServiceQuery(3576, Protocol::TCP), Service::Name("cmc-port")),
        (ServiceQuery(24321, Protocol::TCP), Service::Name("ild")),
        (ServiceQuery(3309, Protocol::UDP), Service::Name("tns-adv")),
        (ServiceQuery(4555, Protocol::TCP), Service::Name("rsip")),
        (ServiceQuery(3078, Protocol::UDP), Service::Name("orbix-cfg-ssl")),
        (ServiceQuery(2940, Protocol::UDP), Service::Name("sm-pas-3")),
        (ServiceQuery(673, Protocol::UDP), Service::Name("cimplex")),
        (ServiceQuery(1992, Protocol::UDP), Service::Name("stun-p3")),
        (ServiceQuery(1745, Protocol::UDP), Service::Name("remote-winsock")),
        (ServiceQuery(3049, Protocol::UDP), Service::Name("cfs")),
        (ServiceQuery(362, Protocol::UDP), Service::Name("srssend")),
        (ServiceQuery(34567, Protocol::TCP), Service::Name("dhanalakshmi")),
        (ServiceQuery(2809, Protocol::UDP), Service::Name("corbaloc")),
        (ServiceQuery(13724, Protocol::TCP), Service::Name("vnetd")),
        (ServiceQuery(4086, Protocol::TCP), Service::Name("ftsync")),
        (ServiceQuery(559, Protocol::UDP), Service::Name("teedtap")),
        (ServiceQuery(1564, Protocol::TCP), Service::Name("pay-per-view")),
        (ServiceQuery(6086, Protocol::TCP), Service::Name("pdtp")),
        (ServiceQuery(10990, Protocol::TCP), Service::Name("rmiaux")),
        (ServiceQuery(6301, Protocol::UDP), Service::Name("bmc_ctd_ldap")),
        (ServiceQuery(4128, Protocol::UDP), Service::Name("nufw")),
        (ServiceQuery(3090, Protocol::TCP), Service::Name("stss")),
        (ServiceQuery(2278, Protocol::UDP), Service::Name("s3db")),
        (ServiceQuery(2368, Protocol::UDP), Service::Name("opentable")),
        (ServiceQuery(32779, Protocol::UDP), Service::Name("sometimes-rpc22")),
        (ServiceQuery(4216, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1436, Protocol::UDP), Service::Name("sas-2")),
        (ServiceQuery(4803, Protocol::TCP), Service::Name("notateit")),
        (ServiceQuery(2067, Protocol::TCP), Service::Name("dlswpn")),
        (ServiceQuery(783, Protocol::TCP), Service::Name("spamassassin")),
        (ServiceQuery(1966, Protocol::TCP), Service::Name("slush")),
        (ServiceQuery(2774, Protocol::TCP), Service::Name("rbakcup2")),
        (ServiceQuery(48049, Protocol::TCP), Service::Name("3gpp-cbsp")),
        (ServiceQuery(4324, Protocol::UDP), Service::Name("balour")),
        (ServiceQuery(7272, Protocol::TCP), Service::Name("watchme-7272")),
        (ServiceQuery(4001, Protocol::UDP), Service::Name("newoak")),
        (ServiceQuery(7274, Protocol::TCP), Service::Name("oma-rlp-s")),
        (ServiceQuery(1696, Protocol::UDP), Service::Name("rrifmm")),
        (ServiceQuery(8501, Protocol::TCP), Service::Name("cmtp-mgt")),
        (ServiceQuery(4207, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1154, Protocol::TCP), Service::Name("resacommunity")),
        (ServiceQuery(4289, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3048, Protocol::UDP), Service::Name("pctrader")),
        (ServiceQuery(2943, Protocol::UDP), Service::Name("ttnrepository")),
        (ServiceQuery(3321, Protocol::TCP), Service::Name("vnsstr")),
        (ServiceQuery(2118, Protocol::UDP), Service::Name("mentaserver")),
        (ServiceQuery(2666, Protocol::UDP), Service::Name("extensis")),
        (ServiceQuery(1381, Protocol::UDP), Service::Name("apple-licman")),
        (ServiceQuery(469, Protocol::TCP), Service::Name("rcp")),
        (ServiceQuery(27665, Protocol::TCP), Service::Name("Trinoo_Master")),
        (ServiceQuery(1220, Protocol::UDP), Service::Name("qt-serveradmin")),
        (ServiceQuery(30003, Protocol::TCP), Service::Name("amicon-fpsu-ra")),
        (ServiceQuery(38203, Protocol::UDP), Service::Name("agpolicy")),
        (ServiceQuery(4986, Protocol::TCP), Service::Name("mrip")),
        (ServiceQuery(3598, Protocol::TCP), Service::Name("a15")),
        (ServiceQuery(3609, Protocol::UDP), Service::Name("cpdi-pidas-cm")),
        (ServiceQuery(5044, Protocol::UDP), Service::Name("lxi-evntsvc")),
        (ServiceQuery(9205, Protocol::TCP), Service::Name("wap-vcal")),
        (ServiceQuery(5900, Protocol::UDP), Service::Name("rfb")),
        (ServiceQuery(3604, Protocol::UDP), Service::Name("bmc-jmx-port")),
        (ServiceQuery(1628, Protocol::UDP), Service::Name("lontalk-norm")),
        (ServiceQuery(900, Protocol::TCP), Service::Name("omginitialrefs")),
        (ServiceQuery(4032, Protocol::UDP), Service::Name("vrts-auth-port")),
        (ServiceQuery(2070, Protocol::TCP), Service::Name("ah-esp-encap")),
        (ServiceQuery(212, Protocol::TCP), Service::Name("anet")),
        (ServiceQuery(829, Protocol::UDP), Service::Name("pkix-3-ca-ra")),
        (ServiceQuery(605, Protocol::TCP), Service::Name("soap-beep")),
        (ServiceQuery(3381, Protocol::TCP), Service::Name("geneous")),
        (ServiceQuery(5687, Protocol::TCP), Service::Name("gog-multiplayer")),
        (ServiceQuery(596, Protocol::UDP), Service::Name("smsd")),
        (ServiceQuery(3932, Protocol::UDP), Service::Name("dyn-site")),
        (ServiceQuery(3921, Protocol::TCP), Service::Name("herodotus-net")),
        (ServiceQuery(2752, Protocol::UDP), Service::Name("rsisysaccess")),
        (ServiceQuery(200, Protocol::UDP), Service::Name("src")),
        (ServiceQuery(3488, Protocol::UDP), Service::Name("fs-rh-srv")),
        (ServiceQuery(3277, Protocol::UDP), Service::Name("awg-proxy")),
        (ServiceQuery(1733, Protocol::TCP), Service::Name("siipat")),
        (ServiceQuery(1125, Protocol::TCP), Service::Name("hpvmmagent")),
        (ServiceQuery(3322, Protocol::UDP), Service::Name("active-net")),
        (ServiceQuery(15002, Protocol::TCP), Service::Name("onep-tls")),
        (ServiceQuery(1374, Protocol::UDP), Service::Name("molly")),
        (ServiceQuery(3782, Protocol::UDP), Service::Name("iso-tp0s")),
        (ServiceQuery(4847, Protocol::UDP), Service::Name("wfc")),
        (ServiceQuery(3795, Protocol::TCP), Service::Name("myblast")),
        (ServiceQuery(4250, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2993, Protocol::TCP), Service::Name("veritas-vis1")),
        (ServiceQuery(133, Protocol::TCP), Service::Name("statsrv")),
        (ServiceQuery(48128, Protocol::UDP), Service::Name("isnetserv")),
        (ServiceQuery(7174, Protocol::UDP), Service::Name("clutild")),
        (ServiceQuery(24554, Protocol::UDP), Service::Name("binkp")),
        (ServiceQuery(1378, Protocol::TCP), Service::Name("elan")),
        (ServiceQuery(4753, Protocol::TCP), Service::Name("simon")),
        (ServiceQuery(1354, Protocol::UDP), Service::Name("rightbrain")),
        (ServiceQuery(2534, Protocol::TCP), Service::Name("combox-web-acc")),
        (ServiceQuery(2246, Protocol::UDP), Service::Name("pc-mta-addrmap")),
        (ServiceQuery(3779, Protocol::UDP), Service::Name("cognima")),
        (ServiceQuery(5023, Protocol::UDP), Service::Name("htuilsrv")),
        (ServiceQuery(3787, Protocol::TCP), Service::Name("fintrx")),
        (ServiceQuery(677, Protocol::TCP), Service::Name("vpp")),
        (ServiceQuery(8768, Protocol::TCP), Service::Name("sandpolis")),
        (ServiceQuery(4290, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(5991, Protocol::TCP), Service::Name("nuxsl")),
        (ServiceQuery(45001, Protocol::TCP), Service::Name("asmps")),
        (ServiceQuery(3938, Protocol::UDP), Service::Name("dbcontrol_agent")),
        (ServiceQuery(7663, Protocol::TCP), Service::Name("rome")),
        (ServiceQuery(27002, Protocol::UDP), Service::Name("flex-lm")),
        (ServiceQuery(7340, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(6406, Protocol::TCP), Service::Name("boe-processsvr")),
        (ServiceQuery(11371, Protocol::UDP), Service::Name("hkp")),
        (ServiceQuery(2377, Protocol::TCP), Service::Name("swarm")),
        (ServiceQuery(30001, Protocol::TCP), Service::Name("pago-services1")),
        (ServiceQuery(4094, Protocol::TCP), Service::Name("sysrqd")),
        (ServiceQuery(211, Protocol::UDP), Service::Name("914c-g")),
        (ServiceQuery(3606, Protocol::TCP), Service::Name("splitlock")),
        (ServiceQuery(5635, Protocol::TCP), Service::Name("sfmsso")),
        (ServiceQuery(2176, Protocol::UDP), Service::Name("rapi")),
        (ServiceQuery(101, Protocol::TCP), Service::Name("hostname")),
        (ServiceQuery(7307, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(710, Protocol::UDP), Service::Name("entrust-ash")),
        (ServiceQuery(4673, Protocol::TCP), Service::Name("cxws")),
        (ServiceQuery(8015, Protocol::TCP), Service::Name("cfg-cloud")),
        (ServiceQuery(7020, Protocol::TCP), Service::Name("dpserve")),
        (ServiceQuery(10128, Protocol::TCP), Service::Name("bmc-perf-sd")),
        (ServiceQuery(1260, Protocol::UDP), Service::Name("ibm-ssd")),
        (ServiceQuery(6039, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(1381, Protocol::TCP), Service::Name("apple-licman")),
        (ServiceQuery(3821, Protocol::UDP), Service::Name("pmcp")),
        (ServiceQuery(359, Protocol::UDP), Service::Name("tenebris_nts")),
        (ServiceQuery(6633, Protocol::TCP), Service::Name("cisco-vpath-tun")),
        (ServiceQuery(4002, Protocol::TCP), Service::Name("mlchat-proxy")),
        (ServiceQuery(2772, Protocol::UDP), Service::Name("auris")),
        (ServiceQuery(18, Protocol::UDP), Service::Name("msp")),
        (ServiceQuery(5102, Protocol::UDP), Service::Name("oms-nonsecure")),
        (ServiceQuery(6061, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(7880, Protocol::UDP), Service::Name("pss")),
        (ServiceQuery(20999, Protocol::UDP), Service::Name("athand-mmp")),
        (ServiceQuery(3329, Protocol::TCP), Service::Name("hp-device-disc")),
        (ServiceQuery(2132, Protocol::UDP), Service::Name("solera-epmap")),
        (ServiceQuery(9215, Protocol::TCP), Service::Name("insis")),
        (ServiceQuery(2758, Protocol::UDP), Service::Name("apollo-status")),
        (ServiceQuery(5909, Protocol::TCP), Service::Name("agma")),
        (ServiceQuery(3023, Protocol::TCP), Service::Name("magicnotes")),
        (ServiceQuery(5900, Protocol::TCP), Service::Name("vnc")),
        (ServiceQuery(4306, Protocol::TCP), Service::Name("pinghgl")),
        (ServiceQuery(3039, Protocol::UDP), Service::Name("cogitate")),
        (ServiceQuery(3131, Protocol::UDP), Service::Name("netbookmark")),
        (ServiceQuery(7335, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(3392, Protocol::TCP), Service::Name("efi-lm")),
        (ServiceQuery(3758, Protocol::TCP), Service::Name("apw-registry")),
        (ServiceQuery(3845, Protocol::TCP), Service::Name("v-one-spp")),
        (ServiceQuery(9344, Protocol::TCP), Service::Name("mphlpdmc")),
        (ServiceQuery(6505, Protocol::UDP), Service::Name("badm_priv")),
        (ServiceQuery(3654, Protocol::TCP), Service::Name("vaprtm")),
        (ServiceQuery(6581, Protocol::UDP), Service::Name("parsec-peer")),
        (ServiceQuery(3657, Protocol::UDP), Service::Name("immedianet-bcn")),
        (ServiceQuery(663, Protocol::UDP), Service::Name("purenoise")),
        (ServiceQuery(15999, Protocol::TCP), Service::Name("programmar")),
        (ServiceQuery(4059, Protocol::TCP), Service::Name("dlms-cosem")),
        (ServiceQuery(5963, Protocol::TCP), Service::Name("indy")),
        (ServiceQuery(4458, Protocol::TCP), Service::Name("mcp")),
        (ServiceQuery(64, Protocol::TCP), Service::Name("covia")),
        (ServiceQuery(3375, Protocol::UDP), Service::Name("vsnm-agent")),
        (ServiceQuery(3879, Protocol::UDP), Service::Name("appss-lm")),
        (ServiceQuery(2552, Protocol::TCP), Service::Name("call-logging")),
        (ServiceQuery(10162, Protocol::UDP), Service::Name("snmpdtls-trap")),
        (ServiceQuery(2728, Protocol::TCP), Service::Name("sqdr")),
        (ServiceQuery(8129, Protocol::TCP), Service::Name("paycash-wbp")),
        (ServiceQuery(3988, Protocol::UDP), Service::Name("dcs-config")),
        (ServiceQuery(2660, Protocol::TCP), Service::Name("gcmonitor")),
        (ServiceQuery(6162, Protocol::UDP), Service::Name("patrol-coll")),
        (ServiceQuery(5359, Protocol::TCP), Service::Name("ms-alerter")),
        (ServiceQuery(106, Protocol::UDP), Service::Name("3com-tsmux")),
        (ServiceQuery(4537, Protocol::TCP), Service::Name("wssauthsvc")),
        (ServiceQuery(5913, Protocol::UDP), Service::Name("ads-c")),
        (ServiceQuery(555, Protocol::UDP), Service::Name("dsf")),
        (ServiceQuery(3134, Protocol::TCP), Service::Name("ecp")),
        (ServiceQuery(6406, Protocol::UDP), Service::Name("boe-processsvr")),
        (ServiceQuery(1832, Protocol::TCP), Service::Name("tht-treasure")),
        (ServiceQuery(1935, Protocol::UDP), Service::Name("macromedia-fcs")),
        (ServiceQuery(2235, Protocol::UDP), Service::Name("sercomm-wlink")),
        (ServiceQuery(1882, Protocol::TCP), Service::Name("ecsqdmn")),
        (ServiceQuery(4040, Protocol::TCP), Service::Name("yo-main")),
        (ServiceQuery(3068, Protocol::TCP), Service::Name("ls3bcast")),
        (ServiceQuery(1559, Protocol::UDP), Service::Name("web2host")),
        (ServiceQuery(4186, Protocol::TCP), Service::Name("boxbackupstore")),
        (ServiceQuery(1755, Protocol::TCP), Service::Name("wms")),
        (ServiceQuery(591, Protocol::UDP), Service::Name("http-alt")),
        (ServiceQuery(8231, Protocol::TCP), Service::Name("hncp-udp-port")),
        (ServiceQuery(5543, Protocol::TCP), Service::Name("qftest-licserve")),
        (ServiceQuery(2163, Protocol::TCP), Service::Name("navisphere-sec")),
        (ServiceQuery(4011, Protocol::TCP), Service::Name("altserviceboot")),
        (ServiceQuery(521, Protocol::UDP), Service::Name("ripng")),
        (ServiceQuery(3437, Protocol::TCP), Service::Name("autocueds")),
        (ServiceQuery(5166, Protocol::UDP), Service::Name("winpcs")),
        (ServiceQuery(3762, Protocol::TCP), Service::Name("gbs-smp")),
        (ServiceQuery(5151, Protocol::UDP), Service::Name("esri_sde")),
        (ServiceQuery(5306, Protocol::UDP), Service::Name("sun-mc-grp")),
        (ServiceQuery(5602, Protocol::TCP), Service::Name("a1-msc")),
        (ServiceQuery(3217, Protocol::TCP), Service::Name("unite")),
        (ServiceQuery(886, Protocol::TCP), Service::Name("iclcnet-locate")),
        (ServiceQuery(24386, Protocol::TCP), Service::Name("intel_rci")),
        (ServiceQuery(2935, Protocol::TCP), Service::Name("qtp")),
        (ServiceQuery(1805, Protocol::UDP), Service::Name("enl-name")),
        (ServiceQuery(490, Protocol::UDP), Service::Name("micom-pfs")),
        (ServiceQuery(5583, Protocol::UDP), Service::Name("tmo-icon-sync")),
        (ServiceQuery(9211, Protocol::UDP), Service::Name("oma-mlp-s")),
        (ServiceQuery(7781, Protocol::UDP), Service::Name("accu-lmgr")),
        (ServiceQuery(5067, Protocol::TCP), Service::Name("authentx")),
        (ServiceQuery(7300, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(2055, Protocol::TCP), Service::Name("iop")),
        (ServiceQuery(992, Protocol::UDP), Service::Name("telnets")),
        (ServiceQuery(3656, Protocol::UDP), Service::Name("abatjss")),
        (ServiceQuery(6105, Protocol::TCP), Service::Name("isdninfo")),
        (ServiceQuery(3269, Protocol::TCP), Service::Name("globalcatLDAPssl")),
        (ServiceQuery(2932, Protocol::TCP), Service::Name("incp")),
        (ServiceQuery(2988, Protocol::TCP), Service::Name("hippad")),
        (ServiceQuery(7437, Protocol::UDP), Service::Name("faximum")),
        (ServiceQuery(1631, Protocol::TCP), Service::Name("visitview")),
        (ServiceQuery(2168, Protocol::TCP), Service::Name("easy-soft-mux")),
        (ServiceQuery(1573, Protocol::TCP), Service::Name("itscomm-ns")),
        (ServiceQuery(5968, Protocol::TCP), Service::Name("mppolicy-v5")),
        (ServiceQuery(516, Protocol::UDP), Service::Name("videotex")),
        (ServiceQuery(2962, Protocol::TCP), Service::Name("iph-policy-cli")),
        (ServiceQuery(2920, Protocol::TCP), Service::Name("roboeda")),
        (ServiceQuery(26260, Protocol::TCP), Service::Name("ezproxy")),
        (ServiceQuery(8002, Protocol::TCP), Service::Name("teradataordbms")),
        (ServiceQuery(7336, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(688, Protocol::TCP), Service::Name("realm-rusd")),
        (ServiceQuery(4199, Protocol::TCP), Service::Name("eims-admin")),
        (ServiceQuery(1284, Protocol::UDP), Service::Name("iee-qfx")),
        (ServiceQuery(321, Protocol::TCP), Service::Name("pip")),
        (ServiceQuery(644, Protocol::TCP), Service::Name("dwr")),
        (ServiceQuery(2312, Protocol::UDP), Service::Name("wanscaler")),
        (ServiceQuery(6626, Protocol::UDP), Service::Name("wago-service")),
        (ServiceQuery(4554, Protocol::TCP), Service::Name("msfrs")),
        (ServiceQuery(767, Protocol::TCP), Service::Name("phonebook")),
        (ServiceQuery(3900, Protocol::TCP), Service::Name("udt_os")),
        (ServiceQuery(1735, Protocol::UDP), Service::Name("privatechat")),
        (ServiceQuery(441, Protocol::UDP), Service::Name("decvms-sysmgt")),
        (ServiceQuery(2766, Protocol::UDP), Service::Name("compaq-scp")),
        (ServiceQuery(1090, Protocol::UDP), Service::Name("ff-fms")),
        (ServiceQuery(6343, Protocol::TCP), Service::Name("sflow")),
        (ServiceQuery(689, Protocol::TCP), Service::Name("nmap")),
        (ServiceQuery(2496, Protocol::UDP), Service::Name("dirgis")),
        (ServiceQuery(5406, Protocol::TCP), Service::Name("systemics-sox")),
        (ServiceQuery(2953, Protocol::UDP), Service::Name("ovalarmsrv")),
        (ServiceQuery(5203, Protocol::UDP), Service::Name("targus-getdata3")),
        (ServiceQuery(198, Protocol::UDP), Service::Name("dls-mon")),
        (ServiceQuery(3079, Protocol::UDP), Service::Name("lv-frontpanel")),
        (ServiceQuery(1288, Protocol::UDP), Service::Name("navbuddy")),
        (ServiceQuery(4555, Protocol::UDP), Service::Name("rsip")),
        (ServiceQuery(11367, Protocol::TCP), Service::Name("atm-uhas")),
        (ServiceQuery(129, Protocol::TCP), Service::Name("pwdgen")),
        (ServiceQuery(4229, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1401, Protocol::TCP), Service::Name("goldleaf-licman")),
        (ServiceQuery(199, Protocol::TCP), Service::Name("smux")),
        (ServiceQuery(1197, Protocol::UDP), Service::Name("carrius-rshell")),
        (ServiceQuery(1148, Protocol::TCP), Service::Name("elfiq-repl")),
        (ServiceQuery(5093, Protocol::TCP), Service::Name("sentinel-lm")),
        (ServiceQuery(12008, Protocol::TCP), Service::Name("accuracer-dbms")),
        (ServiceQuery(4752, Protocol::TCP), Service::Name("snap")),
        (ServiceQuery(3464, Protocol::TCP), Service::Name("edm-mgr-sync")),
        (ServiceQuery(7346, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(1338, Protocol::UDP), Service::Name("wmc-log-svc")),
        (ServiceQuery(2260, Protocol::UDP), Service::Name("apc-2260")),
        (ServiceQuery(3857, Protocol::TCP), Service::Name("trap-port")),
        (ServiceQuery(3551, Protocol::TCP), Service::Name("apcupsd")),
        (ServiceQuery(1868, Protocol::UDP), Service::Name("viziblebrowser")),
        (ServiceQuery(3920, Protocol::TCP), Service::Name("exasoftport1")),
        (ServiceQuery(2124, Protocol::TCP), Service::Name("elatelink")),
        (ServiceQuery(1387, Protocol::UDP), Service::Name("cadsi-lm")),
        (ServiceQuery(1647, Protocol::TCP), Service::Name("rsap")),
        (ServiceQuery(2285, Protocol::TCP), Service::Name("lnvmailmon")),
        (ServiceQuery(2433, Protocol::UDP), Service::Name("codasrv-se")),
        (ServiceQuery(2211, Protocol::TCP), Service::Name("emwin")),
        (ServiceQuery(144, Protocol::TCP), Service::Name("news")),
        (ServiceQuery(509, Protocol::UDP), Service::Name("snare")),
        (ServiceQuery(2719, Protocol::UDP), Service::Name("scan-change")),
        (ServiceQuery(11723, Protocol::TCP), Service::Name("emc-xsw-dcache")),
        (ServiceQuery(93, Protocol::TCP), Service::Name("dcp")),
        (ServiceQuery(1977, Protocol::UDP), Service::Name("tcoaddressbook")),
        (ServiceQuery(2784, Protocol::UDP), Service::Name("www-dev")),
        (ServiceQuery(2086, Protocol::TCP), Service::Name("gnunet")),
        (ServiceQuery(4027, Protocol::TCP), Service::Name("bxp")),
        (ServiceQuery(9750, Protocol::TCP), Service::Name("board-voip")),
        (ServiceQuery(2532, Protocol::UDP), Service::Name("ovtopmd")),
        (ServiceQuery(800, Protocol::UDP), Service::Name("mdbs_daemon")),
        (ServiceQuery(2207, Protocol::TCP), Service::Name("hpssd")),
        (ServiceQuery(2786, Protocol::UDP), Service::Name("aic-oncrpc")),
        (ServiceQuery(7337, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(2338, Protocol::UDP), Service::Name("norton-lambert")),
        (ServiceQuery(2691, Protocol::TCP), Service::Name("itinternet")),
        (ServiceQuery(35004, Protocol::TCP), Service::Name("rt-classmanager")),
        (ServiceQuery(6656, Protocol::TCP), Service::Name("emgmsg")),
        (ServiceQuery(4201, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(9321, Protocol::UDP), Service::Name("guibase")),
        (ServiceQuery(3551, Protocol::UDP), Service::Name("apcupsd")),
        (ServiceQuery(3450, Protocol::UDP), Service::Name("castorproxy")),
        (ServiceQuery(3593, Protocol::TCP), Service::Name("bpmd")),
        (ServiceQuery(432, Protocol::TCP), Service::Name("iasd")),
        (ServiceQuery(5084, Protocol::UDP), Service::Name("llrp")),
        (ServiceQuery(4024, Protocol::TCP), Service::Name("tnp1-port")),
        (ServiceQuery(1812, Protocol::UDP), Service::Name("radius")),
        (ServiceQuery(62, Protocol::TCP), Service::Name("acas")),
        (ServiceQuery(2928, Protocol::UDP), Service::Name("redstone-cpss")),
        (ServiceQuery(3493, Protocol::UDP), Service::Name("nut")),
        (ServiceQuery(29999, Protocol::TCP), Service::Name("bingbang")),
        (ServiceQuery(2439, Protocol::TCP), Service::Name("sybasedbsynch")),
        (ServiceQuery(2088, Protocol::TCP), Service::Name("ip-blf")),
        (ServiceQuery(9201, Protocol::UDP), Service::Name("wap-wsp-wtp")),
        (ServiceQuery(1659, Protocol::UDP), Service::Name("sg-lm")),
        (ServiceQuery(32778, Protocol::UDP), Service::Name("sometimes-rpc20")),
        (ServiceQuery(5550, Protocol::TCP), Service::Name("sdadmind")),
        (ServiceQuery(6998, Protocol::TCP), Service::Name("iatp-highpri")),
        (ServiceQuery(6547, Protocol::UDP), Service::Name("apc-6547")),
        (ServiceQuery(2755, Protocol::TCP), Service::Name("expresspay")),
        (ServiceQuery(7421, Protocol::UDP), Service::Name("mtportmon")),
        (ServiceQuery(2796, Protocol::UDP), Service::Name("ac-tech")),
        (ServiceQuery(3113, Protocol::TCP), Service::Name("cs-auth-svr")),
        (ServiceQuery(4484, Protocol::UDP), Service::Name("hpssmgmt")),
        (ServiceQuery(4169, Protocol::UDP), Service::Name("iadt-disc")),
        (ServiceQuery(5755, Protocol::UDP), Service::Name("openmailg")),
        (ServiceQuery(25005, Protocol::TCP), Service::Name("icl-twobase6")),
        (ServiceQuery(3795, Protocol::UDP), Service::Name("myblast")),
        (ServiceQuery(25005, Protocol::UDP), Service::Name("icl-twobase6")),
        (ServiceQuery(3573, Protocol::TCP), Service::Name("tag-ups-1")),
        (ServiceQuery(1604, Protocol::TCP), Service::Name("icabrowser")),
        (ServiceQuery(15740, Protocol::TCP), Service::Name("ptp")),
        (ServiceQuery(4403, Protocol::UDP), Service::Name("ds-user")),
        (ServiceQuery(3884, Protocol::UDP), Service::Name("softrack-meter")),
        (ServiceQuery(3469, Protocol::TCP), Service::Name("pluribus")),
        (ServiceQuery(3699, Protocol::TCP), Service::Name("kpn-icw")),
        (ServiceQuery(2765, Protocol::TCP), Service::Name("qip-audup")),
        (ServiceQuery(9025, Protocol::UDP), Service::Name("swa-3")),
        (ServiceQuery(1972, Protocol::UDP), Service::Name("intersys-cache")),
        (ServiceQuery(4670, Protocol::TCP), Service::Name("light")),
        (ServiceQuery(6101, Protocol::UDP), Service::Name("synchronet-rtc")),
        (ServiceQuery(4149, Protocol::UDP), Service::Name("agslb")),
        (ServiceQuery(2894, Protocol::TCP), Service::Name("abacus-remote")),
        (ServiceQuery(3803, Protocol::TCP), Service::Name("soniqsync")),
        (ServiceQuery(3245, Protocol::TCP), Service::Name("vieo-fe")),
        (ServiceQuery(1426, Protocol::TCP), Service::Name("sas-1")),
        (ServiceQuery(2806, Protocol::UDP), Service::Name("cspuni")),
        (ServiceQuery(11877, Protocol::UDP), Service::Name("x2e-disc")),
        (ServiceQuery(1449, Protocol::UDP), Service::Name("peport")),
        (ServiceQuery(3949, Protocol::UDP), Service::Name("drip")),
        (ServiceQuery(3492, Protocol::TCP), Service::Name("tvdumtray-port")),
        (ServiceQuery(83, Protocol::TCP), Service::Name("mit-ml-dev")),
        (ServiceQuery(2307, Protocol::TCP), Service::Name("pehelp")),
        (ServiceQuery(1743, Protocol::UDP), Service::Name("cinegrfx-lm")),
        (ServiceQuery(7650, Protocol::UDP), Service::Name("cucme-3")),
        (ServiceQuery(5202, Protocol::UDP), Service::Name("targus-getdata2")),
        (ServiceQuery(3848, Protocol::TCP), Service::Name("item")),
        (ServiceQuery(1508, Protocol::UDP), Service::Name("diagmond")),
        (ServiceQuery(1468, Protocol::TCP), Service::Name("csdm")),
        (ServiceQuery(1307, Protocol::TCP), Service::Name("pacmand")),
        (ServiceQuery(2351, Protocol::UDP), Service::Name("psrserver")),
        (ServiceQuery(137, Protocol::TCP), Service::Name("netbios-ns")),
        (ServiceQuery(2861, Protocol::TCP), Service::Name("dialpad-voice2")),
        (ServiceQuery(12322, Protocol::UDP), Service::Name("warehouse")),
        (ServiceQuery(8764, Protocol::TCP), Service::Name("openqueue")),
        (ServiceQuery(3382, Protocol::UDP), Service::Name("fujitsu-neat")),
        (ServiceQuery(24465, Protocol::UDP), Service::Name("tonidods")),
        (ServiceQuery(20057, Protocol::TCP), Service::Name("avesterra")),
        (ServiceQuery(244, Protocol::UDP), Service::Name("dayna")),
        (ServiceQuery(9203, Protocol::UDP), Service::Name("wap-wsp-wtp-s")),
        (ServiceQuery(592, Protocol::UDP), Service::Name("eudora-set")),
        (ServiceQuery(1085, Protocol::UDP), Service::Name("webobjects")),
        (ServiceQuery(5462, Protocol::UDP), Service::Name("ttl-publisher")),
        (ServiceQuery(1406, Protocol::UDP), Service::Name("netlabs-lm")),
        (ServiceQuery(2101, Protocol::UDP), Service::Name("rtcm-sc104")),
        (ServiceQuery(352, Protocol::UDP), Service::Name("dtag-ste-sb")),
        (ServiceQuery(7003, Protocol::UDP), Service::Name("afs3-vlserver")),
        (ServiceQuery(3894, Protocol::TCP), Service::Name("syam-agent")),
        (ServiceQuery(2106, Protocol::UDP), Service::Name("ekshell")),
        (ServiceQuery(18635, Protocol::UDP), Service::Name("rds-ip")),
        (ServiceQuery(2484, Protocol::TCP), Service::Name("ttc-ssl")),
        (ServiceQuery(8032, Protocol::TCP), Service::Name("pro-ed")),
        (ServiceQuery(1622, Protocol::TCP), Service::Name("ontime")),
        (ServiceQuery(361, Protocol::UDP), Service::Name("semantix")),
        (ServiceQuery(2343, Protocol::UDP), Service::Name("nati-logos")),
        (ServiceQuery(888, Protocol::UDP), Service::Name("accessbuilder")),
        (ServiceQuery(4239, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(7202, Protocol::TCP), Service::Name("pon-ictp")),
        (ServiceQuery(10104, Protocol::UDP), Service::Name("swdtp")),
        (ServiceQuery(1310, Protocol::UDP), Service::Name("husky")),
        (ServiceQuery(1664, Protocol::TCP), Service::Name("netview-aix-4")),
        (ServiceQuery(1441, Protocol::UDP), Service::Name("cadis-1")),
        (ServiceQuery(3584, Protocol::UDP), Service::Name("u-dbap")),
        (ServiceQuery(1709, Protocol::TCP), Service::Name("centra")),
        (ServiceQuery(2436, Protocol::UDP), Service::Name("topx")),
        (ServiceQuery(2138, Protocol::UDP), Service::Name("unbind-cluster")),
        (ServiceQuery(6063, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(6715, Protocol::TCP), Service::Name("fibotrader-com")),
        (ServiceQuery(34379, Protocol::TCP), Service::Name("p-net-remote")),
        (ServiceQuery(5507, Protocol::TCP), Service::Name("psl-management")),
        (ServiceQuery(2472, Protocol::TCP), Service::Name("c3")),
        (ServiceQuery(2550, Protocol::UDP), Service::Name("ads")),
        (ServiceQuery(3517, Protocol::UDP), Service::Name("802-11-iapp")),
        (ServiceQuery(5073, Protocol::UDP), Service::Name("tag-pm")),
        (ServiceQuery(451, Protocol::TCP), Service::Name("sfs-smp-net")),
        (ServiceQuery(8893, Protocol::TCP), Service::Name("ddi-tcp-6")),
        (ServiceQuery(9802, Protocol::UDP), Service::Name("davsrcs")),
        (ServiceQuery(539, Protocol::TCP), Service::Name("apertus-ldp")),
        (ServiceQuery(2348, Protocol::TCP), Service::Name("redstorm_info")),
        (ServiceQuery(439, Protocol::TCP), Service::Name("dasp")),
        (ServiceQuery(1069, Protocol::UDP), Service::Name("cognex-insight")),
        (ServiceQuery(9087, Protocol::UDP), Service::Name("classic")),
        (ServiceQuery(3395, Protocol::TCP), Service::Name("dyna-lm")),
        (ServiceQuery(24006, Protocol::TCP), Service::Name("med-net-svc")),
        (ServiceQuery(4095, Protocol::UDP), Service::Name("xtgui")),
        (ServiceQuery(992, Protocol::TCP), Service::Name("telnets")),
        (ServiceQuery(616, Protocol::UDP), Service::Name("sco-sysmgr")),
        (ServiceQuery(4546, Protocol::TCP), Service::Name("sf-lm")),
        (ServiceQuery(372, Protocol::TCP), Service::Name("ulistserv")),
        (ServiceQuery(3515, Protocol::TCP), Service::Name("must-backplane")),
        (ServiceQuery(2340, Protocol::TCP), Service::Name("wrs_registry")),
        (ServiceQuery(3685, Protocol::UDP), Service::Name("dsx-agent")),
        (ServiceQuery(1279, Protocol::UDP), Service::Name("dellwebadmin-2")),
        (ServiceQuery(2179, Protocol::TCP), Service::Name("vmrdp")),
        (ServiceQuery(3759, Protocol::UDP), Service::Name("exapt-lmgr")),
        (ServiceQuery(6116, Protocol::TCP), Service::Name("xtlserv")),
        (ServiceQuery(568, Protocol::TCP), Service::Name("ms-shuttle")),
        (ServiceQuery(460, Protocol::UDP), Service::Name("skronk")),
        (ServiceQuery(4596, Protocol::UDP), Service::Name("ias-neighbor")),
        (ServiceQuery(2521, Protocol::TCP), Service::Name("adaptecmgr")),
        (ServiceQuery(440, Protocol::UDP), Service::Name("sgcp")),
        (ServiceQuery(4228, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3252, Protocol::TCP), Service::Name("dhe")),
        (ServiceQuery(1002, Protocol::TCP), Service::Name("windows-icfw")),
        (ServiceQuery(4422, Protocol::TCP), Service::Name("tsepisp")),
        (ServiceQuery(1442, Protocol::TCP), Service::Name("cadis-2")),
        (ServiceQuery(8800, Protocol::UDP), Service::Name("sunwebadmin")),
        (ServiceQuery(1040, Protocol::UDP), Service::Name("netarx")),
        (ServiceQuery(3245, Protocol::UDP), Service::Name("vieo-fe")),
        (ServiceQuery(1120, Protocol::UDP), Service::Name("bnetfile")),
        (ServiceQuery(27008, Protocol::UDP), Service::Name("flex-lm")),
        (ServiceQuery(1143, Protocol::TCP), Service::Name("imyx")),
        (ServiceQuery(2704, Protocol::TCP), Service::Name("sms-remctrl")),
        (ServiceQuery(2327, Protocol::UDP), Service::Name("xingcsm")),
        (ServiceQuery(4137, Protocol::TCP), Service::Name("cl-db-remote")),
        (ServiceQuery(3723, Protocol::TCP), Service::Name("sychrond")),
        (ServiceQuery(1141, Protocol::TCP), Service::Name("mxomss")),
        (ServiceQuery(1795, Protocol::TCP), Service::Name("dpi-proxy")),
        (ServiceQuery(9593, Protocol::TCP), Service::Name("cba8")),
        (ServiceQuery(6060, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(3807, Protocol::TCP), Service::Name("spugna")),
        (ServiceQuery(3414, Protocol::UDP), Service::Name("wip-port")),
        (ServiceQuery(8056, Protocol::TCP), Service::Name("senomix05")),
        (ServiceQuery(2917, Protocol::UDP), Service::Name("elvin_client")),
        (ServiceQuery(2768, Protocol::UDP), Service::Name("uacs")),
        (ServiceQuery(6346, Protocol::TCP), Service::Name("gnutella")),
        (ServiceQuery(4143, Protocol::UDP), Service::Name("oidsr")),
        (ServiceQuery(2193, Protocol::TCP), Service::Name("drwcs")),
        (ServiceQuery(533, Protocol::UDP), Service::Name("netwall")),
        (ServiceQuery(3829, Protocol::UDP), Service::Name("neteh-ext")),
        (ServiceQuery(7274, Protocol::UDP), Service::Name("oma-rlp-s")),
        (ServiceQuery(3805, Protocol::UDP), Service::Name("tcpdataserver")),
        (ServiceQuery(198, Protocol::TCP), Service::Name("dls-mon")),
        (ServiceQuery(21221, Protocol::TCP), Service::Name("aigairserver")),
        (ServiceQuery(2872, Protocol::UDP), Service::Name("radix")),
        (ServiceQuery(2476, Protocol::UDP), Service::Name("ace-svr-prop")),
        (ServiceQuery(4016, Protocol::TCP), Service::Name("talarian-mcast2")),
        (ServiceQuery(6456, Protocol::TCP), Service::Name("skip-cert-send")),
        (ServiceQuery(3684, Protocol::UDP), Service::Name("faxstfx-port")),
        (ServiceQuery(1827, Protocol::TCP), Service::Name("pcm")),
        (ServiceQuery(1320, Protocol::TCP), Service::Name("amx-axbnet")),
        (ServiceQuery(1550, Protocol::TCP), Service::Name("3m-image-lm")),
        (ServiceQuery(2718, Protocol::TCP), Service::Name("pn-requester2")),
        (ServiceQuery(24004, Protocol::TCP), Service::Name("med-ovw")),
        (ServiceQuery(1675, Protocol::UDP), Service::Name("pdp")),
        (ServiceQuery(2776, Protocol::UDP), Service::Name("ridgeway1")),
        (ServiceQuery(3208, Protocol::UDP), Service::Name("pfu-prcallback")),
        (ServiceQuery(2188, Protocol::TCP), Service::Name("radware-rpm")),
        (ServiceQuery(41794, Protocol::TCP), Service::Name("crestron-cip")),
        (ServiceQuery(3571, Protocol::TCP), Service::Name("megardsvr-port")),
        (ServiceQuery(4413, Protocol::TCP), Service::Name("avi-nms")),
        (ServiceQuery(1099, Protocol::TCP), Service::Name("rmiregistry")),
        (ServiceQuery(8300, Protocol::UDP), Service::Name("tmi")),
        (ServiceQuery(39213, Protocol::UDP), Service::Name("sygatefw")),
        (ServiceQuery(310, Protocol::TCP), Service::Name("bhmds")),
        (ServiceQuery(1592, Protocol::UDP), Service::Name("commonspace")),
        (ServiceQuery(1813, Protocol::UDP), Service::Name("radacct")),
        (ServiceQuery(9952, Protocol::TCP), Service::Name("apc-9952")),
        (ServiceQuery(4059, Protocol::UDP), Service::Name("dlms-cosem")),
        (ServiceQuery(2152, Protocol::UDP), Service::Name("gtp-user")),
        (ServiceQuery(3287, Protocol::TCP), Service::Name("directvdata")),
        (ServiceQuery(7728, Protocol::TCP), Service::Name("osvr")),
        (ServiceQuery(8834, Protocol::TCP), Service::Name("nessus-xmlrpc")),
        (ServiceQuery(1348, Protocol::TCP), Service::Name("bbn-mmx")),
        (ServiceQuery(2525, Protocol::TCP), Service::Name("ms-v-worlds")),
        (ServiceQuery(9631, Protocol::TCP), Service::Name("peocoll")),
        (ServiceQuery(11164, Protocol::UDP), Service::Name("suncacao-csa")),
        (ServiceQuery(4989, Protocol::UDP), Service::Name("parallel")),
        (ServiceQuery(1142, Protocol::TCP), Service::Name("edtools")),
        (ServiceQuery(8132, Protocol::UDP), Service::Name("dbabble")),
        (ServiceQuery(7080, Protocol::TCP), Service::Name("empowerid")),
        (ServiceQuery(1605, Protocol::TCP), Service::Name("slp")),
        (ServiceQuery(32778, Protocol::TCP), Service::Name("sometimes-rpc19")),
        (ServiceQuery(3353, Protocol::TCP), Service::Name("fatpipe")),
        (ServiceQuery(3560, Protocol::TCP), Service::Name("iniserve-port")),
        (ServiceQuery(1256, Protocol::TCP), Service::Name("de-server")),
        (ServiceQuery(1481, Protocol::UDP), Service::Name("airs")),
        (ServiceQuery(1611, Protocol::UDP), Service::Name("ill")),
        (ServiceQuery(685, Protocol::TCP), Service::Name("mdc-portmapper")),
        (ServiceQuery(2192, Protocol::UDP), Service::Name("asdis")),
        (ServiceQuery(8567, Protocol::UDP), Service::Name("oap-admin")),
        (ServiceQuery(6550, Protocol::UDP), Service::Name("fg-sysupdate")),
        (ServiceQuery(6963, Protocol::UDP), Service::Name("swismgr1")),
        (ServiceQuery(13821, Protocol::UDP), Service::Name("dsmcc-download")),
        (ServiceQuery(3206, Protocol::TCP), Service::Name("ironmail")),
        (ServiceQuery(8417, Protocol::TCP), Service::Name("espeech-rtp")),
        (ServiceQuery(1535, Protocol::TCP), Service::Name("ampr-info")),
        (ServiceQuery(3186, Protocol::UDP), Service::Name("iiw-port")),
        (ServiceQuery(3762, Protocol::UDP), Service::Name("gbs-smp")),
        (ServiceQuery(3365, Protocol::TCP), Service::Name("contentserver")),
        (ServiceQuery(3055, Protocol::UDP), Service::Name("policyserver")),
        (ServiceQuery(3715, Protocol::TCP), Service::Name("anoto-rendezv")),
        (ServiceQuery(3990, Protocol::TCP), Service::Name("bv-is")),
        (ServiceQuery(3993, Protocol::UDP), Service::Name("bv-agent")),
        (ServiceQuery(3805, Protocol::TCP), Service::Name("tcpdataserver")),
        (ServiceQuery(2106, Protocol::TCP), Service::Name("ekshell")),
        (ServiceQuery(3293, Protocol::UDP), Service::Name("fg-fps")),
        (ServiceQuery(22003, Protocol::UDP), Service::Name("optohost003")),
        (ServiceQuery(2171, Protocol::TCP), Service::Name("msfw-storage")),
        (ServiceQuery(2257, Protocol::TCP), Service::Name("simple-tx-rx")),
        (ServiceQuery(1557, Protocol::TCP), Service::Name("arbortext-lm")),
        (ServiceQuery(1031, Protocol::TCP), Service::Name("iad2")),
        (ServiceQuery(6487, Protocol::TCP), Service::Name("sun-sr-iiop-aut")),
        (ServiceQuery(7275, Protocol::TCP), Service::Name("oma-ulp")),
        (ServiceQuery(31457, Protocol::UDP), Service::Name("tetrinet")),
        (ServiceQuery(1178, Protocol::TCP), Service::Name("skkserv")),
        (ServiceQuery(553, Protocol::TCP), Service::Name("pirp")),
        (ServiceQuery(1875, Protocol::UDP), Service::Name("westell-stats")),
        (ServiceQuery(5633, Protocol::UDP), Service::Name("beorl")),
        (ServiceQuery(604, Protocol::UDP), Service::Name("tunnel")),
        (ServiceQuery(4101, Protocol::TCP), Service::Name("brlp-0")),
        (ServiceQuery(4676, Protocol::UDP), Service::Name("dhct-alerts")),
        (ServiceQuery(4127, Protocol::UDP), Service::Name("unikeypro")),
        (ServiceQuery(646, Protocol::TCP), Service::Name("ldp")),
        (ServiceQuery(2777, Protocol::TCP), Service::Name("ridgeway2")),
        (ServiceQuery(47, Protocol::UDP), Service::Name("ni-ftp")),
        (ServiceQuery(5114, Protocol::TCP), Service::Name("ev-services")),
        (ServiceQuery(9802, Protocol::TCP), Service::Name("davsrcs")),
        (ServiceQuery(3501, Protocol::TCP), Service::Name("isoft-p2p")),
        (ServiceQuery(2431, Protocol::TCP), Service::Name("venus-se")),
        (ServiceQuery(2357, Protocol::TCP), Service::Name("unihub-server")),
        (ServiceQuery(2436, Protocol::TCP), Service::Name("topx")),
        (ServiceQuery(4792, Protocol::TCP), Service::Name("unified-bus")),
        (ServiceQuery(2978, Protocol::TCP), Service::Name("ttc-etap-ds")),
        (ServiceQuery(2673, Protocol::TCP), Service::Name("firstcall42")),
        (ServiceQuery(2269, Protocol::TCP), Service::Name("mikey")),
        (ServiceQuery(2636, Protocol::TCP), Service::Name("solve")),
        (ServiceQuery(7998, Protocol::TCP), Service::Name("usicontentpush")),
        (ServiceQuery(1471, Protocol::TCP), Service::Name("csdmbase")),
        (ServiceQuery(6404, Protocol::UDP), Service::Name("boe-filesvr")),
        (ServiceQuery(3959, Protocol::TCP), Service::Name("treehopper")),
        (ServiceQuery(1470, Protocol::TCP), Service::Name("uaiact")),
        (ServiceQuery(4601, Protocol::TCP), Service::Name("piranha2")),
        (ServiceQuery(2644, Protocol::UDP), Service::Name("travsoft-ipx-t")),
        (ServiceQuery(5969, Protocol::UDP), Service::Name("mppolicy-mgr")),
        (ServiceQuery(3552, Protocol::TCP), Service::Name("taserver")),
        (ServiceQuery(2960, Protocol::UDP), Service::Name("dfoxserver")),
        (ServiceQuery(48001, Protocol::UDP), Service::Name("nimspooler")),
        (ServiceQuery(1769, Protocol::UDP), Service::Name("bmc-net-adm")),
        (ServiceQuery(578, Protocol::UDP), Service::Name("ipdd")),
        (ServiceQuery(2788, Protocol::UDP), Service::Name("fryeserv")),
        (ServiceQuery(19333, Protocol::TCP), Service::Name("litecoin")),
        (ServiceQuery(4262, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1082, Protocol::UDP), Service::Name("amt-esd-prot")),
        (ServiceQuery(13786, Protocol::UDP), Service::Name("nomdb")),
        (ServiceQuery(605, Protocol::UDP), Service::Name("soap-beep")),
        (ServiceQuery(4598, Protocol::UDP), Service::Name("a16-an-an")),
        (ServiceQuery(1406, Protocol::TCP), Service::Name("netlabs-lm")),
        (ServiceQuery(6665, Protocol::TCP), Service::Name("irc")),
        (ServiceQuery(2438, Protocol::UDP), Service::Name("msp")),
        (ServiceQuery(2773, Protocol::UDP), Service::Name("rbakcup1")),
        (ServiceQuery(4738, Protocol::TCP), Service::Name("solera-lpn")),
        (ServiceQuery(4357, Protocol::TCP), Service::Name("qsnet-cond")),
        (ServiceQuery(510, Protocol::TCP), Service::Name("fcp")),
        (ServiceQuery(4209, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1287, Protocol::UDP), Service::Name("routematch")),
        (ServiceQuery(18136, Protocol::TCP), Service::Name("racf")),
        (ServiceQuery(8554, Protocol::TCP), Service::Name("rtsp-alt")),
        (ServiceQuery(2730, Protocol::UDP), Service::Name("nec-raidplus")),
        (ServiceQuery(2001, Protocol::TCP), Service::Name("dc")),
        (ServiceQuery(3753, Protocol::UDP), Service::Name("nattyserver")),
        (ServiceQuery(3869, Protocol::TCP), Service::Name("ovsam-mgmt")),
        (ServiceQuery(3508, Protocol::TCP), Service::Name("interactionweb")),
        (ServiceQuery(4375, Protocol::UDP), Service::Name("tolteces")),
        (ServiceQuery(1269, Protocol::UDP), Service::Name("watilapp")),
        (ServiceQuery(2360, Protocol::UDP), Service::Name("nexstorindltd")),
        (ServiceQuery(1937, Protocol::UDP), Service::Name("jwserver")),
        (ServiceQuery(1278, Protocol::TCP), Service::Name("dellwebadmin-1")),
        (ServiceQuery(6671, Protocol::UDP), Service::Name("p4p-portal")),
        (ServiceQuery(7778, Protocol::TCP), Service::Name("interwise")),
        (ServiceQuery(4666, Protocol::TCP), Service::Name("eportcomm")),
        (ServiceQuery(3846, Protocol::UDP), Service::Name("an-pcp")),
        (ServiceQuery(2429, Protocol::UDP), Service::Name("ft-role")),
        (ServiceQuery(5313, Protocol::TCP), Service::Name("rrdp")),
        (ServiceQuery(6110, Protocol::TCP), Service::Name("softcm")),
        (ServiceQuery(7511, Protocol::UDP), Service::Name("pafec-lm")),
        (ServiceQuery(4667, Protocol::UDP), Service::Name("mmacomm")),
        (ServiceQuery(1670, Protocol::TCP), Service::Name("netview-aix-10")),
        (ServiceQuery(2679, Protocol::UDP), Service::Name("syncserverssl")),
        (ServiceQuery(268, Protocol::TCP), Service::Name("td-replica")),
        (ServiceQuery(22003, Protocol::TCP), Service::Name("optohost003")),
        (ServiceQuery(4280, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1738, Protocol::UDP), Service::Name("gamegen1")),
        (ServiceQuery(4008, Protocol::TCP), Service::Name("netcheque")),
        (ServiceQuery(26262, Protocol::TCP), Service::Name("k3software-svr")),
        (ServiceQuery(4213, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(4166, Protocol::TCP), Service::Name("joost")),
        (ServiceQuery(8321, Protocol::UDP), Service::Name("tnp")),
        (ServiceQuery(595, Protocol::UDP), Service::Name("cab-protocol")),
        (ServiceQuery(1110, Protocol::TCP), Service::Name("nfsd-status")),
        (ServiceQuery(3304, Protocol::UDP), Service::Name("opsession-srvr")),
        (ServiceQuery(5741, Protocol::UDP), Service::Name("ida-discover1")),
        (ServiceQuery(2507, Protocol::TCP), Service::Name("spock")),
        (ServiceQuery(1542, Protocol::TCP), Service::Name("gridgen-elmd")),
        (ServiceQuery(152, Protocol::UDP), Service::Name("bftp")),
        (ServiceQuery(8893, Protocol::UDP), Service::Name("ddi-udp-6")),
        (ServiceQuery(5014, Protocol::TCP), Service::Name("onpsocket")),
        (ServiceQuery(3581, Protocol::UDP), Service::Name("kfxaclicensing")),
        (ServiceQuery(1624, Protocol::UDP), Service::Name("udp-sr-port")),
        (ServiceQuery(7348, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(2947, Protocol::UDP), Service::Name("gpsd")),
        (ServiceQuery(1187, Protocol::TCP), Service::Name("alias")),
        (ServiceQuery(7275, Protocol::UDP), Service::Name("oma-ulp")),
        (ServiceQuery(1383, Protocol::TCP), Service::Name("gwha")),
        (ServiceQuery(1147, Protocol::UDP), Service::Name("capioverlan")),
        (ServiceQuery(2242, Protocol::UDP), Service::Name("foliocorp")),
        (ServiceQuery(4839, Protocol::UDP), Service::Name("varadero-2")),
        (ServiceQuery(5246, Protocol::UDP), Service::Name("capwap-control")),
        (ServiceQuery(11876, Protocol::TCP), Service::Name("xoraya")),
        (ServiceQuery(2611, Protocol::TCP), Service::Name("lionhead")),
        (ServiceQuery(1268, Protocol::TCP), Service::Name("propel-msgsys")),
        (ServiceQuery(5433, Protocol::UDP), Service::Name("pyrrho")),
        (ServiceQuery(1066, Protocol::TCP), Service::Name("fpo-fns")),
        (ServiceQuery(2488, Protocol::UDP), Service::Name("moy-corp")),
        (ServiceQuery(1476, Protocol::TCP), Service::Name("clvm-cfg")),
        (ServiceQuery(4108, Protocol::UDP), Service::Name("accel")),
        (ServiceQuery(4804, Protocol::UDP), Service::Name("aja-ntv4-disc")),
        (ServiceQuery(7244, Protocol::TCP), Service::Name("frc-hicp")),
        (ServiceQuery(2100, Protocol::UDP), Service::Name("amiganetfs")),
        (ServiceQuery(2603, Protocol::TCP), Service::Name("ripngd")),
        (ServiceQuery(3494, Protocol::TCP), Service::Name("ibm3494")),
        (ServiceQuery(2440, Protocol::UDP), Service::Name("spearway")),
        (ServiceQuery(2850, Protocol::TCP), Service::Name("metaconsole")),
        (ServiceQuery(2313, Protocol::UDP), Service::Name("iapp")),
        (ServiceQuery(33656, Protocol::UDP), Service::Name("snip")),
        (ServiceQuery(562, Protocol::UDP), Service::Name("chshell")),
        (ServiceQuery(6046, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(3954, Protocol::UDP), Service::Name("adrep")),
        (ServiceQuery(3091, Protocol::UDP), Service::Name("1ci-smcs")),
        (ServiceQuery(7121, Protocol::TCP), Service::Name("virprot-lm")),
        (ServiceQuery(3651, Protocol::UDP), Service::Name("xrpc-registry")),
        (ServiceQuery(3330, Protocol::UDP), Service::Name("mcs-calypsoicf")),
        (ServiceQuery(4675, Protocol::UDP), Service::Name("dhct-status")),
        (ServiceQuery(108, Protocol::UDP), Service::Name("snagas")),
        (ServiceQuery(1155, Protocol::TCP), Service::Name("nfa")),
        (ServiceQuery(4153, Protocol::UDP), Service::Name("mbl-battd")),
        (ServiceQuery(2090, Protocol::UDP), Service::Name("lrp")),
        (ServiceQuery(41111, Protocol::TCP), Service::Name("fs-qos")),
        (ServiceQuery(46, Protocol::UDP), Service::Name("mpm-snd")),
        (ServiceQuery(167, Protocol::UDP), Service::Name("namp")),
        (ServiceQuery(6505, Protocol::TCP), Service::Name("badm_priv")),
        (ServiceQuery(4015, Protocol::UDP), Service::Name("talarian-mcast1")),
        (ServiceQuery(9628, Protocol::TCP), Service::Name("odbcpathway")),
        (ServiceQuery(2669, Protocol::TCP), Service::Name("toad")),
        (ServiceQuery(10117, Protocol::TCP), Service::Name("iqrm")),
        (ServiceQuery(3608, Protocol::TCP), Service::Name("trendchip-dcp")),
        (ServiceQuery(6544, Protocol::TCP), Service::Name("mythtv")),
        (ServiceQuery(9343, Protocol::TCP), Service::Name("mpidcmgr")),
        (ServiceQuery(4080, Protocol::UDP), Service::Name("lorica-in")),
        (ServiceQuery(1208, Protocol::TCP), Service::Name("seagull-ais")),
        (ServiceQuery(3010, Protocol::UDP), Service::Name("ping-pong")),
        (ServiceQuery(1981, Protocol::UDP), Service::Name("p2pq")),
        (ServiceQuery(2940, Protocol::TCP), Service::Name("sm-pas-3")),
        (ServiceQuery(1858, Protocol::UDP), Service::Name("privateark")),
        (ServiceQuery(1852, Protocol::UDP), Service::Name("virtual-time")),
        (ServiceQuery(57, Protocol::TCP), Service::Name("priv-term")),
        (ServiceQuery(2234, Protocol::TCP), Service::Name("directplay")),
        (ServiceQuery(8059, Protocol::TCP), Service::Name("senomix08")),
        (ServiceQuery(7107, Protocol::TCP), Service::Name("aes-x170")),
        (ServiceQuery(5058, Protocol::UDP), Service::Name("locus-disc")),
        (ServiceQuery(2281, Protocol::TCP), Service::Name("lnvconsole")),
        (ServiceQuery(1134, Protocol::UDP), Service::Name("aplx")),
        (ServiceQuery(9164, Protocol::TCP), Service::Name("apani5")),
        (ServiceQuery(1854, Protocol::UDP), Service::Name("buddy-draw")),
        (ServiceQuery(69, Protocol::UDP), Service::Name("tftp")),
        (ServiceQuery(13822, Protocol::UDP), Service::Name("dsmcc-ccp")),
        (ServiceQuery(1274, Protocol::UDP), Service::Name("t1distproc")),
        (ServiceQuery(10005, Protocol::TCP), Service::Name("stel")),
        (ServiceQuery(1106, Protocol::UDP), Service::Name("isoipsigport-1")),
        (ServiceQuery(2174, Protocol::TCP), Service::Name("msfw-array")),
        (ServiceQuery(3626, Protocol::TCP), Service::Name("bvcdaemon-port")),
        (ServiceQuery(7870, Protocol::TCP), Service::Name("rbt-smc")),
        (ServiceQuery(3534, Protocol::UDP), Service::Name("urld-port")),
        (ServiceQuery(774, Protocol::UDP), Service::Name("acmaint_dbd")),
        (ServiceQuery(9801, Protocol::UDP), Service::Name("sstp-2")),
        (ServiceQuery(6038, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(3677, Protocol::TCP), Service::Name("roverlog")),
        (ServiceQuery(1695, Protocol::UDP), Service::Name("rrilwm")),
        (ServiceQuery(13712, Protocol::TCP), Service::Name("netbackup")),
        (ServiceQuery(1062, Protocol::TCP), Service::Name("veracity")),
        (ServiceQuery(6159, Protocol::TCP), Service::Name("efb-aci")),
        (ServiceQuery(2932, Protocol::UDP), Service::Name("incp")),
        (ServiceQuery(5671, Protocol::TCP), Service::Name("amqps")),
        (ServiceQuery(1420, Protocol::UDP), Service::Name("timbuktu-srv4")),
        (ServiceQuery(6558, Protocol::TCP), Service::Name("xdsxdm")),
        (ServiceQuery(782, Protocol::UDP), Service::Name("hp-managed-node")),
        (ServiceQuery(2182, Protocol::UDP), Service::Name("cgn-stat")),
        (ServiceQuery(9283, Protocol::UDP), Service::Name("callwaveiam")),
        (ServiceQuery(12300, Protocol::UDP), Service::Name("linogridengine")),
        (ServiceQuery(24005, Protocol::TCP), Service::Name("med-ci")),
        (ServiceQuery(4188, Protocol::TCP), Service::Name("vatata")),
        (ServiceQuery(28200, Protocol::TCP), Service::Name("voxelstorm")),
        (ServiceQuery(2918, Protocol::UDP), Service::Name("kastenchasepad")),
        (ServiceQuery(3256, Protocol::UDP), Service::Name("cpqrpm-agent")),
        (ServiceQuery(1645, Protocol::UDP), Service::Name("radius")),
        (ServiceQuery(1265, Protocol::UDP), Service::Name("dssiapi")),
        (ServiceQuery(1156, Protocol::TCP), Service::Name("iascontrol-oms")),
        (ServiceQuery(3570, Protocol::UDP), Service::Name("mccwebsvr-port")),
        (ServiceQuery(6370, Protocol::TCP), Service::Name("metaedit-se")),
        (ServiceQuery(1359, Protocol::UDP), Service::Name("ftsrv")),
        (ServiceQuery(1477, Protocol::TCP), Service::Name("ms-sna-server")),
        (ServiceQuery(1662, Protocol::TCP), Service::Name("netview-aix-2")),
        (ServiceQuery(5400, Protocol::TCP), Service::Name("pcduo-old")),
        (ServiceQuery(3612, Protocol::UDP), Service::Name("hp-dataprotect")),
        (ServiceQuery(5742, Protocol::TCP), Service::Name("ida-discover2")),
        (ServiceQuery(1680, Protocol::TCP), Service::Name("CarbonCopy")),
        (ServiceQuery(11171, Protocol::UDP), Service::Name("snss")),
        (ServiceQuery(6034, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(3547, Protocol::UDP), Service::Name("symantec-sim")),
        (ServiceQuery(1615, Protocol::TCP), Service::Name("netbill-auth")),
        (ServiceQuery(3348, Protocol::TCP), Service::Name("pangolin-laser")),
        (ServiceQuery(202, Protocol::UDP), Service::Name("at-nbp")),
        (ServiceQuery(6484, Protocol::UDP), Service::Name("sun-sr-jms")),
        (ServiceQuery(3063, Protocol::UDP), Service::Name("ncadg-ip-udp")),
        (ServiceQuery(4029, Protocol::TCP), Service::Name("ip-qsig")),
        (ServiceQuery(2614, Protocol::TCP), Service::Name("neveroffline")),
        (ServiceQuery(84, Protocol::UDP), Service::Name("ctf")),
        (ServiceQuery(10114, Protocol::UDP), Service::Name("netiq-qcheck")),
        (ServiceQuery(30100, Protocol::TCP), Service::Name("rwp")),
        (ServiceQuery(3726, Protocol::TCP), Service::Name("array-manager")),
        (ServiceQuery(8051, Protocol::TCP), Service::Name("rocrail")),
        (ServiceQuery(24922, Protocol::UDP), Service::Name("find")),
        (ServiceQuery(4090, Protocol::TCP), Service::Name("omasgport")),
        (ServiceQuery(5676, Protocol::TCP), Service::Name("raadmin")),
        (ServiceQuery(11111, Protocol::UDP), Service::Name("vce")),
        (ServiceQuery(4190, Protocol::TCP), Service::Name("sieve")),
        (ServiceQuery(2480, Protocol::UDP), Service::Name("powerexchange")),
        (ServiceQuery(9612, Protocol::TCP), Service::Name("streamcomm-ds")),
        (ServiceQuery(3774, Protocol::TCP), Service::Name("zicom")),
        (ServiceQuery(7301, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(5984, Protocol::TCP), Service::Name("couchdb")),
        (ServiceQuery(2015, Protocol::UDP), Service::Name("raid-cs")),
        (ServiceQuery(693, Protocol::TCP), Service::Name("connendp")),
        (ServiceQuery(2380, Protocol::TCP), Service::Name("etcd-server")),
        (ServiceQuery(2491, Protocol::UDP), Service::Name("conclave-cpp")),
        (ServiceQuery(20048, Protocol::TCP), Service::Name("mountd")),
        (ServiceQuery(1236, Protocol::TCP), Service::Name("bvcontrol")),
        (ServiceQuery(1077, Protocol::UDP), Service::Name("imgames")),
        (ServiceQuery(3127, Protocol::TCP), Service::Name("ctx-bridge")),
        (ServiceQuery(1692, Protocol::TCP), Service::Name("sstsys-lm")),
        (ServiceQuery(27008, Protocol::TCP), Service::Name("flexlm8")),
        (ServiceQuery(2595, Protocol::TCP), Service::Name("worldfusion1")),
        (ServiceQuery(4503, Protocol::TCP), Service::Name("m-bus-oms")),
        (ServiceQuery(32635, Protocol::TCP), Service::Name("sec-ntb-clnt")),
        (ServiceQuery(9091, Protocol::UDP), Service::Name("xmltec-xmlmail")),
        (ServiceQuery(2274, Protocol::UDP), Service::Name("pcttunnell")),
        (ServiceQuery(6021, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(3118, Protocol::UDP), Service::Name("pkagent")),
        (ServiceQuery(1727, Protocol::UDP), Service::Name("winddx")),
        (ServiceQuery(22002, Protocol::UDP), Service::Name("optohost002")),
        (ServiceQuery(16992, Protocol::UDP), Service::Name("amt-soap-http")),
        (ServiceQuery(2000, Protocol::TCP), Service::Name("cisco-sccp")),
        (ServiceQuery(6962, Protocol::TCP), Service::Name("jmevt2")),
        (ServiceQuery(259, Protocol::TCP), Service::Name("esro-gen")),
        (ServiceQuery(2565, Protocol::UDP), Service::Name("coord-svr")),
        (ServiceQuery(6054, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(435, Protocol::UDP), Service::Name("mobilip-mn")),
        (ServiceQuery(9020, Protocol::UDP), Service::Name("tambora")),
        (ServiceQuery(2730, Protocol::TCP), Service::Name("nec-raidplus")),
        (ServiceQuery(6065, Protocol::TCP), Service::Name("winpharaoh")),
        (ServiceQuery(2998, Protocol::TCP), Service::Name("iss-realsec")),
        (ServiceQuery(161, Protocol::TCP), Service::Name("snmp")),
        (ServiceQuery(5506, Protocol::TCP), Service::Name("amc")),
        (ServiceQuery(2327, Protocol::TCP), Service::Name("xingcsm")),
        (ServiceQuery(1463, Protocol::TCP), Service::Name("nucleus")),
        (ServiceQuery(8207, Protocol::UDP), Service::Name("lm-sserver")),
        (ServiceQuery(9287, Protocol::TCP), Service::Name("cumulus")),
        (ServiceQuery(3275, Protocol::UDP), Service::Name("samd")),
        (ServiceQuery(4295, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2204, Protocol::TCP), Service::Name("b2-license")),
        (ServiceQuery(6689, Protocol::TCP), Service::Name("tsa")),
        (ServiceQuery(47, Protocol::TCP), Service::Name("ni-ftp")),
        (ServiceQuery(1061, Protocol::UDP), Service::Name("kiosk")),
        (ServiceQuery(2731, Protocol::TCP), Service::Name("fyre-messanger")),
        (ServiceQuery(4737, Protocol::TCP), Service::Name("ipdr-sp")),
        (ServiceQuery(2748, Protocol::UDP), Service::Name("fjippol-polsvr")),
        (ServiceQuery(1111, Protocol::TCP), Service::Name("lmsocialserver")),
        (ServiceQuery(2812, Protocol::TCP), Service::Name("atmtcp")),
        (ServiceQuery(5630, Protocol::TCP), Service::Name("precise-comm")),
        (ServiceQuery(4298, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(5999, Protocol::UDP), Service::Name("cvsup")),
        (ServiceQuery(9988, Protocol::TCP), Service::Name("nsesrvr")),
        (ServiceQuery(3816, Protocol::UDP), Service::Name("sunlps-http")),
        (ServiceQuery(8276, Protocol::UDP), Service::Name("pando-sec")),
        (ServiceQuery(7301, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(7166, Protocol::UDP), Service::Name("aruba-server")),
        (ServiceQuery(5567, Protocol::UDP), Service::Name("m-oap")),
        (ServiceQuery(5418, Protocol::UDP), Service::Name("mcntp")),
        (ServiceQuery(7343, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(10805, Protocol::TCP), Service::Name("lpdg")),
        (ServiceQuery(3876, Protocol::TCP), Service::Name("dl_agent")),
        (ServiceQuery(1579, Protocol::UDP), Service::Name("ioc-sea-lm")),
        (ServiceQuery(3409, Protocol::UDP), Service::Name("networklens")),
        (ServiceQuery(9086, Protocol::TCP), Service::Name("net2display")),
        (ServiceQuery(1030, Protocol::TCP), Service::Name("iad1")),
        (ServiceQuery(5359, Protocol::UDP), Service::Name("ms-alerter")),
        (ServiceQuery(23002, Protocol::TCP), Service::Name("inovaport3")),
        (ServiceQuery(7161, Protocol::TCP), Service::Name("cabsm-comm")),
        (ServiceQuery(4457, Protocol::TCP), Service::Name("prRegister")),
        (ServiceQuery(1838, Protocol::TCP), Service::Name("talnet")),
        (ServiceQuery(3240, Protocol::UDP), Service::Name("triomotion")),
        (ServiceQuery(3878, Protocol::UDP), Service::Name("fotogcad")),
        (ServiceQuery(262, Protocol::UDP), Service::Name("arcisdms")),
        (ServiceQuery(363, Protocol::TCP), Service::Name("rsvp_tunnel")),
        (ServiceQuery(2193, Protocol::UDP), Service::Name("drwcs")),
        (ServiceQuery(21845, Protocol::UDP), Service::Name("webphone")),
        (ServiceQuery(1238, Protocol::UDP), Service::Name("hacl-qs")),
        (ServiceQuery(9899, Protocol::TCP), Service::Name("sctp-tunneling")),
        (ServiceQuery(3980, Protocol::UDP), Service::Name("acms")),
        (ServiceQuery(1647, Protocol::UDP), Service::Name("rsap")),
        (ServiceQuery(6548, Protocol::UDP), Service::Name("apc-6548")),
        (ServiceQuery(42, Protocol::UDP), Service::Name("nameserver")),
        (ServiceQuery(5744, Protocol::TCP), Service::Name("watchdoc")),
        (ServiceQuery(3739, Protocol::TCP), Service::Name("launchbird-lm")),
        (ServiceQuery(2991, Protocol::UDP), Service::Name("wkstn-mon")),
        (ServiceQuery(1093, Protocol::UDP), Service::Name("proofd")),
        (ServiceQuery(49001, Protocol::TCP), Service::Name("nusrp")),
        (ServiceQuery(9380, Protocol::UDP), Service::Name("boxp")),
        (ServiceQuery(187, Protocol::TCP), Service::Name("aci")),
        (ServiceQuery(9599, Protocol::UDP), Service::Name("robix")),
        (ServiceQuery(3995, Protocol::TCP), Service::Name("iss-mgmt-ssl")),
        (ServiceQuery(6355, Protocol::TCP), Service::Name("pmcs")),
        (ServiceQuery(1762, Protocol::TCP), Service::Name("landesk-rc")),
        (ServiceQuery(209, Protocol::TCP), Service::Name("tam")),
        (ServiceQuery(353, Protocol::UDP), Service::Name("ndsauth")),
        (ServiceQuery(4247, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3634, Protocol::TCP), Service::Name("hlibmgr")),
        (ServiceQuery(563, Protocol::TCP), Service::Name("snews")),
        (ServiceQuery(2862, Protocol::UDP), Service::Name("ttg-protocol")),
        (ServiceQuery(2903, Protocol::UDP), Service::Name("suitcase")),
        (ServiceQuery(1479, Protocol::TCP), Service::Name("dberegister")),
        (ServiceQuery(9281, Protocol::UDP), Service::Name("swtp-port1")),
        (ServiceQuery(426, Protocol::UDP), Service::Name("smartsdp")),
        (ServiceQuery(737, Protocol::UDP), Service::Name("sometimes-rpc2")),
        (ServiceQuery(9954, Protocol::TCP), Service::Name("hinp")),
        (ServiceQuery(2582, Protocol::TCP), Service::Name("argis-ds")),
        (ServiceQuery(31335, Protocol::UDP), Service::Name("Trinoo_Register")),
        (ServiceQuery(3618, Protocol::TCP), Service::Name("aairnet-1")),
        (ServiceQuery(5190, Protocol::UDP), Service::Name("aol")),
        (ServiceQuery(3177, Protocol::UDP), Service::Name("phonex-port")),
        (ServiceQuery(6946, Protocol::TCP), Service::Name("bioserver")),
        (ServiceQuery(6410, Protocol::UDP), Service::Name("boe-resssvr4")),
        (ServiceQuery(1978, Protocol::TCP), Service::Name("unisql")),
        (ServiceQuery(7327, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(3189, Protocol::TCP), Service::Name("pcle-infex")),
        (ServiceQuery(3273, Protocol::UDP), Service::Name("sxmp")),
        (ServiceQuery(2189, Protocol::TCP), Service::Name("radware-rpm-s")),
        (ServiceQuery(2677, Protocol::UDP), Service::Name("gadgetgate1way")),
        (ServiceQuery(2687, Protocol::UDP), Service::Name("pq-lic-mgmt")),
        (ServiceQuery(1327, Protocol::UDP), Service::Name("ultrex")),
        (ServiceQuery(1174, Protocol::TCP), Service::Name("fnet-remote-ui")),
        (ServiceQuery(44321, Protocol::TCP), Service::Name("pmcd")),
        (ServiceQuery(19283, Protocol::TCP), Service::Name("keysrvr")),
        (ServiceQuery(7342, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(8377, Protocol::UDP), Service::Name("cruise-swroute")),
        (ServiceQuery(438, Protocol::TCP), Service::Name("dsfgw")),
        (ServiceQuery(3737, Protocol::TCP), Service::Name("xpanel")),
        (ServiceQuery(1332, Protocol::UDP), Service::Name("pcia-rxp-b")),
        (ServiceQuery(6403, Protocol::TCP), Service::Name("boe-cachesvr")),
        (ServiceQuery(3044, Protocol::UDP), Service::Name("epp")),
        (ServiceQuery(2942, Protocol::TCP), Service::Name("sm-pas-5")),
        (ServiceQuery(72, Protocol::TCP), Service::Name("netrjs-2")),
        (ServiceQuery(8443, Protocol::TCP), Service::Name("https-alt")),
        (ServiceQuery(4125, Protocol::TCP), Service::Name("rww")),
        (ServiceQuery(1909, Protocol::TCP), Service::Name("global-wlink")),
        (ServiceQuery(2420, Protocol::UDP), Service::Name("dslremote-mgmt")),
        (ServiceQuery(4103, Protocol::UDP), Service::Name("brlp-2")),
        (ServiceQuery(1115, Protocol::UDP), Service::Name("ardus-trns")),
        (ServiceQuery(5780, Protocol::TCP), Service::Name("vts-rpc")),
        (ServiceQuery(3968, Protocol::UDP), Service::Name("ianywhere-dbns")),
        (ServiceQuery(15998, Protocol::TCP), Service::Name("2ping")),
        (ServiceQuery(210, Protocol::TCP), Service::Name("z39.50")),
        (ServiceQuery(2300, Protocol::UDP), Service::Name("cvmmon")),
        (ServiceQuery(3151, Protocol::TCP), Service::Name("nm-assessor")),
        (ServiceQuery(3368, Protocol::UDP), Service::Name("satvid-datalnk")),
        (ServiceQuery(2713, Protocol::TCP), Service::Name("raventbs")),
        (ServiceQuery(5009, Protocol::UDP), Service::Name("winfs")),
        (ServiceQuery(3129, Protocol::UDP), Service::Name("netport-id")),
        (ServiceQuery(4153, Protocol::TCP), Service::Name("mbl-battd")),
        (ServiceQuery(1624, Protocol::TCP), Service::Name("udp-sr-port")),
        (ServiceQuery(25000, Protocol::TCP), Service::Name("icl-twobase1")),
        (ServiceQuery(5281, Protocol::TCP), Service::Name("undo-lm")),
        (ServiceQuery(1147, Protocol::TCP), Service::Name("capioverlan")),
        (ServiceQuery(1209, Protocol::UDP), Service::Name("ipcd3")),
        (ServiceQuery(4548, Protocol::UDP), Service::Name("synchromesh")),
        (ServiceQuery(1901, Protocol::UDP), Service::Name("fjicl-tep-a")),
        (ServiceQuery(1281, Protocol::UDP), Service::Name("healthd")),
        (ServiceQuery(386, Protocol::UDP), Service::Name("asa")),
        (ServiceQuery(2680, Protocol::TCP), Service::Name("pxc-sapxom")),
        (ServiceQuery(15998, Protocol::UDP), Service::Name("2ping")),
        (ServiceQuery(2772, Protocol::TCP), Service::Name("auris")),
        (ServiceQuery(1665, Protocol::UDP), Service::Name("netview-aix-5")),
        (ServiceQuery(5473, Protocol::TCP), Service::Name("apsolab-tags")),
        (ServiceQuery(44900, Protocol::TCP), Service::Name("m3da")),
        (ServiceQuery(23402, Protocol::TCP), Service::Name("novar-global")),
        (ServiceQuery(7998, Protocol::UDP), Service::Name("usicontentpush")),
        (ServiceQuery(3225, Protocol::TCP), Service::Name("fcip-port")),
        (ServiceQuery(2820, Protocol::TCP), Service::Name("univision")),
        (ServiceQuery(5043, Protocol::TCP), Service::Name("swxadmin")),
        (ServiceQuery(6070, Protocol::TCP), Service::Name("messageasap")),
        (ServiceQuery(8074, Protocol::UDP), Service::Name("gadugadu")),
        (ServiceQuery(9900, Protocol::UDP), Service::Name("iua")),
        (ServiceQuery(472, Protocol::UDP), Service::Name("ljk-login")),
        (ServiceQuery(1804, Protocol::TCP), Service::Name("enl")),
        (ServiceQuery(7845, Protocol::UDP), Service::Name("apc-7845")),
        (ServiceQuery(3908, Protocol::TCP), Service::Name("hppronetman")),
        (ServiceQuery(3773, Protocol::TCP), Service::Name("ctdhercules")),
        (ServiceQuery(3531, Protocol::UDP), Service::Name("peerenabler")),
        (ServiceQuery(2793, Protocol::TCP), Service::Name("initlsmsad")),
        (ServiceQuery(27500, Protocol::UDP), Service::Name("quakeworld")),
        (ServiceQuery(6062, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(2855, Protocol::UDP), Service::Name("msrp")),
        (ServiceQuery(5786, Protocol::TCP), Service::Name("cisco-redu")),
        (ServiceQuery(3792, Protocol::TCP), Service::Name("sitewatch")),
        (ServiceQuery(5464, Protocol::TCP), Service::Name("quailnet")),
        (ServiceQuery(5403, Protocol::TCP), Service::Name("hpoms-ci-lstn")),
        (ServiceQuery(44544, Protocol::TCP), Service::Name("domiq")),
        (ServiceQuery(24678, Protocol::TCP), Service::Name("proactivate")),
        (ServiceQuery(4353, Protocol::UDP), Service::Name("f5-iquery")),
        (ServiceQuery(71, Protocol::TCP), Service::Name("netrjs-1")),
        (ServiceQuery(7344, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(3565, Protocol::TCP), Service::Name("m2pa")),
        (ServiceQuery(3844, Protocol::TCP), Service::Name("rnm")),
        (ServiceQuery(1933, Protocol::UDP), Service::Name("xmapi")),
        (ServiceQuery(2662, Protocol::TCP), Service::Name("bintec-capi")),
        (ServiceQuery(3881, Protocol::UDP), Service::Name("idac")),
        (ServiceQuery(4026, Protocol::UDP), Service::Name("as-debug")),
        (ServiceQuery(4212, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1151, Protocol::TCP), Service::Name("unizensus")),
        (ServiceQuery(6241, Protocol::TCP), Service::Name("jeol-nsdtp-1")),
        (ServiceQuery(2053, Protocol::TCP), Service::Name("knetd")),
        (ServiceQuery(2992, Protocol::UDP), Service::Name("avenyo")),
        (ServiceQuery(5406, Protocol::UDP), Service::Name("systemics-sox")),
        (ServiceQuery(443, Protocol::UDP), Service::Name("https")),
        (ServiceQuery(5053, Protocol::TCP), Service::Name("rlm")),
        (ServiceQuery(6455, Protocol::TCP), Service::Name("skip-cert-recv")),
        (ServiceQuery(1758, Protocol::TCP), Service::Name("tftp-mcast")),
        (ServiceQuery(1028, Protocol::UDP), Service::Name("ms-lsa")),
        (ServiceQuery(2639, Protocol::TCP), Service::Name("aminet")),
        (ServiceQuery(22343, Protocol::TCP), Service::Name("cis-secure")),
        (ServiceQuery(3921, Protocol::UDP), Service::Name("herodotus-net")),
        (ServiceQuery(55, Protocol::TCP), Service::Name("isi-gl")),
        (ServiceQuery(18262, Protocol::UDP), Service::Name("gv-pf")),
        (ServiceQuery(215, Protocol::TCP), Service::Name("softpc")),
        (ServiceQuery(2892, Protocol::UDP), Service::Name("snifferdata")),
        (ServiceQuery(7006, Protocol::UDP), Service::Name("afs3-errors")),
        (ServiceQuery(1471, Protocol::UDP), Service::Name("csdmbase")),
        (ServiceQuery(640, Protocol::TCP), Service::Name("entrust-sps")),
        (ServiceQuery(1689, Protocol::TCP), Service::Name("firefox")),
        (ServiceQuery(497, Protocol::UDP), Service::Name("retrospect")),
        (ServiceQuery(6488, Protocol::UDP), Service::Name("sun-sr-jmx")),
        (ServiceQuery(2711, Protocol::UDP), Service::Name("sso-control")),
        (ServiceQuery(2501, Protocol::UDP), Service::Name("rtsclient")),
        (ServiceQuery(1396, Protocol::TCP), Service::Name("dvl-activemail")),
        (ServiceQuery(1341, Protocol::TCP), Service::Name("qubes")),
        (ServiceQuery(2675, Protocol::UDP), Service::Name("ttc-etap")),
        (ServiceQuery(5360, Protocol::TCP), Service::Name("ms-sideshow")),
        (ServiceQuery(2481, Protocol::UDP), Service::Name("giop")),
        (ServiceQuery(2645, Protocol::UDP), Service::Name("novell-ipx-cmd")),
        (ServiceQuery(1179, Protocol::UDP), Service::Name("b2n")),
        (ServiceQuery(4234, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3347, Protocol::TCP), Service::Name("phoenix-rpc")),
        (ServiceQuery(3656, Protocol::TCP), Service::Name("abatjss")),
        (ServiceQuery(5270, Protocol::TCP), Service::Name("xmp")),
        (ServiceQuery(23401, Protocol::UDP), Service::Name("novar-alarm")),
        (ServiceQuery(1314, Protocol::UDP), Service::Name("pdps")),
        (ServiceQuery(3175, Protocol::TCP), Service::Name("t1-e1-over-ip")),
        (ServiceQuery(7343, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(3510, Protocol::TCP), Service::Name("xss-port")),
        (ServiceQuery(27999, Protocol::UDP), Service::Name("tw-auth-key")),
        (ServiceQuery(2483, Protocol::UDP), Service::Name("ttc")),
        (ServiceQuery(887, Protocol::TCP), Service::Name("iclcnet_svinfo")),
        (ServiceQuery(3405, Protocol::UDP), Service::Name("nokia-ann-ch1")),
        (ServiceQuery(3459, Protocol::TCP), Service::Name("integral")),
        (ServiceQuery(120, Protocol::TCP), Service::Name("cfdptkt")),
        (ServiceQuery(1522, Protocol::TCP), Service::Name("rna-lm")),
        (ServiceQuery(3021, Protocol::TCP), Service::Name("agriserver")),
        (ServiceQuery(1509, Protocol::UDP), Service::Name("robcad-lm")),
        (ServiceQuery(4343, Protocol::TCP), Service::Name("unicall")),
        (ServiceQuery(3948, Protocol::TCP), Service::Name("apdap")),
        (ServiceQuery(4881, Protocol::UDP), Service::Name("socp-t")),
        (ServiceQuery(3095, Protocol::TCP), Service::Name("panasas")),
        (ServiceQuery(682, Protocol::TCP), Service::Name("xfr")),
        (ServiceQuery(2036, Protocol::UDP), Service::Name("e-dpnet")),
        (ServiceQuery(950, Protocol::TCP), Service::Name("oftep-rpc")),
        (ServiceQuery(4601, Protocol::UDP), Service::Name("piranha2")),
        (ServiceQuery(1915, Protocol::TCP), Service::Name("facelink")),
        (ServiceQuery(367, Protocol::TCP), Service::Name("mortgageware")),
        (ServiceQuery(139, Protocol::UDP), Service::Name("netbios-ssn")),
        (ServiceQuery(2540, Protocol::UDP), Service::Name("lonworks")),
        (ServiceQuery(4051, Protocol::TCP), Service::Name("cppdp")),
        (ServiceQuery(1679, Protocol::UDP), Service::Name("darcorp-lm")),
        (ServiceQuery(4157, Protocol::TCP), Service::Name("stat-scanner")),
        (ServiceQuery(380, Protocol::UDP), Service::Name("is99s")),
        (ServiceQuery(7008, Protocol::TCP), Service::Name("afs3-update")),
        (ServiceQuery(452, Protocol::TCP), Service::Name("sfs-config")),
        (ServiceQuery(2232, Protocol::TCP), Service::Name("ivs-video")),
        (ServiceQuery(2590, Protocol::UDP), Service::Name("idotdist")),
        (ServiceQuery(6507, Protocol::UDP), Service::Name("bdir_priv")),
        (ServiceQuery(1394, Protocol::TCP), Service::Name("iclpv-nlc")),
        (ServiceQuery(2306, Protocol::TCP), Service::Name("tappi-boxnet")),
        (ServiceQuery(2847, Protocol::UDP), Service::Name("aimpp-port-req")),
        (ServiceQuery(1079, Protocol::UDP), Service::Name("asprovatalk")),
        (ServiceQuery(3793, Protocol::TCP), Service::Name("dcsoftware")),
        (ServiceQuery(2756, Protocol::TCP), Service::Name("simplement-tie")),
        (ServiceQuery(2686, Protocol::TCP), Service::Name("mpnjsomg")),
        (ServiceQuery(2598, Protocol::UDP), Service::Name("citriximaclient")),
        (ServiceQuery(20005, Protocol::UDP), Service::Name("openwebnet")),
        (ServiceQuery(2129, Protocol::UDP), Service::Name("cs-live")),
        (ServiceQuery(3033, Protocol::TCP), Service::Name("pdb")),
        (ServiceQuery(2362, Protocol::TCP), Service::Name("digiman")),
        (ServiceQuery(4366, Protocol::TCP), Service::Name("shadowstream")),
        (ServiceQuery(2776, Protocol::TCP), Service::Name("ridgeway1")),
        (ServiceQuery(5350, Protocol::TCP), Service::Name("nat-pmp-status")),
        (ServiceQuery(6243, Protocol::UDP), Service::Name("jeol-nsddp-3")),
        (ServiceQuery(5351, Protocol::TCP), Service::Name("nat-pmp")),
        (ServiceQuery(2559, Protocol::UDP), Service::Name("lstp")),
        (ServiceQuery(4112, Protocol::TCP), Service::Name("apple-vpns-rp")),
        (ServiceQuery(368, Protocol::TCP), Service::Name("qbikgdp")),
        (ServiceQuery(2494, Protocol::UDP), Service::Name("bmc-ar")),
        (ServiceQuery(6716, Protocol::TCP), Service::Name("princity-agent")),
        (ServiceQuery(2315, Protocol::TCP), Service::Name("precise-sft")),
        (ServiceQuery(3890, Protocol::UDP), Service::Name("ndsconnect")),
        (ServiceQuery(1050, Protocol::UDP), Service::Name("cma")),
        (ServiceQuery(3248, Protocol::TCP), Service::Name("procos-lm")),
        (ServiceQuery(3903, Protocol::UDP), Service::Name("charsetmgr")),
        (ServiceQuery(1050, Protocol::TCP), Service::Name("java-or-OTGfileshare")),
        (ServiceQuery(1533, Protocol::UDP), Service::Name("virtual-places")),
        (ServiceQuery(12109, Protocol::UDP), Service::Name("rets-ssl")),
        (ServiceQuery(1416, Protocol::TCP), Service::Name("novell-lu6.2")),
        (ServiceQuery(2405, Protocol::UDP), Service::Name("trc-netpoll")),
        (ServiceQuery(1766, Protocol::UDP), Service::Name("cft-5")),
        (ServiceQuery(4037, Protocol::TCP), Service::Name("ravehd")),
        (ServiceQuery(6004, Protocol::UDP), Service::Name("X11:4")),
        (ServiceQuery(623, Protocol::TCP), Service::Name("oob-ws-http")),
        (ServiceQuery(9629, Protocol::TCP), Service::Name("uniport")),
        (ServiceQuery(2936, Protocol::UDP), Service::Name("otpatch")),
        (ServiceQuery(7687, Protocol::TCP), Service::Name("bolt")),
        (ServiceQuery(4674, Protocol::TCP), Service::Name("appiq-mgmt")),
        (ServiceQuery(7542, Protocol::TCP), Service::Name("saratoga")),
        (ServiceQuery(2476, Protocol::TCP), Service::Name("ace-svr-prop")),
        (ServiceQuery(1316, Protocol::UDP), Service::Name("exbit-escp")),
        (ServiceQuery(556, Protocol::TCP), Service::Name("remotefs")),
        (ServiceQuery(1650, Protocol::UDP), Service::Name("nkd")),
        (ServiceQuery(3189, Protocol::UDP), Service::Name("pcle-infex")),
        (ServiceQuery(5237, Protocol::UDP), Service::Name("mnet-discovery")),
        (ServiceQuery(3602, Protocol::UDP), Service::Name("infiniswitchcl")),
        (ServiceQuery(5397, Protocol::UDP), Service::Name("stresstester")),
        (ServiceQuery(4370, Protocol::UDP), Service::Name("elpro_tunnel")),
        (ServiceQuery(3996, Protocol::UDP), Service::Name("remoteanything")),
        (ServiceQuery(2271, Protocol::UDP), Service::Name("mmcals")),
        (ServiceQuery(1211, Protocol::UDP), Service::Name("groove-dpp")),
        (ServiceQuery(12865, Protocol::TCP), Service::Name("netperf")),
        (ServiceQuery(28015, Protocol::TCP), Service::Name("rethinkdb")),
        (ServiceQuery(20049, Protocol::TCP), Service::Name("nfsrdma")),
        (ServiceQuery(5634, Protocol::TCP), Service::Name("xprtld")),
        (ServiceQuery(3886, Protocol::TCP), Service::Name("nei-management")),
        (ServiceQuery(485, Protocol::TCP), Service::Name("powerburst")),
        (ServiceQuery(3358, Protocol::UDP), Service::Name("mpsysrmsvr")),
        (ServiceQuery(8999, Protocol::UDP), Service::Name("bctp")),
        (ServiceQuery(5298, Protocol::UDP), Service::Name("presence")),
        (ServiceQuery(9390, Protocol::TCP), Service::Name("otp")),
        (ServiceQuery(3438, Protocol::TCP), Service::Name("spiral-admin")),
        (ServiceQuery(1353, Protocol::TCP), Service::Name("relief")),
        (ServiceQuery(365, Protocol::UDP), Service::Name("dtk")),
        (ServiceQuery(5234, Protocol::UDP), Service::Name("eenet")),
        (ServiceQuery(2147, Protocol::UDP), Service::Name("lv-auth")),
        (ServiceQuery(1465, Protocol::UDP), Service::Name("pipes")),
        (ServiceQuery(1158, Protocol::TCP), Service::Name("lsnr")),
        (ServiceQuery(3025, Protocol::TCP), Service::Name("slnp")),
        (ServiceQuery(2489, Protocol::TCP), Service::Name("tsilb")),
        (ServiceQuery(5070, Protocol::TCP), Service::Name("vtsas")),
        (ServiceQuery(203, Protocol::UDP), Service::Name("at-3")),
        (ServiceQuery(6085, Protocol::TCP), Service::Name("konspire2b")),
        (ServiceQuery(3207, Protocol::TCP), Service::Name("vx-auth-port")),
        (ServiceQuery(508, Protocol::UDP), Service::Name("xvttp")),
        (ServiceQuery(2957, Protocol::UDP), Service::Name("jmact5")),
        (ServiceQuery(1094, Protocol::TCP), Service::Name("rootd")),
        (ServiceQuery(6016, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(2980, Protocol::TCP), Service::Name("wimd")),
        (ServiceQuery(2184, Protocol::TCP), Service::Name("nvd")),
        (ServiceQuery(3913, Protocol::TCP), Service::Name("listcrt-port")),
        (ServiceQuery(2643, Protocol::TCP), Service::Name("gte-samp")),
        (ServiceQuery(2276, Protocol::TCP), Service::Name("ibridge-mgmt")),
        (ServiceQuery(3415, Protocol::UDP), Service::Name("bcinameservice")),
        (ServiceQuery(16360, Protocol::UDP), Service::Name("netserialext1")),
        (ServiceQuery(14143, Protocol::TCP), Service::Name("icpps")),
        (ServiceQuery(5800, Protocol::TCP), Service::Name("vnc-http")),
        (ServiceQuery(1717, Protocol::TCP), Service::Name("fj-hdnet")),
        (ServiceQuery(2147, Protocol::TCP), Service::Name("lv-auth")),
        (ServiceQuery(5002, Protocol::UDP), Service::Name("rfe")),
        (ServiceQuery(4040, Protocol::UDP), Service::Name("yo-main")),
        (ServiceQuery(20222, Protocol::TCP), Service::Name("ipulse-ics")),
        (ServiceQuery(1929, Protocol::UDP), Service::Name("bandwiz-system")),
        (ServiceQuery(3472, Protocol::TCP), Service::Name("jaugsremotec-1")),
        (ServiceQuery(143, Protocol::UDP), Service::Name("imap")),
        (ServiceQuery(1089, Protocol::UDP), Service::Name("ff-annunc")),
        (ServiceQuery(181, Protocol::TCP), Service::Name("unify")),
        (ServiceQuery(3124, Protocol::TCP), Service::Name("beacon-port")),
        (ServiceQuery(597, Protocol::TCP), Service::Name("ptcnameservice")),
        (ServiceQuery(1008, Protocol::TCP), Service::Name("ufsd")),
        (ServiceQuery(2449, Protocol::TCP), Service::Name("ratl")),
        (ServiceQuery(1204, Protocol::UDP), Service::Name("ssslog-mgr")),
        (ServiceQuery(3757, Protocol::UDP), Service::Name("grf-port")),
        (ServiceQuery(1776, Protocol::UDP), Service::Name("femis")),
        (ServiceQuery(3108, Protocol::TCP), Service::Name("geolocate")),
        (ServiceQuery(3468, Protocol::UDP), Service::Name("ttcmremotectrl")),
        (ServiceQuery(3215, Protocol::UDP), Service::Name("jmq-daemon-2")),
        (ServiceQuery(2472, Protocol::UDP), Service::Name("c3")),
        (ServiceQuery(7022, Protocol::UDP), Service::Name("ctdp")),
        (ServiceQuery(3814, Protocol::UDP), Service::Name("neto-dcs")),
        (ServiceQuery(7901, Protocol::UDP), Service::Name("tnos-sp")),
        (ServiceQuery(3883, Protocol::TCP), Service::Name("vrpn")),
        (ServiceQuery(4889, Protocol::TCP), Service::Name("xcap-control")),
        (ServiceQuery(4270, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3366, Protocol::TCP), Service::Name("creativepartnr")),
        (ServiceQuery(2236, Protocol::UDP), Service::Name("nani")),
        (ServiceQuery(5195, Protocol::TCP), Service::Name("ampl-lic")),
        (ServiceQuery(3240, Protocol::TCP), Service::Name("triomotion")),
        (ServiceQuery(3000, Protocol::UDP), Service::Name("hbci")),
        (ServiceQuery(5246, Protocol::TCP), Service::Name("capwap-control")),
        (ServiceQuery(1135, Protocol::TCP), Service::Name("omnivision")),
        (ServiceQuery(2077, Protocol::TCP), Service::Name("tsrmagt")),
        (ServiceQuery(4112, Protocol::UDP), Service::Name("apple-vpns-rp")),
        (ServiceQuery(7747, Protocol::UDP), Service::Name("prgp")),
        (ServiceQuery(1302, Protocol::UDP), Service::Name("ci3-software-2")),
        (ServiceQuery(1551, Protocol::TCP), Service::Name("hecmtl-db")),
        (ServiceQuery(185, Protocol::UDP), Service::Name("remote-kis")),
        (ServiceQuery(5475, Protocol::TCP), Service::Name("apsolab-data")),
        (ServiceQuery(22305, Protocol::UDP), Service::Name("cis")),
        (ServiceQuery(1864, Protocol::TCP), Service::Name("paradym-31")),
        (ServiceQuery(3123, Protocol::UDP), Service::Name("edix")),
        (ServiceQuery(9022, Protocol::UDP), Service::Name("paragent")),
        (ServiceQuery(69, Protocol::TCP), Service::Name("tftp")),
        (ServiceQuery(4790, Protocol::TCP), Service::Name("vxlan-gpe")),
        (ServiceQuery(2043, Protocol::UDP), Service::Name("isis-bcast")),
        (ServiceQuery(3906, Protocol::UDP), Service::Name("topovista-data")),
        (ServiceQuery(6621, Protocol::TCP), Service::Name("kftp")),
        (ServiceQuery(5074, Protocol::UDP), Service::Name("alesquery")),
        (ServiceQuery(464, Protocol::UDP), Service::Name("kpasswd5")),
        (ServiceQuery(3259, Protocol::TCP), Service::Name("epncdp2")),
        (ServiceQuery(22000, Protocol::UDP), Service::Name("snapenetio")),
        (ServiceQuery(1460, Protocol::TCP), Service::Name("proshare2")),
        (ServiceQuery(4263, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(5251, Protocol::UDP), Service::Name("caevms")),
        (ServiceQuery(1663, Protocol::UDP), Service::Name("netview-aix-3")),
        (ServiceQuery(3482, Protocol::TCP), Service::Name("vulture")),
        (ServiceQuery(1453, Protocol::UDP), Service::Name("genie-lm")),
        (ServiceQuery(2355, Protocol::UDP), Service::Name("psdbserver")),
        (ServiceQuery(37483, Protocol::TCP), Service::Name("gdrive-sync")),
        (ServiceQuery(1927, Protocol::UDP), Service::Name("videte-cipc")),
        (ServiceQuery(4288, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1560, Protocol::TCP), Service::Name("asci-val")),
        (ServiceQuery(6035, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(5715, Protocol::UDP), Service::Name("prosharedata")),
        (ServiceQuery(48129, Protocol::UDP), Service::Name("blp5")),
        (ServiceQuery(1821, Protocol::UDP), Service::Name("donnyworld")),
        (ServiceQuery(1145, Protocol::UDP), Service::Name("x9-icue")),
        (ServiceQuery(1646, Protocol::TCP), Service::Name("sa-msg-port")),
        (ServiceQuery(1507, Protocol::UDP), Service::Name("symplex")),
        (ServiceQuery(2043, Protocol::TCP), Service::Name("isis-bcast")),
        (ServiceQuery(6789, Protocol::UDP), Service::Name("smc-https")),
        (ServiceQuery(1983, Protocol::TCP), Service::Name("lhtp")),
        (ServiceQuery(1767, Protocol::TCP), Service::Name("cft-6")),
        (ServiceQuery(7314, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(3686, Protocol::TCP), Service::Name("tnmpv2")),
        (ServiceQuery(5099, Protocol::UDP), Service::Name("sentlm-srv2srv")),
        (ServiceQuery(1530, Protocol::TCP), Service::Name("rap-service")),
        (ServiceQuery(6321, Protocol::UDP), Service::Name("emp-server1")),
        (ServiceQuery(1796, Protocol::TCP), Service::Name("vocaltec-admin")),
        (ServiceQuery(359, Protocol::TCP), Service::Name("tenebris_nts")),
        (ServiceQuery(3751, Protocol::TCP), Service::Name("gprs-cube")),
        (ServiceQuery(75, Protocol::TCP), Service::Name("priv-dial")),
        (ServiceQuery(3557, Protocol::UDP), Service::Name("personalos-001")),
        (ServiceQuery(1122, Protocol::TCP), Service::Name("availant-mgr")),
        (ServiceQuery(9089, Protocol::TCP), Service::Name("sqlexec-ssl")),
        (ServiceQuery(1208, Protocol::UDP), Service::Name("seagull-ais")),
        (ServiceQuery(38202, Protocol::UDP), Service::Name("fairview")),
        (ServiceQuery(1452, Protocol::UDP), Service::Name("gtegsc-lm")),
        (ServiceQuery(38037, Protocol::UDP), Service::Name("landesk-cba")),
        (ServiceQuery(1831, Protocol::UDP), Service::Name("myrtle")),
        (ServiceQuery(3817, Protocol::TCP), Service::Name("tapeware")),
        (ServiceQuery(16959, Protocol::TCP), Service::Name("subseven")),
        (ServiceQuery(5743, Protocol::TCP), Service::Name("watchdoc-pod")),
        (ServiceQuery(7410, Protocol::UDP), Service::Name("ionixnetmon")),
        (ServiceQuery(2933, Protocol::UDP), Service::Name("4-tieropmgw")),
        (ServiceQuery(2109, Protocol::TCP), Service::Name("ergolight")),
        (ServiceQuery(1774, Protocol::UDP), Service::Name("global-dtserv")),
        (ServiceQuery(22273, Protocol::TCP), Service::Name("wnn6")),
        (ServiceQuery(3790, Protocol::TCP), Service::Name("quickbooksrds")),
        (ServiceQuery(2594, Protocol::TCP), Service::Name("dts")),
        (ServiceQuery(1048, Protocol::UDP), Service::Name("neod2")),
        (ServiceQuery(3200, Protocol::UDP), Service::Name("tick-port")),
        (ServiceQuery(1496, Protocol::TCP), Service::Name("liberty-lm")),
        (ServiceQuery(22333, Protocol::TCP), Service::Name("showcockpit-net")),
        (ServiceQuery(1870, Protocol::TCP), Service::Name("sunscalar-dns")),
        (ServiceQuery(1613, Protocol::UDP), Service::Name("netbill-keyrep")),
        (ServiceQuery(684, Protocol::UDP), Service::Name("corba-iiop-ssl")),
        (ServiceQuery(8117, Protocol::TCP), Service::Name("purityrpc")),
        (ServiceQuery(164, Protocol::UDP), Service::Name("smip-agent")),
        (ServiceQuery(154, Protocol::TCP), Service::Name("netsc-prod")),
        (ServiceQuery(2040, Protocol::UDP), Service::Name("lam")),
        (ServiceQuery(1925, Protocol::UDP), Service::Name("discovery-port")),
        (ServiceQuery(3769, Protocol::TCP), Service::Name("haipe-otnk")),
        (ServiceQuery(2806, Protocol::TCP), Service::Name("cspuni")),
        (ServiceQuery(3629, Protocol::TCP), Service::Name("escvpnet")),
        (ServiceQuery(43189, Protocol::UDP), Service::Name("ndm-agent-port")),
        (ServiceQuery(4305, Protocol::UDP), Service::Name("batman")),
        (ServiceQuery(4538, Protocol::TCP), Service::Name("swx-gate")),
        (ServiceQuery(1751, Protocol::UDP), Service::Name("swiftnet")),
        (ServiceQuery(11172, Protocol::TCP), Service::Name("oemcacao-jmxmp")),
        (ServiceQuery(1960, Protocol::UDP), Service::Name("nasmanager")),
        (ServiceQuery(4842, Protocol::TCP), Service::Name("gw-asv")),
        (ServiceQuery(48653, Protocol::TCP), Service::Name("robotraconteur")),
        (ServiceQuery(20480, Protocol::UDP), Service::Name("emwavemsg")),
        (ServiceQuery(4412, Protocol::TCP), Service::Name("smallchat")),
        (ServiceQuery(5719, Protocol::TCP), Service::Name("dpm-agent")),
        (ServiceQuery(8086, Protocol::UDP), Service::Name("d-s-n")),
        (ServiceQuery(24321, Protocol::UDP), Service::Name("ild")),
        (ServiceQuery(2720, Protocol::UDP), Service::Name("wkars")),
        (ServiceQuery(14150, Protocol::TCP), Service::Name("vcscmd")),
        (ServiceQuery(8230, Protocol::TCP), Service::Name("rexecj")),
        (ServiceQuery(7305, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(3914, Protocol::UDP), Service::Name("listcrt-port-2")),
        (ServiceQuery(1079, Protocol::TCP), Service::Name("asprovatalk")),
        (ServiceQuery(312, Protocol::TCP), Service::Name("vslmp")),
        (ServiceQuery(1372, Protocol::UDP), Service::Name("fc-ser")),
        (ServiceQuery(9009, Protocol::UDP), Service::Name("pichat")),
        (ServiceQuery(28000, Protocol::TCP), Service::Name("nxlmd")),
        (ServiceQuery(1881, Protocol::TCP), Service::Name("ibm-mqseries2")),
        (ServiceQuery(1205, Protocol::UDP), Service::Name("accord-mgc")),
        (ServiceQuery(5431, Protocol::TCP), Service::Name("park-agent")),
        (ServiceQuery(4047, Protocol::TCP), Service::Name("ctp-state")),
        (ServiceQuery(3292, Protocol::UDP), Service::Name("cart-o-rama")),
        (ServiceQuery(2518, Protocol::TCP), Service::Name("willy")),
        (ServiceQuery(6241, Protocol::UDP), Service::Name("jeol-nsddp-1")),
        (ServiceQuery(4039, Protocol::UDP), Service::Name("fazzt-admin")),
        (ServiceQuery(6073, Protocol::TCP), Service::Name("directplay8")),
        (ServiceQuery(1312, Protocol::UDP), Service::Name("sti-envision")),
        (ServiceQuery(1855, Protocol::TCP), Service::Name("fiorano-rtrsvc")),
        (ServiceQuery(21590, Protocol::TCP), Service::Name("vofr-gateway")),
        (ServiceQuery(6924, Protocol::TCP), Service::Name("split-ping")),
        (ServiceQuery(2268, Protocol::UDP), Service::Name("amt")),
        (ServiceQuery(8500, Protocol::TCP), Service::Name("fmtp")),
        (ServiceQuery(6508, Protocol::TCP), Service::Name("bdir_pub")),
        (ServiceQuery(4327, Protocol::TCP), Service::Name("jaxer-web")),
        (ServiceQuery(3731, Protocol::TCP), Service::Name("smap")),
        (ServiceQuery(3754, Protocol::TCP), Service::Name("timestenbroker")),
        (ServiceQuery(23402, Protocol::UDP), Service::Name("novar-global")),
        (ServiceQuery(3691, Protocol::TCP), Service::Name("magaya-network")),
        (ServiceQuery(8770, Protocol::UDP), Service::Name("dpap")),
        (ServiceQuery(3483, Protocol::TCP), Service::Name("slim-devices")),
        (ServiceQuery(2526, Protocol::UDP), Service::Name("ema-sent-lm")),
        (ServiceQuery(3636, Protocol::TCP), Service::Name("servistaitsm")),
        (ServiceQuery(39063, Protocol::TCP), Service::Name("vroa")),
        (ServiceQuery(1255, Protocol::UDP), Service::Name("de-cache-query")),
        (ServiceQuery(9222, Protocol::TCP), Service::Name("teamcoherence")),
        (ServiceQuery(8457, Protocol::TCP), Service::Name("nexentamv")),
        (ServiceQuery(124, Protocol::UDP), Service::Name("ansatrader")),
        (ServiceQuery(1357, Protocol::TCP), Service::Name("pegboard")),
        (ServiceQuery(5069, Protocol::UDP), Service::Name("i-net-2000-npr")),
        (ServiceQuery(468, Protocol::UDP), Service::Name("photuris")),
        (ServiceQuery(1323, Protocol::TCP), Service::Name("brcd")),
        (ServiceQuery(9594, Protocol::TCP), Service::Name("msgsys")),
        (ServiceQuery(3651, Protocol::TCP), Service::Name("xrpc-registry")),
        (ServiceQuery(6506, Protocol::UDP), Service::Name("badm_pub")),
        (ServiceQuery(2014, Protocol::TCP), Service::Name("troff")),
        (ServiceQuery(1346, Protocol::TCP), Service::Name("alta-ana-lm")),
        (ServiceQuery(1556, Protocol::UDP), Service::Name("veritas_pbx")),
        (ServiceQuery(4317, Protocol::TCP), Service::Name("opentelemetry")),
        (ServiceQuery(34980, Protocol::UDP), Service::Name("ethercat")),
        (ServiceQuery(39681, Protocol::TCP), Service::Name("turbonote-1")),
        (ServiceQuery(31416, Protocol::UDP), Service::Name("xqosd")),
        (ServiceQuery(2458, Protocol::TCP), Service::Name("griffin")),
        (ServiceQuery(8097, Protocol::TCP), Service::Name("sac")),
        (ServiceQuery(7011, Protocol::UDP), Service::Name("talon-disc")),
        (ServiceQuery(8182, Protocol::TCP), Service::Name("vmware-fdm")),
        (ServiceQuery(1642, Protocol::UDP), Service::Name("isis-am")),
        (ServiceQuery(9214, Protocol::TCP), Service::Name("ipdcesgbs")),
        (ServiceQuery(3849, Protocol::UDP), Service::Name("spw-dnspreload")),
        (ServiceQuery(26263, Protocol::TCP), Service::Name("k3software-cli")),
        (ServiceQuery(1180, Protocol::TCP), Service::Name("mc-client")),
        (ServiceQuery(1445, Protocol::TCP), Service::Name("proxima-lm")),
        (ServiceQuery(1427, Protocol::TCP), Service::Name("mloadd")),
        (ServiceQuery(1360, Protocol::UDP), Service::Name("mimer")),
        (ServiceQuery(2691, Protocol::UDP), Service::Name("itinternet")),
        (ServiceQuery(1809, Protocol::UDP), Service::Name("oracle-vp1")),
        (ServiceQuery(27017, Protocol::TCP), Service::Name("mongod")),
        (ServiceQuery(4159, Protocol::UDP), Service::Name("nss")),
        (ServiceQuery(2062, Protocol::UDP), Service::Name("icg-swp")),
        (ServiceQuery(4106, Protocol::UDP), Service::Name("synchronite")),
        (ServiceQuery(421, Protocol::UDP), Service::Name("ariel2")),
        (ServiceQuery(5352, Protocol::TCP), Service::Name("dns-llq")),
        (ServiceQuery(3229, Protocol::TCP), Service::Name("global-cd-port")),
        (ServiceQuery(2791, Protocol::TCP), Service::Name("mtport-regist")),
        (ServiceQuery(61439, Protocol::TCP), Service::Name("netprowler-manager")),
        (ServiceQuery(27005, Protocol::UDP), Service::Name("flex-lm")),
        (ServiceQuery(1578, Protocol::TCP), Service::Name("jacobus-lm")),
        (ServiceQuery(8052, Protocol::TCP), Service::Name("senomix01")),
        (ServiceQuery(143, Protocol::TCP), Service::Name("imap")),
        (ServiceQuery(1819, Protocol::UDP), Service::Name("plato-lm")),
        (ServiceQuery(188, Protocol::UDP), Service::Name("mumps")),
        (ServiceQuery(1576, Protocol::TCP), Service::Name("moldflow-lm")),
        (ServiceQuery(4687, Protocol::TCP), Service::Name("nst")),
        (ServiceQuery(1447, Protocol::TCP), Service::Name("apri-lm")),
        (ServiceQuery(4076, Protocol::UDP), Service::Name("seraph")),
        (ServiceQuery(101, Protocol::UDP), Service::Name("hostname")),
        (ServiceQuery(3579, Protocol::TCP), Service::Name("ttat3lb")),
        (ServiceQuery(611, Protocol::TCP), Service::Name("npmp-gui")),
        (ServiceQuery(3831, Protocol::UDP), Service::Name("dvapps")),
        (ServiceQuery(1144, Protocol::TCP), Service::Name("fuscript")),
        (ServiceQuery(3865, Protocol::UDP), Service::Name("xpl")),
        (ServiceQuery(1760, Protocol::UDP), Service::Name("www-ldap-gw")),
        (ServiceQuery(1795, Protocol::UDP), Service::Name("dpi-proxy")),
        (ServiceQuery(24003, Protocol::UDP), Service::Name("med-supp")),
        (ServiceQuery(3252, Protocol::UDP), Service::Name("dhe")),
        (ServiceQuery(1038, Protocol::TCP), Service::Name("mtqp")),
        (ServiceQuery(2130, Protocol::UDP), Service::Name("xds")),
        (ServiceQuery(6099, Protocol::TCP), Service::Name("raxa-mgmt")),
        (ServiceQuery(685, Protocol::UDP), Service::Name("mdc-portmapper")),
        (ServiceQuery(531, Protocol::UDP), Service::Name("conference")),
        (ServiceQuery(1834, Protocol::UDP), Service::Name("ardusuni")),
        (ServiceQuery(4677, Protocol::TCP), Service::Name("bcs")),
        (ServiceQuery(3937, Protocol::TCP), Service::Name("dvbservdsc")),
        (ServiceQuery(1215, Protocol::UDP), Service::Name("scanstat-1")),
        (ServiceQuery(1955, Protocol::TCP), Service::Name("abr-secure")),
        (ServiceQuery(25793, Protocol::TCP), Service::Name("vocaltec-hos")),
        (ServiceQuery(3258, Protocol::UDP), Service::Name("ivecon-port")),
        (ServiceQuery(3912, Protocol::TCP), Service::Name("gbmt-stars")),
        (ServiceQuery(33656, Protocol::TCP), Service::Name("snip")),
        (ServiceQuery(5597, Protocol::TCP), Service::Name("ininmessaging")),
        (ServiceQuery(2121, Protocol::TCP), Service::Name("ccproxy-ftp")),
        (ServiceQuery(3713, Protocol::UDP), Service::Name("tftps")),
        (ServiceQuery(5902, Protocol::TCP), Service::Name("vnc-2")),
        (ServiceQuery(18888, Protocol::UDP), Service::Name("apc-necmp")),
        (ServiceQuery(4850, Protocol::TCP), Service::Name("sun-as-nodeagt")),
        (ServiceQuery(6124, Protocol::UDP), Service::Name("pnbs")),
        (ServiceQuery(7080, Protocol::UDP), Service::Name("empowerid")),
        (ServiceQuery(8070, Protocol::TCP), Service::Name("ucs-isc")),
        (ServiceQuery(269, Protocol::TCP), Service::Name("manet")),
        (ServiceQuery(4208, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(5083, Protocol::TCP), Service::Name("qfp")),
        (ServiceQuery(1223, Protocol::TCP), Service::Name("tgp")),
        (ServiceQuery(808, Protocol::TCP), Service::Name("ccproxy-http")),
        (ServiceQuery(24242, Protocol::TCP), Service::Name("filesphere")),
        (ServiceQuery(6703, Protocol::UDP), Service::Name("e-design-web")),
        (ServiceQuery(3011, Protocol::UDP), Service::Name("trusted-web")),
        (ServiceQuery(5747, Protocol::TCP), Service::Name("tunatic")),
        (ServiceQuery(2863, Protocol::TCP), Service::Name("sonardata")),
        (ServiceQuery(8193, Protocol::UDP), Service::Name("sophos")),
        (ServiceQuery(3122, Protocol::UDP), Service::Name("vtr-emulator")),
        (ServiceQuery(2590, Protocol::TCP), Service::Name("idotdist")),
        (ServiceQuery(1450, Protocol::TCP), Service::Name("dwf")),
        (ServiceQuery(3580, Protocol::UDP), Service::Name("nati-svrloc")),
        (ServiceQuery(506, Protocol::UDP), Service::Name("ohimsrv")),
        (ServiceQuery(2576, Protocol::TCP), Service::Name("tclprodebugger")),
        (ServiceQuery(4371, Protocol::TCP), Service::Name("l2c-control")),
        (ServiceQuery(2016, Protocol::TCP), Service::Name("bootserver")),
        (ServiceQuery(5987, Protocol::UDP), Service::Name("wbem-rmi")),
        (ServiceQuery(2539, Protocol::TCP), Service::Name("vsiadmin")),
        (ServiceQuery(6489, Protocol::TCP), Service::Name("sun-sr-admin")),
        (ServiceQuery(5673, Protocol::TCP), Service::Name("jms")),
        (ServiceQuery(6456, Protocol::UDP), Service::Name("skip-cert-send")),
        (ServiceQuery(7128, Protocol::TCP), Service::Name("scenidm")),
        (ServiceQuery(6963, Protocol::TCP), Service::Name("swismgr1")),
        (ServiceQuery(4683, Protocol::TCP), Service::Name("spike")),
        (ServiceQuery(3906, Protocol::TCP), Service::Name("topovista-data")),
        (ServiceQuery(5020, Protocol::TCP), Service::Name("zenginkyo-1")),
        (ServiceQuery(5167, Protocol::TCP), Service::Name("scte104")),
        (ServiceQuery(13785, Protocol::TCP), Service::Name("nbdb")),
        (ServiceQuery(2150, Protocol::TCP), Service::Name("dynamic3d")),
        (ServiceQuery(41111, Protocol::UDP), Service::Name("fs-qos")),
        (ServiceQuery(520, Protocol::TCP), Service::Name("efs")),
        (ServiceQuery(1189, Protocol::UDP), Service::Name("unet")),
        (ServiceQuery(5422, Protocol::TCP), Service::Name("salient-mux")),
        (ServiceQuery(1843, Protocol::UDP), Service::Name("netopia-vo5")),
        (ServiceQuery(3322, Protocol::TCP), Service::Name("active-net")),
        (ServiceQuery(1887, Protocol::TCP), Service::Name("filex-lport")),
        (ServiceQuery(2545, Protocol::UDP), Service::Name("sis-emt")),
        (ServiceQuery(3543, Protocol::TCP), Service::Name("qftest-lookup")),
        (ServiceQuery(31727, Protocol::TCP), Service::Name("diagd")),
        (ServiceQuery(5990, Protocol::TCP), Service::Name("wbem-exp-https")),
        (ServiceQuery(3675, Protocol::UDP), Service::Name("calltrax")),
        (ServiceQuery(1181, Protocol::TCP), Service::Name("3comnetman")),
        (ServiceQuery(4231, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1423, Protocol::UDP), Service::Name("essbase")),
        (ServiceQuery(3301, Protocol::TCP), Service::Name("tarantool")),
        (ServiceQuery(2808, Protocol::TCP), Service::Name("j-lan-p")),
        (ServiceQuery(19007, Protocol::TCP), Service::Name("scintilla")),
        (ServiceQuery(10800, Protocol::TCP), Service::Name("gap")),
        (ServiceQuery(5666, Protocol::TCP), Service::Name("nrpe")),
        (ServiceQuery(1601, Protocol::TCP), Service::Name("aas")),
        (ServiceQuery(6123, Protocol::TCP), Service::Name("backup-express")),
        (ServiceQuery(1238, Protocol::TCP), Service::Name("hacl-qs")),
        (ServiceQuery(1969, Protocol::UDP), Service::Name("lipsinc1")),
        (ServiceQuery(353, Protocol::TCP), Service::Name("ndsauth")),
        (ServiceQuery(2408, Protocol::TCP), Service::Name("optimanet")),
        (ServiceQuery(33331, Protocol::UDP), Service::Name("diamondport")),
        (ServiceQuery(1596, Protocol::TCP), Service::Name("radio-sm")),
        (ServiceQuery(4311, Protocol::TCP), Service::Name("p6ssmc")),
        (ServiceQuery(3466, Protocol::TCP), Service::Name("workflow")),
        (ServiceQuery(25006, Protocol::UDP), Service::Name("icl-twobase7")),
        (ServiceQuery(4743, Protocol::TCP), Service::Name("openhpid")),
        (ServiceQuery(5223, Protocol::UDP), Service::Name("hpvirtgrp")),
        (ServiceQuery(9280, Protocol::UDP), Service::Name("pgps")),
        (ServiceQuery(8292, Protocol::TCP), Service::Name("blp3")),
        (ServiceQuery(690, Protocol::UDP), Service::Name("vatp")),
        (ServiceQuery(1703, Protocol::TCP), Service::Name("hb-engine")),
        (ServiceQuery(2086, Protocol::UDP), Service::Name("gnunet")),
        (ServiceQuery(56, Protocol::TCP), Service::Name("xns-auth")),
        (ServiceQuery(3266, Protocol::TCP), Service::Name("ns-cfg-server")),
        (ServiceQuery(3192, Protocol::UDP), Service::Name("firemonrcc")),
        (ServiceQuery(2671, Protocol::TCP), Service::Name("newlixreg")),
        (ServiceQuery(45002, Protocol::TCP), Service::Name("rs-status")),
        (ServiceQuery(764, Protocol::UDP), Service::Name("omserv")),
        (ServiceQuery(2795, Protocol::TCP), Service::Name("livestats")),
        (ServiceQuery(7347, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(1851, Protocol::UDP), Service::Name("ctcd")),
        (ServiceQuery(4727, Protocol::TCP), Service::Name("fcis")),
        (ServiceQuery(9164, Protocol::UDP), Service::Name("apani5")),
        (ServiceQuery(566, Protocol::TCP), Service::Name("streettalk")),
        (ServiceQuery(1215, Protocol::TCP), Service::Name("scanstat-1")),
        (ServiceQuery(1222, Protocol::UDP), Service::Name("nerv")),
        (ServiceQuery(147, Protocol::TCP), Service::Name("iso-ip")),
        (ServiceQuery(3545, Protocol::TCP), Service::Name("camac")),
        (ServiceQuery(3905, Protocol::TCP), Service::Name("mupdate")),
        (ServiceQuery(9111, Protocol::TCP), Service::Name("DragonIDSConsole")),
        (ServiceQuery(4391, Protocol::TCP), Service::Name("apwi-imserver")),
        (ServiceQuery(2596, Protocol::UDP), Service::Name("worldfusion2")),
        (ServiceQuery(2063, Protocol::TCP), Service::Name("icg-bridge")),
        (ServiceQuery(16994, Protocol::TCP), Service::Name("amt-redir-tcp")),
        (ServiceQuery(1677, Protocol::UDP), Service::Name("groupwise")),
        (ServiceQuery(2173, Protocol::TCP), Service::Name("msfw-replica")),
        (ServiceQuery(1392, Protocol::UDP), Service::Name("iclpv-pm")),
        (ServiceQuery(1666, Protocol::TCP), Service::Name("netview-aix-6")),
        (ServiceQuery(5984, Protocol::UDP), Service::Name("couchdb")),
        (ServiceQuery(405, Protocol::UDP), Service::Name("ncld")),
        (ServiceQuery(2264, Protocol::TCP), Service::Name("apx500api-1")),
        (ServiceQuery(140, Protocol::TCP), Service::Name("emfis-data")),
        (ServiceQuery(2605, Protocol::UDP), Service::Name("nsc-posa")),
        (ServiceQuery(18187, Protocol::UDP), Service::Name("opsec-ela")),
        (ServiceQuery(2800, Protocol::TCP), Service::Name("acc-raid")),
        (ServiceQuery(9444, Protocol::UDP), Service::Name("wso2esb-console")),
        (ServiceQuery(9207, Protocol::TCP), Service::Name("wap-vcal-s")),
        (ServiceQuery(406, Protocol::UDP), Service::Name("imsp")),
        (ServiceQuery(6482, Protocol::UDP), Service::Name("ldoms-mgmt")),
        (ServiceQuery(36443, Protocol::TCP), Service::Name("m2ap")),
        (ServiceQuery(2615, Protocol::UDP), Service::Name("firepower")),
        (ServiceQuery(3345, Protocol::TCP), Service::Name("influence")),
        (ServiceQuery(8055, Protocol::TCP), Service::Name("senomix04")),
        (ServiceQuery(5136, Protocol::UDP), Service::Name("minotaur-sa")),
        (ServiceQuery(3956, Protocol::TCP), Service::Name("gvcp")),
        (ServiceQuery(3847, Protocol::UDP), Service::Name("msfw-control")),
        (ServiceQuery(408, Protocol::UDP), Service::Name("prm-sm")),
        (ServiceQuery(1954, Protocol::UDP), Service::Name("abr-api")),
        (ServiceQuery(262, Protocol::TCP), Service::Name("arcisdms")),
        (ServiceQuery(4126, Protocol::UDP), Service::Name("ddrepl")),
        (ServiceQuery(7501, Protocol::TCP), Service::Name("ovbus")),
        (ServiceQuery(311, Protocol::TCP), Service::Name("asip-webadmin")),
        (ServiceQuery(320, Protocol::TCP), Service::Name("ptp-general")),
        (ServiceQuery(2976, Protocol::TCP), Service::Name("cns-srv-port")),
        (ServiceQuery(9295, Protocol::UDP), Service::Name("armcenterhttps")),
        (ServiceQuery(9294, Protocol::TCP), Service::Name("armcenterhttp")),
        (ServiceQuery(9618, Protocol::UDP), Service::Name("condor")),
        (ServiceQuery(2703, Protocol::UDP), Service::Name("sms-chat")),
        (ServiceQuery(11235, Protocol::TCP), Service::Name("xcompute")),
        (ServiceQuery(3007, Protocol::UDP), Service::Name("lotusmtap")),
        (ServiceQuery(3411, Protocol::TCP), Service::Name("biolink-auth")),
        (ServiceQuery(1892, Protocol::UDP), Service::Name("childkey-ctrl")),
        (ServiceQuery(21, Protocol::UDP), Service::Name("ftp")),
        (ServiceQuery(9600, Protocol::UDP), Service::Name("micromuse-ncpw")),
        (ServiceQuery(2425, Protocol::UDP), Service::Name("fjitsuappmgr")),
        (ServiceQuery(2635, Protocol::TCP), Service::Name("backburner")),
        (ServiceQuery(2105, Protocol::UDP), Service::Name("eklogin")),
        (ServiceQuery(2487, Protocol::UDP), Service::Name("pns")),
        (ServiceQuery(8021, Protocol::UDP), Service::Name("intu-ec-client")),
        (ServiceQuery(1705, Protocol::TCP), Service::Name("slingshot")),
        (ServiceQuery(1432, Protocol::TCP), Service::Name("blueberry-lm")),
        (ServiceQuery(3991, Protocol::UDP), Service::Name("bv-smcsrv")),
        (ServiceQuery(2127, Protocol::TCP), Service::Name("index-pc-wb")),
        (ServiceQuery(4685, Protocol::UDP), Service::Name("autopac")),
        (ServiceQuery(3673, Protocol::TCP), Service::Name("mediavault-gui")),
        (ServiceQuery(4019, Protocol::TCP), Service::Name("talarian-mcast5")),
        (ServiceQuery(4261, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(694, Protocol::UDP), Service::Name("ha-cluster")),
        (ServiceQuery(7013, Protocol::TCP), Service::Name("microtalon-dis")),
        (ServiceQuery(3143, Protocol::UDP), Service::Name("seaview")),
        (ServiceQuery(458, Protocol::UDP), Service::Name("appleqtc")),
        (ServiceQuery(2512, Protocol::UDP), Service::Name("citrixima")),
        (ServiceQuery(4430, Protocol::TCP), Service::Name("rsqlserver")),
        (ServiceQuery(5022, Protocol::TCP), Service::Name("mice")),
        (ServiceQuery(1440, Protocol::UDP), Service::Name("eicon-slp")),
        (ServiceQuery(102, Protocol::TCP), Service::Name("iso-tsap")),
        (ServiceQuery(4867, Protocol::TCP), Service::Name("unify-debug")),
        (ServiceQuery(769, Protocol::TCP), Service::Name("vid")),
        (ServiceQuery(3963, Protocol::UDP), Service::Name("thrp")),
        (ServiceQuery(6054, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(461, Protocol::UDP), Service::Name("datasurfsrv")),
        (ServiceQuery(3133, Protocol::UDP), Service::Name("prism-deploy")),
        (ServiceQuery(6471, Protocol::UDP), Service::Name("lvision-lm")),
        (ServiceQuery(9995, Protocol::UDP), Service::Name("palace-4")),
        (ServiceQuery(6057, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(3374, Protocol::TCP), Service::Name("cluster-disc")),
        (ServiceQuery(4063, Protocol::UDP), Service::Name("ice-router")),
        (ServiceQuery(3601, Protocol::TCP), Service::Name("visinet-gui")),
        (ServiceQuery(6404, Protocol::TCP), Service::Name("boe-filesvr")),
        (ServiceQuery(30400, Protocol::TCP), Service::Name("gs-realtime")),
        (ServiceQuery(2964, Protocol::UDP), Service::Name("bullant-srap")),
        (ServiceQuery(2202, Protocol::TCP), Service::Name("imtc-map")),
        (ServiceQuery(6777, Protocol::TCP), Service::Name("ntz-tracker")),
        (ServiceQuery(5639, Protocol::TCP), Service::Name("ics")),
        (ServiceQuery(2324, Protocol::UDP), Service::Name("cosmocall")),
        (ServiceQuery(3448, Protocol::TCP), Service::Name("dnc-port")),
        (ServiceQuery(7546, Protocol::UDP), Service::Name("cfs")),
        (ServiceQuery(1501, Protocol::UDP), Service::Name("sas-3")),
        (ServiceQuery(430, Protocol::TCP), Service::Name("utmpsd")),
        (ServiceQuery(4488, Protocol::UDP), Service::Name("awacs-ice")),
        (ServiceQuery(4553, Protocol::TCP), Service::Name("icshostsvc")),
        (ServiceQuery(759, Protocol::TCP), Service::Name("con")),
        (ServiceQuery(6326, Protocol::TCP), Service::Name("dt-vra")),
        (ServiceQuery(653, Protocol::UDP), Service::Name("repscmd")),
        (ServiceQuery(7323, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(2595, Protocol::UDP), Service::Name("worldfusion1")),
        (ServiceQuery(4752, Protocol::UDP), Service::Name("snap")),
        (ServiceQuery(2628, Protocol::TCP), Service::Name("dict")),
        (ServiceQuery(2280, Protocol::TCP), Service::Name("lnvpoller")),
        (ServiceQuery(3489, Protocol::UDP), Service::Name("dtp-dia")),
        (ServiceQuery(2253, Protocol::UDP), Service::Name("dtv-chan-req")),
        (ServiceQuery(1872, Protocol::TCP), Service::Name("canocentral1")),
        (ServiceQuery(477, Protocol::UDP), Service::Name("ss7ns")),
        (ServiceQuery(1, Protocol::UDP), Service::Name("tcpmux")),
        (ServiceQuery(8128, Protocol::UDP), Service::Name("paycash-online")),
        (ServiceQuery(1800, Protocol::UDP), Service::Name("ansys-lm")),
        (ServiceQuery(2598, Protocol::TCP), Service::Name("citriximaclient")),
        (ServiceQuery(3368, Protocol::TCP), Service::Name("satvid-datalnk")),
        (ServiceQuery(1948, Protocol::TCP), Service::Name("eye2eye")),
        (ServiceQuery(3223, Protocol::UDP), Service::Name("digivote")),
        (ServiceQuery(1221, Protocol::TCP), Service::Name("sweetware-apps")),
        (ServiceQuery(1534, Protocol::TCP), Service::Name("micromuse-lm")),
        (ServiceQuery(3804, Protocol::UDP), Service::Name("iqnet-port")),
        (ServiceQuery(3930, Protocol::UDP), Service::Name("syam-webserver")),
        (ServiceQuery(5004, Protocol::UDP), Service::Name("avt-profile-1")),
        (ServiceQuery(2252, Protocol::UDP), Service::Name("njenet-ssl")),
        (ServiceQuery(5269, Protocol::UDP), Service::Name("xmpp-server")),
        (ServiceQuery(1793, Protocol::TCP), Service::Name("rsc-robot")),
        (ServiceQuery(2664, Protocol::TCP), Service::Name("patrol-mq-gm")),
        (ServiceQuery(5777, Protocol::TCP), Service::Name("dali-port")),
        (ServiceQuery(624, Protocol::UDP), Service::Name("cryptoadmin")),
        (ServiceQuery(49000, Protocol::TCP), Service::Name("matahari")),
        (ServiceQuery(4361, Protocol::TCP), Service::Name("nacnl")),
        (ServiceQuery(31, Protocol::UDP), Service::Name("msg-auth")),
        (ServiceQuery(7800, Protocol::UDP), Service::Name("asr")),
        (ServiceQuery(3285, Protocol::TCP), Service::Name("plato")),
        (ServiceQuery(1042, Protocol::UDP), Service::Name("afrog")),
        (ServiceQuery(5637, Protocol::TCP), Service::Name("cssc")),
        (ServiceQuery(97, Protocol::UDP), Service::Name("swift-rvf")),
        (ServiceQuery(2385, Protocol::TCP), Service::Name("sd-data")),
        (ServiceQuery(1565, Protocol::TCP), Service::Name("winddlb")),
        (ServiceQuery(1262, Protocol::UDP), Service::Name("qnts-orb")),
        (ServiceQuery(9617, Protocol::TCP), Service::Name("erunbook_server")),
        (ServiceQuery(4131, Protocol::TCP), Service::Name("stars")),
        (ServiceQuery(3442, Protocol::TCP), Service::Name("connect-server")),
        (ServiceQuery(5453, Protocol::UDP), Service::Name("surebox")),
        (ServiceQuery(2213, Protocol::UDP), Service::Name("kali")),
        (ServiceQuery(4145, Protocol::UDP), Service::Name("vvr-control")),
        (ServiceQuery(7427, Protocol::UDP), Service::Name("oveadmgr")),
        (ServiceQuery(1091, Protocol::TCP), Service::Name("ff-sm")),
        (ServiceQuery(1503, Protocol::TCP), Service::Name("imtc-mcs")),
        (ServiceQuery(492, Protocol::TCP), Service::Name("ticf-1")),
        (ServiceQuery(7002, Protocol::TCP), Service::Name("afs3-prserver")),
        (ServiceQuery(7161, Protocol::UDP), Service::Name("cabsm-comm")),
        (ServiceQuery(1379, Protocol::UDP), Service::Name("dbreporter")),
        (ServiceQuery(4684, Protocol::TCP), Service::Name("rfid-rp1")),
        (ServiceQuery(1688, Protocol::TCP), Service::Name("nsjtp-data")),
        (ServiceQuery(178, Protocol::TCP), Service::Name("nextstep")),
        (ServiceQuery(1233, Protocol::UDP), Service::Name("univ-appserver")),
        (ServiceQuery(2919, Protocol::UDP), Service::Name("roboer")),
        (ServiceQuery(4173, Protocol::TCP), Service::Name("mma-discovery")),
        (ServiceQuery(3962, Protocol::TCP), Service::Name("sbi-agent")),
        (ServiceQuery(1885, Protocol::UDP), Service::Name("vrtstrapserver")),
        (ServiceQuery(3113, Protocol::UDP), Service::Name("cs-auth-svr")),
        (ServiceQuery(2867, Protocol::TCP), Service::Name("esps-portal")),
        (ServiceQuery(2162, Protocol::TCP), Service::Name("navisphere")),
        (ServiceQuery(351, Protocol::TCP), Service::Name("matip-type-b")),
        (ServiceQuery(3797, Protocol::UDP), Service::Name("idps")),
        (ServiceQuery(5627, Protocol::TCP), Service::Name("ninaf")),
        (ServiceQuery(3223, Protocol::TCP), Service::Name("digivote")),
        (ServiceQuery(1871, Protocol::TCP), Service::Name("canocentral0")),
        (ServiceQuery(5425, Protocol::TCP), Service::Name("br-channel")),
        (ServiceQuery(6629, Protocol::TCP), Service::Name("nexgen-aux")),
        (ServiceQuery(2606, Protocol::UDP), Service::Name("netmon")),
        (ServiceQuery(6997, Protocol::TCP), Service::Name("MobilitySrv")),
        (ServiceQuery(500, Protocol::TCP), Service::Name("isakmp")),
        (ServiceQuery(3939, Protocol::UDP), Service::Name("aamp")),
        (ServiceQuery(1604, Protocol::UDP), Service::Name("icabrowser")),
        (ServiceQuery(247, Protocol::UDP), Service::Name("subntbcst_tftp")),
        (ServiceQuery(1714, Protocol::UDP), Service::Name("sesi-lm")),
        (ServiceQuery(1696, Protocol::TCP), Service::Name("rrifmm")),
        (ServiceQuery(4271, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1989, Protocol::UDP), Service::Name("tr-rsrb-p3")),
        (ServiceQuery(829, Protocol::TCP), Service::Name("pkix-3-ca-ra")),
        (ServiceQuery(8061, Protocol::TCP), Service::Name("nikatron-dev")),
        (ServiceQuery(105, Protocol::TCP), Service::Name("csnet-ns")),
        (ServiceQuery(25004, Protocol::UDP), Service::Name("icl-twobase5")),
        (ServiceQuery(1998, Protocol::TCP), Service::Name("x25-svc-port")),
        (ServiceQuery(13721, Protocol::TCP), Service::Name("netbackup")),
        (ServiceQuery(1982, Protocol::TCP), Service::Name("estamp")),
        (ServiceQuery(2521, Protocol::UDP), Service::Name("adaptecmgr")),
        (ServiceQuery(1359, Protocol::TCP), Service::Name("ftsrv")),
        (ServiceQuery(2660, Protocol::UDP), Service::Name("gcmonitor")),
        (ServiceQuery(1826, Protocol::TCP), Service::Name("ardt")),
        (ServiceQuery(2678, Protocol::TCP), Service::Name("gadgetgate2way")),
        (ServiceQuery(1840, Protocol::TCP), Service::Name("netopia-vo2")),
        (ServiceQuery(3771, Protocol::UDP), Service::Name("paging-port")),
        (ServiceQuery(4341, Protocol::TCP), Service::Name("lisp-data")),
        (ServiceQuery(7330, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(2004, Protocol::UDP), Service::Name("emce")),
        (ServiceQuery(39, Protocol::UDP), Service::Name("rlp")),
        (ServiceQuery(1801, Protocol::TCP), Service::Name("msmq")),
        (ServiceQuery(4423, Protocol::TCP), Service::Name("thingkit")),
        (ServiceQuery(7071, Protocol::TCP), Service::Name("iwg1")),
        (ServiceQuery(3369, Protocol::TCP), Service::Name("satvid-datalnk")),
        (ServiceQuery(2754, Protocol::TCP), Service::Name("apollo-cc")),
        (ServiceQuery(1382, Protocol::TCP), Service::Name("udt_os")),
        (ServiceQuery(40842, Protocol::UDP), Service::Name("csccredir")),
        (ServiceQuery(4443, Protocol::UDP), Service::Name("pharos")),
        (ServiceQuery(3518, Protocol::TCP), Service::Name("artifact-msg")),
        (ServiceQuery(6662, Protocol::TCP), Service::Name("radmind")),
        (ServiceQuery(5222, Protocol::UDP), Service::Name("xmpp-client")),
        (ServiceQuery(4368, Protocol::UDP), Service::Name("wxbrief")),
        (ServiceQuery(3484, Protocol::TCP), Service::Name("gbs-stp")),
        (ServiceQuery(8001, Protocol::UDP), Service::Name("vcom-tunnel")),
        (ServiceQuery(2235, Protocol::TCP), Service::Name("sercomm-wlink")),
        (ServiceQuery(3004, Protocol::UDP), Service::Name("csoftragent")),
        (ServiceQuery(3826, Protocol::UDP), Service::Name("wormux")),
        (ServiceQuery(2844, Protocol::TCP), Service::Name("bpcp-poll")),
        (ServiceQuery(3100, Protocol::TCP), Service::Name("opcon-xps")),
        (ServiceQuery(4403, Protocol::TCP), Service::Name("ds-user")),
        (ServiceQuery(7019, Protocol::TCP), Service::Name("doceri-ctl")),
        (ServiceQuery(2422, Protocol::UDP), Service::Name("crmsbits")),
        (ServiceQuery(5987, Protocol::TCP), Service::Name("wbem-rmi")),
        (ServiceQuery(1037, Protocol::UDP), Service::Name("ams")),
        (ServiceQuery(3431, Protocol::TCP), Service::Name("ndl-als")),
        (ServiceQuery(9443, Protocol::UDP), Service::Name("tungsten-https")),
        (ServiceQuery(5145, Protocol::UDP), Service::Name("rmonitor_secure")),
        (ServiceQuery(22321, Protocol::TCP), Service::Name("wnn6_Tw")),
        (ServiceQuery(1897, Protocol::TCP), Service::Name("metaagent")),
        (ServiceQuery(9161, Protocol::UDP), Service::Name("apani2")),
        (ServiceQuery(1878, Protocol::UDP), Service::Name("drmsmc")),
        (ServiceQuery(2543, Protocol::TCP), Service::Name("reftek")),
        (ServiceQuery(5863, Protocol::TCP), Service::Name("ppsuitemsg")),
        (ServiceQuery(1173, Protocol::TCP), Service::Name("d-cinema-rrp")),
        (ServiceQuery(3678, Protocol::UDP), Service::Name("ipr-dglt")),
        (ServiceQuery(1080, Protocol::UDP), Service::Name("socks")),
        (ServiceQuery(3631, Protocol::TCP), Service::Name("cs-services")),
        (ServiceQuery(2779, Protocol::TCP), Service::Name("lbc-sync")),
        (ServiceQuery(7320, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(23, Protocol::TCP), Service::Name("telnet")),
        (ServiceQuery(2421, Protocol::UDP), Service::Name("g-talk")),
        (ServiceQuery(1476, Protocol::UDP), Service::Name("clvm-cfg")),
        (ServiceQuery(6444, Protocol::UDP), Service::Name("sge_qmaster")),
        (ServiceQuery(1890, Protocol::TCP), Service::Name("wilkenlistener")),
        (ServiceQuery(6141, Protocol::TCP), Service::Name("meta-corp")),
        (ServiceQuery(3600, Protocol::UDP), Service::Name("trap-daemon")),
        (ServiceQuery(4203, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(662, Protocol::TCP), Service::Name("pftp")),
        (ServiceQuery(8088, Protocol::UDP), Service::Name("radan-http")),
        (ServiceQuery(3909, Protocol::TCP), Service::Name("surfcontrolcpa")),
        (ServiceQuery(2668, Protocol::TCP), Service::Name("alarm-clock-c")),
        (ServiceQuery(1183, Protocol::TCP), Service::Name("llsurfup-http")),
        (ServiceQuery(2622, Protocol::UDP), Service::Name("metricadbc")),
        (ServiceQuery(4791, Protocol::TCP), Service::Name("roce")),
        (ServiceQuery(13218, Protocol::UDP), Service::Name("emc-vcas-udp")),
        (ServiceQuery(2858, Protocol::UDP), Service::Name("ecnp")),
        (ServiceQuery(8404, Protocol::TCP), Service::Name("svcloud")),
        (ServiceQuery(2201, Protocol::TCP), Service::Name("ats")),
        (ServiceQuery(12005, Protocol::TCP), Service::Name("dbisamserver1")),
        (ServiceQuery(1203, Protocol::UDP), Service::Name("ssslic-mgr")),
        (ServiceQuery(3306, Protocol::TCP), Service::Name("mysql")),
        (ServiceQuery(2675, Protocol::TCP), Service::Name("ttc-etap")),
        (ServiceQuery(1358, Protocol::UDP), Service::Name("connlcli")),
        (ServiceQuery(7339, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(3395, Protocol::UDP), Service::Name("dyna-lm")),
        (ServiceQuery(9206, Protocol::UDP), Service::Name("wap-vcard-s")),
        (ServiceQuery(4879, Protocol::TCP), Service::Name("wsdl-event")),
        (ServiceQuery(3840, Protocol::UDP), Service::Name("flirtmitmir")),
        (ServiceQuery(2192, Protocol::TCP), Service::Name("asdis")),
        (ServiceQuery(1496, Protocol::UDP), Service::Name("liberty-lm")),
        (ServiceQuery(6409, Protocol::TCP), Service::Name("boe-resssvr3")),
        (ServiceQuery(2697, Protocol::TCP), Service::Name("oce-snmp-trap")),
        (ServiceQuery(514, Protocol::TCP), Service::Name("shell")),
        (ServiceQuery(6121, Protocol::TCP), Service::Name("spdy")),
        (ServiceQuery(4302, Protocol::TCP), Service::Name("d-data-control")),
        (ServiceQuery(281, Protocol::UDP), Service::Name("personal-link")),
        (ServiceQuery(4937, Protocol::TCP), Service::Name("atsc-mh-ssc")),
        (ServiceQuery(1634, Protocol::UDP), Service::Name("loaprobe")),
        (ServiceQuery(1715, Protocol::TCP), Service::Name("houdini-lm")),
        (ServiceQuery(8675, Protocol::TCP), Service::Name("msi-cps-rm")),
        (ServiceQuery(4070, Protocol::TCP), Service::Name("tripe")),
        (ServiceQuery(3635, Protocol::TCP), Service::Name("sdo")),
        (ServiceQuery(6140, Protocol::TCP), Service::Name("pulsonixnls")),
        (ServiceQuery(1837, Protocol::UDP), Service::Name("csoft1")),
        (ServiceQuery(547, Protocol::UDP), Service::Name("dhcpv6-server")),
        (ServiceQuery(48556, Protocol::UDP), Service::Name("com-bardac-dw")),
        (ServiceQuery(2183, Protocol::UDP), Service::Name("cgn-config")),
        (ServiceQuery(3201, Protocol::TCP), Service::Name("cpq-tasksmart")),
        (ServiceQuery(3387, Protocol::UDP), Service::Name("backroomnet")),
        (ServiceQuery(777, Protocol::UDP), Service::Name("multiling-http")),
        (ServiceQuery(3540, Protocol::TCP), Service::Name("pnrp-port")),
        (ServiceQuery(2973, Protocol::TCP), Service::Name("svnetworks")),
        (ServiceQuery(32896, Protocol::UDP), Service::Name("idmgratm")),
        (ServiceQuery(2473, Protocol::UDP), Service::Name("aker-cdp")),
        (ServiceQuery(1178, Protocol::UDP), Service::Name("sgi-storman")),
        (ServiceQuery(19539, Protocol::UDP), Service::Name("fxuptp")),
        (ServiceQuery(6012, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(650, Protocol::TCP), Service::Name("obex")),
        (ServiceQuery(2042, Protocol::UDP), Service::Name("isis")),
        (ServiceQuery(7588, Protocol::TCP), Service::Name("sun-lm")),
        (ServiceQuery(3197, Protocol::UDP), Service::Name("embrace-dp-s")),
        (ServiceQuery(4218, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3336, Protocol::UDP), Service::Name("directv-tick")),
        (ServiceQuery(4969, Protocol::TCP), Service::Name("ccss-qmm")),
        (ServiceQuery(376, Protocol::TCP), Service::Name("nip")),
        (ServiceQuery(1872, Protocol::UDP), Service::Name("canocentral1")),
        (ServiceQuery(4146, Protocol::UDP), Service::Name("tgcconnect")),
        (ServiceQuery(2066, Protocol::TCP), Service::Name("aura")),
        (ServiceQuery(1500, Protocol::TCP), Service::Name("vlsi-lm")),
        (ServiceQuery(2542, Protocol::TCP), Service::Name("udrawgraph")),
        (ServiceQuery(2092, Protocol::UDP), Service::Name("descent3")),
        (ServiceQuery(3605, Protocol::UDP), Service::Name("comcam-io")),
        (ServiceQuery(8190, Protocol::TCP), Service::Name("gcp-rphy")),
        (ServiceQuery(6112, Protocol::UDP), Service::Name("dtspcd")),
        (ServiceQuery(2646, Protocol::UDP), Service::Name("and-lm")),
        (ServiceQuery(6627, Protocol::UDP), Service::Name("nexgen")),
        (ServiceQuery(2969, Protocol::TCP), Service::Name("essp")),
        (ServiceQuery(4868, Protocol::TCP), Service::Name("phrelay")),
        (ServiceQuery(2113, Protocol::UDP), Service::Name("hsl-storm")),
        (ServiceQuery(5304, Protocol::TCP), Service::Name("hacl-local")),
        (ServiceQuery(6149, Protocol::TCP), Service::Name("tal-pod")),
        (ServiceQuery(1462, Protocol::UDP), Service::Name("world-lm")),
        (ServiceQuery(6355, Protocol::UDP), Service::Name("pmcs")),
        (ServiceQuery(7325, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(38, Protocol::UDP), Service::Name("rap")),
        (ServiceQuery(31765, Protocol::TCP), Service::Name("gamesmith-port")),
        (ServiceQuery(910, Protocol::TCP), Service::Name("kink")),
        (ServiceQuery(5429, Protocol::UDP), Service::Name("base")),
        (ServiceQuery(287, Protocol::TCP), Service::Name("k-block")),
        (ServiceQuery(6997, Protocol::UDP), Service::Name("MobilitySrv")),
        (ServiceQuery(10542, Protocol::TCP), Service::Name("MOS-aux")),
        (ServiceQuery(6514, Protocol::UDP), Service::Name("syslog-tls")),
        (ServiceQuery(4595, Protocol::TCP), Service::Name("ias-paging")),
        (ServiceQuery(10007, Protocol::UDP), Service::Name("mvs-capacity")),
        (ServiceQuery(3475, Protocol::TCP), Service::Name("genisar-port")),
        (ServiceQuery(46336, Protocol::TCP), Service::Name("inedo")),
        (ServiceQuery(3876, Protocol::UDP), Service::Name("dl_agent")),
        (ServiceQuery(7316, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(410, Protocol::UDP), Service::Name("decladebug")),
        (ServiceQuery(4344, Protocol::UDP), Service::Name("vinainstall")),
        (ServiceQuery(1206, Protocol::UDP), Service::Name("anthony-data")),
        (ServiceQuery(4941, Protocol::UDP), Service::Name("eq-office-4941")),
        (ServiceQuery(2959, Protocol::TCP), Service::Name("rmopagt")),
        (ServiceQuery(3888, Protocol::TCP), Service::Name("ciphire-serv")),
        (ServiceQuery(7227, Protocol::UDP), Service::Name("ramp")),
        (ServiceQuery(10114, Protocol::TCP), Service::Name("netiq-qcheck")),
        (ServiceQuery(4394, Protocol::TCP), Service::Name("apwi-disc")),
        (ServiceQuery(8376, Protocol::TCP), Service::Name("cruise-enum")),
        (ServiceQuery(634, Protocol::TCP), Service::Name("ginad")),
        (ServiceQuery(94, Protocol::UDP), Service::Name("objcall")),
        (ServiceQuery(3624, Protocol::UDP), Service::Name("dist-upgrade")),
        (ServiceQuery(2943, Protocol::TCP), Service::Name("ttnrepository")),
        (ServiceQuery(3863, Protocol::UDP), Service::Name("asap-tcp")),
        (ServiceQuery(4117, Protocol::TCP), Service::Name("hillrserv")),
        (ServiceQuery(2250, Protocol::TCP), Service::Name("remote-collab")),
        (ServiceQuery(3148, Protocol::UDP), Service::Name("nm-game-admin")),
        (ServiceQuery(13714, Protocol::TCP), Service::Name("netbackup")),
        (ServiceQuery(7726, Protocol::TCP), Service::Name("freezexservice")),
        (ServiceQuery(7563, Protocol::TCP), Service::Name("cfw")),
        (ServiceQuery(3085, Protocol::TCP), Service::Name("pcihreq")),
        (ServiceQuery(2115, Protocol::UDP), Service::Name("kdm")),
        (ServiceQuery(6786, Protocol::UDP), Service::Name("smc-jmx")),
        (ServiceQuery(4660, Protocol::TCP), Service::Name("mosmig")),
        (ServiceQuery(4316, Protocol::TCP), Service::Name("choiceview-clt")),
        (ServiceQuery(1803, Protocol::TCP), Service::Name("hp-hcip-gwy")),
        (ServiceQuery(4389, Protocol::UDP), Service::Name("xandros-cms")),
        (ServiceQuery(25576, Protocol::TCP), Service::Name("sauterdongle")),
        (ServiceQuery(2975, Protocol::UDP), Service::Name("fjmpcm")),
        (ServiceQuery(6671, Protocol::TCP), Service::Name("p4p-portal")),
        (ServiceQuery(1575, Protocol::TCP), Service::Name("oraclenames")),
        (ServiceQuery(10110, Protocol::TCP), Service::Name("nmea-0183")),
        (ServiceQuery(1086, Protocol::TCP), Service::Name("cplscrambler-lg")),
        (ServiceQuery(3495, Protocol::TCP), Service::Name("seclayer-tcp")),
        (ServiceQuery(9595, Protocol::TCP), Service::Name("pds")),
        (ServiceQuery(4192, Protocol::TCP), Service::Name("azeti")),
        (ServiceQuery(997, Protocol::UDP), Service::Name("maitrd")),
        (ServiceQuery(487, Protocol::UDP), Service::Name("saft")),
        (ServiceQuery(1927, Protocol::TCP), Service::Name("videte-cipc")),
        (ServiceQuery(5165, Protocol::UDP), Service::Name("ife_icorp")),
        (ServiceQuery(13710, Protocol::TCP), Service::Name("netbackup")),
        (ServiceQuery(10990, Protocol::UDP), Service::Name("rmiaux")),
        (ServiceQuery(590, Protocol::TCP), Service::Name("tns-cml")),
        (ServiceQuery(551, Protocol::UDP), Service::Name("cybercash")),
        (ServiceQuery(2452, Protocol::UDP), Service::Name("snifferclient")),
        (ServiceQuery(6268, Protocol::TCP), Service::Name("grid")),
        (ServiceQuery(833, Protocol::TCP), Service::Name("netconfsoapbeep")),
        (ServiceQuery(695, Protocol::TCP), Service::Name("ieee-mms-ssl")),
        (ServiceQuery(552, Protocol::UDP), Service::Name("deviceshare")),
        (ServiceQuery(3756, Protocol::UDP), Service::Name("canon-capt")),
        (ServiceQuery(3469, Protocol::UDP), Service::Name("pluribus")),
        (ServiceQuery(8090, Protocol::TCP), Service::Name("opsmessaging")),
        (ServiceQuery(7544, Protocol::UDP), Service::Name("nta-ds")),
        (ServiceQuery(27000, Protocol::UDP), Service::Name("flex-lm")),
        (ServiceQuery(2111, Protocol::UDP), Service::Name("dsatp")),
        (ServiceQuery(10081, Protocol::UDP), Service::Name("famdc")),
        (ServiceQuery(672, Protocol::UDP), Service::Name("vpps-qua")),
        (ServiceQuery(7474, Protocol::TCP), Service::Name("neo4j")),
        (ServiceQuery(22347, Protocol::UDP), Service::Name("WibuKey")),
        (ServiceQuery(9402, Protocol::UDP), Service::Name("sec-pc2fax-srv")),
        (ServiceQuery(2225, Protocol::TCP), Service::Name("rcip-itu")),
        (ServiceQuery(2713, Protocol::UDP), Service::Name("raventbs")),
        (ServiceQuery(17, Protocol::UDP), Service::Name("qotd")),
        (ServiceQuery(8894, Protocol::UDP), Service::Name("ddi-udp-7")),
        (ServiceQuery(619, Protocol::TCP), Service::Name("compaq-evm")),
        (ServiceQuery(2216, Protocol::UDP), Service::Name("vtu-comms")),
        (ServiceQuery(2458, Protocol::UDP), Service::Name("griffin")),
        (ServiceQuery(24850, Protocol::TCP), Service::Name("assoc-disc")),
        (ServiceQuery(3535, Protocol::UDP), Service::Name("ms-la")),
        (ServiceQuery(1871, Protocol::UDP), Service::Name("canocentral0")),
        (ServiceQuery(22370, Protocol::TCP), Service::Name("hpnpd")),
        (ServiceQuery(4022, Protocol::TCP), Service::Name("dnox")),
        (ServiceQuery(9994, Protocol::UDP), Service::Name("palace-3")),
        (ServiceQuery(8809, Protocol::TCP), Service::Name("3gpp-monp")),
        (ServiceQuery(2699, Protocol::UDP), Service::Name("csoft-plusclnt")),
        (ServiceQuery(2279, Protocol::TCP), Service::Name("xmquery")),
        (ServiceQuery(4233, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(7352, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(27782, Protocol::UDP), Service::Name("ars-vista")),
        (ServiceQuery(4290, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1503, Protocol::UDP), Service::Name("imtc-mcs")),
        (ServiceQuery(5399, Protocol::UDP), Service::Name("securitychase")),
        (ServiceQuery(4700, Protocol::UDP), Service::Name("netxms-agent")),
        (ServiceQuery(2391, Protocol::TCP), Service::Name("3com-net-mgmt")),
        (ServiceQuery(2667, Protocol::UDP), Service::Name("alarm-clock-s")),
        (ServiceQuery(3267, Protocol::UDP), Service::Name("ibm-dial-out")),
        (ServiceQuery(1508, Protocol::TCP), Service::Name("diagmond")),
        (ServiceQuery(5011, Protocol::UDP), Service::Name("telelpathattack")),
        (ServiceQuery(24754, Protocol::TCP), Service::Name("cslg")),
        (ServiceQuery(18769, Protocol::UDP), Service::Name("ique")),
        (ServiceQuery(2729, Protocol::UDP), Service::Name("tcim-control")),
        (ServiceQuery(3537, Protocol::UDP), Service::Name("ni-visa-remote")),
        (ServiceQuery(3487, Protocol::UDP), Service::Name("ltcudp")),
        (ServiceQuery(7464, Protocol::TCP), Service::Name("pythonds")),
        (ServiceQuery(514, Protocol::UDP), Service::Name("syslog")),
        (ServiceQuery(3337, Protocol::TCP), Service::Name("directv-catlg")),
        (ServiceQuery(4099, Protocol::UDP), Service::Name("dpcp")),
        (ServiceQuery(4077, Protocol::UDP), Service::Name("ascomalarm")),
        (ServiceQuery(575, Protocol::TCP), Service::Name("vemmi")),
        (ServiceQuery(2245, Protocol::UDP), Service::Name("hao")),
        (ServiceQuery(5502, Protocol::UDP), Service::Name("fcp-srvr-inst1")),
        (ServiceQuery(8901, Protocol::UDP), Service::Name("jmb-cds2")),
        (ServiceQuery(2262, Protocol::TCP), Service::Name("comotionback")),
        (ServiceQuery(3061, Protocol::TCP), Service::Name("cautcpd")),
        (ServiceQuery(2839, Protocol::TCP), Service::Name("nmsigport")),
        (ServiceQuery(1136, Protocol::TCP), Service::Name("hhb-gateway")),
        (ServiceQuery(4357, Protocol::UDP), Service::Name("qsnet-cond")),
        (ServiceQuery(6057, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(122, Protocol::TCP), Service::Name("smakynet")),
        (ServiceQuery(6002, Protocol::UDP), Service::Name("X11:2")),
        (ServiceQuery(1923, Protocol::TCP), Service::Name("spice")),
        (ServiceQuery(7674, Protocol::UDP), Service::Name("imqtunnels")),
        (ServiceQuery(3965, Protocol::UDP), Service::Name("ati-ip-to-ncpe")),
        (ServiceQuery(665, Protocol::TCP), Service::Name("sun-dr")),
        (ServiceQuery(658, Protocol::UDP), Service::Name("tenfold")),
        (ServiceQuery(431, Protocol::UDP), Service::Name("utmpcd")),
        (ServiceQuery(5636, Protocol::TCP), Service::Name("sfm-db-server")),
        (ServiceQuery(8778, Protocol::TCP), Service::Name("uec")),
        (ServiceQuery(3249, Protocol::UDP), Service::Name("ssp")),
        (ServiceQuery(3353, Protocol::UDP), Service::Name("fatpipe")),
        (ServiceQuery(3885, Protocol::UDP), Service::Name("topflow-ssl")),
        (ServiceQuery(2514, Protocol::UDP), Service::Name("facsys-ntp")),
        (ServiceQuery(2506, Protocol::UDP), Service::Name("jbroker")),
        (ServiceQuery(25001, Protocol::TCP), Service::Name("icl-twobase2")),
        (ServiceQuery(5308, Protocol::TCP), Service::Name("cfengine")),
        (ServiceQuery(8766, Protocol::TCP), Service::Name("amcs")),
        (ServiceQuery(1104, Protocol::UDP), Service::Name("xrl")),
        (ServiceQuery(1993, Protocol::TCP), Service::Name("snmp-tcp-port")),
        (ServiceQuery(2937, Protocol::UDP), Service::Name("pnaconsult-lm")),
        (ServiceQuery(8266, Protocol::TCP), Service::Name("espeasy-p2p")),
        (ServiceQuery(774, Protocol::TCP), Service::Name("rpasswd")),
        (ServiceQuery(3957, Protocol::TCP), Service::Name("mqe-broker")),
        (ServiceQuery(10540, Protocol::UDP), Service::Name("MOS-lower")),
        (ServiceQuery(2666, Protocol::TCP), Service::Name("extensis")),
        (ServiceQuery(3262, Protocol::UDP), Service::Name("necp")),
        (ServiceQuery(70, Protocol::UDP), Service::Name("gopher")),
        (ServiceQuery(2869, Protocol::TCP), Service::Name("icslap")),
        (ServiceQuery(1341, Protocol::UDP), Service::Name("qubes")),
        (ServiceQuery(2444, Protocol::UDP), Service::Name("btpp2sectrans")),
        (ServiceQuery(902, Protocol::UDP), Service::Name("ideafarm-door")),
        (ServiceQuery(17756, Protocol::TCP), Service::Name("zigbee-ips")),
        (ServiceQuery(3077, Protocol::UDP), Service::Name("orbix-loc-ssl")),
        (ServiceQuery(6083, Protocol::TCP), Service::Name("miami-bcast")),
        (ServiceQuery(3810, Protocol::TCP), Service::Name("wlanauth")),
        (ServiceQuery(4296, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3496, Protocol::UDP), Service::Name("seclayer-tls")),
        (ServiceQuery(7724, Protocol::UDP), Service::Name("nsdeepfreezectl")),
        (ServiceQuery(3268, Protocol::UDP), Service::Name("msft-gc")),
        (ServiceQuery(1368, Protocol::UDP), Service::Name("screencast")),
        (ServiceQuery(6086, Protocol::UDP), Service::Name("pdtp")),
        (ServiceQuery(1432, Protocol::UDP), Service::Name("blueberry-lm")),
        (ServiceQuery(516, Protocol::TCP), Service::Name("videotex")),
        (ServiceQuery(3486, Protocol::UDP), Service::Name("ifsf-hb-port")),
        (ServiceQuery(246, Protocol::TCP), Service::Name("dsp3270")),
        (ServiceQuery(4167, Protocol::TCP), Service::Name("ddgn")),
        (ServiceQuery(3283, Protocol::TCP), Service::Name("netassistant")),
        (ServiceQuery(3337, Protocol::UDP), Service::Name("directv-catlg")),
        (ServiceQuery(3041, Protocol::UDP), Service::Name("di-traceware")),
        (ServiceQuery(2293, Protocol::TCP), Service::Name("npdbgmngr")),
        (ServiceQuery(6145, Protocol::UDP), Service::Name("statsci2-lm")),
        (ServiceQuery(7676, Protocol::UDP), Service::Name("imqbrokerd")),
        (ServiceQuery(1474, Protocol::TCP), Service::Name("telefinder")),
        (ServiceQuery(3481, Protocol::TCP), Service::Name("cleanerliverc")),
        (ServiceQuery(3953, Protocol::UDP), Service::Name("xmlink-connect")),
        (ServiceQuery(3824, Protocol::TCP), Service::Name("acp-policy")),
        (ServiceQuery(4299, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(7777, Protocol::TCP), Service::Name("cbt")),
        (ServiceQuery(371, Protocol::TCP), Service::Name("clearcase")),
        (ServiceQuery(2033, Protocol::UDP), Service::Name("glogger")),
        (ServiceQuery(3, Protocol::TCP), Service::Name("compressnet")),
        (ServiceQuery(24003, Protocol::TCP), Service::Name("med-supp")),
        (ServiceQuery(71, Protocol::UDP), Service::Name("netrjs-1")),
        (ServiceQuery(9293, Protocol::UDP), Service::Name("storview")),
        (ServiceQuery(1252, Protocol::UDP), Service::Name("bspne-pcc")),
        (ServiceQuery(36411, Protocol::TCP), Service::Name("wlcp")),
        (ServiceQuery(1431, Protocol::UDP), Service::Name("rgtp")),
        (ServiceQuery(2107, Protocol::TCP), Service::Name("msmq-mgmt")),
        (ServiceQuery(3270, Protocol::TCP), Service::Name("verismart")),
        (ServiceQuery(2490, Protocol::UDP), Service::Name("qip-qdhcp")),
        (ServiceQuery(16384, Protocol::TCP), Service::Name("connected")),
        (ServiceQuery(1484, Protocol::UDP), Service::Name("confluent")),
        (ServiceQuery(8116, Protocol::TCP), Service::Name("cp-cluster")),
        (ServiceQuery(1984, Protocol::TCP), Service::Name("bigbrother")),
        (ServiceQuery(483, Protocol::UDP), Service::Name("ulpnet")),
        (ServiceQuery(2407, Protocol::TCP), Service::Name("orion")),
        (ServiceQuery(145, Protocol::UDP), Service::Name("uaac")),
        (ServiceQuery(3533, Protocol::TCP), Service::Name("raven-rdp")),
        (ServiceQuery(4788, Protocol::TCP), Service::Name("xmcp")),
        (ServiceQuery(6012, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(5985, Protocol::UDP), Service::Name("wsman")),
        (ServiceQuery(1279, Protocol::TCP), Service::Name("dellwebadmin-2")),
        (ServiceQuery(32636, Protocol::TCP), Service::Name("DMExpress")),
        (ServiceQuery(2247, Protocol::TCP), Service::Name("antidotemgrsvr")),
        (ServiceQuery(5715, Protocol::TCP), Service::Name("prosharedata")),
        (ServiceQuery(1264, Protocol::UDP), Service::Name("prat")),
        (ServiceQuery(1053, Protocol::UDP), Service::Name("remote-as")),
        (ServiceQuery(1393, Protocol::TCP), Service::Name("iclpv-nls")),
        (ServiceQuery(15555, Protocol::UDP), Service::Name("cisco-snat")),
        (ServiceQuery(48003, Protocol::UDP), Service::Name("nimgtw")),
        (ServiceQuery(441, Protocol::TCP), Service::Name("decvms-sysmgt")),
        (ServiceQuery(7009, Protocol::TCP), Service::Name("afs3-rmtsys")),
        (ServiceQuery(165, Protocol::UDP), Service::Name("xns-courier")),
        (ServiceQuery(2599, Protocol::TCP), Service::Name("snapd")),
        (ServiceQuery(2442, Protocol::UDP), Service::Name("netangel")),
        (ServiceQuery(1469, Protocol::UDP), Service::Name("aal-lm")),
        (ServiceQuery(473, Protocol::UDP), Service::Name("hybrid-pop")),
        (ServiceQuery(4017, Protocol::TCP), Service::Name("talarian-mcast3")),
        (ServiceQuery(611, Protocol::UDP), Service::Name("npmp-gui")),
        (ServiceQuery(9285, Protocol::UDP), Service::Name("n2h2server")),
        (ServiceQuery(498, Protocol::TCP), Service::Name("siam")),
        (ServiceQuery(494, Protocol::TCP), Service::Name("pov-ray")),
        (ServiceQuery(1846, Protocol::UDP), Service::Name("tunstall-pnc")),
        (ServiceQuery(48005, Protocol::TCP), Service::Name("nimbusdbctrl")),
        (ServiceQuery(4665, Protocol::TCP), Service::Name("contclientms")),
        (ServiceQuery(9287, Protocol::UDP), Service::Name("cumulus")),
        (ServiceQuery(1765, Protocol::TCP), Service::Name("cft-4")),
        (ServiceQuery(2835, Protocol::TCP), Service::Name("evtp-data")),
        (ServiceQuery(486, Protocol::TCP), Service::Name("sstats")),
        (ServiceQuery(457, Protocol::UDP), Service::Name("scohelp")),
        (ServiceQuery(3603, Protocol::TCP), Service::Name("int-rcv-cntrl")),
        (ServiceQuery(2069, Protocol::UDP), Service::Name("event-port")),
        (ServiceQuery(674, Protocol::TCP), Service::Name("acap")),
        (ServiceQuery(1757, Protocol::UDP), Service::Name("cnhrp")),
        (ServiceQuery(17221, Protocol::TCP), Service::Name("avdecc")),
        (ServiceQuery(9987, Protocol::TCP), Service::Name("dsm-scm-target")),
        (ServiceQuery(2895, Protocol::UDP), Service::Name("natuslink")),
        (ServiceQuery(4552, Protocol::UDP), Service::Name("menandmice-mon")),
        (ServiceQuery(3661, Protocol::TCP), Service::Name("can-ferret-ssl")),
        (ServiceQuery(3755, Protocol::UDP), Service::Name("sas-remote-hlp")),
        (ServiceQuery(4141, Protocol::TCP), Service::Name("oirtgsvc")),
        (ServiceQuery(470, Protocol::UDP), Service::Name("scx-proxy")),
        (ServiceQuery(5747, Protocol::UDP), Service::Name("tunatic")),
        (ServiceQuery(5717, Protocol::UDP), Service::Name("prosharenotify")),
        (ServiceQuery(2081, Protocol::UDP), Service::Name("kme-trap-port")),
        (ServiceQuery(8083, Protocol::UDP), Service::Name("us-srv")),
        (ServiceQuery(1744, Protocol::TCP), Service::Name("ncpm-ft")),
        (ServiceQuery(2169, Protocol::UDP), Service::Name("brain")),
        (ServiceQuery(5599, Protocol::TCP), Service::Name("esinstall")),
        (ServiceQuery(420, Protocol::UDP), Service::Name("smpte")),
        (ServiceQuery(6016, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(429, Protocol::TCP), Service::Name("ocs_amu")),
        (ServiceQuery(18516, Protocol::TCP), Service::Name("heythings")),
        (ServiceQuery(481, Protocol::TCP), Service::Name("dvs")),
        (ServiceQuery(43440, Protocol::TCP), Service::Name("ew-mgmt")),
        (ServiceQuery(7326, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(2005, Protocol::UDP), Service::Name("oracle")),
        (ServiceQuery(2224, Protocol::UDP), Service::Name("efi-mg")),
        (ServiceQuery(7674, Protocol::TCP), Service::Name("imqtunnels")),
        (ServiceQuery(3613, Protocol::TCP), Service::Name("alaris-disc")),
        (ServiceQuery(7628, Protocol::TCP), Service::Name("zen-pawn")),
        (ServiceQuery(7324, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(3882, Protocol::TCP), Service::Name("msdts1")),
        (ServiceQuery(4234, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(5092, Protocol::UDP), Service::Name("magpie")),
        (ServiceQuery(4486, Protocol::TCP), Service::Name("icms")),
        (ServiceQuery(3390, Protocol::TCP), Service::Name("dsc")),
        (ServiceQuery(7304, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(1764, Protocol::TCP), Service::Name("landesk-rc")),
        (ServiceQuery(2342, Protocol::TCP), Service::Name("manage-exec")),
        (ServiceQuery(6036, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(1511, Protocol::TCP), Service::Name("3l-l1")),
        (ServiceQuery(1118, Protocol::UDP), Service::Name("sacred")),
        (ServiceQuery(4486, Protocol::UDP), Service::Name("icms")),
        (ServiceQuery(142, Protocol::UDP), Service::Name("bl-idm")),
        (ServiceQuery(2879, Protocol::TCP), Service::Name("ucentric-ds")),
        (ServiceQuery(1213, Protocol::TCP), Service::Name("mpc-lifenet")),
        (ServiceQuery(4331, Protocol::TCP), Service::Name("ktickets-rest")),
        (ServiceQuery(70, Protocol::TCP), Service::Name("gopher")),
        (ServiceQuery(20202, Protocol::TCP), Service::Name("ipdtp-port")),
        (ServiceQuery(4418, Protocol::TCP), Service::Name("axysbridge")),
        (ServiceQuery(4093, Protocol::TCP), Service::Name("pvxpluscs")),
        (ServiceQuery(4089, Protocol::UDP), Service::Name("opencore")),
        (ServiceQuery(5025, Protocol::TCP), Service::Name("scpi-raw")),
        (ServiceQuery(7800, Protocol::TCP), Service::Name("asr")),
        (ServiceQuery(4104, Protocol::TCP), Service::Name("brlp-3")),
        (ServiceQuery(13832, Protocol::TCP), Service::Name("a-trust-rpc")),
        (ServiceQuery(3545, Protocol::UDP), Service::Name("camac")),
        (ServiceQuery(4538, Protocol::UDP), Service::Name("swx-gate")),
        (ServiceQuery(2982, Protocol::TCP), Service::Name("iwb-whiteboard")),
        (ServiceQuery(3171, Protocol::UDP), Service::Name("serverview-gf")),
        (ServiceQuery(3331, Protocol::UDP), Service::Name("mcs-messaging")),
        (ServiceQuery(1459, Protocol::UDP), Service::Name("proshare1")),
        (ServiceQuery(1970, Protocol::TCP), Service::Name("netop-rc")),
        (ServiceQuery(4894, Protocol::TCP), Service::Name("lyskom")),
        (ServiceQuery(7332, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(579, Protocol::TCP), Service::Name("decbsrv")),
        (ServiceQuery(4287, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(30260, Protocol::TCP), Service::Name("kingdomsonline")),
        (ServiceQuery(1990, Protocol::UDP), Service::Name("stun-p1")),
        (ServiceQuery(3385, Protocol::UDP), Service::Name("qnxnetman")),
        (ServiceQuery(5501, Protocol::UDP), Service::Name("fcp-addr-srvr2")),
        (ServiceQuery(9993, Protocol::UDP), Service::Name("palace-2")),
        (ServiceQuery(7627, Protocol::UDP), Service::Name("soap-http")),
        (ServiceQuery(81, Protocol::TCP), Service::Name("hosts2-ns")),
        (ServiceQuery(3195, Protocol::UDP), Service::Name("ncu-1")),
        (ServiceQuery(2464, Protocol::UDP), Service::Name("direcpc-si")),
        (ServiceQuery(1539, Protocol::TCP), Service::Name("intellistor-lm")),
        (ServiceQuery(1394, Protocol::UDP), Service::Name("iclpv-nlc")),
        (ServiceQuery(1455, Protocol::TCP), Service::Name("esl-lm")),
        (ServiceQuery(1411, Protocol::TCP), Service::Name("af")),
        (ServiceQuery(4165, Protocol::TCP), Service::Name("altcp")),
        (ServiceQuery(26, Protocol::TCP), Service::Name("rsftp")),
        (ServiceQuery(4046, Protocol::UDP), Service::Name("acp-proto")),
        (ServiceQuery(3642, Protocol::UDP), Service::Name("juxml-port")),
        (ServiceQuery(16950, Protocol::UDP), Service::Name("sgcip")),
        (ServiceQuery(661, Protocol::TCP), Service::Name("hap")),
        (ServiceQuery(3719, Protocol::TCP), Service::Name("itelserverport")),
        (ServiceQuery(4749, Protocol::TCP), Service::Name("profilemac")),
        (ServiceQuery(66, Protocol::UDP), Service::Name("sqlnet")),
        (ServiceQuery(4238, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(503, Protocol::TCP), Service::Name("intrinsa")),
        (ServiceQuery(873, Protocol::TCP), Service::Name("rsync")),
        (ServiceQuery(3014, Protocol::UDP), Service::Name("broker_service")),
        (ServiceQuery(4417, Protocol::TCP), Service::Name("workflowdir")),
        (ServiceQuery(5618, Protocol::TCP), Service::Name("efr")),
        (ServiceQuery(1135, Protocol::UDP), Service::Name("omnivision")),
        (ServiceQuery(1691, Protocol::UDP), Service::Name("empire-empuma")),
        (ServiceQuery(40842, Protocol::TCP), Service::Name("csccredir")),
        (ServiceQuery(2441, Protocol::UDP), Service::Name("pvsw-inet")),
        (ServiceQuery(3566, Protocol::TCP), Service::Name("quest-data-hub")),
        (ServiceQuery(1083, Protocol::TCP), Service::Name("ansoft-lm-1")),
        (ServiceQuery(1911, Protocol::TCP), Service::Name("mtp")),
        (ServiceQuery(3842, Protocol::UDP), Service::Name("nhci")),
        (ServiceQuery(2383, Protocol::UDP), Service::Name("ms-olap4")),
        (ServiceQuery(7478, Protocol::TCP), Service::Name("openit")),
        (ServiceQuery(1960, Protocol::TCP), Service::Name("nasmanager")),
        (ServiceQuery(2992, Protocol::TCP), Service::Name("avenyo")),
        (ServiceQuery(2569, Protocol::TCP), Service::Name("sonuscallsig")),
        (ServiceQuery(4074, Protocol::UDP), Service::Name("cequint-cityid")),
        (ServiceQuery(2726, Protocol::UDP), Service::Name("tams")),
        (ServiceQuery(1757, Protocol::TCP), Service::Name("cnhrp")),
        (ServiceQuery(2011, Protocol::UDP), Service::Name("servserv")),
        (ServiceQuery(3638, Protocol::TCP), Service::Name("ehp-backup")),
        (ServiceQuery(1888, Protocol::TCP), Service::Name("ncconfig")),
        (ServiceQuery(4113, Protocol::TCP), Service::Name("aipn-reg")),
        (ServiceQuery(2240, Protocol::UDP), Service::Name("recipe")),
        (ServiceQuery(1746, Protocol::TCP), Service::Name("ftrapid-1")),
        (ServiceQuery(3777, Protocol::TCP), Service::Name("jibe-eb")),
        (ServiceQuery(1349, Protocol::TCP), Service::Name("sbook")),
        (ServiceQuery(656, Protocol::TCP), Service::Name("spmp")),
        (ServiceQuery(5579, Protocol::TCP), Service::Name("fdtracks")),
        (ServiceQuery(24680, Protocol::TCP), Service::Name("tcc-http")),
        (ServiceQuery(19020, Protocol::TCP), Service::Name("j-link")),
        (ServiceQuery(2349, Protocol::UDP), Service::Name("redstorm_diag")),
        (ServiceQuery(3001, Protocol::TCP), Service::Name("nessus")),
        (ServiceQuery(2702, Protocol::UDP), Service::Name("sms-xfer")),
        (ServiceQuery(86, Protocol::UDP), Service::Name("mfcobol")),
        (ServiceQuery(1160, Protocol::UDP), Service::Name("olsv")),
        (ServiceQuery(2859, Protocol::TCP), Service::Name("activememory")),
        (ServiceQuery(98, Protocol::TCP), Service::Name("linuxconf")),
        (ServiceQuery(3226, Protocol::UDP), Service::Name("isi-irp")),
        (ServiceQuery(1021, Protocol::UDP), Service::Name("exp1")),
        (ServiceQuery(4713, Protocol::TCP), Service::Name("pulseaudio")),
        (ServiceQuery(5729, Protocol::UDP), Service::Name("openmail")),
        (ServiceQuery(626, Protocol::UDP), Service::Name("serialnumberd")),
        (ServiceQuery(1244, Protocol::TCP), Service::Name("isbconference1")),
        (ServiceQuery(4454, Protocol::TCP), Service::Name("nssagentmgr")),
        (ServiceQuery(1697, Protocol::UDP), Service::Name("rrisat")),
        (ServiceQuery(5456, Protocol::UDP), Service::Name("apc-5456")),
        (ServiceQuery(4067, Protocol::TCP), Service::Name("idp")),
        (ServiceQuery(9597, Protocol::UDP), Service::Name("pd-admin")),
        (ServiceQuery(1836, Protocol::TCP), Service::Name("ste-smsc")),
        (ServiceQuery(2935, Protocol::UDP), Service::Name("qtp")),
        (ServiceQuery(1472, Protocol::TCP), Service::Name("csdm")),
        (ServiceQuery(6050, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(7569, Protocol::TCP), Service::Name("dell-eql-asm")),
        (ServiceQuery(2223, Protocol::TCP), Service::Name("rockwell-csp2")),
        (ServiceQuery(2024, Protocol::TCP), Service::Name("xinuexpansion4")),
        (ServiceQuery(3767, Protocol::TCP), Service::Name("listmgr-port")),
        (ServiceQuery(6901, Protocol::TCP), Service::Name("jetstream")),
        (ServiceQuery(4566, Protocol::TCP), Service::Name("kwtc")),
        (ServiceQuery(2581, Protocol::UDP), Service::Name("argis-te")),
        (ServiceQuery(5413, Protocol::UDP), Service::Name("wwiotalk")),
        (ServiceQuery(6801, Protocol::UDP), Service::Name("acnet")),
        (ServiceQuery(6064, Protocol::TCP), Service::Name("ndl-ahp-svc")),
        (ServiceQuery(3016, Protocol::TCP), Service::Name("notify_srvr")),
        (ServiceQuery(4264, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(5454, Protocol::TCP), Service::Name("apc-5454")),
        (ServiceQuery(379, Protocol::UDP), Service::Name("is99c")),
        (ServiceQuery(4016, Protocol::UDP), Service::Name("talarian-mcast2")),
        (ServiceQuery(6000, Protocol::UDP), Service::Name("X11")),
        (ServiceQuery(10103, Protocol::UDP), Service::Name("ezrelay")),
        (ServiceQuery(1259, Protocol::TCP), Service::Name("opennl-voice")),
        (ServiceQuery(7318, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(5363, Protocol::UDP), Service::Name("net-projection")),
        (ServiceQuery(3734, Protocol::TCP), Service::Name("synel-data")),
        (ServiceQuery(2251, Protocol::UDP), Service::Name("dif-port")),
        (ServiceQuery(2658, Protocol::TCP), Service::Name("sns-admin")),
        (ServiceQuery(1165, Protocol::UDP), Service::Name("qsm-gui")),
        (ServiceQuery(2813, Protocol::UDP), Service::Name("llm-pass")),
        (ServiceQuery(2700, Protocol::TCP), Service::Name("tqdata")),
        (ServiceQuery(2144, Protocol::UDP), Service::Name("lv-ffx")),
        (ServiceQuery(5423, Protocol::TCP), Service::Name("virtualuser")),
        (ServiceQuery(1446, Protocol::TCP), Service::Name("ora-lm")),
        (ServiceQuery(129, Protocol::UDP), Service::Name("pwdgen")),
        (ServiceQuery(2623, Protocol::TCP), Service::Name("lmdp")),
        (ServiceQuery(3165, Protocol::UDP), Service::Name("newgenpay")),
        (ServiceQuery(7328, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(9204, Protocol::TCP), Service::Name("wap-vcard")),
        (ServiceQuery(2802, Protocol::UDP), Service::Name("veritas-udp1")),
        (ServiceQuery(7365, Protocol::UDP), Service::Name("lcm-server")),
        (ServiceQuery(1205, Protocol::TCP), Service::Name("accord-mgc")),
        (ServiceQuery(27005, Protocol::TCP), Service::Name("flexlm5")),
        (ServiceQuery(4292, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(27442, Protocol::UDP), Service::Name("jesmsjc")),
        (ServiceQuery(3141, Protocol::TCP), Service::Name("vmodem")),
        (ServiceQuery(2915, Protocol::TCP), Service::Name("tksocket")),
        (ServiceQuery(12172, Protocol::UDP), Service::Name("hivep")),
        (ServiceQuery(6045, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(1889, Protocol::TCP), Service::Name("unify-adapter")),
        (ServiceQuery(1193, Protocol::TCP), Service::Name("fiveacross")),
        (ServiceQuery(3398, Protocol::TCP), Service::Name("sapcomm")),
        (ServiceQuery(3843, Protocol::UDP), Service::Name("quest-agent")),
        (ServiceQuery(17235, Protocol::TCP), Service::Name("ssh-mgmt")),
        (ServiceQuery(2917, Protocol::TCP), Service::Name("elvin_client")),
        (ServiceQuery(1821, Protocol::TCP), Service::Name("donnyworld")),
        (ServiceQuery(9222, Protocol::UDP), Service::Name("teamcoherence")),
        (ServiceQuery(9104, Protocol::TCP), Service::Name("jetdirect")),
        (ServiceQuery(2999, Protocol::TCP), Service::Name("remoteware-un")),
        (ServiceQuery(4942, Protocol::TCP), Service::Name("eq-office-4942")),
        (ServiceQuery(914, Protocol::TCP), Service::Name("rift-lies")),
        (ServiceQuery(635, Protocol::TCP), Service::Name("rlzdbase")),
        (ServiceQuery(2230, Protocol::UDP), Service::Name("queueadm")),
        (ServiceQuery(1996, Protocol::UDP), Service::Name("tr-rsrb-port")),
        (ServiceQuery(2971, Protocol::TCP), Service::Name("netclip")),
        (ServiceQuery(10007, Protocol::TCP), Service::Name("mvs-capacity")),
        (ServiceQuery(2198, Protocol::UDP), Service::Name("onehome-remote")),
        (ServiceQuery(33333, Protocol::TCP), Service::Name("dgi-serv")),
        (ServiceQuery(2453, Protocol::TCP), Service::Name("madge-ltd")),
        (ServiceQuery(16161, Protocol::TCP), Service::Name("sun-sea-port")),
        (ServiceQuery(615, Protocol::UDP), Service::Name("sco-inetmgr")),
        (ServiceQuery(4450, Protocol::TCP), Service::Name("camp")),
        (ServiceQuery(1403, Protocol::TCP), Service::Name("prm-nm-np")),
        (ServiceQuery(3447, Protocol::UDP), Service::Name("directnet")),
        (ServiceQuery(1617, Protocol::UDP), Service::Name("nimrod-agent")),
        (ServiceQuery(6841, Protocol::TCP), Service::Name("netmo-default")),
        (ServiceQuery(3564, Protocol::TCP), Service::Name("esimport")),
        (ServiceQuery(4184, Protocol::UDP), Service::Name("universe_suite")),
        (ServiceQuery(2737, Protocol::UDP), Service::Name("srp-feedback")),
        (ServiceQuery(3797, Protocol::TCP), Service::Name("idps")),
        (ServiceQuery(4745, Protocol::TCP), Service::Name("fmp")),
        (ServiceQuery(1010, Protocol::UDP), Service::Name("surf")),
        (ServiceQuery(23400, Protocol::TCP), Service::Name("novar-dbase")),
        (ServiceQuery(3985, Protocol::UDP), Service::Name("mapper-mapethd")),
        (ServiceQuery(2429, Protocol::TCP), Service::Name("ft-role")),
        (ServiceQuery(1352, Protocol::TCP), Service::Name("lotusnotes")),
        (ServiceQuery(4071, Protocol::UDP), Service::Name("aibkup")),
        (ServiceQuery(379, Protocol::TCP), Service::Name("is99c")),
        (ServiceQuery(1765, Protocol::UDP), Service::Name("cft-4")),
        (ServiceQuery(2801, Protocol::TCP), Service::Name("igcp")),
        (ServiceQuery(7999, Protocol::TCP), Service::Name("irdmi2")),
        (ServiceQuery(6390, Protocol::UDP), Service::Name("metaedit-ws")),
        (ServiceQuery(4149, Protocol::TCP), Service::Name("agslb")),
        (ServiceQuery(316, Protocol::TCP), Service::Name("decauth")),
        (ServiceQuery(9956, Protocol::TCP), Service::Name("alljoyn")),
        (ServiceQuery(1412, Protocol::UDP), Service::Name("innosys")),
        (ServiceQuery(2868, Protocol::TCP), Service::Name("npep-messaging")),
        (ServiceQuery(109, Protocol::TCP), Service::Name("pop2")),
        (ServiceQuery(1414, Protocol::UDP), Service::Name("ibm-mqseries")),
        (ServiceQuery(3224, Protocol::TCP), Service::Name("aes-discovery")),
        (ServiceQuery(1362, Protocol::TCP), Service::Name("timeflies")),
        (ServiceQuery(8160, Protocol::UDP), Service::Name("patrol")),
        (ServiceQuery(3502, Protocol::TCP), Service::Name("avinstalldisc")),
        (ServiceQuery(712, Protocol::UDP), Service::Name("tbrpf")),
        (ServiceQuery(1722, Protocol::TCP), Service::Name("hks-lm")),
        (ServiceQuery(3513, Protocol::TCP), Service::Name("arcpd")),
        (ServiceQuery(6050, Protocol::TCP), Service::Name("arcserve")),
        (ServiceQuery(2183, Protocol::TCP), Service::Name("cgn-config")),
        (ServiceQuery(3172, Protocol::UDP), Service::Name("serverview-rm")),
        (ServiceQuery(1851, Protocol::TCP), Service::Name("ctcd")),
        (ServiceQuery(3191, Protocol::UDP), Service::Name("csvr-sslproxy")),
        (ServiceQuery(2836, Protocol::UDP), Service::Name("catalyst")),
        (ServiceQuery(5421, Protocol::UDP), Service::Name("netsupport2")),
        (ServiceQuery(3117, Protocol::UDP), Service::Name("mctet-jserv")),
        (ServiceQuery(10631, Protocol::TCP), Service::Name("printopia")),
        (ServiceQuery(131, Protocol::UDP), Service::Name("cisco-tna")),
        (ServiceQuery(16361, Protocol::TCP), Service::Name("netserialext2")),
        (ServiceQuery(11104, Protocol::TCP), Service::Name("netapp-icmgmt")),
        (ServiceQuery(3105, Protocol::UDP), Service::Name("cardbox")),
        (ServiceQuery(3230, Protocol::UDP), Service::Name("sftdst-port")),
        (ServiceQuery(2804, Protocol::UDP), Service::Name("dvr-esm")),
        (ServiceQuery(6787, Protocol::TCP), Service::Name("smc-admin")),
        (ServiceQuery(43438, Protocol::TCP), Service::Name("hmip-routing")),
        (ServiceQuery(4988, Protocol::UDP), Service::Name("smar-se-port2")),
        (ServiceQuery(5172, Protocol::TCP), Service::Name("pcoip-mgmt")),
        (ServiceQuery(5820, Protocol::TCP), Service::Name("autopassdaemon")),
        (ServiceQuery(4672, Protocol::UDP), Service::Name("rfa")),
        (ServiceQuery(9592, Protocol::UDP), Service::Name("ldgateway")),
        (ServiceQuery(2027, Protocol::TCP), Service::Name("shadowserver")),
        (ServiceQuery(4165, Protocol::UDP), Service::Name("altcp")),
        (ServiceQuery(2588, Protocol::UDP), Service::Name("privilege")),
        (ServiceQuery(1466, Protocol::TCP), Service::Name("oceansoft-lm")),
        (ServiceQuery(3442, Protocol::UDP), Service::Name("connect-server")),
        (ServiceQuery(19, Protocol::UDP), Service::Name("chargen")),
        (ServiceQuery(6900, Protocol::TCP), Service::Name("rtimeviewer")),
        (ServiceQuery(3614, Protocol::UDP), Service::Name("sigma-port")),
        (ServiceQuery(2152, Protocol::TCP), Service::Name("gtp-user")),
        (ServiceQuery(135, Protocol::TCP), Service::Name("msrpc")),
        (ServiceQuery(525, Protocol::UDP), Service::Name("timed")),
        (ServiceQuery(3677, Protocol::UDP), Service::Name("roverlog")),
        (ServiceQuery(3548, Protocol::UDP), Service::Name("interworld")),
        (ServiceQuery(16665, Protocol::TCP), Service::Name("axon-tunnel")),
        (ServiceQuery(6321, Protocol::TCP), Service::Name("emp-server1")),
        (ServiceQuery(3855, Protocol::TCP), Service::Name("opentrac")),
        (ServiceQuery(6306, Protocol::UDP), Service::Name("ufmp")),
        (ServiceQuery(4118, Protocol::UDP), Service::Name("netscript")),
        (ServiceQuery(2047, Protocol::TCP), Service::Name("dls")),
        (ServiceQuery(3402, Protocol::TCP), Service::Name("fxaengine-net")),
        (ServiceQuery(5061, Protocol::TCP), Service::Name("sip-tls")),
        (ServiceQuery(3424, Protocol::UDP), Service::Name("xtrms")),
        (ServiceQuery(1465, Protocol::TCP), Service::Name("pipes")),
        (ServiceQuery(3478, Protocol::TCP), Service::Name("stun")),
        (ServiceQuery(2639, Protocol::UDP), Service::Name("aminet")),
        (ServiceQuery(3458, Protocol::TCP), Service::Name("d3winosfi")),
        (ServiceQuery(7979, Protocol::UDP), Service::Name("micromuse-ncps")),
        (ServiceQuery(3769, Protocol::UDP), Service::Name("haipe-otnk")),
        (ServiceQuery(3690, Protocol::UDP), Service::Name("svn")),
        (ServiceQuery(51, Protocol::UDP), Service::Name("la-maint")),
        (ServiceQuery(2888, Protocol::UDP), Service::Name("spcsdlobby")),
        (ServiceQuery(10006, Protocol::TCP), Service::Name("netapp-sync")),
        (ServiceQuery(3978, Protocol::UDP), Service::Name("secure-cfg-svr")),
        (ServiceQuery(20046, Protocol::UDP), Service::Name("tmophl7mts")),
        (ServiceQuery(3918, Protocol::UDP), Service::Name("pktcablemmcops")),
        (ServiceQuery(1231, Protocol::TCP), Service::Name("menandmice-lpm")),
        (ServiceQuery(23272, Protocol::UDP), Service::Name("s102")),
        (ServiceQuery(2466, Protocol::TCP), Service::Name("lbf")),
        (ServiceQuery(2779, Protocol::UDP), Service::Name("lbc-sync")),
        (ServiceQuery(1778, Protocol::UDP), Service::Name("prodigy-intrnet")),
        (ServiceQuery(3170, Protocol::UDP), Service::Name("serverview-asn")),
        (ServiceQuery(14001, Protocol::UDP), Service::Name("sua")),
        (ServiceQuery(3528, Protocol::UDP), Service::Name("jboss-iiop")),
        (ServiceQuery(5344, Protocol::UDP), Service::Name("xkotodrcp")),
        (ServiceQuery(7394, Protocol::UDP), Service::Name("fse")),
        (ServiceQuery(2839, Protocol::UDP), Service::Name("nmsigport")),
        (ServiceQuery(7680, Protocol::TCP), Service::Name("pando-pub")),
        (ServiceQuery(395, Protocol::TCP), Service::Name("netcp")),
        (ServiceQuery(1044, Protocol::UDP), Service::Name("dcutility")),
        (ServiceQuery(4847, Protocol::TCP), Service::Name("wfc")),
        (ServiceQuery(990, Protocol::UDP), Service::Name("ftps")),
        (ServiceQuery(4901, Protocol::TCP), Service::Name("flr_agent")),
        (ServiceQuery(3352, Protocol::UDP), Service::Name("ssql")),
        (ServiceQuery(1291, Protocol::TCP), Service::Name("seagulllms")),
        (ServiceQuery(2310, Protocol::TCP), Service::Name("sdclient")),
        (ServiceQuery(2874, Protocol::UDP), Service::Name("dxmessagebase1")),
        (ServiceQuery(2827, Protocol::TCP), Service::Name("slc-ctrlrloops")),
        (ServiceQuery(7331, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(6666, Protocol::TCP), Service::Name("irc")),
        (ServiceQuery(1651, Protocol::UDP), Service::Name("shiva_confsrvr")),
        (ServiceQuery(1714, Protocol::TCP), Service::Name("sesi-lm")),
        (ServiceQuery(5721, Protocol::TCP), Service::Name("dtpt")),
        (ServiceQuery(3035, Protocol::UDP), Service::Name("fjsv-gssagt")),
        (ServiceQuery(1252, Protocol::TCP), Service::Name("bspne-pcc")),
        (ServiceQuery(1939, Protocol::TCP), Service::Name("jvserver")),
        (ServiceQuery(4845, Protocol::TCP), Service::Name("wcr-remlib")),
        (ServiceQuery(5314, Protocol::UDP), Service::Name("opalis-rbt-ipc")),
        (ServiceQuery(1984, Protocol::UDP), Service::Name("bb")),
        (ServiceQuery(3619, Protocol::TCP), Service::Name("aairnet-2")),
        (ServiceQuery(2968, Protocol::TCP), Service::Name("enpp")),
        (ServiceQuery(5727, Protocol::TCP), Service::Name("asgenf")),
        (ServiceQuery(7016, Protocol::TCP), Service::Name("spg")),
        (ServiceQuery(1327, Protocol::TCP), Service::Name("ultrex")),
        (ServiceQuery(3901, Protocol::UDP), Service::Name("nimsh")),
        (ServiceQuery(2866, Protocol::UDP), Service::Name("iwlistener")),
        (ServiceQuery(4330, Protocol::TCP), Service::Name("dey-sapi")),
        (ServiceQuery(2544, Protocol::UDP), Service::Name("novell-zen")),
        (ServiceQuery(3663, Protocol::UDP), Service::Name("dtp")),
        (ServiceQuery(6041, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(4215, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2006, Protocol::UDP), Service::Name("raid-cc")),
        (ServiceQuery(5501, Protocol::TCP), Service::Name("fcp-addr-srvr2")),
        (ServiceQuery(6551, Protocol::TCP), Service::Name("sum")),
        (ServiceQuery(2528, Protocol::UDP), Service::Name("ncr_ccl")),
        (ServiceQuery(1380, Protocol::UDP), Service::Name("telesis-licman")),
        (ServiceQuery(1348, Protocol::UDP), Service::Name("bbn-mmx")),
        (ServiceQuery(8054, Protocol::TCP), Service::Name("senomix03")),
        (ServiceQuery(1709, Protocol::UDP), Service::Name("centra")),
        (ServiceQuery(4129, Protocol::UDP), Service::Name("nuauth")),
        (ServiceQuery(4679, Protocol::UDP), Service::Name("mgesupervision")),
        (ServiceQuery(266, Protocol::TCP), Service::Name("sst")),
        (ServiceQuery(32801, Protocol::UDP), Service::Name("mlsn")),
        (ServiceQuery(1728, Protocol::UDP), Service::Name("telindus")),
        (ServiceQuery(1420, Protocol::TCP), Service::Name("timbuktu-srv4")),
        (ServiceQuery(48129, Protocol::TCP), Service::Name("blp5")),
        (ServiceQuery(3, Protocol::UDP), Service::Name("compressnet")),
        (ServiceQuery(3297, Protocol::TCP), Service::Name("cytel-lm")),
        (ServiceQuery(1906, Protocol::TCP), Service::Name("tpmd")),
        (ServiceQuery(3300, Protocol::TCP), Service::Name("ceph")),
        (ServiceQuery(8376, Protocol::UDP), Service::Name("cruise-enum")),
        (ServiceQuery(2348, Protocol::UDP), Service::Name("redstorm_info")),
        (ServiceQuery(148, Protocol::TCP), Service::Name("cronus")),
        (ServiceQuery(561, Protocol::UDP), Service::Name("monitor")),
        (ServiceQuery(4105, Protocol::UDP), Service::Name("shofarplayer")),
        (ServiceQuery(1468, Protocol::UDP), Service::Name("csdm")),
        (ServiceQuery(385, Protocol::TCP), Service::Name("ibm-app")),
        (ServiceQuery(2474, Protocol::UDP), Service::Name("vitalanalysis")),
        (ServiceQuery(2765, Protocol::UDP), Service::Name("qip-audup")),
        (ServiceQuery(1839, Protocol::TCP), Service::Name("netopia-vo1")),
        (ServiceQuery(243, Protocol::UDP), Service::Name("sur-meas")),
        (ServiceQuery(8384, Protocol::TCP), Service::Name("marathontp")),
        (ServiceQuery(2062, Protocol::TCP), Service::Name("icg-swp")),
        (ServiceQuery(4597, Protocol::TCP), Service::Name("a21-an-1xbs")),
        (ServiceQuery(3428, Protocol::UDP), Service::Name("twcss")),
        (ServiceQuery(2578, Protocol::UDP), Service::Name("rvs-isdn-dcp")),
        (ServiceQuery(752, Protocol::UDP), Service::Name("qrh")),
        (ServiceQuery(8201, Protocol::UDP), Service::Name("trivnet2")),
        (ServiceQuery(1559, Protocol::TCP), Service::Name("web2host")),
        (ServiceQuery(1962, Protocol::UDP), Service::Name("biap-mp")),
        (ServiceQuery(21010, Protocol::TCP), Service::Name("notezilla-lan")),
        (ServiceQuery(3549, Protocol::TCP), Service::Name("tellumat-nms")),
        (ServiceQuery(2203, Protocol::TCP), Service::Name("b2-runtime")),
        (ServiceQuery(2684, Protocol::UDP), Service::Name("mpnjsosv")),
        (ServiceQuery(1168, Protocol::TCP), Service::Name("vchat")),
        (ServiceQuery(2364, Protocol::TCP), Service::Name("oi-2000")),
        (ServiceQuery(3247, Protocol::UDP), Service::Name("dvt-data")),
        (ServiceQuery(4155, Protocol::UDP), Service::Name("bzr")),
        (ServiceQuery(1034, Protocol::UDP), Service::Name("activesync-notify")),
        (ServiceQuery(36700, Protocol::TCP), Service::Name("mapx")),
        (ServiceQuery(7624, Protocol::TCP), Service::Name("indi")),
        (ServiceQuery(1505, Protocol::TCP), Service::Name("funkproxy")),
        (ServiceQuery(32775, Protocol::TCP), Service::Name("sometimes-rpc13")),
        (ServiceQuery(7229, Protocol::TCP), Service::Name("citrixuppg")),
        (ServiceQuery(2939, Protocol::UDP), Service::Name("sm-pas-2")),
        (ServiceQuery(2501, Protocol::TCP), Service::Name("rtsclient")),
        (ServiceQuery(3363, Protocol::UDP), Service::Name("nati-vi-server")),
        (ServiceQuery(9310, Protocol::TCP), Service::Name("sapms")),
        (ServiceQuery(33890, Protocol::TCP), Service::Name("digilent-adept")),
        (ServiceQuery(6667, Protocol::TCP), Service::Name("irc")),
        (ServiceQuery(1357, Protocol::UDP), Service::Name("pegboard")),
        (ServiceQuery(403, Protocol::TCP), Service::Name("decap")),
        (ServiceQuery(626, Protocol::TCP), Service::Name("apple-imap-admin")),
        (ServiceQuery(5133, Protocol::UDP), Service::Name("nbt-pc")),
        (ServiceQuery(216, Protocol::TCP), Service::Name("atls")),
        (ServiceQuery(1746, Protocol::UDP), Service::Name("ftrapid-1")),
        (ServiceQuery(2596, Protocol::TCP), Service::Name("worldfusion2")),
        (ServiceQuery(7401, Protocol::TCP), Service::Name("rtps-dd-ut")),
        (ServiceQuery(1754, Protocol::TCP), Service::Name("oracle-em2")),
        (ServiceQuery(2722, Protocol::UDP), Service::Name("proactivesrvr")),
        (ServiceQuery(2208, Protocol::TCP), Service::Name("hpiod")),
        (ServiceQuery(744, Protocol::UDP), Service::Name("flexlm")),
        (ServiceQuery(4248, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1218, Protocol::TCP), Service::Name("aeroflight-ads")),
        (ServiceQuery(8889, Protocol::TCP), Service::Name("ddi-tcp-2")),
        (ServiceQuery(1241, Protocol::UDP), Service::Name("nessus")),
        (ServiceQuery(4145, Protocol::TCP), Service::Name("vvr-control")),
        (ServiceQuery(1637, Protocol::UDP), Service::Name("islc")),
        (ServiceQuery(5145, Protocol::TCP), Service::Name("rmonitor_secure")),
        (ServiceQuery(661, Protocol::UDP), Service::Name("hap")),
        (ServiceQuery(5575, Protocol::TCP), Service::Name("ora-oap")),
        (ServiceQuery(3833, Protocol::UDP), Service::Name("aipn-auth")),
        (ServiceQuery(9418, Protocol::UDP), Service::Name("git")),
        (ServiceQuery(2654, Protocol::TCP), Service::Name("corel_vncadmin")),
        (ServiceQuery(2510, Protocol::TCP), Service::Name("fjappmgrbulk")),
        (ServiceQuery(1913, Protocol::TCP), Service::Name("armadp")),
        (ServiceQuery(680, Protocol::UDP), Service::Name("entrust-aaas")),
        (ServiceQuery(1243, Protocol::UDP), Service::Name("serialgateway")),
        (ServiceQuery(2557, Protocol::TCP), Service::Name("nicetec-mgmt")),
        (ServiceQuery(8710, Protocol::TCP), Service::Name("semi-grpc")),
        (ServiceQuery(19412, Protocol::TCP), Service::Name("hp-sessmon")),
        (ServiceQuery(9278, Protocol::UDP), Service::Name("pegasus")),
        (ServiceQuery(4322, Protocol::UDP), Service::Name("trim-event")),
        (ServiceQuery(5015, Protocol::TCP), Service::Name("fmwp")),
        (ServiceQuery(6015, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(3532, Protocol::UDP), Service::Name("raven-rmp")),
        (ServiceQuery(3271, Protocol::UDP), Service::Name("csoft-prev")),
        (ServiceQuery(3302, Protocol::UDP), Service::Name("mcs-fastmail")),
        (ServiceQuery(5106, Protocol::TCP), Service::Name("actifioudsagent")),
        (ServiceQuery(409, Protocol::UDP), Service::Name("prm-nm")),
        (ServiceQuery(3743, Protocol::UDP), Service::Name("ipcs-command")),
        (ServiceQuery(2024, Protocol::UDP), Service::Name("xinuexpansion4")),
        (ServiceQuery(7689, Protocol::TCP), Service::Name("collaber")),
        (ServiceQuery(1770, Protocol::UDP), Service::Name("bmc-net-svc")),
        (ServiceQuery(3583, Protocol::UDP), Service::Name("canex-watch")),
        (ServiceQuery(19540, Protocol::UDP), Service::Name("sxuptp")),
        (ServiceQuery(5215, Protocol::TCP), Service::Name("noteza")),
        (ServiceQuery(25003, Protocol::TCP), Service::Name("icl-twobase4")),
        (ServiceQuery(731, Protocol::UDP), Service::Name("netviewdm3")),
        (ServiceQuery(5404, Protocol::TCP), Service::Name("hpoms-dps-lstn")),
        (ServiceQuery(4115, Protocol::UDP), Service::Name("cds")),
        (ServiceQuery(1576, Protocol::UDP), Service::Name("moldflow-lm")),
        (ServiceQuery(10540, Protocol::TCP), Service::Name("MOS-lower")),
        (ServiceQuery(4013, Protocol::TCP), Service::Name("acl-manager")),
        (ServiceQuery(1261, Protocol::UDP), Service::Name("mpshrsv")),
        (ServiceQuery(4303, Protocol::UDP), Service::Name("srcp")),
        (ServiceQuery(1780, Protocol::TCP), Service::Name("dpkeyserv")),
        (ServiceQuery(4254, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2, Protocol::UDP), Service::Name("compressnet")),
        (ServiceQuery(4271, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3527, Protocol::TCP), Service::Name("beserver-msg-q")),
        (ServiceQuery(6037, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(1606, Protocol::TCP), Service::Name("slm-api")),
        (ServiceQuery(3204, Protocol::TCP), Service::Name("netwatcher-db")),
        (ServiceQuery(9200, Protocol::UDP), Service::Name("wap-wsp")),
        (ServiceQuery(36444, Protocol::TCP), Service::Name("m3ap")),
        (ServiceQuery(1120, Protocol::TCP), Service::Name("bnetfile")),
        (ServiceQuery(6115, Protocol::TCP), Service::Name("xic")),
        (ServiceQuery(1342, Protocol::UDP), Service::Name("esbroker")),
        (ServiceQuery(3877, Protocol::UDP), Service::Name("xmpcr-interface")),
        (ServiceQuery(8086, Protocol::TCP), Service::Name("d-s-n")),
        (ServiceQuery(40853, Protocol::UDP), Service::Name("ortec-disc")),
        (ServiceQuery(404, Protocol::UDP), Service::Name("nced")),
        (ServiceQuery(4566, Protocol::UDP), Service::Name("kwtc")),
        (ServiceQuery(183, Protocol::UDP), Service::Name("ocbinder")),
        (ServiceQuery(1844, Protocol::UDP), Service::Name("direcpc-dll")),
        (ServiceQuery(3260, Protocol::UDP), Service::Name("iscsi")),
        (ServiceQuery(913, Protocol::UDP), Service::Name("apex-edge")),
        (ServiceQuery(149, Protocol::UDP), Service::Name("aed-512")),
        (ServiceQuery(25471, Protocol::TCP), Service::Name("rna")),
        (ServiceQuery(861, Protocol::TCP), Service::Name("owamp-control")),
        (ServiceQuery(2283, Protocol::TCP), Service::Name("lnvstatus")),
        (ServiceQuery(5026, Protocol::TCP), Service::Name("strexec-d")),
        (ServiceQuery(26261, Protocol::UDP), Service::Name("ezmeeting")),
        (ServiceQuery(1384, Protocol::UDP), Service::Name("os-licman")),
        (ServiceQuery(43191, Protocol::TCP), Service::Name("noit-transport")),
        (ServiceQuery(377, Protocol::TCP), Service::Name("tnETOS")),
        (ServiceQuery(1815, Protocol::TCP), Service::Name("mmpft")),
        (ServiceQuery(1998, Protocol::UDP), Service::Name("x25-svc-port")),
        (ServiceQuery(7315, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(3002, Protocol::UDP), Service::Name("exlm-agent")),
        (ServiceQuery(673, Protocol::TCP), Service::Name("cimplex")),
        (ServiceQuery(2263, Protocol::UDP), Service::Name("ecwcfg")),
        (ServiceQuery(511, Protocol::TCP), Service::Name("passgo")),
        (ServiceQuery(2238, Protocol::TCP), Service::Name("aviva-sna")),
        (ServiceQuery(2447, Protocol::TCP), Service::Name("ovwdb")),
        (ServiceQuery(2293, Protocol::UDP), Service::Name("npdbgmngr")),
        (ServiceQuery(3341, Protocol::UDP), Service::Name("anet-h")),
        (ServiceQuery(2129, Protocol::TCP), Service::Name("cs-live")),
        (ServiceQuery(3073, Protocol::UDP), Service::Name("vcrp")),
        (ServiceQuery(1981, Protocol::TCP), Service::Name("p2pq")),
        (ServiceQuery(2617, Protocol::UDP), Service::Name("cmadmin")),
        (ServiceQuery(4071, Protocol::TCP), Service::Name("aibkup")),
        (ServiceQuery(3119, Protocol::TCP), Service::Name("d2000kernel")),
        (ServiceQuery(5597, Protocol::UDP), Service::Name("ininmessaging")),
        (ServiceQuery(450, Protocol::TCP), Service::Name("tserver")),
        (ServiceQuery(2287, Protocol::UDP), Service::Name("dna")),
        (ServiceQuery(2256, Protocol::UDP), Service::Name("pcc-mfp")),
        (ServiceQuery(1554, Protocol::TCP), Service::Name("caci-lm")),
        (ServiceQuery(5249, Protocol::TCP), Service::Name("caaclang2")),
        (ServiceQuery(24601, Protocol::TCP), Service::Name("udpstp")),
        (ServiceQuery(5307, Protocol::TCP), Service::Name("sco-aip")),
        (ServiceQuery(2683, Protocol::TCP), Service::Name("ncdloadbalance")),
        (ServiceQuery(3683, Protocol::UDP), Service::Name("bmc-ea")),
        (ServiceQuery(2229, Protocol::UDP), Service::Name("datalens")),
        (ServiceQuery(3198, Protocol::UDP), Service::Name("embrace-dp-c")),
        (ServiceQuery(1619, Protocol::UDP), Service::Name("xs-openstorage")),
        (ServiceQuery(12004, Protocol::TCP), Service::Name("entextlow")),
        (ServiceQuery(17729, Protocol::UDP), Service::Name("ea")),
        (ServiceQuery(2329, Protocol::UDP), Service::Name("nvd")),
        (ServiceQuery(1741, Protocol::TCP), Service::Name("cisco-net-mgmt")),
        (ServiceQuery(5768, Protocol::TCP), Service::Name("openmailpxy")),
        (ServiceQuery(1742, Protocol::TCP), Service::Name("3Com-nsd")),
        (ServiceQuery(1716, Protocol::UDP), Service::Name("xmsg")),
        (ServiceQuery(2061, Protocol::TCP), Service::Name("netmount")),
        (ServiceQuery(6961, Protocol::UDP), Service::Name("jmact3")),
        (ServiceQuery(2109, Protocol::UDP), Service::Name("ergolight")),
        (ServiceQuery(30260, Protocol::UDP), Service::Name("kingdomsonline")),
        (ServiceQuery(3450, Protocol::TCP), Service::Name("castorproxy")),
        (ServiceQuery(2707, Protocol::UDP), Service::Name("emcsymapiport")),
        (ServiceQuery(5978, Protocol::TCP), Service::Name("ncd-diag-tcp")),
        (ServiceQuery(3366, Protocol::UDP), Service::Name("creativepartnr")),
        (ServiceQuery(3206, Protocol::UDP), Service::Name("ironmail")),
        (ServiceQuery(1497, Protocol::TCP), Service::Name("rfx-lm")),
        (ServiceQuery(1828, Protocol::UDP), Service::Name("itm-mcell-u")),
        (ServiceQuery(1201, Protocol::UDP), Service::Name("nucleus-sand")),
        (ServiceQuery(4208, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1879, Protocol::UDP), Service::Name("nettgain-nms")),
        (ServiceQuery(6582, Protocol::TCP), Service::Name("parsec-game")),
        (ServiceQuery(2749, Protocol::TCP), Service::Name("fjippol-cnsl")),
        (ServiceQuery(8205, Protocol::UDP), Service::Name("lm-instmgr")),
        (ServiceQuery(1423, Protocol::TCP), Service::Name("essbase")),
        (ServiceQuery(4902, Protocol::TCP), Service::Name("magiccontrol")),
        (ServiceQuery(3752, Protocol::UDP), Service::Name("vipremoteagent")),
        (ServiceQuery(5429, Protocol::TCP), Service::Name("base")),
        (ServiceQuery(2057, Protocol::TCP), Service::Name("rich-cp")),
        (ServiceQuery(25900, Protocol::UDP), Service::Name("tasp-net")),
        (ServiceQuery(1946, Protocol::UDP), Service::Name("tekpls")),
        (ServiceQuery(3538, Protocol::UDP), Service::Name("ibm-diradm")),
        (ServiceQuery(3697, Protocol::UDP), Service::Name("nw-license")),
        (ServiceQuery(4675, Protocol::TCP), Service::Name("dhct-status")),
        (ServiceQuery(3760, Protocol::UDP), Service::Name("adtempusclient")),
        (ServiceQuery(5083, Protocol::UDP), Service::Name("qfp")),
        (ServiceQuery(7282, Protocol::TCP), Service::Name("mzca-action")),
        (ServiceQuery(1542, Protocol::UDP), Service::Name("gridgen-elmd")),
        (ServiceQuery(4069, Protocol::UDP), Service::Name("minger")),
        (ServiceQuery(20999, Protocol::TCP), Service::Name("athand-mmp")),
        (ServiceQuery(19398, Protocol::TCP), Service::Name("mtrgtrans")),
        (ServiceQuery(2604, Protocol::UDP), Service::Name("nsc-ccs")),
        (ServiceQuery(5785, Protocol::UDP), Service::Name("3par-rcopy")),
        (ServiceQuery(2852, Protocol::UDP), Service::Name("bears-01")),
        (ServiceQuery(3625, Protocol::UDP), Service::Name("volley")),
        (ServiceQuery(1675, Protocol::TCP), Service::Name("pdp")),
        (ServiceQuery(2723, Protocol::TCP), Service::Name("watchdog-nt")),
        (ServiceQuery(4744, Protocol::TCP), Service::Name("ifsp")),
        (ServiceQuery(3059, Protocol::TCP), Service::Name("qsoft")),
        (ServiceQuery(8058, Protocol::UDP), Service::Name("senomix07")),
        (ServiceQuery(2357, Protocol::UDP), Service::Name("unihub-server")),
        (ServiceQuery(2833, Protocol::UDP), Service::Name("glishd")),
        (ServiceQuery(6032, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(1166, Protocol::TCP), Service::Name("qsm-remote")),
        (ServiceQuery(8567, Protocol::TCP), Service::Name("oap-admin")),
        (ServiceQuery(3139, Protocol::UDP), Service::Name("incognitorv")),
        (ServiceQuery(995, Protocol::TCP), Service::Name("pop3s")),
        (ServiceQuery(3501, Protocol::UDP), Service::Name("isoft-p2p")),
        (ServiceQuery(1350, Protocol::UDP), Service::Name("editbench")),
        (ServiceQuery(150, Protocol::TCP), Service::Name("sql-net")),
        (ServiceQuery(7316, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(44600, Protocol::TCP), Service::Name("asihpi")),
        (ServiceQuery(5628, Protocol::TCP), Service::Name("htrust")),
        (ServiceQuery(3687, Protocol::UDP), Service::Name("simple-push")),
        (ServiceQuery(3033, Protocol::UDP), Service::Name("pdb")),
        (ServiceQuery(9597, Protocol::TCP), Service::Name("pd-admin")),
        (ServiceQuery(9091, Protocol::TCP), Service::Name("xmltec-xmlmail")),
        (ServiceQuery(9215, Protocol::UDP), Service::Name("insis")),
        (ServiceQuery(2214, Protocol::TCP), Service::Name("rpi")),
        (ServiceQuery(8129, Protocol::UDP), Service::Name("paycash-wbp")),
        (ServiceQuery(1116, Protocol::UDP), Service::Name("ardus-cntl")),
        (ServiceQuery(1060, Protocol::UDP), Service::Name("polestar")),
        (ServiceQuery(3725, Protocol::TCP), Service::Name("na-er-tip")),
        (ServiceQuery(3896, Protocol::TCP), Service::Name("sdo-tls")),
        (ServiceQuery(17300, Protocol::TCP), Service::Name("kuang2")),
        (ServiceQuery(2400, Protocol::UDP), Service::Name("opequus-server")),
        (ServiceQuery(2023, Protocol::TCP), Service::Name("xinuexpansion3")),
        (ServiceQuery(4350, Protocol::TCP), Service::Name("net-device")),
        (ServiceQuery(1091, Protocol::UDP), Service::Name("ff-sm")),
        (ServiceQuery(6379, Protocol::TCP), Service::Name("redis")),
        (ServiceQuery(9695, Protocol::TCP), Service::Name("ccnx")),
        (ServiceQuery(3076, Protocol::TCP), Service::Name("orbix-config")),
        (ServiceQuery(420, Protocol::TCP), Service::Name("smpte")),
        (ServiceQuery(43439, Protocol::TCP), Service::Name("eq3-update")),
        (ServiceQuery(3923, Protocol::TCP), Service::Name("symb-sb-port")),
        (ServiceQuery(7073, Protocol::TCP), Service::Name("martalk")),
        (ServiceQuery(7273, Protocol::TCP), Service::Name("openmanage")),
        (ServiceQuery(3662, Protocol::UDP), Service::Name("pserver")),
        (ServiceQuery(213, Protocol::UDP), Service::Name("ipx")),
        (ServiceQuery(2695, Protocol::UDP), Service::Name("vspread")),
        (ServiceQuery(2764, Protocol::UDP), Service::Name("data-insurance")),
        (ServiceQuery(1693, Protocol::TCP), Service::Name("rrirtr")),
        (ServiceQuery(4369, Protocol::UDP), Service::Name("epmd")),
        (ServiceQuery(2299, Protocol::TCP), Service::Name("pc-telecommute")),
        (ServiceQuery(2053, Protocol::UDP), Service::Name("lot105-ds-upd")),
        (ServiceQuery(1191, Protocol::UDP), Service::Name("gpfs")),
        (ServiceQuery(2799, Protocol::TCP), Service::Name("icon-discover")),
        (ServiceQuery(9345, Protocol::TCP), Service::Name("rancher")),
        (ServiceQuery(696, Protocol::TCP), Service::Name("rushd")),
        (ServiceQuery(5023, Protocol::TCP), Service::Name("htuilsrv")),
        (ServiceQuery(7314, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(8202, Protocol::TCP), Service::Name("aesop")),
        (ServiceQuery(7426, Protocol::TCP), Service::Name("pmdmgr")),
        (ServiceQuery(8473, Protocol::TCP), Service::Name("vp2p")),
        (ServiceQuery(3369, Protocol::UDP), Service::Name("satvid-datalnk")),
        (ServiceQuery(2412, Protocol::TCP), Service::Name("cdn")),
        (ServiceQuery(4122, Protocol::TCP), Service::Name("fprams")),
        (ServiceQuery(1217, Protocol::TCP), Service::Name("hpss-ndapi")),
        (ServiceQuery(206, Protocol::TCP), Service::Name("at-zis")),
        (ServiceQuery(2010, Protocol::UDP), Service::Name("pipe_server")),
        (ServiceQuery(7781, Protocol::TCP), Service::Name("accu-lmgr")),
        (ServiceQuery(3143, Protocol::TCP), Service::Name("seaview")),
        (ServiceQuery(4273, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1123, Protocol::TCP), Service::Name("murray")),
        (ServiceQuery(3180, Protocol::UDP), Service::Name("mc-brk-srv")),
        (ServiceQuery(3173, Protocol::TCP), Service::Name("serverview-icc")),
        (ServiceQuery(4132, Protocol::TCP), Service::Name("nuts_dem")),
        (ServiceQuery(2695, Protocol::TCP), Service::Name("vspread")),
        (ServiceQuery(4882, Protocol::TCP), Service::Name("socp-c")),
        (ServiceQuery(4568, Protocol::TCP), Service::Name("bmc-reporting")),
        (ServiceQuery(4692, Protocol::UDP), Service::Name("conspiracy")),
        (ServiceQuery(18186, Protocol::UDP), Service::Name("ohsc")),
        (ServiceQuery(118, Protocol::TCP), Service::Name("sqlserv")),
        (ServiceQuery(4310, Protocol::UDP), Service::Name("mirrtex")),
        (ServiceQuery(8160, Protocol::TCP), Service::Name("patrol")),
        (ServiceQuery(314, Protocol::UDP), Service::Name("opalis-robot")),
        (ServiceQuery(4304, Protocol::UDP), Service::Name("owserver")),
        (ServiceQuery(1870, Protocol::UDP), Service::Name("sunscalar-dns")),
        (ServiceQuery(1038, Protocol::UDP), Service::Name("mtqp")),
        (ServiceQuery(3209, Protocol::UDP), Service::Name("netwkpathengine")),
        (ServiceQuery(1730, Protocol::TCP), Service::Name("roketz")),
        (ServiceQuery(9216, Protocol::TCP), Service::Name("acme")),
        (ServiceQuery(3065, Protocol::UDP), Service::Name("slinterbase")),
        (ServiceQuery(1403, Protocol::UDP), Service::Name("prm-nm-np")),
        (ServiceQuery(2515, Protocol::UDP), Service::Name("facsys-router")),
        (ServiceQuery(3102, Protocol::UDP), Service::Name("sl-mon")),
        (ServiceQuery(6343, Protocol::UDP), Service::Name("sflow")),
        (ServiceQuery(5424, Protocol::TCP), Service::Name("beyond-remote")),
        (ServiceQuery(374, Protocol::UDP), Service::Name("legent-2")),
        (ServiceQuery(3449, Protocol::TCP), Service::Name("hotu-chat")),
        (ServiceQuery(7355, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(4419, Protocol::TCP), Service::Name("cbp")),
        (ServiceQuery(1056, Protocol::TCP), Service::Name("vfo")),
        (ServiceQuery(5190, Protocol::TCP), Service::Name("aol")),
        (ServiceQuery(3554, Protocol::UDP), Service::Name("questnotify")),
        (ServiceQuery(522, Protocol::UDP), Service::Name("ulp")),
        (ServiceQuery(11165, Protocol::TCP), Service::Name("suncacao-websvc")),
        (ServiceQuery(3024, Protocol::UDP), Service::Name("nds_sso")),
        (ServiceQuery(40843, Protocol::UDP), Service::Name("csccfirewall")),
        (ServiceQuery(2652, Protocol::UDP), Service::Name("interpathpanel")),
        (ServiceQuery(5235, Protocol::TCP), Service::Name("galaxy-network")),
        (ServiceQuery(2390, Protocol::TCP), Service::Name("rsmtp")),
        (ServiceQuery(34980, Protocol::TCP), Service::Name("ethercat")),
        (ServiceQuery(247, Protocol::TCP), Service::Name("subntbcst_tftp")),
        (ServiceQuery(4326, Protocol::UDP), Service::Name("geognosis")),
        (ServiceQuery(5112, Protocol::UDP), Service::Name("pm-cmdsvr")),
        (ServiceQuery(613, Protocol::TCP), Service::Name("hmmp-op")),
        (ServiceQuery(5227, Protocol::UDP), Service::Name("perfd")),
        (ServiceQuery(15740, Protocol::UDP), Service::Name("ptp")),
        (ServiceQuery(1595, Protocol::UDP), Service::Name("radio")),
        (ServiceQuery(4109, Protocol::TCP), Service::Name("izm")),
        (ServiceQuery(2122, Protocol::TCP), Service::Name("caupc-remote")),
        (ServiceQuery(6317, Protocol::TCP), Service::Name("nav-data-cmd")),
        (ServiceQuery(35005, Protocol::TCP), Service::Name("rt-labtracker")),
        (ServiceQuery(5557, Protocol::TCP), Service::Name("farenet")),
        (ServiceQuery(3328, Protocol::TCP), Service::Name("egptlm")),
        (ServiceQuery(2388, Protocol::UDP), Service::Name("mynahautostart")),
        (ServiceQuery(5748, Protocol::UDP), Service::Name("tunalyzer")),
        (ServiceQuery(217, Protocol::TCP), Service::Name("dbase")),
        (ServiceQuery(1514, Protocol::TCP), Service::Name("fujitsu-dtcns")),
        (ServiceQuery(3218, Protocol::UDP), Service::Name("smartpackets")),
        (ServiceQuery(3272, Protocol::UDP), Service::Name("user-manager")),
        (ServiceQuery(2619, Protocol::TCP), Service::Name("bruce")),
        (ServiceQuery(1614, Protocol::TCP), Service::Name("netbill-cred")),
        (ServiceQuery(5420, Protocol::TCP), Service::Name("cylink-c")),
        (ServiceQuery(319, Protocol::TCP), Service::Name("ptp-event")),
        (ServiceQuery(5105, Protocol::UDP), Service::Name("hughes-ap")),
        (ServiceQuery(25954, Protocol::TCP), Service::Name("bf-game")),
        (ServiceQuery(2391, Protocol::UDP), Service::Name("3com-net-mgmt")),
        (ServiceQuery(223, Protocol::UDP), Service::Name("cdc")),
        (ServiceQuery(1978, Protocol::UDP), Service::Name("unisql")),
        (ServiceQuery(8910, Protocol::TCP), Service::Name("manyone-http")),
        (ServiceQuery(621, Protocol::TCP), Service::Name("escp-ip")),
        (ServiceQuery(1126, Protocol::UDP), Service::Name("hpvmmdata")),
        (ServiceQuery(3109, Protocol::TCP), Service::Name("personnel")),
        (ServiceQuery(3653, Protocol::TCP), Service::Name("tsp")),
        (ServiceQuery(621, Protocol::UDP), Service::Name("escp-ip")),
        (ServiceQuery(1318, Protocol::UDP), Service::Name("krb5gatekeeper")),
        (ServiceQuery(32772, Protocol::UDP), Service::Name("sometimes-rpc8")),
        (ServiceQuery(4557, Protocol::UDP), Service::Name("mtcevrunqss")),
        (ServiceQuery(3547, Protocol::TCP), Service::Name("symantec-sim")),
        (ServiceQuery(6040, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(32767, Protocol::TCP), Service::Name("filenet-powsrm")),
        (ServiceQuery(35355, Protocol::TCP), Service::Name("altova-lm")),
        (ServiceQuery(1235, Protocol::UDP), Service::Name("mosaicsyssvc1")),
        (ServiceQuery(16000, Protocol::TCP), Service::Name("fmsas")),
        (ServiceQuery(3176, Protocol::TCP), Service::Name("ars-master")),
        (ServiceQuery(3718, Protocol::TCP), Service::Name("opus-services")),
        (ServiceQuery(25003, Protocol::UDP), Service::Name("icl-twobase4")),
        (ServiceQuery(2117, Protocol::TCP), Service::Name("mentaclient")),
        (ServiceQuery(43190, Protocol::TCP), Service::Name("ip-provision")),
        (ServiceQuery(4951, Protocol::UDP), Service::Name("pwgwims")),
        (ServiceQuery(2936, Protocol::TCP), Service::Name("otpatch")),
        (ServiceQuery(65301, Protocol::TCP), Service::Name("pcanywhere")),
        (ServiceQuery(6567, Protocol::TCP), Service::Name("esp")),
        (ServiceQuery(6831, Protocol::TCP), Service::Name("ambit-lm")),
        (ServiceQuery(1602, Protocol::UDP), Service::Name("inspect")),
        (ServiceQuery(5415, Protocol::UDP), Service::Name("ns-server")),
        (ServiceQuery(499, Protocol::TCP), Service::Name("iso-ill")),
        (ServiceQuery(3542, Protocol::UDP), Service::Name("hacl-monitor")),
        (ServiceQuery(205, Protocol::UDP), Service::Name("at-5")),
        (ServiceQuery(4030, Protocol::TCP), Service::Name("jdmn-port")),
        (ServiceQuery(1298, Protocol::UDP), Service::Name("lpcp")),
        (ServiceQuery(2034, Protocol::UDP), Service::Name("scoremgr")),
        (ServiceQuery(88, Protocol::TCP), Service::Name("kerberos-sec")),
        (ServiceQuery(3787, Protocol::UDP), Service::Name("fintrx")),
        (ServiceQuery(7011, Protocol::TCP), Service::Name("talon-disc")),
        (ServiceQuery(3234, Protocol::TCP), Service::Name("alchemy")),
        (ServiceQuery(786, Protocol::TCP), Service::Name("concert")),
        (ServiceQuery(1814, Protocol::UDP), Service::Name("tdp-suite")),
        (ServiceQuery(207, Protocol::UDP), Service::Name("at-7")),
        (ServiceQuery(10115, Protocol::TCP), Service::Name("netiq-endpt")),
        (ServiceQuery(3672, Protocol::TCP), Service::Name("lispworks-orb")),
        (ServiceQuery(1365, Protocol::TCP), Service::Name("adapt-sna")),
        (ServiceQuery(205, Protocol::TCP), Service::Name("at-5")),
        (ServiceQuery(3871, Protocol::UDP), Service::Name("avocent-adsap")),
        (ServiceQuery(2294, Protocol::UDP), Service::Name("konshus-lm")),
        (ServiceQuery(6028, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(48, Protocol::UDP), Service::Name("auditd")),
        (ServiceQuery(1790, Protocol::UDP), Service::Name("nmsp")),
        (ServiceQuery(161, Protocol::UDP), Service::Name("snmp")),
        (ServiceQuery(3462, Protocol::UDP), Service::Name("edm-std-notify")),
        (ServiceQuery(2105, Protocol::TCP), Service::Name("eklogin")),
        (ServiceQuery(8378, Protocol::TCP), Service::Name("cruise-config")),
        (ServiceQuery(3738, Protocol::TCP), Service::Name("versatalk")),
        (ServiceQuery(17, Protocol::TCP), Service::Name("qotd")),
        (ServiceQuery(383, Protocol::TCP), Service::Name("hp-alarm-mgr")),
        (ServiceQuery(1572, Protocol::UDP), Service::Name("chip-lm")),
        (ServiceQuery(2282, Protocol::UDP), Service::Name("lnvalarm")),
        (ServiceQuery(1621, Protocol::UDP), Service::Name("softdataphone")),
        (ServiceQuery(2630, Protocol::TCP), Service::Name("sitaramgmt")),
        (ServiceQuery(9978, Protocol::TCP), Service::Name("xybrid-rt")),
        (ServiceQuery(1540, Protocol::UDP), Service::Name("rds")),
        (ServiceQuery(8042, Protocol::TCP), Service::Name("fs-agent")),
        (ServiceQuery(2464, Protocol::TCP), Service::Name("direcpc-si")),
        (ServiceQuery(4192, Protocol::UDP), Service::Name("azeti-bd")),
        (ServiceQuery(14154, Protocol::UDP), Service::Name("vad")),
        (ServiceQuery(1434, Protocol::TCP), Service::Name("ms-sql-m")),
        (ServiceQuery(2094, Protocol::TCP), Service::Name("nbx-au")),
        (ServiceQuery(2066, Protocol::UDP), Service::Name("aura")),
        (ServiceQuery(1725, Protocol::TCP), Service::Name("iden-ralp")),
        (ServiceQuery(1185, Protocol::TCP), Service::Name("catchpole")),
        (ServiceQuery(8699, Protocol::TCP), Service::Name("vnyx")),
        (ServiceQuery(6669, Protocol::TCP), Service::Name("irc")),
        (ServiceQuery(10055, Protocol::TCP), Service::Name("qptlmd")),
        (ServiceQuery(1537, Protocol::UDP), Service::Name("sdsc-lm")),
        (ServiceQuery(3537, Protocol::TCP), Service::Name("ni-visa-remote")),
        (ServiceQuery(3441, Protocol::TCP), Service::Name("connect-client")),
        (ServiceQuery(4548, Protocol::TCP), Service::Name("synchromesh")),
        (ServiceQuery(1, Protocol::TCP), Service::Name("tcpmux")),
        (ServiceQuery(7017, Protocol::TCP), Service::Name("grasp")),
        (ServiceQuery(3421, Protocol::UDP), Service::Name("bmap")),
        (ServiceQuery(3480, Protocol::TCP), Service::Name("plethora")),
        (ServiceQuery(3784, Protocol::TCP), Service::Name("bfd-control")),
        (ServiceQuery(4030, Protocol::UDP), Service::Name("jdmn-port")),
        (ServiceQuery(1055, Protocol::TCP), Service::Name("ansyslmd")),
        (ServiceQuery(3445, Protocol::TCP), Service::Name("monp")),
        (ServiceQuery(24, Protocol::UDP), Service::Name("priv-mail")),
        (ServiceQuery(4225, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3197, Protocol::TCP), Service::Name("embrace-dp-s")),
        (ServiceQuery(1916, Protocol::TCP), Service::Name("persona")),
        (ServiceQuery(7312, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(5105, Protocol::TCP), Service::Name("hughes-ap")),
        (ServiceQuery(1424, Protocol::TCP), Service::Name("hybrid")),
        (ServiceQuery(1848, Protocol::TCP), Service::Name("fjdocdist")),
        (ServiceQuery(4448, Protocol::UDP), Service::Name("asc-slmd")),
        (ServiceQuery(3822, Protocol::TCP), Service::Name("acp-discovery")),
        (ServiceQuery(4220, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(4415, Protocol::TCP), Service::Name("brcd-vr-req")),
        (ServiceQuery(3880, Protocol::UDP), Service::Name("igrs")),
        (ServiceQuery(2418, Protocol::TCP), Service::Name("cas")),
        (ServiceQuery(9321, Protocol::TCP), Service::Name("guibase")),
        (ServiceQuery(3452, Protocol::UDP), Service::Name("sabp-signal")),
        (ServiceQuery(9255, Protocol::TCP), Service::Name("mon")),
        (ServiceQuery(702, Protocol::UDP), Service::Name("iris-beep")),
        (ServiceQuery(9418, Protocol::TCP), Service::Name("git")),
        (ServiceQuery(3596, Protocol::UDP), Service::Name("iw-mmogame")),
        (ServiceQuery(1286, Protocol::TCP), Service::Name("netuitive")),
        (ServiceQuery(2920, Protocol::UDP), Service::Name("roboeda")),
        (ServiceQuery(7030, Protocol::TCP), Service::Name("op-probe")),
        (ServiceQuery(5314, Protocol::TCP), Service::Name("opalis-rbt-ipc")),
        (ServiceQuery(1520, Protocol::TCP), Service::Name("atm-zip-office")),
        (ServiceQuery(1324, Protocol::TCP), Service::Name("delta-mcp")),
        (ServiceQuery(2367, Protocol::UDP), Service::Name("service-ctrl")),
        (ServiceQuery(6058, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(989, Protocol::UDP), Service::Name("ftps-data")),
        (ServiceQuery(1790, Protocol::TCP), Service::Name("nmsp")),
        (ServiceQuery(4452, Protocol::TCP), Service::Name("ctiprogramload")),
        (ServiceQuery(2477, Protocol::TCP), Service::Name("ssm-cvs")),
        (ServiceQuery(2904, Protocol::UDP), Service::Name("m2ua")),
        (ServiceQuery(4390, Protocol::UDP), Service::Name("wiegand")),
        (ServiceQuery(4480, Protocol::TCP), Service::Name("proxy-plus")),
        (ServiceQuery(2778, Protocol::UDP), Service::Name("gwen-sonya")),
        (ServiceQuery(1648, Protocol::UDP), Service::Name("concurrent-lm")),
        (ServiceQuery(1515, Protocol::UDP), Service::Name("ifor-protocol")),
        (ServiceQuery(3977, Protocol::UDP), Service::Name("opswmanager")),
        (ServiceQuery(4487, Protocol::TCP), Service::Name("prex-tcp")),
        (ServiceQuery(10100, Protocol::UDP), Service::Name("itap-ddtp")),
        (ServiceQuery(15363, Protocol::UDP), Service::Name("3link")),
        (ServiceQuery(7697, Protocol::TCP), Service::Name("klio")),
        (ServiceQuery(6147, Protocol::UDP), Service::Name("montage-lm")),
        (ServiceQuery(2395, Protocol::UDP), Service::Name("lan900_remote")),
        (ServiceQuery(828, Protocol::UDP), Service::Name("itm-mcell-s")),
        (ServiceQuery(42509, Protocol::TCP), Service::Name("candrp")),
        (ServiceQuery(5165, Protocol::TCP), Service::Name("ife_icorp")),
        (ServiceQuery(5351, Protocol::UDP), Service::Name("nat-pmp")),
        (ServiceQuery(653, Protocol::TCP), Service::Name("repscmd")),
        (ServiceQuery(11371, Protocol::TCP), Service::Name("pksd")),
        (ServiceQuery(2735, Protocol::UDP), Service::Name("netiq-mc")),
        (ServiceQuery(2026, Protocol::TCP), Service::Name("scrabble")),
        (ServiceQuery(3689, Protocol::UDP), Service::Name("daap")),
        (ServiceQuery(43440, Protocol::UDP), Service::Name("ew-disc-cmd")),
        (ServiceQuery(419, Protocol::UDP), Service::Name("ariel1")),
        (ServiceQuery(431, Protocol::TCP), Service::Name("utmpcd")),
        (ServiceQuery(7235, Protocol::TCP), Service::Name("aspcoordination")),
        (ServiceQuery(770, Protocol::TCP), Service::Name("cadlock")),
        (ServiceQuery(5600, Protocol::UDP), Service::Name("esmmanager")),
        (ServiceQuery(7003, Protocol::TCP), Service::Name("afs3-vlserver")),
        (ServiceQuery(17184, Protocol::TCP), Service::Name("vestasdlp")),
        (ServiceQuery(1944, Protocol::TCP), Service::Name("close-combat")),
        (ServiceQuery(1797, Protocol::UDP), Service::Name("uma")),
        (ServiceQuery(4003, Protocol::TCP), Service::Name("pxc-splr-ft")),
        (ServiceQuery(1506, Protocol::UDP), Service::Name("utcd")),
        (ServiceQuery(43210, Protocol::TCP), Service::Name("shaperai")),
        (ServiceQuery(5423, Protocol::UDP), Service::Name("virtualuser")),
        (ServiceQuery(3573, Protocol::UDP), Service::Name("tag-ups-1")),
        (ServiceQuery(8612, Protocol::UDP), Service::Name("canon-bjnp2")),
        (ServiceQuery(2909, Protocol::TCP), Service::Name("funk-dialout")),
        (ServiceQuery(184, Protocol::UDP), Service::Name("ocserver")),
        (ServiceQuery(5000, Protocol::UDP), Service::Name("upnp")),
        (ServiceQuery(25901, Protocol::TCP), Service::Name("niobserver")),
        (ServiceQuery(3348, Protocol::UDP), Service::Name("pangolin-laser")),
        (ServiceQuery(14250, Protocol::TCP), Service::Name("cps")),
        (ServiceQuery(9160, Protocol::TCP), Service::Name("apani1")),
        (ServiceQuery(543, Protocol::TCP), Service::Name("klogin")),
        (ServiceQuery(3776, Protocol::UDP), Service::Name("dvcprov-port")),
        (ServiceQuery(2842, Protocol::UDP), Service::Name("l3-hawk")),
        (ServiceQuery(3005, Protocol::TCP), Service::Name("deslogin")),
        (ServiceQuery(5671, Protocol::UDP), Service::Name("amqps")),
        (ServiceQuery(1810, Protocol::UDP), Service::Name("jerand-lm")),
        (ServiceQuery(2656, Protocol::UDP), Service::Name("kana")),
        (ServiceQuery(3415, Protocol::TCP), Service::Name("bcinameservice")),
        (ServiceQuery(3817, Protocol::UDP), Service::Name("tapeware")),
        (ServiceQuery(6042, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(4756, Protocol::TCP), Service::Name("RDCenter")),
        (ServiceQuery(10321, Protocol::TCP), Service::Name("cosir")),
        (ServiceQuery(2915, Protocol::UDP), Service::Name("tksocket")),
        (ServiceQuery(27016, Protocol::TCP), Service::Name("chlenix")),
        (ServiceQuery(1784, Protocol::UDP), Service::Name("finle-lm")),
        (ServiceQuery(3194, Protocol::TCP), Service::Name("magbind")),
        (ServiceQuery(1900, Protocol::TCP), Service::Name("upnp")),
        (ServiceQuery(1654, Protocol::TCP), Service::Name("stargatealerts")),
        (ServiceQuery(4952, Protocol::TCP), Service::Name("sagxtsds")),
        (ServiceQuery(1916, Protocol::UDP), Service::Name("persona")),
        (ServiceQuery(2580, Protocol::TCP), Service::Name("tributary")),
        (ServiceQuery(4404, Protocol::UDP), Service::Name("ds-admin")),
        (ServiceQuery(312, Protocol::UDP), Service::Name("vslmp")),
        (ServiceQuery(5745, Protocol::TCP), Service::Name("fcopy-server")),
        (ServiceQuery(2718, Protocol::UDP), Service::Name("pn-requester2")),
        (ServiceQuery(481, Protocol::UDP), Service::Name("ph")),
        (ServiceQuery(7128, Protocol::UDP), Service::Name("scenidm")),
        (ServiceQuery(9875, Protocol::TCP), Service::Name("sapv1")),
        (ServiceQuery(706, Protocol::TCP), Service::Name("silc")),
        (ServiceQuery(3111, Protocol::TCP), Service::Name("wsynch")),
        (ServiceQuery(2338, Protocol::TCP), Service::Name("norton-lambert")),
        (ServiceQuery(1635, Protocol::TCP), Service::Name("edb-server1")),
        (ServiceQuery(2689, Protocol::UDP), Service::Name("fastlynx")),
        (ServiceQuery(2947, Protocol::TCP), Service::Name("gpsd")),
        (ServiceQuery(5033, Protocol::TCP), Service::Name("jtnetd-server")),
        (ServiceQuery(3689, Protocol::TCP), Service::Name("rendezvous")),
        (ServiceQuery(3740, Protocol::UDP), Service::Name("heartbeat")),
        (ServiceQuery(121, Protocol::UDP), Service::Name("erpc")),
        (ServiceQuery(5503, Protocol::TCP), Service::Name("fcp-srvr-inst2")),
        (ServiceQuery(446, Protocol::UDP), Service::Name("ddm-rdb")),
        (ServiceQuery(700, Protocol::TCP), Service::Name("epp")),
        (ServiceQuery(1395, Protocol::TCP), Service::Name("iclpv-wsm")),
        (ServiceQuery(2136, Protocol::UDP), Service::Name("appworxsrv")),
        (ServiceQuery(622, Protocol::TCP), Service::Name("collaborator")),
        (ServiceQuery(2504, Protocol::UDP), Service::Name("wlbs")),
        (ServiceQuery(4116, Protocol::UDP), Service::Name("smartcard-tls")),
        (ServiceQuery(11720, Protocol::UDP), Service::Name("h323callsigalt")),
        (ServiceQuery(13160, Protocol::TCP), Service::Name("i-zipqd")),
        (ServiceQuery(3854, Protocol::TCP), Service::Name("stryker-com")),
        (ServiceQuery(3418, Protocol::UDP), Service::Name("rnmap")),
        (ServiceQuery(571, Protocol::TCP), Service::Name("umeter")),
        (ServiceQuery(4312, Protocol::TCP), Service::Name("pscl-mgt")),
        (ServiceQuery(180, Protocol::TCP), Service::Name("ris")),
        (ServiceQuery(915, Protocol::TCP), Service::Name("rift-ties")),
        (ServiceQuery(4371, Protocol::UDP), Service::Name("l2c-disc")),
        (ServiceQuery(2630, Protocol::UDP), Service::Name("sitaramgmt")),
        (ServiceQuery(2698, Protocol::UDP), Service::Name("mck-ivpip")),
        (ServiceQuery(38002, Protocol::TCP), Service::Name("cresco-control")),
        (ServiceQuery(5627, Protocol::UDP), Service::Name("ninaf")),
        (ServiceQuery(9396, Protocol::UDP), Service::Name("fjinvmgr")),
        (ServiceQuery(2492, Protocol::TCP), Service::Name("groove")),
        (ServiceQuery(6141, Protocol::UDP), Service::Name("meta-corp")),
        (ServiceQuery(127, Protocol::UDP), Service::Name("locus-con")),
        (ServiceQuery(1398, Protocol::TCP), Service::Name("video-activmail")),
        (ServiceQuery(1293, Protocol::TCP), Service::Name("pkt-krb-ipsec")),
        (ServiceQuery(2200, Protocol::UDP), Service::Name("ici")),
        (ServiceQuery(1661, Protocol::UDP), Service::Name("netview-aix-1")),
        (ServiceQuery(425, Protocol::TCP), Service::Name("icad-el")),
        (ServiceQuery(3535, Protocol::TCP), Service::Name("ms-la")),
        (ServiceQuery(1704, Protocol::UDP), Service::Name("bcs-broker")),
        (ServiceQuery(3840, Protocol::TCP), Service::Name("flirtmitmir")),
        (ServiceQuery(3653, Protocol::UDP), Service::Name("tsp")),
        (ServiceQuery(2933, Protocol::TCP), Service::Name("4-tieropmgw")),
        (ServiceQuery(166, Protocol::TCP), Service::Name("s-net")),
        (ServiceQuery(2953, Protocol::TCP), Service::Name("ovalarmsrv")),
        (ServiceQuery(4105, Protocol::TCP), Service::Name("shofarplayer")),
        (ServiceQuery(196, Protocol::UDP), Service::Name("dn6-smm-red")),
        (ServiceQuery(7551, Protocol::TCP), Service::Name("controlone-con")),
        (ServiceQuery(5465, Protocol::TCP), Service::Name("netops-broker")),
        (ServiceQuery(7306, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(4243, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(4313, Protocol::TCP), Service::Name("perrla")),
        (ServiceQuery(3398, Protocol::UDP), Service::Name("mercantile")),
        (ServiceQuery(4082, Protocol::UDP), Service::Name("lorica-out")),
        (ServiceQuery(5047, Protocol::UDP), Service::Name("iscape")),
        (ServiceQuery(3936, Protocol::TCP), Service::Name("mailprox")),
        (ServiceQuery(4658, Protocol::UDP), Service::Name("playsta2-app")),
        (ServiceQuery(39681, Protocol::UDP), Service::Name("turbonote-1")),
        (ServiceQuery(3351, Protocol::TCP), Service::Name("btrieve")),
        (ServiceQuery(7282, Protocol::UDP), Service::Name("mzca-alert")),
        (ServiceQuery(10548, Protocol::TCP), Service::Name("serverdocs")),
        (ServiceQuery(707, Protocol::UDP), Service::Name("borland-dsj")),
        (ServiceQuery(2184, Protocol::UDP), Service::Name("nvd")),
        (ServiceQuery(1857, Protocol::TCP), Service::Name("datacaptor")),
        (ServiceQuery(2965, Protocol::UDP), Service::Name("bullant-rap")),
        (ServiceQuery(1591, Protocol::UDP), Service::Name("ncpm-pm")),
        (ServiceQuery(47100, Protocol::TCP), Service::Name("jvl-mactalk")),
        (ServiceQuery(5433, Protocol::TCP), Service::Name("pyrrho")),
        (ServiceQuery(110, Protocol::UDP), Service::Name("pop3")),
        (ServiceQuery(24322, Protocol::TCP), Service::Name("hid")),
        (ServiceQuery(8989, Protocol::TCP), Service::Name("sunwebadmins")),
        (ServiceQuery(21800, Protocol::TCP), Service::Name("tvpm")),
        (ServiceQuery(749, Protocol::TCP), Service::Name("kerberos-adm")),
        (ServiceQuery(553, Protocol::UDP), Service::Name("pirp")),
        (ServiceQuery(4286, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1116, Protocol::TCP), Service::Name("ardus-cntl")),
        (ServiceQuery(1139, Protocol::TCP), Service::Name("cce3x")),
        (ServiceQuery(2040, Protocol::TCP), Service::Name("lam")),
        (ServiceQuery(3151, Protocol::UDP), Service::Name("nm-assessor")),
        (ServiceQuery(1324, Protocol::UDP), Service::Name("delta-mcp")),
        (ServiceQuery(5863, Protocol::UDP), Service::Name("ppsuitemsg")),
        (ServiceQuery(1788, Protocol::TCP), Service::Name("psmond")),
        (ServiceQuery(356, Protocol::UDP), Service::Name("cloanto-net-1")),
        (ServiceQuery(2451, Protocol::UDP), Service::Name("netchat")),
        (ServiceQuery(8763, Protocol::UDP), Service::Name("mc-appserver")),
        (ServiceQuery(1763, Protocol::TCP), Service::Name("landesk-rc")),
        (ServiceQuery(2763, Protocol::UDP), Service::Name("desktop-dna")),
        (ServiceQuery(4374, Protocol::TCP), Service::Name("psi-ptt")),
        (ServiceQuery(4038, Protocol::UDP), Service::Name("fazzt-ptp")),
        (ServiceQuery(4686, Protocol::TCP), Service::Name("msp-os")),
        (ServiceQuery(11000, Protocol::TCP), Service::Name("irisa")),
        (ServiceQuery(2863, Protocol::UDP), Service::Name("sonardata")),
        (ServiceQuery(1527, Protocol::UDP), Service::Name("tlisrv")),
        (ServiceQuery(657, Protocol::TCP), Service::Name("rmc")),
        (ServiceQuery(14145, Protocol::UDP), Service::Name("gcm-app")),
        (ServiceQuery(5317, Protocol::TCP), Service::Name("hpdevms")),
        (ServiceQuery(7742, Protocol::TCP), Service::Name("msss")),
        (ServiceQuery(3668, Protocol::TCP), Service::Name("dell-rm-port")),
        (ServiceQuery(1914, Protocol::TCP), Service::Name("elm-momentum")),
        (ServiceQuery(3818, Protocol::UDP), Service::Name("crinis-hb")),
        (ServiceQuery(3555, Protocol::UDP), Service::Name("razor")),
        (ServiceQuery(2593, Protocol::UDP), Service::Name("mns-mail")),
        (ServiceQuery(4135, Protocol::UDP), Service::Name("cl-db-attach")),
        (ServiceQuery(9300, Protocol::TCP), Service::Name("vrace")),
        (ServiceQuery(8320, Protocol::TCP), Service::Name("tnp-discover")),
        (ServiceQuery(5062, Protocol::UDP), Service::Name("na-localise")),
        (ServiceQuery(603, Protocol::UDP), Service::Name("idxp")),
        (ServiceQuery(2681, Protocol::TCP), Service::Name("mpnjsomb")),
        (ServiceQuery(4320, Protocol::UDP), Service::Name("fdt-rcatp")),
        (ServiceQuery(4894, Protocol::UDP), Service::Name("lyskom")),
        (ServiceQuery(4850, Protocol::UDP), Service::Name("sun-as-nodeagt")),
        (ServiceQuery(5201, Protocol::UDP), Service::Name("targus-getdata1")),
        (ServiceQuery(5724, Protocol::TCP), Service::Name("omsdk")),
        (ServiceQuery(28000, Protocol::UDP), Service::Name("nxlmd")),
        (ServiceQuery(2162, Protocol::UDP), Service::Name("navisphere")),
        (ServiceQuery(4154, Protocol::TCP), Service::Name("atlinks")),
        (ServiceQuery(2694, Protocol::TCP), Service::Name("pwrsevent")),
        (ServiceQuery(2460, Protocol::UDP), Service::Name("ms-theater")),
        (ServiceQuery(3471, Protocol::TCP), Service::Name("jt400-ssl")),
        (ServiceQuery(2180, Protocol::UDP), Service::Name("mc-gt-srv")),
        (ServiceQuery(3599, Protocol::UDP), Service::Name("quasar-server")),
        (ServiceQuery(2631, Protocol::UDP), Service::Name("sitaradir")),
        (ServiceQuery(1467, Protocol::TCP), Service::Name("csdmbase")),
        (ServiceQuery(185, Protocol::TCP), Service::Name("remote-kis")),
        (ServiceQuery(5072, Protocol::TCP), Service::Name("ayiya")),
        (ServiceQuery(3916, Protocol::TCP), Service::Name("wysdmc")),
        (ServiceQuery(1301, Protocol::UDP), Service::Name("ci3-software-1")),
        (ServiceQuery(58, Protocol::UDP), Service::Name("xns-mail")),
        (ServiceQuery(3474, Protocol::UDP), Service::Name("ttntspauto")),
        (ServiceQuery(6951, Protocol::UDP), Service::Name("otlp")),
        (ServiceQuery(25100, Protocol::TCP), Service::Name("db2c-tls")),
        (ServiceQuery(4170, Protocol::TCP), Service::Name("d-cinema-csp")),
        (ServiceQuery(2686, Protocol::UDP), Service::Name("mpnjsomg")),
        (ServiceQuery(2646, Protocol::TCP), Service::Name("and-lm")),
        (ServiceQuery(1878, Protocol::TCP), Service::Name("drmsmc")),
        (ServiceQuery(1336, Protocol::TCP), Service::Name("ischat")),
        (ServiceQuery(6966, Protocol::TCP), Service::Name("swispol")),
        (ServiceQuery(1502, Protocol::UDP), Service::Name("shivadiscovery")),
        (ServiceQuery(5264, Protocol::UDP), Service::Name("3com-njack-1")),
        (ServiceQuery(3178, Protocol::UDP), Service::Name("radclientport")),
        (ServiceQuery(3349, Protocol::UDP), Service::Name("chevinservices")),
        (ServiceQuery(5726, Protocol::TCP), Service::Name("ms-ilm-sts")),
        (ServiceQuery(350, Protocol::UDP), Service::Name("matip-type-a")),
        (ServiceQuery(1409, Protocol::TCP), Service::Name("here-lm")),
        (ServiceQuery(4278, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(9100, Protocol::TCP), Service::Name("jetdirect")),
        (ServiceQuery(21849, Protocol::UDP), Service::Name("netspeak-cps")),
        (ServiceQuery(1446, Protocol::UDP), Service::Name("ora-lm")),
        (ServiceQuery(4406, Protocol::UDP), Service::Name("ds-slp")),
        (ServiceQuery(1366, Protocol::UDP), Service::Name("netware-csp")),
        (ServiceQuery(2012, Protocol::UDP), Service::Name("raid-ac")),
        (ServiceQuery(3346, Protocol::TCP), Service::Name("trnsprntproxy")),
        (ServiceQuery(6771, Protocol::UDP), Service::Name("plysrv-https")),
        (ServiceQuery(2977, Protocol::UDP), Service::Name("ttc-etap-ns")),
        (ServiceQuery(267, Protocol::UDP), Service::Name("td-service")),
        (ServiceQuery(2956, Protocol::TCP), Service::Name("ovrimosdbman")),
        (ServiceQuery(4309, Protocol::UDP), Service::Name("dserver")),
        (ServiceQuery(55, Protocol::UDP), Service::Name("isi-gl")),
        (ServiceQuery(4045, Protocol::TCP), Service::Name("lockd")),
        (ServiceQuery(2161, Protocol::UDP), Service::Name("apc-2161")),
        (ServiceQuery(4083, Protocol::UDP), Service::Name("lorica-out-sec")),
        (ServiceQuery(628, Protocol::UDP), Service::Name("qmqp")),
        (ServiceQuery(3732, Protocol::TCP), Service::Name("m-wnn")),
        (ServiceQuery(8472, Protocol::TCP), Service::Name("otv")),
        (ServiceQuery(2721, Protocol::UDP), Service::Name("smart-diagnose")),
        (ServiceQuery(4143, Protocol::TCP), Service::Name("oidsr")),
        (ServiceQuery(4606, Protocol::TCP), Service::Name("sixid")),
        (ServiceQuery(1575, Protocol::UDP), Service::Name("oraclenames")),
        (ServiceQuery(44323, Protocol::TCP), Service::Name("pmwebapi")),
        (ServiceQuery(6507, Protocol::TCP), Service::Name("bdir_priv")),
        (ServiceQuery(4015, Protocol::TCP), Service::Name("talarian-mcast1")),
        (ServiceQuery(11163, Protocol::TCP), Service::Name("suncacao-rmi")),
        (ServiceQuery(1739, Protocol::UDP), Service::Name("webaccess")),
        (ServiceQuery(22004, Protocol::TCP), Service::Name("optohost004")),
        (ServiceQuery(1586, Protocol::TCP), Service::Name("ibm-abtact")),
        (ServiceQuery(3185, Protocol::UDP), Service::Name("smpppd")),
        (ServiceQuery(2015, Protocol::TCP), Service::Name("cypress")),
        (ServiceQuery(4742, Protocol::UDP), Service::Name("sicct-sdp")),
        (ServiceQuery(3074, Protocol::UDP), Service::Name("xbox")),
        (ServiceQuery(6688, Protocol::TCP), Service::Name("clever-tcpip")),
        (ServiceQuery(1175, Protocol::UDP), Service::Name("dossier")),
        (ServiceQuery(2945, Protocol::UDP), Service::Name("h248-binary")),
        (ServiceQuery(2039, Protocol::TCP), Service::Name("prizma")),
        (ServiceQuery(4851, Protocol::TCP), Service::Name("derby-repli")),
        (ServiceQuery(11103, Protocol::TCP), Service::Name("origo-sync")),
        (ServiceQuery(3167, Protocol::UDP), Service::Name("nowcontact")),
        (ServiceQuery(5757, Protocol::UDP), Service::Name("x500ms")),
        (ServiceQuery(3296, Protocol::TCP), Service::Name("rib-slm")),
        (ServiceQuery(2403, Protocol::UDP), Service::Name("taskmaster2000")),
        (ServiceQuery(1370, Protocol::TCP), Service::Name("us-gv")),
        (ServiceQuery(2541, Protocol::TCP), Service::Name("lonworks2")),
        (ServiceQuery(31457, Protocol::TCP), Service::Name("tetrinet")),
        (ServiceQuery(3604, Protocol::TCP), Service::Name("bmc-jmx-port")),
        (ServiceQuery(2351, Protocol::TCP), Service::Name("psrserver")),
        (ServiceQuery(1452, Protocol::TCP), Service::Name("gtegsc-lm")),
        (ServiceQuery(3336, Protocol::TCP), Service::Name("directv-tick")),
        (ServiceQuery(2267, Protocol::TCP), Service::Name("ontobroker")),
        (ServiceQuery(8321, Protocol::TCP), Service::Name("tnp")),
        (ServiceQuery(360, Protocol::TCP), Service::Name("scoi2odialog")),
        (ServiceQuery(9991, Protocol::TCP), Service::Name("issa")),
        (ServiceQuery(2205, Protocol::UDP), Service::Name("jps")),
        (ServiceQuery(5605, Protocol::TCP), Service::Name("a4-sdunode")),
        (ServiceQuery(1593, Protocol::TCP), Service::Name("mainsoft-lm")),
        (ServiceQuery(3848, Protocol::UDP), Service::Name("item")),
        (ServiceQuery(5912, Protocol::UDP), Service::Name("fis")),
        (ServiceQuery(2583, Protocol::TCP), Service::Name("mon")),
        (ServiceQuery(5883, Protocol::TCP), Service::Name("jute")),
        (ServiceQuery(2807, Protocol::TCP), Service::Name("cspmulti")),
        (ServiceQuery(3981, Protocol::TCP), Service::Name("starfish")),
        (ServiceQuery(4126, Protocol::TCP), Service::Name("ddrepl")),
        (ServiceQuery(4377, Protocol::TCP), Service::Name("cp-spxsvr")),
        (ServiceQuery(3326, Protocol::UDP), Service::Name("sftu")),
        (ServiceQuery(3827, Protocol::UDP), Service::Name("netmpi")),
        (ServiceQuery(9107, Protocol::TCP), Service::Name("jetdirect")),
        (ServiceQuery(2762, Protocol::UDP), Service::Name("dicom-tls")),
        (ServiceQuery(373, Protocol::TCP), Service::Name("legent-1")),
        (ServiceQuery(7002, Protocol::UDP), Service::Name("afs3-prserver")),
        (ServiceQuery(16310, Protocol::UDP), Service::Name("pduncs")),
        (ServiceQuery(504, Protocol::UDP), Service::Name("citadel")),
        (ServiceQuery(11211, Protocol::UDP), Service::Name("memcache")),
        (ServiceQuery(21553, Protocol::TCP), Service::Name("rdm-tfs")),
        (ServiceQuery(1536, Protocol::UDP), Service::Name("ampr-inter")),
        (ServiceQuery(224, Protocol::TCP), Service::Name("masqdialer")),
        (ServiceQuery(1922, Protocol::TCP), Service::Name("tapestry")),
        (ServiceQuery(8004, Protocol::TCP), Service::Name("p2pevolvenet")),
        (ServiceQuery(2140, Protocol::UDP), Service::Name("ias-reg")),
        (ServiceQuery(2862, Protocol::TCP), Service::Name("ttg-protocol")),
        (ServiceQuery(6486, Protocol::UDP), Service::Name("sun-sr-iiops")),
        (ServiceQuery(3812, Protocol::UDP), Service::Name("neto-wol-server")),
        (ServiceQuery(158, Protocol::UDP), Service::Name("pcmail-srv")),
        (ServiceQuery(3478, Protocol::UDP), Service::Name("stun")),
        (ServiceQuery(3700, Protocol::UDP), Service::Name("lrs-paging")),
        (ServiceQuery(2459, Protocol::TCP), Service::Name("community")),
        (ServiceQuery(7338, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(6390, Protocol::TCP), Service::Name("metaedit-ws")),
        (ServiceQuery(7030, Protocol::UDP), Service::Name("op-probe")),
        (ServiceQuery(2358, Protocol::TCP), Service::Name("futrix")),
        (ServiceQuery(4262, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1485, Protocol::UDP), Service::Name("lansource")),
        (ServiceQuery(3132, Protocol::UDP), Service::Name("ms-rule-engine")),
        (ServiceQuery(5417, Protocol::TCP), Service::Name("sns-agent")),
        (ServiceQuery(3958, Protocol::UDP), Service::Name("mqe-agent")),
        (ServiceQuery(4124, Protocol::UDP), Service::Name("tigv2")),
        (ServiceQuery(38001, Protocol::TCP), Service::Name("ivs-insertion")),
        (ServiceQuery(1356, Protocol::UDP), Service::Name("cuillamartin")),
        (ServiceQuery(1089, Protocol::TCP), Service::Name("ff-annunc")),
        (ServiceQuery(1290, Protocol::UDP), Service::Name("winjaserver")),
        (ServiceQuery(2529, Protocol::TCP), Service::Name("utsftp")),
        (ServiceQuery(422, Protocol::TCP), Service::Name("ariel3")),
        (ServiceQuery(1499, Protocol::TCP), Service::Name("fhc")),
        (ServiceQuery(6511, Protocol::TCP), Service::Name("dccp-udp")),
        (ServiceQuery(32776, Protocol::UDP), Service::Name("sometimes-rpc16")),
        (ServiceQuery(244, Protocol::TCP), Service::Name("dayna")),
        (ServiceQuery(8001, Protocol::TCP), Service::Name("vcom-tunnel")),
        (ServiceQuery(115, Protocol::UDP), Service::Name("sftp")),
        (ServiceQuery(4590, Protocol::TCP), Service::Name("rid")),
        (ServiceQuery(4202, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(6252, Protocol::TCP), Service::Name("tl1-ssh")),
        (ServiceQuery(3003, Protocol::TCP), Service::Name("cgms")),
        (ServiceQuery(6514, Protocol::TCP), Service::Name("syslog-tls")),
        (ServiceQuery(27999, Protocol::TCP), Service::Name("tw-auth-key")),
        (ServiceQuery(2289, Protocol::TCP), Service::Name("dict-lookup")),
        (ServiceQuery(1046, Protocol::UDP), Service::Name("wfremotertm")),
        (ServiceQuery(3680, Protocol::UDP), Service::Name("npds-tracker")),
        (ServiceQuery(3355, Protocol::UDP), Service::Name("ordinox-dbase")),
        (ServiceQuery(14145, Protocol::TCP), Service::Name("gcm-app")),
        (ServiceQuery(3746, Protocol::TCP), Service::Name("linktest")),
        (ServiceQuery(5192, Protocol::UDP), Service::Name("aol-2")),
        (ServiceQuery(2217, Protocol::UDP), Service::Name("gotodevice")),
        (ServiceQuery(4144, Protocol::TCP), Service::Name("wincim")),
        (ServiceQuery(5029, Protocol::TCP), Service::Name("infobright")),
        (ServiceQuery(16789, Protocol::TCP), Service::Name("cadsisvr")),
        (ServiceQuery(1527, Protocol::TCP), Service::Name("tlisrv")),
        (ServiceQuery(3145, Protocol::TCP), Service::Name("csi-lfap")),
        (ServiceQuery(1073, Protocol::TCP), Service::Name("bridgecontrol")),
        (ServiceQuery(9162, Protocol::TCP), Service::Name("apani3")),
        (ServiceQuery(5194, Protocol::TCP), Service::Name("cpscomm")),
        (ServiceQuery(7402, Protocol::UDP), Service::Name("rtps-dd-mt")),
        (ServiceQuery(6678, Protocol::TCP), Service::Name("vfbp")),
        (ServiceQuery(32896, Protocol::TCP), Service::Name("idmgratm")),
        (ServiceQuery(9344, Protocol::UDP), Service::Name("mphlpdmc")),
        (ServiceQuery(2280, Protocol::UDP), Service::Name("lnvpoller")),
        (ServiceQuery(1305, Protocol::TCP), Service::Name("pe-mike")),
        (ServiceQuery(713, Protocol::UDP), Service::Name("iris-xpc")),
        (ServiceQuery(48002, Protocol::TCP), Service::Name("nimhub")),
        (ServiceQuery(9695, Protocol::UDP), Service::Name("ccnx")),
        (ServiceQuery(5465, Protocol::UDP), Service::Name("netops-broker")),
        (ServiceQuery(670, Protocol::UDP), Service::Name("vacdsm-sws")),
        (ServiceQuery(3121, Protocol::TCP), Service::Name("pcmk-remote")),
        (ServiceQuery(2421, Protocol::TCP), Service::Name("g-talk")),
        (ServiceQuery(10933, Protocol::TCP), Service::Name("octopustentacle")),
        (ServiceQuery(2715, Protocol::UDP), Service::Name("hpstgmgr2")),
        (ServiceQuery(4003, Protocol::UDP), Service::Name("pxc-splr-ft")),
        (ServiceQuery(3830, Protocol::TCP), Service::Name("cernsysmgmtagt")),
        (ServiceQuery(1367, Protocol::TCP), Service::Name("dcs")),
        (ServiceQuery(27, Protocol::TCP), Service::Name("nsw-fe")),
        (ServiceQuery(1250, Protocol::TCP), Service::Name("swldy-sias")),
        (ServiceQuery(2926, Protocol::UDP), Service::Name("mobile-file-dl")),
        (ServiceQuery(1125, Protocol::UDP), Service::Name("hpvmmagent")),
        (ServiceQuery(27002, Protocol::TCP), Service::Name("flexlm2")),
        (ServiceQuery(3160, Protocol::UDP), Service::Name("tip-app-server")),
        (ServiceQuery(854, Protocol::TCP), Service::Name("dlep")),
        (ServiceQuery(5674, Protocol::UDP), Service::Name("hyperscsi-port")),
        (ServiceQuery(310, Protocol::UDP), Service::Name("bhmds")),
        (ServiceQuery(4740, Protocol::UDP), Service::Name("ipfixs")),
        (ServiceQuery(1388, Protocol::TCP), Service::Name("objective-dbc")),
        (ServiceQuery(3071, Protocol::TCP), Service::Name("csd-mgmt-port")),
        (ServiceQuery(705, Protocol::TCP), Service::Name("agentx")),
        (ServiceQuery(22125, Protocol::TCP), Service::Name("dcap")),
        (ServiceQuery(3622, Protocol::TCP), Service::Name("ff-lr-port")),
        (ServiceQuery(257, Protocol::TCP), Service::Name("fw1-mc-fwmodule")),
        (ServiceQuery(3268, Protocol::TCP), Service::Name("globalcatLDAP")),
        (ServiceQuery(1162, Protocol::TCP), Service::Name("health-trap")),
        (ServiceQuery(132, Protocol::UDP), Service::Name("cisco-sys")),
        (ServiceQuery(19, Protocol::TCP), Service::Name("chargen")),
        (ServiceQuery(2618, Protocol::TCP), Service::Name("priority-e-com")),
        (ServiceQuery(1505, Protocol::UDP), Service::Name("funkproxy")),
        (ServiceQuery(4123, Protocol::TCP), Service::Name("z-wave")),
        (ServiceQuery(572, Protocol::TCP), Service::Name("sonar")),
        (ServiceQuery(3109, Protocol::UDP), Service::Name("personnel")),
        (ServiceQuery(1119, Protocol::TCP), Service::Name("bnetgame")),
        (ServiceQuery(3630, Protocol::TCP), Service::Name("cs-remote-db")),
        (ServiceQuery(5672, Protocol::UDP), Service::Name("amqp")),
        (ServiceQuery(1898, Protocol::UDP), Service::Name("cymtec-port")),
        (ServiceQuery(502, Protocol::UDP), Service::Name("mbap")),
        (ServiceQuery(3264, Protocol::UDP), Service::Name("ccmail")),
        (ServiceQuery(2457, Protocol::UDP), Service::Name("rapido-ip")),
        (ServiceQuery(603, Protocol::TCP), Service::Name("mnotes")),
        (ServiceQuery(5426, Protocol::UDP), Service::Name("devbasic")),
        (ServiceQuery(24577, Protocol::TCP), Service::Name("bilobit")),
        (ServiceQuery(2523, Protocol::UDP), Service::Name("qke-llc-v3")),
        (ServiceQuery(3490, Protocol::UDP), Service::Name("colubris")),
        (ServiceQuery(6417, Protocol::UDP), Service::Name("faxcomservice")),
        (ServiceQuery(2560, Protocol::UDP), Service::Name("labrat")),
        (ServiceQuery(280, Protocol::TCP), Service::Name("http-mgmt")),
        (ServiceQuery(4009, Protocol::UDP), Service::Name("chimera-hwm")),
        (ServiceQuery(3473, Protocol::TCP), Service::Name("jaugsremotec-2")),
        (ServiceQuery(8200, Protocol::UDP), Service::Name("trivnet1")),
        (ServiceQuery(1329, Protocol::UDP), Service::Name("netdb-export")),
        (ServiceQuery(3525, Protocol::UDP), Service::Name("eisport")),
        (ServiceQuery(22343, Protocol::UDP), Service::Name("cis-secure")),
        (ServiceQuery(7012, Protocol::UDP), Service::Name("talon-engine")),
        (ServiceQuery(1153, Protocol::UDP), Service::Name("c1222-acse")),
        (ServiceQuery(2858, Protocol::TCP), Service::Name("ecnp")),
        (ServiceQuery(2566, Protocol::TCP), Service::Name("pcs-pcw")),
        (ServiceQuery(18463, Protocol::TCP), Service::Name("ac-cluster")),
        (ServiceQuery(3372, Protocol::UDP), Service::Name("tip2")),
        (ServiceQuery(513, Protocol::UDP), Service::Name("who")),
        (ServiceQuery(48128, Protocol::TCP), Service::Name("isnetserv")),
        (ServiceQuery(3388, Protocol::TCP), Service::Name("cbserver")),
        (ServiceQuery(3331, Protocol::TCP), Service::Name("mcs-messaging")),
        (ServiceQuery(4430, Protocol::UDP), Service::Name("rsqlserver")),
        (ServiceQuery(9559, Protocol::TCP), Service::Name("p4runtime")),
        (ServiceQuery(1334, Protocol::UDP), Service::Name("writesrv")),
        (ServiceQuery(3218, Protocol::TCP), Service::Name("smartpackets")),
        (ServiceQuery(5500, Protocol::TCP), Service::Name("hotline")),
        (ServiceQuery(7014, Protocol::UDP), Service::Name("microtalon-com")),
        (ServiceQuery(2976, Protocol::UDP), Service::Name("cns-srv-port")),
        (ServiceQuery(4358, Protocol::UDP), Service::Name("qsnet-nucl")),
        (ServiceQuery(2714, Protocol::TCP), Service::Name("raventdm")),
        (ServiceQuery(3363, Protocol::TCP), Service::Name("nati-vi-server")),
        (ServiceQuery(2989, Protocol::UDP), Service::Name("zarkov")),
        (ServiceQuery(13823, Protocol::TCP), Service::Name("bmdss")),
        (ServiceQuery(4840, Protocol::UDP), Service::Name("opcua-udp")),
        (ServiceQuery(3124, Protocol::UDP), Service::Name("beacon-port")),
        (ServiceQuery(4786, Protocol::TCP), Service::Name("smart-install")),
        (ServiceQuery(1792, Protocol::TCP), Service::Name("ibm-dt-2")),
        (ServiceQuery(7004, Protocol::TCP), Service::Name("afs3-kaserver")),
        (ServiceQuery(1953, Protocol::TCP), Service::Name("rapidbase")),
        (ServiceQuery(3098, Protocol::TCP), Service::Name("umm-port")),
        (ServiceQuery(3977, Protocol::TCP), Service::Name("opswmanager")),
        (ServiceQuery(2931, Protocol::UDP), Service::Name("circle-x")),
        (ServiceQuery(22555, Protocol::TCP), Service::Name("vocaltec-wconf")),
        (ServiceQuery(3678, Protocol::TCP), Service::Name("ipr-dglt")),
        (ServiceQuery(6668, Protocol::TCP), Service::Name("irc")),
        (ServiceQuery(6026, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(1564, Protocol::UDP), Service::Name("pay-per-view")),
        (ServiceQuery(3396, Protocol::TCP), Service::Name("printer_agent")),
        (ServiceQuery(3738, Protocol::UDP), Service::Name("versatalk")),
        (ServiceQuery(3342, Protocol::UDP), Service::Name("webtie")),
        (ServiceQuery(1030, Protocol::UDP), Service::Name("iad1")),
        (ServiceQuery(1923, Protocol::UDP), Service::Name("spice")),
        (ServiceQuery(2206, Protocol::TCP), Service::Name("hpocbus")),
        (ServiceQuery(350, Protocol::TCP), Service::Name("matip-type-a")),
        (ServiceQuery(6030, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(4223, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1335, Protocol::UDP), Service::Name("digital-notary")),
        (ServiceQuery(1715, Protocol::UDP), Service::Name("houdini-lm")),
        (ServiceQuery(2237, Protocol::TCP), Service::Name("optech-port1-lm")),
        (ServiceQuery(2997, Protocol::UDP), Service::Name("rebol")),
        (ServiceQuery(4871, Protocol::TCP), Service::Name("wired")),
        (ServiceQuery(1055, Protocol::UDP), Service::Name("ansyslmd")),
        (ServiceQuery(1299, Protocol::UDP), Service::Name("hp-sci")),
        (ServiceQuery(31400, Protocol::TCP), Service::Name("pace-licensed")),
        (ServiceQuery(8888, Protocol::UDP), Service::Name("ddi-udp-1")),
        (ServiceQuery(4033, Protocol::UDP), Service::Name("sanavigator")),
        (ServiceQuery(2239, Protocol::UDP), Service::Name("imagequery")),
        (ServiceQuery(2892, Protocol::TCP), Service::Name("snifferdata")),
        (ServiceQuery(5034, Protocol::TCP), Service::Name("jtnetd-status")),
        (ServiceQuery(3636, Protocol::UDP), Service::Name("servistaitsm")),
        (ServiceQuery(1507, Protocol::TCP), Service::Name("symplex")),
        (ServiceQuery(1948, Protocol::UDP), Service::Name("eye2eye")),
        (ServiceQuery(3257, Protocol::TCP), Service::Name("cpqrpm-server")),
        (ServiceQuery(4187, Protocol::TCP), Service::Name("csc_proxy")),
        (ServiceQuery(1546, Protocol::TCP), Service::Name("abbaccuray")),
        (ServiceQuery(1427, Protocol::UDP), Service::Name("mloadd")),
        (ServiceQuery(20013, Protocol::TCP), Service::Name("ss-idi")),
        (ServiceQuery(31020, Protocol::TCP), Service::Name("autotrac-acp")),
        (ServiceQuery(5349, Protocol::TCP), Service::Name("stuns")),
        (ServiceQuery(13819, Protocol::TCP), Service::Name("dsmcc-session")),
        (ServiceQuery(3389, Protocol::UDP), Service::Name("ms-wbt-server")),
        (ServiceQuery(344, Protocol::TCP), Service::Name("pdap")),
        (ServiceQuery(6269, Protocol::UDP), Service::Name("grid-alt")),
        (ServiceQuery(1719, Protocol::UDP), Service::Name("h323gatestat")),
        (ServiceQuery(2734, Protocol::TCP), Service::Name("ccs-software")),
        (ServiceQuery(132, Protocol::TCP), Service::Name("cisco-sys")),
        (ServiceQuery(7932, Protocol::TCP), Service::Name("t2-drm")),
        (ServiceQuery(1056, Protocol::UDP), Service::Name("vfo")),
        (ServiceQuery(9889, Protocol::TCP), Service::Name("gt-proxy")),
        (ServiceQuery(1534, Protocol::UDP), Service::Name("micromuse-lm")),
        (ServiceQuery(9700, Protocol::UDP), Service::Name("board-roar")),
        (ServiceQuery(1904, Protocol::UDP), Service::Name("fjicl-tep-c")),
        (ServiceQuery(2324, Protocol::TCP), Service::Name("cosmocall")),
        (ServiceQuery(8243, Protocol::UDP), Service::Name("synapse-nhttps")),
        (ServiceQuery(2768, Protocol::TCP), Service::Name("uacs")),
        (ServiceQuery(1294, Protocol::TCP), Service::Name("cmmdriver")),
        (ServiceQuery(2085, Protocol::UDP), Service::Name("ada-cip")),
        (ServiceQuery(2593, Protocol::TCP), Service::Name("mns-mail")),
        (ServiceQuery(4420, Protocol::TCP), Service::Name("nvm-express")),
        (ServiceQuery(2570, Protocol::UDP), Service::Name("hs-port")),
        (ServiceQuery(7313, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(1306, Protocol::TCP), Service::Name("re-conn-proto")),
        (ServiceQuery(598, Protocol::TCP), Service::Name("sco-websrvrmg3")),
        (ServiceQuery(2990, Protocol::TCP), Service::Name("boscap")),
        (ServiceQuery(3211, Protocol::TCP), Service::Name("avsecuremgmt")),
        (ServiceQuery(457, Protocol::TCP), Service::Name("scohelp")),
        (ServiceQuery(7070, Protocol::UDP), Service::Name("arcp")),
        (ServiceQuery(3576, Protocol::UDP), Service::Name("cmc-port")),
        (ServiceQuery(2042, Protocol::TCP), Service::Name("isis")),
        (ServiceQuery(2513, Protocol::UDP), Service::Name("citrixadmin")),
        (ServiceQuery(2067, Protocol::UDP), Service::Name("dlswpn")),
        (ServiceQuery(1943, Protocol::UDP), Service::Name("beeyond-media")),
        (ServiceQuery(1610, Protocol::UDP), Service::Name("taurus-wh")),
        (ServiceQuery(1182, Protocol::TCP), Service::Name("accelenet")),
        (ServiceQuery(399, Protocol::UDP), Service::Name("iso-tsap-c2")),
        (ServiceQuery(16444, Protocol::TCP), Service::Name("overnet")),
        (ServiceQuery(10439, Protocol::TCP), Service::Name("bngsync")),
        (ServiceQuery(5688, Protocol::UDP), Service::Name("ggz")),
        (ServiceQuery(3497, Protocol::UDP), Service::Name("ipether232port")),
        (ServiceQuery(6143, Protocol::TCP), Service::Name("watershed-lm")),
        (ServiceQuery(1494, Protocol::TCP), Service::Name("citrix-ica")),
        (ServiceQuery(1193, Protocol::UDP), Service::Name("fiveacross")),
        (ServiceQuery(6679, Protocol::TCP), Service::Name("osaut")),
        (ServiceQuery(802, Protocol::TCP), Service::Name("mbap-s")),
        (ServiceQuery(1607, Protocol::TCP), Service::Name("stt")),
        (ServiceQuery(7169, Protocol::UDP), Service::Name("ccag-pib")),
        (ServiceQuery(5583, Protocol::TCP), Service::Name("tmo-icon-sync")),
        (ServiceQuery(1932, Protocol::TCP), Service::Name("ctt-broker")),
        (ServiceQuery(50, Protocol::UDP), Service::Name("re-mail-ck")),
        (ServiceQuery(9903, Protocol::TCP), Service::Name("multicast-ping")),
        (ServiceQuery(3257, Protocol::UDP), Service::Name("cpqrpm-server")),
        (ServiceQuery(2537, Protocol::UDP), Service::Name("upgrade")),
        (ServiceQuery(639, Protocol::TCP), Service::Name("msdp")),
        (ServiceQuery(12172, Protocol::TCP), Service::Name("hivep")),
        (ServiceQuery(5282, Protocol::UDP), Service::Name("transmit-port")),
        (ServiceQuery(4839, Protocol::TCP), Service::Name("varadero-2")),
        (ServiceQuery(3391, Protocol::TCP), Service::Name("savant")),
        (ServiceQuery(1598, Protocol::UDP), Service::Name("picknfs")),
        (ServiceQuery(2475, Protocol::UDP), Service::Name("ace-server")),
        (ServiceQuery(672, Protocol::TCP), Service::Name("vpps-qua")),
        (ServiceQuery(26262, Protocol::UDP), Service::Name("k3software-svr")),
        (ServiceQuery(1250, Protocol::UDP), Service::Name("swldy-sias")),
        (ServiceQuery(7744, Protocol::TCP), Service::Name("raqmon-pdu")),
        (ServiceQuery(9640, Protocol::TCP), Service::Name("pqsflows")),
        (ServiceQuery(1286, Protocol::UDP), Service::Name("netuitive")),
        (ServiceQuery(391, Protocol::UDP), Service::Name("synotics-relay")),
        (ServiceQuery(2240, Protocol::TCP), Service::Name("recipe")),
        (ServiceQuery(1863, Protocol::TCP), Service::Name("msnp")),
        (ServiceQuery(486, Protocol::UDP), Service::Name("avian")),
        (ServiceQuery(25007, Protocol::TCP), Service::Name("icl-twobase8")),
        (ServiceQuery(65, Protocol::TCP), Service::Name("tacacs-ds")),
        (ServiceQuery(2172, Protocol::TCP), Service::Name("msfw-s-storage")),
        (ServiceQuery(7351, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(4282, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1088, Protocol::UDP), Service::Name("cplscrambler-al")),
        (ServiceQuery(154, Protocol::UDP), Service::Name("netsc-prod")),
        (ServiceQuery(3181, Protocol::UDP), Service::Name("bmcpatrolagent")),
        (ServiceQuery(2914, Protocol::UDP), Service::Name("gamelobby")),
        (ServiceQuery(8379, Protocol::UDP), Service::Name("cruise-diags")),
        (ServiceQuery(8128, Protocol::TCP), Service::Name("paycash-online")),
        (ServiceQuery(4211, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(8123, Protocol::TCP), Service::Name("polipo")),
        (ServiceQuery(3953, Protocol::TCP), Service::Name("xmlink-connect")),
        (ServiceQuery(9997, Protocol::TCP), Service::Name("palace-6")),
        (ServiceQuery(2392, Protocol::UDP), Service::Name("tacticalauth")),
        (ServiceQuery(4346, Protocol::UDP), Service::Name("elanlm")),
        (ServiceQuery(6998, Protocol::UDP), Service::Name("iatp-highpri")),
        (ServiceQuery(8140, Protocol::TCP), Service::Name("puppet")),
        (ServiceQuery(24680, Protocol::UDP), Service::Name("tcc-http")),
        (ServiceQuery(546, Protocol::TCP), Service::Name("dhcpv6-client")),
        (ServiceQuery(1568, Protocol::TCP), Service::Name("tsspmap")),
        (ServiceQuery(1058, Protocol::TCP), Service::Name("nim")),
        (ServiceQuery(3722, Protocol::UDP), Service::Name("xserveraid")),
        (ServiceQuery(3539, Protocol::TCP), Service::Name("ibm-diradm-ssl")),
        (ServiceQuery(2857, Protocol::UDP), Service::Name("simctlp")),
        (ServiceQuery(3488, Protocol::TCP), Service::Name("fs-rh-srv")),
        (ServiceQuery(48001, Protocol::TCP), Service::Name("nimspooler")),
        (ServiceQuery(6046, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(1614, Protocol::UDP), Service::Name("netbill-cred")),
        (ServiceQuery(7997, Protocol::TCP), Service::Name("pushns")),
        (ServiceQuery(2145, Protocol::TCP), Service::Name("lv-pici")),
        (ServiceQuery(320, Protocol::UDP), Service::Name("ptp-general")),
        (ServiceQuery(4061, Protocol::UDP), Service::Name("ice-location")),
        (ServiceQuery(4157, Protocol::UDP), Service::Name("stat-scanner")),
        (ServiceQuery(10162, Protocol::TCP), Service::Name("snmptls-trap")),
        (ServiceQuery(3053, Protocol::UDP), Service::Name("dsom-server")),
        (ServiceQuery(1671, Protocol::TCP), Service::Name("netview-aix-11")),
        (ServiceQuery(8615, Protocol::TCP), Service::Name("imink")),
        (ServiceQuery(2523, Protocol::TCP), Service::Name("qke-llc-v3")),
        (ServiceQuery(1202, Protocol::TCP), Service::Name("caiccipc")),
        (ServiceQuery(5150, Protocol::UDP), Service::Name("atmp")),
        (ServiceQuery(4450, Protocol::UDP), Service::Name("camp")),
        (ServiceQuery(3047, Protocol::UDP), Service::Name("hlserver")),
        (ServiceQuery(8474, Protocol::TCP), Service::Name("noteshare")),
        (ServiceQuery(2406, Protocol::UDP), Service::Name("jediserver")),
        (ServiceQuery(3930, Protocol::TCP), Service::Name("syam-webserver")),
        (ServiceQuery(6013, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(9202, Protocol::TCP), Service::Name("wap-wsp-s")),
        (ServiceQuery(787, Protocol::TCP), Service::Name("qsc")),
        (ServiceQuery(3536, Protocol::UDP), Service::Name("snac")),
        (ServiceQuery(3270, Protocol::UDP), Service::Name("verismart")),
        (ServiceQuery(7095, Protocol::TCP), Service::Name("jdp-disc")),
        (ServiceQuery(548, Protocol::TCP), Service::Name("afp")),
        (ServiceQuery(1453, Protocol::TCP), Service::Name("genie-lm")),
        (ServiceQuery(9346, Protocol::TCP), Service::Name("ctechlicensing")),
        (ServiceQuery(2155, Protocol::UDP), Service::Name("brdptc")),
        (ServiceQuery(1048, Protocol::TCP), Service::Name("neod2")),
        (ServiceQuery(25902, Protocol::TCP), Service::Name("nilinkanalyst")),
        (ServiceQuery(1899, Protocol::TCP), Service::Name("mc2studios")),
        (ServiceQuery(5013, Protocol::UDP), Service::Name("fmpro-v6")),
        (ServiceQuery(2356, Protocol::TCP), Service::Name("gxtelmd")),
        (ServiceQuery(47809, Protocol::TCP), Service::Name("presonus-ucnet")),
        (ServiceQuery(4355, Protocol::TCP), Service::Name("qsnet-workst")),
        (ServiceQuery(13818, Protocol::TCP), Service::Name("dsmcc-config")),
        (ServiceQuery(2375, Protocol::TCP), Service::Name("docker")),
        (ServiceQuery(5150, Protocol::TCP), Service::Name("atmp")),
        (ServiceQuery(2386, Protocol::UDP), Service::Name("virtualtape")),
        (ServiceQuery(469, Protocol::UDP), Service::Name("rcp")),
        (ServiceQuery(5005, Protocol::UDP), Service::Name("avt-profile-2")),
        (ServiceQuery(26900, Protocol::UDP), Service::Name("hexen2")),
        (ServiceQuery(3009, Protocol::TCP), Service::Name("pxc-ntfy")),
        (ServiceQuery(1736, Protocol::TCP), Service::Name("street-stream")),
        (ServiceQuery(14154, Protocol::TCP), Service::Name("vad")),
        (ServiceQuery(2074, Protocol::UDP), Service::Name("vrtl-vmf-sa")),
        (ServiceQuery(1333, Protocol::UDP), Service::Name("passwrd-policy")),
        (ServiceQuery(2346, Protocol::TCP), Service::Name("redstorm_join")),
        (ServiceQuery(416, Protocol::TCP), Service::Name("silverplatter")),
        (ServiceQuery(9306, Protocol::TCP), Service::Name("sphinxql")),
        (ServiceQuery(5408, Protocol::UDP), Service::Name("foresyte-sec")),
        (ServiceQuery(7228, Protocol::TCP), Service::Name("citrixupp")),
        (ServiceQuery(2967, Protocol::UDP), Service::Name("symantec-av")),
        (ServiceQuery(1720, Protocol::TCP), Service::Name("h323q931")),
        (ServiceQuery(3065, Protocol::TCP), Service::Name("slinterbase")),
        (ServiceQuery(3884, Protocol::TCP), Service::Name("softrack-meter")),
        (ServiceQuery(4096, Protocol::TCP), Service::Name("bre")),
        (ServiceQuery(3935, Protocol::UDP), Service::Name("sdp-portmapper")),
        (ServiceQuery(3765, Protocol::UDP), Service::Name("rtraceroute")),
        (ServiceQuery(5031, Protocol::UDP), Service::Name("dmp")),
        (ServiceQuery(3243, Protocol::TCP), Service::Name("timelot")),
        (ServiceQuery(1557, Protocol::UDP), Service::Name("arbortext-lm")),
        (ServiceQuery(51, Protocol::TCP), Service::Name("la-maint")),
        (ServiceQuery(3256, Protocol::TCP), Service::Name("cpqrpm-agent")),
        (ServiceQuery(7801, Protocol::UDP), Service::Name("ssp-client")),
        (ServiceQuery(33, Protocol::UDP), Service::Name("dsp")),
        (ServiceQuery(558, Protocol::UDP), Service::Name("sdnskmp")),
        (ServiceQuery(9402, Protocol::TCP), Service::Name("sec-pc2fax-srv")),
        (ServiceQuery(13701, Protocol::TCP), Service::Name("netbackup")),
        (ServiceQuery(4138, Protocol::TCP), Service::Name("nettest")),
        (ServiceQuery(5989, Protocol::TCP), Service::Name("wbem-https")),
        (ServiceQuery(2845, Protocol::TCP), Service::Name("bpcp-trap")),
        (ServiceQuery(20167, Protocol::TCP), Service::Name("tolfab")),
        (ServiceQuery(1261, Protocol::TCP), Service::Name("mpshrsv")),
        (ServiceQuery(1487, Protocol::UDP), Service::Name("localinfosrvr")),
        (ServiceQuery(1742, Protocol::UDP), Service::Name("3Com-nsd")),
        (ServiceQuery(267, Protocol::TCP), Service::Name("td-service")),
        (ServiceQuery(2345, Protocol::TCP), Service::Name("dbm")),
        (ServiceQuery(1700, Protocol::TCP), Service::Name("mps-raft")),
        (ServiceQuery(2613, Protocol::UDP), Service::Name("smntubootstrap")),
        (ServiceQuery(2173, Protocol::UDP), Service::Name("msfw-replica")),
        (ServiceQuery(111, Protocol::UDP), Service::Name("rpcbind")),
        (ServiceQuery(8130, Protocol::UDP), Service::Name("indigo-vrmi")),
        (ServiceQuery(4900, Protocol::TCP), Service::Name("hfcs")),
        (ServiceQuery(402, Protocol::TCP), Service::Name("genie")),
        (ServiceQuery(485, Protocol::UDP), Service::Name("powerburst")),
        (ServiceQuery(1613, Protocol::TCP), Service::Name("netbill-keyrep")),
        (ServiceQuery(2466, Protocol::UDP), Service::Name("lbf")),
        (ServiceQuery(4942, Protocol::UDP), Service::Name("eq-office-4942")),
        (ServiceQuery(22002, Protocol::TCP), Service::Name("optohost002")),
        (ServiceQuery(3779, Protocol::TCP), Service::Name("cognima")),
        (ServiceQuery(9694, Protocol::UDP), Service::Name("client-wakeup")),
        (ServiceQuery(2059, Protocol::UDP), Service::Name("bmc-messaging")),
        (ServiceQuery(11109, Protocol::TCP), Service::Name("sgi-dmfmgr")),
        (ServiceQuery(635, Protocol::UDP), Service::Name("mount")),
        (ServiceQuery(3283, Protocol::UDP), Service::Name("netassistant")),
        (ServiceQuery(6113, Protocol::TCP), Service::Name("dayliteserver")),
        (ServiceQuery(2353, Protocol::UDP), Service::Name("pspserver")),
        (ServiceQuery(4279, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(5271, Protocol::TCP), Service::Name("cuelink")),
        (ServiceQuery(7902, Protocol::TCP), Service::Name("tnos-dp")),
        (ServiceQuery(9397, Protocol::UDP), Service::Name("mpidcagt")),
        (ServiceQuery(6025, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(6244, Protocol::TCP), Service::Name("jeol-nsdtp-4")),
        (ServiceQuery(3040, Protocol::TCP), Service::Name("tomato-springs")),
        (ServiceQuery(20000, Protocol::TCP), Service::Name("dnp")),
        (ServiceQuery(6568, Protocol::TCP), Service::Name("canit_store")),
        (ServiceQuery(2703, Protocol::TCP), Service::Name("sms-chat")),
        (ServiceQuery(2645, Protocol::TCP), Service::Name("novell-ipx-cmd")),
        (ServiceQuery(5814, Protocol::TCP), Service::Name("spt-automation")),
        (ServiceQuery(3242, Protocol::TCP), Service::Name("sdp-id-port")),
        (ServiceQuery(2258, Protocol::UDP), Service::Name("rcts")),
        (ServiceQuery(6027, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(3704, Protocol::TCP), Service::Name("adobeserver-4")),
        (ServiceQuery(3794, Protocol::TCP), Service::Name("jaus")),
        (ServiceQuery(6025, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(2883, Protocol::TCP), Service::Name("ndnp")),
        (ServiceQuery(73, Protocol::TCP), Service::Name("netrjs-3")),
        (ServiceQuery(7550, Protocol::TCP), Service::Name("cloudsignaling")),
        (ServiceQuery(6601, Protocol::TCP), Service::Name("mstmg-sstp")),
        (ServiceQuery(2281, Protocol::UDP), Service::Name("lnvconsole")),
        (ServiceQuery(5070, Protocol::UDP), Service::Name("vtsas")),
        (ServiceQuery(9200, Protocol::TCP), Service::Name("wap-wsp")),
        (ServiceQuery(54, Protocol::TCP), Service::Name("xns-ch")),
        (ServiceQuery(17500, Protocol::UDP), Service::Name("db-lsp-disc")),
        (ServiceQuery(1867, Protocol::TCP), Service::Name("udrive")),
        (ServiceQuery(3169, Protocol::TCP), Service::Name("serverview-as")),
        (ServiceQuery(4970, Protocol::TCP), Service::Name("ccss-qsm")),
        (ServiceQuery(3820, Protocol::TCP), Service::Name("scp")),
        (ServiceQuery(9668, Protocol::UDP), Service::Name("tec5-sdctp")),
        (ServiceQuery(6108, Protocol::TCP), Service::Name("sercomm-scadmin")),
        (ServiceQuery(8207, Protocol::TCP), Service::Name("lm-sserver")),
        (ServiceQuery(9598, Protocol::UDP), Service::Name("vscp")),
        (ServiceQuery(6980, Protocol::TCP), Service::Name("qolyester")),
        (ServiceQuery(3371, Protocol::UDP), Service::Name("satvid-datalnk")),
        (ServiceQuery(1620, Protocol::UDP), Service::Name("faxportwinport")),
        (ServiceQuery(1762, Protocol::UDP), Service::Name("cft-1")),
        (ServiceQuery(35354, Protocol::TCP), Service::Name("kitim")),
        (ServiceQuery(1146, Protocol::TCP), Service::Name("audit-transfer")),
        (ServiceQuery(3729, Protocol::TCP), Service::Name("fksp-audit")),
        (ServiceQuery(6149, Protocol::UDP), Service::Name("tal-pod")),
        (ServiceQuery(2878, Protocol::TCP), Service::Name("aap")),
        (ServiceQuery(3917, Protocol::UDP), Service::Name("aftmux")),
        (ServiceQuery(7171, Protocol::TCP), Service::Name("drm-production")),
        (ServiceQuery(1690, Protocol::UDP), Service::Name("ng-umds")),
        (ServiceQuery(9080, Protocol::UDP), Service::Name("glrpc")),
        (ServiceQuery(1159, Protocol::TCP), Service::Name("oracle-oms")),
        (ServiceQuery(2516, Protocol::TCP), Service::Name("maincontrol")),
        (ServiceQuery(3688, Protocol::UDP), Service::Name("simple-push-s")),
        (ServiceQuery(3571, Protocol::UDP), Service::Name("megardsvr-port")),
        (ServiceQuery(23546, Protocol::TCP), Service::Name("areaguard-neo")),
        (ServiceQuery(1980, Protocol::UDP), Service::Name("pearldoc-xact")),
        (ServiceQuery(1816, Protocol::UDP), Service::Name("harp")),
        (ServiceQuery(9295, Protocol::TCP), Service::Name("armcenterhttps")),
        (ServiceQuery(170, Protocol::UDP), Service::Name("print-srv")),
        (ServiceQuery(4006, Protocol::TCP), Service::Name("pxc-spvr")),
        (ServiceQuery(3306, Protocol::UDP), Service::Name("mysql")),
        (ServiceQuery(4373, Protocol::TCP), Service::Name("remctl")),
        (ServiceQuery(41794, Protocol::UDP), Service::Name("crestron-cip")),
        (ServiceQuery(7726, Protocol::UDP), Service::Name("freezexservice")),
        (ServiceQuery(2207, Protocol::UDP), Service::Name("hpssd")),
        (ServiceQuery(3649, Protocol::UDP), Service::Name("nmmp")),
        (ServiceQuery(2884, Protocol::UDP), Service::Name("flashmsg")),
        (ServiceQuery(5271, Protocol::UDP), Service::Name("cuelink-disc")),
        (ServiceQuery(263, Protocol::TCP), Service::Name("hdap")),
        (ServiceQuery(2376, Protocol::TCP), Service::Name("docker")),
        (ServiceQuery(3047, Protocol::TCP), Service::Name("hlserver")),
        (ServiceQuery(2203, Protocol::UDP), Service::Name("b2-runtime")),
        (ServiceQuery(5248, Protocol::TCP), Service::Name("caacws")),
        (ServiceQuery(744, Protocol::TCP), Service::Name("flexlm")),
        (ServiceQuery(1880, Protocol::UDP), Service::Name("vsat-control")),
        (ServiceQuery(544, Protocol::TCP), Service::Name("kshell")),
        (ServiceQuery(1213, Protocol::UDP), Service::Name("mpc-lifenet")),
        (ServiceQuery(4985, Protocol::TCP), Service::Name("gerhcs")),
        (ServiceQuery(30999, Protocol::UDP), Service::Name("ovobs")),
        (ServiceQuery(7349, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(2981, Protocol::TCP), Service::Name("mylxamport")),
        (ServiceQuery(3280, Protocol::TCP), Service::Name("vs-server")),
        (ServiceQuery(2346, Protocol::UDP), Service::Name("redstorm_join")),
        (ServiceQuery(31949, Protocol::UDP), Service::Name("iceedcp_rx")),
        (ServiceQuery(1438, Protocol::TCP), Service::Name("eicon-server")),
        (ServiceQuery(2832, Protocol::UDP), Service::Name("silkp4")),
        (ServiceQuery(1745, Protocol::TCP), Service::Name("remote-winsock")),
        (ServiceQuery(534, Protocol::UDP), Service::Name("mm-admin")),
        (ServiceQuery(1429, Protocol::UDP), Service::Name("nms")),
        (ServiceQuery(258, Protocol::TCP), Service::Name("fw1-mc-gui")),
        (ServiceQuery(9374, Protocol::TCP), Service::Name("fjdmimgr")),
        (ServiceQuery(2032, Protocol::TCP), Service::Name("blackboard")),
        (ServiceQuery(2570, Protocol::TCP), Service::Name("hs-port")),
        (ServiceQuery(3316, Protocol::TCP), Service::Name("aicc-cmi")),
        (ServiceQuery(11111, Protocol::TCP), Service::Name("vce")),
        (ServiceQuery(6370, Protocol::UDP), Service::Name("metaedit-se")),
        (ServiceQuery(3077, Protocol::TCP), Service::Name("orbix-loc-ssl")),
        (ServiceQuery(1421, Protocol::TCP), Service::Name("gandalf-lm")),
        (ServiceQuery(3317, Protocol::TCP), Service::Name("vsaiport")),
        (ServiceQuery(1726, Protocol::TCP), Service::Name("iberiagames")),
        (ServiceQuery(54, Protocol::UDP), Service::Name("xns-ch")),
        (ServiceQuery(9080, Protocol::TCP), Service::Name("glrpc")),
        (ServiceQuery(8087, Protocol::UDP), Service::Name("simplifymedia")),
        (ServiceQuery(5993, Protocol::TCP), Service::Name("cim-rs")),
        (ServiceQuery(7005, Protocol::TCP), Service::Name("afs3-volser")),
        (ServiceQuery(3896, Protocol::UDP), Service::Name("sdo-tls")),
        (ServiceQuery(2426, Protocol::TCP), Service::Name("vcmp")),
        (ServiceQuery(47000, Protocol::UDP), Service::Name("mbus")),
        (ServiceQuery(20014, Protocol::TCP), Service::Name("opendeploy")),
        (ServiceQuery(2017, Protocol::UDP), Service::Name("bootclient")),
        (ServiceQuery(2221, Protocol::TCP), Service::Name("rockwell-csp1")),
        (ServiceQuery(152, Protocol::TCP), Service::Name("bftp")),
        (ServiceQuery(1255, Protocol::TCP), Service::Name("de-cache-query")),
        (ServiceQuery(5044, Protocol::TCP), Service::Name("lxi-evntsvc")),
        (ServiceQuery(1144, Protocol::UDP), Service::Name("fuscript")),
        (ServiceQuery(6052, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(4801, Protocol::UDP), Service::Name("iwec")),
        (ServiceQuery(395, Protocol::UDP), Service::Name("netcp")),
        (ServiceQuery(1156, Protocol::UDP), Service::Name("iascontrol-oms")),
        (ServiceQuery(1829, Protocol::TCP), Service::Name("optika-emedia")),
        (ServiceQuery(1176, Protocol::UDP), Service::Name("indigo-server")),
        (ServiceQuery(2374, Protocol::TCP), Service::Name("hydra")),
        (ServiceQuery(4569, Protocol::UDP), Service::Name("iax")),
        (ServiceQuery(418, Protocol::UDP), Service::Name("hyper-g")),
        (ServiceQuery(3655, Protocol::UDP), Service::Name("abatemgr")),
        (ServiceQuery(4293, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(4404, Protocol::TCP), Service::Name("ds-admin")),
        (ServiceQuery(4074, Protocol::TCP), Service::Name("cequint-cityid")),
        (ServiceQuery(3684, Protocol::TCP), Service::Name("faxstfx-port")),
        (ServiceQuery(5677, Protocol::UDP), Service::Name("questdb2-lnchr")),
        (ServiceQuery(620, Protocol::TCP), Service::Name("sco-websrvrmgr")),
        (ServiceQuery(1249, Protocol::UDP), Service::Name("mesavistaco")),
        (ServiceQuery(4741, Protocol::TCP), Service::Name("lumimgrd")),
        (ServiceQuery(8442, Protocol::TCP), Service::Name("cybro-a-bus")),
        (ServiceQuery(512, Protocol::UDP), Service::Name("biff")),
        (ServiceQuery(456, Protocol::TCP), Service::Name("macon")),
        (ServiceQuery(5769, Protocol::TCP), Service::Name("spramsca")),
        (ServiceQuery(3144, Protocol::UDP), Service::Name("tarantella")),
        (ServiceQuery(2287, Protocol::TCP), Service::Name("dna")),
        (ServiceQuery(7174, Protocol::TCP), Service::Name("clutild")),
        (ServiceQuery(5233, Protocol::TCP), Service::Name("enfs")),
        (ServiceQuery(4267, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1635, Protocol::UDP), Service::Name("edb-server1")),
        (ServiceQuery(8402, Protocol::TCP), Service::Name("abarsd")),
        (ServiceQuery(2629, Protocol::TCP), Service::Name("sitaraserver")),
        (ServiceQuery(6509, Protocol::TCP), Service::Name("mgcs-mfp-port")),
        (ServiceQuery(3626, Protocol::UDP), Service::Name("bvcdaemon-port")),
        (ServiceQuery(3181, Protocol::TCP), Service::Name("bmcpatrolagent")),
        (ServiceQuery(43188, Protocol::TCP), Service::Name("reachout")),
        (ServiceQuery(2492, Protocol::UDP), Service::Name("groove")),
        (ServiceQuery(4150, Protocol::TCP), Service::Name("PowerAlert-nsa")),
        (ServiceQuery(1528, Protocol::TCP), Service::Name("mciautoreg")),
        (ServiceQuery(4740, Protocol::TCP), Service::Name("ipfixs")),
        (ServiceQuery(1181, Protocol::UDP), Service::Name("3comnetman")),
        (ServiceQuery(3969, Protocol::UDP), Service::Name("landmarks")),
        (ServiceQuery(31685, Protocol::TCP), Service::Name("dsx_monitor")),
        (ServiceQuery(2527, Protocol::UDP), Service::Name("iqserver")),
        (ServiceQuery(2347, Protocol::TCP), Service::Name("redstorm_find")),
        (ServiceQuery(1289, Protocol::TCP), Service::Name("jwalkserver")),
        (ServiceQuery(6672, Protocol::TCP), Service::Name("vision_server")),
        (ServiceQuery(4042, Protocol::TCP), Service::Name("ldxp")),
        (ServiceQuery(3511, Protocol::TCP), Service::Name("webmail-2")),
        (ServiceQuery(2487, Protocol::TCP), Service::Name("pns")),
        (ServiceQuery(4075, Protocol::UDP), Service::Name("perimlan")),
        (ServiceQuery(4270, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(32772, Protocol::TCP), Service::Name("sometimes-rpc7")),
        (ServiceQuery(2706, Protocol::TCP), Service::Name("ncdmirroring")),
        (ServiceQuery(5231, Protocol::TCP), Service::Name("crusecontrol")),
        (ServiceQuery(5357, Protocol::UDP), Service::Name("wsdapi")),
        (ServiceQuery(2241, Protocol::TCP), Service::Name("ivsd")),
        (ServiceQuery(2111, Protocol::TCP), Service::Name("kx")),
        (ServiceQuery(7430, Protocol::UDP), Service::Name("xmpv7")),
        (ServiceQuery(760, Protocol::UDP), Service::Name("ns")),
        (ServiceQuery(9191, Protocol::UDP), Service::Name("sun-as-jpda")),
        (ServiceQuery(4171, Protocol::TCP), Service::Name("ml-svnet")),
        (ServiceQuery(4181, Protocol::TCP), Service::Name("macbak")),
        (ServiceQuery(7333, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(1326, Protocol::TCP), Service::Name("wimsic")),
        (ServiceQuery(3889, Protocol::UDP), Service::Name("dandv-tester")),
        (ServiceQuery(2984, Protocol::UDP), Service::Name("hpidsadmin")),
        (ServiceQuery(1710, Protocol::TCP), Service::Name("impera")),
        (ServiceQuery(652, Protocol::TCP), Service::Name("hello-port")),
        (ServiceQuery(3682, Protocol::TCP), Service::Name("cas-mapi")),
        (ServiceQuery(6558, Protocol::UDP), Service::Name("xdsxdm")),
        (ServiceQuery(1883, Protocol::UDP), Service::Name("ibm-mqisdp")),
        (ServiceQuery(3572, Protocol::TCP), Service::Name("megaregsvrport")),
        (ServiceQuery(518, Protocol::UDP), Service::Name("ntalk")),
        (ServiceQuery(11208, Protocol::TCP), Service::Name("wifree")),
        (ServiceQuery(6360, Protocol::UDP), Service::Name("metaedit-mu")),
        (ServiceQuery(7777, Protocol::UDP), Service::Name("cbt")),
        (ServiceQuery(1896, Protocol::TCP), Service::Name("b-novative-ls")),
        (ServiceQuery(1224, Protocol::UDP), Service::Name("vpnz")),
        (ServiceQuery(5425, Protocol::UDP), Service::Name("br-channel")),
        (ServiceQuery(5450, Protocol::TCP), Service::Name("tiepie")),
        (ServiceQuery(1957, Protocol::UDP), Service::Name("unix-status")),
        (ServiceQuery(2224, Protocol::TCP), Service::Name("efi-mg")),
        (ServiceQuery(2562, Protocol::UDP), Service::Name("delibo")),
        (ServiceQuery(47557, Protocol::UDP), Service::Name("dbbrowse")),
        (ServiceQuery(3837, Protocol::TCP), Service::Name("mkm-discovery")),
        (ServiceQuery(781, Protocol::TCP), Service::Name("hp-collector")),
        (ServiceQuery(5046, Protocol::TCP), Service::Name("vpm-udp")),
        (ServiceQuery(19540, Protocol::TCP), Service::Name("sxuptp")),
        (ServiceQuery(549, Protocol::UDP), Service::Name("idfp")),
        (ServiceQuery(9208, Protocol::TCP), Service::Name("rjcdb-vcards")),
        (ServiceQuery(475, Protocol::TCP), Service::Name("tcpnethaspsrv")),
        (ServiceQuery(361, Protocol::TCP), Service::Name("semantix")),
        (ServiceQuery(2455, Protocol::UDP), Service::Name("wago-io-system")),
        (ServiceQuery(12753, Protocol::UDP), Service::Name("tsaf")),
        (ServiceQuery(49, Protocol::TCP), Service::Name("tacacs")),
        (ServiceQuery(6061, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(501, Protocol::UDP), Service::Name("stmf")),
        (ServiceQuery(27015, Protocol::UDP), Service::Name("halflife")),
        (ServiceQuery(3294, Protocol::UDP), Service::Name("fg-gip")),
        (ServiceQuery(4228, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(4682, Protocol::TCP), Service::Name("finisar")),
        (ServiceQuery(1269, Protocol::TCP), Service::Name("watilapp")),
        (ServiceQuery(4078, Protocol::TCP), Service::Name("cssp")),
        (ServiceQuery(155, Protocol::TCP), Service::Name("netsc-dev")),
        (ServiceQuery(7356, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(2286, Protocol::TCP), Service::Name("nas-metering")),
        (ServiceQuery(4845, Protocol::UDP), Service::Name("wcr-remlib")),
        (ServiceQuery(7317, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(597, Protocol::UDP), Service::Name("ptcnameservice")),
        (ServiceQuery(6548, Protocol::TCP), Service::Name("powerchuteplus")),
        (ServiceQuery(3961, Protocol::UDP), Service::Name("proaxess")),
        (ServiceQuery(22351, Protocol::TCP), Service::Name("codemeter-cmwan")),
        (ServiceQuery(12321, Protocol::UDP), Service::Name("warehouse-sss")),
        (ServiceQuery(4053, Protocol::TCP), Service::Name("ccu-comm-1")),
        (ServiceQuery(3826, Protocol::TCP), Service::Name("wormux")),
        (ServiceQuery(1072, Protocol::UDP), Service::Name("cardax")),
        (ServiceQuery(1925, Protocol::TCP), Service::Name("discovery-port")),
        (ServiceQuery(121, Protocol::TCP), Service::Name("erpc")),
        (ServiceQuery(4062, Protocol::TCP), Service::Name("ice-slocation")),
        (ServiceQuery(7323, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(3440, Protocol::UDP), Service::Name("ans-console")),
        (ServiceQuery(2717, Protocol::TCP), Service::Name("pn-requester")),
        (ServiceQuery(9152, Protocol::TCP), Service::Name("ms-sql2000")),
        (ServiceQuery(54320, Protocol::TCP), Service::Name("bo2k")),
        (ServiceQuery(4701, Protocol::TCP), Service::Name("netxms-mgmt")),
        (ServiceQuery(3496, Protocol::TCP), Service::Name("seclayer-tls")),
        (ServiceQuery(2468, Protocol::TCP), Service::Name("qip-msgd")),
        (ServiceQuery(30999, Protocol::TCP), Service::Name("ovobs")),
        (ServiceQuery(4154, Protocol::UDP), Service::Name("atlinks")),
        (ServiceQuery(2902, Protocol::UDP), Service::Name("netaspi")),
        (ServiceQuery(2292, Protocol::TCP), Service::Name("mib-streaming")),
        (ServiceQuery(3679, Protocol::UDP), Service::Name("newton-dock")),
        (ServiceQuery(3503, Protocol::UDP), Service::Name("lsp-ping")),
        (ServiceQuery(8786, Protocol::TCP), Service::Name("msgclnt")),
        (ServiceQuery(1081, Protocol::UDP), Service::Name("pvuniwien")),
        (ServiceQuery(567, Protocol::TCP), Service::Name("banyan-rpc")),
        (ServiceQuery(536, Protocol::UDP), Service::Name("opalis-rdv")),
        (ServiceQuery(2337, Protocol::UDP), Service::Name("ideesrv")),
        (ServiceQuery(106, Protocol::TCP), Service::Name("pop3pw")),
        (ServiceQuery(3923, Protocol::UDP), Service::Name("symb-sb-port")),
        (ServiceQuery(2322, Protocol::TCP), Service::Name("ofsd")),
        (ServiceQuery(1226, Protocol::TCP), Service::Name("stgxfws")),
        (ServiceQuery(7543, Protocol::UDP), Service::Name("atul")),
        (ServiceQuery(2767, Protocol::UDP), Service::Name("uadtc")),
        (ServiceQuery(3295, Protocol::UDP), Service::Name("dyniplookup")),
        (ServiceQuery(3333, Protocol::UDP), Service::Name("dec-notes")),
        (ServiceQuery(7015, Protocol::TCP), Service::Name("talon-webserver")),
        (ServiceQuery(4025, Protocol::UDP), Service::Name("partimage")),
        (ServiceQuery(4352, Protocol::TCP), Service::Name("pjlink")),
        (ServiceQuery(14034, Protocol::UDP), Service::Name("sage-best-com2")),
        (ServiceQuery(5638, Protocol::TCP), Service::Name("flcrs")),
        (ServiceQuery(7031, Protocol::TCP), Service::Name("iposplanet")),
        (ServiceQuery(2655, Protocol::TCP), Service::Name("unglue")),
        (ServiceQuery(7072, Protocol::TCP), Service::Name("iba-cfg")),
        (ServiceQuery(222, Protocol::UDP), Service::Name("rsh-spx")),
        (ServiceQuery(3687, Protocol::TCP), Service::Name("simple-push")),
        (ServiceQuery(3638, Protocol::UDP), Service::Name("ehp-backup")),
        (ServiceQuery(3069, Protocol::TCP), Service::Name("ls3")),
        (ServiceQuery(3052, Protocol::UDP), Service::Name("apc-3052")),
        (ServiceQuery(1750, Protocol::TCP), Service::Name("sslp")),
        (ServiceQuery(3809, Protocol::UDP), Service::Name("apocd")),
        (ServiceQuery(747, Protocol::UDP), Service::Name("fujitsu-dev")),
        (ServiceQuery(2478, Protocol::TCP), Service::Name("ssm-cssps")),
        (ServiceQuery(16993, Protocol::TCP), Service::Name("amt-soap-https")),
        (ServiceQuery(3005, Protocol::UDP), Service::Name("geniuslm")),
        (ServiceQuery(5914, Protocol::TCP), Service::Name("ipsdtls")),
        (ServiceQuery(2065, Protocol::UDP), Service::Name("dlsrpn")),
        (ServiceQuery(8034, Protocol::TCP), Service::Name("vantronix-mgmt")),
        (ServiceQuery(3041, Protocol::TCP), Service::Name("di-traceware")),
        (ServiceQuery(3937, Protocol::UDP), Service::Name("dvbservdsc")),
        (ServiceQuery(5688, Protocol::TCP), Service::Name("ggz")),
        (ServiceQuery(3006, Protocol::TCP), Service::Name("deslogind")),
        (ServiceQuery(27001, Protocol::UDP), Service::Name("flex-lm")),
        (ServiceQuery(5117, Protocol::TCP), Service::Name("gradecam")),
        (ServiceQuery(4005, Protocol::TCP), Service::Name("pxc-pin")),
        (ServiceQuery(3618, Protocol::UDP), Service::Name("aairnet-1")),
        (ServiceQuery(9213, Protocol::TCP), Service::Name("serverstart")),
        (ServiceQuery(170, Protocol::TCP), Service::Name("print-srv")),
        (ServiceQuery(6018, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(2251, Protocol::TCP), Service::Name("dif-port")),
        (ServiceQuery(3354, Protocol::UDP), Service::Name("suitjd")),
        (ServiceQuery(3519, Protocol::TCP), Service::Name("nvmsgd")),
        (ServiceQuery(4323, Protocol::UDP), Service::Name("trim-ice")),
        (ServiceQuery(109, Protocol::UDP), Service::Name("pop2")),
        (ServiceQuery(1731, Protocol::TCP), Service::Name("msiccp")),
        (ServiceQuery(2773, Protocol::TCP), Service::Name("rbakcup1")),
        (ServiceQuery(1904, Protocol::TCP), Service::Name("fjicl-tep-c")),
        (ServiceQuery(3597, Protocol::TCP), Service::Name("a14")),
        (ServiceQuery(4376, Protocol::TCP), Service::Name("bip")),
        (ServiceQuery(3832, Protocol::TCP), Service::Name("xxnetserver")),
        (ServiceQuery(24677, Protocol::TCP), Service::Name("flashfiler")),
        (ServiceQuery(1820, Protocol::TCP), Service::Name("mcagent")),
        (ServiceQuery(2117, Protocol::UDP), Service::Name("mentaclient")),
        (ServiceQuery(1700, Protocol::UDP), Service::Name("mps-raft")),
        (ServiceQuery(1651, Protocol::TCP), Service::Name("shiva_confsrvr")),
        (ServiceQuery(5047, Protocol::TCP), Service::Name("iscape")),
        (ServiceQuery(3679, Protocol::TCP), Service::Name("newton-dock")),
        (ServiceQuery(2432, Protocol::UDP), Service::Name("codasrv")),
        (ServiceQuery(853, Protocol::TCP), Service::Name("domain-s")),
        (ServiceQuery(654, Protocol::UDP), Service::Name("aodv")),
        (ServiceQuery(3996, Protocol::TCP), Service::Name("abcsoftware")),
        (ServiceQuery(3051, Protocol::TCP), Service::Name("galaxy-server")),
        (ServiceQuery(5411, Protocol::UDP), Service::Name("actnet")),
        (ServiceQuery(1467, Protocol::UDP), Service::Name("csdmbase")),
        (ServiceQuery(9598, Protocol::TCP), Service::Name("vscp")),
        (ServiceQuery(1724, Protocol::UDP), Service::Name("csbphone")),
        (ServiceQuery(2362, Protocol::UDP), Service::Name("digiman")),
        (ServiceQuery(3951, Protocol::TCP), Service::Name("pwgippfax")),
        (ServiceQuery(1822, Protocol::UDP), Service::Name("es-elmd")),
        (ServiceQuery(2089, Protocol::TCP), Service::Name("sep")),
        (ServiceQuery(6253, Protocol::TCP), Service::Name("crip")),
        (ServiceQuery(5024, Protocol::TCP), Service::Name("scpi-telnet")),
        (ServiceQuery(41, Protocol::UDP), Service::Name("graphics")),
        (ServiceQuery(1571, Protocol::TCP), Service::Name("rdb-dbs-disp")),
        (ServiceQuery(2246, Protocol::TCP), Service::Name("pc-mta-addrmap")),
        (ServiceQuery(3666, Protocol::UDP), Service::Name("eserver-pap")),
        (ServiceQuery(1963, Protocol::TCP), Service::Name("webmachine")),
        (ServiceQuery(1880, Protocol::TCP), Service::Name("vsat-control")),
        (ServiceQuery(4109, Protocol::UDP), Service::Name("izm")),
        (ServiceQuery(1123, Protocol::UDP), Service::Name("murray")),
        (ServiceQuery(9085, Protocol::UDP), Service::Name("ibm-rsyscon")),
        (ServiceQuery(1095, Protocol::TCP), Service::Name("nicelink")),
        (ServiceQuery(3364, Protocol::TCP), Service::Name("creativeserver")),
        (ServiceQuery(4150, Protocol::UDP), Service::Name("PowerAlert-nsa")),
        (ServiceQuery(1522, Protocol::UDP), Service::Name("rna-lm")),
        (ServiceQuery(3931, Protocol::TCP), Service::Name("msr-plugin-port")),
        (ServiceQuery(18769, Protocol::TCP), Service::Name("ique")),
        (ServiceQuery(6031, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(155, Protocol::UDP), Service::Name("netsc-dev")),
        (ServiceQuery(2669, Protocol::UDP), Service::Name("toad")),
        (ServiceQuery(4062, Protocol::UDP), Service::Name("ice-slocation")),
        (ServiceQuery(8600, Protocol::TCP), Service::Name("asterix")),
        (ServiceQuery(6446, Protocol::UDP), Service::Name("mysql-proxy")),
        (ServiceQuery(709, Protocol::TCP), Service::Name("entrustmanager")),
        (ServiceQuery(6006, Protocol::TCP), Service::Name("X11:6")),
        (ServiceQuery(6784, Protocol::TCP), Service::Name("bfd-lag")),
        (ServiceQuery(4323, Protocol::TCP), Service::Name("trim-ice")),
        (ServiceQuery(313, Protocol::UDP), Service::Name("magenta-logic")),
        (ServiceQuery(1237, Protocol::UDP), Service::Name("tsdos390")),
        (ServiceQuery(654, Protocol::TCP), Service::Name("aodv")),
        (ServiceQuery(1052, Protocol::TCP), Service::Name("ddt")),
        (ServiceQuery(2016, Protocol::UDP), Service::Name("bootserver")),
        (ServiceQuery(1439, Protocol::TCP), Service::Name("eicon-x25")),
        (ServiceQuery(3265, Protocol::TCP), Service::Name("altav-tunnel")),
        (ServiceQuery(4022, Protocol::UDP), Service::Name("dnox")),
        (ServiceQuery(3439, Protocol::TCP), Service::Name("hri-port")),
        (ServiceQuery(137, Protocol::UDP), Service::Name("netbios-ns")),
        (ServiceQuery(1138, Protocol::UDP), Service::Name("encrypted_admin")),
        (ServiceQuery(6009, Protocol::UDP), Service::Name("X11:9")),
        (ServiceQuery(1447, Protocol::UDP), Service::Name("apri-lm")),
        (ServiceQuery(7779, Protocol::TCP), Service::Name("vstat")),
        (ServiceQuery(2443, Protocol::UDP), Service::Name("powerclientcsf")),
        (ServiceQuery(5155, Protocol::TCP), Service::Name("asctrl-agent")),
        (ServiceQuery(7549, Protocol::TCP), Service::Name("nls-tl")),
        (ServiceQuery(1951, Protocol::TCP), Service::Name("bcs-lmserver")),
        (ServiceQuery(2786, Protocol::TCP), Service::Name("aic-oncrpc")),
        (ServiceQuery(1321, Protocol::UDP), Service::Name("pip")),
        (ServiceQuery(284, Protocol::UDP), Service::Name("corerjd")),
        (ServiceQuery(22222, Protocol::TCP), Service::Name("easyengine")),
        (ServiceQuery(7648, Protocol::UDP), Service::Name("cucme-1")),
        (ServiceQuery(6636, Protocol::TCP), Service::Name("mpls-udp-dtls")),
        (ServiceQuery(1315, Protocol::TCP), Service::Name("els")),
        (ServiceQuery(394, Protocol::UDP), Service::Name("embl-ndt")),
        (ServiceQuery(1999, Protocol::UDP), Service::Name("tcp-id-port")),
        (ServiceQuery(5474, Protocol::TCP), Service::Name("apsolab-rpc")),
        (ServiceQuery(4042, Protocol::UDP), Service::Name("ldxp")),
        (ServiceQuery(7123, Protocol::TCP), Service::Name("snif")),
        (ServiceQuery(39, Protocol::TCP), Service::Name("rlp")),
        (ServiceQuery(3775, Protocol::TCP), Service::Name("ispmmgr")),
        (ServiceQuery(1159, Protocol::UDP), Service::Name("oracle-oms")),
        (ServiceQuery(2807, Protocol::UDP), Service::Name("cspmulti")),
        (ServiceQuery(3171, Protocol::TCP), Service::Name("serverview-gf")),
        (ServiceQuery(6696, Protocol::TCP), Service::Name("babel")),
        (ServiceQuery(3316, Protocol::UDP), Service::Name("aicc-cmi")),
        (ServiceQuery(5573, Protocol::TCP), Service::Name("sdmmp")),
        (ServiceQuery(8184, Protocol::TCP), Service::Name("itach")),
        (ServiceQuery(7544, Protocol::TCP), Service::Name("nta-ds")),
        (ServiceQuery(6009, Protocol::TCP), Service::Name("X11:9")),
        (ServiceQuery(1545, Protocol::TCP), Service::Name("vistium-share")),
        (ServiceQuery(17219, Protocol::UDP), Service::Name("chipper")),
        (ServiceQuery(23333, Protocol::UDP), Service::Name("elxmgmt")),
        (ServiceQuery(1824, Protocol::TCP), Service::Name("metrics-pas")),
        (ServiceQuery(2364, Protocol::UDP), Service::Name("oi-2000")),
        (ServiceQuery(4209, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3531, Protocol::TCP), Service::Name("peerenabler")),
        (ServiceQuery(2315, Protocol::UDP), Service::Name("precise-sft")),
        (ServiceQuery(4164, Protocol::UDP), Service::Name("silverpeakcomm")),
        (ServiceQuery(2946, Protocol::UDP), Service::Name("fjsvmpor")),
        (ServiceQuery(6401, Protocol::TCP), Service::Name("crystalenterprise")),
        (ServiceQuery(3440, Protocol::TCP), Service::Name("ans-console")),
        (ServiceQuery(2102, Protocol::UDP), Service::Name("zephyr-srv")),
        (ServiceQuery(2200, Protocol::TCP), Service::Name("ici")),
        (ServiceQuery(32773, Protocol::TCP), Service::Name("sometimes-rpc9")),
        (ServiceQuery(2410, Protocol::UDP), Service::Name("vrts-registry")),
        (ServiceQuery(5567, Protocol::TCP), Service::Name("m-oap")),
        (ServiceQuery(16161, Protocol::UDP), Service::Name("sun-sea-port")),
        (ServiceQuery(3728, Protocol::UDP), Service::Name("e-woa")),
        (ServiceQuery(3325, Protocol::TCP), Service::Name("active-net")),
        (ServiceQuery(6026, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(7310, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(462, Protocol::UDP), Service::Name("datasurfsrvsec")),
        (ServiceQuery(2396, Protocol::UDP), Service::Name("wusage")),
        (ServiceQuery(4020, Protocol::UDP), Service::Name("trap")),
        (ServiceQuery(2463, Protocol::TCP), Service::Name("lsi-raid-mgmt")),
        (ServiceQuery(489, Protocol::TCP), Service::Name("nest-protocol")),
        (ServiceQuery(4012, Protocol::UDP), Service::Name("pda-gate")),
        (ServiceQuery(18888, Protocol::TCP), Service::Name("apc-necmp")),
        (ServiceQuery(2678, Protocol::UDP), Service::Name("gadgetgate2way")),
        (ServiceQuery(996, Protocol::TCP), Service::Name("xtreelic")),
        (ServiceQuery(2574, Protocol::UDP), Service::Name("blockade-bpsp")),
        (ServiceQuery(5361, Protocol::TCP), Service::Name("ms-s-sideshow")),
        (ServiceQuery(4884, Protocol::TCP), Service::Name("hivestor")),
        (ServiceQuery(1247, Protocol::TCP), Service::Name("visionpyramid")),
        (ServiceQuery(2555, Protocol::UDP), Service::Name("compaq-wcp")),
        (ServiceQuery(4663, Protocol::UDP), Service::Name("noteit")),
        (ServiceQuery(3979, Protocol::TCP), Service::Name("smwan")),
        (ServiceQuery(6881, Protocol::TCP), Service::Name("bittorrent-tracker")),
        (ServiceQuery(26486, Protocol::UDP), Service::Name("exoline-udp")),
        (ServiceQuery(2831, Protocol::TCP), Service::Name("silkp3")),
        (ServiceQuery(196, Protocol::TCP), Service::Name("dn6-smm-red")),
        (ServiceQuery(1894, Protocol::TCP), Service::Name("o2server-port")),
        (ServiceQuery(11000, Protocol::UDP), Service::Name("irisa")),
        (ServiceQuery(2112, Protocol::TCP), Service::Name("kip")),
        (ServiceQuery(480, Protocol::TCP), Service::Name("loadsrv")),
        (ServiceQuery(754, Protocol::TCP), Service::Name("krb_prop")),
        (ServiceQuery(2254, Protocol::TCP), Service::Name("seispoc")),
        (ServiceQuery(4282, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3238, Protocol::UDP), Service::Name("apparenet-as")),
        (ServiceQuery(1555, Protocol::TCP), Service::Name("livelan")),
        (ServiceQuery(5202, Protocol::TCP), Service::Name("targus-getdata2")),
        (ServiceQuery(4146, Protocol::TCP), Service::Name("tgcconnect")),
        (ServiceQuery(20034, Protocol::TCP), Service::Name("nburn_id")),
        (ServiceQuery(759, Protocol::UDP), Service::Name("con")),
        (ServiceQuery(3744, Protocol::UDP), Service::Name("sasg")),
        (ServiceQuery(16361, Protocol::UDP), Service::Name("netserialext2")),
        (ServiceQuery(4091, Protocol::UDP), Service::Name("ewinstaller")),
        (ServiceQuery(2212, Protocol::TCP), Service::Name("leecoposserver")),
        (ServiceQuery(1609, Protocol::UDP), Service::Name("isysg-lm")),
        (ServiceQuery(4754, Protocol::TCP), Service::Name("gre-in-udp")),
        (ServiceQuery(1617, Protocol::TCP), Service::Name("nimrod-agent")),
        (ServiceQuery(4704, Protocol::TCP), Service::Name("assuria-ins")),
        (ServiceQuery(2530, Protocol::TCP), Service::Name("vrcommerce")),
        (ServiceQuery(2044, Protocol::TCP), Service::Name("rimsl")),
        (ServiceQuery(5750, Protocol::TCP), Service::Name("rscd")),
        (ServiceQuery(2382, Protocol::UDP), Service::Name("ms-olap3")),
        (ServiceQuery(1673, Protocol::TCP), Service::Name("proshare-mc-1")),
        (ServiceQuery(6081, Protocol::TCP), Service::Name("geneve")),
        (ServiceQuery(5234, Protocol::TCP), Service::Name("eenet")),
        (ServiceQuery(2944, Protocol::UDP), Service::Name("megaco-h248")),
        (ServiceQuery(9762, Protocol::TCP), Service::Name("tungsten-http")),
        (ServiceQuery(3799, Protocol::TCP), Service::Name("radius-dynauth")),
        (ServiceQuery(1398, Protocol::UDP), Service::Name("video-activmail")),
        (ServiceQuery(1601, Protocol::UDP), Service::Name("aas")),
        (ServiceQuery(2874, Protocol::TCP), Service::Name("dxmessagebase1")),
        (ServiceQuery(2316, Protocol::TCP), Service::Name("sent-lm")),
        (ServiceQuery(3400, Protocol::UDP), Service::Name("csms2")),
        (ServiceQuery(24249, Protocol::UDP), Service::Name("vista-4gl")),
        (ServiceQuery(6251, Protocol::TCP), Service::Name("tl1-raw-ssl")),
        (ServiceQuery(2977, Protocol::TCP), Service::Name("ttc-etap-ns")),
        (ServiceQuery(3387, Protocol::TCP), Service::Name("backroomnet")),
        (ServiceQuery(9001, Protocol::UDP), Service::Name("etlservicemgr")),
        (ServiceQuery(7346, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(5471, Protocol::TCP), Service::Name("apsolab-cols")),
        (ServiceQuery(3188, Protocol::TCP), Service::Name("brcm-comm-port")),
        (ServiceQuery(3159, Protocol::UDP), Service::Name("navegaweb-port")),
        (ServiceQuery(2131, Protocol::TCP), Service::Name("avantageb2b")),
        (ServiceQuery(393, Protocol::UDP), Service::Name("dis")),
        (ServiceQuery(1693, Protocol::UDP), Service::Name("rrirtr")),
        (ServiceQuery(3727, Protocol::UDP), Service::Name("e-mdu")),
        (ServiceQuery(2835, Protocol::UDP), Service::Name("evtp-data")),
        (ServiceQuery(13706, Protocol::TCP), Service::Name("netbackup")),
        (ServiceQuery(4194, Protocol::TCP), Service::Name("spdm")),
        (ServiceQuery(2556, Protocol::TCP), Service::Name("nicetec-nmsvc")),
        (ServiceQuery(2494, Protocol::TCP), Service::Name("bmc-ar")),
        (ServiceQuery(9285, Protocol::TCP), Service::Name("n2h2server")),
        (ServiceQuery(4151, Protocol::UDP), Service::Name("menandmice_noh")),
        (ServiceQuery(7708, Protocol::TCP), Service::Name("scinet")),
        (ServiceQuery(394, Protocol::TCP), Service::Name("embl-ndt")),
        (ServiceQuery(1302, Protocol::TCP), Service::Name("ci3-software-2")),
        (ServiceQuery(711, Protocol::TCP), Service::Name("cisco-tdp")),
        (ServiceQuery(3704, Protocol::UDP), Service::Name("adobeserver-4")),
        (ServiceQuery(1171, Protocol::UDP), Service::Name("atc-appserver")),
        (ServiceQuery(34379, Protocol::UDP), Service::Name("p-net-remote")),
        (ServiceQuery(1131, Protocol::UDP), Service::Name("caspssl")),
        (ServiceQuery(358, Protocol::UDP), Service::Name("shrinkwrap")),
        (ServiceQuery(4291, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(4120, Protocol::TCP), Service::Name("minirem")),
        (ServiceQuery(2629, Protocol::UDP), Service::Name("sitaraserver")),
        (ServiceQuery(8083, Protocol::TCP), Service::Name("us-srv")),
        (ServiceQuery(7420, Protocol::TCP), Service::Name("ipluminary")),
        (ServiceQuery(1445, Protocol::UDP), Service::Name("proxima-lm")),
        (ServiceQuery(3927, Protocol::TCP), Service::Name("natdataservice")),
        (ServiceQuery(4755, Protocol::TCP), Service::Name("gre-udp-dtls")),
        (ServiceQuery(2255, Protocol::TCP), Service::Name("vrtp")),
        (ServiceQuery(6102, Protocol::TCP), Service::Name("synchronet-upd")),
        (ServiceQuery(159, Protocol::TCP), Service::Name("nss-routing")),
        (ServiceQuery(1546, Protocol::UDP), Service::Name("abbaccuray")),
        (ServiceQuery(901, Protocol::TCP), Service::Name("samba-swat")),
        (ServiceQuery(17234, Protocol::UDP), Service::Name("integrius-stp")),
        (ServiceQuery(1850, Protocol::TCP), Service::Name("gsi")),
        (ServiceQuery(2049, Protocol::UDP), Service::Name("nfs")),
        (ServiceQuery(264, Protocol::UDP), Service::Name("fw1-or-bgmp")),
        (ServiceQuery(3639, Protocol::UDP), Service::Name("xap-ha")),
        (ServiceQuery(5298, Protocol::TCP), Service::Name("presence")),
        (ServiceQuery(2410, Protocol::TCP), Service::Name("vrts-registry")),
        (ServiceQuery(158, Protocol::TCP), Service::Name("pcmail-srv")),
        (ServiceQuery(5302, Protocol::TCP), Service::Name("hacl-cfg")),
        (ServiceQuery(4292, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(10129, Protocol::TCP), Service::Name("bmc-gms")),
        (ServiceQuery(6014, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(3184, Protocol::UDP), Service::Name("apogeex-port")),
        (ServiceQuery(3605, Protocol::TCP), Service::Name("comcam-io")),
        (ServiceQuery(1750, Protocol::UDP), Service::Name("sslp")),
        (ServiceQuery(4002, Protocol::UDP), Service::Name("pxc-spvr-ft")),
        (ServiceQuery(21337, Protocol::TCP), Service::Name("xahaud")),
        (ServiceQuery(1853, Protocol::UDP), Service::Name("vids-avtp")),
        (ServiceQuery(2350, Protocol::UDP), Service::Name("psbserver")),
        (ServiceQuery(20046, Protocol::TCP), Service::Name("tmophl7mts")),
        (ServiceQuery(1170, Protocol::TCP), Service::Name("atc-lm")),
        (ServiceQuery(1165, Protocol::TCP), Service::Name("qsm-gui")),
        (ServiceQuery(22350, Protocol::UDP), Service::Name("CodeMeter")),
        (ServiceQuery(7747, Protocol::TCP), Service::Name("prgp")),
        (ServiceQuery(3700, Protocol::TCP), Service::Name("lrs-paging")),
        (ServiceQuery(2418, Protocol::UDP), Service::Name("cas")),
        (ServiceQuery(6697, Protocol::UDP), Service::Name("babel")),
        (ServiceQuery(4692, Protocol::TCP), Service::Name("conspiracy")),
        (ServiceQuery(2007, Protocol::TCP), Service::Name("dectalk")),
        (ServiceQuery(1225, Protocol::UDP), Service::Name("slinkysearch")),
        (ServiceQuery(17219, Protocol::TCP), Service::Name("chipper")),
        (ServiceQuery(1140, Protocol::TCP), Service::Name("autonoc")),
        (ServiceQuery(3018, Protocol::TCP), Service::Name("srvc_registry")),
        (ServiceQuery(9105, Protocol::TCP), Service::Name("jetdirect")),
        (ServiceQuery(6144, Protocol::UDP), Service::Name("statsci1-lm")),
        (ServiceQuery(390, Protocol::TCP), Service::Name("uis")),
        (ServiceQuery(549, Protocol::TCP), Service::Name("idfp")),
        (ServiceQuery(6999, Protocol::TCP), Service::Name("iatp-normalpri")),
        (ServiceQuery(3710, Protocol::TCP), Service::Name("portgate-auth")),
        (ServiceQuery(8292, Protocol::UDP), Service::Name("blp3")),
        (ServiceQuery(6045, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(1492, Protocol::UDP), Service::Name("stone-design-1")),
        (ServiceQuery(392, Protocol::TCP), Service::Name("synotics-broker")),
        (ServiceQuery(4004, Protocol::UDP), Service::Name("pxc-roid")),
        (ServiceQuery(1227, Protocol::UDP), Service::Name("dns2go")),
        (ServiceQuery(1777, Protocol::TCP), Service::Name("powerguardian")),
        (ServiceQuery(41121, Protocol::TCP), Service::Name("tentacle")),
        (ServiceQuery(3508, Protocol::UDP), Service::Name("interactionweb")),
        (ServiceQuery(3317, Protocol::UDP), Service::Name("vsaiport")),
        (ServiceQuery(2971, Protocol::UDP), Service::Name("netclip")),
        (ServiceQuery(2406, Protocol::TCP), Service::Name("jediserver")),
        (ServiceQuery(3766, Protocol::TCP), Service::Name("sitewatch-s")),
        (ServiceQuery(194, Protocol::TCP), Service::Name("irc")),
        (ServiceQuery(7720, Protocol::TCP), Service::Name("medimageportal")),
        (ServiceQuery(2701, Protocol::TCP), Service::Name("sms-rcinfo")),
        (ServiceQuery(3281, Protocol::TCP), Service::Name("sysopt")),
        (ServiceQuery(4084, Protocol::TCP), Service::Name("fortisphere-vm")),
        (ServiceQuery(3514, Protocol::TCP), Service::Name("must-p2p")),
        (ServiceQuery(4595, Protocol::UDP), Service::Name("ias-paging")),
        (ServiceQuery(7306, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(3377, Protocol::TCP), Service::Name("cogsys-lm")),
        (ServiceQuery(3503, Protocol::TCP), Service::Name("lsp-ping")),
        (ServiceQuery(456, Protocol::UDP), Service::Name("macon")),
        (ServiceQuery(4085, Protocol::TCP), Service::Name("ezmessagesrv")),
        (ServiceQuery(4235, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3360, Protocol::UDP), Service::Name("kv-server")),
        (ServiceQuery(2846, Protocol::TCP), Service::Name("aimpp-hello")),
        (ServiceQuery(68, Protocol::TCP), Service::Name("dhcpc")),
        (ServiceQuery(2509, Protocol::TCP), Service::Name("fjmpss")),
        (ServiceQuery(4849, Protocol::UDP), Service::Name("appserv-https")),
        (ServiceQuery(64, Protocol::UDP), Service::Name("covia")),
        (ServiceQuery(49400, Protocol::TCP), Service::Name("compaqdiag")),
        (ServiceQuery(32786, Protocol::UDP), Service::Name("sometimes-rpc26")),
        (ServiceQuery(6549, Protocol::TCP), Service::Name("apc-6549")),
        (ServiceQuery(6445, Protocol::TCP), Service::Name("sge_execd")),
        (ServiceQuery(647, Protocol::TCP), Service::Name("dhcp-failover")),
        (ServiceQuery(3101, Protocol::UDP), Service::Name("hp-pxpib")),
        (ServiceQuery(26489, Protocol::UDP), Service::Name("exonet")),
        (ServiceQuery(48004, Protocol::TCP), Service::Name("nimbusdb")),
        (ServiceQuery(8554, Protocol::UDP), Service::Name("rtsp-alt")),
        (ServiceQuery(90, Protocol::UDP), Service::Name("dnsix")),
        (ServiceQuery(4221, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(5061, Protocol::UDP), Service::Name("sip-tls")),
        (ServiceQuery(5721, Protocol::UDP), Service::Name("dtpt")),
        (ServiceQuery(38638, Protocol::TCP), Service::Name("psqlmws")),
        (ServiceQuery(7309, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(283, Protocol::TCP), Service::Name("rescap")),
        (ServiceQuery(38293, Protocol::UDP), Service::Name("landesk-cba")),
        (ServiceQuery(3062, Protocol::TCP), Service::Name("ncacn-ip-tcp")),
        (ServiceQuery(2413, Protocol::UDP), Service::Name("orion-rmi-reg")),
        (ServiceQuery(7328, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(1763, Protocol::UDP), Service::Name("cft-2")),
        (ServiceQuery(2325, Protocol::TCP), Service::Name("ansysli")),
        (ServiceQuery(4411, Protocol::TCP), Service::Name("found")),
        (ServiceQuery(6515, Protocol::UDP), Service::Name("elipse-rec")),
        (ServiceQuery(3789, Protocol::UDP), Service::Name("remotedeploy")),
        (ServiceQuery(3487, Protocol::TCP), Service::Name("ltctcp")),
        (ServiceQuery(20167, Protocol::UDP), Service::Name("tolfab")),
        (ServiceQuery(1045, Protocol::TCP), Service::Name("fpitp")),
        (ServiceQuery(9500, Protocol::TCP), Service::Name("ismserver")),
        (ServiceQuery(2664, Protocol::UDP), Service::Name("patrol-mq-gm")),
        (ServiceQuery(24678, Protocol::UDP), Service::Name("proactivate")),
        (ServiceQuery(2652, Protocol::TCP), Service::Name("interpathpanel")),
        (ServiceQuery(1629, Protocol::TCP), Service::Name("lontalk-urgnt")),
        (ServiceQuery(1638, Protocol::UDP), Service::Name("ismc")),
        (ServiceQuery(6106, Protocol::UDP), Service::Name("mpsserver")),
        (ServiceQuery(1096, Protocol::TCP), Service::Name("cnrprotocol")),
        (ServiceQuery(2163, Protocol::UDP), Service::Name("navisphere-sec")),
        (ServiceQuery(3320, Protocol::UDP), Service::Name("officelink2000")),
        (ServiceQuery(6056, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(16003, Protocol::UDP), Service::Name("alfin")),
        (ServiceQuery(6130, Protocol::TCP), Service::Name("damewaremobgtwy")),
        (ServiceQuery(4559, Protocol::TCP), Service::Name("hylafax")),
        (ServiceQuery(6300, Protocol::UDP), Service::Name("bmc-grx")),
        (ServiceQuery(7300, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(9593, Protocol::UDP), Service::Name("cba8")),
        (ServiceQuery(5272, Protocol::UDP), Service::Name("pk")),
        (ServiceQuery(16992, Protocol::TCP), Service::Name("amt-soap-http")),
        (ServiceQuery(9282, Protocol::TCP), Service::Name("swtp-port2")),
        (ServiceQuery(8444, Protocol::TCP), Service::Name("pcsync-http")),
        (ServiceQuery(1941, Protocol::TCP), Service::Name("dic-aida")),
        (ServiceQuery(1902, Protocol::UDP), Service::Name("fjicl-tep-b")),
        (ServiceQuery(4449, Protocol::TCP), Service::Name("privatewire")),
        (ServiceQuery(1088, Protocol::TCP), Service::Name("cplscrambler-al")),
        (ServiceQuery(1684, Protocol::TCP), Service::Name("snaresecure")),
        (ServiceQuery(175, Protocol::TCP), Service::Name("vmnet")),
        (ServiceQuery(14033, Protocol::TCP), Service::Name("sage-best-com1")),
        (ServiceQuery(8232, Protocol::TCP), Service::Name("hncp-dtls-port")),
        (ServiceQuery(2985, Protocol::UDP), Service::Name("hpidsagent")),
        (ServiceQuery(397, Protocol::UDP), Service::Name("mptn")),
        (ServiceQuery(3320, Protocol::TCP), Service::Name("officelink2000")),
        (ServiceQuery(3820, Protocol::UDP), Service::Name("scp")),
        (ServiceQuery(7402, Protocol::TCP), Service::Name("rtps-dd-mt")),
        (ServiceQuery(3365, Protocol::UDP), Service::Name("contentserver")),
        (ServiceQuery(27006, Protocol::TCP), Service::Name("flexlm6")),
        (ServiceQuery(1521, Protocol::TCP), Service::Name("oracle")),
        (ServiceQuery(3138, Protocol::TCP), Service::Name("rtnt-2")),
        (ServiceQuery(2096, Protocol::UDP), Service::Name("nbx-dir")),
        (ServiceQuery(2249, Protocol::TCP), Service::Name("rfmp")),
        (ServiceQuery(33123, Protocol::TCP), Service::Name("aurora-balaena")),
        (ServiceQuery(12322, Protocol::TCP), Service::Name("warehouse")),
        (ServiceQuery(1619, Protocol::TCP), Service::Name("xs-openstorage")),
        (ServiceQuery(11600, Protocol::UDP), Service::Name("tempest-port")),
        (ServiceQuery(8009, Protocol::TCP), Service::Name("ajp13")),
        (ServiceQuery(1140, Protocol::UDP), Service::Name("autonoc")),
        (ServiceQuery(2753, Protocol::TCP), Service::Name("de-spot")),
        (ServiceQuery(6022, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(550, Protocol::TCP), Service::Name("new-rwho")),
        (ServiceQuery(3983, Protocol::UDP), Service::Name("eisp")),
        (ServiceQuery(2335, Protocol::UDP), Service::Name("ace-proxy")),
        (ServiceQuery(2226, Protocol::TCP), Service::Name("di-drm")),
        (ServiceQuery(1903, Protocol::TCP), Service::Name("linkname")),
        (ServiceQuery(4019, Protocol::UDP), Service::Name("talarian-mcast5")),
        (ServiceQuery(25903, Protocol::TCP), Service::Name("niprobe")),
        (ServiceQuery(4359, Protocol::TCP), Service::Name("omabcastltkm")),
        (ServiceQuery(3046, Protocol::TCP), Service::Name("di-ase")),
        (ServiceQuery(2821, Protocol::UDP), Service::Name("vrts-at-port")),
        (ServiceQuery(6471, Protocol::TCP), Service::Name("lvision-lm")),
        (ServiceQuery(886, Protocol::UDP), Service::Name("iclcnet-locate")),
        (ServiceQuery(9966, Protocol::UDP), Service::Name("odnsp")),
        (ServiceQuery(2138, Protocol::TCP), Service::Name("unbind-cluster")),
        (ServiceQuery(1198, Protocol::UDP), Service::Name("cajo-discovery")),
        (ServiceQuery(5755, Protocol::TCP), Service::Name("openmailg")),
        (ServiceQuery(27910, Protocol::UDP), Service::Name("quake2")),
        (ServiceQuery(8433, Protocol::TCP), Service::Name("aws-as2")),
        (ServiceQuery(1873, Protocol::TCP), Service::Name("fjmpjps")),
        (ServiceQuery(9955, Protocol::TCP), Service::Name("alljoyn-stm")),
        (ServiceQuery(1594, Protocol::UDP), Service::Name("sixtrak")),
        (ServiceQuery(1898, Protocol::TCP), Service::Name("cymtec-port")),
        (ServiceQuery(1707, Protocol::UDP), Service::Name("vdmplay")),
        (ServiceQuery(9952, Protocol::UDP), Service::Name("apc-9952")),
        (ServiceQuery(9596, Protocol::TCP), Service::Name("mercury-disc")),
        (ServiceQuery(177, Protocol::TCP), Service::Name("xdmcp")),
        (ServiceQuery(2608, Protocol::UDP), Service::Name("wag-service")),
        (ServiceQuery(1115, Protocol::TCP), Service::Name("ardus-trns")),
        (ServiceQuery(2182, Protocol::TCP), Service::Name("cgn-stat")),
        (ServiceQuery(2724, Protocol::UDP), Service::Name("qotps")),
        (ServiceQuery(4743, Protocol::UDP), Service::Name("openhpid")),
        (ServiceQuery(1278, Protocol::UDP), Service::Name("dellwebadmin-1")),
        (ServiceQuery(3020, Protocol::UDP), Service::Name("cifs")),
        (ServiceQuery(1516, Protocol::TCP), Service::Name("vpad")),
        (ServiceQuery(447, Protocol::UDP), Service::Name("ddm-dfm")),
        (ServiceQuery(2548, Protocol::UDP), Service::Name("vytalvaultpipe")),
        (ServiceQuery(1386, Protocol::UDP), Service::Name("checksum")),
        (ServiceQuery(6053, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(1277, Protocol::TCP), Service::Name("miva-mqs")),
        (ServiceQuery(5200, Protocol::UDP), Service::Name("targus-getdata")),
        (ServiceQuery(4238, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1483, Protocol::TCP), Service::Name("afs")),
        (ServiceQuery(357, Protocol::UDP), Service::Name("bhevent")),
        (ServiceQuery(5043, Protocol::UDP), Service::Name("swxadmin")),
        (ServiceQuery(2430, Protocol::TCP), Service::Name("venus")),
        (ServiceQuery(9374, Protocol::UDP), Service::Name("fjdmimgr")),
        (ServiceQuery(4068, Protocol::TCP), Service::Name("ipfltbcst")),
        (ServiceQuery(190, Protocol::UDP), Service::Name("cacp")),
        (ServiceQuery(1869, Protocol::TCP), Service::Name("transact")),
        (ServiceQuery(42999, Protocol::TCP), Service::Name("curiosity")),
        (ServiceQuery(4500, Protocol::UDP), Service::Name("nat-t-ike")),
        (ServiceQuery(3870, Protocol::TCP), Service::Name("ovsam-d-agent")),
        (ServiceQuery(2549, Protocol::TCP), Service::Name("ipass")),
        (ServiceQuery(1294, Protocol::UDP), Service::Name("cmmdriver")),
        (ServiceQuery(5462, Protocol::TCP), Service::Name("ttl-publisher")),
        (ServiceQuery(7707, Protocol::UDP), Service::Name("sync-em7")),
        (ServiceQuery(3409, Protocol::TCP), Service::Name("networklens")),
        (ServiceQuery(544, Protocol::UDP), Service::Name("kshell")),
        (ServiceQuery(5247, Protocol::TCP), Service::Name("capwap-data")),
        (ServiceQuery(2302, Protocol::UDP), Service::Name("binderysupport")),
        (ServiceQuery(4870, Protocol::UDP), Service::Name("cc-tracking")),
        (ServiceQuery(6143, Protocol::UDP), Service::Name("watershed-lm")),
        (ServiceQuery(2991, Protocol::TCP), Service::Name("wkstn-mon")),
        (ServiceQuery(5585, Protocol::TCP), Service::Name("bis-sync")),
        (ServiceQuery(413, Protocol::UDP), Service::Name("smsp")),
        (ServiceQuery(444, Protocol::UDP), Service::Name("snpp")),
        (ServiceQuery(532, Protocol::UDP), Service::Name("netnews")),
        (ServiceQuery(1105, Protocol::TCP), Service::Name("ftranhc")),
        (ServiceQuery(7588, Protocol::UDP), Service::Name("sun-lm")),
        (ServiceQuery(5436, Protocol::TCP), Service::Name("pmip6-cntl")),
        (ServiceQuery(2469, Protocol::TCP), Service::Name("mti-tcs-comm")),
        (ServiceQuery(9060, Protocol::TCP), Service::Name("CardWeb-IO")),
        (ServiceQuery(1195, Protocol::UDP), Service::Name("rsf-1")),
        (ServiceQuery(9162, Protocol::UDP), Service::Name("apani3")),
        (ServiceQuery(3875, Protocol::TCP), Service::Name("pnbscada")),
        (ServiceQuery(2519, Protocol::UDP), Service::Name("globmsgsvc")),
        (ServiceQuery(8686, Protocol::TCP), Service::Name("sun-as-jmxrmi")),
        (ServiceQuery(5456, Protocol::TCP), Service::Name("apc-5456")),
        (ServiceQuery(3284, Protocol::TCP), Service::Name("4talk")),
        (ServiceQuery(453, Protocol::TCP), Service::Name("creativeserver")),
        (ServiceQuery(1407, Protocol::TCP), Service::Name("dbsa-lm")),
        (ServiceQuery(19999, Protocol::UDP), Service::Name("dnp-sec")),
        (ServiceQuery(3482, Protocol::UDP), Service::Name("vulture")),
        (ServiceQuery(3641, Protocol::TCP), Service::Name("netplay-port2")),
        (ServiceQuery(5410, Protocol::TCP), Service::Name("salient-usrmgr")),
        (ServiceQuery(3711, Protocol::UDP), Service::Name("edb-server2")),
        (ServiceQuery(1316, Protocol::TCP), Service::Name("exbit-escp")),
        (ServiceQuery(3807, Protocol::UDP), Service::Name("spugna")),
        (ServiceQuery(3750, Protocol::UDP), Service::Name("cbos-ip-port")),
        (ServiceQuery(2452, Protocol::TCP), Service::Name("snifferclient")),
        (ServiceQuery(5629, Protocol::UDP), Service::Name("symantec-sfdb")),
        (ServiceQuery(2808, Protocol::UDP), Service::Name("j-lan-p")),
        (ServiceQuery(9, Protocol::UDP), Service::Name("discard")),
        (ServiceQuery(11211, Protocol::TCP), Service::Name("memcache")),
        (ServiceQuery(5302, Protocol::UDP), Service::Name("hacl-cfg")),
        (ServiceQuery(3796, Protocol::TCP), Service::Name("spw-dialer")),
        (ServiceQuery(1755, Protocol::UDP), Service::Name("ms-streaming")),
        (ServiceQuery(4217, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1973, Protocol::TCP), Service::Name("dlsrap")),
        (ServiceQuery(7309, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(4301, Protocol::TCP), Service::Name("d-data")),
        (ServiceQuery(4075, Protocol::TCP), Service::Name("perimlan")),
        (ServiceQuery(3116, Protocol::UDP), Service::Name("mctet-gateway")),
        (ServiceQuery(2176, Protocol::TCP), Service::Name("rapi")),
        (ServiceQuery(1891, Protocol::UDP), Service::Name("childkey-notif")),
        (ServiceQuery(3236, Protocol::UDP), Service::Name("apparenet-ts")),
        (ServiceQuery(3474, Protocol::TCP), Service::Name("ttntspauto")),
        (ServiceQuery(2397, Protocol::UDP), Service::Name("ncl")),
        (ServiceQuery(3915, Protocol::TCP), Service::Name("agcat")),
        (ServiceQuery(1076, Protocol::TCP), Service::Name("sns_credit")),
        (ServiceQuery(323, Protocol::TCP), Service::Name("rpki-rtr")),
        (ServiceQuery(3785, Protocol::UDP), Service::Name("bfd-echo")),
        (ServiceQuery(2170, Protocol::UDP), Service::Name("eyetv")),
        (ServiceQuery(5718, Protocol::UDP), Service::Name("dpm")),
        (ServiceQuery(1944, Protocol::UDP), Service::Name("close-combat")),
        (ServiceQuery(4871, Protocol::UDP), Service::Name("wired")),
        (ServiceQuery(5682, Protocol::TCP), Service::Name("brightcore")),
        (ServiceQuery(2747, Protocol::TCP), Service::Name("fjippol-swrly")),
        (ServiceQuery(465, Protocol::UDP), Service::Name("smtps")),
        (ServiceQuery(3444, Protocol::UDP), Service::Name("denali-server")),
        (ServiceQuery(34963, Protocol::TCP), Service::Name("profinet-rtm")),
        (ServiceQuery(5, Protocol::UDP), Service::Name("rje")),
        (ServiceQuery(473, Protocol::TCP), Service::Name("hybrid-pop")),
        (ServiceQuery(4662, Protocol::TCP), Service::Name("edonkey")),
        (ServiceQuery(2008, Protocol::UDP), Service::Name("terminaldb")),
        (ServiceQuery(17222, Protocol::TCP), Service::Name("cpsp")),
        (ServiceQuery(43189, Protocol::TCP), Service::Name("ndm-agent-port")),
        (ServiceQuery(7473, Protocol::TCP), Service::Name("rise")),
        (ServiceQuery(2041, Protocol::TCP), Service::Name("interbase")),
        (ServiceQuery(7810, Protocol::UDP), Service::Name("rbt-wanopt")),
        (ServiceQuery(222, Protocol::TCP), Service::Name("rsh-spx")),
        (ServiceQuery(4028, Protocol::UDP), Service::Name("dtserver-port")),
        (ServiceQuery(3471, Protocol::UDP), Service::Name("jt400-ssl")),
        (ServiceQuery(13718, Protocol::TCP), Service::Name("netbackup")),
        (ServiceQuery(3383, Protocol::UDP), Service::Name("esp-lm")),
        (ServiceQuery(861, Protocol::UDP), Service::Name("owamp-control")),
        (ServiceQuery(1052, Protocol::UDP), Service::Name("ddt")),
        (ServiceQuery(451, Protocol::UDP), Service::Name("sfs-smp-net")),
        (ServiceQuery(1440, Protocol::TCP), Service::Name("eicon-slp")),
        (ServiceQuery(7023, Protocol::TCP), Service::Name("ct2nmcs")),
        (ServiceQuery(1231, Protocol::UDP), Service::Name("menandmice-lpm")),
        (ServiceQuery(4352, Protocol::UDP), Service::Name("pjlink")),
        (ServiceQuery(2587, Protocol::TCP), Service::Name("masc")),
        (ServiceQuery(201, Protocol::TCP), Service::Name("at-rtmp")),
        (ServiceQuery(2575, Protocol::UDP), Service::Name("hl7")),
        (ServiceQuery(3850, Protocol::TCP), Service::Name("qtms-bootstrap")),
        (ServiceQuery(1552, Protocol::UDP), Service::Name("pciarray")),
        (ServiceQuery(1210, Protocol::TCP), Service::Name("eoss")),
        (ServiceQuery(38472, Protocol::TCP), Service::Name("f1-control")),
        (ServiceQuery(3080, Protocol::UDP), Service::Name("stm_pproc")),
        (ServiceQuery(22347, Protocol::TCP), Service::Name("WibuKey")),
        (ServiceQuery(1776, Protocol::TCP), Service::Name("femis")),
        (ServiceQuery(2766, Protocol::TCP), Service::Name("listen")),
        (ServiceQuery(2415, Protocol::UDP), Service::Name("codima-rtp")),
        (ServiceQuery(1759, Protocol::UDP), Service::Name("spss-lm")),
        (ServiceQuery(27782, Protocol::TCP), Service::Name("ars-vista")),
        (ServiceQuery(4326, Protocol::TCP), Service::Name("geognosis")),
        (ServiceQuery(3825, Protocol::TCP), Service::Name("ffserver")),
        (ServiceQuery(5700, Protocol::TCP), Service::Name("supportassist")),
        (ServiceQuery(4245, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(5224, Protocol::UDP), Service::Name("hpvirtctrl")),
        (ServiceQuery(1343, Protocol::UDP), Service::Name("re101")),
        (ServiceQuery(833, Protocol::UDP), Service::Name("netconfsoapbeep")),
        (ServiceQuery(1280, Protocol::UDP), Service::Name("pictrography")),
        (ServiceQuery(7393, Protocol::UDP), Service::Name("nfoldman")),
        (ServiceQuery(2305, Protocol::UDP), Service::Name("mt-scaleserver")),
        (ServiceQuery(7395, Protocol::TCP), Service::Name("winqedit")),
        (ServiceQuery(6325, Protocol::TCP), Service::Name("dt-mgmtsvc")),
        (ServiceQuery(3541, Protocol::UDP), Service::Name("voispeed-port")),
        (ServiceQuery(692, Protocol::TCP), Service::Name("hyperwave-isp")),
        (ServiceQuery(1428, Protocol::TCP), Service::Name("informatik-lm")),
        (ServiceQuery(7101, Protocol::TCP), Service::Name("elcn")),
        (ServiceQuery(4108, Protocol::TCP), Service::Name("accel")),
        (ServiceQuery(4804, Protocol::TCP), Service::Name("aja-ntv4-disc")),
        (ServiceQuery(2383, Protocol::TCP), Service::Name("ms-olap4")),
        (ServiceQuery(1010, Protocol::TCP), Service::Name("surf")),
        (ServiceQuery(2775, Protocol::TCP), Service::Name("smpp")),
        (ServiceQuery(2924, Protocol::UDP), Service::Name("precise-vip")),
        (ServiceQuery(1691, Protocol::TCP), Service::Name("empire-empuma")),
        (ServiceQuery(5152, Protocol::TCP), Service::Name("sde-discovery")),
        (ServiceQuery(9595, Protocol::UDP), Service::Name("pds")),
        (ServiceQuery(6767, Protocol::UDP), Service::Name("bmc-perf-agent")),
        (ServiceQuery(10800, Protocol::UDP), Service::Name("gap")),
        (ServiceQuery(12012, Protocol::TCP), Service::Name("vipera")),
        (ServiceQuery(3922, Protocol::UDP), Service::Name("sor-update")),
        (ServiceQuery(4844, Protocol::TCP), Service::Name("gw-log")),
        (ServiceQuery(5005, Protocol::TCP), Service::Name("avt-profile-2")),
        (ServiceQuery(6047, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(4010, Protocol::TCP), Service::Name("samsung-unidex")),
        (ServiceQuery(1430, Protocol::UDP), Service::Name("tpdu")),
        (ServiceQuery(13929, Protocol::UDP), Service::Name("dta-systems")),
        (ServiceQuery(1190, Protocol::TCP), Service::Name("commlinx-avl")),
        (ServiceQuery(9104, Protocol::UDP), Service::Name("peerwire")),
        (ServiceQuery(3696, Protocol::UDP), Service::Name("telnetcpcd")),
        (ServiceQuery(1049, Protocol::TCP), Service::Name("td-postman")),
        (ServiceQuery(540, Protocol::TCP), Service::Name("uucp")),
        (ServiceQuery(78, Protocol::TCP), Service::Name("vettcp")),
        (ServiceQuery(3004, Protocol::TCP), Service::Name("csoftragent")),
        (ServiceQuery(1788, Protocol::UDP), Service::Name("psmond")),
        (ServiceQuery(5064, Protocol::TCP), Service::Name("ca-1")),
        (ServiceQuery(2397, Protocol::TCP), Service::Name("ncl")),
        (ServiceQuery(2263, Protocol::TCP), Service::Name("ecwcfg")),
        (ServiceQuery(1317, Protocol::UDP), Service::Name("vrts-ipcserver")),
        (ServiceQuery(6051, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(1620, Protocol::TCP), Service::Name("faxportwinport")),
        (ServiceQuery(4179, Protocol::TCP), Service::Name("MaxumSP")),
        (ServiceQuery(3014, Protocol::TCP), Service::Name("broker_service")),
        (ServiceQuery(1694, Protocol::TCP), Service::Name("rrimwm")),
        (ServiceQuery(7778, Protocol::UDP), Service::Name("interwise")),
        (ServiceQuery(751, Protocol::UDP), Service::Name("kadmin")),
        (ServiceQuery(448, Protocol::TCP), Service::Name("ddm-ssl")),
        (ServiceQuery(6640, Protocol::TCP), Service::Name("ovsdb")),
        (ServiceQuery(3975, Protocol::TCP), Service::Name("airshot")),
        (ServiceQuery(831, Protocol::TCP), Service::Name("netconf-beep")),
        (ServiceQuery(628, Protocol::TCP), Service::Name("qmqp")),
        (ServiceQuery(487, Protocol::TCP), Service::Name("saft")),
        (ServiceQuery(5629, Protocol::TCP), Service::Name("symantec-sfdb")),
        (ServiceQuery(2591, Protocol::UDP), Service::Name("maytagshuffle")),
        (ServiceQuery(7000, Protocol::UDP), Service::Name("afs3-fileserver")),
        (ServiceQuery(2704, Protocol::UDP), Service::Name("sms-remctrl")),
        (ServiceQuery(375, Protocol::TCP), Service::Name("hassle")),
        (ServiceQuery(2405, Protocol::TCP), Service::Name("trc-netpoll")),
        (ServiceQuery(21845, Protocol::TCP), Service::Name("webphone")),
        (ServiceQuery(5986, Protocol::TCP), Service::Name("wsmans")),
        (ServiceQuery(2146, Protocol::UDP), Service::Name("lv-not")),
        (ServiceQuery(3910, Protocol::TCP), Service::Name("prnrequest")),
        (ServiceQuery(1952, Protocol::UDP), Service::Name("mpnjsc")),
        (ServiceQuery(4750, Protocol::UDP), Service::Name("ssad")),
        (ServiceQuery(2167, Protocol::TCP), Service::Name("raw-serial")),
        (ServiceQuery(3855, Protocol::UDP), Service::Name("opentrac")),
        (ServiceQuery(1535, Protocol::UDP), Service::Name("ampr-info")),
        (ServiceQuery(2440, Protocol::TCP), Service::Name("spearway")),
        (ServiceQuery(2424, Protocol::TCP), Service::Name("kofax-svr")),
        (ServiceQuery(4745, Protocol::UDP), Service::Name("fmp")),
        (ServiceQuery(748, Protocol::UDP), Service::Name("ris-cm")),
        (ServiceQuery(7394, Protocol::TCP), Service::Name("fse")),
        (ServiceQuery(3732, Protocol::UDP), Service::Name("m-wnn")),
        (ServiceQuery(7938, Protocol::TCP), Service::Name("lgtomapper")),
        (ServiceQuery(7200, Protocol::UDP), Service::Name("fodms")),
        (ServiceQuery(1949, Protocol::TCP), Service::Name("ismaeasdaqlive")),
        (ServiceQuery(4882, Protocol::UDP), Service::Name("socp-c")),
        (ServiceQuery(2739, Protocol::UDP), Service::Name("tn-timing")),
        (ServiceQuery(3984, Protocol::UDP), Service::Name("mapper-nodemgr")),
        (ServiceQuery(5598, Protocol::UDP), Service::Name("mctfeed")),
        (ServiceQuery(4249, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(4351, Protocol::UDP), Service::Name("plcy-net-svcs")),
        (ServiceQuery(3332, Protocol::UDP), Service::Name("mcs-mailsvr")),
        (ServiceQuery(1150, Protocol::UDP), Service::Name("blaze")),
        (ServiceQuery(3400, Protocol::TCP), Service::Name("csms2")),
        (ServiceQuery(2983, Protocol::TCP), Service::Name("netplan")),
        (ServiceQuery(10051, Protocol::TCP), Service::Name("zabbix-trapper")),
        (ServiceQuery(7341, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(4156, Protocol::UDP), Service::Name("stat-results")),
        (ServiceQuery(6024, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(5081, Protocol::UDP), Service::Name("sdl-ets")),
        (ServiceQuery(4827, Protocol::UDP), Service::Name("squid-htcp")),
        (ServiceQuery(1162, Protocol::UDP), Service::Name("health-trap")),
        (ServiceQuery(31029, Protocol::TCP), Service::Name("yawn")),
        (ServiceQuery(4152, Protocol::TCP), Service::Name("idig_mux")),
        (ServiceQuery(1224, Protocol::TCP), Service::Name("vpnz")),
        (ServiceQuery(388, Protocol::UDP), Service::Name("unidata-ldm")),
        (ServiceQuery(4133, Protocol::UDP), Service::Name("nuts_bootp")),
        (ServiceQuery(3251, Protocol::UDP), Service::Name("sysscanner")),
        (ServiceQuery(1253, Protocol::UDP), Service::Name("q55-pcc")),
        (ServiceQuery(2223, Protocol::UDP), Service::Name("rockwell-csp2")),
        (ServiceQuery(381, Protocol::UDP), Service::Name("hp-collector")),
        (ServiceQuery(3362, Protocol::UDP), Service::Name("dj-ilm")),
        (ServiceQuery(5435, Protocol::UDP), Service::Name("sceanics")),
        (ServiceQuery(1263, Protocol::TCP), Service::Name("dka")),
        (ServiceQuery(2688, Protocol::UDP), Service::Name("md-cg-http")),
        (ServiceQuery(3448, Protocol::UDP), Service::Name("dnc-port")),
        (ServiceQuery(1480, Protocol::UDP), Service::Name("pacerforum")),
        (ServiceQuery(1739, Protocol::TCP), Service::Name("webaccess")),
        (ServiceQuery(6160, Protocol::TCP), Service::Name("ecmp")),
        (ServiceQuery(9387, Protocol::TCP), Service::Name("d2dconfig")),
        (ServiceQuery(4335, Protocol::TCP), Service::Name("netconf-ch-tls")),
        (ServiceQuery(151, Protocol::UDP), Service::Name("hems")),
        (ServiceQuery(3045, Protocol::UDP), Service::Name("responsenet")),
        (ServiceQuery(1608, Protocol::TCP), Service::Name("smart-lm")),
        (ServiceQuery(2665, Protocol::UDP), Service::Name("patrol-mq-nm")),
        (ServiceQuery(3290, Protocol::UDP), Service::Name("caps-lm")),
        (ServiceQuery(2610, Protocol::UDP), Service::Name("versa-tek")),
        (ServiceQuery(315, Protocol::UDP), Service::Name("dpsi")),
        (ServiceQuery(3577, Protocol::UDP), Service::Name("config-port")),
        (ServiceQuery(3187, Protocol::TCP), Service::Name("odi-port")),
        (ServiceQuery(6122, Protocol::TCP), Service::Name("bex-webadmin")),
        (ServiceQuery(3835, Protocol::UDP), Service::Name("spectardb")),
        (ServiceQuery(2649, Protocol::UDP), Service::Name("vpsipport")),
        (ServiceQuery(3228, Protocol::TCP), Service::Name("dwmsgserver")),
        (ServiceQuery(650, Protocol::UDP), Service::Name("bwnfs")),
        (ServiceQuery(2618, Protocol::UDP), Service::Name("priority-e-com")),
        (ServiceQuery(5265, Protocol::UDP), Service::Name("3com-njack-2")),
        (ServiceQuery(3236, Protocol::TCP), Service::Name("apparenet-ts")),
        (ServiceQuery(3319, Protocol::UDP), Service::Name("sdt-lmd")),
        (ServiceQuery(5470, Protocol::TCP), Service::Name("apsolab-col")),
        (ServiceQuery(4018, Protocol::UDP), Service::Name("talarian-mcast4")),
        (ServiceQuery(1330, Protocol::UDP), Service::Name("streetperfect")),
        (ServiceQuery(23005, Protocol::UDP), Service::Name("inovaport6")),
        (ServiceQuery(6020, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(5065, Protocol::TCP), Service::Name("ca-2")),
        (ServiceQuery(59, Protocol::UDP), Service::Name("priv-file")),
        (ServiceQuery(8500, Protocol::UDP), Service::Name("fmtp")),
        (ServiceQuery(368, Protocol::UDP), Service::Name("qbikgdp")),
        (ServiceQuery(9632, Protocol::TCP), Service::Name("mc-comm")),
        (ServiceQuery(4058, Protocol::TCP), Service::Name("kingfisher")),
        (ServiceQuery(9088, Protocol::UDP), Service::Name("sqlexec")),
        (ServiceQuery(82, Protocol::UDP), Service::Name("xfer")),
        (ServiceQuery(8711, Protocol::TCP), Service::Name("nvc")),
        (ServiceQuery(1355, Protocol::UDP), Service::Name("intuitive-edge")),
        (ServiceQuery(8873, Protocol::UDP), Service::Name("dxspider")),
        (ServiceQuery(10443, Protocol::TCP), Service::Name("cirrossp")),
        (ServiceQuery(1315, Protocol::UDP), Service::Name("els")),
        (ServiceQuery(174, Protocol::TCP), Service::Name("mailq")),
        (ServiceQuery(4265, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(4049, Protocol::UDP), Service::Name("wafs")),
        (ServiceQuery(2978, Protocol::UDP), Service::Name("ttc-etap-ds")),
        (ServiceQuery(2885, Protocol::TCP), Service::Name("topflow")),
        (ServiceQuery(1228, Protocol::UDP), Service::Name("florence")),
        (ServiceQuery(77, Protocol::UDP), Service::Name("priv-rje")),
        (ServiceQuery(270, Protocol::UDP), Service::Name("gist")),
        (ServiceQuery(18181, Protocol::TCP), Service::Name("opsec-cvp")),
        (ServiceQuery(8066, Protocol::TCP), Service::Name("toad-bi-appsrvr")),
        (ServiceQuery(1072, Protocol::TCP), Service::Name("cardax")),
        (ServiceQuery(6485, Protocol::UDP), Service::Name("sun-sr-iiop")),
        (ServiceQuery(3706, Protocol::UDP), Service::Name("rt-event")),
        (ServiceQuery(2014, Protocol::UDP), Service::Name("raid-sf")),
        (ServiceQuery(1293, Protocol::UDP), Service::Name("pkt-krb-ipsec")),
        (ServiceQuery(35006, Protocol::TCP), Service::Name("rt-helper")),
        (ServiceQuery(1433, Protocol::UDP), Service::Name("ms-sql-s")),
        (ServiceQuery(3220, Protocol::UDP), Service::Name("xnm-ssl")),
        (ServiceQuery(5689, Protocol::UDP), Service::Name("qmvideo")),
        (ServiceQuery(2852, Protocol::TCP), Service::Name("bears-01")),
        (ServiceQuery(3259, Protocol::UDP), Service::Name("epncdp2")),
        (ServiceQuery(3860, Protocol::UDP), Service::Name("sasp")),
        (ServiceQuery(10541, Protocol::UDP), Service::Name("MOS-upper")),
        (ServiceQuery(3616, Protocol::UDP), Service::Name("cd3o-protocol")),
        (ServiceQuery(21554, Protocol::TCP), Service::Name("dfserver")),
        (ServiceQuery(284, Protocol::TCP), Service::Name("corerjd")),
        (ServiceQuery(3253, Protocol::TCP), Service::Name("pda-data")),
        (ServiceQuery(381, Protocol::TCP), Service::Name("hp-collector")),
        (ServiceQuery(9021, Protocol::UDP), Service::Name("panagolin-ident")),
        (ServiceQuery(2089, Protocol::UDP), Service::Name("sep")),
        (ServiceQuery(1124, Protocol::TCP), Service::Name("hpvmmcontrol")),
        (ServiceQuery(7548, Protocol::TCP), Service::Name("tidp")),
        (ServiceQuery(1103, Protocol::UDP), Service::Name("adobeserver-2")),
        (ServiceQuery(4139, Protocol::TCP), Service::Name("thrtx")),
        (ServiceQuery(1142, Protocol::UDP), Service::Name("edtools")),
        (ServiceQuery(4442, Protocol::TCP), Service::Name("saris")),
        (ServiceQuery(5769, Protocol::UDP), Service::Name("spramsca")),
        (ServiceQuery(4353, Protocol::TCP), Service::Name("f5-iquery")),
        (ServiceQuery(6105, Protocol::UDP), Service::Name("primaserver")),
        (ServiceQuery(1823, Protocol::UDP), Service::Name("unisys-lm")),
        (ServiceQuery(7, Protocol::TCP), Service::Name("echo")),
        (ServiceQuery(697, Protocol::UDP), Service::Name("uuidgen")),
        (ServiceQuery(2503, Protocol::TCP), Service::Name("nms-dpnss")),
        (ServiceQuery(2090, Protocol::TCP), Service::Name("lrp")),
        (ServiceQuery(1129, Protocol::TCP), Service::Name("saphostctrls")),
        (ServiceQuery(5064, Protocol::UDP), Service::Name("ca-1")),
        (ServiceQuery(1376, Protocol::TCP), Service::Name("ibm-pps")),
        (ServiceQuery(22001, Protocol::UDP), Service::Name("optocontrol")),
        (ServiceQuery(9131, Protocol::TCP), Service::Name("dddp")),
        (ServiceQuery(38000, Protocol::TCP), Service::Name("ivs-database")),
        (ServiceQuery(3857, Protocol::UDP), Service::Name("trap-port")),
        (ServiceQuery(507, Protocol::TCP), Service::Name("crs")),
        (ServiceQuery(6021, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(1934, Protocol::TCP), Service::Name("xaapi")),
        (ServiceQuery(3522, Protocol::TCP), Service::Name("nssocketport")),
        (ServiceQuery(753, Protocol::TCP), Service::Name("rrh")),
        (ServiceQuery(4264, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(4684, Protocol::UDP), Service::Name("rfid-rp1")),
        (ServiceQuery(4210, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(141, Protocol::UDP), Service::Name("emfis-cntl")),
        (ServiceQuery(3941, Protocol::TCP), Service::Name("homeportal-web")),
        (ServiceQuery(5191, Protocol::UDP), Service::Name("aol-1")),
        (ServiceQuery(3988, Protocol::TCP), Service::Name("dcs-config")),
        (ServiceQuery(4951, Protocol::TCP), Service::Name("pwgwims")),
        (ServiceQuery(5102, Protocol::TCP), Service::Name("admeng")),
        (ServiceQuery(18182, Protocol::TCP), Service::Name("opsec-ufp")),
        (ServiceQuery(2231, Protocol::TCP), Service::Name("wimaxasncp")),
        (ServiceQuery(1590, Protocol::UDP), Service::Name("gemini-lm")),
        (ServiceQuery(3658, Protocol::UDP), Service::Name("ps-ams")),
        (ServiceQuery(3530, Protocol::UDP), Service::Name("gf")),
        (ServiceQuery(190, Protocol::TCP), Service::Name("gacp")),
        (ServiceQuery(31765, Protocol::UDP), Service::Name("gamesmith-port")),
        (ServiceQuery(433, Protocol::TCP), Service::Name("nnsp")),
        (ServiceQuery(3584, Protocol::TCP), Service::Name("u-dbap")),
        (ServiceQuery(2954, Protocol::UDP), Service::Name("ovalarmsrv-cmd")),
        (ServiceQuery(3446, Protocol::UDP), Service::Name("3comfaxrpc")),
        (ServiceQuery(1413, Protocol::TCP), Service::Name("innosys-acl")),
        (ServiceQuery(3168, Protocol::TCP), Service::Name("poweronnud")),
        (ServiceQuery(49, Protocol::UDP), Service::Name("tacacs")),
        (ServiceQuery(4728, Protocol::TCP), Service::Name("capmux")),
        (ServiceQuery(13713, Protocol::TCP), Service::Name("netbackup")),
        (ServiceQuery(1932, Protocol::UDP), Service::Name("ctt-broker")),
        (ServiceQuery(3149, Protocol::UDP), Service::Name("nm-game-server")),
        (ServiceQuery(2379, Protocol::TCP), Service::Name("etcd-client")),
        (ServiceQuery(5803, Protocol::TCP), Service::Name("vnc-http-3")),
        (ServiceQuery(32779, Protocol::TCP), Service::Name("sometimes-rpc21")),
        (ServiceQuery(2774, Protocol::UDP), Service::Name("rbakcup2")),
        (ServiceQuery(3522, Protocol::UDP), Service::Name("nssocketport")),
        (ServiceQuery(1731, Protocol::UDP), Service::Name("msiccp")),
        (ServiceQuery(4174, Protocol::TCP), Service::Name("smcluster")),
        (ServiceQuery(2520, Protocol::TCP), Service::Name("pvsw")),
        (ServiceQuery(3783, Protocol::TCP), Service::Name("bim-pem")),
        (ServiceQuery(4551, Protocol::TCP), Service::Name("ieee-mih")),
        (ServiceQuery(1434, Protocol::UDP), Service::Name("ms-sql-m")),
        (ServiceQuery(1287, Protocol::TCP), Service::Name("routematch")),
        (ServiceQuery(1579, Protocol::TCP), Service::Name("ioc-sea-lm")),
        (ServiceQuery(7117, Protocol::TCP), Service::Name("rothaga")),
        (ServiceQuery(8057, Protocol::TCP), Service::Name("senomix06")),
        (ServiceQuery(5226, Protocol::UDP), Service::Name("hp-status")),
        (ServiceQuery(495, Protocol::TCP), Service::Name("intecourier")),
        (ServiceQuery(3834, Protocol::UDP), Service::Name("spectardata")),
        (ServiceQuery(1070, Protocol::TCP), Service::Name("gmrupdateserv")),
        (ServiceQuery(6632, Protocol::TCP), Service::Name("mxodbc-connect")),
        (ServiceQuery(3802, Protocol::UDP), Service::Name("vhd")),
        (ServiceQuery(8077, Protocol::TCP), Service::Name("mles")),
        (ServiceQuery(404, Protocol::TCP), Service::Name("nced")),
        (ServiceQuery(2115, Protocol::TCP), Service::Name("kdm")),
        (ServiceQuery(2508, Protocol::TCP), Service::Name("jdatastore")),
        (ServiceQuery(2996, Protocol::TCP), Service::Name("vsixml")),
        (ServiceQuery(2573, Protocol::TCP), Service::Name("trustestablish")),
        (ServiceQuery(6350, Protocol::UDP), Service::Name("adap")),
        (ServiceQuery(2905, Protocol::TCP), Service::Name("m3ua")),
        (ServiceQuery(26133, Protocol::UDP), Service::Name("scscp")),
        (ServiceQuery(7021, Protocol::UDP), Service::Name("dpserveadmin")),
        (ServiceQuery(1284, Protocol::TCP), Service::Name("iee-qfx")),
        (ServiceQuery(3743, Protocol::TCP), Service::Name("ipcs-command")),
        (ServiceQuery(1947, Protocol::TCP), Service::Name("sentinelsrm")),
        (ServiceQuery(1254, Protocol::UDP), Service::Name("de-noc")),
        (ServiceQuery(2647, Protocol::TCP), Service::Name("syncserver")),
        (ServiceQuery(40841, Protocol::TCP), Service::Name("cscp")),
        (ServiceQuery(26486, Protocol::TCP), Service::Name("exoline-tcp")),
        (ServiceQuery(4785, Protocol::UDP), Service::Name("cncp")),
        (ServiceQuery(7629, Protocol::UDP), Service::Name("xdas")),
        (ServiceQuery(3526, Protocol::TCP), Service::Name("starquiz-port")),
        (ServiceQuery(7281, Protocol::UDP), Service::Name("itactionserver2")),
        (ServiceQuery(422, Protocol::UDP), Service::Name("ariel3")),
        (ServiceQuery(2584, Protocol::TCP), Service::Name("cyaserv")),
        (ServiceQuery(286, Protocol::TCP), Service::Name("fxp")),
        (ServiceQuery(4218, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2504, Protocol::TCP), Service::Name("wlbs")),
        (ServiceQuery(488, Protocol::UDP), Service::Name("gss-http")),
        (ServiceQuery(412, Protocol::UDP), Service::Name("synoptics-trap")),
        (ServiceQuery(7680, Protocol::UDP), Service::Name("pando-pub")),
        (ServiceQuery(7428, Protocol::UDP), Service::Name("ovladmgr")),
        (ServiceQuery(2895, Protocol::TCP), Service::Name("natuslink")),
        (ServiceQuery(660, Protocol::TCP), Service::Name("mac-srvr-admin")),
        (ServiceQuery(1525, Protocol::TCP), Service::Name("orasrv")),
        (ServiceQuery(418, Protocol::TCP), Service::Name("hyper-g")),
        (ServiceQuery(2006, Protocol::TCP), Service::Name("invokator")),
        (ServiceQuery(4256, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(10288, Protocol::TCP), Service::Name("blocks")),
        (ServiceQuery(4881, Protocol::TCP), Service::Name("socp-t")),
        (ServiceQuery(5055, Protocol::TCP), Service::Name("unot")),
        (ServiceQuery(1791, Protocol::UDP), Service::Name("ea1")),
        (ServiceQuery(2934, Protocol::TCP), Service::Name("4-tieropmcli")),
        (ServiceQuery(21212, Protocol::TCP), Service::Name("trinket-agent")),
        (ServiceQuery(16900, Protocol::UDP), Service::Name("newbay-snc-mc")),
        (ServiceQuery(6583, Protocol::UDP), Service::Name("joaJewelSuite")),
        (ServiceQuery(4739, Protocol::TCP), Service::Name("ipfix")),
        (ServiceQuery(5226, Protocol::TCP), Service::Name("hp-status")),
        (ServiceQuery(8610, Protocol::UDP), Service::Name("canon-mfnp")),
        (ServiceQuery(452, Protocol::UDP), Service::Name("sfs-config")),
        (ServiceQuery(524, Protocol::UDP), Service::Name("ncp")),
        (ServiceQuery(3567, Protocol::TCP), Service::Name("oap")),
        (ServiceQuery(1987, Protocol::TCP), Service::Name("tr-rsrb-p1")),
        (ServiceQuery(4849, Protocol::TCP), Service::Name("appserv-https")),
        (ServiceQuery(411, Protocol::UDP), Service::Name("rmt")),
        (ServiceQuery(3536, Protocol::TCP), Service::Name("snac")),
        (ServiceQuery(2948, Protocol::UDP), Service::Name("wap-push")),
        (ServiceQuery(1164, Protocol::UDP), Service::Name("qsm-proxy")),
        (ServiceQuery(7967, Protocol::TCP), Service::Name("supercell")),
        (ServiceQuery(3600, Protocol::TCP), Service::Name("trap-daemon")),
        (ServiceQuery(2580, Protocol::UDP), Service::Name("tributary")),
        (ServiceQuery(1049, Protocol::UDP), Service::Name("td-postman")),
        (ServiceQuery(8148, Protocol::TCP), Service::Name("isdd")),
        (ServiceQuery(34249, Protocol::TCP), Service::Name("turbonote-2")),
        (ServiceQuery(2254, Protocol::UDP), Service::Name("seispoc")),
        (ServiceQuery(4263, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(209, Protocol::UDP), Service::Name("tam")),
        (ServiceQuery(526, Protocol::TCP), Service::Name("tempo")),
        (ServiceQuery(206, Protocol::UDP), Service::Name("at-zis")),
        (ServiceQuery(1345, Protocol::UDP), Service::Name("vpjp")),
        (ServiceQuery(4456, Protocol::UDP), Service::Name("prchat-server")),
        (ServiceQuery(5906, Protocol::TCP), Service::Name("rpas-c2")),
        (ServiceQuery(2823, Protocol::UDP), Service::Name("cqg-netlan")),
        (ServiceQuery(1806, Protocol::UDP), Service::Name("musiconline")),
        (ServiceQuery(3009, Protocol::UDP), Service::Name("pxc-ntfy")),
        (ServiceQuery(5022, Protocol::UDP), Service::Name("mice")),
        (ServiceQuery(4558, Protocol::TCP), Service::Name("mtcevrunqman")),
        (ServiceQuery(4444, Protocol::TCP), Service::Name("krb524")),
        (ServiceQuery(5060, Protocol::TCP), Service::Name("sip")),
        (ServiceQuery(12109, Protocol::TCP), Service::Name("rets-ssl")),
        (ServiceQuery(3140, Protocol::UDP), Service::Name("ariliamulti")),
        (ServiceQuery(9020, Protocol::TCP), Service::Name("tambora")),
        (ServiceQuery(6500, Protocol::TCP), Service::Name("boks")),
        (ServiceQuery(3094, Protocol::TCP), Service::Name("rapidmq-reg")),
        (ServiceQuery(2124, Protocol::UDP), Service::Name("elatelink")),
        (ServiceQuery(176, Protocol::TCP), Service::Name("genrad-mux")),
        (ServiceQuery(63, Protocol::UDP), Service::Name("via-ftp")),
        (ServiceQuery(3262, Protocol::TCP), Service::Name("necp")),
        (ServiceQuery(622, Protocol::UDP), Service::Name("collaborator")),
        (ServiceQuery(6242, Protocol::TCP), Service::Name("jeol-nsdtp-2")),
        (ServiceQuery(3809, Protocol::TCP), Service::Name("apocd")),
        (ServiceQuery(153, Protocol::TCP), Service::Name("sgmp")),
        (ServiceQuery(2899, Protocol::UDP), Service::Name("powergemplus")),
        (ServiceQuery(3227, Protocol::TCP), Service::Name("dwnmshttp")),
        (ServiceQuery(3008, Protocol::TCP), Service::Name("midnight-tech")),
        (ServiceQuery(246, Protocol::UDP), Service::Name("dsp3270")),
        (ServiceQuery(2178, Protocol::TCP), Service::Name("bitspeer")),
        (ServiceQuery(4681, Protocol::TCP), Service::Name("parliant")),
        (ServiceQuery(2056, Protocol::UDP), Service::Name("omnisky")),
        (ServiceQuery(44818, Protocol::TCP), Service::Name("EtherNetIP-2")),
        (ServiceQuery(1666, Protocol::UDP), Service::Name("netview-aix-6")),
        (ServiceQuery(8415, Protocol::TCP), Service::Name("dlpx-sp")),
        (ServiceQuery(577, Protocol::TCP), Service::Name("vnas")),
        (ServiceQuery(77, Protocol::TCP), Service::Name("priv-rje")),
        (ServiceQuery(6019, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(695, Protocol::UDP), Service::Name("ieee-mms-ssl")),
        (ServiceQuery(6965, Protocol::TCP), Service::Name("swistrap")),
        (ServiceQuery(594, Protocol::TCP), Service::Name("tpip")),
        (ServiceQuery(1584, Protocol::TCP), Service::Name("tn-tl-fd2")),
        (ServiceQuery(3067, Protocol::TCP), Service::Name("fjhpjp")),
        (ServiceQuery(2815, Protocol::UDP), Service::Name("lbc-measure")),
        (ServiceQuery(5134, Protocol::TCP), Service::Name("ppactivation")),
        (ServiceQuery(4460, Protocol::TCP), Service::Name("ntske")),
        (ServiceQuery(1368, Protocol::TCP), Service::Name("screencast")),
        (ServiceQuery(1353, Protocol::UDP), Service::Name("relief")),
        (ServiceQuery(22, Protocol::UDP), Service::Name("ssh")),
        (ServiceQuery(1167, Protocol::UDP), Service::Name("cisco-ipsla")),
        (ServiceQuery(8401, Protocol::TCP), Service::Name("sabarsd")),
        (ServiceQuery(1464, Protocol::UDP), Service::Name("msl_lmd")),
        (ServiceQuery(3931, Protocol::UDP), Service::Name("msr-plugin-port")),
        (ServiceQuery(1325, Protocol::TCP), Service::Name("dx-instrument")),
        (ServiceQuery(527, Protocol::UDP), Service::Name("stx")),
        (ServiceQuery(3666, Protocol::TCP), Service::Name("eserver-pap")),
        (ServiceQuery(901, Protocol::UDP), Service::Name("smpnameres")),
        (ServiceQuery(3621, Protocol::UDP), Service::Name("ep-nsp")),
        (ServiceQuery(5055, Protocol::UDP), Service::Name("unot")),
        (ServiceQuery(245, Protocol::TCP), Service::Name("link")),
        (ServiceQuery(76, Protocol::UDP), Service::Name("deos")),
        (ServiceQuery(2926, Protocol::TCP), Service::Name("mobile-file-dl")),
        (ServiceQuery(551, Protocol::TCP), Service::Name("cybercash")),
        (ServiceQuery(1289, Protocol::UDP), Service::Name("jwalkserver")),
        (ServiceQuery(5082, Protocol::UDP), Service::Name("qcp")),
        (ServiceQuery(1375, Protocol::UDP), Service::Name("bytex")),
        (ServiceQuery(1267, Protocol::TCP), Service::Name("epc")),
        (ServiceQuery(3110, Protocol::TCP), Service::Name("sim-control")),
        (ServiceQuery(27003, Protocol::UDP), Service::Name("flex-lm")),
        (ServiceQuery(2877, Protocol::TCP), Service::Name("bluelance")),
        (ServiceQuery(3191, Protocol::TCP), Service::Name("csvr-sslproxy")),
        (ServiceQuery(2055, Protocol::UDP), Service::Name("iop")),
        (ServiceQuery(3119, Protocol::UDP), Service::Name("d2000kernel")),
        (ServiceQuery(1940, Protocol::TCP), Service::Name("jvclient")),
        (ServiceQuery(1300, Protocol::UDP), Service::Name("h323hostcallsc")),
        (ServiceQuery(4035, Protocol::TCP), Service::Name("wap-push-http")),
        (ServiceQuery(3520, Protocol::TCP), Service::Name("galileolog")),
        (ServiceQuery(3425, Protocol::TCP), Service::Name("agps-port")),
        (ServiceQuery(7162, Protocol::UDP), Service::Name("caistoragemgr")),
        (ServiceQuery(3194, Protocol::UDP), Service::Name("magbind")),
        (ServiceQuery(2896, Protocol::UDP), Service::Name("ecovisiong6-1")),
        (ServiceQuery(2536, Protocol::UDP), Service::Name("btpp2audctr1")),
        (ServiceQuery(4295, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(4305, Protocol::TCP), Service::Name("batman")),
        (ServiceQuery(430, Protocol::UDP), Service::Name("utmpsd")),
        (ServiceQuery(773, Protocol::UDP), Service::Name("notify")),
        (ServiceQuery(6714, Protocol::UDP), Service::Name("ibprotocol")),
        (ServiceQuery(3552, Protocol::UDP), Service::Name("taserver")),
        (ServiceQuery(2836, Protocol::TCP), Service::Name("catalyst")),
        (ServiceQuery(3070, Protocol::UDP), Service::Name("mgxswitch")),
        (ServiceQuery(37654, Protocol::TCP), Service::Name("unisys-eportal")),
        (ServiceQuery(2299, Protocol::UDP), Service::Name("pc-telecommute")),
        (ServiceQuery(204, Protocol::TCP), Service::Name("at-echo")),
        (ServiceQuery(3649, Protocol::TCP), Service::Name("nmmp")),
        (ServiceQuery(2295, Protocol::TCP), Service::Name("advant-lm")),
        (ServiceQuery(1777, Protocol::UDP), Service::Name("powerguardian")),
        (ServiceQuery(1424, Protocol::UDP), Service::Name("hybrid")),
        (ServiceQuery(4089, Protocol::TCP), Service::Name("opencore")),
        (ServiceQuery(191, Protocol::TCP), Service::Name("prospero")),
        (ServiceQuery(2525, Protocol::UDP), Service::Name("ms-v-worlds")),
        (ServiceQuery(4751, Protocol::TCP), Service::Name("spocp")),
        (ServiceQuery(2092, Protocol::TCP), Service::Name("descent3")),
        (ServiceQuery(1769, Protocol::TCP), Service::Name("bmc-net-adm")),
        (ServiceQuery(4678, Protocol::TCP), Service::Name("traversal")),
        (ServiceQuery(5014, Protocol::UDP), Service::Name("onpsocket")),
        (ServiceQuery(643, Protocol::UDP), Service::Name("sanity")),
        (ServiceQuery(5424, Protocol::UDP), Service::Name("beyond-remote")),
        (ServiceQuery(4242, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3974, Protocol::TCP), Service::Name("citysearch")),
        (ServiceQuery(4090, Protocol::UDP), Service::Name("omasgport")),
        (ServiceQuery(2155, Protocol::TCP), Service::Name("brdptc")),
        (ServiceQuery(3662, Protocol::TCP), Service::Name("pserver")),
        (ServiceQuery(6769, Protocol::TCP), Service::Name("adi-gxp-srvprt")),
        (ServiceQuery(5693, Protocol::TCP), Service::Name("rbsystem")),
        (ServiceQuery(471, Protocol::TCP), Service::Name("mondex")),
        (ServiceQuery(7349, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(4177, Protocol::TCP), Service::Name("wello")),
        (ServiceQuery(115, Protocol::TCP), Service::Name("sftp")),
        (ServiceQuery(2031, Protocol::TCP), Service::Name("mobrien-chat")),
        (ServiceQuery(3871, Protocol::TCP), Service::Name("avocent-adsap")),
        (ServiceQuery(2624, Protocol::UDP), Service::Name("aria")),
        (ServiceQuery(1283, Protocol::UDP), Service::Name("productinfo")),
        (ServiceQuery(24922, Protocol::TCP), Service::Name("find")),
        (ServiceQuery(4172, Protocol::TCP), Service::Name("pcoip")),
        (ServiceQuery(668, Protocol::TCP), Service::Name("mecomm")),
        (ServiceQuery(1993, Protocol::UDP), Service::Name("snmp-tcp-port")),
        (ServiceQuery(6653, Protocol::TCP), Service::Name("openflow")),
        (ServiceQuery(2149, Protocol::TCP), Service::Name("acptsys")),
        (ServiceQuery(3729, Protocol::UDP), Service::Name("fksp-audit")),
        (ServiceQuery(3370, Protocol::UDP), Service::Name("satvid-datalnk")),
        (ServiceQuery(24002, Protocol::UDP), Service::Name("med-fsp-tx")),
        (ServiceQuery(7020, Protocol::UDP), Service::Name("dpserve")),
        (ServiceQuery(2273, Protocol::UDP), Service::Name("mysql-im")),
        (ServiceQuery(5988, Protocol::TCP), Service::Name("wbem-http")),
        (ServiceQuery(3702, Protocol::UDP), Service::Name("ws-discovery")),
        (ServiceQuery(22537, Protocol::TCP), Service::Name("caldsoft-backup")),
        (ServiceQuery(2640, Protocol::TCP), Service::Name("sai_sentlm")),
        (ServiceQuery(3420, Protocol::UDP), Service::Name("ifcp-port")),
        (ServiceQuery(3641, Protocol::UDP), Service::Name("netplay-port2")),
        (ServiceQuery(7170, Protocol::UDP), Service::Name("nsrp")),
        (ServiceQuery(5413, Protocol::TCP), Service::Name("wwiotalk")),
        (ServiceQuery(1529, Protocol::TCP), Service::Name("support")),
        (ServiceQuery(3323, Protocol::UDP), Service::Name("active-net")),
        (ServiceQuery(16002, Protocol::TCP), Service::Name("gsms")),
        (ServiceQuery(1812, Protocol::TCP), Service::Name("radius")),
        (ServiceQuery(7005, Protocol::UDP), Service::Name("afs3-volser")),
        (ServiceQuery(658, Protocol::TCP), Service::Name("tenfold")),
        (ServiceQuery(2897, Protocol::TCP), Service::Name("citrix-rtmp")),
        (ServiceQuery(3423, Protocol::UDP), Service::Name("xtrm")),
        (ServiceQuery(3671, Protocol::UDP), Service::Name("efcp")),
        (ServiceQuery(1975, Protocol::UDP), Service::Name("tcoflashagent")),
        (ServiceQuery(2456, Protocol::TCP), Service::Name("altav-remmgt")),
        (ServiceQuery(2902, Protocol::TCP), Service::Name("netaspi")),
        (ServiceQuery(1047, Protocol::TCP), Service::Name("neod1")),
        (ServiceQuery(3757, Protocol::TCP), Service::Name("grf-port")),
        (ServiceQuery(1977, Protocol::TCP), Service::Name("tcoaddressbook")),
        (ServiceQuery(900, Protocol::UDP), Service::Name("omginitialrefs")),
        (ServiceQuery(1489, Protocol::TCP), Service::Name("dmdocbroker")),
        (ServiceQuery(11720, Protocol::TCP), Service::Name("h323callsigalt")),
        (ServiceQuery(317, Protocol::TCP), Service::Name("zannet")),
        (ServiceQuery(3529, Protocol::UDP), Service::Name("jboss-iiop-ssl")),
        (ServiceQuery(2154, Protocol::TCP), Service::Name("stdptc")),
        (ServiceQuery(601, Protocol::TCP), Service::Name("syslog-conn")),
        (ServiceQuery(5309, Protocol::UDP), Service::Name("jprinter")),
        (ServiceQuery(9616, Protocol::TCP), Service::Name("erunbook_agent")),
        (ServiceQuery(2365, Protocol::UDP), Service::Name("dbref")),
        (ServiceQuery(20670, Protocol::TCP), Service::Name("track")),
        (ServiceQuery(2798, Protocol::UDP), Service::Name("tmesis-upshot")),
        (ServiceQuery(1551, Protocol::UDP), Service::Name("hecmtl-db")),
        (ServiceQuery(3591, Protocol::UDP), Service::Name("gtrack-server")),
        (ServiceQuery(4672, Protocol::TCP), Service::Name("rfa")),
        (ServiceQuery(3987, Protocol::UDP), Service::Name("centerline")),
        (ServiceQuery(11108, Protocol::TCP), Service::Name("myq-termlink")),
        (ServiceQuery(3142, Protocol::UDP), Service::Name("rdc-wh-eos")),
        (ServiceQuery(2333, Protocol::UDP), Service::Name("snapp")),
        (ServiceQuery(10002, Protocol::TCP), Service::Name("documentum")),
        (ServiceQuery(3315, Protocol::UDP), Service::Name("cdid")),
        (ServiceQuery(8611, Protocol::TCP), Service::Name("canon-bjnp1")),
        (ServiceQuery(13708, Protocol::TCP), Service::Name("netbackup")),
        (ServiceQuery(8082, Protocol::UDP), Service::Name("us-cli")),
        (ServiceQuery(3725, Protocol::UDP), Service::Name("na-er-tip")),
        (ServiceQuery(2303, Protocol::TCP), Service::Name("proxy-gateway")),
        (ServiceQuery(3100, Protocol::UDP), Service::Name("opcon-xps")),
        (ServiceQuery(14936, Protocol::UDP), Service::Name("hde-lcesrvr-1")),
        (ServiceQuery(1640, Protocol::TCP), Service::Name("cert-responder")),
        (ServiceQuery(1627, Protocol::TCP), Service::Name("t128-gateway")),
        (ServiceQuery(4449, Protocol::UDP), Service::Name("privatewire")),
        (ServiceQuery(28017, Protocol::TCP), Service::Name("mongod")),
        (ServiceQuery(3992, Protocol::TCP), Service::Name("bv-ds")),
        (ServiceQuery(5419, Protocol::UDP), Service::Name("dj-ice")),
        (ServiceQuery(3926, Protocol::UDP), Service::Name("winport")),
        (ServiceQuery(5249, Protocol::UDP), Service::Name("caaclang2")),
        (ServiceQuery(15151, Protocol::TCP), Service::Name("bo2k")),
        (ServiceQuery(2430, Protocol::UDP), Service::Name("venus")),
        (ServiceQuery(3628, Protocol::UDP), Service::Name("ept-machine")),
        (ServiceQuery(1523, Protocol::UDP), Service::Name("cichild-lm")),
        (ServiceQuery(357, Protocol::TCP), Service::Name("bhevent")),
        (ServiceQuery(4168, Protocol::UDP), Service::Name("pslicser")),
        (ServiceQuery(8450, Protocol::UDP), Service::Name("npmp")),
        (ServiceQuery(10051, Protocol::UDP), Service::Name("zabbix-trapper")),
        (ServiceQuery(5723, Protocol::UDP), Service::Name("omhs")),
        (ServiceQuery(2074, Protocol::TCP), Service::Name("vrtl-vmf-sa")),
        (ServiceQuery(3480, Protocol::UDP), Service::Name("plethora")),
        (ServiceQuery(2740, Protocol::TCP), Service::Name("alarm")),
        (ServiceQuery(3156, Protocol::UDP), Service::Name("indura")),
        (ServiceQuery(27018, Protocol::TCP), Service::Name("mongod")),
        (ServiceQuery(4092, Protocol::TCP), Service::Name("ewdgs")),
        (ServiceQuery(2341, Protocol::TCP), Service::Name("xiostatus")),
        (ServiceQuery(7400, Protocol::TCP), Service::Name("rtps-discovery")),
        (ServiceQuery(3639, Protocol::TCP), Service::Name("xap-ha")),
        (ServiceQuery(1182, Protocol::UDP), Service::Name("accelenet-data")),
        (ServiceQuery(9051, Protocol::TCP), Service::Name("tor-control")),
        (ServiceQuery(125, Protocol::TCP), Service::Name("locus-map")),
        (ServiceQuery(1022, Protocol::TCP), Service::Name("exp2")),
        (ServiceQuery(9101, Protocol::UDP), Service::Name("bacula-dir")),
        (ServiceQuery(715, Protocol::TCP), Service::Name("iris-lwz")),
        (ServiceQuery(3253, Protocol::UDP), Service::Name("pda-data")),
        (ServiceQuery(3486, Protocol::TCP), Service::Name("ifsf-hb-port")),
        (ServiceQuery(61, Protocol::UDP), Service::Name("ni-mail")),
        (ServiceQuery(13722, Protocol::UDP), Service::Name("bpjava-msvc")),
        (ServiceQuery(5030, Protocol::UDP), Service::Name("surfpass")),
        (ServiceQuery(8432, Protocol::TCP), Service::Name("pgbackrest")),
        (ServiceQuery(2706, Protocol::UDP), Service::Name("ncdmirroring")),
        (ServiceQuery(3265, Protocol::UDP), Service::Name("altav-tunnel")),
        (ServiceQuery(9380, Protocol::TCP), Service::Name("boxp")),
        (ServiceQuery(2318, Protocol::TCP), Service::Name("cadencecontrol")),
        (ServiceQuery(9694, Protocol::TCP), Service::Name("client-wakeup")),
        (ServiceQuery(3905, Protocol::UDP), Service::Name("mupdate")),
        (ServiceQuery(3039, Protocol::TCP), Service::Name("cogitate")),
        (ServiceQuery(4148, Protocol::UDP), Service::Name("hhb-handheld")),
        (ServiceQuery(177, Protocol::UDP), Service::Name("xdmcp")),
        (ServiceQuery(8610, Protocol::TCP), Service::Name("canon-mfnp")),
        (ServiceQuery(4273, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(6243, Protocol::TCP), Service::Name("jeol-nsdtp-3")),
        (ServiceQuery(9106, Protocol::TCP), Service::Name("jetdirect")),
        (ServiceQuery(1792, Protocol::UDP), Service::Name("ibm-dt-2")),
        (ServiceQuery(1497, Protocol::UDP), Service::Name("rfx-lm")),
        (ServiceQuery(4937, Protocol::UDP), Service::Name("atsc-mh-ssc")),
        (ServiceQuery(2175, Protocol::TCP), Service::Name("airsync")),
        (ServiceQuery(1751, Protocol::TCP), Service::Name("swiftnet")),
        (ServiceQuery(9292, Protocol::TCP), Service::Name("armtechdaemon")),
        (ServiceQuery(5568, Protocol::UDP), Service::Name("sdt")),
        (ServiceQuery(8003, Protocol::TCP), Service::Name("mcreport")),
        (ServiceQuery(3709, Protocol::UDP), Service::Name("ca-idms")),
        (ServiceQuery(4210, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(5006, Protocol::TCP), Service::Name("wsm-server")),
        (ServiceQuery(3319, Protocol::TCP), Service::Name("sdt-lmd")),
        (ServiceQuery(3178, Protocol::TCP), Service::Name("radclientport")),
        (ServiceQuery(2579, Protocol::UDP), Service::Name("mpfoncl")),
        (ServiceQuery(554, Protocol::TCP), Service::Name("rtsp")),
        (ServiceQuery(3364, Protocol::UDP), Service::Name("creativeserver")),
        (ServiceQuery(1271, Protocol::TCP), Service::Name("excw")),
        (ServiceQuery(7798, Protocol::TCP), Service::Name("pnet-enc")),
        (ServiceQuery(3217, Protocol::UDP), Service::Name("unite")),
        (ServiceQuery(3099, Protocol::UDP), Service::Name("chmd")),
        (ServiceQuery(2049, Protocol::TCP), Service::Name("nfs")),
        (ServiceQuery(7359, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(4660, Protocol::UDP), Service::Name("smaclmgr")),
        (ServiceQuery(9281, Protocol::TCP), Service::Name("swtp-port1")),
        (ServiceQuery(4067, Protocol::UDP), Service::Name("idp")),
        (ServiceQuery(3377, Protocol::UDP), Service::Name("cogsys-lm")),
        (ServiceQuery(8767, Protocol::TCP), Service::Name("core-of-source")),
        (ServiceQuery(3056, Protocol::UDP), Service::Name("cdl-server")),
        (ServiceQuery(6085, Protocol::UDP), Service::Name("konspire2b")),
        (ServiceQuery(268, Protocol::UDP), Service::Name("td-replica")),
        (ServiceQuery(20001, Protocol::TCP), Service::Name("microsan")),
        (ServiceQuery(570, Protocol::TCP), Service::Name("meter")),
        (ServiceQuery(8899, Protocol::UDP), Service::Name("ospf-lite")),
        (ServiceQuery(1189, Protocol::TCP), Service::Name("unet")),
        (ServiceQuery(21800, Protocol::UDP), Service::Name("tvpm")),
        (ServiceQuery(8892, Protocol::TCP), Service::Name("seosload")),
        (ServiceQuery(4129, Protocol::TCP), Service::Name("nuauth")),
        (ServiceQuery(1847, Protocol::UDP), Service::Name("slp-notify")),
        (ServiceQuery(1789, Protocol::UDP), Service::Name("hello")),
        (ServiceQuery(1057, Protocol::TCP), Service::Name("startron")),
        (ServiceQuery(6003, Protocol::TCP), Service::Name("X11:3")),
        (ServiceQuery(16384, Protocol::UDP), Service::Name("connected")),
        (ServiceQuery(37472, Protocol::TCP), Service::Name("3gpp-w1ap")),
        (ServiceQuery(1352, Protocol::UDP), Service::Name("lotusnotes")),
        (ServiceQuery(4376, Protocol::UDP), Service::Name("bip")),
        (ServiceQuery(173, Protocol::TCP), Service::Name("xyplex-mux")),
        (ServiceQuery(165, Protocol::TCP), Service::Name("xns-courier")),
        (ServiceQuery(1683, Protocol::TCP), Service::Name("ncpm-hip")),
        (ServiceQuery(2826, Protocol::TCP), Service::Name("slc-systemlog")),
        (ServiceQuery(62078, Protocol::TCP), Service::Name("iphone-sync")),
        (ServiceQuery(390, Protocol::UDP), Service::Name("uis")),
        (ServiceQuery(1000, Protocol::TCP), Service::Name("cadlock")),
        (ServiceQuery(1449, Protocol::TCP), Service::Name("peport")),
        (ServiceQuery(7302, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(3507, Protocol::TCP), Service::Name("nesh-broker")),
        (ServiceQuery(3419, Protocol::TCP), Service::Name("softaudit")),
        (ServiceQuery(678, Protocol::TCP), Service::Name("ggf-ncp")),
        (ServiceQuery(25001, Protocol::UDP), Service::Name("icl-twobase2")),
        (ServiceQuery(1450, Protocol::UDP), Service::Name("dwf")),
        (ServiceQuery(7676, Protocol::TCP), Service::Name("imqbrokerd")),
        (ServiceQuery(4842, Protocol::UDP), Service::Name("gw-asv")),
        (ServiceQuery(4183, Protocol::TCP), Service::Name("gmmp")),
        (ServiceQuery(1897, Protocol::UDP), Service::Name("metaagent")),
        (ServiceQuery(1375, Protocol::TCP), Service::Name("bytex")),
        (ServiceQuery(9026, Protocol::UDP), Service::Name("swa-4")),
        (ServiceQuery(3071, Protocol::UDP), Service::Name("csd-mgmt-port")),
        (ServiceQuery(22005, Protocol::TCP), Service::Name("optohost004")),
        (ServiceQuery(2515, Protocol::TCP), Service::Name("facsys-router")),
        (ServiceQuery(2256, Protocol::TCP), Service::Name("pcc-mfp")),
        (ServiceQuery(31416, Protocol::TCP), Service::Name("boinc")),
        (ServiceQuery(2634, Protocol::UDP), Service::Name("pk-electronics")),
        (ServiceQuery(2219, Protocol::TCP), Service::Name("netiq-ncap")),
        (ServiceQuery(33334, Protocol::TCP), Service::Name("speedtrace")),
        (ServiceQuery(2621, Protocol::TCP), Service::Name("miles-apart")),
        (ServiceQuery(2119, Protocol::UDP), Service::Name("gsigatekeeper")),
        (ServiceQuery(6969, Protocol::TCP), Service::Name("acmsoda")),
        (ServiceQuery(6071, Protocol::TCP), Service::Name("ssdtp")),
        (ServiceQuery(2019, Protocol::UDP), Service::Name("about")),
        (ServiceQuery(1577, Protocol::UDP), Service::Name("hypercube-lm")),
        (ServiceQuery(6044, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(1177, Protocol::UDP), Service::Name("dkmessenger")),
        (ServiceQuery(1873, Protocol::UDP), Service::Name("fjmpjps")),
        (ServiceQuery(17007, Protocol::UDP), Service::Name("isode-dua")),
        (ServiceQuery(2064, Protocol::UDP), Service::Name("icg-iprelay")),
        (ServiceQuery(2179, Protocol::UDP), Service::Name("vmrdp")),
        (ServiceQuery(3642, Protocol::TCP), Service::Name("juxml-port")),
        (ServiceQuery(1945, Protocol::UDP), Service::Name("dialogic-elmd")),
        (ServiceQuery(438, Protocol::UDP), Service::Name("dsfgw")),
        (ServiceQuery(3519, Protocol::UDP), Service::Name("galileo")),
        (ServiceQuery(6252, Protocol::UDP), Service::Name("tl1-ssh")),
        (ServiceQuery(7303, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(1618, Protocol::TCP), Service::Name("skytelnet")),
        (ServiceQuery(10102, Protocol::TCP), Service::Name("ezproxy-2")),
        (ServiceQuery(3850, Protocol::UDP), Service::Name("qtms-bootstrap")),
        (ServiceQuery(2522, Protocol::UDP), Service::Name("windb")),
        (ServiceQuery(4988, Protocol::TCP), Service::Name("smar-se-port2")),
        (ServiceQuery(369, Protocol::TCP), Service::Name("rpc2portmap")),
        (ServiceQuery(1567, Protocol::TCP), Service::Name("jlicelmd")),
        (ServiceQuery(3560, Protocol::UDP), Service::Name("iniserve-port")),
        (ServiceQuery(3084, Protocol::TCP), Service::Name("itm-mccs")),
        (ServiceQuery(2330, Protocol::TCP), Service::Name("tscchat")),
        (ServiceQuery(3720, Protocol::TCP), Service::Name("ufastro-instr")),
        (ServiceQuery(705, Protocol::UDP), Service::Name("agentx")),
        (ServiceQuery(2531, Protocol::TCP), Service::Name("ito-e-gui")),
        (ServiceQuery(7280, Protocol::TCP), Service::Name("itactionserver1")),
        (ServiceQuery(1117, Protocol::UDP), Service::Name("ardus-mtrns")),
        (ServiceQuery(7979, Protocol::TCP), Service::Name("micromuse-ncps")),
        (ServiceQuery(42510, Protocol::TCP), Service::Name("caerpc")),
        (ServiceQuery(3637, Protocol::UDP), Service::Name("scservp")),
        (ServiceQuery(42508, Protocol::UDP), Service::Name("candp")),
        (ServiceQuery(5115, Protocol::TCP), Service::Name("autobuild")),
        (ServiceQuery(2123, Protocol::TCP), Service::Name("gtp-control")),
        (ServiceQuery(5072, Protocol::UDP), Service::Name("ayiya")),
        (ServiceQuery(32483, Protocol::TCP), Service::Name("apm-link")),
        (ServiceQuery(1148, Protocol::UDP), Service::Name("elfiq-repl")),
        (ServiceQuery(3873, Protocol::UDP), Service::Name("fagordnc")),
        (ServiceQuery(3485, Protocol::TCP), Service::Name("celatalk")),
        (ServiceQuery(2712, Protocol::UDP), Service::Name("aocp")),
        (ServiceQuery(2454, Protocol::TCP), Service::Name("indx-dds")),
        (ServiceQuery(11321, Protocol::TCP), Service::Name("arena-server")),
        (ServiceQuery(197, Protocol::TCP), Service::Name("dls")),
        (ServiceQuery(2186, Protocol::TCP), Service::Name("gtaua")),
        (ServiceQuery(3458, Protocol::UDP), Service::Name("d3winosfi")),
        (ServiceQuery(1797, Protocol::TCP), Service::Name("uma")),
        (ServiceQuery(995, Protocol::UDP), Service::Name("pop3s")),
        (ServiceQuery(43441, Protocol::UDP), Service::Name("ciscocsdb")),
        (ServiceQuery(1952, Protocol::TCP), Service::Name("mpnjsc")),
        (ServiceQuery(2617, Protocol::TCP), Service::Name("cmadmin")),
        (ServiceQuery(2883, Protocol::UDP), Service::Name("ndnp")),
        (ServiceQuery(9217, Protocol::TCP), Service::Name("fsc-port")),
        (ServiceQuery(468, Protocol::TCP), Service::Name("photuris")),
        (ServiceQuery(666, Protocol::TCP), Service::Name("doom")),
        (ServiceQuery(1594, Protocol::TCP), Service::Name("sixtrak")),
        (ServiceQuery(4846, Protocol::UDP), Service::Name("contamac_icm")),
        (ServiceQuery(4167, Protocol::UDP), Service::Name("ddgn")),
        (ServiceQuery(3162, Protocol::TCP), Service::Name("sflm")),
        (ServiceQuery(1243, Protocol::TCP), Service::Name("serialgateway")),
        (ServiceQuery(21201, Protocol::TCP), Service::Name("memcachedb")),
        (ServiceQuery(3751, Protocol::UDP), Service::Name("gprs-cube")),
        (ServiceQuery(6087, Protocol::TCP), Service::Name("ldss")),
        (ServiceQuery(1577, Protocol::TCP), Service::Name("hypercube-lm")),
        (ServiceQuery(150, Protocol::UDP), Service::Name("sql-net")),
        (ServiceQuery(7429, Protocol::UDP), Service::Name("opi-sock")),
        (ServiceQuery(2159, Protocol::UDP), Service::Name("gdbremote")),
        (ServiceQuery(10100, Protocol::TCP), Service::Name("itap-ddtp")),
        (ServiceQuery(61441, Protocol::TCP), Service::Name("netprowler-sensor")),
        (ServiceQuery(2619, Protocol::UDP), Service::Name("bruce")),
        (ServiceQuery(1280, Protocol::TCP), Service::Name("pictrography")),
        (ServiceQuery(9217, Protocol::UDP), Service::Name("fsc-port")),
        (ServiceQuery(1155, Protocol::UDP), Service::Name("nfa")),
        (ServiceQuery(2600, Protocol::TCP), Service::Name("zebrasrv")),
        (ServiceQuery(145, Protocol::TCP), Service::Name("uaac")),
        (ServiceQuery(638, Protocol::UDP), Service::Name("mcns-sec")),
        (ServiceQuery(44322, Protocol::UDP), Service::Name("pmcdproxy")),
        (ServiceQuery(1585, Protocol::TCP), Service::Name("intv")),
        (ServiceQuery(8080, Protocol::UDP), Service::Name("http-alt")),
        (ServiceQuery(4160, Protocol::TCP), Service::Name("jini-discovery")),
        (ServiceQuery(3157, Protocol::UDP), Service::Name("e3consultants")),
        (ServiceQuery(362, Protocol::TCP), Service::Name("srssend")),
        (ServiceQuery(4285, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(5603, Protocol::UDP), Service::Name("a1-bs")),
        (ServiceQuery(8184, Protocol::UDP), Service::Name("itach")),
        (ServiceQuery(1667, Protocol::UDP), Service::Name("netview-aix-7")),
        (ServiceQuery(2290, Protocol::TCP), Service::Name("sonus-logging")),
        (ServiceQuery(5801, Protocol::TCP), Service::Name("vnc-http-1")),
        (ServiceQuery(2589, Protocol::TCP), Service::Name("quartus-tcl")),
        (ServiceQuery(723, Protocol::TCP), Service::Name("omfs")),
        (ServiceQuery(18635, Protocol::TCP), Service::Name("rds-ip")),
        (ServiceQuery(2001, Protocol::UDP), Service::Name("wizard")),
        (ServiceQuery(4255, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3334, Protocol::TCP), Service::Name("directv-web")),
        (ServiceQuery(2181, Protocol::TCP), Service::Name("eforward")),
        (ServiceQuery(3724, Protocol::UDP), Service::Name("blizwow")),
        (ServiceQuery(25, Protocol::TCP), Service::Name("smtp")),
        (ServiceQuery(9279, Protocol::TCP), Service::Name("pegasus-ctl")),
        (ServiceQuery(998, Protocol::UDP), Service::Name("puparp")),
        (ServiceQuery(7677, Protocol::UDP), Service::Name("sun-user-https")),
        (ServiceQuery(5725, Protocol::TCP), Service::Name("ms-ilm")),
        (ServiceQuery(102, Protocol::UDP), Service::Name("iso-tsap")),
        (ServiceQuery(4274, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2840, Protocol::TCP), Service::Name("l3-exprt")),
        (ServiceQuery(989, Protocol::TCP), Service::Name("ftps-data")),
        (ServiceQuery(5057, Protocol::TCP), Service::Name("intecom-ps2")),
        (ServiceQuery(3780, Protocol::UDP), Service::Name("nnp")),
        (ServiceQuery(125, Protocol::UDP), Service::Name("locus-map")),
        (ServiceQuery(7802, Protocol::TCP), Service::Name("vns-tp")),
        (ServiceQuery(2, Protocol::TCP), Service::Name("compressnet")),
        (ServiceQuery(2190, Protocol::UDP), Service::Name("tivoconnect")),
        (ServiceQuery(3030, Protocol::TCP), Service::Name("arepa-cas")),
        (ServiceQuery(532, Protocol::TCP), Service::Name("netnews")),
        (ServiceQuery(3695, Protocol::UDP), Service::Name("bmc-data-coll")),
        (ServiceQuery(1781, Protocol::TCP), Service::Name("answersoft-lm")),
        (ServiceQuery(2144, Protocol::TCP), Service::Name("lv-ffx")),
        (ServiceQuery(9009, Protocol::TCP), Service::Name("pichat")),
        (ServiceQuery(345, Protocol::UDP), Service::Name("pawserv")),
        (ServiceQuery(2288, Protocol::TCP), Service::Name("netml")),
        (ServiceQuery(2165, Protocol::TCP), Service::Name("x-bone-api")),
        (ServiceQuery(2103, Protocol::UDP), Service::Name("zephyr-clt")),
        (ServiceQuery(6059, Protocol::TCP), Service::Name("X11:59")),
        (ServiceQuery(3811, Protocol::TCP), Service::Name("amp")),
        (ServiceQuery(315, Protocol::TCP), Service::Name("dpsi")),
        (ServiceQuery(1132, Protocol::TCP), Service::Name("kvm-via-ip")),
        (ServiceQuery(4065, Protocol::UDP), Service::Name("avanti_cdp")),
        (ServiceQuery(711, Protocol::UDP), Service::Name("cisco-tdp")),
        (ServiceQuery(3844, Protocol::UDP), Service::Name("rnm")),
        (ServiceQuery(1861, Protocol::TCP), Service::Name("lecroy-vicp")),
        (ServiceQuery(1257, Protocol::UDP), Service::Name("shockwave2")),
        (ServiceQuery(1275, Protocol::UDP), Service::Name("ivcollector")),
        (ServiceQuery(21847, Protocol::UDP), Service::Name("netspeak-cs")),
        (ServiceQuery(2233, Protocol::UDP), Service::Name("infocrypt")),
        (ServiceQuery(1061, Protocol::TCP), Service::Name("kiosk")),
        (ServiceQuery(3426, Protocol::UDP), Service::Name("arkivio")),
        (ServiceQuery(5994, Protocol::TCP), Service::Name("rms-agent")),
        (ServiceQuery(43, Protocol::TCP), Service::Name("whois")),
        (ServiceQuery(1530, Protocol::UDP), Service::Name("rap-service")),
        (ServiceQuery(3900, Protocol::UDP), Service::Name("udt_os")),
        (ServiceQuery(3386, Protocol::TCP), Service::Name("gprs-data")),
        (ServiceQuery(2684, Protocol::TCP), Service::Name("mpnjsosv")),
        (ServiceQuery(7007, Protocol::TCP), Service::Name("afs3-bos")),
        (ServiceQuery(8688, Protocol::TCP), Service::Name("openremote-ctrl")),
        (ServiceQuery(300, Protocol::TCP), Service::Name("tacacss")),
        (ServiceQuery(19998, Protocol::TCP), Service::Name("iec-104-sec")),
        (ServiceQuery(3007, Protocol::TCP), Service::Name("lotusmtap")),
        (ServiceQuery(5574, Protocol::TCP), Service::Name("lsi-bobcat")),
        (ServiceQuery(6701, Protocol::TCP), Service::Name("carracho")),
        (ServiceQuery(1833, Protocol::TCP), Service::Name("udpradio")),
        (ServiceQuery(9161, Protocol::TCP), Service::Name("apani2")),
        (ServiceQuery(1185, Protocol::UDP), Service::Name("catchpole")),
        (ServiceQuery(413, Protocol::TCP), Service::Name("smsp")),
        (ServiceQuery(6951, Protocol::TCP), Service::Name("otlp")),
        (ServiceQuery(384, Protocol::TCP), Service::Name("arns")),
        (ServiceQuery(4406, Protocol::TCP), Service::Name("ds-slp")),
        (ServiceQuery(1429, Protocol::TCP), Service::Name("nms")),
        (ServiceQuery(3287, Protocol::UDP), Service::Name("directvdata")),
        (ServiceQuery(7727, Protocol::UDP), Service::Name("trident-data")),
        (ServiceQuery(1443, Protocol::TCP), Service::Name("ies-lm")),
        (ServiceQuery(3027, Protocol::TCP), Service::Name("LiebDevMgmt_C")),
        (ServiceQuery(1807, Protocol::TCP), Service::Name("fhsp")),
        (ServiceQuery(5193, Protocol::UDP), Service::Name("aol-3")),
        (ServiceQuery(3314, Protocol::TCP), Service::Name("uohost")),
        (ServiceQuery(5904, Protocol::TCP), Service::Name("ag-swim")),
        (ServiceQuery(1488, Protocol::TCP), Service::Name("docstor")),
        (ServiceQuery(3026, Protocol::UDP), Service::Name("agri-gateway")),
        (ServiceQuery(6023, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(3623, Protocol::TCP), Service::Name("haipe-discover")),
        (ServiceQuery(1907, Protocol::TCP), Service::Name("intrastar")),
        (ServiceQuery(1637, Protocol::TCP), Service::Name("islc")),
        (ServiceQuery(4914, Protocol::TCP), Service::Name("bones")),
        (ServiceQuery(6023, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(20012, Protocol::TCP), Service::Name("ss-idi-disc")),
        (ServiceQuery(4351, Protocol::TCP), Service::Name("plcy-net-svcs")),
        (ServiceQuery(3341, Protocol::TCP), Service::Name("anet-h")),
        (ServiceQuery(1774, Protocol::TCP), Service::Name("global-dtserv")),
        (ServiceQuery(4224, Protocol::TCP), Service::Name("xtell")),
        (ServiceQuery(2399, Protocol::TCP), Service::Name("fmpro-fdal")),
        (ServiceQuery(3263, Protocol::UDP), Service::Name("ecolor-imager")),
        (ServiceQuery(540, Protocol::UDP), Service::Name("uucp")),
        (ServiceQuery(7311, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(3659, Protocol::TCP), Service::Name("apple-sasl")),
        (ServiceQuery(6770, Protocol::TCP), Service::Name("plysrv-http")),
        (ServiceQuery(3129, Protocol::TCP), Service::Name("netport-id")),
        (ServiceQuery(4392, Protocol::TCP), Service::Name("apwi-rxserver")),
        (ServiceQuery(2050, Protocol::UDP), Service::Name("av-emb-config")),
        (ServiceQuery(4020, Protocol::TCP), Service::Name("trap")),
        (ServiceQuery(5678, Protocol::TCP), Service::Name("rrac")),
        (ServiceQuery(4111, Protocol::TCP), Service::Name("xgrid")),
        (ServiceQuery(2950, Protocol::UDP), Service::Name("esip")),
        (ServiceQuery(629, Protocol::UDP), Service::Name("3com-amp3")),
        (ServiceQuery(2402, Protocol::TCP), Service::Name("taskmaster2000")),
        (ServiceQuery(5402, Protocol::TCP), Service::Name("mftp")),
        (ServiceQuery(2034, Protocol::TCP), Service::Name("scoremgr")),
        (ServiceQuery(313, Protocol::TCP), Service::Name("magenta-logic")),
        (ServiceQuery(1131, Protocol::TCP), Service::Name("caspssl")),
        (ServiceQuery(1008, Protocol::UDP), Service::Name("ufsd")),
        (ServiceQuery(2354, Protocol::UDP), Service::Name("psprserver")),
        (ServiceQuery(1435, Protocol::UDP), Service::Name("ibm-cics")),
        (ServiceQuery(2045, Protocol::TCP), Service::Name("cdfunc")),
        (ServiceQuery(27007, Protocol::TCP), Service::Name("flexlm7")),
        (ServiceQuery(5599, Protocol::UDP), Service::Name("esinstall")),
        (ServiceQuery(256, Protocol::TCP), Service::Name("fw1-secureremote")),
        (ServiceQuery(19539, Protocol::TCP), Service::Name("fxuptp")),
        (ServiceQuery(2714, Protocol::UDP), Service::Name("raventdm")),
        (ServiceQuery(3509, Protocol::TCP), Service::Name("vt-ssl")),
        (ServiceQuery(2497, Protocol::TCP), Service::Name("quaddb")),
        (ServiceQuery(2716, Protocol::UDP), Service::Name("inova-ip-disco")),
        (ServiceQuery(1964, Protocol::TCP), Service::Name("solid-e-engine")),
        (ServiceQuery(3521, Protocol::TCP), Service::Name("mc3ss")),
        (ServiceQuery(1631, Protocol::UDP), Service::Name("visitview")),
        (ServiceQuery(3946, Protocol::TCP), Service::Name("backupedge")),
        (ServiceQuery(3349, Protocol::TCP), Service::Name("chevinservices")),
        (ServiceQuery(2665, Protocol::TCP), Service::Name("patrol-mq-nm")),
        (ServiceQuery(19194, Protocol::TCP), Service::Name("ua-secureagent")),
        (ServiceQuery(26487, Protocol::UDP), Service::Name("exoconfig")),
        (ServiceQuery(1118, Protocol::TCP), Service::Name("sacred")),
        (ServiceQuery(3428, Protocol::TCP), Service::Name("twcss")),
        (ServiceQuery(4260, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(9212, Protocol::UDP), Service::Name("serverviewdbms")),
        (ServiceQuery(1262, Protocol::TCP), Service::Name("qnts-orb")),
        (ServiceQuery(9630, Protocol::TCP), Service::Name("peoctlr")),
        (ServiceQuery(32787, Protocol::TCP), Service::Name("sometimes-rpc27")),
        (ServiceQuery(2434, Protocol::UDP), Service::Name("pxc-epmap")),
        (ServiceQuery(3942, Protocol::TCP), Service::Name("srdp")),
        (ServiceQuery(3280, Protocol::UDP), Service::Name("vs-server")),
        (ServiceQuery(3575, Protocol::UDP), Service::Name("ccm-port")),
        (ServiceQuery(1418, Protocol::UDP), Service::Name("timbuktu-srv2")),
        (ServiceQuery(204, Protocol::UDP), Service::Name("at-echo")),
        (ServiceQuery(2985, Protocol::TCP), Service::Name("hpidsagent")),
        (ServiceQuery(3965, Protocol::TCP), Service::Name("ati-ip-to-ncpe")),
        (ServiceQuery(1727, Protocol::TCP), Service::Name("winddx")),
        (ServiceQuery(1236, Protocol::UDP), Service::Name("bvcontrol")),
        (ServiceQuery(1422, Protocol::TCP), Service::Name("autodesk-lm")),
        (ServiceQuery(6888, Protocol::UDP), Service::Name("muse")),
        (ServiceQuery(3107, Protocol::TCP), Service::Name("business")),
        (ServiceQuery(2591, Protocol::TCP), Service::Name("maytagshuffle")),
        (ServiceQuery(6402, Protocol::UDP), Service::Name("boe-eventsrv")),
        (ServiceQuery(4662, Protocol::UDP), Service::Name("oms")),
        (ServiceQuery(1756, Protocol::UDP), Service::Name("capfast-lmd")),
        (ServiceQuery(6161, Protocol::UDP), Service::Name("patrol-ism")),
        (ServiceQuery(1451, Protocol::UDP), Service::Name("infoman")),
        (ServiceQuery(6600, Protocol::TCP), Service::Name("mshvlm")),
        (ServiceQuery(17185, Protocol::TCP), Service::Name("soundsvirtual")),
        (ServiceQuery(2060, Protocol::UDP), Service::Name("teleniumdaemon")),
        (ServiceQuery(1477, Protocol::UDP), Service::Name("ms-sna-server")),
        (ServiceQuery(5757, Protocol::TCP), Service::Name("x500ms")),
        (ServiceQuery(6502, Protocol::UDP), Service::Name("netop-rc")),
        (ServiceQuery(3927, Protocol::UDP), Service::Name("natdataservice")),
        (ServiceQuery(6036, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(3699, Protocol::UDP), Service::Name("kpn-icw")),
        (ServiceQuery(1566, Protocol::TCP), Service::Name("corelvideo")),
        (ServiceQuery(4402, Protocol::TCP), Service::Name("ds-clnt")),
        (ServiceQuery(3214, Protocol::UDP), Service::Name("jmq-daemon-1")),
        (ServiceQuery(13821, Protocol::TCP), Service::Name("dsmcc-download")),
        (ServiceQuery(1711, Protocol::UDP), Service::Name("pptconference")),
        (ServiceQuery(2993, Protocol::UDP), Service::Name("veritas-vis1")),
        (ServiceQuery(1322, Protocol::UDP), Service::Name("novation")),
        (ServiceQuery(217, Protocol::UDP), Service::Name("dbase")),
        (ServiceQuery(3203, Protocol::UDP), Service::Name("netwatcher-mon")),
        (ServiceQuery(461, Protocol::TCP), Service::Name("datasurfsrv")),
        (ServiceQuery(6114, Protocol::TCP), Service::Name("wrspice")),
        (ServiceQuery(4328, Protocol::UDP), Service::Name("jaxer-manager")),
        (ServiceQuery(799, Protocol::TCP), Service::Name("controlit")),
        (ServiceQuery(7164, Protocol::UDP), Service::Name("fsr")),
        (ServiceQuery(3893, Protocol::UDP), Service::Name("cgi-starapi")),
        (ServiceQuery(7901, Protocol::TCP), Service::Name("tnos-sp")),
        (ServiceQuery(2931, Protocol::TCP), Service::Name("circle-x")),
        (ServiceQuery(4060, Protocol::UDP), Service::Name("dsmeter_iatc")),
        (ServiceQuery(2336, Protocol::UDP), Service::Name("appleugcontrol")),
        (ServiceQuery(3940, Protocol::TCP), Service::Name("xecp-node")),
        (ServiceQuery(3643, Protocol::TCP), Service::Name("audiojuggler")),
        (ServiceQuery(4230, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3914, Protocol::TCP), Service::Name("listcrt-port-2")),
        (ServiceQuery(4661, Protocol::UDP), Service::Name("kar2ouche")),
        (ServiceQuery(2423, Protocol::UDP), Service::Name("rnrp")),
        (ServiceQuery(3164, Protocol::TCP), Service::Name("imprs")),
        (ServiceQuery(4870, Protocol::TCP), Service::Name("cc-tracking")),
        (ServiceQuery(372, Protocol::UDP), Service::Name("ulistserv")),
        (ServiceQuery(2334, Protocol::UDP), Service::Name("ace-client")),
        (ServiceQuery(2467, Protocol::TCP), Service::Name("high-criteria")),
        (ServiceQuery(2668, Protocol::UDP), Service::Name("alarm-clock-c")),
        (ServiceQuery(2632, Protocol::UDP), Service::Name("irdg-post")),
        (ServiceQuery(400, Protocol::UDP), Service::Name("work-sol")),
        (ServiceQuery(8183, Protocol::TCP), Service::Name("proremote")),
        (ServiceQuery(2636, Protocol::UDP), Service::Name("solve")),
        (ServiceQuery(3051, Protocol::UDP), Service::Name("galaxy-server")),
        (ServiceQuery(521, Protocol::TCP), Service::Name("ripng")),
        (ServiceQuery(3452, Protocol::TCP), Service::Name("sabp-signal")),
        (ServiceQuery(2363, Protocol::UDP), Service::Name("mediacntrlnfsd")),
        (ServiceQuery(599, Protocol::UDP), Service::Name("acp")),
        (ServiceQuery(2168, Protocol::UDP), Service::Name("easy-soft-mux")),
        (ServiceQuery(2267, Protocol::UDP), Service::Name("ontobroker")),
        (ServiceQuery(2600, Protocol::UDP), Service::Name("hpstgmgr")),
        (ServiceQuery(4733, Protocol::TCP), Service::Name("resorcs")),
        (ServiceQuery(3075, Protocol::UDP), Service::Name("orbix-locator")),
        (ServiceQuery(2798, Protocol::TCP), Service::Name("tmesis-upshot")),
        (ServiceQuery(2278, Protocol::TCP), Service::Name("s3db")),
        (ServiceQuery(1541, Protocol::TCP), Service::Name("rds2")),
        (ServiceQuery(3966, Protocol::TCP), Service::Name("bflckmgr")),
        (ServiceQuery(1425, Protocol::TCP), Service::Name("zion-lm")),
        (ServiceQuery(3228, Protocol::UDP), Service::Name("dwmsgserver")),
        (ServiceQuery(714, Protocol::UDP), Service::Name("iris-xpcs")),
        (ServiceQuery(11320, Protocol::TCP), Service::Name("imip-channels")),
        (ServiceQuery(1299, Protocol::TCP), Service::Name("hp-sci")),
        (ServiceQuery(547, Protocol::TCP), Service::Name("dhcpv6-server")),
        (ServiceQuery(6634, Protocol::TCP), Service::Name("mpls-pm")),
        (ServiceQuery(4226, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2676, Protocol::UDP), Service::Name("simslink")),
        (ServiceQuery(3378, Protocol::UDP), Service::Name("wsicopy")),
        (ServiceQuery(1946, Protocol::TCP), Service::Name("tekpls")),
        (ServiceQuery(698, Protocol::UDP), Service::Name("olsr")),
        (ServiceQuery(1212, Protocol::TCP), Service::Name("lupa")),
        (ServiceQuery(9667, Protocol::TCP), Service::Name("xmms2")),
        (ServiceQuery(3463, Protocol::TCP), Service::Name("edm-adm-notify")),
        (ServiceQuery(3956, Protocol::UDP), Service::Name("gvcp")),
        (ServiceQuery(2190, Protocol::TCP), Service::Name("tivoconnect")),
        (ServiceQuery(7887, Protocol::TCP), Service::Name("ubroker")),
        (ServiceQuery(19000, Protocol::TCP), Service::Name("igrid")),
        (ServiceQuery(2297, Protocol::TCP), Service::Name("d2k-datamover1")),
        (ServiceQuery(5628, Protocol::UDP), Service::Name("htrust")),
        (ServiceQuery(2039, Protocol::UDP), Service::Name("prizma")),
        (ServiceQuery(8081, Protocol::TCP), Service::Name("blackice-icecap")),
        (ServiceQuery(7871, Protocol::TCP), Service::Name("mdm")),
        (ServiceQuery(7333, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(4162, Protocol::UDP), Service::Name("omstopology")),
        (ServiceQuery(1597, Protocol::UDP), Service::Name("orbplus-iiop")),
        (ServiceQuery(629, Protocol::TCP), Service::Name("3com-amp3")),
        (ServiceQuery(2343, Protocol::TCP), Service::Name("nati-logos")),
        (ServiceQuery(15555, Protocol::TCP), Service::Name("cisco-snat")),
        (ServiceQuery(1356, Protocol::TCP), Service::Name("cuillamartin")),
        (ServiceQuery(2907, Protocol::TCP), Service::Name("webmethods-b2b")),
        (ServiceQuery(7437, Protocol::TCP), Service::Name("faximum")),
        (ServiceQuery(772, Protocol::UDP), Service::Name("cycleserv2")),
        (ServiceQuery(4990, Protocol::TCP), Service::Name("busycal")),
        (ServiceQuery(28001, Protocol::TCP), Service::Name("pqsp")),
        (ServiceQuery(4843, Protocol::TCP), Service::Name("opcua-tls")),
        (ServiceQuery(2762, Protocol::TCP), Service::Name("dicom-tls")),
        (ServiceQuery(4395, Protocol::TCP), Service::Name("omnivisionesx")),
        (ServiceQuery(1845, Protocol::TCP), Service::Name("altalink")),
        (ServiceQuery(7332, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(33060, Protocol::TCP), Service::Name("mysqlx")),
        (ServiceQuery(1404, Protocol::UDP), Service::Name("igi-lm")),
        (ServiceQuery(1687, Protocol::UDP), Service::Name("nsjtp-ctrl")),
        (ServiceQuery(2108, Protocol::UDP), Service::Name("rkinit")),
        (ServiceQuery(1096, Protocol::UDP), Service::Name("cnrprotocol")),
        (ServiceQuery(3587, Protocol::TCP), Service::Name("p2pgroup")),
        (ServiceQuery(2044, Protocol::UDP), Service::Name("rimsl")),
        (ServiceQuery(6510, Protocol::TCP), Service::Name("mcer-port")),
        (ServiceQuery(41796, Protocol::TCP), Service::Name("crestron-cips")),
        (ServiceQuery(1955, Protocol::UDP), Service::Name("abr-secure")),
        (ServiceQuery(1595, Protocol::TCP), Service::Name("radio")),
        (ServiceQuery(7354, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(1173, Protocol::UDP), Service::Name("d-cinema-rrp")),
        (ServiceQuery(2547, Protocol::UDP), Service::Name("vytalvaultvsmp")),
        (ServiceQuery(7981, Protocol::TCP), Service::Name("sossd-collect")),
        (ServiceQuery(6769, Protocol::UDP), Service::Name("adi-gxp-srvprt")),
        (ServiceQuery(428, Protocol::TCP), Service::Name("ocs_cmu")),
        (ServiceQuery(1276, Protocol::TCP), Service::Name("ivmanager")),
        (ServiceQuery(2005, Protocol::TCP), Service::Name("deslogin")),
        (ServiceQuery(1518, Protocol::UDP), Service::Name("vpvd")),
        (ServiceQuery(6501, Protocol::UDP), Service::Name("boks_servc")),
        (ServiceQuery(3866, Protocol::UDP), Service::Name("dzdaemon")),
        (ServiceQuery(16001, Protocol::TCP), Service::Name("fmsascon")),
        (ServiceQuery(509, Protocol::TCP), Service::Name("snare")),
        (ServiceQuery(7629, Protocol::TCP), Service::Name("xdas")),
        (ServiceQuery(454, Protocol::UDP), Service::Name("contentserver")),
        (ServiceQuery(1343, Protocol::TCP), Service::Name("re101")),
        (ServiceQuery(508, Protocol::TCP), Service::Name("xvttp")),
        (ServiceQuery(1928, Protocol::TCP), Service::Name("emsd-port")),
        (ServiceQuery(3824, Protocol::UDP), Service::Name("acp-policy")),
        (ServiceQuery(474, Protocol::UDP), Service::Name("tn-tl-w2")),
        (ServiceQuery(675, Protocol::TCP), Service::Name("dctp")),
        (ServiceQuery(9333, Protocol::TCP), Service::Name("litecoin")),
        (ServiceQuery(3216, Protocol::UDP), Service::Name("ferrari-foam")),
        (ServiceQuery(50, Protocol::TCP), Service::Name("re-mail-ck")),
        (ServiceQuery(4007, Protocol::TCP), Service::Name("pxc-splr")),
        (ServiceQuery(630, Protocol::TCP), Service::Name("rda")),
        (ServiceQuery(2270, Protocol::UDP), Service::Name("starschool")),
        (ServiceQuery(2872, Protocol::TCP), Service::Name("radix")),
        (ServiceQuery(5313, Protocol::UDP), Service::Name("rrdp")),
        (ServiceQuery(347, Protocol::UDP), Service::Name("fatserv")),
        (ServiceQuery(4079, Protocol::TCP), Service::Name("santools")),
        (ServiceQuery(383, Protocol::UDP), Service::Name("hp-alarm-mgr")),
        (ServiceQuery(5604, Protocol::UDP), Service::Name("a3-sdunode")),
        (ServiceQuery(2462, Protocol::TCP), Service::Name("qadmifevent")),
        (ServiceQuery(594, Protocol::UDP), Service::Name("tpip")),
        (ServiceQuery(1859, Protocol::TCP), Service::Name("gammafetchsvr")),
        (ServiceQuery(7401, Protocol::UDP), Service::Name("rtps-dd-ut")),
        (ServiceQuery(3226, Protocol::TCP), Service::Name("isi-irp")),
        (ServiceQuery(5232, Protocol::TCP), Service::Name("sgi-dgl")),
        (ServiceQuery(2597, Protocol::TCP), Service::Name("homesteadglory")),
        (ServiceQuery(4159, Protocol::TCP), Service::Name("nss")),
        (ServiceQuery(4291, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(7675, Protocol::TCP), Service::Name("imqtunnel")),
        (ServiceQuery(4204, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(6103, Protocol::UDP), Service::Name("rets")),
        (ServiceQuery(1698, Protocol::TCP), Service::Name("rsvp-encap-1")),
        (ServiceQuery(2132, Protocol::TCP), Service::Name("solera-epmap")),
        (ServiceQuery(1566, Protocol::UDP), Service::Name("corelvideo")),
        (ServiceQuery(189, Protocol::TCP), Service::Name("qft")),
        (ServiceQuery(1744, Protocol::UDP), Service::Name("ncpm-ft")),
        (ServiceQuery(308, Protocol::TCP), Service::Name("novastorbakcup")),
        (ServiceQuery(4727, Protocol::UDP), Service::Name("fcis-disc")),
        (ServiceQuery(9950, Protocol::UDP), Service::Name("apc-9950")),
        (ServiceQuery(2344, Protocol::UDP), Service::Name("fcmsys")),
        (ServiceQuery(5530, Protocol::TCP), Service::Name("sdserv")),
        (ServiceQuery(2071, Protocol::TCP), Service::Name("acp-port")),
        (ServiceQuery(740, Protocol::TCP), Service::Name("netcp")),
        (ServiceQuery(4373, Protocol::UDP), Service::Name("remctl")),
        (ServiceQuery(2110, Protocol::TCP), Service::Name("umsp")),
        (ServiceQuery(4161, Protocol::UDP), Service::Name("omscontact")),
        (ServiceQuery(6316, Protocol::TCP), Service::Name("abb-escp")),
        (ServiceQuery(5728, Protocol::TCP), Service::Name("io-dist-data")),
        (ServiceQuery(4536, Protocol::TCP), Service::Name("ehs-ssl")),
        (ServiceQuery(3814, Protocol::TCP), Service::Name("neto-dcs")),
        (ServiceQuery(2816, Protocol::TCP), Service::Name("lbc-watchdog")),
        (ServiceQuery(4276, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2540, Protocol::TCP), Service::Name("lonworks")),
        (ServiceQuery(3222, Protocol::UDP), Service::Name("glbp")),
        (ServiceQuery(3219, Protocol::UDP), Service::Name("wms-messenger")),
        (ServiceQuery(3135, Protocol::UDP), Service::Name("peerbook-port")),
        (ServiceQuery(202, Protocol::TCP), Service::Name("at-nbp")),
        (ServiceQuery(2725, Protocol::UDP), Service::Name("msolap-ptp2")),
        (ServiceQuery(2059, Protocol::TCP), Service::Name("bmc-messaging")),
        (ServiceQuery(3813, Protocol::UDP), Service::Name("rap-ip")),
        (ServiceQuery(156, Protocol::UDP), Service::Name("sqlsrv")),
        (ServiceQuery(4200, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(6888, Protocol::TCP), Service::Name("muse")),
        (ServiceQuery(2388, Protocol::TCP), Service::Name("mynahautostart")),
        (ServiceQuery(7304, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(2972, Protocol::TCP), Service::Name("pmsm-webrctl")),
        (ServiceQuery(758, Protocol::TCP), Service::Name("nlogin")),
        (ServiceQuery(648, Protocol::TCP), Service::Name("rrp")),
        (ServiceQuery(2156, Protocol::TCP), Service::Name("trp")),
        (ServiceQuery(6028, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(1818, Protocol::UDP), Service::Name("etftp")),
        (ServiceQuery(4668, Protocol::UDP), Service::Name("mmaeds")),
        (ServiceQuery(2930, Protocol::UDP), Service::Name("amx-weblinx")),
        (ServiceQuery(11877, Protocol::TCP), Service::Name("x2e-disc")),
        (ServiceQuery(15660, Protocol::UDP), Service::Name("bex-xr")),
        (ServiceQuery(6048, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(79, Protocol::TCP), Service::Name("finger")),
        (ServiceQuery(4297, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3583, Protocol::TCP), Service::Name("canex-watch")),
        (ServiceQuery(2628, Protocol::UDP), Service::Name("dict")),
        (ServiceQuery(432, Protocol::UDP), Service::Name("iasd")),
        (ServiceQuery(2007, Protocol::UDP), Service::Name("raid-am")),
        (ServiceQuery(2554, Protocol::TCP), Service::Name("vcnet-link-v10")),
        (ServiceQuery(2499, Protocol::UDP), Service::Name("unicontrol")),
        (ServiceQuery(6109, Protocol::UDP), Service::Name("globecast-id")),
        (ServiceQuery(2558, Protocol::UDP), Service::Name("pclemultimedia")),
        (ServiceQuery(1491, Protocol::UDP), Service::Name("anynetgateway")),
        (ServiceQuery(2865, Protocol::TCP), Service::Name("pit-vpn")),
        (ServiceQuery(22005, Protocol::UDP), Service::Name("optohost004")),
        (ServiceQuery(5555, Protocol::TCP), Service::Name("freeciv")),
        (ServiceQuery(911, Protocol::TCP), Service::Name("xact-backup")),
        (ServiceQuery(1489, Protocol::UDP), Service::Name("dmdocbroker")),
        (ServiceQuery(2116, Protocol::UDP), Service::Name("ccowcmr")),
        (ServiceQuery(614, Protocol::UDP), Service::Name("sshell")),
        (ServiceQuery(599, Protocol::TCP), Service::Name("acp")),
        (ServiceQuery(118, Protocol::UDP), Service::Name("sqlserv")),
        (ServiceQuery(9747, Protocol::TCP), Service::Name("l5nas-parchan")),
        (ServiceQuery(3034, Protocol::UDP), Service::Name("osmosis-aeea")),
        (ServiceQuery(45000, Protocol::UDP), Service::Name("ciscopop")),
        (ServiceQuery(27, Protocol::UDP), Service::Name("nsw-fe")),
        (ServiceQuery(8913, Protocol::TCP), Service::Name("dragonfly")),
        (ServiceQuery(8010, Protocol::TCP), Service::Name("xmpp")),
        (ServiceQuery(3590, Protocol::UDP), Service::Name("wv-csp-sms")),
        (ServiceQuery(7129, Protocol::TCP), Service::Name("scenccs")),
        (ServiceQuery(9888, Protocol::TCP), Service::Name("cyborg-systems")),
        (ServiceQuery(6401, Protocol::UDP), Service::Name("boe-was")),
        (ServiceQuery(3589, Protocol::UDP), Service::Name("isomair")),
        (ServiceQuery(583, Protocol::UDP), Service::Name("philips-vc")),
        (ServiceQuery(3162, Protocol::UDP), Service::Name("sflm")),
        (ServiceQuery(203, Protocol::TCP), Service::Name("at-3")),
        (ServiceQuery(1134, Protocol::TCP), Service::Name("aplx")),
        (ServiceQuery(8991, Protocol::UDP), Service::Name("https-wmap")),
        (ServiceQuery(1101, Protocol::UDP), Service::Name("pt2-discover")),
        (ServiceQuery(5032, Protocol::TCP), Service::Name("signacert-agent")),
        (ServiceQuery(1901, Protocol::TCP), Service::Name("fjicl-tep-a")),
        (ServiceQuery(3924, Protocol::UDP), Service::Name("mpl-gprs-port")),
        (ServiceQuery(2333, Protocol::TCP), Service::Name("snapp")),
        (ServiceQuery(4848, Protocol::UDP), Service::Name("appserv-http")),
        (ServiceQuery(2497, Protocol::UDP), Service::Name("quaddb")),
        (ServiceQuery(24005, Protocol::UDP), Service::Name("med-ci")),
        (ServiceQuery(355, Protocol::TCP), Service::Name("datex-asn")),
        (ServiceQuery(6787, Protocol::UDP), Service::Name("smc-admin")),
        (ServiceQuery(10104, Protocol::TCP), Service::Name("swdtp")),
        (ServiceQuery(5584, Protocol::UDP), Service::Name("bis-web")),
        (ServiceQuery(4567, Protocol::UDP), Service::Name("tram")),
        (ServiceQuery(2780, Protocol::UDP), Service::Name("lbc-control")),
        (ServiceQuery(4665, Protocol::UDP), Service::Name("contclientms")),
        (ServiceQuery(7845, Protocol::TCP), Service::Name("apc-7845")),
        (ServiceQuery(2291, Protocol::TCP), Service::Name("eapsp")),
        (ServiceQuery(1990, Protocol::TCP), Service::Name("stun-p1")),
        (ServiceQuery(7560, Protocol::TCP), Service::Name("sncp")),
        (ServiceQuery(8502, Protocol::TCP), Service::Name("ftnmtp")),
        (ServiceQuery(462, Protocol::TCP), Service::Name("datasurfsrvsec")),
        (ServiceQuery(7356, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(3304, Protocol::TCP), Service::Name("opsession-srvr")),
        (ServiceQuery(9343, Protocol::UDP), Service::Name("mpidcmgr")),
        (ServiceQuery(370, Protocol::UDP), Service::Name("codaauth2")),
        (ServiceQuery(9666, Protocol::TCP), Service::Name("zoomcp")),
        (ServiceQuery(1164, Protocol::TCP), Service::Name("qsm-proxy")),
        (ServiceQuery(2125, Protocol::TCP), Service::Name("lockstep")),
        (ServiceQuery(1385, Protocol::UDP), Service::Name("atex_elmd")),
        (ServiceQuery(2725, Protocol::TCP), Service::Name("msolap-ptp2")),
        (ServiceQuery(7626, Protocol::TCP), Service::Name("simco")),
        (ServiceQuery(5203, Protocol::TCP), Service::Name("targus-getdata3")),
        (ServiceQuery(66, Protocol::TCP), Service::Name("sqlnet")),
        (ServiceQuery(4121, Protocol::TCP), Service::Name("e-builder")),
        (ServiceQuery(1905, Protocol::UDP), Service::Name("sugp")),
        (ServiceQuery(1456, Protocol::TCP), Service::Name("dca")),
        (ServiceQuery(10111, Protocol::TCP), Service::Name("nmea-onenet")),
        (ServiceQuery(3911, Protocol::TCP), Service::Name("prnstatus")),
        (ServiceQuery(3389, Protocol::TCP), Service::Name("ms-wbt-server")),
        (ServiceQuery(8291, Protocol::TCP), Service::Name("winbox")),
        (ServiceQuery(1097, Protocol::UDP), Service::Name("sunclustermgr")),
        (ServiceQuery(2557, Protocol::UDP), Service::Name("nicetec-mgmt")),
        (ServiceQuery(640, Protocol::UDP), Service::Name("pcnfs")),
        (ServiceQuery(286, Protocol::UDP), Service::Name("fxp")),
        (ServiceQuery(3165, Protocol::TCP), Service::Name("newgenpay")),
        (ServiceQuery(1113, Protocol::TCP), Service::Name("ltp-deepspace")),
        (ServiceQuery(4182, Protocol::TCP), Service::Name("pcptcpservice")),
        (ServiceQuery(4307, Protocol::UDP), Service::Name("visicron-vs")),
        (ServiceQuery(7303, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(4275, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(426, Protocol::TCP), Service::Name("smartsdp")),
        (ServiceQuery(2587, Protocol::UDP), Service::Name("masc")),
        (ServiceQuery(3327, Protocol::TCP), Service::Name("bbars")),
        (ServiceQuery(3599, Protocol::TCP), Service::Name("quasar-server")),
        (ServiceQuery(9103, Protocol::UDP), Service::Name("bacula-sd")),
        (ServiceQuery(2076, Protocol::TCP), Service::Name("newlixconfig")),
        (ServiceQuery(8908, Protocol::TCP), Service::Name("dpp")),
        (ServiceQuery(9103, Protocol::TCP), Service::Name("jetdirect")),
        (ServiceQuery(3145, Protocol::UDP), Service::Name("csi-lfap")),
        (ServiceQuery(5670, Protocol::TCP), Service::Name("filemq")),
        (ServiceQuery(281, Protocol::TCP), Service::Name("personal-link")),
        (ServiceQuery(709, Protocol::UDP), Service::Name("entrustmanager")),
        (ServiceQuery(591, Protocol::TCP), Service::Name("http-alt")),
        (ServiceQuery(181, Protocol::UDP), Service::Name("unify")),
        (ServiceQuery(1517, Protocol::TCP), Service::Name("vpac")),
        (ServiceQuery(8301, Protocol::UDP), Service::Name("amberon")),
        (ServiceQuery(1850, Protocol::UDP), Service::Name("gsi")),
        (ServiceQuery(2900, Protocol::UDP), Service::Name("quicksuite")),
        (ServiceQuery(3589, Protocol::TCP), Service::Name("isomair")),
        (ServiceQuery(2433, Protocol::TCP), Service::Name("codasrv-se")),
        (ServiceQuery(3770, Protocol::UDP), Service::Name("cindycollab")),
        (ServiceQuery(2916, Protocol::TCP), Service::Name("elvin_server")),
        (ServiceQuery(2830, Protocol::TCP), Service::Name("silkp2")),
        (ServiceQuery(3714, Protocol::TCP), Service::Name("delos-dms")),
        (ServiceQuery(2485, Protocol::UDP), Service::Name("netobjects1")),
        (ServiceQuery(4179, Protocol::UDP), Service::Name("MaxumSP")),
        (ServiceQuery(467, Protocol::UDP), Service::Name("mylex-mapd")),
        (ServiceQuery(1685, Protocol::UDP), Service::Name("n2nremote")),
        (ServiceQuery(1174, Protocol::UDP), Service::Name("fnet-remote-ui")),
        (ServiceQuery(10110, Protocol::UDP), Service::Name("nmea-0183")),
        (ServiceQuery(903, Protocol::TCP), Service::Name("iss-console-mgr")),
        (ServiceQuery(1410, Protocol::TCP), Service::Name("hiq")),
        (ServiceQuery(580, Protocol::UDP), Service::Name("sntp-heartbeat")),
        (ServiceQuery(2088, Protocol::UDP), Service::Name("ip-blf")),
        (ServiceQuery(23005, Protocol::TCP), Service::Name("inovaport6")),
        (ServiceQuery(1417, Protocol::TCP), Service::Name("timbuktu-srv1")),
        (ServiceQuery(9209, Protocol::TCP), Service::Name("almobile-system")),
        (ServiceQuery(364, Protocol::UDP), Service::Name("aurora-cmgr")),
        (ServiceQuery(3401, Protocol::UDP), Service::Name("squid-snmp")),
        (ServiceQuery(9889, Protocol::UDP), Service::Name("gt-proxy")),
        (ServiceQuery(22289, Protocol::TCP), Service::Name("wnn6_Cn")),
        (ServiceQuery(3913, Protocol::UDP), Service::Name("listcrt-port")),
        (ServiceQuery(4668, Protocol::TCP), Service::Name("mmaeds")),
        (ServiceQuery(4283, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3610, Protocol::UDP), Service::Name("echonet")),
        (ServiceQuery(89, Protocol::TCP), Service::Name("su-mit-tg")),
        (ServiceQuery(2091, Protocol::UDP), Service::Name("prp")),
        (ServiceQuery(260, Protocol::TCP), Service::Name("openport")),
        (ServiceQuery(169, Protocol::UDP), Service::Name("send")),
        (ServiceQuery(649, Protocol::TCP), Service::Name("cadview-3d")),
        (ServiceQuery(30002, Protocol::UDP), Service::Name("pago-services2")),
        (ServiceQuery(386, Protocol::TCP), Service::Name("asa")),
        (ServiceQuery(2577, Protocol::TCP), Service::Name("scipticslsrvr")),
        (ServiceQuery(4010, Protocol::UDP), Service::Name("samsung-unidex")),
        (ServiceQuery(4106, Protocol::TCP), Service::Name("synchronite")),
        (ServiceQuery(2119, Protocol::TCP), Service::Name("gsigatekeeper")),
        (ServiceQuery(5634, Protocol::UDP), Service::Name("xprtld")),
        (ServiceQuery(570, Protocol::UDP), Service::Name("meter")),
        (ServiceQuery(3401, Protocol::TCP), Service::Name("filecast")),
        (ServiceQuery(5990, Protocol::UDP), Service::Name("wbem-exp-https")),
        (ServiceQuery(1226, Protocol::UDP), Service::Name("stgxfws")),
        (ServiceQuery(4011, Protocol::UDP), Service::Name("altserviceboot")),
        (ServiceQuery(17756, Protocol::UDP), Service::Name("zigbee-ips")),
        (ServiceQuery(9216, Protocol::UDP), Service::Name("acme")),
        (ServiceQuery(25008, Protocol::UDP), Service::Name("icl-twobase9")),
        (ServiceQuery(5427, Protocol::TCP), Service::Name("sco-peer-tta")),
        (ServiceQuery(48899, Protocol::UDP), Service::Name("tc_ads_discovery")),
        (ServiceQuery(1306, Protocol::UDP), Service::Name("re-conn-proto")),
        (ServiceQuery(3504, Protocol::TCP), Service::Name("ironstorm")),
        (ServiceQuery(8008, Protocol::UDP), Service::Name("http-alt")),
        (ServiceQuery(5007, Protocol::TCP), Service::Name("wsm-server-ssl")),
        (ServiceQuery(3088, Protocol::TCP), Service::Name("xdtp")),
        (ServiceQuery(4147, Protocol::UDP), Service::Name("vrxpservman")),
        (ServiceQuery(126, Protocol::UDP), Service::Name("unitary")),
        (ServiceQuery(750, Protocol::TCP), Service::Name("kerberos")),
        (ServiceQuery(3932, Protocol::TCP), Service::Name("dyn-site")),
        (ServiceQuery(9022, Protocol::TCP), Service::Name("paragent")),
        (ServiceQuery(2029, Protocol::TCP), Service::Name("hsrpv6")),
        (ServiceQuery(1027, Protocol::TCP), Service::Name("IIS")),
        (ServiceQuery(5977, Protocol::TCP), Service::Name("ncd-pref-tcp")),
        (ServiceQuery(2987, Protocol::TCP), Service::Name("identify")),
        (ServiceQuery(6513, Protocol::TCP), Service::Name("netconf-tls")),
        (ServiceQuery(1574, Protocol::UDP), Service::Name("mvel-lm")),
        (ServiceQuery(4021, Protocol::UDP), Service::Name("nexus-portal")),
        (ServiceQuery(1816, Protocol::TCP), Service::Name("harp")),
        (ServiceQuery(5221, Protocol::TCP), Service::Name("3exmp")),
        (ServiceQuery(582, Protocol::UDP), Service::Name("scc-security")),
        (ServiceQuery(3303, Protocol::TCP), Service::Name("opsession-clnt")),
        (ServiceQuery(3470, Protocol::TCP), Service::Name("jt400")),
        (ServiceQuery(7633, Protocol::TCP), Service::Name("pmdfmgt")),
        (ServiceQuery(3692, Protocol::UDP), Service::Name("intelsync")),
        (ServiceQuery(4064, Protocol::UDP), Service::Name("ice-srouter")),
        (ServiceQuery(3463, Protocol::UDP), Service::Name("edm-adm-notify")),
        (ServiceQuery(3586, Protocol::TCP), Service::Name("emprise-lsc")),
        (ServiceQuery(5104, Protocol::UDP), Service::Name("tinymessage")),
        (ServiceQuery(1884, Protocol::UDP), Service::Name("idmaps")),
        (ServiceQuery(3875, Protocol::UDP), Service::Name("pnbscada")),
        (ServiceQuery(3783, Protocol::UDP), Service::Name("bim-pem")),
        (ServiceQuery(2218, Protocol::TCP), Service::Name("bounzza")),
        (ServiceQuery(1058, Protocol::UDP), Service::Name("nim")),
        (ServiceQuery(8890, Protocol::TCP), Service::Name("ddi-tcp-3")),
        (ServiceQuery(4878, Protocol::TCP), Service::Name("inst-discovery")),
        (ServiceQuery(4949, Protocol::UDP), Service::Name("munin")),
        (ServiceQuery(2780, Protocol::TCP), Service::Name("lbc-control")),
        (ServiceQuery(8194, Protocol::TCP), Service::Name("sophos")),
        (ServiceQuery(3564, Protocol::UDP), Service::Name("esimport")),
        (ServiceQuery(17223, Protocol::TCP), Service::Name("isa100-gci")),
        (ServiceQuery(612, Protocol::TCP), Service::Name("hmmp-ind")),
        (ServiceQuery(7344, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(3384, Protocol::TCP), Service::Name("hp-clic")),
        (ServiceQuery(1668, Protocol::UDP), Service::Name("netview-aix-8")),
        (ServiceQuery(2128, Protocol::UDP), Service::Name("net-steward")),
        (ServiceQuery(1078, Protocol::TCP), Service::Name("avocent-proxy")),
        (ServiceQuery(32249, Protocol::TCP), Service::Name("t1distproc60")),
        (ServiceQuery(1548, Protocol::UDP), Service::Name("axon-lm")),
        (ServiceQuery(1214, Protocol::UDP), Service::Name("fasttrack")),
        (ServiceQuery(3498, Protocol::UDP), Service::Name("dashpas-port")),
        (ServiceQuery(4600, Protocol::UDP), Service::Name("piranha1")),
        (ServiceQuery(3224, Protocol::UDP), Service::Name("aes-discovery")),
        (ServiceQuery(9762, Protocol::UDP), Service::Name("tungsten-http")),
        (ServiceQuery(1364, Protocol::TCP), Service::Name("ndm-server")),
        (ServiceQuery(3231, Protocol::TCP), Service::Name("vidigo")),
        (ServiceQuery(1371, Protocol::TCP), Service::Name("fc-cli")),
        (ServiceQuery(1251, Protocol::UDP), Service::Name("servergraph")),
        (ServiceQuery(1986, Protocol::UDP), Service::Name("licensedaemon")),
        (ServiceQuery(10101, Protocol::UDP), Service::Name("ezmeeting-2")),
        (ServiceQuery(7012, Protocol::TCP), Service::Name("talon-engine")),
        (ServiceQuery(93, Protocol::UDP), Service::Name("dcp")),
        (ServiceQuery(3439, Protocol::UDP), Service::Name("hri-port")),
        (ServiceQuery(3003, Protocol::UDP), Service::Name("cgms")),
        (ServiceQuery(1311, Protocol::TCP), Service::Name("rxmon")),
        (ServiceQuery(1130, Protocol::UDP), Service::Name("casp")),
        (ServiceQuery(5092, Protocol::TCP), Service::Name("magpie")),
        (ServiceQuery(2442, Protocol::TCP), Service::Name("netangel")),
        (ServiceQuery(2905, Protocol::UDP), Service::Name("m3ua")),
        (ServiceQuery(1860, Protocol::UDP), Service::Name("sunscalar-svc")),
        (ServiceQuery(2775, Protocol::UDP), Service::Name("smpp")),
        (ServiceQuery(17729, Protocol::TCP), Service::Name("ea")),
        (ServiceQuery(3553, Protocol::UDP), Service::Name("rbr-discovery")),
        (ServiceQuery(4876, Protocol::TCP), Service::Name("tritium-can")),
        (ServiceQuery(769, Protocol::UDP), Service::Name("vid")),
        (ServiceQuery(2792, Protocol::UDP), Service::Name("f5-globalsite")),
        (ServiceQuery(5678, Protocol::UDP), Service::Name("rrac")),
        (ServiceQuery(5420, Protocol::UDP), Service::Name("cylink-c")),
        (ServiceQuery(3986, Protocol::TCP), Service::Name("mapper-ws_ethd")),
        (ServiceQuery(215, Protocol::UDP), Service::Name("softpc")),
        (ServiceQuery(3521, Protocol::UDP), Service::Name("mc3ss")),
        (ServiceQuery(19398, Protocol::UDP), Service::Name("mtrgtrans")),
        (ServiceQuery(4088, Protocol::TCP), Service::Name("npsp")),
        (ServiceQuery(1360, Protocol::TCP), Service::Name("mimer")),
        (ServiceQuery(513, Protocol::TCP), Service::Name("login")),
        (ServiceQuery(4232, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(4191, Protocol::UDP), Service::Name("dsmipv6")),
        (ServiceQuery(1922, Protocol::UDP), Service::Name("tapestry")),
        (ServiceQuery(9911, Protocol::UDP), Service::Name("sype-transport")),
        (ServiceQuery(8804, Protocol::UDP), Service::Name("truecm")),
        (ServiceQuery(20480, Protocol::TCP), Service::Name("emwavemsg")),
        (ServiceQuery(3740, Protocol::TCP), Service::Name("heartbeat")),
        (ServiceQuery(32811, Protocol::TCP), Service::Name("retp")),
        (ServiceQuery(17007, Protocol::TCP), Service::Name("isode-dua")),
        (ServiceQuery(1219, Protocol::TCP), Service::Name("aeroflight-ret")),
        (ServiceQuery(2554, Protocol::UDP), Service::Name("vcnet-link-v10")),
        (ServiceQuery(848, Protocol::TCP), Service::Name("gdoi")),
        (ServiceQuery(1928, Protocol::UDP), Service::Name("emsd-port")),
        (ServiceQuery(4036, Protocol::TCP), Service::Name("wap-push-https")),
        (ServiceQuery(2746, Protocol::TCP), Service::Name("cpudpencap")),
        (ServiceQuery(9123, Protocol::TCP), Service::Name("grcp")),
        (ServiceQuery(1791, Protocol::TCP), Service::Name("ea1")),
        (ServiceQuery(1211, Protocol::TCP), Service::Name("groove-dpp")),
        (ServiceQuery(2323, Protocol::UDP), Service::Name("3d-nfsd")),
        (ServiceQuery(3836, Protocol::TCP), Service::Name("markem-dcp")),
        (ServiceQuery(5168, Protocol::UDP), Service::Name("scte30")),
        (ServiceQuery(1399, Protocol::UDP), Service::Name("cadkey-licman")),
        (ServiceQuery(1391, Protocol::UDP), Service::Name("iclpv-sas")),
        (ServiceQuery(144, Protocol::UDP), Service::Name("news")),
        (ServiceQuery(1129, Protocol::UDP), Service::Name("saphostctrls")),
        (ServiceQuery(5252, Protocol::UDP), Service::Name("movaz-ssc")),
        (ServiceQuery(3030, Protocol::UDP), Service::Name("arepa-cas")),
        (ServiceQuery(2909, Protocol::UDP), Service::Name("funk-dialout")),
        (ServiceQuery(1313, Protocol::TCP), Service::Name("bmc_patroldb")),
        (ServiceQuery(1749, Protocol::UDP), Service::Name("aspen-services")),
        (ServiceQuery(4446, Protocol::UDP), Service::Name("n1-fwp")),
        (ServiceQuery(7738, Protocol::UDP), Service::Name("aiagent")),
        (ServiceQuery(1799, Protocol::TCP), Service::Name("netrisk")),
        (ServiceQuery(2553, Protocol::UDP), Service::Name("efidiningport")),
        (ServiceQuery(3635, Protocol::UDP), Service::Name("sdo")),
        (ServiceQuery(5164, Protocol::TCP), Service::Name("vpa")),
        (ServiceQuery(6063, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(3928, Protocol::TCP), Service::Name("netboot-pxe")),
        (ServiceQuery(3362, Protocol::TCP), Service::Name("dj-ilm")),
        (ServiceQuery(2908, Protocol::TCP), Service::Name("mao")),
        (ServiceQuery(47624, Protocol::UDP), Service::Name("directplaysrvr")),
        (ServiceQuery(3569, Protocol::TCP), Service::Name("mbg-ctrl")),
        (ServiceQuery(9277, Protocol::TCP), Service::Name("traingpsdata")),
        (ServiceQuery(1711, Protocol::TCP), Service::Name("pptconference")),
        (ServiceQuery(3681, Protocol::UDP), Service::Name("bts-x73")),
        (ServiceQuery(8609, Protocol::TCP), Service::Name("canon-cpp-disc")),
        (ServiceQuery(5901, Protocol::TCP), Service::Name("vnc-1")),
        (ServiceQuery(1931, Protocol::UDP), Service::Name("amdsched")),
        (ServiceQuery(10810, Protocol::TCP), Service::Name("nmc-disc")),
        (ServiceQuery(9284, Protocol::UDP), Service::Name("visd")),
        (ServiceQuery(14250, Protocol::UDP), Service::Name("cps")),
        (ServiceQuery(3017, Protocol::UDP), Service::Name("event_listener")),
        (ServiceQuery(3611, Protocol::TCP), Service::Name("six-degrees")),
        (ServiceQuery(5058, Protocol::TCP), Service::Name("locus-disc")),
        (ServiceQuery(5908, Protocol::TCP), Service::Name("ipsma")),
        (ServiceQuery(3652, Protocol::TCP), Service::Name("vxcrnbuport")),
        (ServiceQuery(2275, Protocol::UDP), Service::Name("ibridge-data")),
        (ServiceQuery(2191, Protocol::UDP), Service::Name("tvbus")),
        (ServiceQuery(4347, Protocol::UDP), Service::Name("lansurveyor")),
        (ServiceQuery(99, Protocol::TCP), Service::Name("metagram")),
        (ServiceQuery(6017, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(6481, Protocol::UDP), Service::Name("servicetags")),
        (ServiceQuery(3152, Protocol::UDP), Service::Name("feitianrockey")),
        (ServiceQuery(4134, Protocol::TCP), Service::Name("nifty-hmi")),
        (ServiceQuery(2667, Protocol::TCP), Service::Name("alarm-clock-s")),
        (ServiceQuery(1510, Protocol::TCP), Service::Name("mvx-lm")),
        (ServiceQuery(3643, Protocol::UDP), Service::Name("audiojuggler")),
        (ServiceQuery(2709, Protocol::TCP), Service::Name("supermon")),
        (ServiceQuery(2314, Protocol::TCP), Service::Name("cr-websystems")),
        (ServiceQuery(8613, Protocol::UDP), Service::Name("canon-bjnp3")),
        (ServiceQuery(3887, Protocol::TCP), Service::Name("ciphire-data")),
        (ServiceQuery(3138, Protocol::UDP), Service::Name("rtnt-2")),
        (ServiceQuery(702, Protocol::TCP), Service::Name("iris-beep")),
        (ServiceQuery(2653, Protocol::TCP), Service::Name("sonus")),
        (ServiceQuery(3344, Protocol::UDP), Service::Name("bnt-manager")),
        (ServiceQuery(2626, Protocol::TCP), Service::Name("gbjd816")),
        (ServiceQuery(1809, Protocol::TCP), Service::Name("oracle-vp1")),
        (ServiceQuery(4445, Protocol::TCP), Service::Name("upnotifyp")),
        (ServiceQuery(3862, Protocol::UDP), Service::Name("giga-pocket")),
        (ServiceQuery(1464, Protocol::TCP), Service::Name("msl_lmd")),
        (ServiceQuery(174, Protocol::UDP), Service::Name("mailq")),
        (ServiceQuery(4241, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2366, Protocol::UDP), Service::Name("qip-login")),
        (ServiceQuery(523, Protocol::TCP), Service::Name("ibm-db2")),
        (ServiceQuery(8765, Protocol::UDP), Service::Name("ultraseek-http")),
        (ServiceQuery(4229, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2510, Protocol::UDP), Service::Name("fjappmgrbulk")),
        (ServiceQuery(828, Protocol::TCP), Service::Name("itm-mcell-s")),
        (ServiceQuery(9995, Protocol::TCP), Service::Name("palace-4")),
        (ServiceQuery(4682, Protocol::UDP), Service::Name("finisar")),
        (ServiceQuery(3096, Protocol::TCP), Service::Name("ndl-aps")),
        (ServiceQuery(1194, Protocol::TCP), Service::Name("openvpn")),
        (ServiceQuery(2244, Protocol::UDP), Service::Name("nmsserver")),
        (ServiceQuery(7338, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(1087, Protocol::UDP), Service::Name("cplscrambler-in")),
        (ServiceQuery(3897, Protocol::TCP), Service::Name("sdo-ssh")),
        (ServiceQuery(770, Protocol::UDP), Service::Name("cadlock")),
        (ServiceQuery(9100, Protocol::UDP), Service::Name("hp-pdl-datastr")),
        (ServiceQuery(20014, Protocol::UDP), Service::Name("opendeploy")),
        (ServiceQuery(38800, Protocol::TCP), Service::Name("sruth")),
        (ServiceQuery(2787, Protocol::UDP), Service::Name("piccolo")),
        (ServiceQuery(2063, Protocol::UDP), Service::Name("icg-bridge")),
        (ServiceQuery(13217, Protocol::TCP), Service::Name("rs-pias")),
        (ServiceQuery(2113, Protocol::TCP), Service::Name("hsl-storm")),
        (ServiceQuery(7277, Protocol::UDP), Service::Name("oma-ilp-s")),
        (ServiceQuery(1036, Protocol::TCP), Service::Name("nsstp")),
        (ServiceQuery(7933, Protocol::TCP), Service::Name("t2-brm")),
        (ServiceQuery(2489, Protocol::UDP), Service::Name("tsilb")),
        (ServiceQuery(5445, Protocol::TCP), Service::Name("smbdirect")),
        (ServiceQuery(4443, Protocol::TCP), Service::Name("pharos")),
        (ServiceQuery(3506, Protocol::TCP), Service::Name("apc-3506")),
        (ServiceQuery(2519, Protocol::TCP), Service::Name("globmsgsvc")),
        (ServiceQuery(1242, Protocol::UDP), Service::Name("nmasoverip")),
        (ServiceQuery(7421, Protocol::TCP), Service::Name("mtportmon")),
        (ServiceQuery(9925, Protocol::TCP), Service::Name("xybrid-cloud")),
        (ServiceQuery(44334, Protocol::TCP), Service::Name("tinyfw")),
        (ServiceQuery(3183, Protocol::UDP), Service::Name("cops-tls")),
        (ServiceQuery(758, Protocol::UDP), Service::Name("nlogin")),
        (ServiceQuery(32768, Protocol::TCP), Service::Name("filenet-tms")),
        (ServiceQuery(4324, Protocol::TCP), Service::Name("balour")),
        (ServiceQuery(4369, Protocol::TCP), Service::Name("epmd")),
        (ServiceQuery(3410, Protocol::TCP), Service::Name("networklenss")),
        (ServiceQuery(5676, Protocol::UDP), Service::Name("raadmin")),
        (ServiceQuery(16444, Protocol::UDP), Service::Name("overnet")),
        (ServiceQuery(407, Protocol::UDP), Service::Name("timbuktu")),
        (ServiceQuery(3892, Protocol::TCP), Service::Name("pcc-image-port")),
        (ServiceQuery(2048, Protocol::TCP), Service::Name("dls-monitor")),
        (ServiceQuery(11175, Protocol::TCP), Service::Name("oemcacao-websvc")),
        (ServiceQuery(4224, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(8005, Protocol::TCP), Service::Name("mxi")),
        (ServiceQuery(280, Protocol::UDP), Service::Name("http-mgmt")),
        (ServiceQuery(3267, Protocol::TCP), Service::Name("ibm-dial-out")),
        (ServiceQuery(6144, Protocol::TCP), Service::Name("statsci1-lm")),
        (ServiceQuery(1282, Protocol::UDP), Service::Name("emperion")),
        (ServiceQuery(2911, Protocol::TCP), Service::Name("blockade")),
        (ServiceQuery(7001, Protocol::UDP), Service::Name("afs3-callback")),
        (ServiceQuery(2270, Protocol::TCP), Service::Name("starschool")),
        (ServiceQuery(4844, Protocol::UDP), Service::Name("gw-log")),
        (ServiceQuery(3163, Protocol::UDP), Service::Name("res-sap")),
        (ServiceQuery(1972, Protocol::TCP), Service::Name("intersys-cache")),
        (ServiceQuery(3691, Protocol::UDP), Service::Name("magaya-network")),
        (ServiceQuery(1482, Protocol::TCP), Service::Name("miteksys-lm")),
        (ServiceQuery(2143, Protocol::TCP), Service::Name("lv-jc")),
        (ServiceQuery(427, Protocol::UDP), Service::Name("svrloc")),
        (ServiceQuery(3447, Protocol::TCP), Service::Name("directnet")),
        (ServiceQuery(3454, Protocol::TCP), Service::Name("mira")),
        (ServiceQuery(3085, Protocol::UDP), Service::Name("pcihreq")),
        (ServiceQuery(3037, Protocol::UDP), Service::Name("hp-san-mgmt")),
        (ServiceQuery(6140, Protocol::UDP), Service::Name("pulsonixnls")),
        (ServiceQuery(9401, Protocol::UDP), Service::Name("sec-t4net-clt")),
        (ServiceQuery(4058, Protocol::UDP), Service::Name("kingfisher")),
        (ServiceQuery(7427, Protocol::TCP), Service::Name("oveadmgr")),
        (ServiceQuery(2502, Protocol::UDP), Service::Name("kentrox-prot")),
        (ServiceQuery(1956, Protocol::UDP), Service::Name("vrtl-vmf-ds")),
        (ServiceQuery(7170, Protocol::TCP), Service::Name("nsrp")),
        (ServiceQuery(525, Protocol::TCP), Service::Name("timed")),
        (ServiceQuery(542, Protocol::TCP), Service::Name("commerce")),
        (ServiceQuery(45, Protocol::TCP), Service::Name("mpm")),
        (ServiceQuery(537, Protocol::TCP), Service::Name("nmsp")),
        (ServiceQuery(2822, Protocol::UDP), Service::Name("ka0wuc")),
        (ServiceQuery(1584, Protocol::UDP), Service::Name("tn-tl-fd2")),
        (ServiceQuery(19411, Protocol::UDP), Service::Name("hp-sca")),
        (ServiceQuery(5679, Protocol::UDP), Service::Name("dccm")),
        (ServiceQuery(3623, Protocol::UDP), Service::Name("haipe-discover")),
        (ServiceQuery(3213, Protocol::TCP), Service::Name("neon24x7")),
        (ServiceQuery(3168, Protocol::UDP), Service::Name("poweronnud")),
        (ServiceQuery(3637, Protocol::TCP), Service::Name("scservp")),
        (ServiceQuery(1473, Protocol::UDP), Service::Name("openmath")),
        (ServiceQuery(2037, Protocol::UDP), Service::Name("applus")),
        (ServiceQuery(4328, Protocol::TCP), Service::Name("jaxer-manager")),
        (ServiceQuery(2166, Protocol::UDP), Service::Name("iwserver")),
        (ServiceQuery(2656, Protocol::TCP), Service::Name("kana")),
        (ServiceQuery(775, Protocol::UDP), Service::Name("acmaint_transd")),
        (ServiceQuery(453, Protocol::UDP), Service::Name("creativeserver")),
        (ServiceQuery(3394, Protocol::TCP), Service::Name("d2k-tapestry2")),
        (ServiceQuery(2070, Protocol::UDP), Service::Name("ah-esp-encap")),
        (ServiceQuery(12009, Protocol::TCP), Service::Name("ghvpn")),
        (ServiceQuery(257, Protocol::UDP), Service::Name("set")),
        (ServiceQuery(3023, Protocol::UDP), Service::Name("magicnotes")),
        (ServiceQuery(7321, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(2395, Protocol::TCP), Service::Name("lan900_remote")),
        (ServiceQuery(25955, Protocol::TCP), Service::Name("bf-master")),
        (ServiceQuery(2272, Protocol::UDP), Service::Name("mmcal")),
        (ServiceQuery(7342, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(5586, Protocol::TCP), Service::Name("att-mt-sms")),
        (ServiceQuery(4936, Protocol::TCP), Service::Name("an-signaling")),
        (ServiceQuery(5050, Protocol::TCP), Service::Name("mmcc")),
        (ServiceQuery(393, Protocol::TCP), Service::Name("dis")),
        (ServiceQuery(4867, Protocol::UDP), Service::Name("unify-debug")),
        (ServiceQuery(2551, Protocol::TCP), Service::Name("isg-uda-server")),
        (ServiceQuery(1767, Protocol::UDP), Service::Name("cft-6")),
        (ServiceQuery(11320, Protocol::UDP), Service::Name("imip-channels")),
        (ServiceQuery(2881, Protocol::UDP), Service::Name("ndsp")),
        (ServiceQuery(133, Protocol::UDP), Service::Name("statsrv")),
        (ServiceQuery(4124, Protocol::TCP), Service::Name("tigv2")),
        (ServiceQuery(8200, Protocol::TCP), Service::Name("trivnet1")),
        (ServiceQuery(5630, Protocol::UDP), Service::Name("precise-comm")),
        (ServiceQuery(902, Protocol::TCP), Service::Name("iss-realsecure")),
        (ServiceQuery(4683, Protocol::UDP), Service::Name("spike")),
        (ServiceQuery(4368, Protocol::TCP), Service::Name("wxbrief")),
        (ServiceQuery(4320, Protocol::TCP), Service::Name("fdt-rcatp")),
        (ServiceQuery(7627, Protocol::TCP), Service::Name("soap-http")),
        (ServiceQuery(2363, Protocol::TCP), Service::Name("mediacntrlnfsd")),
        (ServiceQuery(1084, Protocol::UDP), Service::Name("ansoft-lm-2")),
        (ServiceQuery(9700, Protocol::TCP), Service::Name("board-roar")),
        (ServiceQuery(8026, Protocol::UDP), Service::Name("ca-audit-ds")),
        (ServiceQuery(1569, Protocol::UDP), Service::Name("ets")),
        (ServiceQuery(5316, Protocol::UDP), Service::Name("hpdevms")),
        (ServiceQuery(1350, Protocol::TCP), Service::Name("editbench")),
        (ServiceQuery(7913, Protocol::TCP), Service::Name("qo-secure")),
        (ServiceQuery(2972, Protocol::UDP), Service::Name("pmsm-webrctl")),
        (ServiceQuery(23001, Protocol::TCP), Service::Name("inovaport2")),
        (ServiceQuery(2955, Protocol::TCP), Service::Name("csnotify")),
        (ServiceQuery(2804, Protocol::TCP), Service::Name("dvr-esm")),
        (ServiceQuery(6200, Protocol::TCP), Service::Name("lm-x")),
        (ServiceQuery(3461, Protocol::UDP), Service::Name("edm-stager")),
        (ServiceQuery(910, Protocol::UDP), Service::Name("kink")),
        (ServiceQuery(6142, Protocol::UDP), Service::Name("aspentec-lm")),
        (ServiceQuery(26489, Protocol::TCP), Service::Name("exonet")),
        (ServiceQuery(3388, Protocol::UDP), Service::Name("cbserver")),
        (ServiceQuery(3594, Protocol::TCP), Service::Name("mediaspace")),
        (ServiceQuery(4409, Protocol::TCP), Service::Name("netcabinet-com")),
        (ServiceQuery(1973, Protocol::UDP), Service::Name("dlsrap")),
        (ServiceQuery(2071, Protocol::UDP), Service::Name("acp-port")),
        (ServiceQuery(602, Protocol::TCP), Service::Name("xmlrpc-beep")),
        (ServiceQuery(3879, Protocol::TCP), Service::Name("appss-lm")),
        (ServiceQuery(2518, Protocol::UDP), Service::Name("willy")),
        (ServiceQuery(2498, Protocol::TCP), Service::Name("odn-castraq")),
        (ServiceQuery(1113, Protocol::UDP), Service::Name("ltp-deepspace")),
        (ServiceQuery(3852, Protocol::TCP), Service::Name("sse-app-config")),
        (ServiceQuery(1698, Protocol::UDP), Service::Name("rsvp-encap-1")),
        (ServiceQuery(3086, Protocol::UDP), Service::Name("jdl-dbkitchen")),
        (ServiceQuery(1161, Protocol::TCP), Service::Name("health-polling")),
        (ServiceQuery(4594, Protocol::UDP), Service::Name("ias-session")),
        (ServiceQuery(18184, Protocol::UDP), Service::Name("opsec-lea")),
        (ServiceQuery(2239, Protocol::TCP), Service::Name("imagequery")),
        (ServiceQuery(8082, Protocol::TCP), Service::Name("blackice-alerts")),
        (ServiceQuery(1567, Protocol::UDP), Service::Name("jlicelmd")),
        (ServiceQuery(3288, Protocol::UDP), Service::Name("cops")),
        (ServiceQuery(1186, Protocol::UDP), Service::Name("mysql-cluster")),
        (ServiceQuery(18185, Protocol::UDP), Service::Name("opsec-omi")),
        (ServiceQuery(1854, Protocol::TCP), Service::Name("buddy-draw")),
        (ServiceQuery(2157, Protocol::UDP), Service::Name("xnds")),
        (ServiceQuery(3081, Protocol::TCP), Service::Name("tl1-lv")),
        (ServiceQuery(5404, Protocol::UDP), Service::Name("hpoms-dps-lstn")),
        (ServiceQuery(356, Protocol::TCP), Service::Name("cloanto-net-1")),
        (ServiceQuery(1039, Protocol::TCP), Service::Name("sbl")),
        (ServiceQuery(1268, Protocol::UDP), Service::Name("propel-msgsys")),
        (ServiceQuery(996, Protocol::UDP), Service::Name("vsinet")),
        (ServiceQuery(4246, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3992, Protocol::UDP), Service::Name("bv-ds")),
        (ServiceQuery(3089, Protocol::TCP), Service::Name("ptk-alink")),
        (ServiceQuery(3436, Protocol::UDP), Service::Name("gc-config")),
        (ServiceQuery(2616, Protocol::UDP), Service::Name("appswitch-emp")),
        (ServiceQuery(7355, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(4869, Protocol::TCP), Service::Name("phrelaydbg")),
        (ServiceQuery(4900, Protocol::UDP), Service::Name("hfcs")),
        (ServiceQuery(2075, Protocol::UDP), Service::Name("newlixengine")),
        (ServiceQuery(4663, Protocol::TCP), Service::Name("noteit")),
        (ServiceQuery(3130, Protocol::UDP), Service::Name("squid-ipc")),
        (ServiceQuery(1945, Protocol::TCP), Service::Name("dialogic-elmd")),
        (ServiceQuery(9050, Protocol::TCP), Service::Name("tor-socks")),
        (ServiceQuery(680, Protocol::TCP), Service::Name("entrust-aaas")),
        (ServiceQuery(887, Protocol::UDP), Service::Name("iclcnet_svinfo")),
        (ServiceQuery(2018, Protocol::UDP), Service::Name("rellpack")),
        (ServiceQuery(1491, Protocol::TCP), Service::Name("anynetgateway")),
        (ServiceQuery(1141, Protocol::UDP), Service::Name("mxomss")),
        (ServiceQuery(5116, Protocol::UDP), Service::Name("emb-proj-cmd")),
        (ServiceQuery(6791, Protocol::TCP), Service::Name("hnm")),
        (ServiceQuery(2511, Protocol::TCP), Service::Name("metastorm")),
        (ServiceQuery(5162, Protocol::TCP), Service::Name("snmpssh-trap")),
        (ServiceQuery(3772, Protocol::TCP), Service::Name("ctp")),
        (ServiceQuery(4458, Protocol::UDP), Service::Name("mcp")),
        (ServiceQuery(1152, Protocol::TCP), Service::Name("winpoplanmess")),
        (ServiceQuery(7350, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(2626, Protocol::UDP), Service::Name("gbjd816")),
        (ServiceQuery(6550, Protocol::TCP), Service::Name("fg-sysupdate")),
        (ServiceQuery(89, Protocol::UDP), Service::Name("su-mit-tg")),
        (ServiceQuery(3081, Protocol::UDP), Service::Name("tl1-lv")),
        (ServiceQuery(5363, Protocol::TCP), Service::Name("net-projection")),
        (ServiceQuery(5013, Protocol::TCP), Service::Name("fmpro-v6")),
        (ServiceQuery(1229, Protocol::TCP), Service::Name("zented")),
        (ServiceQuery(608, Protocol::UDP), Service::Name("sift-uft")),
        (ServiceQuery(3617, Protocol::TCP), Service::Name("sharp-server")),
        (ServiceQuery(22951, Protocol::UDP), Service::Name("brf-gw")),
        (ServiceQuery(4451, Protocol::TCP), Service::Name("ctisystemmsg")),
        (ServiceQuery(8130, Protocol::TCP), Service::Name("indigo-vrmi")),
        (ServiceQuery(4537, Protocol::UDP), Service::Name("wssauthsvc")),
        (ServiceQuery(2220, Protocol::UDP), Service::Name("netiq")),
        (ServiceQuery(1907, Protocol::UDP), Service::Name("intrastar")),
        (ServiceQuery(14937, Protocol::UDP), Service::Name("hde-lcesrvr-2")),
        (ServiceQuery(2692, Protocol::TCP), Service::Name("admins-lms")),
        (ServiceQuery(403, Protocol::UDP), Service::Name("decap")),
        (ServiceQuery(1563, Protocol::TCP), Service::Name("cadabra-lm")),
        (ServiceQuery(4163, Protocol::UDP), Service::Name("silverpeakpeer")),
        (ServiceQuery(4456, Protocol::TCP), Service::Name("prchat-server")),
        (ServiceQuery(7471, Protocol::TCP), Service::Name("sttunnel")),
        (ServiceQuery(3859, Protocol::UDP), Service::Name("nav-port")),
        (ServiceQuery(5153, Protocol::TCP), Service::Name("toruxserver")),
        (ServiceQuery(108, Protocol::TCP), Service::Name("snagas")),
        (ServiceQuery(478, Protocol::TCP), Service::Name("spsc")),
        (ServiceQuery(12003, Protocol::UDP), Service::Name("entextmed")),
        (ServiceQuery(530, Protocol::UDP), Service::Name("courier")),
        (ServiceQuery(3645, Protocol::UDP), Service::Name("cyc")),
        (ServiceQuery(11967, Protocol::TCP), Service::Name("sysinfo-sp")),
        (ServiceQuery(6251, Protocol::UDP), Service::Name("tl1-raw-ssl")),
        (ServiceQuery(3577, Protocol::TCP), Service::Name("config-port")),
        (ServiceQuery(3054, Protocol::TCP), Service::Name("amt-cnf-prot")),
        (ServiceQuery(333, Protocol::UDP), Service::Name("texar")),
        (ServiceQuery(3667, Protocol::TCP), Service::Name("infoexch")),
        (ServiceQuery(3241, Protocol::TCP), Service::Name("sysorb")),
        (ServiceQuery(3705, Protocol::TCP), Service::Name("adobeserver-5")),
        (ServiceQuery(2701, Protocol::UDP), Service::Name("sms-rcinfo")),
        (ServiceQuery(581, Protocol::UDP), Service::Name("bdp")),
        (ServiceQuery(15000, Protocol::TCP), Service::Name("hydap")),
        (ServiceQuery(1591, Protocol::TCP), Service::Name("ncpm-pm")),
        (ServiceQuery(3010, Protocol::TCP), Service::Name("gw")),
        (ServiceQuery(2186, Protocol::UDP), Service::Name("gtaua")),
        (ServiceQuery(37, Protocol::UDP), Service::Name("time")),
        (ServiceQuery(8148, Protocol::UDP), Service::Name("isdd")),
        (ServiceQuery(3771, Protocol::TCP), Service::Name("paging-port")),
        (ServiceQuery(620, Protocol::UDP), Service::Name("sco-websrvrmgr")),
        (ServiceQuery(47808, Protocol::TCP), Service::Name("bacnet")),
        (ServiceQuery(3993, Protocol::TCP), Service::Name("bv-agent")),
        (ServiceQuery(4568, Protocol::UDP), Service::Name("bmc-reporting")),
        (ServiceQuery(1207, Protocol::UDP), Service::Name("metasage")),
        (ServiceQuery(1553, Protocol::TCP), Service::Name("sna-cs")),
        (ServiceQuery(5399, Protocol::TCP), Service::Name("securitychase")),
        (ServiceQuery(32768, Protocol::UDP), Service::Name("omad")),
        (ServiceQuery(4359, Protocol::UDP), Service::Name("omabcastltkm")),
        (ServiceQuery(9596, Protocol::UDP), Service::Name("mercury-disc")),
        (ServiceQuery(1837, Protocol::TCP), Service::Name("csoft1")),
        (ServiceQuery(1319, Protocol::UDP), Service::Name("amx-icsp")),
        (ServiceQuery(5868, Protocol::TCP), Service::Name("diameters")),
        (ServiceQuery(15363, Protocol::TCP), Service::Name("3link")),
        (ServiceQuery(2994, Protocol::TCP), Service::Name("veritas-vis2")),
        (ServiceQuery(35357, Protocol::TCP), Service::Name("openstack-id")),
        (ServiceQuery(862, Protocol::UDP), Service::Name("twamp-control")),
        (ServiceQuery(8022, Protocol::UDP), Service::Name("oa-system")),
        (ServiceQuery(2831, Protocol::UDP), Service::Name("silkp3")),
        (ServiceQuery(1139, Protocol::UDP), Service::Name("evm")),
        (ServiceQuery(5969, Protocol::TCP), Service::Name("mppolicy-mgr")),
        (ServiceQuery(3510, Protocol::UDP), Service::Name("xss-port")),
        (ServiceQuery(6464, Protocol::TCP), Service::Name("ieee11073-20701")),
        (ServiceQuery(6320, Protocol::UDP), Service::Name("repsvc")),
        (ServiceQuery(2547, Protocol::TCP), Service::Name("vytalvaultvsmp")),
        (ServiceQuery(32777, Protocol::UDP), Service::Name("sometimes-rpc18")),
        (ServiceQuery(2671, Protocol::UDP), Service::Name("newlixreg")),
        (ServiceQuery(21849, Protocol::TCP), Service::Name("netspeak-cps")),
        (ServiceQuery(1806, Protocol::TCP), Service::Name("musiconline")),
        (ServiceQuery(3272, Protocol::TCP), Service::Name("user-manager")),
        (ServiceQuery(1161, Protocol::UDP), Service::Name("health-polling")),
        (ServiceQuery(3399, Protocol::UDP), Service::Name("csms")),
        (ServiceQuery(3971, Protocol::TCP), Service::Name("lanrevserver")),
        (ServiceQuery(48000, Protocol::UDP), Service::Name("nimcontroller")),
        (ServiceQuery(1059, Protocol::TCP), Service::Name("nimreg")),
        (ServiceQuery(32249, Protocol::UDP), Service::Name("t1distproc60")),
        (ServiceQuery(103, Protocol::TCP), Service::Name("gppitnp")),
        (ServiceQuery(3490, Protocol::TCP), Service::Name("colubris")),
        (ServiceQuery(5254, Protocol::TCP), Service::Name("logcabin")),
        (ServiceQuery(587, Protocol::TCP), Service::Name("submission")),
        (ServiceQuery(2700, Protocol::UDP), Service::Name("tqdata")),
        (ServiceQuery(3557, Protocol::TCP), Service::Name("personalos-001")),
        (ServiceQuery(6253, Protocol::UDP), Service::Name("crip")),
        (ServiceQuery(1154, Protocol::UDP), Service::Name("resacommunity")),
        (ServiceQuery(27960, Protocol::UDP), Service::Name("quake3")),
        (ServiceQuery(3513, Protocol::UDP), Service::Name("arcpd")),
        (ServiceQuery(1448, Protocol::UDP), Service::Name("oc-lm")),
        (ServiceQuery(1041, Protocol::TCP), Service::Name("danf-ak2")),
        (ServiceQuery(378, Protocol::UDP), Service::Name("dsETOS")),
        (ServiceQuery(1994, Protocol::UDP), Service::Name("stun-port")),
        (ServiceQuery(4746, Protocol::TCP), Service::Name("intelliadm-disc")),
        (ServiceQuery(3210, Protocol::UDP), Service::Name("flamenco-proxy")),
        (ServiceQuery(4554, Protocol::UDP), Service::Name("msfrs")),
        (ServiceQuery(11, Protocol::TCP), Service::Name("systat")),
        (ServiceQuery(1426, Protocol::UDP), Service::Name("sas-1")),
        (ServiceQuery(9668, Protocol::TCP), Service::Name("tec5-sdctp")),
        (ServiceQuery(5554, Protocol::UDP), Service::Name("sgi-esphttp")),
        (ServiceQuery(2499, Protocol::TCP), Service::Name("unicontrol")),
        (ServiceQuery(5407, Protocol::UDP), Service::Name("foresyte-clear")),
        (ServiceQuery(163, Protocol::TCP), Service::Name("cmip-man")),
        (ServiceQuery(3087, Protocol::UDP), Service::Name("asoki-sma")),
        (ServiceQuery(4118, Protocol::TCP), Service::Name("netscript")),
        (ServiceQuery(589, Protocol::UDP), Service::Name("eyelink")),
        (ServiceQuery(14414, Protocol::TCP), Service::Name("ca-web-update")),
        (ServiceQuery(4687, Protocol::UDP), Service::Name("nst")),
        (ServiceQuery(23003, Protocol::TCP), Service::Name("inovaport4")),
        (ServiceQuery(3962, Protocol::UDP), Service::Name("sbi-agent")),
        (ServiceQuery(14149, Protocol::UDP), Service::Name("vrts-tdd")),
        (ServiceQuery(59, Protocol::TCP), Service::Name("priv-file")),
        (ServiceQuery(762, Protocol::UDP), Service::Name("quotad")),
        (ServiceQuery(200, Protocol::TCP), Service::Name("src")),
        (ServiceQuery(5786, Protocol::UDP), Service::Name("cisco-redu")),
        (ServiceQuery(12168, Protocol::UDP), Service::Name("cawas")),
        (ServiceQuery(54321, Protocol::UDP), Service::Name("bo2k")),
        (ServiceQuery(4072, Protocol::TCP), Service::Name("zieto-sock")),
        (ServiceQuery(2097, Protocol::UDP), Service::Name("jetformpreview")),
        (ServiceQuery(192, Protocol::UDP), Service::Name("osu-nms")),
        (ServiceQuery(182, Protocol::UDP), Service::Name("audit")),
        (ServiceQuery(2078, Protocol::TCP), Service::Name("tpcsrvr")),
        (ServiceQuery(3251, Protocol::TCP), Service::Name("sysscanner")),
        (ServiceQuery(5745, Protocol::UDP), Service::Name("fcopy-server")),
        (ServiceQuery(14001, Protocol::TCP), Service::Name("sua")),
        (ServiceQuery(3938, Protocol::TCP), Service::Name("dbcontrol_agent")),
        (ServiceQuery(1618, Protocol::UDP), Service::Name("skytelnet")),
        (ServiceQuery(21848, Protocol::TCP), Service::Name("netspeak-acd")),
        (ServiceQuery(1706, Protocol::TCP), Service::Name("jetform")),
        (ServiceQuery(13716, Protocol::TCP), Service::Name("netbackup")),
        (ServiceQuery(3083, Protocol::TCP), Service::Name("tl1-telnet")),
        (ServiceQuery(2339, Protocol::UDP), Service::Name("3com-webview")),
        (ServiceQuery(321, Protocol::UDP), Service::Name("pip")),
        (ServiceQuery(4250, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(21000, Protocol::TCP), Service::Name("irtrans")),
        (ServiceQuery(7022, Protocol::TCP), Service::Name("ctdp")),
        (ServiceQuery(1917, Protocol::TCP), Service::Name("noagent")),
        (ServiceQuery(25, Protocol::UDP), Service::Name("smtp")),
        (ServiceQuery(23457, Protocol::TCP), Service::Name("aequus-alt")),
        (ServiceQuery(5722, Protocol::UDP), Service::Name("msdfsr")),
        (ServiceQuery(6004, Protocol::TCP), Service::Name("X11:4")),
        (ServiceQuery(3644, Protocol::TCP), Service::Name("ssowatch")),
        (ServiceQuery(25004, Protocol::TCP), Service::Name("icl-twobase5")),
        (ServiceQuery(830, Protocol::UDP), Service::Name("netconf-ssh")),
        (ServiceQuery(2717, Protocol::UDP), Service::Name("pn-requester")),
        (ServiceQuery(8294, Protocol::UDP), Service::Name("blp4")),
        (ServiceQuery(4343, Protocol::UDP), Service::Name("unicall")),
        (ServiceQuery(9088, Protocol::TCP), Service::Name("sqlexec")),
        (ServiceQuery(7100, Protocol::UDP), Service::Name("font-service")),
        (ServiceQuery(5437, Protocol::TCP), Service::Name("pmip6-data")),
        (ServiceQuery(2210, Protocol::TCP), Service::Name("noaaport")),
        (ServiceQuery(10020, Protocol::TCP), Service::Name("abb-hw")),
        (ServiceQuery(5137, Protocol::UDP), Service::Name("ctsd")),
        (ServiceQuery(6013, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(3998, Protocol::TCP), Service::Name("dnx")),
        (ServiceQuery(2091, Protocol::TCP), Service::Name("prp")),
        (ServiceQuery(3505, Protocol::UDP), Service::Name("ccmcomm")),
        (ServiceQuery(3763, Protocol::UDP), Service::Name("xo-wave")),
        (ServiceQuery(3318, Protocol::UDP), Service::Name("ssrip")),
        (ServiceQuery(2320, Protocol::TCP), Service::Name("siebel-ns")),
        (ServiceQuery(25009, Protocol::UDP), Service::Name("icl-twobase10")),
        (ServiceQuery(107, Protocol::UDP), Service::Name("rtelnet")),
        (ServiceQuery(1270, Protocol::TCP), Service::Name("ssserver")),
        (ServiceQuery(7308, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(9592, Protocol::TCP), Service::Name("ldgateway")),
        (ServiceQuery(4100, Protocol::TCP), Service::Name("igo-incognito")),
        (ServiceQuery(4325, Protocol::TCP), Service::Name("geognosisman")),
        (ServiceQuery(496, Protocol::UDP), Service::Name("pim-rp-disc")),
        (ServiceQuery(9900, Protocol::TCP), Service::Name("iua")),
        (ServiceQuery(32773, Protocol::UDP), Service::Name("sometimes-rpc10")),
        (ServiceQuery(2916, Protocol::UDP), Service::Name("elvin_server")),
        (ServiceQuery(2399, Protocol::UDP), Service::Name("fmpro-fdal")),
        (ServiceQuery(2334, Protocol::TCP), Service::Name("ace-client")),
        (ServiceQuery(3972, Protocol::TCP), Service::Name("iconp")),
        (ServiceQuery(142, Protocol::TCP), Service::Name("bl-idm")),
        (ServiceQuery(24001, Protocol::TCP), Service::Name("med-fsp-rx")),
        (ServiceQuery(10081, Protocol::TCP), Service::Name("famdc")),
        (ServiceQuery(5343, Protocol::TCP), Service::Name("kfserver")),
        (ServiceQuery(3591, Protocol::TCP), Service::Name("gtrack-server")),
        (ServiceQuery(1046, Protocol::TCP), Service::Name("wfremotertm")),
        (ServiceQuery(84, Protocol::TCP), Service::Name("ctf")),
        (ServiceQuery(1328, Protocol::TCP), Service::Name("ewall")),
        (ServiceQuery(367, Protocol::UDP), Service::Name("mortgageware")),
        (ServiceQuery(9010, Protocol::TCP), Service::Name("sdr")),
        (ServiceQuery(449, Protocol::UDP), Service::Name("as-servermap")),
        (ServiceQuery(4989, Protocol::TCP), Service::Name("parallel")),
        (ServiceQuery(8989, Protocol::UDP), Service::Name("sunwebadmins")),
        (ServiceQuery(3419, Protocol::UDP), Service::Name("softaudit")),
        (ServiceQuery(399, Protocol::TCP), Service::Name("iso-tsap-c2")),
        (ServiceQuery(742, Protocol::TCP), Service::Name("netrcs")),
        (ServiceQuery(3673, Protocol::UDP), Service::Name("mediavault-gui")),
        (ServiceQuery(701, Protocol::TCP), Service::Name("lmp")),
        (ServiceQuery(3177, Protocol::TCP), Service::Name("phonex-port")),
        (ServiceQuery(7018, Protocol::TCP), Service::Name("fisa-svc")),
        (ServiceQuery(3574, Protocol::UDP), Service::Name("dmaf-caster")),
        (ServiceQuery(5696, Protocol::TCP), Service::Name("kmip")),
        (ServiceQuery(3207, Protocol::UDP), Service::Name("vx-auth-port")),
        (ServiceQuery(4050, Protocol::UDP), Service::Name("cisco-wafs")),
        (ServiceQuery(5316, Protocol::TCP), Service::Name("hpdevms")),
        (ServiceQuery(2082, Protocol::UDP), Service::Name("infowave")),
        (ServiceQuery(754, Protocol::UDP), Service::Name("tell")),
        (ServiceQuery(2745, Protocol::UDP), Service::Name("urbisnet")),
        (ServiceQuery(1108, Protocol::TCP), Service::Name("ratio-adp")),
        (ServiceQuery(94, Protocol::TCP), Service::Name("objcall")),
        (ServiceQuery(7283, Protocol::TCP), Service::Name("genstat")),
        (ServiceQuery(2009, Protocol::UDP), Service::Name("whosockami")),
        (ServiceQuery(2983, Protocol::UDP), Service::Name("netplan")),
        (ServiceQuery(801, Protocol::UDP), Service::Name("device")),
        (ServiceQuery(2812, Protocol::UDP), Service::Name("atmtcp")),
        (ServiceQuery(2035, Protocol::UDP), Service::Name("imsldoc")),
        (ServiceQuery(1536, Protocol::TCP), Service::Name("ampr-inter")),
        (ServiceQuery(3791, Protocol::UDP), Service::Name("tvnetworkvideo")),
        (ServiceQuery(3441, Protocol::UDP), Service::Name("connect-client")),
        (ServiceQuery(4100, Protocol::UDP), Service::Name("igo-incognito")),
        (ServiceQuery(616, Protocol::TCP), Service::Name("sco-sysmgr")),
        (ServiceQuery(2571, Protocol::UDP), Service::Name("cecsvc")),
        (ServiceQuery(2160, Protocol::TCP), Service::Name("apc-2160")),
        (ServiceQuery(2471, Protocol::UDP), Service::Name("seaodbc")),
        (ServiceQuery(38201, Protocol::TCP), Service::Name("galaxy7-data")),
        (ServiceQuery(5428, Protocol::UDP), Service::Name("omid")),
        (ServiceQuery(48, Protocol::TCP), Service::Name("auditd")),
        (ServiceQuery(2356, Protocol::UDP), Service::Name("gxtelmd")),
        (ServiceQuery(1448, Protocol::TCP), Service::Name("oc-lm")),
        (ServiceQuery(2255, Protocol::UDP), Service::Name("vrtp")),
        (ServiceQuery(3505, Protocol::TCP), Service::Name("ccmcomm")),
        (ServiceQuery(8470, Protocol::TCP), Service::Name("cisco-avp")),
        (ServiceQuery(3052, Protocol::TCP), Service::Name("powerchute")),
        (ServiceQuery(1997, Protocol::TCP), Service::Name("gdp-port")),
        (ServiceQuery(9632, Protocol::UDP), Service::Name("mc-comm")),
        (ServiceQuery(2960, Protocol::TCP), Service::Name("dfoxserver")),
        (ServiceQuery(6011, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(8805, Protocol::TCP), Service::Name("pfcp")),
        (ServiceQuery(3874, Protocol::TCP), Service::Name("sixxsconfig")),
        (ServiceQuery(112, Protocol::UDP), Service::Name("mcidas")),
        (ServiceQuery(3396, Protocol::UDP), Service::Name("printer_agent")),
        (ServiceQuery(1492, Protocol::TCP), Service::Name("stone-design-1")),
        (ServiceQuery(1229, Protocol::UDP), Service::Name("zented")),
        (ServiceQuery(46, Protocol::TCP), Service::Name("mpm-snd")),
        (ServiceQuery(7547, Protocol::TCP), Service::Name("cwmp")),
        (ServiceQuery(8181, Protocol::TCP), Service::Name("intermapper")),
        (ServiceQuery(8199, Protocol::UDP), Service::Name("vvr-data")),
        (ServiceQuery(20031, Protocol::UDP), Service::Name("bakbonenetvault")),
        (ServiceQuery(6850, Protocol::UDP), Service::Name("iccrushmore")),
        (ServiceQuery(9899, Protocol::UDP), Service::Name("sctp-tunneling")),
        (ServiceQuery(4046, Protocol::TCP), Service::Name("acp-proto")),
        (ServiceQuery(2632, Protocol::TCP), Service::Name("irdg-post")),
        (ServiceQuery(742, Protocol::UDP), Service::Name("netrcs")),
        (ServiceQuery(3148, Protocol::TCP), Service::Name("nm-game-admin")),
        (ServiceQuery(3909, Protocol::UDP), Service::Name("surfcontrolcpa")),
        (ServiceQuery(2759, Protocol::UDP), Service::Name("apollo-gms")),
        (ServiceQuery(8008, Protocol::TCP), Service::Name("http")),
        (ServiceQuery(5569, Protocol::TCP), Service::Name("rdmnet-ctrl")),
        (ServiceQuery(9898, Protocol::TCP), Service::Name("monkeycom")),
        (ServiceQuery(1200, Protocol::TCP), Service::Name("scol")),
        (ServiceQuery(3378, Protocol::TCP), Service::Name("wsicopy")),
        (ServiceQuery(1636, Protocol::TCP), Service::Name("isdc")),
        (ServiceQuery(4272, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(27006, Protocol::UDP), Service::Name("flex-lm")),
        (ServiceQuery(5771, Protocol::TCP), Service::Name("netagent")),
        (ServiceQuery(2797, Protocol::TCP), Service::Name("esp-encap")),
        (ServiceQuery(6547, Protocol::TCP), Service::Name("powerchuteplus")),
        (ServiceQuery(1585, Protocol::UDP), Service::Name("intv")),
        (ServiceQuery(3180, Protocol::TCP), Service::Name("mc-brk-srv")),
        (ServiceQuery(2352, Protocol::TCP), Service::Name("pslserver")),
        (ServiceQuery(7724, Protocol::TCP), Service::Name("nsdeepfreezectl")),
        (ServiceQuery(7201, Protocol::TCP), Service::Name("dlip")),
        (ServiceQuery(7025, Protocol::UDP), Service::Name("vmsvc-2")),
        (ServiceQuery(6667, Protocol::UDP), Service::Name("ircu")),
        (ServiceQuery(3273, Protocol::TCP), Service::Name("sxmp")),
        (ServiceQuery(10001, Protocol::TCP), Service::Name("scp-config")),
        (ServiceQuery(7508, Protocol::TCP), Service::Name("adcp")),
        (ServiceQuery(4094, Protocol::UDP), Service::Name("sysrqd")),
        (ServiceQuery(68, Protocol::UDP), Service::Name("dhcpc")),
        (ServiceQuery(2414, Protocol::TCP), Service::Name("beeyond")),
        (ServiceQuery(2738, Protocol::TCP), Service::Name("ndl-tcp-ois-gw")),
        (ServiceQuery(3282, Protocol::UDP), Service::Name("datusorb")),
        (ServiceQuery(1298, Protocol::TCP), Service::Name("lpcp")),
        (ServiceQuery(2204, Protocol::UDP), Service::Name("b2-license")),
        (ServiceQuery(9400, Protocol::UDP), Service::Name("sec-t4net-srv")),
        (ServiceQuery(2887, Protocol::UDP), Service::Name("aironetddp")),
        (ServiceQuery(1808, Protocol::TCP), Service::Name("oracle-vp2")),
        (ServiceQuery(120, Protocol::UDP), Service::Name("cfdptkt")),
        (ServiceQuery(3928, Protocol::UDP), Service::Name("netboot-pxe")),
        (ServiceQuery(9951, Protocol::UDP), Service::Name("apc-9951")),
        (ServiceQuery(6501, Protocol::TCP), Service::Name("boks_servc")),
        (ServiceQuery(2131, Protocol::UDP), Service::Name("avantageb2b")),
        (ServiceQuery(2081, Protocol::TCP), Service::Name("kme-trap-port")),
        (ServiceQuery(24000, Protocol::UDP), Service::Name("med-ltp")),
        (ServiceQuery(4121, Protocol::UDP), Service::Name("e-builder")),
        (ServiceQuery(5580, Protocol::UDP), Service::Name("tmosms0")),
        (ServiceQuery(1101, Protocol::TCP), Service::Name("pt2-discover")),
        (ServiceQuery(4081, Protocol::TCP), Service::Name("lorica-in-sec")),
        (ServiceQuery(6488, Protocol::TCP), Service::Name("sun-sr-jmx")),
        (ServiceQuery(2238, Protocol::UDP), Service::Name("aviva-sna")),
        (ServiceQuery(3202, Protocol::UDP), Service::Name("intraintra")),
        (ServiceQuery(2854, Protocol::TCP), Service::Name("infomover")),
        (ServiceQuery(1064, Protocol::TCP), Service::Name("jstel")),
        (ServiceQuery(2096, Protocol::TCP), Service::Name("nbx-dir")),
        (ServiceQuery(1051, Protocol::TCP), Service::Name("optima-vnet")),
        (ServiceQuery(1826, Protocol::UDP), Service::Name("ardt")),
        (ServiceQuery(9878, Protocol::TCP), Service::Name("kca-service")),
        (ServiceQuery(2868, Protocol::UDP), Service::Name("npep-messaging")),
        (ServiceQuery(18184, Protocol::TCP), Service::Name("opsec-lea")),
        (ServiceQuery(5031, Protocol::TCP), Service::Name("dmp")),
        (ServiceQuery(636, Protocol::TCP), Service::Name("ldapssl")),
        (ServiceQuery(7797, Protocol::UDP), Service::Name("pnet-conn")),
        (ServiceQuery(442, Protocol::TCP), Service::Name("cvc_hostd")),
        (ServiceQuery(10023, Protocol::TCP), Service::Name("cefd-vmp")),
        (ServiceQuery(9800, Protocol::UDP), Service::Name("davsrc")),
        (ServiceQuery(242, Protocol::UDP), Service::Name("direct")),
        (ServiceQuery(3518, Protocol::UDP), Service::Name("artifact-msg")),
        (ServiceQuery(2582, Protocol::UDP), Service::Name("argis-ds")),
        (ServiceQuery(4535, Protocol::TCP), Service::Name("ehs")),
        (ServiceQuery(7668, Protocol::TCP), Service::Name("authoritygate")),
        (ServiceQuery(7707, Protocol::TCP), Service::Name("sync-em7")),
        (ServiceQuery(148, Protocol::UDP), Service::Name("cronus")),
        (ServiceQuery(5007, Protocol::UDP), Service::Name("wsm-server-ssl")),
        (ServiceQuery(96, Protocol::TCP), Service::Name("dixie")),
        (ServiceQuery(6107, Protocol::TCP), Service::Name("etc-control")),
        (ServiceQuery(2864, Protocol::UDP), Service::Name("astromed-main")),
        (ServiceQuery(6133, Protocol::UDP), Service::Name("nbt-wol")),
        (ServiceQuery(5191, Protocol::TCP), Service::Name("aol-1")),
        (ServiceQuery(4869, Protocol::UDP), Service::Name("phrelaydbg")),
        (ServiceQuery(2243, Protocol::TCP), Service::Name("magicom")),
        (ServiceQuery(1391, Protocol::TCP), Service::Name("iclpv-sas")),
        (ServiceQuery(3749, Protocol::UDP), Service::Name("cimtrak")),
        (ServiceQuery(1582, Protocol::UDP), Service::Name("msims")),
        (ServiceQuery(2520, Protocol::UDP), Service::Name("pvsw")),
        (ServiceQuery(7025, Protocol::TCP), Service::Name("vmsvc-2")),
        (ServiceQuery(4014, Protocol::TCP), Service::Name("taiclock")),
        (ServiceQuery(1106, Protocol::TCP), Service::Name("isoipsigport-1")),
        (ServiceQuery(3659, Protocol::UDP), Service::Name("apple-sasl")),
        (ServiceQuery(5766, Protocol::UDP), Service::Name("openmailns")),
        (ServiceQuery(2683, Protocol::UDP), Service::Name("ncdloadbalance")),
        (ServiceQuery(1051, Protocol::UDP), Service::Name("optima-vnet")),
        (ServiceQuery(1737, Protocol::UDP), Service::Name("ultimad")),
        (ServiceQuery(32767, Protocol::UDP), Service::Name("filenet-powsrm")),
        (ServiceQuery(6657, Protocol::TCP), Service::Name("palcom-disc")),
        (ServiceQuery(13705, Protocol::TCP), Service::Name("netbackup")),
        (ServiceQuery(2586, Protocol::UDP), Service::Name("netx-agent")),
        (ServiceQuery(2849, Protocol::TCP), Service::Name("fxp")),
        (ServiceQuery(2657, Protocol::UDP), Service::Name("sns-dispatcher")),
        (ServiceQuery(2663, Protocol::TCP), Service::Name("bintec-tapi")),
        (ServiceQuery(2134, Protocol::UDP), Service::Name("avenue")),
        (ServiceQuery(9396, Protocol::TCP), Service::Name("fjinvmgr")),
        (ServiceQuery(4306, Protocol::UDP), Service::Name("pinghgl")),
        (ServiceQuery(4999, Protocol::TCP), Service::Name("hfcs-manager")),
        (ServiceQuery(4104, Protocol::UDP), Service::Name("brlp-3")),
        (ServiceQuery(3128, Protocol::TCP), Service::Name("squid-http")),
        (ServiceQuery(4536, Protocol::UDP), Service::Name("ehs-ssl")),
        (ServiceQuery(2058, Protocol::UDP), Service::Name("newwavesearch")),
        (ServiceQuery(21590, Protocol::UDP), Service::Name("vofr-gateway")),
        (ServiceQuery(3297, Protocol::UDP), Service::Name("cytel-lm")),
        (ServiceQuery(41230, Protocol::TCP), Service::Name("z-wave-s")),
        (ServiceQuery(4009, Protocol::TCP), Service::Name("chimera-hwm")),
        (ServiceQuery(1112, Protocol::UDP), Service::Name("icp")),
        (ServiceQuery(2511, Protocol::UDP), Service::Name("metastorm")),
        (ServiceQuery(2137, Protocol::UDP), Service::Name("connect")),
        (ServiceQuery(2882, Protocol::UDP), Service::Name("ndtp")),
        (ServiceQuery(2028, Protocol::UDP), Service::Name("submitserver")),
        (ServiceQuery(9007, Protocol::UDP), Service::Name("ogs-client")),
        (ServiceQuery(3929, Protocol::UDP), Service::Name("smauth-port")),
        (ServiceQuery(5783, Protocol::UDP), Service::Name("3par-mgmt-ssl")),
        (ServiceQuery(3435, Protocol::TCP), Service::Name("pacom")),
        (ServiceQuery(1097, Protocol::TCP), Service::Name("sunclustermgr")),
        (ServiceQuery(1346, Protocol::UDP), Service::Name("alta-ana-lm")),
        (ServiceQuery(830, Protocol::TCP), Service::Name("netconf-ssh")),
        (ServiceQuery(10125, Protocol::TCP), Service::Name("cimple")),
        (ServiceQuery(19220, Protocol::TCP), Service::Name("cora")),
        (ServiceQuery(3529, Protocol::TCP), Service::Name("jboss-iiop-ssl")),
        (ServiceQuery(2663, Protocol::UDP), Service::Name("bintec-tapi")),
        (ServiceQuery(2225, Protocol::UDP), Service::Name("rcip-itu")),
        (ServiceQuery(4546, Protocol::UDP), Service::Name("sf-lm")),
        (ServiceQuery(3294, Protocol::TCP), Service::Name("fg-gip")),
        (ServiceQuery(2046, Protocol::UDP), Service::Name("sdfunc")),
        (ServiceQuery(2385, Protocol::UDP), Service::Name("sd-data")),
        (ServiceQuery(3015, Protocol::UDP), Service::Name("nati-dstp")),
        (ServiceQuery(5992, Protocol::TCP), Service::Name("consul-insight")),
        (ServiceQuery(8474, Protocol::UDP), Service::Name("noteshare")),
        (ServiceQuery(613, Protocol::UDP), Service::Name("hmmp-op")),
        (ServiceQuery(9909, Protocol::UDP), Service::Name("domaintime")),
        (ServiceQuery(4841, Protocol::TCP), Service::Name("quosa")),
        (ServiceQuery(11321, Protocol::UDP), Service::Name("arena-server")),
        (ServiceQuery(2923, Protocol::TCP), Service::Name("wta-wsp-wtp-s")),
        (ServiceQuery(2763, Protocol::TCP), Service::Name("desktop-dna")),
        (ServiceQuery(1560, Protocol::UDP), Service::Name("asci-val")),
        (ServiceQuery(173, Protocol::UDP), Service::Name("xyplex-mux")),
        (ServiceQuery(5416, Protocol::UDP), Service::Name("sns-gateway")),
        (ServiceQuery(135, Protocol::UDP), Service::Name("msrpc")),
        (ServiceQuery(9312, Protocol::TCP), Service::Name("sphinxapi")),
        (ServiceQuery(2685, Protocol::UDP), Service::Name("mpnjsocl")),
        (ServiceQuery(369, Protocol::UDP), Service::Name("rpc2portmap")),
        (ServiceQuery(4182, Protocol::UDP), Service::Name("pcptcpservice")),
        (ServiceQuery(319, Protocol::UDP), Service::Name("ptp-event")),
        (ServiceQuery(6029, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(7744, Protocol::UDP), Service::Name("raqmon-pdu")),
        (ServiceQuery(1893, Protocol::TCP), Service::Name("elad")),
        (ServiceQuery(4053, Protocol::UDP), Service::Name("ccu-comm-1")),
        (ServiceQuery(3060, Protocol::UDP), Service::Name("interserver")),
        (ServiceQuery(1321, Protocol::TCP), Service::Name("pip")),
        (ServiceQuery(2485, Protocol::TCP), Service::Name("netobjects1")),
        (ServiceQuery(2502, Protocol::TCP), Service::Name("kentrox-prot")),
        (ServiceQuery(1070, Protocol::UDP), Service::Name("gmrupdateserv")),
        (ServiceQuery(2803, Protocol::TCP), Service::Name("btprjctrl")),
        (ServiceQuery(13216, Protocol::UDP), Service::Name("bcslogc")),
        (ServiceQuery(47557, Protocol::TCP), Service::Name("dbbrowse")),
        (ServiceQuery(1063, Protocol::UDP), Service::Name("kyoceranetdev")),
        (ServiceQuery(3790, Protocol::UDP), Service::Name("quickbooksrds")),
        (ServiceQuery(6043, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(3865, Protocol::TCP), Service::Name("xpl")),
        (ServiceQuery(4592, Protocol::UDP), Service::Name("hrpd-ith-at-an")),
        (ServiceQuery(8208, Protocol::TCP), Service::Name("lm-webwatcher")),
        (ServiceQuery(1444, Protocol::TCP), Service::Name("marcam-lm")),
        (ServiceQuery(4669, Protocol::UDP), Service::Name("eportcommdata")),
        (ServiceQuery(476, Protocol::TCP), Service::Name("tn-tl-fd1")),
        (ServiceQuery(4201, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(8954, Protocol::UDP), Service::Name("cumulus-admin")),
        (ServiceQuery(5409, Protocol::TCP), Service::Name("salient-dtasrv")),
        (ServiceQuery(3233, Protocol::UDP), Service::Name("whisker")),
        (ServiceQuery(2819, Protocol::UDP), Service::Name("fc-faultnotify")),
        (ServiceQuery(3970, Protocol::TCP), Service::Name("lanrevagent")),
        (ServiceQuery(2959, Protocol::UDP), Service::Name("rmopagt")),
        (ServiceQuery(8131, Protocol::UDP), Service::Name("indigo-vbcp")),
        (ServiceQuery(1524, Protocol::UDP), Service::Name("ingreslock")),
        (ServiceQuery(2741, Protocol::UDP), Service::Name("tsb")),
        (ServiceQuery(5428, Protocol::TCP), Service::Name("telaconsole")),
        (ServiceQuery(898, Protocol::TCP), Service::Name("sun-manageconsole")),
        (ServiceQuery(1600, Protocol::TCP), Service::Name("issd")),
        (ServiceQuery(3877, Protocol::TCP), Service::Name("xmpcr-interface")),
        (ServiceQuery(7673, Protocol::TCP), Service::Name("imqstomps")),
        (ServiceQuery(410, Protocol::TCP), Service::Name("decladebug")),
        (ServiceQuery(1102, Protocol::TCP), Service::Name("adobeserver-1")),
        (ServiceQuery(10008, Protocol::TCP), Service::Name("octopus")),
        (ServiceQuery(6625, Protocol::TCP), Service::Name("datascaler-ctl")),
        (ServiceQuery(1939, Protocol::UDP), Service::Name("jvserver")),
        (ServiceQuery(3102, Protocol::TCP), Service::Name("sl-mon")),
        (ServiceQuery(1210, Protocol::UDP), Service::Name("eoss")),
        (ServiceQuery(3967, Protocol::UDP), Service::Name("ppsms")),
        (ServiceQuery(2685, Protocol::TCP), Service::Name("mpnjsocl")),
        (ServiceQuery(1748, Protocol::UDP), Service::Name("oracle-em1")),
        (ServiceQuery(2185, Protocol::UDP), Service::Name("onbase-dds")),
        (ServiceQuery(3831, Protocol::TCP), Service::Name("dvapps")),
        (ServiceQuery(13216, Protocol::TCP), Service::Name("bcslogc")),
        (ServiceQuery(35001, Protocol::TCP), Service::Name("rt-viewer")),
        (ServiceQuery(3269, Protocol::UDP), Service::Name("msft-gc-ssl")),
        (ServiceQuery(5100, Protocol::UDP), Service::Name("socalia")),
        (ServiceQuery(8793, Protocol::TCP), Service::Name("acd-pm")),
        (ServiceQuery(3967, Protocol::TCP), Service::Name("ppsms")),
        (ServiceQuery(3648, Protocol::TCP), Service::Name("fjcp")),
        (ServiceQuery(4742, Protocol::TCP), Service::Name("sicct")),
        (ServiceQuery(671, Protocol::UDP), Service::Name("vacdsm-app")),
        (ServiceQuery(2927, Protocol::TCP), Service::Name("unimobilectrl")),
        (ServiceQuery(2483, Protocol::TCP), Service::Name("ttc")),
        (ServiceQuery(1951, Protocol::UDP), Service::Name("bcs-lmserver")),
        (ServiceQuery(7010, Protocol::UDP), Service::Name("ups-onlinet")),
        (ServiceQuery(114, Protocol::TCP), Service::Name("audionews")),
        (ServiceQuery(4244, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2285, Protocol::UDP), Service::Name("lnvmailmon")),
        (ServiceQuery(7353, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(5673, Protocol::UDP), Service::Name("jms")),
        (ServiceQuery(12006, Protocol::UDP), Service::Name("dbisamserver2")),
        (ServiceQuery(1029, Protocol::TCP), Service::Name("ms-lsa")),
        (ServiceQuery(1157, Protocol::TCP), Service::Name("iascontrol")),
        (ServiceQuery(5010, Protocol::UDP), Service::Name("telelpathstart")),
        (ServiceQuery(1915, Protocol::UDP), Service::Name("facelink")),
        (ServiceQuery(19191, Protocol::UDP), Service::Name("opsec-uaa")),
        (ServiceQuery(4691, Protocol::TCP), Service::Name("mtn")),
        (ServiceQuery(87, Protocol::TCP), Service::Name("priv-term-l")),
        (ServiceQuery(3473, Protocol::UDP), Service::Name("jaugsremotec-2")),
        (ServiceQuery(3120, Protocol::UDP), Service::Name("d2000webserver")),
        (ServiceQuery(6133, Protocol::TCP), Service::Name("nbt-wol")),
        (ServiceQuery(5050, Protocol::UDP), Service::Name("mmcc")),
        (ServiceQuery(1587, Protocol::TCP), Service::Name("pra_elmd")),
        (ServiceQuery(11, Protocol::UDP), Service::Name("systat")),
        (ServiceQuery(2732, Protocol::UDP), Service::Name("g5m")),
        (ServiceQuery(370, Protocol::TCP), Service::Name("codaauth2")),
        (ServiceQuery(4183, Protocol::UDP), Service::Name("gmmp")),
        (ServiceQuery(110, Protocol::TCP), Service::Name("pop3")),
        (ServiceQuery(4246, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1311, Protocol::UDP), Service::Name("rxmon")),
        (ServiceQuery(3723, Protocol::UDP), Service::Name("sychrond")),
        (ServiceQuery(3136, Protocol::UDP), Service::Name("grubd")),
        (ServiceQuery(4092, Protocol::UDP), Service::Name("ewdgs")),
        (ServiceQuery(459, Protocol::UDP), Service::Name("ampr-rcmd")),
        (ServiceQuery(419, Protocol::TCP), Service::Name("ariel1")),
        (ServiceQuery(2359, Protocol::TCP), Service::Name("flukeserver")),
        (ServiceQuery(2402, Protocol::UDP), Service::Name("taskmaster2000")),
        (ServiceQuery(498, Protocol::UDP), Service::Name("siam")),
        (ServiceQuery(6701, Protocol::UDP), Service::Name("kti-icad-srvr")),
        (ServiceQuery(3952, Protocol::TCP), Service::Name("i3-sessionmgr")),
        (ServiceQuery(4076, Protocol::TCP), Service::Name("seraph")),
        (ServiceQuery(2370, Protocol::TCP), Service::Name("l3-hbmon")),
        (ServiceQuery(1206, Protocol::TCP), Service::Name("anthony-data")),
        (ServiceQuery(27350, Protocol::TCP), Service::Name("mdcs-scheduler")),
        (ServiceQuery(2021, Protocol::UDP), Service::Name("xinuexpansion1")),
        (ServiceQuery(1149, Protocol::TCP), Service::Name("bvtsonar")),
        (ServiceQuery(365, Protocol::TCP), Service::Name("dtk")),
        (ServiceQuery(5581, Protocol::TCP), Service::Name("tmosms1")),
        (ServiceQuery(5052, Protocol::TCP), Service::Name("ita-manager")),
        (ServiceQuery(667, Protocol::TCP), Service::Name("disclose")),
        (ServiceQuery(45966, Protocol::UDP), Service::Name("ssr-servermgr")),
        (ServiceQuery(1196, Protocol::UDP), Service::Name("netmagic")),
        (ServiceQuery(2000, Protocol::UDP), Service::Name("cisco-sccp")),
        (ServiceQuery(4065, Protocol::TCP), Service::Name("avanti_cdp")),
        (ServiceQuery(997, Protocol::TCP), Service::Name("maitrd")),
        (ServiceQuery(1660, Protocol::TCP), Service::Name("skip-mc-gikreq")),
        (ServiceQuery(2855, Protocol::TCP), Service::Name("msrp")),
        (ServiceQuery(1098, Protocol::TCP), Service::Name("rmiactivation")),
        (ServiceQuery(1191, Protocol::TCP), Service::Name("gpfs")),
        (ServiceQuery(2586, Protocol::TCP), Service::Name("netx-agent")),
        (ServiceQuery(1950, Protocol::UDP), Service::Name("ismaeasdaqtest")),
        (ServiceQuery(8800, Protocol::TCP), Service::Name("sunwebadmin")),
        (ServiceQuery(3355, Protocol::TCP), Service::Name("ordinox-dbase")),
        (ServiceQuery(2805, Protocol::TCP), Service::Name("wta-wsp-s")),
        (ServiceQuery(47624, Protocol::TCP), Service::Name("directplaysrvr")),
        (ServiceQuery(3140, Protocol::TCP), Service::Name("ariliamulti")),
        (ServiceQuery(3634, Protocol::UDP), Service::Name("hlibmgr")),
        (ServiceQuery(1435, Protocol::TCP), Service::Name("ibm-cics")),
        (ServiceQuery(1273, Protocol::TCP), Service::Name("emc-gateway")),
        (ServiceQuery(17224, Protocol::TCP), Service::Name("trdp-pd")),
        (ServiceQuery(1367, Protocol::UDP), Service::Name("dcs")),
        (ServiceQuery(999, Protocol::UDP), Service::Name("applix")),
        (ServiceQuery(8118, Protocol::TCP), Service::Name("privoxy")),
        (ServiceQuery(27374, Protocol::TCP), Service::Name("subseven")),
        (ServiceQuery(566, Protocol::UDP), Service::Name("streettalk")),
        (ServiceQuery(3050, Protocol::TCP), Service::Name("gds_db")),
        (ServiceQuery(681, Protocol::UDP), Service::Name("entrust-aams")),
        (ServiceQuery(6344, Protocol::TCP), Service::Name("streletz")),
        (ServiceQuery(3940, Protocol::UDP), Service::Name("xecp-node")),
        (ServiceQuery(1246, Protocol::TCP), Service::Name("payrouter")),
        (ServiceQuery(1384, Protocol::TCP), Service::Name("os-licman")),
        (ServiceQuery(1708, Protocol::UDP), Service::Name("gat-lmd")),
        (ServiceQuery(4269, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(546, Protocol::UDP), Service::Name("dhcpv6-client")),
        (ServiceQuery(24676, Protocol::TCP), Service::Name("canditv")),
        (ServiceQuery(8990, Protocol::TCP), Service::Name("http-wmap")),
        (ServiceQuery(4340, Protocol::UDP), Service::Name("gaia")),
        (ServiceQuery(1612, Protocol::TCP), Service::Name("netbill-trans")),
        (ServiceQuery(3093, Protocol::UDP), Service::Name("rapidmq-center")),
        (ServiceQuery(4342, Protocol::UDP), Service::Name("lisp-control")),
        (ServiceQuery(5814, Protocol::UDP), Service::Name("spt-automation")),
        (ServiceQuery(710, Protocol::TCP), Service::Name("entrust-ash")),
        (ServiceQuery(5632, Protocol::UDP), Service::Name("pcanywherestat")),
        (ServiceQuery(2620, Protocol::TCP), Service::Name("lpsrecommender")),
        (ServiceQuery(3274, Protocol::TCP), Service::Name("ordinox-server")),
        (ServiceQuery(2970, Protocol::UDP), Service::Name("index-net")),
        (ServiceQuery(4666, Protocol::UDP), Service::Name("edonkey")),
        (ServiceQuery(1784, Protocol::TCP), Service::Name("finle-lm")),
        (ServiceQuery(665, Protocol::UDP), Service::Name("sun-dr")),
        (ServiceQuery(3883, Protocol::UDP), Service::Name("vrpn")),
        (ServiceQuery(2870, Protocol::TCP), Service::Name("daishi")),
        (ServiceQuery(2413, Protocol::TCP), Service::Name("orion-rmi-reg")),
        (ServiceQuery(7311, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(3885, Protocol::TCP), Service::Name("topflow-ssl")),
        (ServiceQuery(3295, Protocol::TCP), Service::Name("dyniplookup")),
        (ServiceQuery(524, Protocol::TCP), Service::Name("ncp")),
        (ServiceQuery(2810, Protocol::TCP), Service::Name("netsteward")),
        (ServiceQuery(1541, Protocol::UDP), Service::Name("rds2")),
        (ServiceQuery(2148, Protocol::TCP), Service::Name("veritas-ucl")),
        (ServiceQuery(3311, Protocol::TCP), Service::Name("mcns-tel-ret")),
        (ServiceQuery(3044, Protocol::TCP), Service::Name("epp")),
        (ServiceQuery(6048, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(10002, Protocol::UDP), Service::Name("documentum")),
        (ServiceQuery(1399, Protocol::TCP), Service::Name("cadkey-licman")),
        (ServiceQuery(3568, Protocol::TCP), Service::Name("oap-s")),
        (ServiceQuery(3484, Protocol::UDP), Service::Name("gbs-stp")),
        (ServiceQuery(7473, Protocol::UDP), Service::Name("rise")),
        (ServiceQuery(2003, Protocol::UDP), Service::Name("brutus")),
        (ServiceQuery(3761, Protocol::UDP), Service::Name("gsakmp")),
        (ServiceQuery(3468, Protocol::TCP), Service::Name("ttcmremotectrl")),
        (ServiceQuery(1436, Protocol::TCP), Service::Name("sas-2")),
        (ServiceQuery(414, Protocol::UDP), Service::Name("infoseek")),
        (ServiceQuery(507, Protocol::UDP), Service::Name("crs")),
        (ServiceQuery(1383, Protocol::UDP), Service::Name("gwha")),
        (ServiceQuery(4107, Protocol::TCP), Service::Name("j-ac")),
        (ServiceQuery(12002, Protocol::TCP), Service::Name("entexthigh")),
        (ServiceQuery(2906, Protocol::UDP), Service::Name("caller9")),
        (ServiceQuery(4885, Protocol::UDP), Service::Name("abbs")),
        (ServiceQuery(1504, Protocol::TCP), Service::Name("evb-elm")),
        (ServiceQuery(3379, Protocol::TCP), Service::Name("socorfs")),
        (ServiceQuery(2712, Protocol::TCP), Service::Name("aocp")),
        (ServiceQuery(1472, Protocol::UDP), Service::Name("csdm")),
        (ServiceQuery(2912, Protocol::TCP), Service::Name("epicon")),
        (ServiceQuery(9801, Protocol::TCP), Service::Name("sstp-2")),
        (ServiceQuery(2365, Protocol::TCP), Service::Name("dbref")),
        (ServiceQuery(2569, Protocol::UDP), Service::Name("sonuscallsig")),
        (ServiceQuery(1914, Protocol::UDP), Service::Name("elm-momentum")),
        (ServiceQuery(2749, Protocol::UDP), Service::Name("fjippol-cnsl")),
        (ServiceQuery(6075, Protocol::TCP), Service::Name("dpm-acm")),
        (ServiceQuery(2359, Protocol::UDP), Service::Name("flukeserver")),
        (ServiceQuery(4747, Protocol::TCP), Service::Name("buschtrommel")),
        (ServiceQuery(4155, Protocol::TCP), Service::Name("bzr")),
        (ServiceQuery(5415, Protocol::TCP), Service::Name("ns-server")),
        (ServiceQuery(5080, Protocol::UDP), Service::Name("onscreen")),
        (ServiceQuery(3652, Protocol::UDP), Service::Name("vxcrnbuport")),
        (ServiceQuery(4690, Protocol::TCP), Service::Name("prelude")),
        (ServiceQuery(45514, Protocol::TCP), Service::Name("cloudcheck")),
        (ServiceQuery(2099, Protocol::UDP), Service::Name("h2250-annex-g")),
        (ServiceQuery(4563, Protocol::TCP), Service::Name("amahi-anywhere")),
        (ServiceQuery(780, Protocol::TCP), Service::Name("wpgs")),
        (ServiceQuery(5270, Protocol::UDP), Service::Name("xmp")),
        (ServiceQuery(9205, Protocol::UDP), Service::Name("wap-vcal")),
        (ServiceQuery(5071, Protocol::TCP), Service::Name("powerschool")),
        (ServiceQuery(4013, Protocol::UDP), Service::Name("acl-manager")),
        (ServiceQuery(3049, Protocol::TCP), Service::Name("cfs")),
        (ServiceQuery(9083, Protocol::TCP), Service::Name("emc-pp-mgmtsvc")),
        (ServiceQuery(533, Protocol::TCP), Service::Name("netwall")),
        (ServiceQuery(3476, Protocol::UDP), Service::Name("nppmp")),
        (ServiceQuery(1999, Protocol::TCP), Service::Name("tcp-id-port")),
        (ServiceQuery(2734, Protocol::UDP), Service::Name("ccs-software")),
        (ServiceQuery(427, Protocol::TCP), Service::Name("svrloc")),
        (ServiceQuery(3539, Protocol::UDP), Service::Name("ibm-diradm-ssl")),
        (ServiceQuery(3139, Protocol::TCP), Service::Name("incognitorv")),
        (ServiceQuery(186, Protocol::TCP), Service::Name("kis")),
        (ServiceQuery(2964, Protocol::TCP), Service::Name("bullant-srap")),
        (ServiceQuery(4447, Protocol::TCP), Service::Name("n1-rmgmt")),
        (ServiceQuery(8403, Protocol::TCP), Service::Name("admind")),
        (ServiceQuery(5012, Protocol::UDP), Service::Name("nsp")),
        (ServiceQuery(6620, Protocol::UDP), Service::Name("kftp-data")),
        (ServiceQuery(3455, Protocol::UDP), Service::Name("prsvp")),
        (ServiceQuery(8911, Protocol::TCP), Service::Name("manyone-xml")),
        (ServiceQuery(1047, Protocol::UDP), Service::Name("neod1")),
        (ServiceQuery(2202, Protocol::UDP), Service::Name("imtc-map")),
        (ServiceQuery(1133, Protocol::TCP), Service::Name("dfn")),
        (ServiceQuery(9800, Protocol::TCP), Service::Name("davsrc")),
        (ServiceQuery(358, Protocol::TCP), Service::Name("shrinkwrap")),
        (ServiceQuery(1533, Protocol::TCP), Service::Name("virtual-places")),
        (ServiceQuery(740, Protocol::UDP), Service::Name("netcp")),
        (ServiceQuery(2151, Protocol::TCP), Service::Name("docent")),
        (ServiceQuery(1385, Protocol::TCP), Service::Name("atex_elmd")),
        (ServiceQuery(4221, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(436, Protocol::UDP), Service::Name("dna-cml")),
        (ServiceQuery(27009, Protocol::UDP), Service::Name("flex-lm")),
        (ServiceQuery(1331, Protocol::TCP), Service::Name("intersan")),
        (ServiceQuery(4941, Protocol::TCP), Service::Name("eq-office-4941")),
        (ServiceQuery(1877, Protocol::TCP), Service::Name("hp-webqosdb")),
        (ServiceQuery(119, Protocol::TCP), Service::Name("nntp")),
        (ServiceQuery(1730, Protocol::UDP), Service::Name("roketz")),
        (ServiceQuery(3866, Protocol::TCP), Service::Name("dzdaemon")),
        (ServiceQuery(1414, Protocol::TCP), Service::Name("ibm-mqseries")),
        (ServiceQuery(10809, Protocol::TCP), Service::Name("nbd")),
        (ServiceQuery(2384, Protocol::TCP), Service::Name("sd-request")),
        (ServiceQuery(2832, Protocol::TCP), Service::Name("silkp4")),
        (ServiceQuery(264, Protocol::TCP), Service::Name("bgmp")),
        (ServiceQuery(1285, Protocol::UDP), Service::Name("neoiface")),
        (ServiceQuery(1475, Protocol::TCP), Service::Name("taligent-lm")),
        (ServiceQuery(3718, Protocol::UDP), Service::Name("opus-services")),
        (ServiceQuery(36865, Protocol::TCP), Service::Name("kastenxpipe")),
        (ServiceQuery(122, Protocol::UDP), Service::Name("smakynet")),
        (ServiceQuery(1192, Protocol::TCP), Service::Name("caids-sensor")),
        (ServiceQuery(14500, Protocol::TCP), Service::Name("xpra")),
        (ServiceQuery(6628, Protocol::TCP), Service::Name("afesc-mc")),
        (ServiceQuery(11174, Protocol::TCP), Service::Name("oemcacao-rmi")),
        (ServiceQuery(4669, Protocol::TCP), Service::Name("eportcommdata")),
        (ServiceQuery(4051, Protocol::UDP), Service::Name("cppdp")),
        (ServiceQuery(2737, Protocol::TCP), Service::Name("srp-feedback")),
        (ServiceQuery(3291, Protocol::UDP), Service::Name("sah-lm")),
        (ServiceQuery(6100, Protocol::TCP), Service::Name("synchronet-db")),
        (ServiceQuery(4098, Protocol::UDP), Service::Name("drmsfsd")),
        (ServiceQuery(14142, Protocol::UDP), Service::Name("icpp")),
        (ServiceQuery(6222, Protocol::UDP), Service::Name("radmind")),
        (ServiceQuery(2097, Protocol::TCP), Service::Name("jetformpreview")),
        (ServiceQuery(1415, Protocol::TCP), Service::Name("dbstar")),
        (ServiceQuery(2607, Protocol::UDP), Service::Name("connection")),
        (ServiceQuery(4395, Protocol::UDP), Service::Name("omnivisionesx")),
        (ServiceQuery(3063, Protocol::TCP), Service::Name("ncadg-ip-udp")),
        (ServiceQuery(42, Protocol::TCP), Service::Name("nameserver")),
        (ServiceQuery(4247, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2870, Protocol::UDP), Service::Name("daishi")),
        (ServiceQuery(11105, Protocol::TCP), Service::Name("netapp-icdata")),
        (ServiceQuery(4241, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(28910, Protocol::UDP), Service::Name("heretic2")),
        (ServiceQuery(3213, Protocol::UDP), Service::Name("neon24x7")),
        (ServiceQuery(1396, Protocol::UDP), Service::Name("dvl-activemail")),
        (ServiceQuery(1258, Protocol::UDP), Service::Name("opennl")),
        (ServiceQuery(8211, Protocol::TCP), Service::Name("aruba-papi")),
        (ServiceQuery(67, Protocol::TCP), Service::Name("dhcps")),
        (ServiceQuery(8900, Protocol::UDP), Service::Name("jmb-cds1")),
        (ServiceQuery(3660, Protocol::UDP), Service::Name("can-nds-ssl")),
        (ServiceQuery(382, Protocol::UDP), Service::Name("hp-managed-node")),
        (ServiceQuery(2522, Protocol::TCP), Service::Name("windb")),
        (ServiceQuery(4360, Protocol::TCP), Service::Name("matrix_vnet")),
        (ServiceQuery(3975, Protocol::UDP), Service::Name("airshot")),
        (ServiceQuery(2535, Protocol::TCP), Service::Name("madcap")),
        (ServiceQuery(1392, Protocol::TCP), Service::Name("iclpv-pm")),
        (ServiceQuery(4199, Protocol::UDP), Service::Name("eims-admin")),
        (ServiceQuery(4688, Protocol::TCP), Service::Name("mobile-p2p")),
        (ServiceQuery(466, Protocol::UDP), Service::Name("digital-vrc")),
        (ServiceQuery(3778, Protocol::TCP), Service::Name("c-h-it-port")),
        (ServiceQuery(712, Protocol::TCP), Service::Name("tbrpf")),
        (ServiceQuery(3357, Protocol::UDP), Service::Name("adtech-test")),
        (ServiceQuery(2178, Protocol::UDP), Service::Name("bitspeer")),
        (ServiceQuery(6583, Protocol::TCP), Service::Name("joaJewelSuite")),
        (ServiceQuery(1137, Protocol::TCP), Service::Name("trim")),
        (ServiceQuery(2974, Protocol::TCP), Service::Name("signal")),
        (ServiceQuery(6410, Protocol::TCP), Service::Name("boe-resssvr4")),
        (ServiceQuery(12003, Protocol::TCP), Service::Name("entextmed")),
        (ServiceQuery(9500, Protocol::UDP), Service::Name("ismserver")),
        (ServiceQuery(242, Protocol::TCP), Service::Name("direct")),
        (ServiceQuery(2679, Protocol::TCP), Service::Name("syncserverssl")),
        (ServiceQuery(2982, Protocol::UDP), Service::Name("iwb-whiteboard")),
        (ServiceQuery(3606, Protocol::UDP), Service::Name("splitlock")),
        (ServiceQuery(2987, Protocol::UDP), Service::Name("identify")),
        (ServiceQuery(2151, Protocol::UDP), Service::Name("docent")),
        (ServiceQuery(3231, Protocol::UDP), Service::Name("vidigo")),
        (ServiceQuery(27001, Protocol::TCP), Service::Name("flexlm1")),
        (ServiceQuery(2326, Protocol::UDP), Service::Name("idcp")),
        (ServiceQuery(3212, Protocol::UDP), Service::Name("surveyinst")),
        (ServiceQuery(405, Protocol::TCP), Service::Name("ncld")),
        (ServiceQuery(25901, Protocol::UDP), Service::Name("niobserver")),
        (ServiceQuery(179, Protocol::TCP), Service::Name("bgp")),
        (ServiceQuery(579, Protocol::UDP), Service::Name("decbsrv")),
        (ServiceQuery(8026, Protocol::TCP), Service::Name("ca-audit-ds")),
        (ServiceQuery(105, Protocol::UDP), Service::Name("csnet-ns")),
        (ServiceQuery(9, Protocol::TCP), Service::Name("discard")),
        (ServiceQuery(7672, Protocol::TCP), Service::Name("imqstomp")),
        (ServiceQuery(1183, Protocol::UDP), Service::Name("llsurfup-http")),
        (ServiceQuery(5434, Protocol::UDP), Service::Name("sgi-arrayd")),
        (ServiceQuery(2310, Protocol::UDP), Service::Name("sdclient")),
        (ServiceQuery(1119, Protocol::UDP), Service::Name("bnetgame")),
        (ServiceQuery(4433, Protocol::TCP), Service::Name("vop")),
        (ServiceQuery(666, Protocol::UDP), Service::Name("doom")),
        (ServiceQuery(1798, Protocol::UDP), Service::Name("etp")),
        (ServiceQuery(4086, Protocol::UDP), Service::Name("ftsync")),
        (ServiceQuery(1109, Protocol::TCP), Service::Name("kpop")),
        (ServiceQuery(677, Protocol::UDP), Service::Name("vpp")),
        (ServiceQuery(4133, Protocol::TCP), Service::Name("nuts_bootp")),
        (ServiceQuery(4184, Protocol::TCP), Service::Name("universe_suite")),
        (ServiceQuery(627, Protocol::UDP), Service::Name("passgo-tivoli")),
        (ServiceQuery(8999, Protocol::TCP), Service::Name("bctp")),
        (ServiceQuery(6935, Protocol::TCP), Service::Name("ethoscan")),
        (ServiceQuery(8880, Protocol::TCP), Service::Name("cddbp-alt")),
        (ServiceQuery(8403, Protocol::UDP), Service::Name("admind")),
        (ServiceQuery(1475, Protocol::UDP), Service::Name("taligent-lm")),
        (ServiceQuery(3721, Protocol::UDP), Service::Name("xsync")),
        (ServiceQuery(6417, Protocol::TCP), Service::Name("faxcomservice")),
        (ServiceQuery(1722, Protocol::UDP), Service::Name("hks-lm")),
        (ServiceQuery(3246, Protocol::TCP), Service::Name("dvt-system")),
        (ServiceQuery(3792, Protocol::UDP), Service::Name("sitewatch")),
        (ServiceQuery(6484, Protocol::TCP), Service::Name("sun-sr-jms")),
        (ServiceQuery(698, Protocol::TCP), Service::Name("olsr")),
        (ServiceQuery(7880, Protocol::TCP), Service::Name("pss")),
        (ServiceQuery(1227, Protocol::TCP), Service::Name("dns2go")),
        (ServiceQuery(2505, Protocol::TCP), Service::Name("ppcontrol")),
        (ServiceQuery(4137, Protocol::UDP), Service::Name("cl-db-remote")),
        (ServiceQuery(1511, Protocol::UDP), Service::Name("3l-l1")),
        (ServiceQuery(1921, Protocol::UDP), Service::Name("noadmin")),
        (ServiceQuery(3640, Protocol::UDP), Service::Name("netplay-port1")),
        (ServiceQuery(163, Protocol::UDP), Service::Name("cmip-man")),
        (ServiceQuery(2721, Protocol::TCP), Service::Name("smart-diagnose")),
        (ServiceQuery(1630, Protocol::TCP), Service::Name("oraclenet8cman")),
        (ServiceQuery(2386, Protocol::TCP), Service::Name("virtualtape")),
        (ServiceQuery(1107, Protocol::TCP), Service::Name("isoipsigport-2")),
        (ServiceQuery(1043, Protocol::TCP), Service::Name("boinc")),
        (ServiceQuery(577, Protocol::UDP), Service::Name("vnas")),
        (ServiceQuery(4300, Protocol::TCP), Service::Name("corelccam")),
        (ServiceQuery(7334, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(4235, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(7329, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(3154, Protocol::UDP), Service::Name("ccmrmi")),
        (ServiceQuery(3115, Protocol::UDP), Service::Name("mctet-master")),
        (ServiceQuery(609, Protocol::UDP), Service::Name("npmp-trap")),
        (ServiceQuery(5767, Protocol::TCP), Service::Name("s-openmail")),
        (ServiceQuery(97, Protocol::TCP), Service::Name("swift-rvf")),
        (ServiceQuery(3381, Protocol::UDP), Service::Name("geneous")),
        (ServiceQuery(3852, Protocol::UDP), Service::Name("sse-app-config")),
        (ServiceQuery(1638, Protocol::TCP), Service::Name("ismc")),
        (ServiceQuery(9981, Protocol::TCP), Service::Name("pumpkindb")),
        (ServiceQuery(3238, Protocol::TCP), Service::Name("apparenet-as")),
        (ServiceQuery(1740, Protocol::UDP), Service::Name("encore")),
        (ServiceQuery(415, Protocol::TCP), Service::Name("bnet")),
        (ServiceQuery(697, Protocol::TCP), Service::Name("uuidgen")),
        (ServiceQuery(38037, Protocol::TCP), Service::Name("landesk-cba")),
        (ServiceQuery(218, Protocol::UDP), Service::Name("mpp")),
        (ServiceQuery(3556, Protocol::UDP), Service::Name("sky-transport")),
        (ServiceQuery(2143, Protocol::UDP), Service::Name("lv-jc")),
        (ServiceQuery(5197, Protocol::TCP), Service::Name("tunstall-lwp")),
        (ServiceQuery(1108, Protocol::UDP), Service::Name("ratio-adp")),
        (ServiceQuery(4671, Protocol::UDP), Service::Name("acter")),
        (ServiceQuery(1729, Protocol::TCP), Service::Name("citynl")),
        (ServiceQuery(6146, Protocol::TCP), Service::Name("lonewolf-lm")),
        (ServiceQuery(6074, Protocol::TCP), Service::Name("max")),
        (ServiceQuery(5770, Protocol::UDP), Service::Name("spramsd")),
        (ServiceQuery(182, Protocol::TCP), Service::Name("audit")),
        (ServiceQuery(7331, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(3706, Protocol::TCP), Service::Name("rt-event")),
        (ServiceQuery(584, Protocol::UDP), Service::Name("keyserver")),
        (ServiceQuery(1303, Protocol::UDP), Service::Name("sftsrv")),
        (ServiceQuery(1469, Protocol::TCP), Service::Name("aal-lm")),
        (ServiceQuery(208, Protocol::TCP), Service::Name("at-8")),
        (ServiceQuery(1729, Protocol::UDP), Service::Name("citynl")),
        (ServiceQuery(4787, Protocol::TCP), Service::Name("sia-ctrl-plane")),
        (ServiceQuery(1085, Protocol::TCP), Service::Name("webobjects")),
        (ServiceQuery(2181, Protocol::UDP), Service::Name("eforward")),
        (ServiceQuery(1502, Protocol::TCP), Service::Name("shivadiscovery")),
        (ServiceQuery(36602, Protocol::TCP), Service::Name("observium-agent")),
        (ServiceQuery(48050, Protocol::TCP), Service::Name("weandsf")),
        (ServiceQuery(1752, Protocol::UDP), Service::Name("lofr-lm")),
        (ServiceQuery(4670, Protocol::UDP), Service::Name("light")),
        (ServiceQuery(23004, Protocol::TCP), Service::Name("inovaport5")),
        (ServiceQuery(1132, Protocol::UDP), Service::Name("kvm-via-ip")),
        (ServiceQuery(3515, Protocol::UDP), Service::Name("must-backplane")),
        (ServiceQuery(4599, Protocol::UDP), Service::Name("a17-an-an")),
        (ServiceQuery(1627, Protocol::UDP), Service::Name("t128-gateway")),
        (ServiceQuery(5251, Protocol::TCP), Service::Name("caevms")),
        (ServiceQuery(21846, Protocol::TCP), Service::Name("netspeak-is")),
        (ServiceQuery(2303, Protocol::UDP), Service::Name("proxy-gateway")),
        (ServiceQuery(1102, Protocol::UDP), Service::Name("adobeserver-1")),
        (ServiceQuery(3133, Protocol::TCP), Service::Name("prism-deploy")),
        (ServiceQuery(2003, Protocol::TCP), Service::Name("finger")),
        (ServiceQuery(3668, Protocol::UDP), Service::Name("dell-rm-port")),
        (ServiceQuery(8131, Protocol::TCP), Service::Name("indigo-vbcp")),
        (ServiceQuery(9122, Protocol::TCP), Service::Name("grcmp")),
        (ServiceQuery(1376, Protocol::UDP), Service::Name("ibm-pps")),
        (ServiceQuery(9206, Protocol::TCP), Service::Name("wap-vcard-s")),
        (ServiceQuery(8990, Protocol::UDP), Service::Name("http-wmap")),
        (ServiceQuery(4407, Protocol::TCP), Service::Name("nacagent")),
        (ServiceQuery(3982, Protocol::UDP), Service::Name("eis")),
        (ServiceQuery(3221, Protocol::TCP), Service::Name("xnm-clear-text")),
        (ServiceQuery(700, Protocol::UDP), Service::Name("epp")),
        (ServiceQuery(1596, Protocol::UDP), Service::Name("radio-bc")),
        (ServiceQuery(4987, Protocol::UDP), Service::Name("smar-se-port1")),
        (ServiceQuery(2984, Protocol::TCP), Service::Name("hpidsadmin")),
        (ServiceQuery(6503, Protocol::TCP), Service::Name("boks_clntd")),
        (ServiceQuery(1035, Protocol::TCP), Service::Name("multidropper")),
        (ServiceQuery(4550, Protocol::TCP), Service::Name("gds-adppiw-db")),
        (ServiceQuery(3800, Protocol::TCP), Service::Name("pwgpsi")),
        (ServiceQuery(5242, Protocol::TCP), Service::Name("attune")),
        (ServiceQuery(4260, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3107, Protocol::UDP), Service::Name("business")),
        (ServiceQuery(366, Protocol::UDP), Service::Name("odmr")),
        (ServiceQuery(34962, Protocol::UDP), Service::Name("profinet-rt")),
        (ServiceQuery(1405, Protocol::UDP), Service::Name("ibm-res")),
        (ServiceQuery(9966, Protocol::TCP), Service::Name("odnsp")),
        (ServiceQuery(3372, Protocol::TCP), Service::Name("msdtc")),
        (ServiceQuery(5426, Protocol::TCP), Service::Name("devbasic")),
        (ServiceQuery(3839, Protocol::UDP), Service::Name("amx-rms")),
        (ServiceQuery(3961, Protocol::TCP), Service::Name("proaxess")),
        (ServiceQuery(953, Protocol::TCP), Service::Name("rndc")),
        (ServiceQuery(5503, Protocol::UDP), Service::Name("fcp-srvr-inst2")),
        (ServiceQuery(21847, Protocol::TCP), Service::Name("netspeak-cs")),
        (ServiceQuery(2871, Protocol::UDP), Service::Name("msi-selectplay")),
        (ServiceQuery(7307, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(168, Protocol::UDP), Service::Name("rsvd")),
        (ServiceQuery(2396, Protocol::TCP), Service::Name("wusage")),
        (ServiceQuery(19000, Protocol::UDP), Service::Name("igrid")),
        (ServiceQuery(7319, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(444, Protocol::TCP), Service::Name("snpp")),
        (ServiceQuery(4073, Protocol::UDP), Service::Name("iRAPP")),
        (ServiceQuery(2899, Protocol::TCP), Service::Name("powergemplus")),
        (ServiceQuery(4297, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2864, Protocol::TCP), Service::Name("astromed-main")),
        (ServiceQuery(3562, Protocol::TCP), Service::Name("sdbproxy")),
        (ServiceQuery(1991, Protocol::TCP), Service::Name("stun-p2")),
        (ServiceQuery(48003, Protocol::TCP), Service::Name("nimgtw")),
        (ServiceQuery(502, Protocol::TCP), Service::Name("mbap")),
        (ServiceQuery(651, Protocol::TCP), Service::Name("ieee-mms")),
        (ServiceQuery(496, Protocol::TCP), Service::Name("pim-rp-disc")),
        (ServiceQuery(3500, Protocol::UDP), Service::Name("rtmp-port")),
        (ServiceQuery(11201, Protocol::TCP), Service::Name("smsqp")),
        (ServiceQuery(10082, Protocol::TCP), Service::Name("amandaidx")),
        (ServiceQuery(3841, Protocol::TCP), Service::Name("zfirm-shiprush3")),
        (ServiceQuery(13711, Protocol::TCP), Service::Name("netbackup")),
        (ServiceQuery(3791, Protocol::TCP), Service::Name("tvnetworkvideo")),
        (ServiceQuery(3477, Protocol::UDP), Service::Name("ecomm")),
        (ServiceQuery(6008, Protocol::TCP), Service::Name("X11:8")),
        (ServiceQuery(5582, Protocol::UDP), Service::Name("fac-restore")),
        (ServiceQuery(1409, Protocol::UDP), Service::Name("here-lm")),
        (ServiceQuery(2011, Protocol::TCP), Service::Name("raid-cc")),
        (ServiceQuery(6657, Protocol::UDP), Service::Name("palcom-disc")),
        (ServiceQuery(3748, Protocol::TCP), Service::Name("webdata")),
        (ServiceQuery(479, Protocol::TCP), Service::Name("iafserver")),
        (ServiceQuery(1272, Protocol::TCP), Service::Name("cspmlockmgr")),
        (ServiceQuery(7305, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(391, Protocol::TCP), Service::Name("synotics-relay")),
        (ServiceQuery(6347, Protocol::UDP), Service::Name("gnutella2")),
        (ServiceQuery(8471, Protocol::UDP), Service::Name("pim-port")),
        (ServiceQuery(4063, Protocol::TCP), Service::Name("ice-router")),
        (ServiceQuery(10117, Protocol::UDP), Service::Name("iqrm")),
        (ServiceQuery(9445, Protocol::TCP), Service::Name("mindarray-ca")),
        (ServiceQuery(9522, Protocol::TCP), Service::Name("sma-spw")),
        (ServiceQuery(2802, Protocol::TCP), Service::Name("veritas-tcp1")),
        (ServiceQuery(2245, Protocol::TCP), Service::Name("hao")),
        (ServiceQuery(5601, Protocol::UDP), Service::Name("esmagent")),
        (ServiceQuery(409, Protocol::TCP), Service::Name("prm-nm")),
        (ServiceQuery(2199, Protocol::TCP), Service::Name("onehome-help")),
        (ServiceQuery(6044, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(10116, Protocol::TCP), Service::Name("netiq-voipa")),
        (ServiceQuery(6318, Protocol::TCP), Service::Name("iona-data")),
        (ServiceQuery(1653, Protocol::TCP), Service::Name("alphatech-lm")),
        (ServiceQuery(1580, Protocol::UDP), Service::Name("tn-tl-r2")),
        (ServiceQuery(9994, Protocol::TCP), Service::Name("palace-3")),
        (ServiceQuery(2727, Protocol::TCP), Service::Name("mgcp-callagent")),
        (ServiceQuery(5315, Protocol::UDP), Service::Name("hacl-poll")),
        (ServiceQuery(40404, Protocol::TCP), Service::Name("sptx")),
        (ServiceQuery(3321, Protocol::UDP), Service::Name("vnsstr")),
        (ServiceQuery(3123, Protocol::TCP), Service::Name("edix")),
        (ServiceQuery(7345, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(3103, Protocol::TCP), Service::Name("autocuesmi")),
        (ServiceQuery(117, Protocol::TCP), Service::Name("uucp-path")),
        (ServiceQuery(3237, Protocol::UDP), Service::Name("apparenet-tps")),
        (ServiceQuery(3467, Protocol::TCP), Service::Name("rcst")),
        (ServiceQuery(2419, Protocol::UDP), Service::Name("attachmate-s2s")),
        (ServiceQuery(7237, Protocol::TCP), Service::Name("pads")),
        (ServiceQuery(2785, Protocol::TCP), Service::Name("aic-np")),
        (ServiceQuery(1961, Protocol::UDP), Service::Name("bts-appserver")),
        (ServiceQuery(9214, Protocol::UDP), Service::Name("ipdcesgbs")),
        (ServiceQuery(2809, Protocol::TCP), Service::Name("corbaloc")),
        (ServiceQuery(749, Protocol::UDP), Service::Name("kerberos-adm")),
        (ServiceQuery(3453, Protocol::TCP), Service::Name("pscupd")),
        (ServiceQuery(32771, Protocol::UDP), Service::Name("sometimes-rpc6")),
        (ServiceQuery(2290, Protocol::UDP), Service::Name("sonus-logging")),
        (ServiceQuery(5436, Protocol::UDP), Service::Name("pmip6-cntl")),
        (ServiceQuery(2563, Protocol::TCP), Service::Name("cti-redwood")),
        (ServiceQuery(810, Protocol::TCP), Service::Name("fcp-udp")),
        (ServiceQuery(1890, Protocol::UDP), Service::Name("wilkenlistener")),
        (ServiceQuery(7215, Protocol::TCP), Service::Name("PS-Server")),
        (ServiceQuery(10860, Protocol::TCP), Service::Name("helix")),
        (ServiceQuery(423, Protocol::UDP), Service::Name("opc-job-start")),
        (ServiceQuery(5793, Protocol::TCP), Service::Name("xtreamx")),
        (ServiceQuery(4180, Protocol::UDP), Service::Name("httpx")),
        (ServiceQuery(2409, Protocol::UDP), Service::Name("sns-protocol")),
        (ServiceQuery(4726, Protocol::TCP), Service::Name("a26-fap-fgw")),
        (ServiceQuery(38202, Protocol::TCP), Service::Name("fairview")),
        (ServiceQuery(7000, Protocol::TCP), Service::Name("afs3-fileserver")),
        (ServiceQuery(1773, Protocol::UDP), Service::Name("kmscontrol")),
        (ServiceQuery(3780, Protocol::TCP), Service::Name("nnp")),
        (ServiceQuery(3872, Protocol::UDP), Service::Name("oem-agent")),
        (ServiceQuery(543, Protocol::UDP), Service::Name("klogin")),
        (ServiceQuery(3919, Protocol::TCP), Service::Name("hyperip")),
        (ServiceQuery(5085, Protocol::TCP), Service::Name("encrypted-llrp")),
        (ServiceQuery(7429, Protocol::TCP), Service::Name("opi-sock")),
        (ServiceQuery(6111, Protocol::TCP), Service::Name("spc")),
        (ServiceQuery(3080, Protocol::TCP), Service::Name("stm_pproc")),
        (ServiceQuery(3592, Protocol::UDP), Service::Name("gtrack-ne")),
        (ServiceQuery(8443, Protocol::UDP), Service::Name("pcsync-https")),
        (ServiceQuery(4732, Protocol::UDP), Service::Name("ohmtrigger")),
        (ServiceQuery(1749, Protocol::TCP), Service::Name("aspen-services")),
        (ServiceQuery(2592, Protocol::UDP), Service::Name("netrek")),
        (ServiceQuery(1253, Protocol::TCP), Service::Name("q55-pcc")),
        (ServiceQuery(1593, Protocol::UDP), Service::Name("mainsoft-lm")),
        (ServiceQuery(669, Protocol::UDP), Service::Name("meregister")),
        (ServiceQuery(4110, Protocol::TCP), Service::Name("g2tag")),
        (ServiceQuery(1336, Protocol::UDP), Service::Name("ischat")),
        (ServiceQuery(2201, Protocol::UDP), Service::Name("ats")),
        (ServiceQuery(5209, Protocol::TCP), Service::Name("nomad")),
        (ServiceQuery(6483, Protocol::UDP), Service::Name("SunVTS-RMI")),
        (ServiceQuery(3027, Protocol::UDP), Service::Name("LiebDevMgmt_C")),
        (ServiceQuery(5463, Protocol::UDP), Service::Name("ttlpriceproxy")),
        (ServiceQuery(8153, Protocol::TCP), Service::Name("quantastor")),
        (ServiceQuery(2032, Protocol::UDP), Service::Name("blackboard")),
        (ServiceQuery(2896, Protocol::TCP), Service::Name("ecovisiong6-1")),
        (ServiceQuery(4205, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(25008, Protocol::TCP), Service::Name("icl-twobase9")),
        (ServiceQuery(2311, Protocol::TCP), Service::Name("messageservice")),
        (ServiceQuery(800, Protocol::TCP), Service::Name("mdbs_daemon")),
        (ServiceQuery(1924, Protocol::UDP), Service::Name("xiip")),
        (ServiceQuery(2778, Protocol::TCP), Service::Name("gwen-sonya")),
        (ServiceQuery(4405, Protocol::UDP), Service::Name("ds-mail")),
        (ServiceQuery(2342, Protocol::UDP), Service::Name("manage-exec")),
        (ServiceQuery(4289, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(9318, Protocol::TCP), Service::Name("secure-ts")),
        (ServiceQuery(9026, Protocol::TCP), Service::Name("swa-4")),
        (ServiceQuery(8116, Protocol::UDP), Service::Name("cp-cluster")),
        (ServiceQuery(1512, Protocol::TCP), Service::Name("wins")),
        (ServiceQuery(8808, Protocol::TCP), Service::Name("ssports-bcast")),
        (ServiceQuery(4047, Protocol::UDP), Service::Name("ctp-state")),
        (ServiceQuery(2041, Protocol::UDP), Service::Name("interbase")),
        (ServiceQuery(2036, Protocol::TCP), Service::Name("e-dpnet")),
        (ServiceQuery(1317, Protocol::TCP), Service::Name("vrts-ipcserver")),
        (ServiceQuery(5008, Protocol::TCP), Service::Name("synapsis-edge")),
        (ServiceQuery(2206, Protocol::UDP), Service::Name("hpocbus")),
        (ServiceQuery(5716, Protocol::TCP), Service::Name("prosharerequest")),
        (ServiceQuery(607, Protocol::TCP), Service::Name("nqs")),
        (ServiceQuery(3918, Protocol::TCP), Service::Name("pktcablemmcops")),
        (ServiceQuery(1443, Protocol::UDP), Service::Name("ies-lm")),
        (ServiceQuery(3978, Protocol::TCP), Service::Name("secure-cfg-svr")),
        (ServiceQuery(4427, Protocol::TCP), Service::Name("drizzle")),
        (ServiceQuery(26257, Protocol::TCP), Service::Name("cockroach")),
        (ServiceQuery(3125, Protocol::UDP), Service::Name("a13-an")),
        (ServiceQuery(3611, Protocol::UDP), Service::Name("six-degrees")),
        (ServiceQuery(8088, Protocol::TCP), Service::Name("radan-http")),
        (ServiceQuery(4110, Protocol::UDP), Service::Name("g2tag")),
        (ServiceQuery(3279, Protocol::UDP), Service::Name("admind")),
        (ServiceQuery(10200, Protocol::TCP), Service::Name("trisoap")),
        (ServiceQuery(23000, Protocol::TCP), Service::Name("inovaport1")),
        (ServiceQuery(1077, Protocol::TCP), Service::Name("imgames")),
        (ServiceQuery(4252, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(32034, Protocol::TCP), Service::Name("iracinghelper")),
        (ServiceQuery(5059, Protocol::UDP), Service::Name("sds")),
        (ServiceQuery(675, Protocol::UDP), Service::Name("dctp")),
        (ServiceQuery(2572, Protocol::UDP), Service::Name("ibp")),
        (ServiceQuery(11796, Protocol::TCP), Service::Name("lanschool")),
        (ServiceQuery(6579, Protocol::TCP), Service::Name("affiliate")),
        (ServiceQuery(5986, Protocol::UDP), Service::Name("wsmans")),
        (ServiceQuery(17010, Protocol::TCP), Service::Name("ncpu")),
        (ServiceQuery(28119, Protocol::TCP), Service::Name("a27-ran-ran")),
        (ServiceQuery(3045, Protocol::TCP), Service::Name("slnp")),
        (ServiceQuery(1177, Protocol::TCP), Service::Name("dkmessenger")),
        (ServiceQuery(45825, Protocol::TCP), Service::Name("qdb2service")),
        (ServiceQuery(3526, Protocol::UDP), Service::Name("starquiz-port")),
        (ServiceQuery(583, Protocol::TCP), Service::Name("philips-vc")),
        (ServiceQuery(1959, Protocol::UDP), Service::Name("simp-all")),
        (ServiceQuery(2296, Protocol::UDP), Service::Name("theta-lm")),
        (ServiceQuery(4591, Protocol::TCP), Service::Name("l3t-at-an")),
        (ServiceQuery(7810, Protocol::TCP), Service::Name("rbt-wanopt")),
        (ServiceQuery(192, Protocol::TCP), Service::Name("osu-nms")),
        (ServiceQuery(1531, Protocol::UDP), Service::Name("rap-listen")),
        (ServiceQuery(4200, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(4730, Protocol::TCP), Service::Name("gearman")),
        (ServiceQuery(2818, Protocol::TCP), Service::Name("rmlnk")),
        (ServiceQuery(7358, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(3136, Protocol::TCP), Service::Name("grubd")),
        (ServiceQuery(4401, Protocol::UDP), Service::Name("ds-srvr")),
        (ServiceQuery(3329, Protocol::UDP), Service::Name("hp-device-disc")),
        (ServiceQuery(20049, Protocol::UDP), Service::Name("nfsrdma")),
        (ServiceQuery(7335, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(3493, Protocol::TCP), Service::Name("nut")),
        (ServiceQuery(4056, Protocol::TCP), Service::Name("lms")),
        (ServiceQuery(4664, Protocol::UDP), Service::Name("ems")),
        (ServiceQuery(3735, Protocol::TCP), Service::Name("pwdis")),
        (ServiceQuery(4004, Protocol::TCP), Service::Name("pxc-roid")),
        (ServiceQuery(3544, Protocol::UDP), Service::Name("teredo")),
        (ServiceQuery(2250, Protocol::UDP), Service::Name("remote-collab")),
        (ServiceQuery(6549, Protocol::UDP), Service::Name("powerchuteplus")),
        (ServiceQuery(1245, Protocol::UDP), Service::Name("isbconference2")),
        (ServiceQuery(610, Protocol::UDP), Service::Name("npmp-local")),
        (ServiceQuery(3338, Protocol::UDP), Service::Name("anet-b")),
        (ServiceQuery(6033, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(729, Protocol::TCP), Service::Name("netviewdm1")),
        (ServiceQuery(8686, Protocol::UDP), Service::Name("sun-as-jmxrmi")),
        (ServiceQuery(1474, Protocol::UDP), Service::Name("telefinder")),
        (ServiceQuery(1661, Protocol::TCP), Service::Name("netview-aix-1")),
        (ServiceQuery(1936, Protocol::TCP), Service::Name("jetcmeserver")),
        (ServiceQuery(8765, Protocol::TCP), Service::Name("ultraseek-http")),
        (ServiceQuery(1893, Protocol::UDP), Service::Name("elad")),
        (ServiceQuery(3534, Protocol::TCP), Service::Name("urld-port")),
        (ServiceQuery(3974, Protocol::UDP), Service::Name("citysearch")),
        (ServiceQuery(2811, Protocol::TCP), Service::Name("gsiftp")),
        (ServiceQuery(398, Protocol::TCP), Service::Name("kryptolan")),
        (ServiceQuery(3874, Protocol::UDP), Service::Name("sixxsconfig")),
        (ServiceQuery(528, Protocol::UDP), Service::Name("custix")),
        (ServiceQuery(6041, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(1479, Protocol::UDP), Service::Name("dberegister")),
        (ServiceQuery(3425, Protocol::UDP), Service::Name("agps-port")),
        (ServiceQuery(3299, Protocol::TCP), Service::Name("saprouter")),
        (ServiceQuery(2288, Protocol::UDP), Service::Name("netml")),
        (ServiceQuery(5355, Protocol::UDP), Service::Name("llmnr")),
        (ServiceQuery(6389, Protocol::TCP), Service::Name("clariion-evr01")),
        (ServiceQuery(3028, Protocol::TCP), Service::Name("LiebDevMgmt_DM")),
        (ServiceQuery(1695, Protocol::TCP), Service::Name("rrilwm")),
        (ServiceQuery(2135, Protocol::UDP), Service::Name("gris")),
        (ServiceQuery(32786, Protocol::TCP), Service::Name("sometimes-rpc25")),
        (ServiceQuery(9081, Protocol::TCP), Service::Name("cisco-aqos")),
        (ServiceQuery(26260, Protocol::UDP), Service::Name("ezproxy")),
        (ServiceQuery(3782, Protocol::TCP), Service::Name("iso-tp0s")),
        (ServiceQuery(5461, Protocol::TCP), Service::Name("silkmeter")),
        (ServiceQuery(90, Protocol::TCP), Service::Name("dnsix")),
        (ServiceQuery(4567, Protocol::TCP), Service::Name("tram")),
        (ServiceQuery(1158, Protocol::UDP), Service::Name("dbcontrol-oms")),
        (ServiceQuery(2941, Protocol::UDP), Service::Name("sm-pas-4")),
        (ServiceQuery(3845, Protocol::UDP), Service::Name("v-one-spp")),
        (ServiceQuery(2952, Protocol::UDP), Service::Name("mpfwsas")),
        (ServiceQuery(3886, Protocol::UDP), Service::Name("nei-management")),
        (ServiceQuery(1390, Protocol::TCP), Service::Name("iclpv-sc")),
        (ServiceQuery(6689, Protocol::UDP), Service::Name("tsa")),
        (ServiceQuery(65, Protocol::UDP), Service::Name("tacacs-ds")),
        (ServiceQuery(1634, Protocol::TCP), Service::Name("loaprobe")),
        (ServiceQuery(1967, Protocol::TCP), Service::Name("sns-quote")),
        (ServiceQuery(6445, Protocol::UDP), Service::Name("sge_execd")),
        (ServiceQuery(3429, Protocol::TCP), Service::Name("gcsp")),
        (ServiceQuery(4603, Protocol::TCP), Service::Name("menandmice-upg")),
        (ServiceQuery(1848, Protocol::UDP), Service::Name("fjdocdist")),
        (ServiceQuery(1553, Protocol::UDP), Service::Name("sna-cs")),
        (ServiceQuery(36462, Protocol::TCP), Service::Name("xw-control")),
        (ServiceQuery(6969, Protocol::UDP), Service::Name("acmsoda")),
        (ServiceQuery(8025, Protocol::TCP), Service::Name("ca-audit-da")),
        (ServiceQuery(3296, Protocol::UDP), Service::Name("rib-slm")),
        (ServiceQuery(506, Protocol::TCP), Service::Name("ohimsrv")),
        (ServiceQuery(625, Protocol::UDP), Service::Name("dec_dlm")),
        (ServiceQuery(4838, Protocol::TCP), Service::Name("varadero-1")),
        (ServiceQuery(78, Protocol::UDP), Service::Name("vettcp")),
        (ServiceQuery(1241, Protocol::TCP), Service::Name("nessus")),
        (ServiceQuery(483, Protocol::TCP), Service::Name("ulpnet")),
        (ServiceQuery(4000, Protocol::TCP), Service::Name("remoteanything")),
        (ServiceQuery(3200, Protocol::TCP), Service::Name("tick-port")),
        (ServiceQuery(7700, Protocol::TCP), Service::Name("em7-secom")),
        (ServiceQuery(5573, Protocol::UDP), Service::Name("sdmmp")),
        (ServiceQuery(5430, Protocol::UDP), Service::Name("radec-corp")),
        (ServiceQuery(2699, Protocol::TCP), Service::Name("csoft-plusclnt")),
        (ServiceQuery(4914, Protocol::UDP), Service::Name("bones")),
        (ServiceQuery(2048, Protocol::UDP), Service::Name("dls-monitor")),
        (ServiceQuery(8076, Protocol::TCP), Service::Name("slnp")),
        (ServiceQuery(3788, Protocol::UDP), Service::Name("isrp-port")),
        (ServiceQuery(7566, Protocol::UDP), Service::Name("vsi-omega")),
        (ServiceQuery(92, Protocol::UDP), Service::Name("npp")),
        (ServiceQuery(2171, Protocol::UDP), Service::Name("msfw-storage")),
        (ServiceQuery(3190, Protocol::UDP), Service::Name("csvr-proxy")),
        (ServiceQuery(3955, Protocol::UDP), Service::Name("p2pcommunity")),
        (ServiceQuery(1404, Protocol::TCP), Service::Name("igi-lm")),
        (ServiceQuery(1419, Protocol::UDP), Service::Name("timbuktu-srv3")),
        (ServiceQuery(7982, Protocol::UDP), Service::Name("sossd-disc")),
        (ServiceQuery(3187, Protocol::UDP), Service::Name("odi-port")),
        (ServiceQuery(6400, Protocol::TCP), Service::Name("crystalreports")),
        (ServiceQuery(9339, Protocol::TCP), Service::Name("gnmi-gnoi")),
        (ServiceQuery(13224, Protocol::UDP), Service::Name("powwow-server")),
        (ServiceQuery(5454, Protocol::UDP), Service::Name("apc-5454")),
        (ServiceQuery(761, Protocol::TCP), Service::Name("kpasswd")),
        (ServiceQuery(1143, Protocol::UDP), Service::Name("imyx")),
        (ServiceQuery(4347, Protocol::TCP), Service::Name("lansurveyor")),
        (ServiceQuery(9008, Protocol::TCP), Service::Name("ogs-server")),
        (ServiceQuery(184, Protocol::TCP), Service::Name("ocserver")),
        (ServiceQuery(5312, Protocol::UDP), Service::Name("permabit-cs")),
        (ServiceQuery(1995, Protocol::UDP), Service::Name("perf-port")),
        (ServiceQuery(31337, Protocol::UDP), Service::Name("BackOrifice")),
        (ServiceQuery(3657, Protocol::TCP), Service::Name("immedianet-bcn")),
        (ServiceQuery(18634, Protocol::UDP), Service::Name("rds-ib")),
        (ServiceQuery(123, Protocol::UDP), Service::Name("ntp")),
        (ServiceQuery(7510, Protocol::UDP), Service::Name("ovhpas")),
        (ServiceQuery(1457, Protocol::TCP), Service::Name("valisys-lm")),
        (ServiceQuery(2268, Protocol::TCP), Service::Name("amt")),
        (ServiceQuery(6841, Protocol::UDP), Service::Name("netmo-default")),
        (ServiceQuery(6053, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(11163, Protocol::UDP), Service::Name("suncacao-rmi")),
        (ServiceQuery(4700, Protocol::TCP), Service::Name("netxms-agent")),
        (ServiceQuery(4314, Protocol::TCP), Service::Name("choiceview-agt")),
        (ServiceQuery(7431, Protocol::TCP), Service::Name("pmd")),
        (ServiceQuery(1095, Protocol::UDP), Service::Name("nicelink")),
        (ServiceQuery(15118, Protocol::TCP), Service::Name("v2g-secc")),
        (ServiceQuery(1989, Protocol::TCP), Service::Name("tr-rsrb-p3")),
        (ServiceQuery(4268, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3908, Protocol::UDP), Service::Name("hppronetman")),
        (ServiceQuery(2527, Protocol::TCP), Service::Name("iqserver")),
        (ServiceQuery(684, Protocol::TCP), Service::Name("corba-iiop-ssl")),
        (ServiceQuery(3411, Protocol::UDP), Service::Name("biolink-auth")),
        (ServiceQuery(3943, Protocol::UDP), Service::Name("tig")),
        (ServiceQuery(22305, Protocol::TCP), Service::Name("wnn6_Kr")),
        (ServiceQuery(5723, Protocol::TCP), Service::Name("omhs")),
        (ServiceQuery(5781, Protocol::TCP), Service::Name("3par-evts")),
        (ServiceQuery(3681, Protocol::TCP), Service::Name("bts-x73")),
        (ServiceQuery(8034, Protocol::UDP), Service::Name("vantronix-mgmt")),
        (ServiceQuery(4296, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1419, Protocol::TCP), Service::Name("timbuktu-srv3")),
        (ServiceQuery(1931, Protocol::TCP), Service::Name("amdsched")),
        (ServiceQuery(731, Protocol::TCP), Service::Name("netviewdm3")),
        (ServiceQuery(1828, Protocol::TCP), Service::Name("itm-mcell-u")),
        (ServiceQuery(6350, Protocol::TCP), Service::Name("adap")),
        (ServiceQuery(220, Protocol::TCP), Service::Name("imap3")),
        (ServiceQuery(612, Protocol::UDP), Service::Name("hmmp-ind")),
        (ServiceQuery(13724, Protocol::UDP), Service::Name("vnetd")),
        (ServiceQuery(37654, Protocol::UDP), Service::Name("unisys-eportal")),
        (ServiceQuery(679, Protocol::UDP), Service::Name("mrm")),
        (ServiceQuery(7071, Protocol::UDP), Service::Name("iwg1")),
        (ServiceQuery(4325, Protocol::UDP), Service::Name("geognosisman")),
        (ServiceQuery(2358, Protocol::UDP), Service::Name("futrix")),
        (ServiceQuery(74, Protocol::TCP), Service::Name("netrjs-4")),
        (ServiceQuery(1332, Protocol::TCP), Service::Name("pcia-rxp-b")),
        (ServiceQuery(44123, Protocol::TCP), Service::Name("z-wave-tunnel")),
        (ServiceQuery(750, Protocol::UDP), Service::Name("kerberos")),
        (ServiceQuery(7869, Protocol::TCP), Service::Name("mobileanalyzer")),
        (ServiceQuery(199, Protocol::UDP), Service::Name("smux")),
        (ServiceQuery(4332, Protocol::TCP), Service::Name("getty-focus")),
        (ServiceQuery(4303, Protocol::TCP), Service::Name("srcp")),
        (ServiceQuery(5094, Protocol::TCP), Service::Name("hart-ip")),
        (ServiceQuery(2157, Protocol::TCP), Service::Name("xnds")),
        (ServiceQuery(2818, Protocol::UDP), Service::Name("rmlnk")),
        (ServiceQuery(3617, Protocol::UDP), Service::Name("sharp-server")),
        (ServiceQuery(3987, Protocol::TCP), Service::Name("centerline")),
        (ServiceQuery(159, Protocol::UDP), Service::Name("nss-routing")),
        (ServiceQuery(4789, Protocol::TCP), Service::Name("vxlan")),
        (ServiceQuery(1625, Protocol::UDP), Service::Name("svs-omagent")),
        (ServiceQuery(3082, Protocol::TCP), Service::Name("tl1-raw")),
        (ServiceQuery(2543, Protocol::UDP), Service::Name("reftek")),
        (ServiceQuery(2028, Protocol::TCP), Service::Name("submitserver")),
        (ServiceQuery(2631, Protocol::TCP), Service::Name("sitaradir")),
        (ServiceQuery(3741, Protocol::TCP), Service::Name("wysdma")),
        (ServiceQuery(1395, Protocol::UDP), Service::Name("iclpv-wsm")),
        (ServiceQuery(2981, Protocol::UDP), Service::Name("mylxamport")),
        (ServiceQuery(5308, Protocol::UDP), Service::Name("cfengine")),
        (ServiceQuery(4800, Protocol::TCP), Service::Name("iims")),
        (ServiceQuery(2247, Protocol::UDP), Service::Name("antidotemgrsvr")),
        (ServiceQuery(2454, Protocol::UDP), Service::Name("indx-dds")),
        (ServiceQuery(2505, Protocol::UDP), Service::Name("ppcontrol")),
        (ServiceQuery(2650, Protocol::UDP), Service::Name("eristwoguns")),
        (ServiceQuery(6666, Protocol::UDP), Service::Name("ircu")),
        (ServiceQuery(3822, Protocol::UDP), Service::Name("acp-discovery")),
        (ServiceQuery(193, Protocol::UDP), Service::Name("srmp")),
        (ServiceQuery(13820, Protocol::UDP), Service::Name("dsmcc-passthru")),
        (ServiceQuery(2735, Protocol::TCP), Service::Name("netiq-mc")),
        (ServiceQuery(1736, Protocol::UDP), Service::Name("street-stream")),
        (ServiceQuery(7350, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(2567, Protocol::TCP), Service::Name("clp")),
        (ServiceQuery(4082, Protocol::TCP), Service::Name("lorica-out")),
        (ServiceQuery(3856, Protocol::TCP), Service::Name("informer")),
        (ServiceQuery(37, Protocol::TCP), Service::Name("time")),
        (ServiceQuery(2800, Protocol::UDP), Service::Name("acc-raid")),
        (ServiceQuery(3235, Protocol::TCP), Service::Name("mdap-port")),
        (ServiceQuery(4288, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3693, Protocol::TCP), Service::Name("easl")),
        (ServiceQuery(2649, Protocol::TCP), Service::Name("vpsipport")),
        (ServiceQuery(593, Protocol::TCP), Service::Name("http-rpc-epmap")),
        (ServiceQuery(606, Protocol::UDP), Service::Name("urm")),
        (ServiceQuery(3750, Protocol::TCP), Service::Name("cbos-ip-port")),
        (ServiceQuery(32780, Protocol::TCP), Service::Name("sometimes-rpc23")),
        (ServiceQuery(9210, Protocol::TCP), Service::Name("oma-mlp")),
        (ServiceQuery(2508, Protocol::UDP), Service::Name("jdatastore")),
        (ServiceQuery(3674, Protocol::UDP), Service::Name("wininstall-ipc")),
        (ServiceQuery(14033, Protocol::UDP), Service::Name("sage-best-com1")),
        (ServiceQuery(243, Protocol::TCP), Service::Name("sur-meas")),
        (ServiceQuery(6503, Protocol::UDP), Service::Name("boks_clntd")),
        (ServiceQuery(7357, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(5472, Protocol::TCP), Service::Name("apsolab-tag")),
        (ServiceQuery(2230, Protocol::TCP), Service::Name("queueadm")),
        (ServiceQuery(2934, Protocol::UDP), Service::Name("4-tieropmcli")),
        (ServiceQuery(5435, Protocol::TCP), Service::Name("sceanics")),
        (ServiceQuery(2769, Protocol::TCP), Service::Name("exce")),
        (ServiceQuery(3786, Protocol::UDP), Service::Name("upstriggervsw")),
        (ServiceQuery(2122, Protocol::UDP), Service::Name("caupc-remote")),
        (ServiceQuery(4729, Protocol::UDP), Service::Name("gsmtap")),
        (ServiceQuery(2843, Protocol::UDP), Service::Name("pdnet")),
        (ServiceQuery(9450, Protocol::UDP), Service::Name("sntlkeyssrvr")),
        (ServiceQuery(10107, Protocol::UDP), Service::Name("bctp-server")),
        (ServiceQuery(6970, Protocol::TCP), Service::Name("conductor")),
        (ServiceQuery(1234, Protocol::TCP), Service::Name("hotline")),
        (ServiceQuery(3423, Protocol::TCP), Service::Name("xtrm")),
        (ServiceQuery(2928, Protocol::TCP), Service::Name("redstone-cpss")),
        (ServiceQuery(1674, Protocol::TCP), Service::Name("proshare-mc-2")),
        (ServiceQuery(9211, Protocol::TCP), Service::Name("oma-mlp-s")),
        (ServiceQuery(2023, Protocol::UDP), Service::Name("xinuexpansion3")),
        (ServiceQuery(636, Protocol::UDP), Service::Name("ldaps")),
        (ServiceQuery(1779, Protocol::UDP), Service::Name("pharmasoft")),
        (ServiceQuery(3752, Protocol::TCP), Service::Name("vipremoteagent")),
        (ServiceQuery(4251, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(9300, Protocol::UDP), Service::Name("vrace")),
        (ServiceQuery(5137, Protocol::TCP), Service::Name("ctsd")),
        (ServiceQuery(32774, Protocol::TCP), Service::Name("sometimes-rpc11")),
        (ServiceQuery(2731, Protocol::UDP), Service::Name("fyre-messanger")),
        (ServiceQuery(1794, Protocol::UDP), Service::Name("cera-bcm")),
        (ServiceQuery(1490, Protocol::TCP), Service::Name("insitu-conf")),
        (ServiceQuery(3103, Protocol::UDP), Service::Name("autocuesmi")),
        (ServiceQuery(9000, Protocol::UDP), Service::Name("cslistener")),
        (ServiceQuery(15345, Protocol::TCP), Service::Name("xpilot")),
        (ServiceQuery(3220, Protocol::TCP), Service::Name("xnm-ssl")),
        (ServiceQuery(1813, Protocol::TCP), Service::Name("radius-acct")),
        (ServiceQuery(5500, Protocol::UDP), Service::Name("securid")),
        (ServiceQuery(2401, Protocol::UDP), Service::Name("cvspserver")),
        (ServiceQuery(3985, Protocol::TCP), Service::Name("mapper-mapethd")),
        (ServiceQuery(1136, Protocol::UDP), Service::Name("hhb-gateway")),
        (ServiceQuery(3925, Protocol::UDP), Service::Name("zmp")),
        (ServiceQuery(1240, Protocol::UDP), Service::Name("instantia")),
        (ServiceQuery(3250, Protocol::TCP), Service::Name("hicp")),
        (ServiceQuery(495, Protocol::UDP), Service::Name("intecourier")),
        (ServiceQuery(1598, Protocol::TCP), Service::Name("picknfs")),
        (ServiceQuery(4885, Protocol::TCP), Service::Name("abbs")),
        (ServiceQuery(5785, Protocol::TCP), Service::Name("3par-rcopy")),
        (ServiceQuery(3925, Protocol::TCP), Service::Name("zmp")),
        (ServiceQuery(5046, Protocol::UDP), Service::Name("vpm-udp")),
        (ServiceQuery(4069, Protocol::TCP), Service::Name("minger")),
        (ServiceQuery(5100, Protocol::TCP), Service::Name("admd")),
        (ServiceQuery(141, Protocol::TCP), Service::Name("emfis-cntl")),
        (ServiceQuery(6038, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(2339, Protocol::TCP), Service::Name("3com-webview")),
        (ServiceQuery(1035, Protocol::UDP), Service::Name("mxxrlogin")),
        (ServiceQuery(2633, Protocol::UDP), Service::Name("interintelli")),
        (ServiceQuery(2215, Protocol::TCP), Service::Name("ipcore")),
        (ServiceQuery(6624, Protocol::TCP), Service::Name("datascaler-db")),
        (ServiceQuery(1894, Protocol::UDP), Service::Name("o2server-port")),
        (ServiceQuery(1642, Protocol::TCP), Service::Name("isis-am")),
        (ServiceQuery(1706, Protocol::UDP), Service::Name("jetform")),
        (ServiceQuery(2082, Protocol::TCP), Service::Name("infowave")),
        (ServiceQuery(5979, Protocol::TCP), Service::Name("ncd-conf-tcp")),
        (ServiceQuery(1721, Protocol::TCP), Service::Name("caicci")),
        (ServiceQuery(3949, Protocol::TCP), Service::Name("drip")),
        (ServiceQuery(2939, Protocol::TCP), Service::Name("sm-pas-2")),
        (ServiceQuery(2811, Protocol::UDP), Service::Name("gsiftp")),
        (ServiceQuery(2608, Protocol::TCP), Service::Name("wag-service")),
        (ServiceQuery(5099, Protocol::TCP), Service::Name("sentlm-srv2srv")),
        (ServiceQuery(20001, Protocol::UDP), Service::Name("microsan")),
        (ServiceQuery(3164, Protocol::UDP), Service::Name("imprs")),
        (ServiceQuery(2095, Protocol::TCP), Service::Name("nbx-ser")),
        (ServiceQuery(1176, Protocol::TCP), Service::Name("indigo-server")),
        (ServiceQuery(3980, Protocol::TCP), Service::Name("acms")),
        (ServiceQuery(2079, Protocol::UDP), Service::Name("idware-router")),
        (ServiceQuery(2424, Protocol::UDP), Service::Name("kofax-svr")),
        (ServiceQuery(1305, Protocol::UDP), Service::Name("pe-mike")),
        (ServiceQuery(6020, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(48619, Protocol::TCP), Service::Name("iqobject")),
        (ServiceQuery(26208, Protocol::UDP), Service::Name("wnn6-ds")),
        (ServiceQuery(4596, Protocol::TCP), Service::Name("ias-neighbor")),
        (ServiceQuery(3533, Protocol::UDP), Service::Name("raven-rdp")),
        (ServiceQuery(3120, Protocol::TCP), Service::Name("d2000webserver")),
        (ServiceQuery(5401, Protocol::TCP), Service::Name("excerpts")),
        (ServiceQuery(3431, Protocol::UDP), Service::Name("ndl-als")),
        (ServiceQuery(3898, Protocol::TCP), Service::Name("senip")),
        (ServiceQuery(3735, Protocol::UDP), Service::Name("pwdis")),
        (ServiceQuery(2409, Protocol::TCP), Service::Name("sns-protocol")),
        (ServiceQuery(7794, Protocol::UDP), Service::Name("q3ade")),
        (ServiceQuery(4801, Protocol::TCP), Service::Name("iwec")),
        (ServiceQuery(3408, Protocol::TCP), Service::Name("BESApi")),
        (ServiceQuery(2841, Protocol::UDP), Service::Name("l3-ranger")),
        (ServiceQuery(7545, Protocol::UDP), Service::Name("nta-us")),
        (ServiceQuery(9340, Protocol::TCP), Service::Name("gribi")),
        (ServiceQuery(8665, Protocol::TCP), Service::Name("monetra")),
        (ServiceQuery(4604, Protocol::TCP), Service::Name("irp")),
        (ServiceQuery(10080, Protocol::TCP), Service::Name("amanda")),
        (ServiceQuery(7392, Protocol::UDP), Service::Name("mrssrendezvous")),
        (ServiceQuery(27504, Protocol::UDP), Service::Name("kopek-httphead")),
        (ServiceQuery(9876, Protocol::TCP), Service::Name("sd")),
        (ServiceQuery(541, Protocol::UDP), Service::Name("uucp-rlogin")),
        (ServiceQuery(5841, Protocol::TCP), Service::Name("shiprush-d-ch")),
        (ServiceQuery(4678, Protocol::UDP), Service::Name("traversal")),
        (ServiceQuery(2722, Protocol::TCP), Service::Name("proactivesrvr")),
        (ServiceQuery(4281, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(6037, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(1351, Protocol::UDP), Service::Name("equationbuilder")),
        (ServiceQuery(9998, Protocol::UDP), Service::Name("distinct32")),
        (ServiceQuery(1558, Protocol::TCP), Service::Name("xingmpeg")),
        (ServiceQuery(3058, Protocol::TCP), Service::Name("videobeans")),
        (ServiceQuery(2170, Protocol::TCP), Service::Name("eyetv")),
        (ServiceQuery(6005, Protocol::UDP), Service::Name("X11:5")),
        (ServiceQuery(13720, Protocol::UDP), Service::Name("bprd")),
        (ServiceQuery(5071, Protocol::UDP), Service::Name("powerschool")),
        (ServiceQuery(5269, Protocol::TCP), Service::Name("xmpp-server")),
        (ServiceQuery(6668, Protocol::UDP), Service::Name("ircu")),
        (ServiceQuery(691, Protocol::UDP), Service::Name("msexch-routing")),
        (ServiceQuery(2052, Protocol::UDP), Service::Name("clearvisn")),
        (ServiceQuery(1369, Protocol::TCP), Service::Name("gv-us")),
        (ServiceQuery(261, Protocol::TCP), Service::Name("nsiiops")),
        (ServiceQuery(112, Protocol::TCP), Service::Name("mcidas")),
        (ServiceQuery(8019, Protocol::UDP), Service::Name("qbdb")),
        (ServiceQuery(2857, Protocol::TCP), Service::Name("simctlp")),
        (ServiceQuery(27444, Protocol::UDP), Service::Name("Trinoo_Bcast")),
        (ServiceQuery(1644, Protocol::TCP), Service::Name("saiseh")),
        (ServiceQuery(1167, Protocol::TCP), Service::Name("cisco-ipsla")),
        (ServiceQuery(9929, Protocol::TCP), Service::Name("nping-echo")),
        (ServiceQuery(1130, Protocol::TCP), Service::Name("casp")),
        (ServiceQuery(4751, Protocol::UDP), Service::Name("spocp")),
        (ServiceQuery(4257, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1606, Protocol::UDP), Service::Name("slm-api")),
        (ServiceQuery(1845, Protocol::UDP), Service::Name("altalink")),
        (ServiceQuery(3184, Protocol::TCP), Service::Name("apogeex-port")),
        (ServiceQuery(2309, Protocol::TCP), Service::Name("sdserver")),
        (ServiceQuery(113, Protocol::UDP), Service::Name("auth")),
        (ServiceQuery(8733, Protocol::UDP), Service::Name("ibus")),
        (ServiceQuery(32774, Protocol::UDP), Service::Name("sometimes-rpc12")),
        (ServiceQuery(3682, Protocol::UDP), Service::Name("cas-mapi")),
        (ServiceQuery(3540, Protocol::UDP), Service::Name("pnrp-port")),
        (ServiceQuery(8892, Protocol::UDP), Service::Name("ddi-udp-5")),
        (ServiceQuery(7900, Protocol::TCP), Service::Name("mevent")),
        (ServiceQuery(4219, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(387, Protocol::TCP), Service::Name("aurp")),
        (ServiceQuery(1862, Protocol::TCP), Service::Name("mysql-cm-agent")),
        (ServiceQuery(5714, Protocol::TCP), Service::Name("prosharevideo")),
        (ServiceQuery(2232, Protocol::UDP), Service::Name("ivs-video")),
        (ServiceQuery(3249, Protocol::TCP), Service::Name("ssp")),
        (ServiceQuery(2949, Protocol::UDP), Service::Name("wap-pushsecure")),
        (ServiceQuery(1380, Protocol::TCP), Service::Name("telesis-licman")),
        (ServiceQuery(6506, Protocol::TCP), Service::Name("badm_pub")),
        (ServiceQuery(18262, Protocol::TCP), Service::Name("gv-pf")),
        (ServiceQuery(1529, Protocol::UDP), Service::Name("coauthor")),
        (ServiceQuery(1478, Protocol::UDP), Service::Name("ms-sna-base")),
        (ServiceQuery(1582, Protocol::TCP), Service::Name("msims")),
        (ServiceQuery(138, Protocol::UDP), Service::Name("netbios-dgm")),
        (ServiceQuery(2243, Protocol::UDP), Service::Name("magicom")),
        (ServiceQuery(2328, Protocol::TCP), Service::Name("netrix-sftm")),
        (ServiceQuery(3512, Protocol::TCP), Service::Name("aztec")),
        (ServiceQuery(3919, Protocol::UDP), Service::Name("hyperip")),
        (ServiceQuery(3549, Protocol::UDP), Service::Name("tellumat-nms")),
        (ServiceQuery(5437, Protocol::UDP), Service::Name("pmip6-data")),
        (ServiceQuery(1780, Protocol::UDP), Service::Name("dpkeyserv")),
        (ServiceQuery(3435, Protocol::UDP), Service::Name("pacom")),
        (ServiceQuery(592, Protocol::TCP), Service::Name("eudora-set")),
        (ServiceQuery(1554, Protocol::UDP), Service::Name("caci-lm")),
        (ServiceQuery(11112, Protocol::TCP), Service::Name("dicom")),
        (ServiceQuery(1532, Protocol::TCP), Service::Name("miroconnect")),
        (ServiceQuery(1037, Protocol::TCP), Service::Name("ams")),
        (ServiceQuery(8804, Protocol::TCP), Service::Name("truecm")),
        (ServiceQuery(13786, Protocol::TCP), Service::Name("nomdb")),
        (ServiceQuery(2653, Protocol::UDP), Service::Name("sonus")),
        (ServiceQuery(33123, Protocol::UDP), Service::Name("aurora-balaena")),
        (ServiceQuery(2567, Protocol::UDP), Service::Name("clp")),
        (ServiceQuery(3717, Protocol::UDP), Service::Name("wv-csp-udp-cir")),
        (ServiceQuery(2877, Protocol::UDP), Service::Name("bluelance")),
        (ServiceQuery(5265, Protocol::TCP), Service::Name("3com-njack-2")),
        (ServiceQuery(4378, Protocol::UDP), Service::Name("cp-spxdpy")),
        (ServiceQuery(3622, Protocol::UDP), Service::Name("ff-lr-port")),
        (ServiceQuery(2687, Protocol::TCP), Service::Name("pq-lic-mgmt")),
        (ServiceQuery(7167, Protocol::TCP), Service::Name("casrmagent")),
        (ServiceQuery(99, Protocol::UDP), Service::Name("metagram")),
        (ServiceQuery(2337, Protocol::TCP), Service::Name("ideesrv")),
        (ServiceQuery(8380, Protocol::TCP), Service::Name("cruise-update")),
        (ServiceQuery(589, Protocol::TCP), Service::Name("eyelink")),
        (ServiceQuery(20012, Protocol::UDP), Service::Name("ss-idi-disc")),
        (ServiceQuery(5051, Protocol::UDP), Service::Name("ita-agent")),
        (ServiceQuery(3094, Protocol::UDP), Service::Name("rapidmq-reg")),
        (ServiceQuery(2850, Protocol::UDP), Service::Name("metaconsole")),
        (ServiceQuery(195, Protocol::TCP), Service::Name("dn6-nlm-aud")),
        (ServiceQuery(3586, Protocol::UDP), Service::Name("emprise-lsc")),
        (ServiceQuery(5555, Protocol::UDP), Service::Name("rplay")),
        (ServiceQuery(20, Protocol::UDP), Service::Name("ftp-data")),
        (ServiceQuery(2345, Protocol::UDP), Service::Name("dbm")),
        (ServiceQuery(4971, Protocol::TCP), Service::Name("burp")),
        (ServiceQuery(1881, Protocol::UDP), Service::Name("ibm-mqseries2")),
        (ServiceQuery(5554, Protocol::TCP), Service::Name("sgi-esphttp")),
        (ServiceQuery(2393, Protocol::TCP), Service::Name("ms-olap1")),
        (ServiceQuery(4007, Protocol::UDP), Service::Name("pxc-splr")),
        (ServiceQuery(1856, Protocol::TCP), Service::Name("fiorano-msgsvc")),
        (ServiceQuery(2546, Protocol::TCP), Service::Name("vytalvaultbrtp")),
        (ServiceQuery(1860, Protocol::TCP), Service::Name("sunscalar-svc")),
        (ServiceQuery(4591, Protocol::UDP), Service::Name("l3t-at-an")),
        (ServiceQuery(9600, Protocol::TCP), Service::Name("micromuse-ncpw")),
        (ServiceQuery(10860, Protocol::UDP), Service::Name("helix")),
        (ServiceQuery(2394, Protocol::UDP), Service::Name("ms-olap2")),
        (ServiceQuery(2827, Protocol::UDP), Service::Name("slc-ctrlrloops")),
        (ServiceQuery(2177, Protocol::UDP), Service::Name("qwave")),
        (ServiceQuery(2744, Protocol::UDP), Service::Name("honyaku")),
        (ServiceQuery(10260, Protocol::TCP), Service::Name("axis-wimp-port")),
        (ServiceQuery(6000, Protocol::TCP), Service::Name("X11")),
        (ServiceQuery(3608, Protocol::UDP), Service::Name("trendchip-dcp")),
        (ServiceQuery(1571, Protocol::UDP), Service::Name("rdb-dbs-disp")),
        (ServiceQuery(4225, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(585, Protocol::UDP), Service::Name("imap4-ssl")),
        (ServiceQuery(2819, Protocol::TCP), Service::Name("fc-faultnotify")),
        (ServiceQuery(4940, Protocol::UDP), Service::Name("eq-office-4940")),
        (ServiceQuery(3821, Protocol::TCP), Service::Name("pmcp")),
        (ServiceQuery(2449, Protocol::UDP), Service::Name("ratl")),
        (ServiceQuery(4673, Protocol::UDP), Service::Name("cxws")),
        (ServiceQuery(2910, Protocol::TCP), Service::Name("tdaccess")),
        (ServiceQuery(560, Protocol::TCP), Service::Name("rmonitor")),
        (ServiceQuery(10003, Protocol::UDP), Service::Name("documentum_s")),
        (ServiceQuery(2514, Protocol::TCP), Service::Name("facsys-ntp")),
        (ServiceQuery(417, Protocol::TCP), Service::Name("onmux")),
        (ServiceQuery(2597, Protocol::UDP), Service::Name("homesteadglory")),
        (ServiceQuery(8991, Protocol::TCP), Service::Name("https-wmap")),
        (ServiceQuery(3332, Protocol::TCP), Service::Name("mcs-mailsvr")),
        (ServiceQuery(21000, Protocol::UDP), Service::Name("irtrans")),
        (ServiceQuery(167, Protocol::TCP), Service::Name("namp")),
        (ServiceQuery(7651, Protocol::UDP), Service::Name("cucme-4")),
        (ServiceQuery(3935, Protocol::TCP), Service::Name("sdp-portmapper")),
        (ServiceQuery(1259, Protocol::UDP), Service::Name("opennl-voice")),
        (ServiceQuery(4001, Protocol::TCP), Service::Name("newoak")),
        (ServiceQuery(2445, Protocol::UDP), Service::Name("dtn1")),
        (ServiceQuery(1800, Protocol::TCP), Service::Name("ansys-lm")),
        (ServiceQuery(4283, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(656, Protocol::UDP), Service::Name("spmp")),
        (ServiceQuery(2906, Protocol::TCP), Service::Name("caller9")),
        (ServiceQuery(4281, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1281, Protocol::TCP), Service::Name("healthd")),
        (ServiceQuery(8021, Protocol::TCP), Service::Name("ftp-proxy")),
        (ServiceQuery(1219, Protocol::UDP), Service::Name("aeroflight-ret")),
        (ServiceQuery(1084, Protocol::TCP), Service::Name("ansoft-lm-2")),
        (ServiceQuery(2875, Protocol::UDP), Service::Name("dxmessagebase2")),
        (ServiceQuery(3433, Protocol::TCP), Service::Name("alta-smp")),
        (ServiceQuery(762, Protocol::TCP), Service::Name("quotad")),
        (ServiceQuery(617, Protocol::UDP), Service::Name("sco-dtmgr")),
        (ServiceQuery(7325, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(2728, Protocol::UDP), Service::Name("sqdr")),
        (ServiceQuery(8280, Protocol::UDP), Service::Name("synapse-nhttp")),
        (ServiceQuery(2269, Protocol::UDP), Service::Name("mikey")),
        (ServiceQuery(2697, Protocol::UDP), Service::Name("oce-snmp-trap")),
        (ServiceQuery(6087, Protocol::UDP), Service::Name("ldss")),
        (ServiceQuery(2738, Protocol::UDP), Service::Name("ndl-tcp-ois-gw")),
        (ServiceQuery(387, Protocol::UDP), Service::Name("aurp")),
        (ServiceQuery(52, Protocol::TCP), Service::Name("xns-time")),
        (ServiceQuery(2107, Protocol::UDP), Service::Name("bintec-admin")),
        (ServiceQuery(484, Protocol::UDP), Service::Name("integra-sme")),
        (ServiceQuery(6142, Protocol::TCP), Service::Name("aspentec-lm")),
        (ServiceQuery(3778, Protocol::UDP), Service::Name("c-h-it-port")),
        (ServiceQuery(4269, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3434, Protocol::TCP), Service::Name("opencm")),
        (ServiceQuery(602, Protocol::UDP), Service::Name("xmlrpc-beep")),
        (ServiceQuery(5112, Protocol::TCP), Service::Name("pm-cmdsvr")),
        (ServiceQuery(2797, Protocol::UDP), Service::Name("esp-encap")),
        (ServiceQuery(5417, Protocol::UDP), Service::Name("sns-agent")),
        (ServiceQuery(2237, Protocol::UDP), Service::Name("optech-port1-lm")),
        (ServiceQuery(3648, Protocol::UDP), Service::Name("fjcp")),
        (ServiceQuery(765, Protocol::UDP), Service::Name("webster")),
        (ServiceQuery(3356, Protocol::UDP), Service::Name("upnotifyps")),
        (ServiceQuery(446, Protocol::TCP), Service::Name("ddm-rdb")),
        (ServiceQuery(2072, Protocol::UDP), Service::Name("msync")),
        (ServiceQuery(1221, Protocol::UDP), Service::Name("sweetware-apps")),
        (ServiceQuery(4101, Protocol::UDP), Service::Name("brlp-0")),
        (ServiceQuery(686, Protocol::UDP), Service::Name("hcp-wismar")),
        (ServiceQuery(38292, Protocol::TCP), Service::Name("landesk-cba")),
        (ServiceQuery(3084, Protocol::UDP), Service::Name("itm-mccs")),
        (ServiceQuery(4033, Protocol::TCP), Service::Name("sanavigator")),
        (ServiceQuery(3781, Protocol::TCP), Service::Name("abcvoice-port")),
        (ServiceQuery(10101, Protocol::TCP), Service::Name("ezmeeting-2")),
        (ServiceQuery(3834, Protocol::TCP), Service::Name("spectardata")),
        (ServiceQuery(19194, Protocol::UDP), Service::Name("ua-secureagent")),
        (ServiceQuery(1974, Protocol::UDP), Service::Name("drp")),
        (ServiceQuery(2575, Protocol::TCP), Service::Name("hl7")),
        (ServiceQuery(1861, Protocol::UDP), Service::Name("lecroy-vicp")),
        (ServiceQuery(623, Protocol::UDP), Service::Name("asf-rmcp")),
        (ServiceQuery(3745, Protocol::TCP), Service::Name("gw-call-port")),
        (ServiceQuery(529, Protocol::UDP), Service::Name("irc")),
        (ServiceQuery(3756, Protocol::TCP), Service::Name("canon-capt")),
        (ServiceQuery(2479, Protocol::TCP), Service::Name("ssm-els")),
        (ServiceQuery(11319, Protocol::UDP), Service::Name("imip")),
        (ServiceQuery(2448, Protocol::TCP), Service::Name("hpppssvr")),
        (ServiceQuery(1607, Protocol::UDP), Service::Name("stt")),
        (ServiceQuery(3491, Protocol::UDP), Service::Name("swr-port")),
        (ServiceQuery(4309, Protocol::TCP), Service::Name("dserver")),
        (ServiceQuery(3479, Protocol::TCP), Service::Name("twrpc")),
        (ServiceQuery(1438, Protocol::UDP), Service::Name("eicon-server")),
        (ServiceQuery(7280, Protocol::UDP), Service::Name("itactionserver1")),
        (ServiceQuery(4141, Protocol::UDP), Service::Name("oirtgsvc")),
        (ServiceQuery(9286, Protocol::TCP), Service::Name("n2receive")),
        (ServiceQuery(19191, Protocol::TCP), Service::Name("opsec-uaa")),
        (ServiceQuery(1334, Protocol::TCP), Service::Name("writesrv")),
        (ServiceQuery(3562, Protocol::UDP), Service::Name("sdbproxy")),
        (ServiceQuery(3933, Protocol::UDP), Service::Name("plbserve-port")),
        (ServiceQuery(4784, Protocol::UDP), Service::Name("bfd-multi-ctl")),
        (ServiceQuery(2948, Protocol::TCP), Service::Name("wap-push")),
        (ServiceQuery(1081, Protocol::TCP), Service::Name("pvuniwien")),
        (ServiceQuery(1060, Protocol::TCP), Service::Name("polestar")),
        (ServiceQuery(9024, Protocol::TCP), Service::Name("swa-2")),
        (ServiceQuery(2447, Protocol::UDP), Service::Name("ovwdb")),
        (ServiceQuery(9160, Protocol::UDP), Service::Name("apani1")),
        (ServiceQuery(4950, Protocol::UDP), Service::Name("sybasesrvmon")),
        (ServiceQuery(2702, Protocol::TCP), Service::Name("sms-xfer")),
        (ServiceQuery(2058, Protocol::TCP), Service::Name("newwavesearch")),
        (ServiceQuery(8052, Protocol::UDP), Service::Name("senomix01")),
        (ServiceQuery(1235, Protocol::TCP), Service::Name("mosaicsyssvc1")),
        (ServiceQuery(1500, Protocol::UDP), Service::Name("vlsi-lm")),
        (ServiceQuery(5225, Protocol::UDP), Service::Name("hp-server")),
        (ServiceQuery(9898, Protocol::UDP), Service::Name("monkeycom")),
        (ServiceQuery(3833, Protocol::TCP), Service::Name("aipn-auth")),
        (ServiceQuery(8132, Protocol::TCP), Service::Name("dbabble")),
        (ServiceQuery(4236, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(5604, Protocol::TCP), Service::Name("a3-sdunode")),
        (ServiceQuery(564, Protocol::UDP), Service::Name("9pfs")),
        (ServiceQuery(11600, Protocol::TCP), Service::Name("tempest-port")),
        (ServiceQuery(9599, Protocol::TCP), Service::Name("robix")),
        (ServiceQuery(263, Protocol::UDP), Service::Name("hdap")),
        (ServiceQuery(2434, Protocol::TCP), Service::Name("pxc-epmap")),
        (ServiceQuery(111, Protocol::TCP), Service::Name("rpcbind")),
        (ServiceQuery(5164, Protocol::UDP), Service::Name("vpa-disc")),
        (ServiceQuery(1735, Protocol::TCP), Service::Name("privatechat")),
        (ServiceQuery(3275, Protocol::TCP), Service::Name("samd")),
        (ServiceQuery(1853, Protocol::TCP), Service::Name("vids-avtp")),
        (ServiceQuery(74, Protocol::UDP), Service::Name("netrjs-4")),
        (ServiceQuery(16619, Protocol::TCP), Service::Name("xoms")),
        (ServiceQuery(16995, Protocol::UDP), Service::Name("amt-redir-tls")),
        (ServiceQuery(472, Protocol::TCP), Service::Name("ljk-login")),
        (ServiceQuery(385, Protocol::UDP), Service::Name("ibm-app")),
        (ServiceQuery(3284, Protocol::UDP), Service::Name("4talk")),
        (ServiceQuery(95, Protocol::TCP), Service::Name("supdup")),
        (ServiceQuery(2997, Protocol::TCP), Service::Name("rebol")),
        (ServiceQuery(515, Protocol::UDP), Service::Name("printer")),
        (ServiceQuery(3460, Protocol::TCP), Service::Name("edm-manager")),
        (ServiceQuery(482, Protocol::TCP), Service::Name("bgs-nsi")),
        (ServiceQuery(3111, Protocol::UDP), Service::Name("wsynch")),
        (ServiceQuery(3026, Protocol::TCP), Service::Name("agri-gateway")),
        (ServiceQuery(2453, Protocol::UDP), Service::Name("madge-ltd")),
        (ServiceQuery(2446, Protocol::TCP), Service::Name("bues_service")),
        (ServiceQuery(1622, Protocol::UDP), Service::Name("ontime")),
        (ServiceQuery(17555, Protocol::TCP), Service::Name("ailith")),
        (ServiceQuery(3881, Protocol::TCP), Service::Name("idac")),
        (ServiceQuery(7315, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(3462, Protocol::TCP), Service::Name("track")),
        (ServiceQuery(3416, Protocol::TCP), Service::Name("commandport")),
        (ServiceQuery(1563, Protocol::UDP), Service::Name("cadabra-lm")),
        (ServiceQuery(2149, Protocol::UDP), Service::Name("acptsys")),
        (ServiceQuery(4066, Protocol::UDP), Service::Name("pmas")),
        (ServiceQuery(3902, Protocol::TCP), Service::Name("nimaux")),
        (ServiceQuery(8195, Protocol::TCP), Service::Name("blp2")),
        (ServiceQuery(1053, Protocol::TCP), Service::Name("remote-as")),
        (ServiceQuery(3574, Protocol::TCP), Service::Name("dmaf-server")),
        (ServiceQuery(4680, Protocol::UDP), Service::Name("mgemanagement")),
        (ServiceQuery(3456, Protocol::TCP), Service::Name("vat")),
        (ServiceQuery(7779, Protocol::UDP), Service::Name("vstat")),
        (ServiceQuery(1549, Protocol::UDP), Service::Name("shivasound")),
        (ServiceQuery(4345, Protocol::UDP), Service::Name("m4-network-as")),
        (ServiceQuery(4251, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(686, Protocol::TCP), Service::Name("hcp-wismar")),
        (ServiceQuery(3558, Protocol::UDP), Service::Name("mcp-port")),
        (ServiceQuery(4350, Protocol::UDP), Service::Name("net-device")),
        (ServiceQuery(4064, Protocol::TCP), Service::Name("ice-srouter")),
        (ServiceQuery(5200, Protocol::TCP), Service::Name("targus-getdata")),
        (ServiceQuery(3819, Protocol::TCP), Service::Name("epl-slp")),
        (ServiceQuery(5713, Protocol::UDP), Service::Name("proshareaudio")),
        (ServiceQuery(1127, Protocol::UDP), Service::Name("kwdb-commn")),
        (ServiceQuery(1900, Protocol::UDP), Service::Name("upnp")),
        (ServiceQuery(4883, Protocol::TCP), Service::Name("wmlserver")),
        (ServiceQuery(6620, Protocol::TCP), Service::Name("kftp-data")),
        (ServiceQuery(2026, Protocol::UDP), Service::Name("scrabble")),
        (ServiceQuery(1408, Protocol::TCP), Service::Name("sophia-lm")),
        (ServiceQuery(3075, Protocol::TCP), Service::Name("orbix-locator")),
        (ServiceQuery(2180, Protocol::TCP), Service::Name("mc-gt-srv")),
        (ServiceQuery(5463, Protocol::TCP), Service::Name("ttlpriceproxy")),
        (ServiceQuery(4097, Protocol::UDP), Service::Name("patrolview")),
        (ServiceQuery(2104, Protocol::UDP), Service::Name("zephyr-hm")),
        (ServiceQuery(8699, Protocol::UDP), Service::Name("vnyx")),
        (ServiceQuery(1237, Protocol::TCP), Service::Name("tsdos390")),
        (ServiceQuery(2425, Protocol::TCP), Service::Name("fjitsuappmgr")),
        (ServiceQuery(5633, Protocol::TCP), Service::Name("beorl")),
        (ServiceQuery(5029, Protocol::UDP), Service::Name("infobright")),
        (ServiceQuery(10805, Protocol::UDP), Service::Name("lpdg")),
        (ServiceQuery(1610, Protocol::TCP), Service::Name("taurus-wh")),
        (ServiceQuery(3091, Protocol::TCP), Service::Name("1ci-smcs")),
        (ServiceQuery(8020, Protocol::UDP), Service::Name("intu-ec-svcdisc")),
        (ServiceQuery(6673, Protocol::TCP), Service::Name("vision_elmd")),
        (ServiceQuery(6964, Protocol::TCP), Service::Name("swismgr2")),
        (ServiceQuery(832, Protocol::TCP), Service::Name("netconfsoaphttp")),
        (ServiceQuery(2099, Protocol::TCP), Service::Name("h2250-annex-g")),
        (ServiceQuery(7227, Protocol::TCP), Service::Name("ramp")),
        (ServiceQuery(45045, Protocol::TCP), Service::Name("synctest")),
        (ServiceQuery(1950, Protocol::TCP), Service::Name("ismaeasdaqtest")),
        (ServiceQuery(1882, Protocol::UDP), Service::Name("ecsqdmn")),
        (ServiceQuery(1752, Protocol::TCP), Service::Name("lofr-lm")),
        (ServiceQuery(6502, Protocol::TCP), Service::Name("netop-rc")),
        (ServiceQuery(3465, Protocol::TCP), Service::Name("edm-mgr-cntrl")),
        (ServiceQuery(7443, Protocol::UDP), Service::Name("oracleas-https")),
        (ServiceQuery(4181, Protocol::UDP), Service::Name("macbak")),
        (ServiceQuery(6056, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(2897, Protocol::UDP), Service::Name("citrix-rtmp")),
        (ServiceQuery(1456, Protocol::UDP), Service::Name("dca")),
        (ServiceQuery(3528, Protocol::TCP), Service::Name("jboss-iiop")),
        (ServiceQuery(1407, Protocol::UDP), Service::Name("dbsa-lm")),
        (ServiceQuery(5842, Protocol::TCP), Service::Name("reversion")),
        (ServiceQuery(2856, Protocol::UDP), Service::Name("cesdinv")),
        (ServiceQuery(9953, Protocol::UDP), Service::Name("acis")),
        (ServiceQuery(564, Protocol::TCP), Service::Name("9pfs")),
        (ServiceQuery(20003, Protocol::TCP), Service::Name("commtact-https")),
        (ServiceQuery(4987, Protocol::TCP), Service::Name("maybe-veritas")),
        (ServiceQuery(2321, Protocol::UDP), Service::Name("rdlap")),
        (ServiceQuery(3384, Protocol::UDP), Service::Name("hp-clic")),
        (ServiceQuery(1910, Protocol::TCP), Service::Name("ultrabac")),
        (ServiceQuery(2891, Protocol::UDP), Service::Name("cinegrfx-elmd")),
        (ServiceQuery(2419, Protocol::TCP), Service::Name("attachmate-s2s")),
        (ServiceQuery(2153, Protocol::TCP), Service::Name("ctlptc")),
        (ServiceQuery(1859, Protocol::UDP), Service::Name("gammafetchsvr")),
        (ServiceQuery(1126, Protocol::TCP), Service::Name("hpvmmdata")),
        (ServiceQuery(5042, Protocol::UDP), Service::Name("asnaacceler8db")),
        (ServiceQuery(4310, Protocol::TCP), Service::Name("mirrtex")),
        (ServiceQuery(5156, Protocol::TCP), Service::Name("rugameonline")),
        (ServiceQuery(9119, Protocol::TCP), Service::Name("mxit")),
        (ServiceQuery(3012, Protocol::UDP), Service::Name("twsdss")),
        (ServiceQuery(13721, Protocol::UDP), Service::Name("bpdbm")),
        (ServiceQuery(4535, Protocol::UDP), Service::Name("ehs")),
        (ServiceQuery(3225, Protocol::UDP), Service::Name("fcip-port")),
        (ServiceQuery(10010, Protocol::TCP), Service::Name("rxapi")),
        (ServiceQuery(1639, Protocol::TCP), Service::Name("cert-initiator")),
        (ServiceQuery(4258, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(6628, Protocol::UDP), Service::Name("afesc-mc")),
        (ServiceQuery(16995, Protocol::TCP), Service::Name("amt-redir-tls")),
        (ServiceQuery(8055, Protocol::UDP), Service::Name("senomix04")),
        (ServiceQuery(2764, Protocol::TCP), Service::Name("data-insurance")),
        (ServiceQuery(2677, Protocol::TCP), Service::Name("gadgetgate1way")),
        (ServiceQuery(76, Protocol::TCP), Service::Name("deos")),
        (ServiceQuery(1970, Protocol::UDP), Service::Name("netop-rc")),
        (ServiceQuery(3420, Protocol::TCP), Service::Name("ifcp-port")),
        (ServiceQuery(3115, Protocol::TCP), Service::Name("mctet-master")),
        (ServiceQuery(3960, Protocol::UDP), Service::Name("bess")),
        (ServiceQuery(9875, Protocol::UDP), Service::Name("sapv1")),
        (ServiceQuery(6510, Protocol::UDP), Service::Name("mcer-port")),
        (ServiceQuery(171, Protocol::TCP), Service::Name("multiplex")),
        (ServiceQuery(2640, Protocol::UDP), Service::Name("sai_sentlm")),
        (ServiceQuery(3774, Protocol::UDP), Service::Name("zicom")),
        (ServiceQuery(9876, Protocol::UDP), Service::Name("sd")),
        (ServiceQuery(24002, Protocol::TCP), Service::Name("med-fsp-tx")),
        (ServiceQuery(2400, Protocol::TCP), Service::Name("opequus-server")),
        (ServiceQuery(2601, Protocol::UDP), Service::Name("discp-client")),
        (ServiceQuery(3241, Protocol::UDP), Service::Name("sysorb")),
        (ServiceQuery(699, Protocol::TCP), Service::Name("accessnetwork")),
        (ServiceQuery(3646, Protocol::UDP), Service::Name("xss-srv-port")),
        (ServiceQuery(6623, Protocol::TCP), Service::Name("ktelnet")),
        (ServiceQuery(139, Protocol::TCP), Service::Name("netbios-ssn")),
        (ServiceQuery(9753, Protocol::TCP), Service::Name("rasadv")),
        (ServiceQuery(3944, Protocol::TCP), Service::Name("sops")),
        (ServiceQuery(1330, Protocol::TCP), Service::Name("streetperfect")),
        (ServiceQuery(5540, Protocol::TCP), Service::Name("sdreport")),
        (ServiceQuery(10161, Protocol::UDP), Service::Name("snmpdtls")),
        (ServiceQuery(2849, Protocol::UDP), Service::Name("fxp")),
        (ServiceQuery(355, Protocol::UDP), Service::Name("datex-asn")),
        (ServiceQuery(3939, Protocol::TCP), Service::Name("aamp")),
        (ServiceQuery(1581, Protocol::UDP), Service::Name("mil-2045-47001")),
        (ServiceQuery(2865, Protocol::UDP), Service::Name("pit-vpn")),
        (ServiceQuery(2602, Protocol::TCP), Service::Name("ripd")),
        (ServiceQuery(1153, Protocol::TCP), Service::Name("c1222-acse")),
        (ServiceQuery(6669, Protocol::UDP), Service::Name("ircu")),
        (ServiceQuery(2744, Protocol::TCP), Service::Name("honyaku")),
        (ServiceQuery(2949, Protocol::TCP), Service::Name("wap-pushsecure")),
        (ServiceQuery(8417, Protocol::UDP), Service::Name("espeech-rtp")),
        (ServiceQuery(2279, Protocol::UDP), Service::Name("xmquery")),
        (ServiceQuery(3512, Protocol::UDP), Service::Name("aztec")),
        (ServiceQuery(36423, Protocol::TCP), Service::Name("slmap")),
        (ServiceQuery(10080, Protocol::UDP), Service::Name("amanda")),
        (ServiceQuery(463, Protocol::TCP), Service::Name("alpes")),
        (ServiceQuery(6489, Protocol::UDP), Service::Name("sun-sr-admin")),
        (ServiceQuery(3032, Protocol::TCP), Service::Name("redwood-chat")),
        (ServiceQuery(1626, Protocol::UDP), Service::Name("shockwave")),
        (ServiceQuery(3570, Protocol::TCP), Service::Name("mccwebsvr-port")),
        (ServiceQuery(3899, Protocol::UDP), Service::Name("itv-control")),
        (ServiceQuery(13720, Protocol::TCP), Service::Name("netbackup")),
        (ServiceQuery(8763, Protocol::TCP), Service::Name("mc-appserver")),
        (ServiceQuery(5300, Protocol::TCP), Service::Name("hacl-hb")),
        (ServiceQuery(3703, Protocol::UDP), Service::Name("adobeserver-3")),
        (ServiceQuery(2975, Protocol::TCP), Service::Name("fjmpcm")),
        (ServiceQuery(2185, Protocol::TCP), Service::Name("onbase-dds")),
        (ServiceQuery(41797, Protocol::TCP), Service::Name("crestron-ctps")),
        (ServiceQuery(1439, Protocol::UDP), Service::Name("eicon-x25")),
        (ServiceQuery(493, Protocol::TCP), Service::Name("ticf-2")),
        (ServiceQuery(2470, Protocol::UDP), Service::Name("taskman-port")),
        (ServiceQuery(1802, Protocol::UDP), Service::Name("concomp1")),
        (ServiceQuery(4077, Protocol::TCP), Service::Name("ascomalarm")),
        (ServiceQuery(4175, Protocol::TCP), Service::Name("bccp")),
        (ServiceQuery(3890, Protocol::TCP), Service::Name("ndsconnect")),
        (ServiceQuery(3991, Protocol::TCP), Service::Name("bv-smcsrv")),
        (ServiceQuery(3407, Protocol::TCP), Service::Name("ldap-admin")),
        (ServiceQuery(3418, Protocol::TCP), Service::Name("rnmap")),
        (ServiceQuery(5362, Protocol::UDP), Service::Name("serverwsd2")),
        (ServiceQuery(3025, Protocol::UDP), Service::Name("arepa-raft")),
        (ServiceQuery(9278, Protocol::TCP), Service::Name("pegasus")),
        (ServiceQuery(34378, Protocol::TCP), Service::Name("p-net-local")),
        (ServiceQuery(449, Protocol::TCP), Service::Name("as-servermap")),
        (ServiceQuery(1761, Protocol::TCP), Service::Name("landesk-rc")),
        (ServiceQuery(1899, Protocol::UDP), Service::Name("mc2studios")),
        (ServiceQuery(701, Protocol::UDP), Service::Name("lmp")),
        (ServiceQuery(2169, Protocol::TCP), Service::Name("brain")),
        (ServiceQuery(10288, Protocol::UDP), Service::Name("blocks")),
        (ServiceQuery(1422, Protocol::UDP), Service::Name("autodesk-lm")),
        (ServiceQuery(5248, Protocol::UDP), Service::Name("caacws")),
        (ServiceQuery(175, Protocol::UDP), Service::Name("vmnet")),
        (ServiceQuery(18668, Protocol::TCP), Service::Name("vdmmesh")),
        (ServiceQuery(19410, Protocol::TCP), Service::Name("hp-sco")),
        (ServiceQuery(1673, Protocol::UDP), Service::Name("proshare-mc-1")),
        (ServiceQuery(8005, Protocol::UDP), Service::Name("mxi")),
        (ServiceQuery(3731, Protocol::UDP), Service::Name("smap")),
        (ServiceQuery(587, Protocol::UDP), Service::Name("submission")),
        (ServiceQuery(5049, Protocol::UDP), Service::Name("ivocalize")),
        (ServiceQuery(1083, Protocol::UDP), Service::Name("ansoft-lm-1")),
        (ServiceQuery(6715, Protocol::UDP), Service::Name("fibotrader-com")),
        (ServiceQuery(3430, Protocol::TCP), Service::Name("ssdispatch")),
        (ServiceQuery(1609, Protocol::TCP), Service::Name("isysg-lm")),
        (ServiceQuery(3575, Protocol::TCP), Service::Name("ccm-port")),
        (ServiceQuery(489, Protocol::UDP), Service::Name("nest-protocol")),
        (ServiceQuery(2995, Protocol::UDP), Service::Name("idrs")),
        (ServiceQuery(9612, Protocol::UDP), Service::Name("streamcomm-ds")),
        (ServiceQuery(1454, Protocol::TCP), Service::Name("interhdl_elmd")),
        (ServiceQuery(3179, Protocol::TCP), Service::Name("h2gf-w-2m")),
        (ServiceQuery(8111, Protocol::TCP), Service::Name("skynetflow")),
        (ServiceQuery(4750, Protocol::TCP), Service::Name("ssad")),
        (ServiceQuery(3613, Protocol::UDP), Service::Name("alaris-disc")),
        (ServiceQuery(3076, Protocol::UDP), Service::Name("orbix-config")),
        (ServiceQuery(4066, Protocol::TCP), Service::Name("pmas")),
        (ServiceQuery(18000, Protocol::UDP), Service::Name("biimenu")),
        (ServiceQuery(5646, Protocol::TCP), Service::Name("vfmobile")),
        (ServiceQuery(4211, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1574, Protocol::TCP), Service::Name("mvel-lm")),
        (ServiceQuery(5631, Protocol::TCP), Service::Name("pcanywheredata")),
        (ServiceQuery(2025, Protocol::TCP), Service::Name("ellpack")),
        (ServiceQuery(3976, Protocol::UDP), Service::Name("opswagent")),
        (ServiceQuery(2216, Protocol::TCP), Service::Name("vtu-comms")),
        (ServiceQuery(4024, Protocol::UDP), Service::Name("tnp1-port")),
        (ServiceQuery(3344, Protocol::TCP), Service::Name("bnt-manager")),
        (ServiceQuery(1067, Protocol::UDP), Service::Name("instl_boots")),
        (ServiceQuery(5746, Protocol::TCP), Service::Name("fcopys-server")),
        (ServiceQuery(2860, Protocol::UDP), Service::Name("dialpad-voice1")),
        (ServiceQuery(10107, Protocol::TCP), Service::Name("bctp-server")),
        (ServiceQuery(2052, Protocol::TCP), Service::Name("clearvisn")),
        (ServiceQuery(156, Protocol::TCP), Service::Name("sqlsrv")),
        (ServiceQuery(504, Protocol::TCP), Service::Name("citadel")),
        (ServiceQuery(2733, Protocol::UDP), Service::Name("signet-ctf")),
        (ServiceQuery(3443, Protocol::UDP), Service::Name("ov-nnm-websrv")),
        (ServiceQuery(1066, Protocol::UDP), Service::Name("fpo-fns")),
        (ServiceQuery(3841, Protocol::UDP), Service::Name("zfirm-shiprush3")),
        (ServiceQuery(3847, Protocol::TCP), Service::Name("msfw-control")),
        (ServiceQuery(9025, Protocol::TCP), Service::Name("swa-3")),
        (ServiceQuery(7277, Protocol::TCP), Service::Name("oma-ilp-s")),
        (ServiceQuery(1844, Protocol::TCP), Service::Name("direcpc-dll")),
        (ServiceQuery(79, Protocol::UDP), Service::Name("finger")),
        (ServiceQuery(5010, Protocol::TCP), Service::Name("telelpathstart")),
        (ServiceQuery(4321, Protocol::TCP), Service::Name("rwhois")),
        (ServiceQuery(6011, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(2244, Protocol::TCP), Service::Name("nmsserver")),
        (ServiceQuery(8378, Protocol::UDP), Service::Name("cruise-config")),
        (ServiceQuery(23053, Protocol::TCP), Service::Name("gntp")),
        (ServiceQuery(2709, Protocol::UDP), Service::Name("supermon")),
        (ServiceQuery(6022, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(4940, Protocol::TCP), Service::Name("eq-office-4940")),
        (ServiceQuery(3475, Protocol::UDP), Service::Name("genisar-port")),
        (ServiceQuery(3705, Protocol::UDP), Service::Name("adobeserver-5")),
        (ServiceQuery(5675, Protocol::UDP), Service::Name("v5ua")),
        (ServiceQuery(3563, Protocol::UDP), Service::Name("watcomdebug")),
        (ServiceQuery(4178, Protocol::UDP), Service::Name("storman")),
        (ServiceQuery(445, Protocol::UDP), Service::Name("microsoft-ds")),
        (ServiceQuery(5006, Protocol::UDP), Service::Name("wsm-server")),
        (ServiceQuery(2318, Protocol::UDP), Service::Name("cadencecontrol")),
        (ServiceQuery(5782, Protocol::TCP), Service::Name("3par-mgmt")),
        (ServiceQuery(5146, Protocol::TCP), Service::Name("social-alarm")),
        (ServiceQuery(3150, Protocol::TCP), Service::Name("nm-asses-admin")),
        (ServiceQuery(1756, Protocol::TCP), Service::Name("capfast-lmd")),
        (ServiceQuery(2741, Protocol::TCP), Service::Name("tsb")),
        (ServiceQuery(3946, Protocol::UDP), Service::Name("backupedge")),
        (ServiceQuery(384, Protocol::UDP), Service::Name("arns")),
        (ServiceQuery(265, Protocol::UDP), Service::Name("x-bone-ctl")),
        (ServiceQuery(7677, Protocol::TCP), Service::Name("sun-user-https")),
        (ServiceQuery(1868, Protocol::TCP), Service::Name("viziblebrowser")),
        (ServiceQuery(2148, Protocol::UDP), Service::Name("veritas-ucl")),
        (ServiceQuery(683, Protocol::TCP), Service::Name("corba-iiop")),
        (ServiceQuery(4340, Protocol::TCP), Service::Name("gaia")),
        (ServiceQuery(1318, Protocol::TCP), Service::Name("krb5gatekeeper")),
        (ServiceQuery(1393, Protocol::UDP), Service::Name("iclpv-nls")),
        (ServiceQuery(3550, Protocol::UDP), Service::Name("ssmpp")),
        (ServiceQuery(8032, Protocol::UDP), Service::Name("pro-ed")),
        (ServiceQuery(1969, Protocol::TCP), Service::Name("lipsinc1")),
        (ServiceQuery(2886, Protocol::UDP), Service::Name("responselogic")),
        (ServiceQuery(1364, Protocol::UDP), Service::Name("ndm-server")),
        (ServiceQuery(6047, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(8000, Protocol::UDP), Service::Name("irdmi")),
        (ServiceQuery(690, Protocol::TCP), Service::Name("vatp")),
        (ServiceQuery(6670, Protocol::TCP), Service::Name("irc")),
        (ServiceQuery(5001, Protocol::UDP), Service::Name("commplex-link")),
        (ServiceQuery(1779, Protocol::TCP), Service::Name("pharmasoft")),
        (ServiceQuery(2561, Protocol::TCP), Service::Name("mosaixcc")),
        (ServiceQuery(617, Protocol::TCP), Service::Name("sco-dtmgr")),
        (ServiceQuery(6702, Protocol::TCP), Service::Name("e-design-net")),
        (ServiceQuery(6074, Protocol::UDP), Service::Name("max")),
        (ServiceQuery(3760, Protocol::TCP), Service::Name("adtempusclient")),
        (ServiceQuery(13929, Protocol::TCP), Service::Name("dta-systems")),
        (ServiceQuery(2837, Protocol::TCP), Service::Name("repliweb")),
        (ServiceQuery(8888, Protocol::TCP), Service::Name("sun-answerbook")),
        (ServiceQuery(1138, Protocol::TCP), Service::Name("encrypted_admin")),
        (ServiceQuery(10050, Protocol::UDP), Service::Name("zabbix-agent")),
        (ServiceQuery(1721, Protocol::UDP), Service::Name("caicci")),
        (ServiceQuery(2810, Protocol::UDP), Service::Name("netsteward")),
        (ServiceQuery(1152, Protocol::UDP), Service::Name("winpoplanmess")),
        (ServiceQuery(31337, Protocol::TCP), Service::Name("Elite")),
        (ServiceQuery(4034, Protocol::UDP), Service::Name("ubxd")),
        (ServiceQuery(21554, Protocol::UDP), Service::Name("dfserver")),
        (ServiceQuery(4239, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2154, Protocol::UDP), Service::Name("stdptc")),
        (ServiceQuery(480, Protocol::UDP), Service::Name("iafdbase")),
        (ServiceQuery(3810, Protocol::UDP), Service::Name("wlanauth")),
        (ServiceQuery(1290, Protocol::TCP), Service::Name("winjaserver")),
        (ServiceQuery(3934, Protocol::TCP), Service::Name("sunfm-port")),
        (ServiceQuery(2460, Protocol::TCP), Service::Name("ms-theater")),
        (ServiceQuery(1965, Protocol::TCP), Service::Name("tivoli-npm")),
        (ServiceQuery(25000, Protocol::UDP), Service::Name("icl-twobase1")),
        (ServiceQuery(1100, Protocol::UDP), Service::Name("mctp")),
        (ServiceQuery(2822, Protocol::TCP), Service::Name("ka0wuc")),
        (ServiceQuery(3453, Protocol::UDP), Service::Name("pscupd")),
        (ServiceQuery(3754, Protocol::UDP), Service::Name("timestenbroker")),
        (ServiceQuery(8666, Protocol::TCP), Service::Name("monetra-admin")),
        (ServiceQuery(269, Protocol::UDP), Service::Name("manet")),
        (ServiceQuery(6360, Protocol::TCP), Service::Name("metaedit-mu")),
        (ServiceQuery(6627, Protocol::TCP), Service::Name("nexgen")),
        (ServiceQuery(1823, Protocol::TCP), Service::Name("unisys-lm")),
        (ServiceQuery(3313, Protocol::TCP), Service::Name("uorb")),
        (ServiceQuery(1498, Protocol::UDP), Service::Name("watcom-sql")),
        (ServiceQuery(3373, Protocol::UDP), Service::Name("lavenir-lm")),
        (ServiceQuery(3629, Protocol::UDP), Service::Name("escvpnet")),
        (ServiceQuery(2172, Protocol::UDP), Service::Name("msfw-s-storage")),
        (ServiceQuery(83, Protocol::UDP), Service::Name("mit-ml-dev")),
        (ServiceQuery(6515, Protocol::TCP), Service::Name("elipse-rec")),
        (ServiceQuery(5224, Protocol::TCP), Service::Name("hpvirtctrl")),
        (ServiceQuery(3183, Protocol::TCP), Service::Name("cops-tls")),
        (ServiceQuery(1719, Protocol::TCP), Service::Name("h323gatestat")),
        (ServiceQuery(151, Protocol::TCP), Service::Name("hems")),
        (ServiceQuery(1207, Protocol::TCP), Service::Name("metasage")),
        (ServiceQuery(4659, Protocol::TCP), Service::Name("playsta2-lob")),
        (ServiceQuery(4131, Protocol::UDP), Service::Name("stars")),
        (ServiceQuery(5963, Protocol::UDP), Service::Name("indy")),
        (ServiceQuery(12002, Protocol::UDP), Service::Name("entexthigh")),
        (ServiceQuery(2130, Protocol::TCP), Service::Name("xds")),
        (ServiceQuery(52, Protocol::UDP), Service::Name("xns-time")),
        (ServiceQuery(4043, Protocol::UDP), Service::Name("nirp")),
        (ServiceQuery(3632, Protocol::TCP), Service::Name("distccd")),
        (ServiceQuery(3746, Protocol::UDP), Service::Name("linktest")),
        (ServiceQuery(25002, Protocol::TCP), Service::Name("icl-twobase3")),
        (ServiceQuery(1103, Protocol::TCP), Service::Name("xaudio")),
        (ServiceQuery(1197, Protocol::TCP), Service::Name("carrius-rshell")),
        (ServiceQuery(2301, Protocol::UDP), Service::Name("cpq-wbem")),
        (ServiceQuery(1402, Protocol::UDP), Service::Name("prm-sm-np")),
        (ServiceQuery(8555, Protocol::UDP), Service::Name("d-fence")),
        (ServiceQuery(2961, Protocol::TCP), Service::Name("boldsoft-lm")),
        (ServiceQuery(539, Protocol::UDP), Service::Name("apertus-ldp")),
        (ServiceQuery(8901, Protocol::TCP), Service::Name("jmb-cds2")),
        (ServiceQuery(11164, Protocol::TCP), Service::Name("suncacao-csa")),
        (ServiceQuery(3286, Protocol::UDP), Service::Name("e-net")),
        (ServiceQuery(5553, Protocol::TCP), Service::Name("sgi-eventmond")),
        (ServiceQuery(3943, Protocol::TCP), Service::Name("tig")),
        (ServiceQuery(2952, Protocol::TCP), Service::Name("mpfwsas")),
        (ServiceQuery(3303, Protocol::UDP), Service::Name("opsession-clnt")),
        (ServiceQuery(4000, Protocol::UDP), Service::Name("icq")),
        (ServiceQuery(5201, Protocol::TCP), Service::Name("targus-getdata1")),
        (ServiceQuery(2199, Protocol::UDP), Service::Name("onehome-help")),
        (ServiceQuery(95, Protocol::UDP), Service::Name("supdup")),
        (ServiceQuery(7181, Protocol::TCP), Service::Name("janus-disc")),
        (ServiceQuery(3443, Protocol::TCP), Service::Name("ov-nnm-websrv")),
        (ServiceQuery(1516, Protocol::UDP), Service::Name("vpad")),
        (ServiceQuery(6509, Protocol::UDP), Service::Name("mgcs-mfp-port")),
        (ServiceQuery(5104, Protocol::TCP), Service::Name("tinymessage")),
        (ServiceQuery(4416, Protocol::TCP), Service::Name("pjj-player")),
        (ServiceQuery(1903, Protocol::UDP), Service::Name("linkname")),
        (ServiceQuery(3261, Protocol::TCP), Service::Name("winshadow")),
        (ServiceQuery(6440, Protocol::TCP), Service::Name("heliosd")),
        (ServiceQuery(3895, Protocol::UDP), Service::Name("syam-smc")),
        (ServiceQuery(6032, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(2013, Protocol::UDP), Service::Name("raid-cd")),
        (ServiceQuery(3758, Protocol::UDP), Service::Name("apw-registry")),
        (ServiceQuery(503, Protocol::UDP), Service::Name("intrinsa")),
        (ServiceQuery(2277, Protocol::TCP), Service::Name("bluectrlproxy")),
        (ServiceQuery(3942, Protocol::UDP), Service::Name("srdp")),
        (ServiceQuery(1761, Protocol::UDP), Service::Name("cft-0")),
        (ServiceQuery(7330, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(2355, Protocol::TCP), Service::Name("psdbserver")),
        (ServiceQuery(2038, Protocol::UDP), Service::Name("objectmanager")),
        (ServiceQuery(62, Protocol::UDP), Service::Name("acas")),
        (ServiceQuery(4308, Protocol::TCP), Service::Name("compx-lockview")),
        (ServiceQuery(3997, Protocol::TCP), Service::Name("agentsease-db")),
        (ServiceQuery(10009, Protocol::TCP), Service::Name("swdtp-sv")),
        (ServiceQuery(7967, Protocol::UDP), Service::Name("supercell")),
        (ServiceQuery(3031, Protocol::UDP), Service::Name("eppc")),
        (ServiceQuery(2474, Protocol::TCP), Service::Name("vitalanalysis")),
        (ServiceQuery(8193, Protocol::TCP), Service::Name("sophos")),
        (ServiceQuery(3661, Protocol::UDP), Service::Name("can-ferret-ssl")),
        (ServiceQuery(7397, Protocol::UDP), Service::Name("hexarc")),
        (ServiceQuery(3182, Protocol::TCP), Service::Name("bmcpatrolrnvu")),
        (ServiceQuery(687, Protocol::UDP), Service::Name("asipregistry")),
        (ServiceQuery(3042, Protocol::UDP), Service::Name("journee")),
        (ServiceQuery(5361, Protocol::UDP), Service::Name("ms-s-sideshow")),
        (ServiceQuery(5412, Protocol::TCP), Service::Name("continuus")),
        (ServiceQuery(33331, Protocol::TCP), Service::Name("diamondport")),
        (ServiceQuery(3916, Protocol::UDP), Service::Name("wysdmc")),
        (ServiceQuery(1942, Protocol::UDP), Service::Name("res")),
        (ServiceQuery(3154, Protocol::TCP), Service::Name("ccmrmi")),
        (ServiceQuery(3008, Protocol::UDP), Service::Name("midnight-tech")),
        (ServiceQuery(15, Protocol::TCP), Service::Name("netstat")),
        (ServiceQuery(116, Protocol::UDP), Service::Name("ansanotify")),
        (ServiceQuery(4148, Protocol::TCP), Service::Name("hhb-handheld")),
        (ServiceQuery(3356, Protocol::TCP), Service::Name("upnotifyps")),
        (ServiceQuery(1363, Protocol::TCP), Service::Name("ndm-requester")),
        (ServiceQuery(1179, Protocol::TCP), Service::Name("b2n")),
        (ServiceQuery(2790, Protocol::UDP), Service::Name("plgproxy")),
        (ServiceQuery(1543, Protocol::TCP), Service::Name("simba-cs")),
        (ServiceQuery(1172, Protocol::UDP), Service::Name("dnap")),
        (ServiceQuery(4800, Protocol::UDP), Service::Name("iims")),
        (ServiceQuery(2886, Protocol::TCP), Service::Name("responselogic")),
        (ServiceQuery(2648, Protocol::TCP), Service::Name("upsnotifyprot")),
        (ServiceQuery(660, Protocol::UDP), Service::Name("mac-srvr-admin")),
        (ServiceQuery(3793, Protocol::UDP), Service::Name("dcsoftware")),
        (ServiceQuery(22335, Protocol::TCP), Service::Name("shrewd-control")),
        (ServiceQuery(3305, Protocol::UDP), Service::Name("odette-ftp")),
        (ServiceQuery(1824, Protocol::UDP), Service::Name("metrics-pas")),
        (ServiceQuery(7630, Protocol::TCP), Service::Name("hawk")),
        (ServiceQuery(309, Protocol::TCP), Service::Name("entrusttime")),
        (ServiceQuery(9318, Protocol::UDP), Service::Name("secure-ts")),
        (ServiceQuery(625, Protocol::TCP), Service::Name("apple-xsrvr-admin")),
        (ServiceQuery(3506, Protocol::UDP), Service::Name("apc-3506")),
        (ServiceQuery(1498, Protocol::TCP), Service::Name("watcom-sql")),
        (ServiceQuery(16309, Protocol::TCP), Service::Name("etb4j")),
        (ServiceQuery(147, Protocol::UDP), Service::Name("iso-ip")),
        (ServiceQuery(2694, Protocol::UDP), Service::Name("pwrsevent")),
        (ServiceQuery(10261, Protocol::TCP), Service::Name("tile-ml")),
        (ServiceQuery(2146, Protocol::TCP), Service::Name("lv-not")),
        (ServiceQuery(760, Protocol::TCP), Service::Name("krbupdate")),
        (ServiceQuery(491, Protocol::UDP), Service::Name("go-login")),
        (ServiceQuery(4551, Protocol::UDP), Service::Name("ieee-mih")),
        (ServiceQuery(16991, Protocol::TCP), Service::Name("intel-rci-mp")),
        (ServiceQuery(645, Protocol::UDP), Service::Name("pssc")),
        (ServiceQuery(687, Protocol::TCP), Service::Name("asipregistry")),
        (ServiceQuery(3147, Protocol::UDP), Service::Name("rfio")),
        (ServiceQuery(3594, Protocol::UDP), Service::Name("mediaspace")),
        (ServiceQuery(436, Protocol::TCP), Service::Name("dna-cml")),
        (ServiceQuery(3509, Protocol::UDP), Service::Name("vt-ssl")),
        (ServiceQuery(4378, Protocol::TCP), Service::Name("cp-spxdpy")),
        (ServiceQuery(1632, Protocol::UDP), Service::Name("pammratc")),
        (ServiceQuery(6163, Protocol::UDP), Service::Name("pscribe")),
        (ServiceQuery(3830, Protocol::UDP), Service::Name("cernsysmgmtagt")),
        (ServiceQuery(10003, Protocol::TCP), Service::Name("documentum_s")),
        (ServiceQuery(8981, Protocol::TCP), Service::Name("nod-client")),
        (ServiceQuery(2824, Protocol::UDP), Service::Name("cqg-netlan-1")),
        (ServiceQuery(30000, Protocol::TCP), Service::Name("ndmps")),
        (ServiceQuery(4156, Protocol::TCP), Service::Name("stat-results")),
        (ServiceQuery(3432, Protocol::TCP), Service::Name("osdcp")),
        (ServiceQuery(2332, Protocol::TCP), Service::Name("rcc-host")),
        (ServiceQuery(8883, Protocol::TCP), Service::Name("secure-mqtt")),
        (ServiceQuery(8445, Protocol::TCP), Service::Name("copy")),
        (ServiceQuery(43188, Protocol::UDP), Service::Name("reachout")),
        (ServiceQuery(1304, Protocol::UDP), Service::Name("boomerang")),
        (ServiceQuery(2298, Protocol::TCP), Service::Name("d2k-datamover2")),
        (ServiceQuery(641, Protocol::TCP), Service::Name("repcmd")),
        (ServiceQuery(8199, Protocol::TCP), Service::Name("vvr-data")),
        (ServiceQuery(2435, Protocol::TCP), Service::Name("optilogic")),
        (ServiceQuery(3739, Protocol::UDP), Service::Name("launchbird-lm")),
        (ServiceQuery(1787, Protocol::UDP), Service::Name("funk-license")),
        (ServiceQuery(1127, Protocol::TCP), Service::Name("supfiledbg")),
        (ServiceQuery(7933, Protocol::UDP), Service::Name("t2-brm")),
        (ServiceQuery(2613, Protocol::TCP), Service::Name("smntubootstrap")),
        (ServiceQuery(1708, Protocol::TCP), Service::Name("gat-lmd")),
        (ServiceQuery(3665, Protocol::TCP), Service::Name("ent-engine")),
        (ServiceQuery(2253, Protocol::TCP), Service::Name("dtv-chan-req")),
        (ServiceQuery(4236, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3655, Protocol::TCP), Service::Name("abatemgr")),
        (ServiceQuery(4680, Protocol::TCP), Service::Name("mgemanagement")),
        (ServiceQuery(3002, Protocol::TCP), Service::Name("exlm-agent")),
        (ServiceQuery(2844, Protocol::UDP), Service::Name("bpcp-poll")),
        (ServiceQuery(1121, Protocol::TCP), Service::Name("rmpp")),
        (ServiceQuery(1074, Protocol::TCP), Service::Name("warmspotMgmt")),
        (ServiceQuery(5021, Protocol::UDP), Service::Name("zenginkyo-2")),
        (ServiceQuery(2491, Protocol::TCP), Service::Name("conclave-cpp")),
        (ServiceQuery(3163, Protocol::TCP), Service::Name("res-sap")),
        (ServiceQuery(1657, Protocol::TCP), Service::Name("fujitsu-mmpdc")),
        (ServiceQuery(1964, Protocol::UDP), Service::Name("solid-e-engine")),
        (ServiceQuery(4147, Protocol::TCP), Service::Name("vrxpservman")),
        (ServiceQuery(2524, Protocol::UDP), Service::Name("optiwave-lm")),
        (ServiceQuery(4054, Protocol::TCP), Service::Name("ccu-comm-2")),
        (ServiceQuery(5403, Protocol::UDP), Service::Name("hpoms-ci-lstn")),
        (ServiceQuery(13785, Protocol::UDP), Service::Name("nbdb")),
        (ServiceQuery(1986, Protocol::TCP), Service::Name("licensedaemon")),
        (ServiceQuery(3476, Protocol::TCP), Service::Name("nppmp")),
        (ServiceQuery(3696, Protocol::TCP), Service::Name("telnetcpcd")),
        (ServiceQuery(3351, Protocol::UDP), Service::Name("btrieve")),
        (ServiceQuery(3248, Protocol::UDP), Service::Name("procos-lm")),
        (ServiceQuery(8053, Protocol::UDP), Service::Name("senomix02")),
        (ServiceQuery(6064, Protocol::UDP), Service::Name("ndl-ahp-svc")),
        (ServiceQuery(3323, Protocol::TCP), Service::Name("active-net")),
        (ServiceQuery(3390, Protocol::UDP), Service::Name("dsc")),
        (ServiceQuery(1600, Protocol::UDP), Service::Name("issd")),
        (ServiceQuery(344, Protocol::UDP), Service::Name("pdap")),
        (ServiceQuery(1092, Protocol::UDP), Service::Name("obrpd")),
        (ServiceQuery(2387, Protocol::TCP), Service::Name("vsamredirector")),
        (ServiceQuery(1849, Protocol::UDP), Service::Name("alpha-sms")),
        (ServiceQuery(8614, Protocol::UDP), Service::Name("canon-bjnp4")),
        (ServiceQuery(9536, Protocol::TCP), Service::Name("laes-bf")),
        (ServiceQuery(1963, Protocol::UDP), Service::Name("webmachine")),
        (ServiceQuery(8444, Protocol::UDP), Service::Name("pcsync-http")),
        (ServiceQuery(753, Protocol::UDP), Service::Name("rrh")),
        (ServiceQuery(2439, Protocol::UDP), Service::Name("sybasedbsynch")),
        (ServiceQuery(4057, Protocol::UDP), Service::Name("wfm")),
        (ServiceQuery(3499, Protocol::UDP), Service::Name("sccip-media")),
        (ServiceQuery(1067, Protocol::TCP), Service::Name("instl_boots")),
        (ServiceQuery(16080, Protocol::TCP), Service::Name("osxwebadmin")),
        (ServiceQuery(16310, Protocol::TCP), Service::Name("pduncs")),
        (ServiceQuery(35003, Protocol::TCP), Service::Name("rt-devicemapper")),
        (ServiceQuery(1830, Protocol::TCP), Service::Name("net8-cman")),
        (ServiceQuery(4348, Protocol::TCP), Service::Name("itose")),
        (ServiceQuery(6065, Protocol::UDP), Service::Name("winpharaoh")),
        (ServiceQuery(3582, Protocol::UDP), Service::Name("press")),
        (ServiceQuery(18463, Protocol::UDP), Service::Name("ac-cluster")),
        (ServiceQuery(1031, Protocol::UDP), Service::Name("iad2")),
        (ServiceQuery(6789, Protocol::TCP), Service::Name("ibm-db2-admin")),
        (ServiceQuery(4308, Protocol::UDP), Service::Name("compx-lockview")),
        (ServiceQuery(2794, Protocol::TCP), Service::Name("urp")),
        (ServiceQuery(3426, Protocol::TCP), Service::Name("arkivio")),
        (ServiceQuery(9750, Protocol::UDP), Service::Name("board-voip")),
        (ServiceQuery(3298, Protocol::TCP), Service::Name("deskview")),
        (ServiceQuery(2248, Protocol::UDP), Service::Name("ums")),
        (ServiceQuery(4169, Protocol::TCP), Service::Name("iadt")),
        (ServiceQuery(2208, Protocol::UDP), Service::Name("hpiod")),
        (ServiceQuery(7166, Protocol::TCP), Service::Name("aruba-server")),
        (ServiceQuery(1308, Protocol::TCP), Service::Name("odsi")),
        (ServiceQuery(7570, Protocol::UDP), Service::Name("aries-kfinder")),
        (ServiceQuery(5419, Protocol::TCP), Service::Name("dj-ice")),
        (ServiceQuery(2427, Protocol::TCP), Service::Name("mgcp-gateway")),
        (ServiceQuery(1442, Protocol::UDP), Service::Name("cadis-2")),
        (ServiceQuery(1943, Protocol::TCP), Service::Name("beeyond-media")),
        (ServiceQuery(416, Protocol::UDP), Service::Name("silverplatter")),
        (ServiceQuery(33434, Protocol::UDP), Service::Name("traceroute")),
        (ServiceQuery(3593, Protocol::UDP), Service::Name("bpmd")),
        (ServiceQuery(3158, Protocol::TCP), Service::Name("stvp")),
        (ServiceQuery(3370, Protocol::TCP), Service::Name("satvid-datalnk")),
        (ServiceQuery(219, Protocol::TCP), Service::Name("uarps")),
        (ServiceQuery(8889, Protocol::UDP), Service::Name("ddi-udp-2")),
        (ServiceQuery(2234, Protocol::UDP), Service::Name("directplay")),
        (ServiceQuery(6581, Protocol::TCP), Service::Name("parsec-peer")),
        (ServiceQuery(124, Protocol::TCP), Service::Name("ansatrader")),
        (ServiceQuery(9283, Protocol::TCP), Service::Name("callwaveiam")),
        (ServiceQuery(1114, Protocol::TCP), Service::Name("mini-sql")),
        (ServiceQuery(2957, Protocol::TCP), Service::Name("jmact5")),
        (ServiceQuery(2829, Protocol::UDP), Service::Name("silkp1")),
        (ServiceQuery(1605, Protocol::UDP), Service::Name("slp")),
        (ServiceQuery(741, Protocol::UDP), Service::Name("netgw")),
        (ServiceQuery(6936, Protocol::TCP), Service::Name("xsmsvc")),
        (ServiceQuery(1710, Protocol::UDP), Service::Name("impera")),
        (ServiceQuery(4884, Protocol::UDP), Service::Name("hivestor")),
        (ServiceQuery(7262, Protocol::UDP), Service::Name("cnap")),
        (ServiceQuery(3455, Protocol::TCP), Service::Name("prsvp")),
        (ServiceQuery(4253, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2093, Protocol::TCP), Service::Name("nbx-cc")),
        (ServiceQuery(10083, Protocol::TCP), Service::Name("amidxtape")),
        (ServiceQuery(2635, Protocol::UDP), Service::Name("backburner")),
        (ServiceQuery(2750, Protocol::TCP), Service::Name("fjippol-port1")),
        (ServiceQuery(3201, Protocol::UDP), Service::Name("cpq-tasksmart")),
        (ServiceQuery(763, Protocol::UDP), Service::Name("cycleserv")),
        (ServiceQuery(1198, Protocol::TCP), Service::Name("cajo-discovery")),
        (ServiceQuery(1163, Protocol::UDP), Service::Name("sddp")),
        (ServiceQuery(3040, Protocol::UDP), Service::Name("tomato-springs")),
        (ServiceQuery(9255, Protocol::UDP), Service::Name("mon")),
        (ServiceQuery(2098, Protocol::UDP), Service::Name("dialog-port")),
        (ServiceQuery(3952, Protocol::UDP), Service::Name("i3-sessionmgr")),
        (ServiceQuery(4081, Protocol::UDP), Service::Name("lorica-in-sec")),
        (ServiceQuery(3056, Protocol::TCP), Service::Name("cdl-server")),
        (ServiceQuery(2641, Protocol::TCP), Service::Name("hdl-srv")),
        (ServiceQuery(2274, Protocol::TCP), Service::Name("pcttunnell")),
        (ServiceQuery(5455, Protocol::TCP), Service::Name("apc-5455")),
        (ServiceQuery(23004, Protocol::UDP), Service::Name("inovaport5")),
        (ServiceQuery(2004, Protocol::TCP), Service::Name("mailbox")),
        (ServiceQuery(3343, Protocol::TCP), Service::Name("ms-cluster-net")),
        (ServiceQuery(5434, Protocol::TCP), Service::Name("sgi-arrayd")),
        (ServiceQuery(2887, Protocol::TCP), Service::Name("aironetddp")),
        (ServiceQuery(3214, Protocol::TCP), Service::Name("jmq-daemon-1")),
        (ServiceQuery(1335, Protocol::TCP), Service::Name("digital-notary")),
        (ServiceQuery(1386, Protocol::TCP), Service::Name("checksum")),
        (ServiceQuery(7500, Protocol::TCP), Service::Name("silhouette")),
        (ServiceQuery(5315, Protocol::TCP), Service::Name("hacl-poll")),
        (ServiceQuery(5632, Protocol::TCP), Service::Name("pcanywherestat")),
        (ServiceQuery(4025, Protocol::TCP), Service::Name("partimage")),
        (ServiceQuery(2486, Protocol::UDP), Service::Name("netobjects2")),
        (ServiceQuery(1494, Protocol::UDP), Service::Name("citrix-ica")),
        (ServiceQuery(1033, Protocol::UDP), Service::Name("netinfo-local")),
        (ServiceQuery(7007, Protocol::UDP), Service::Name("afs3-bos")),
        (ServiceQuery(10260, Protocol::UDP), Service::Name("axis-wimp-port")),
        (ServiceQuery(860, Protocol::TCP), Service::Name("iscsi")),
        (ServiceQuery(2325, Protocol::UDP), Service::Name("ansysli")),
        (ServiceQuery(5913, Protocol::TCP), Service::Name("ads-c")),
        (ServiceQuery(12012, Protocol::UDP), Service::Name("vipera")),
        (ServiceQuery(18187, Protocol::TCP), Service::Name("opsec-ela")),
        (ServiceQuery(2462, Protocol::UDP), Service::Name("qadmifevent")),
        (ServiceQuery(3340, Protocol::TCP), Service::Name("anet-m")),
        (ServiceQuery(1303, Protocol::TCP), Service::Name("sftsrv")),
        (ServiceQuery(35, Protocol::UDP), Service::Name("priv-print")),
        (ServiceQuery(604, Protocol::TCP), Service::Name("tunnel")),
        (ServiceQuery(1417, Protocol::UDP), Service::Name("timbuktu-srv1")),
        (ServiceQuery(2787, Protocol::TCP), Service::Name("piccolo")),
        (ServiceQuery(11751, Protocol::TCP), Service::Name("intrepid-ssl")),
        (ServiceQuery(1288, Protocol::TCP), Service::Name("navbuddy")),
        (ServiceQuery(1184, Protocol::TCP), Service::Name("llsurfup-https")),
        (ServiceQuery(396, Protocol::UDP), Service::Name("netware-ip")),
        (ServiceQuery(588, Protocol::TCP), Service::Name("cal")),
        (ServiceQuery(6817, Protocol::TCP), Service::Name("pentbox-sim")),
        (ServiceQuery(2996, Protocol::UDP), Service::Name("vsixml")),
        (ServiceQuery(4400, Protocol::UDP), Service::Name("ds-srv")),
        (ServiceQuery(4185, Protocol::UDP), Service::Name("wcpp")),
        (ServiceQuery(871, Protocol::TCP), Service::Name("supfilesrv")),
        (ServiceQuery(2672, Protocol::UDP), Service::Name("nhserver")),
        (ServiceQuery(4219, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(7799, Protocol::UDP), Service::Name("altbsdp")),
        (ServiceQuery(8301, Protocol::TCP), Service::Name("amberon")),
        (ServiceQuery(18242, Protocol::TCP), Service::Name("iclid")),
        (ServiceQuery(505, Protocol::UDP), Service::Name("mailbox-lm")),
        (ServiceQuery(1654, Protocol::UDP), Service::Name("stargatealerts")),
        (ServiceQuery(5585, Protocol::UDP), Service::Name("bis-sync")),
        (ServiceQuery(4223, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(8786, Protocol::UDP), Service::Name("msgclnt")),
        (ServiceQuery(767, Protocol::UDP), Service::Name("phonebook")),
        (ServiceQuery(3603, Protocol::UDP), Service::Name("int-rcv-cntrl")),
        (ServiceQuery(5681, Protocol::TCP), Service::Name("ncxcp")),
        (ServiceQuery(7787, Protocol::TCP), Service::Name("popup-reminders")),
        (ServiceQuery(4176, Protocol::TCP), Service::Name("tl-ipcproxy")),
        (ServiceQuery(3470, Protocol::UDP), Service::Name("jt400")),
        (ServiceQuery(2821, Protocol::TCP), Service::Name("vrts-at-port")),
        (ServiceQuery(2212, Protocol::UDP), Service::Name("leecoposserver")),
        (ServiceQuery(27876, Protocol::TCP), Service::Name("astrolink")),
        (ServiceQuery(6001, Protocol::TCP), Service::Name("X11:1")),
        (ServiceQuery(1482, Protocol::UDP), Service::Name("miteksys-lm")),
        (ServiceQuery(38462, Protocol::TCP), Service::Name("e1-interface")),
        (ServiceQuery(3326, Protocol::TCP), Service::Name("sftu")),
        (ServiceQuery(5301, Protocol::TCP), Service::Name("hacl-gs")),
        (ServiceQuery(1405, Protocol::TCP), Service::Name("ibm-res")),
        (ServiceQuery(1271, Protocol::UDP), Service::Name("excw")),
        (ServiceQuery(6002, Protocol::TCP), Service::Name("X11:2")),
        (ServiceQuery(1087, Protocol::TCP), Service::Name("cplscrambler-in")),
        (ServiceQuery(4913, Protocol::TCP), Service::Name("lutcp")),
        (ServiceQuery(3325, Protocol::UDP), Service::Name("active-net")),
        (ServiceQuery(2535, Protocol::UDP), Service::Name("madcap")),
        (ServiceQuery(913, Protocol::TCP), Service::Name("apex-edge")),
        (ServiceQuery(1734, Protocol::TCP), Service::Name("cambertx-lm")),
        (ServiceQuery(1689, Protocol::UDP), Service::Name("firefox")),
        (ServiceQuery(1307, Protocol::UDP), Service::Name("pacmand")),
        (ServiceQuery(6566, Protocol::UDP), Service::Name("sane-port")),
        (ServiceQuery(548, Protocol::UDP), Service::Name("afp")),
        (ServiceQuery(3563, Protocol::TCP), Service::Name("watcomdebug")),
        (ServiceQuery(2736, Protocol::TCP), Service::Name("radwiz-nms-srv")),
        (ServiceQuery(1919, Protocol::UDP), Service::Name("can-dch")),
        (ServiceQuery(3504, Protocol::UDP), Service::Name("ironstorm")),
        (ServiceQuery(3957, Protocol::UDP), Service::Name("mqe-broker")),
        (ServiceQuery(7352, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(408, Protocol::TCP), Service::Name("prm-sm")),
        (ServiceQuery(4054, Protocol::UDP), Service::Name("ccu-comm-2")),
        (ServiceQuery(2145, Protocol::UDP), Service::Name("lv-pici")),
        (ServiceQuery(1896, Protocol::UDP), Service::Name("b-novative-ls")),
        (ServiceQuery(3811, Protocol::UDP), Service::Name("amp")),
        (ServiceQuery(466, Protocol::TCP), Service::Name("digital-vrc")),
        (ServiceQuery(6483, Protocol::TCP), Service::Name("SunVTS-RMI")),
        (ServiceQuery(4354, Protocol::UDP), Service::Name("qsnet-trans")),
        (ServiceQuery(2500, Protocol::UDP), Service::Name("rtsserv")),
        (ServiceQuery(32400, Protocol::TCP), Service::Name("plex")),
        (ServiceQuery(903, Protocol::UDP), Service::Name("ideafarm-panic")),
        (ServiceQuery(7334, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(136, Protocol::UDP), Service::Name("profile")),
        (ServiceQuery(4547, Protocol::UDP), Service::Name("lanner-lm")),
        (ServiceQuery(7801, Protocol::TCP), Service::Name("ssp-client")),
        (ServiceQuery(763, Protocol::TCP), Service::Name("cycleserv")),
        (ServiceQuery(4044, Protocol::UDP), Service::Name("ltp")),
        (ServiceQuery(4197, Protocol::TCP), Service::Name("hctl")),
        (ServiceQuery(1665, Protocol::TCP), Service::Name("netview-aix-5")),
        (ServiceQuery(1815, Protocol::UDP), Service::Name("mmpft")),
        (ServiceQuery(3543, Protocol::UDP), Service::Name("qftest-lookup")),
        (ServiceQuery(1549, Protocol::TCP), Service::Name("shivahose")),
        (ServiceQuery(576, Protocol::TCP), Service::Name("ipcd")),
        (ServiceQuery(4570, Protocol::TCP), Service::Name("deploymentmap")),
        (ServiceQuery(43, Protocol::UDP), Service::Name("whois")),
        (ServiceQuery(696, Protocol::UDP), Service::Name("rushd")),
        (ServiceQuery(3873, Protocol::TCP), Service::Name("fagordnc")),
        (ServiceQuery(2019, Protocol::TCP), Service::Name("whosockami")),
        (ServiceQuery(32771, Protocol::TCP), Service::Name("sometimes-rpc5")),
        (ServiceQuery(15000, Protocol::UDP), Service::Name("hydap")),
        (ServiceQuery(994, Protocol::UDP), Service::Name("ircs")),
        (ServiceQuery(559, Protocol::TCP), Service::Name("teedtap")),
        (ServiceQuery(19790, Protocol::TCP), Service::Name("faircom-db")),
        (ServiceQuery(1877, Protocol::UDP), Service::Name("hp-webqosdb")),
        (ServiceQuery(2226, Protocol::UDP), Service::Name("di-drm")),
        (ServiceQuery(9990, Protocol::UDP), Service::Name("osm-appsrvr")),
        (ServiceQuery(9951, Protocol::TCP), Service::Name("apc-9951")),
        (ServiceQuery(3734, Protocol::UDP), Service::Name("synel-data")),
        (ServiceQuery(1157, Protocol::UDP), Service::Name("iascontrol")),
        (ServiceQuery(1093, Protocol::TCP), Service::Name("proofd")),
        (ServiceQuery(4006, Protocol::UDP), Service::Name("pxc-spvr")),
        (ServiceQuery(1676, Protocol::TCP), Service::Name("netcomm1")),
        (ServiceQuery(9209, Protocol::UDP), Service::Name("almobile-system")),
        (ServiceQuery(3658, Protocol::TCP), Service::Name("ps-ams")),
        (ServiceQuery(2372, Protocol::UDP), Service::Name("lanmessenger")),
        (ServiceQuery(8002, Protocol::UDP), Service::Name("teradataordbms")),
        (ServiceQuery(2331, Protocol::TCP), Service::Name("agentview")),
        (ServiceQuery(2295, Protocol::UDP), Service::Name("advant-lm")),
        (ServiceQuery(3380, Protocol::TCP), Service::Name("sns-channels")),
        (ServiceQuery(5111, Protocol::UDP), Service::Name("taep-as-svc")),
        (ServiceQuery(2566, Protocol::UDP), Service::Name("pcs-pcw")),
        (ServiceQuery(2861, Protocol::UDP), Service::Name("dialpad-voice2")),
        (ServiceQuery(3457, Protocol::TCP), Service::Name("vat-control")),
        (ServiceQuery(2153, Protocol::UDP), Service::Name("ctlptc")),
        (ServiceQuery(4278, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1569, Protocol::TCP), Service::Name("ets")),
        (ServiceQuery(542, Protocol::UDP), Service::Name("commerce")),
        (ServiceQuery(13820, Protocol::TCP), Service::Name("dsmcc-passthru")),
        (ServiceQuery(2209, Protocol::UDP), Service::Name("rimf-ps")),
        (ServiceQuery(45678, Protocol::TCP), Service::Name("eba")),
        (ServiceQuery(558, Protocol::TCP), Service::Name("sdnskmp")),
        (ServiceQuery(2866, Protocol::TCP), Service::Name("iwlistener")),
        (ServiceQuery(455, Protocol::TCP), Service::Name("creativepartnr")),
        (ServiceQuery(1495, Protocol::TCP), Service::Name("cvc")),
        (ServiceQuery(7272, Protocol::UDP), Service::Name("watchme-7272")),
        (ServiceQuery(6117, Protocol::TCP), Service::Name("daylitetouch")),
        (ServiceQuery(36001, Protocol::TCP), Service::Name("allpeers")),
        (ServiceQuery(4598, Protocol::TCP), Service::Name("a16-an-an")),
        (ServiceQuery(3708, Protocol::UDP), Service::Name("sun-as-iiops")),
        (ServiceQuery(2969, Protocol::UDP), Service::Name("essp")),
        (ServiceQuery(9084, Protocol::TCP), Service::Name("aurora")),
        (ServiceQuery(3166, Protocol::UDP), Service::Name("sossecollector")),
        (ServiceQuery(4158, Protocol::TCP), Service::Name("stat-cc")),
        (ServiceQuery(3019, Protocol::TCP), Service::Name("resource_mgr")),
        (ServiceQuery(3709, Protocol::TCP), Service::Name("ca-idms")),
        (ServiceQuery(389, Protocol::TCP), Service::Name("ldap")),
        (ServiceQuery(2286, Protocol::UDP), Service::Name("nas-metering")),
        (ServiceQuery(1354, Protocol::TCP), Service::Name("rightbrain")),
        (ServiceQuery(4140, Protocol::TCP), Service::Name("cedros_fds")),
        (ServiceQuery(5003, Protocol::TCP), Service::Name("filemaker")),
        (ServiceQuery(3088, Protocol::UDP), Service::Name("xdtp")),
        (ServiceQuery(6071, Protocol::UDP), Service::Name("ssdtp")),
        (ServiceQuery(4237, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2398, Protocol::UDP), Service::Name("orbiter")),
        (ServiceQuery(1754, Protocol::UDP), Service::Name("oracle-em2")),
        (ServiceQuery(4558, Protocol::UDP), Service::Name("mtcevrunqman")),
        (ServiceQuery(5307, Protocol::UDP), Service::Name("sco-aip")),
        (ServiceQuery(9450, Protocol::TCP), Service::Name("sntlkeyssrvr")),
        (ServiceQuery(1408, Protocol::UDP), Service::Name("sophia-lm")),
        (ServiceQuery(2912, Protocol::UDP), Service::Name("epicon")),
        (ServiceQuery(3380, Protocol::UDP), Service::Name("sns-channels")),
        (ServiceQuery(3630, Protocol::UDP), Service::Name("cs-remote-db")),
        (ServiceQuery(4432, Protocol::TCP), Service::Name("l-acoustics")),
        (ServiceQuery(1493, Protocol::UDP), Service::Name("netmap_lm")),
        (ServiceQuery(2692, Protocol::UDP), Service::Name("admins-lms")),
        (ServiceQuery(1233, Protocol::TCP), Service::Name("univ-appserver")),
        (ServiceQuery(2859, Protocol::UDP), Service::Name("activememory")),
        (ServiceQuery(1741, Protocol::UDP), Service::Name("cisco-net-mgmt")),
        (ServiceQuery(6622, Protocol::UDP), Service::Name("mcftp")),
        (ServiceQuery(2075, Protocol::TCP), Service::Name("newlixengine")),
        (ServiceQuery(5719, Protocol::UDP), Service::Name("dpm-agent")),
        (ServiceQuery(1539, Protocol::UDP), Service::Name("intellistor-lm")),
        (ServiceQuery(1623, Protocol::TCP), Service::Name("jaleosnd")),
        (ServiceQuery(2084, Protocol::UDP), Service::Name("sunclustergeo")),
        (ServiceQuery(2921, Protocol::TCP), Service::Name("cesdcdman")),
        (ServiceQuery(3186, Protocol::TCP), Service::Name("iiw-port")),
        (ServiceQuery(3581, Protocol::TCP), Service::Name("kfxaclicensing")),
        (ServiceQuery(3292, Protocol::TCP), Service::Name("meetingmaker")),
        (ServiceQuery(2930, Protocol::TCP), Service::Name("amx-weblinx")),
        (ServiceQuery(8294, Protocol::TCP), Service::Name("blp4")),
        (ServiceQuery(1887, Protocol::UDP), Service::Name("filex-lport")),
        (ServiceQuery(3068, Protocol::UDP), Service::Name("ls3bcast")),
        (ServiceQuery(1976, Protocol::UDP), Service::Name("tcoregagent")),
        (ServiceQuery(2816, Protocol::UDP), Service::Name("lbc-watchdog")),
        (ServiceQuery(7649, Protocol::UDP), Service::Name("cucme-2")),
        (ServiceQuery(3028, Protocol::UDP), Service::Name("LiebDevMgmt_DM")),
        (ServiceQuery(9163, Protocol::UDP), Service::Name("apani4")),
        (ServiceQuery(1248, Protocol::UDP), Service::Name("hermes")),
        (ServiceQuery(4193, Protocol::TCP), Service::Name("pvxplusio")),
        (ServiceQuery(1908, Protocol::UDP), Service::Name("dawn")),
        (ServiceQuery(3646, Protocol::TCP), Service::Name("xss-srv-port")),
        (ServiceQuery(3160, Protocol::TCP), Service::Name("tip-app-server")),
        (ServiceQuery(24242, Protocol::UDP), Service::Name("filesphere")),
        (ServiceQuery(14002, Protocol::TCP), Service::Name("scotty-disc")),
        (ServiceQuery(2951, Protocol::UDP), Service::Name("ottp")),
        (ServiceQuery(3204, Protocol::UDP), Service::Name("netwatcher-db")),
        (ServiceQuery(3714, Protocol::UDP), Service::Name("delos-dms")),
        (ServiceQuery(1184, Protocol::UDP), Service::Name("llsurfup-https")),
        (ServiceQuery(3134, Protocol::UDP), Service::Name("ecp")),
        (ServiceQuery(31948, Protocol::UDP), Service::Name("iceedcp_tx")),
        (ServiceQuery(4096, Protocol::UDP), Service::Name("bre")),
        (ServiceQuery(3615, Protocol::UDP), Service::Name("start-network")),
        (ServiceQuery(3038, Protocol::UDP), Service::Name("santak-ups")),
        (ServiceQuery(2002, Protocol::UDP), Service::Name("globe")),
        (ServiceQuery(512, Protocol::TCP), Service::Name("exec")),
        (ServiceQuery(3712, Protocol::TCP), Service::Name("sentinel-ent")),
        (ServiceQuery(4301, Protocol::UDP), Service::Name("d-data")),
        (ServiceQuery(3358, Protocol::TCP), Service::Name("mpsysrmsvr")),
        (ServiceQuery(2999, Protocol::UDP), Service::Name("remoteware-un")),
        (ServiceQuery(363, Protocol::UDP), Service::Name("rsvp_tunnel")),
        (ServiceQuery(5682, Protocol::UDP), Service::Name("brightcore")),
        (ServiceQuery(2417, Protocol::UDP), Service::Name("composit-server")),
        (ServiceQuery(3112, Protocol::UDP), Service::Name("ksysguard")),
        (ServiceQuery(8027, Protocol::TCP), Service::Name("papachi-p2p-srv")),
        (ServiceQuery(2538, Protocol::TCP), Service::Name("vnwk-prapi")),
        (ServiceQuery(3695, Protocol::TCP), Service::Name("bmc-data-coll")),
        (ServiceQuery(5025, Protocol::UDP), Service::Name("scpi-raw")),
        (ServiceQuery(642, Protocol::TCP), Service::Name("esro-emsdp")),
        (ServiceQuery(1956, Protocol::TCP), Service::Name("vrtl-vmf-ds")),
        (ServiceQuery(1397, Protocol::TCP), Service::Name("audio-activmail")),
        (ServiceQuery(1781, Protocol::UDP), Service::Name("answersoft-lm")),
        (ServiceQuery(911, Protocol::UDP), Service::Name("xact-backup")),
        (ServiceQuery(4414, Protocol::TCP), Service::Name("updog")),
        (ServiceQuery(3590, Protocol::TCP), Service::Name("wv-csp-sms")),
        (ServiceQuery(29, Protocol::UDP), Service::Name("msg-icp")),
        (ServiceQuery(2316, Protocol::UDP), Service::Name("sent-lm")),
        (ServiceQuery(11876, Protocol::UDP), Service::Name("xoraya")),
        (ServiceQuery(8471, Protocol::TCP), Service::Name("pim-port")),
        (ServiceQuery(6621, Protocol::UDP), Service::Name("kftp")),
        (ServiceQuery(8900, Protocol::TCP), Service::Name("jmb-cds1")),
        (ServiceQuery(4452, Protocol::UDP), Service::Name("ctiprogramload")),
        (ServiceQuery(569, Protocol::UDP), Service::Name("ms-rome")),
        (ServiceQuery(2444, Protocol::TCP), Service::Name("btpp2sectrans")),
        (ServiceQuery(8400, Protocol::UDP), Service::Name("cvd")),
        (ServiceQuery(5453, Protocol::TCP), Service::Name("surebox")),
        (ServiceQuery(2759, Protocol::TCP), Service::Name("apollo-gms")),
        (ServiceQuery(260, Protocol::UDP), Service::Name("openport")),
        (ServiceQuery(3277, Protocol::TCP), Service::Name("awg-proxy")),
        (ServiceQuery(1908, Protocol::TCP), Service::Name("dawn")),
        (ServiceQuery(2118, Protocol::TCP), Service::Name("mentaserver")),
        (ServiceQuery(1976, Protocol::TCP), Service::Name("tcoregagent")),
        (ServiceQuery(1517, Protocol::UDP), Service::Name("vpac")),
        (ServiceQuery(1583, Protocol::UDP), Service::Name("simbaexpress")),
        (ServiceQuery(6785, Protocol::UDP), Service::Name("dgpf-exchg")),
        (ServiceQuery(8448, Protocol::TCP), Service::Name("matrix-fed")),
        (ServiceQuery(5151, Protocol::TCP), Service::Name("esri_sde")),
        (ServiceQuery(6400, Protocol::UDP), Service::Name("boe-cms")),
        (ServiceQuery(3960, Protocol::TCP), Service::Name("bess")),
        (ServiceQuery(1249, Protocol::TCP), Service::Name("mesavistaco")),
        (ServiceQuery(157, Protocol::UDP), Service::Name("knet-cmp")),
        (ServiceQuery(912, Protocol::UDP), Service::Name("apex-mesh")),
        (ServiceQuery(2784, Protocol::TCP), Service::Name("www-dev")),
        (ServiceQuery(801, Protocol::TCP), Service::Name("device")),
        (ServiceQuery(9024, Protocol::UDP), Service::Name("swa-2")),
        (ServiceQuery(5720, Protocol::UDP), Service::Name("ms-licensing")),
        (ServiceQuery(4152, Protocol::UDP), Service::Name("idig_mux")),
        (ServiceQuery(4275, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2010, Protocol::TCP), Service::Name("search")),
        (ServiceQuery(5568, Protocol::TCP), Service::Name("sdt")),
        (ServiceQuery(2541, Protocol::UDP), Service::Name("lonworks2")),
        (ServiceQuery(5601, Protocol::TCP), Service::Name("esmagent")),
        (ServiceQuery(1838, Protocol::UDP), Service::Name("talnet")),
        (ServiceQuery(3596, Protocol::TCP), Service::Name("iw-mmogame")),
        (ServiceQuery(5236, Protocol::TCP), Service::Name("padl2sim")),
        (ServiceQuery(8953, Protocol::TCP), Service::Name("ub-dns-control")),
        (ServiceQuery(1430, Protocol::TCP), Service::Name("tpdu")),
        (ServiceQuery(13783, Protocol::UDP), Service::Name("vopied")),
        (ServiceQuery(3891, Protocol::UDP), Service::Name("rtc-pm-port")),
        (ServiceQuery(1170, Protocol::UDP), Service::Name("atc-lm")),
        (ServiceQuery(3972, Protocol::UDP), Service::Name("iconp")),
        (ServiceQuery(1856, Protocol::UDP), Service::Name("fiorano-msgsvc")),
        (ServiceQuery(2743, Protocol::UDP), Service::Name("murx")),
        (ServiceQuery(8059, Protocol::UDP), Service::Name("senomix08")),
        (ServiceQuery(2060, Protocol::TCP), Service::Name("teleniumdaemon")),
        (ServiceQuery(1032, Protocol::UDP), Service::Name("iad3")),
        (ServiceQuery(1547, Protocol::UDP), Service::Name("laplink")),
        (ServiceQuery(2782, Protocol::TCP), Service::Name("everydayrc")),
        (ServiceQuery(12345, Protocol::TCP), Service::Name("netbus")),
        (ServiceQuery(5461, Protocol::UDP), Service::Name("silkmeter")),
        (ServiceQuery(4203, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3497, Protocol::TCP), Service::Name("ipether232port")),
        (ServiceQuery(1794, Protocol::TCP), Service::Name("cera-bcm")),
        (ServiceQuery(2854, Protocol::UDP), Service::Name("infomover")),
        (ServiceQuery(5059, Protocol::TCP), Service::Name("sds")),
        (ServiceQuery(28589, Protocol::TCP), Service::Name("bosswave")),
        (ServiceQuery(2304, Protocol::UDP), Service::Name("attachmate-uts")),
        (ServiceQuery(2873, Protocol::TCP), Service::Name("psrt")),
        (ServiceQuery(2696, Protocol::TCP), Service::Name("unifyadmin")),
        (ServiceQuery(1802, Protocol::TCP), Service::Name("concomp1")),
        (ServiceQuery(6200, Protocol::UDP), Service::Name("lm-x")),
        (ServiceQuery(2438, Protocol::TCP), Service::Name("msp")),
        (ServiceQuery(1775, Protocol::TCP), Service::Name("vdab")),
        (ServiceQuery(2445, Protocol::TCP), Service::Name("dtn1")),
        (ServiceQuery(4195, Protocol::TCP), Service::Name("aws-wsp")),
        (ServiceQuery(4346, Protocol::TCP), Service::Name("elanlm")),
        (ServiceQuery(2471, Protocol::TCP), Service::Name("seaodbc")),
        (ServiceQuery(5783, Protocol::TCP), Service::Name("3par-mgmt-ssl")),
        (ServiceQuery(11208, Protocol::UDP), Service::Name("wifree")),
        (ServiceQuery(1347, Protocol::UDP), Service::Name("bbn-mmc")),
        (ServiceQuery(3254, Protocol::UDP), Service::Name("pda-sys")),
        (ServiceQuery(1296, Protocol::UDP), Service::Name("dproxy")),
        (ServiceQuery(3973, Protocol::TCP), Service::Name("progistics")),
        (ServiceQuery(3105, Protocol::TCP), Service::Name("cardbox")),
        (ServiceQuery(3421, Protocol::TCP), Service::Name("bmap")),
        (ServiceQuery(5716, Protocol::UDP), Service::Name("prosharerequest")),
        (ServiceQuery(3558, Protocol::TCP), Service::Name("mcp-port")),
        (ServiceQuery(3736, Protocol::TCP), Service::Name("rs-rmi")),
        (ServiceQuery(1988, Protocol::UDP), Service::Name("tr-rsrb-p2")),
        (ServiceQuery(2893, Protocol::UDP), Service::Name("vseconnector")),
        (ServiceQuery(2480, Protocol::TCP), Service::Name("powerexchange")),
        (ServiceQuery(3212, Protocol::TCP), Service::Name("surveyinst")),
        (ServiceQuery(35002, Protocol::TCP), Service::Name("rt-sound")),
        (ServiceQuery(7431, Protocol::UDP), Service::Name("pmd")),
        (ServiceQuery(1074, Protocol::UDP), Service::Name("warmspotMgmt")),
        (ServiceQuery(4952, Protocol::UDP), Service::Name("sagxtsds")),
        (ServiceQuery(12000, Protocol::UDP), Service::Name("entextxid")),
        (ServiceQuery(3263, Protocol::TCP), Service::Name("ecolor-imager")),
        (ServiceQuery(3361, Protocol::UDP), Service::Name("kv-agent")),
        (ServiceQuery(34963, Protocol::UDP), Service::Name("profinet-rtm")),
        (ServiceQuery(5355, Protocol::TCP), Service::Name("llmnr")),
        (ServiceQuery(627, Protocol::TCP), Service::Name("passgo-tivoli")),
        (ServiceQuery(1086, Protocol::UDP), Service::Name("cplscrambler-lg")),
        (ServiceQuery(7024, Protocol::UDP), Service::Name("vmsvc")),
        (ServiceQuery(2470, Protocol::TCP), Service::Name("taskman-port")),
        (ServiceQuery(2360, Protocol::TCP), Service::Name("nexstorindltd")),
        (ServiceQuery(4599, Protocol::TCP), Service::Name("a17-an-an")),
        (ServiceQuery(24676, Protocol::UDP), Service::Name("canditv")),
        (ServiceQuery(1063, Protocol::TCP), Service::Name("kyoceranetdev")),
        (ServiceQuery(1345, Protocol::TCP), Service::Name("vpjp")),
        (ServiceQuery(3747, Protocol::TCP), Service::Name("linktest-s")),
        (ServiceQuery(176, Protocol::UDP), Service::Name("genrad-mux")),
        (ServiceQuery(29167, Protocol::UDP), Service::Name("otmp")),
        (ServiceQuery(3276, Protocol::UDP), Service::Name("maxim-asics")),
        (ServiceQuery(18182, Protocol::UDP), Service::Name("opsec-ufp")),
        (ServiceQuery(9990, Protocol::TCP), Service::Name("osm-appsrvr")),
        (ServiceQuery(2838, Protocol::TCP), Service::Name("starbot")),
        (ServiceQuery(1701, Protocol::UDP), Service::Name("L2TP")),
        (ServiceQuery(3717, Protocol::TCP), Service::Name("wv-csp-udp-cir")),
        (ServiceQuery(38203, Protocol::TCP), Service::Name("agpolicy")),
        (ServiceQuery(682, Protocol::UDP), Service::Name("xfr")),
        (ServiceQuery(15345, Protocol::UDP), Service::Name("xpilot")),
        (ServiceQuery(1325, Protocol::UDP), Service::Name("dx-instrument")),
        (ServiceQuery(4358, Protocol::TCP), Service::Name("qsnet-nucl")),
        (ServiceQuery(6961, Protocol::TCP), Service::Name("jmact3")),
        (ServiceQuery(2292, Protocol::UDP), Service::Name("mib-streaming")),
        (ServiceQuery(3768, Protocol::UDP), Service::Name("rblcheckd")),
        (ServiceQuery(80, Protocol::UDP), Service::Name("http")),
        (ServiceQuery(3174, Protocol::TCP), Service::Name("armi-server")),
        (ServiceQuery(847, Protocol::UDP), Service::Name("dhcp-failover2")),
        (ServiceQuery(1615, Protocol::UDP), Service::Name("netbill-auth")),
        (ServiceQuery(1495, Protocol::UDP), Service::Name("cvc")),
        (ServiceQuery(5353, Protocol::UDP), Service::Name("zeroconf")),
        (ServiceQuery(2463, Protocol::UDP), Service::Name("lsi-raid-mgmt")),
        (ServiceQuery(1382, Protocol::UDP), Service::Name("udt_os")),
        (ServiceQuery(2659, Protocol::TCP), Service::Name("sns-query")),
        (ServiceQuery(2769, Protocol::UDP), Service::Name("exce")),
        (ServiceQuery(2609, Protocol::TCP), Service::Name("system-monitor")),
        (ServiceQuery(45825, Protocol::UDP), Service::Name("qdb2service")),
        (ServiceQuery(4214, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(351, Protocol::UDP), Service::Name("matip-type-b")),
        (ServiceQuery(631, Protocol::UDP), Service::Name("ipp")),
        (ServiceQuery(3430, Protocol::UDP), Service::Name("ssdispatch")),
        (ServiceQuery(5770, Protocol::TCP), Service::Name("spramsd")),
        (ServiceQuery(1043, Protocol::UDP), Service::Name("boinc")),
        (ServiceQuery(1737, Protocol::TCP), Service::Name("ultimad")),
        (ServiceQuery(1117, Protocol::TCP), Service::Name("ardus-mtrns")),
        (ServiceQuery(1997, Protocol::UDP), Service::Name("gdp-port")),
        (ServiceQuery(9023, Protocol::TCP), Service::Name("swa-1")),
        (ServiceQuery(5672, Protocol::TCP), Service::Name("amqp")),
        (ServiceQuery(4485, Protocol::TCP), Service::Name("assyst-dr")),
        (ServiceQuery(4356, Protocol::UDP), Service::Name("qsnet-assist")),
        (ServiceQuery(1361, Protocol::TCP), Service::Name("linx")),
        (ServiceQuery(2266, Protocol::UDP), Service::Name("mfserver")),
        (ServiceQuery(5167, Protocol::UDP), Service::Name("scte104")),
        (ServiceQuery(2493, Protocol::UDP), Service::Name("talarian-mqs")),
        (ServiceQuery(7008, Protocol::UDP), Service::Name("afs3-update")),
        (ServiceQuery(2133, Protocol::TCP), Service::Name("zymed-zpp")),
        (ServiceQuery(2594, Protocol::UDP), Service::Name("dts")),
        (ServiceQuery(655, Protocol::UDP), Service::Name("tinc")),
        (ServiceQuery(2581, Protocol::TCP), Service::Name("argis-te")),
        (ServiceQuery(6771, Protocol::TCP), Service::Name("plysrv-https")),
        (ServiceQuery(3724, Protocol::TCP), Service::Name("blizwow")),
        (ServiceQuery(41795, Protocol::UDP), Service::Name("crestron-ctp")),
        (ServiceQuery(2422, Protocol::TCP), Service::Name("crmsbits")),
        (ServiceQuery(18241, Protocol::TCP), Service::Name("checkpoint-rtm")),
        (ServiceQuery(27504, Protocol::TCP), Service::Name("kopek-httphead")),
        (ServiceQuery(63, Protocol::TCP), Service::Name("via-ftp")),
        (ServiceQuery(1876, Protocol::TCP), Service::Name("ewcappsrv")),
        (ServiceQuery(1819, Protocol::TCP), Service::Name("plato-lm")),
        (ServiceQuery(3902, Protocol::UDP), Service::Name("nimaux")),
        (ServiceQuery(5052, Protocol::UDP), Service::Name("ita-manager")),
        (ServiceQuery(1677, Protocol::TCP), Service::Name("groupwise")),
        (ServiceQuery(3538, Protocol::TCP), Service::Name("ibm-diradm")),
        (ServiceQuery(3235, Protocol::UDP), Service::Name("mdap-port")),
        (ServiceQuery(1339, Protocol::UDP), Service::Name("kjtsiteserver")),
        (ServiceQuery(1078, Protocol::UDP), Service::Name("avocent-proxy")),
        (ServiceQuery(114, Protocol::UDP), Service::Name("audionews")),
        (ServiceQuery(11489, Protocol::TCP), Service::Name("asgcypresstcps")),
        (ServiceQuery(8333, Protocol::TCP), Service::Name("bitcoin")),
        (ServiceQuery(1195, Protocol::TCP), Service::Name("rsf-1")),
        (ServiceQuery(3798, Protocol::UDP), Service::Name("minilock")),
        (ServiceQuery(4689, Protocol::TCP), Service::Name("altovacentral")),
        (ServiceQuery(6346, Protocol::UDP), Service::Name("gnutella")),
        (ServiceQuery(3013, Protocol::UDP), Service::Name("gilatskysurfer")),
        (ServiceQuery(3541, Protocol::TCP), Service::Name("voispeed-port")),
        (ServiceQuery(3712, Protocol::UDP), Service::Name("sentinel-ent")),
        (ServiceQuery(5080, Protocol::TCP), Service::Name("onscreen")),
        (ServiceQuery(1548, Protocol::TCP), Service::Name("axon-lm")),
        (ServiceQuery(488, Protocol::TCP), Service::Name("gss-http")),
        (ServiceQuery(7273, Protocol::UDP), Service::Name("oma-rlp")),
        (ServiceQuery(4646, Protocol::TCP), Service::Name("dots-signal")),
        (ServiceQuery(8149, Protocol::TCP), Service::Name("eor-game")),
        (ServiceQuery(1957, Protocol::TCP), Service::Name("unix-status")),
        (ServiceQuery(5356, Protocol::TCP), Service::Name("ms-smlbiz")),
        (ServiceQuery(2022, Protocol::UDP), Service::Name("xinuexpansion2")),
        (ServiceQuery(9979, Protocol::TCP), Service::Name("visweather")),
        (ServiceQuery(2490, Protocol::TCP), Service::Name("qip-qdhcp")),
        (ServiceQuery(526, Protocol::UDP), Service::Name("tempo")),
        (ServiceQuery(1552, Protocol::TCP), Service::Name("pciarray")),
        (ServiceQuery(2650, Protocol::TCP), Service::Name("eristwoguns")),
        (ServiceQuery(4980, Protocol::TCP), Service::Name("ctxs-vpp")),
        (ServiceQuery(1064, Protocol::UDP), Service::Name("jstel")),
        (ServiceQuery(3516, Protocol::UDP), Service::Name("smartcard-port")),
        (ServiceQuery(1839, Protocol::UDP), Service::Name("netopia-vo1")),
        (ServiceQuery(4593, Protocol::TCP), Service::Name("ipt-anri-anri")),
        (ServiceQuery(764, Protocol::TCP), Service::Name("omserv")),
        (ServiceQuery(2456, Protocol::UDP), Service::Name("altav-remmgt")),
        (ServiceQuery(8194, Protocol::UDP), Service::Name("sophos")),
        (ServiceQuery(9131, Protocol::UDP), Service::Name("dddp")),
        (ServiceQuery(1703, Protocol::UDP), Service::Name("hb-engine")),
        (ServiceQuery(19315, Protocol::UDP), Service::Name("keyshadow")),
        (ServiceQuery(1865, Protocol::UDP), Service::Name("entp")),
        (ServiceQuery(2848, Protocol::UDP), Service::Name("amt-blc-port")),
        (ServiceQuery(4243, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1971, Protocol::TCP), Service::Name("netop-school")),
        (ServiceQuery(8121, Protocol::TCP), Service::Name("apollo-data")),
        (ServiceQuery(6040, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(4207, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(4027, Protocol::UDP), Service::Name("bxp")),
        (ServiceQuery(3621, Protocol::TCP), Service::Name("ep-nsp")),
        (ServiceQuery(4421, Protocol::TCP), Service::Name("scaleft")),
        (ServiceQuery(6051, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(1022, Protocol::UDP), Service::Name("exp2")),
        (ServiceQuery(476, Protocol::UDP), Service::Name("tn-tl-fd1")),
        (ServiceQuery(4401, Protocol::TCP), Service::Name("ds-srvr")),
        (ServiceQuery(1538, Protocol::UDP), Service::Name("3ds-lm")),
        (ServiceQuery(1001, Protocol::TCP), Service::Name("webpush")),
        (ServiceQuery(1387, Protocol::TCP), Service::Name("cadsi-lm")),
        (ServiceQuery(4702, Protocol::UDP), Service::Name("netxms-sync")),
        (ServiceQuery(4394, Protocol::UDP), Service::Name("apwi-disc")),
        (ServiceQuery(314, Protocol::TCP), Service::Name("opalis-robot")),
        (ServiceQuery(2743, Protocol::TCP), Service::Name("murx")),
        (ServiceQuery(5992, Protocol::UDP), Service::Name("consul-insight")),
        (ServiceQuery(4023, Protocol::UDP), Service::Name("esnm-zoning")),
        (ServiceQuery(214, Protocol::UDP), Service::Name("vmpwscs")),
        (ServiceQuery(4592, Protocol::TCP), Service::Name("hrpd-ith-at-an")),
        (ServiceQuery(2801, Protocol::UDP), Service::Name("igcp")),
        (ServiceQuery(1071, Protocol::TCP), Service::Name("bsquare-voip")),
        (ServiceQuery(10161, Protocol::TCP), Service::Name("snmptls")),
        (ServiceQuery(4079, Protocol::UDP), Service::Name("santools")),
        (ServiceQuery(2891, Protocol::TCP), Service::Name("cinegrfx-elmd")),
        (ServiceQuery(8230, Protocol::UDP), Service::Name("rexecj")),
        (ServiceQuery(3339, Protocol::TCP), Service::Name("anet-l")),
        (ServiceQuery(1962, Protocol::TCP), Service::Name("biap-mp")),
        (ServiceQuery(1248, Protocol::TCP), Service::Name("hermes")),
        (ServiceQuery(1300, Protocol::TCP), Service::Name("h323hostcallsc")),
        (ServiceQuery(11095, Protocol::TCP), Service::Name("weave")),
        (ServiceQuery(4451, Protocol::UDP), Service::Name("ctisystemmsg")),
        (ServiceQuery(1026, Protocol::TCP), Service::Name("LSA-or-nterm")),
        (ServiceQuery(3436, Protocol::TCP), Service::Name("gc-config")),
        (ServiceQuery(1641, Protocol::TCP), Service::Name("invision")),
        (ServiceQuery(2265, Protocol::UDP), Service::Name("apx500api-2")),
        (ServiceQuery(3417, Protocol::UDP), Service::Name("csvr")),
        (ServiceQuery(4021, Protocol::TCP), Service::Name("nexus-portal")),
        (ServiceQuery(18881, Protocol::UDP), Service::Name("infotos")),
        (ServiceQuery(22951, Protocol::TCP), Service::Name("brf-gw")),
        (ServiceQuery(3816, Protocol::TCP), Service::Name("sunlps-http")),
        (ServiceQuery(7932, Protocol::UDP), Service::Name("t2-drm")),
        (ServiceQuery(2638, Protocol::TCP), Service::Name("sybase")),
        (ServiceQuery(6267, Protocol::TCP), Service::Name("gld")),
        (ServiceQuery(12168, Protocol::TCP), Service::Name("cawas")),
        (ServiceQuery(3042, Protocol::TCP), Service::Name("journee")),
        (ServiceQuery(16666, Protocol::TCP), Service::Name("vtp")),
        (ServiceQuery(4162, Protocol::TCP), Service::Name("omstopology")),
        (ServiceQuery(1194, Protocol::UDP), Service::Name("openvpn")),
        (ServiceQuery(2393, Protocol::UDP), Service::Name("ms-olap1")),
        (ServiceQuery(1716, Protocol::TCP), Service::Name("xmsg")),
        (ServiceQuery(7411, Protocol::TCP), Service::Name("daqstream")),
        (ServiceQuery(3472, Protocol::UDP), Service::Name("jaugsremotec-1")),
        (ServiceQuery(5101, Protocol::TCP), Service::Name("admdog")),
        (ServiceQuery(103, Protocol::UDP), Service::Name("gppitnp")),
        (ServiceQuery(3444, Protocol::TCP), Service::Name("denali-server")),
        (ServiceQuery(2020, Protocol::UDP), Service::Name("xinupageserver")),
        (ServiceQuery(1671, Protocol::UDP), Service::Name("netview-aix-11")),
        (ServiceQuery(1188, Protocol::UDP), Service::Name("hp-webadmin")),
        (ServiceQuery(5418, Protocol::TCP), Service::Name("mcntp")),
        (ServiceQuery(7846, Protocol::UDP), Service::Name("apc-7846")),
        (ServiceQuery(3823, Protocol::TCP), Service::Name("acp-conduit")),
        (ServiceQuery(7999, Protocol::UDP), Service::Name("irdmi2")),
        (ServiceQuery(3018, Protocol::UDP), Service::Name("srvc_registry")),
        (ServiceQuery(8280, Protocol::TCP), Service::Name("synapse-nhttp")),
        (ServiceQuery(3309, Protocol::TCP), Service::Name("tns-adv")),
        (ServiceQuery(12005, Protocol::UDP), Service::Name("dbisamserver1")),
        (ServiceQuery(1629, Protocol::UDP), Service::Name("lontalk-urgnt")),
        (ServiceQuery(1723, Protocol::UDP), Service::Name("pptp")),
        (ServiceQuery(22128, Protocol::TCP), Service::Name("gsidcap")),
        (ServiceQuery(4267, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3446, Protocol::TCP), Service::Name("3comfaxrpc")),
        (ServiceQuery(6702, Protocol::UDP), Service::Name("e-design-net")),
        (ServiceQuery(4266, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3417, Protocol::TCP), Service::Name("csvr")),
        (ServiceQuery(1265, Protocol::TCP), Service::Name("dssiapi")),
        (ServiceQuery(1747, Protocol::TCP), Service::Name("ftrapid-2")),
        (ServiceQuery(562, Protocol::TCP), Service::Name("chshell")),
        (ServiceQuery(2716, Protocol::TCP), Service::Name("inova-ip-disco")),
        (ServiceQuery(1682, Protocol::UDP), Service::Name("lanyon-lantern")),
        (ServiceQuery(3237, Protocol::TCP), Service::Name("apparenet-tps")),
        (ServiceQuery(8100, Protocol::UDP), Service::Name("xprint-server")),
        (ServiceQuery(1402, Protocol::TCP), Service::Name("prm-sm-np")),
        (ServiceQuery(565, Protocol::TCP), Service::Name("whoami")),
        (ServiceQuery(29, Protocol::TCP), Service::Name("msg-icp")),
        (ServiceQuery(2265, Protocol::TCP), Service::Name("apx500api-2")),
        (ServiceQuery(3465, Protocol::UDP), Service::Name("edm-mgr-cntrl")),
        (ServiceQuery(117, Protocol::UDP), Service::Name("uucp-path")),
        (ServiceQuery(3607, Protocol::TCP), Service::Name("precise-i3")),
        (ServiceQuery(4255, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(37601, Protocol::TCP), Service::Name("eftp")),
        (ServiceQuery(1441, Protocol::TCP), Service::Name("cadis-1")),
        (ServiceQuery(4220, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(6244, Protocol::UDP), Service::Name("jeol-nsddp-4")),
        (ServiceQuery(1930, Protocol::TCP), Service::Name("driveappserver")),
        (ServiceQuery(3416, Protocol::UDP), Service::Name("commandport")),
        (ServiceQuery(3796, Protocol::UDP), Service::Name("spw-dialer")),
        (ServiceQuery(7326, Protocol::TCP), Service::Name("icb")),
        (ServiceQuery(3650, Protocol::UDP), Service::Name("prismiq-plugin")),
        (ServiceQuery(5120, Protocol::TCP), Service::Name("barracuda-bbs")),
        (ServiceQuery(32780, Protocol::UDP), Service::Name("sometimes-rpc24")),
        (ServiceQuery(454, Protocol::TCP), Service::Name("contentserver")),
        (ServiceQuery(3891, Protocol::TCP), Service::Name("rtc-pm-port")),
        (ServiceQuery(6635, Protocol::TCP), Service::Name("mpls-udp")),
        (ServiceQuery(2347, Protocol::UDP), Service::Name("redstorm_find")),
        (ServiceQuery(6567, Protocol::UDP), Service::Name("esp")),
        (ServiceQuery(765, Protocol::TCP), Service::Name("webster")),
        (ServiceQuery(4191, Protocol::TCP), Service::Name("dsmipv6")),
        (ServiceQuery(2690, Protocol::TCP), Service::Name("hp-nnm-data")),
        (ServiceQuery(3832, Protocol::UDP), Service::Name("xxnetserver")),
        (ServiceQuery(3310, Protocol::UDP), Service::Name("dyna-access")),
        (ServiceQuery(8416, Protocol::TCP), Service::Name("espeech")),
        (ServiceQuery(12753, Protocol::TCP), Service::Name("tsaf")),
        (ServiceQuery(3523, Protocol::UDP), Service::Name("odeumservlink")),
        (ServiceQuery(4784, Protocol::TCP), Service::Name("bfd-multi-ctl")),
        (ServiceQuery(5677, Protocol::TCP), Service::Name("questdb2-lnchr")),
        (ServiceQuery(2966, Protocol::UDP), Service::Name("idp-infotrieve")),
        (ServiceQuery(1905, Protocol::TCP), Service::Name("sugp")),
        (ServiceQuery(4084, Protocol::UDP), Service::Name("fortisphere-vm")),
        (ServiceQuery(4425, Protocol::TCP), Service::Name("netrockey6")),
        (ServiceQuery(2529, Protocol::UDP), Service::Name("utsftp")),
        (ServiceQuery(545, Protocol::TCP), Service::Name("ekshell")),
        (ServiceQuery(1351, Protocol::TCP), Service::Name("equationbuilder")),
        (ServiceQuery(1694, Protocol::UDP), Service::Name("rrimwm")),
        (ServiceQuery(2275, Protocol::TCP), Service::Name("ibridge-data")),
        (ServiceQuery(9594, Protocol::UDP), Service::Name("msgsys")),
        (ServiceQuery(2918, Protocol::TCP), Service::Name("kastenchasepad")),
        (ServiceQuery(1653, Protocol::UDP), Service::Name("alphatech-lm")),
        (ServiceQuery(8191, Protocol::TCP), Service::Name("limnerpressure")),
        (ServiceQuery(1100, Protocol::TCP), Service::Name("mctp")),
        (ServiceQuery(1659, Protocol::TCP), Service::Name("sg-lm")),
        (ServiceQuery(5999, Protocol::TCP), Service::Name("ncd-conf")),
        (ServiceQuery(1892, Protocol::TCP), Service::Name("childkey-ctrl")),
        (ServiceQuery(10543, Protocol::TCP), Service::Name("MOS-soap")),
        (ServiceQuery(3587, Protocol::UDP), Service::Name("p2pgroup")),
        (ServiceQuery(25007, Protocol::UDP), Service::Name("icl-twobase8")),
        (ServiceQuery(6619, Protocol::TCP), Service::Name("odette-ftps")),
        (ServiceQuery(5067, Protocol::UDP), Service::Name("authentx")),
        (ServiceQuery(2533, Protocol::TCP), Service::Name("snifferserver")),
        (ServiceQuery(7339, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(3951, Protocol::UDP), Service::Name("pwgippfax")),
        (ServiceQuery(282, Protocol::UDP), Service::Name("cableport-ax")),
        (ServiceQuery(2443, Protocol::TCP), Service::Name("powerclientcsf")),
        (ServiceQuery(4685, Protocol::TCP), Service::Name("autopac")),
        (ServiceQuery(2475, Protocol::TCP), Service::Name("ace-server")),
        (ServiceQuery(1979, Protocol::UDP), Service::Name("unisql-java")),
        (ServiceQuery(1400, Protocol::UDP), Service::Name("cadkey-tablet")),
        (ServiceQuery(662, Protocol::UDP), Service::Name("pftp")),
        (ServiceQuery(1137, Protocol::UDP), Service::Name("trim")),
        (ServiceQuery(2381, Protocol::TCP), Service::Name("compaq-https")),
        (ServiceQuery(3066, Protocol::TCP), Service::Name("netattachsdmp")),
        (ServiceQuery(8873, Protocol::TCP), Service::Name("dxspider")),
        (ServiceQuery(3955, Protocol::TCP), Service::Name("p2pcommunity")),
        (ServiceQuery(5398, Protocol::TCP), Service::Name("elektron-admin")),
        (ServiceQuery(3343, Protocol::UDP), Service::Name("ms-cluster-net")),
        (ServiceQuery(1378, Protocol::UDP), Service::Name("elan")),
        (ServiceQuery(7633, Protocol::UDP), Service::Name("pmdfmgt")),
        (ServiceQuery(7023, Protocol::UDP), Service::Name("ct2nmcs")),
        (ServiceQuery(26261, Protocol::TCP), Service::Name("ezmeeting")),
        (ServiceQuery(1825, Protocol::TCP), Service::Name("direcpc-video")),
        (ServiceQuery(3276, Protocol::TCP), Service::Name("maxim-asics")),
        (ServiceQuery(2605, Protocol::TCP), Service::Name("bgpd")),
        (ServiceQuery(2530, Protocol::UDP), Service::Name("vrcommerce")),
        (ServiceQuery(2116, Protocol::TCP), Service::Name("ccowcmr")),
        (ServiceQuery(354, Protocol::TCP), Service::Name("bh611")),
        (ServiceQuery(1682, Protocol::TCP), Service::Name("lanyon-lantern")),
        (ServiceQuery(5066, Protocol::TCP), Service::Name("stanag-5066")),
        (ServiceQuery(5094, Protocol::UDP), Service::Name("hart-ip")),
        (ServiceQuery(3298, Protocol::UDP), Service::Name("deskview")),
        (ServiceQuery(20034, Protocol::UDP), Service::Name("nburn_id")),
        (ServiceQuery(3610, Protocol::TCP), Service::Name("echonet")),
        (ServiceQuery(1759, Protocol::TCP), Service::Name("spss-lm")),
        (ServiceQuery(1670, Protocol::UDP), Service::Name("netview-aix-10")),
        (ServiceQuery(6770, Protocol::UDP), Service::Name("plysrv-http")),
        (ServiceQuery(1455, Protocol::UDP), Service::Name("esl-lm")),
        (ServiceQuery(706, Protocol::UDP), Service::Name("silc")),
        (ServiceQuery(1726, Protocol::UDP), Service::Name("iberiagames")),
        (ServiceQuery(4793, Protocol::TCP), Service::Name("uet")),
        (ServiceQuery(4276, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(4679, Protocol::TCP), Service::Name("mgesupervision")),
        (ServiceQuery(3934, Protocol::UDP), Service::Name("sunfm-port")),
        (ServiceQuery(8208, Protocol::UDP), Service::Name("lm-webwatcher")),
        (ServiceQuery(123, Protocol::TCP), Service::Name("ntp")),
        (ServiceQuery(2103, Protocol::TCP), Service::Name("zephyr-clt")),
        (ServiceQuery(1674, Protocol::UDP), Service::Name("proshare-mc-2")),
        (ServiceQuery(7327, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(1687, Protocol::TCP), Service::Name("nsjtp-ctrl")),
        (ServiceQuery(4557, Protocol::TCP), Service::Name("fax")),
        (ServiceQuery(40000, Protocol::UDP), Service::Name("safetynetp")),
        (ServiceQuery(8270, Protocol::TCP), Service::Name("robot-remote")),
        (ServiceQuery(1550, Protocol::UDP), Service::Name("3m-image-lm")),
        (ServiceQuery(2913, Protocol::TCP), Service::Name("boosterware")),
        (ServiceQuery(3815, Protocol::TCP), Service::Name("lansurveyorxml")),
        (ServiceQuery(72, Protocol::UDP), Service::Name("netrjs-2")),
        (ServiceQuery(2241, Protocol::UDP), Service::Name("ivsd")),
        (ServiceQuery(554, Protocol::UDP), Service::Name("rtsp")),
        (ServiceQuery(5057, Protocol::UDP), Service::Name("intecom-ps2")),
        (ServiceQuery(1041, Protocol::UDP), Service::Name("danf-ak2")),
        (ServiceQuery(519, Protocol::TCP), Service::Name("utime")),
        (ServiceQuery(1217, Protocol::UDP), Service::Name("hpss-ndapi")),
        (ServiceQuery(5030, Protocol::TCP), Service::Name("surfpass")),
        (ServiceQuery(3169, Protocol::UDP), Service::Name("serverview-as")),
        (ServiceQuery(34378, Protocol::UDP), Service::Name("p-net-local")),
        (ServiceQuery(3053, Protocol::TCP), Service::Name("dsom-server")),
        (ServiceQuery(593, Protocol::UDP), Service::Name("http-rpc-epmap")),
        (ServiceQuery(455, Protocol::UDP), Service::Name("creativepartnr")),
        (ServiceQuery(1328, Protocol::UDP), Service::Name("ewall")),
        (ServiceQuery(429, Protocol::UDP), Service::Name("ocs_amu")),
        (ServiceQuery(3172, Protocol::TCP), Service::Name("serverview-rm")),
        (ServiceQuery(7549, Protocol::UDP), Service::Name("nls-tl")),
        (ServiceQuery(7121, Protocol::UDP), Service::Name("virprot-lm")),
        (ServiceQuery(26133, Protocol::TCP), Service::Name("scscp")),
        (ServiceQuery(2448, Protocol::UDP), Service::Name("hpppssvr")),
        (ServiceQuery(1941, Protocol::UDP), Service::Name("dic-aida")),
        (ServiceQuery(1466, Protocol::UDP), Service::Name("oceansoft-lm")),
        (ServiceQuery(4534, Protocol::TCP), Service::Name("armagetronad")),
        (ServiceQuery(4676, Protocol::TCP), Service::Name("dhct-alerts")),
        (ServiceQuery(5714, Protocol::UDP), Service::Name("prosharevideo")),
        (ServiceQuery(140, Protocol::UDP), Service::Name("emfis-data")),
        (ServiceQuery(572, Protocol::UDP), Service::Name("sonar")),
        (ServiceQuery(2517, Protocol::TCP), Service::Name("call-sig-trans")),
        (ServiceQuery(2642, Protocol::UDP), Service::Name("tragic")),
        (ServiceQuery(2127, Protocol::UDP), Service::Name("index-pc-wb")),
        (ServiceQuery(2853, Protocol::UDP), Service::Name("ispipes")),
        (ServiceQuery(2781, Protocol::UDP), Service::Name("whosells")),
        (ServiceQuery(2796, Protocol::TCP), Service::Name("ac-tech")),
        (ServiceQuery(490, Protocol::TCP), Service::Name("micom-pfs")),
        (ServiceQuery(2457, Protocol::TCP), Service::Name("rapido-ip")),
        (ServiceQuery(1451, Protocol::TCP), Service::Name("infoman")),
        (ServiceQuery(3060, Protocol::TCP), Service::Name("interserver")),
        (ServiceQuery(6088, Protocol::TCP), Service::Name("doglms")),
        (ServiceQuery(5679, Protocol::TCP), Service::Name("activesync")),
        (ServiceQuery(464, Protocol::TCP), Service::Name("kpasswd5")),
        (ServiceQuery(3057, Protocol::TCP), Service::Name("goahead-fldup")),
        (ServiceQuery(4840, Protocol::TCP), Service::Name("opcua-tcp")),
        (ServiceQuery(31, Protocol::TCP), Service::Name("msg-auth")),
        (ServiceQuery(1266, Protocol::UDP), Service::Name("dellpwrappks")),
        (ServiceQuery(1314, Protocol::TCP), Service::Name("pdps")),
        (ServiceQuery(5136, Protocol::TCP), Service::Name("minotaur-sa")),
        (ServiceQuery(2509, Protocol::UDP), Service::Name("fjmpss")),
        (ServiceQuery(6480, Protocol::UDP), Service::Name("sun-sr-http")),
        (ServiceQuery(3029, Protocol::UDP), Service::Name("LiebDevMgmt_A")),
        (ServiceQuery(5584, Protocol::TCP), Service::Name("bis-web")),
        (ServiceQuery(6003, Protocol::UDP), Service::Name("X11:3")),
        (ServiceQuery(2783, Protocol::TCP), Service::Name("aises")),
        (ServiceQuery(1869, Protocol::UDP), Service::Name("transact")),
        (ServiceQuery(11106, Protocol::UDP), Service::Name("sgi-lk")),
        (ServiceQuery(3669, Protocol::UDP), Service::Name("casanswmgmt")),
        (ServiceQuery(2615, Protocol::TCP), Service::Name("firepower")),
        (ServiceQuery(1379, Protocol::TCP), Service::Name("dbreporter")),
        (ServiceQuery(1531, Protocol::TCP), Service::Name("rap-listen")),
        (ServiceQuery(7329, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(3680, Protocol::TCP), Service::Name("npds-tracker")),
        (ServiceQuery(6077, Protocol::TCP), Service::Name("iconstructsrv")),
        (ServiceQuery(1885, Protocol::TCP), Service::Name("vrtstrapserver")),
        (ServiceQuery(2658, Protocol::UDP), Service::Name("sns-admin")),
        (ServiceQuery(1811, Protocol::TCP), Service::Name("scientia-sdb")),
        (ServiceQuery(2795, Protocol::UDP), Service::Name("livestats")),
        (ServiceQuery(11202, Protocol::TCP), Service::Name("dcsl-backup")),
        (ServiceQuery(1263, Protocol::UDP), Service::Name("dka")),
        (ServiceQuery(1734, Protocol::UDP), Service::Name("cambertx-lm")),
        (ServiceQuery(6405, Protocol::UDP), Service::Name("boe-pagesvr")),
        (ServiceQuery(8206, Protocol::TCP), Service::Name("lm-dta")),
        (ServiceQuery(2012, Protocol::TCP), Service::Name("ttyinfo")),
        (ServiceQuery(3785, Protocol::TCP), Service::Name("bfd-echo")),
        (ServiceQuery(2125, Protocol::UDP), Service::Name("lockstep")),
        (ServiceQuery(1025, Protocol::TCP), Service::Name("NFS-or-IIS")),
        (ServiceQuery(6619, Protocol::UDP), Service::Name("odette-ftps")),
        (ServiceQuery(7004, Protocol::UDP), Service::Name("afs3-kaserver")),
        (ServiceQuery(3202, Protocol::TCP), Service::Name("intraintra")),
        (ServiceQuery(3210, Protocol::TCP), Service::Name("flamenco-proxy")),
        (ServiceQuery(1065, Protocol::UDP), Service::Name("syscomlan")),
        (ServiceQuery(2689, Protocol::TCP), Service::Name("fastlynx")),
        (ServiceQuery(1309, Protocol::UDP), Service::Name("jtag-server")),
        (ServiceQuery(7738, Protocol::TCP), Service::Name("aiagent")),
        (ServiceQuery(3227, Protocol::UDP), Service::Name("dwnmshttp")),
        (ServiceQuery(7169, Protocol::TCP), Service::Name("ccag-pib")),
        (ServiceQuery(3289, Protocol::TCP), Service::Name("enpc")),
        (ServiceQuery(2788, Protocol::TCP), Service::Name("fryeserv")),
        (ServiceQuery(18881, Protocol::TCP), Service::Name("infotos")),
        (ServiceQuery(1565, Protocol::UDP), Service::Name("winddlb")),
        (ServiceQuery(1803, Protocol::UDP), Service::Name("hp-hcip-gwy")),
        (ServiceQuery(5581, Protocol::UDP), Service::Name("tmosms1")),
        (ServiceQuery(3020, Protocol::TCP), Service::Name("cifs")),
        (ServiceQuery(3315, Protocol::TCP), Service::Name("cdid")),
        (ServiceQuery(2851, Protocol::UDP), Service::Name("webemshttp")),
        (ServiceQuery(523, Protocol::UDP), Service::Name("ibm-db2")),
        (ServiceQuery(5427, Protocol::UDP), Service::Name("sco-peer-tta")),
        (ServiceQuery(2815, Protocol::TCP), Service::Name("lbc-measure")),
        (ServiceQuery(4041, Protocol::UDP), Service::Name("houston")),
        (ServiceQuery(2968, Protocol::UDP), Service::Name("enpp")),
        (ServiceQuery(3479, Protocol::UDP), Service::Name("twrpc")),
        (ServiceQuery(24, Protocol::TCP), Service::Name("priv-mail")),
        (ServiceQuery(2349, Protocol::TCP), Service::Name("redstorm_diag")),
        (ServiceQuery(31949, Protocol::TCP), Service::Name("iceedcp_rx")),
        (ServiceQuery(567, Protocol::UDP), Service::Name("banyan-rpc")),
        (ServiceQuery(4206, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1437, Protocol::UDP), Service::Name("tabula")),
        (ServiceQuery(2970, Protocol::TCP), Service::Name("index-net")),
        (ServiceQuery(3775, Protocol::UDP), Service::Name("ispmmgr")),
        (ServiceQuery(2078, Protocol::UDP), Service::Name("tpcsrvr")),
        (ServiceQuery(6072, Protocol::TCP), Service::Name("diagnose-proc")),
        (ServiceQuery(2696, Protocol::UDP), Service::Name("unifyadmin")),
        (ServiceQuery(1319, Protocol::TCP), Service::Name("amx-icsp")),
        (ServiceQuery(1785, Protocol::TCP), Service::Name("windlm")),
        (ServiceQuery(3461, Protocol::TCP), Service::Name("edm-stager")),
        (ServiceQuery(3742, Protocol::UDP), Service::Name("cst-port")),
        (ServiceQuery(4689, Protocol::UDP), Service::Name("altovacentral")),
        (ServiceQuery(3527, Protocol::UDP), Service::Name("beserver-msg-q")),
        (ServiceQuery(3211, Protocol::UDP), Service::Name("avsecuremgmt")),
        (ServiceQuery(7980, Protocol::UDP), Service::Name("quest-vista")),
        (ServiceQuery(8807, Protocol::TCP), Service::Name("hes-clip")),
        (ServiceQuery(1865, Protocol::TCP), Service::Name("entp")),
        (ServiceQuery(1831, Protocol::TCP), Service::Name("myrtle")),
        (ServiceQuery(3043, Protocol::UDP), Service::Name("brp")),
        (ServiceQuery(4356, Protocol::TCP), Service::Name("qsnet-assist")),
        (ServiceQuery(1112, Protocol::TCP), Service::Name("msql")),
        (ServiceQuery(1712, Protocol::TCP), Service::Name("registrar")),
        (ServiceQuery(3568, Protocol::UDP), Service::Name("oap-s")),
        (ServiceQuery(1490, Protocol::UDP), Service::Name("insitu-conf")),
        (ServiceQuery(2746, Protocol::UDP), Service::Name("cpudpencap")),
        (ServiceQuery(5154, Protocol::TCP), Service::Name("bzflag")),
        (ServiceQuery(43441, Protocol::TCP), Service::Name("ciscocsdb")),
        (ServiceQuery(3278, Protocol::UDP), Service::Name("lkcmserver")),
        (ServiceQuery(1683, Protocol::UDP), Service::Name("ncpm-hip")),
        (ServiceQuery(6687, Protocol::TCP), Service::Name("clever-ctrace")),
        (ServiceQuery(2817, Protocol::UDP), Service::Name("nmsigport")),
        (ServiceQuery(7164, Protocol::TCP), Service::Name("fsr")),
        (ServiceQuery(1985, Protocol::TCP), Service::Name("hsrp")),
        (ServiceQuery(3711, Protocol::TCP), Service::Name("edb-server2")),
        (ServiceQuery(172, Protocol::TCP), Service::Name("cl-1")),
        (ServiceQuery(348, Protocol::UDP), Service::Name("csi-sgwp")),
        (ServiceQuery(2317, Protocol::TCP), Service::Name("attachmate-g32")),
        (ServiceQuery(43190, Protocol::UDP), Service::Name("ip-provision")),
        (ServiceQuery(8204, Protocol::TCP), Service::Name("lm-perfworks")),
        (ServiceQuery(3542, Protocol::TCP), Service::Name("hacl-monitor")),
        (ServiceQuery(615, Protocol::TCP), Service::Name("sco-inetmgr")),
        (ServiceQuery(5223, Protocol::TCP), Service::Name("hpvirtgrp")),
        (ServiceQuery(16950, Protocol::TCP), Service::Name("sgcip")),
        (ServiceQuery(8006, Protocol::TCP), Service::Name("wpl-analytics")),
        (ServiceQuery(5358, Protocol::TCP), Service::Name("wsdapi-s")),
        (ServiceQuery(3882, Protocol::UDP), Service::Name("msdts1")),
        (ServiceQuery(5911, Protocol::UDP), Service::Name("cpdlc")),
        (ServiceQuery(531, Protocol::TCP), Service::Name("conference")),
        (ServiceQuery(2417, Protocol::TCP), Service::Name("composit-server")),
        (ServiceQuery(1454, Protocol::UDP), Service::Name("interhdl_elmd")),
        (ServiceQuery(4068, Protocol::UDP), Service::Name("ipfltbcst")),
        (ServiceQuery(2461, Protocol::TCP), Service::Name("qadmifoper")),
        (ServiceQuery(5250, Protocol::UDP), Service::Name("soagateway")),
        (ServiceQuery(7337, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(4227, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1720, Protocol::UDP), Service::Name("h323hostcall")),
        (ServiceQuery(4379, Protocol::UDP), Service::Name("ctdb")),
        (ServiceQuery(44321, Protocol::UDP), Service::Name("pmcd")),
        (ServiceQuery(25009, Protocol::TCP), Service::Name("icl-twobase10")),
        (ServiceQuery(1712, Protocol::UDP), Service::Name("registrar")),
        (ServiceQuery(5024, Protocol::UDP), Service::Name("scpi-telnet")),
        (ServiceQuery(5079, Protocol::UDP), Service::Name("cp-spxrpts")),
        (ServiceQuery(3843, Protocol::TCP), Service::Name("quest-agent")),
        (ServiceQuery(3671, Protocol::TCP), Service::Name("efcp")),
        (ServiceQuery(676, Protocol::UDP), Service::Name("vpps-via")),
        (ServiceQuery(375, Protocol::UDP), Service::Name("hassle")),
        (ServiceQuery(3588, Protocol::UDP), Service::Name("sentinel")),
        (ServiceQuery(3836, Protocol::UDP), Service::Name("markem-dcp")),
        (ServiceQuery(16368, Protocol::UDP), Service::Name("netserialext4")),
        (ServiceQuery(2469, Protocol::UDP), Service::Name("mti-tcs-comm")),
        (ServiceQuery(4102, Protocol::TCP), Service::Name("brlp-1")),
        (ServiceQuery(2946, Protocol::TCP), Service::Name("fjsvmpor")),
        (ServiceQuery(6055, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(584, Protocol::TCP), Service::Name("keyserver")),
        (ServiceQuery(2938, Protocol::UDP), Service::Name("sm-pas-1")),
        (ServiceQuery(5069, Protocol::TCP), Service::Name("i-net-2000-npr")),
        (ServiceQuery(40841, Protocol::UDP), Service::Name("cscp")),
        (ServiceQuery(3853, Protocol::UDP), Service::Name("sscan")),
        (ServiceQuery(5505, Protocol::TCP), Service::Name("checkoutdb")),
        (ServiceQuery(772, Protocol::TCP), Service::Name("cycleserv2")),
        (ServiceQuery(3973, Protocol::UDP), Service::Name("progistics")),
        (ServiceQuery(6007, Protocol::UDP), Service::Name("X11:7")),
        (ServiceQuery(2423, Protocol::TCP), Service::Name("rnrp")),
        (ServiceQuery(10128, Protocol::UDP), Service::Name("bmc-perf-sd")),
        (ServiceQuery(160, Protocol::UDP), Service::Name("sgmp-traps")),
        (ServiceQuery(3561, Protocol::UDP), Service::Name("bmc-onekey")),
        (ServiceQuery(912, Protocol::TCP), Service::Name("apex-mesh")),
        (ServiceQuery(1947, Protocol::UDP), Service::Name("sentinelsrm")),
        (ServiceQuery(2126, Protocol::UDP), Service::Name("pktcable-cops")),
        (ServiceQuery(1201, Protocol::TCP), Service::Name("nucleus-sand")),
        (ServiceQuery(3726, Protocol::UDP), Service::Name("array-manager")),
        (ServiceQuery(6069, Protocol::TCP), Service::Name("trip")),
        (ServiceQuery(5357, Protocol::TCP), Service::Name("wsdapi")),
        (ServiceQuery(360, Protocol::UDP), Service::Name("scoi2odialog")),
        (ServiceQuery(529, Protocol::TCP), Service::Name("irc")),
        (ServiceQuery(4457, Protocol::UDP), Service::Name("prRegister")),
        (ServiceQuery(1926, Protocol::TCP), Service::Name("egs")),
        (ServiceQuery(3849, Protocol::TCP), Service::Name("spw-dnspreload")),
        (ServiceQuery(477, Protocol::TCP), Service::Name("ss7ns")),
        (ServiceQuery(9628, Protocol::UDP), Service::Name("odbcpathway")),
        (ServiceQuery(6778, Protocol::TCP), Service::Name("ntz-p2p-storage")),
        (ServiceQuery(7743, Protocol::TCP), Service::Name("sstp-1")),
        (ServiceQuery(1510, Protocol::UDP), Service::Name("mvx-lm")),
        (ServiceQuery(3064, Protocol::TCP), Service::Name("dnet-tstproxy")),
        (ServiceQuery(9203, Protocol::TCP), Service::Name("wap-wsp-wtp-s")),
        (ServiceQuery(4400, Protocol::TCP), Service::Name("ds-srv")),
        (ServiceQuery(3153, Protocol::TCP), Service::Name("s8-client-port")),
        (ServiceQuery(3742, Protocol::TCP), Service::Name("cst-port")),
        (ServiceQuery(7165, Protocol::UDP), Service::Name("doc-server")),
        (ServiceQuery(1589, Protocol::TCP), Service::Name("vqp")),
        (ServiceQuery(21801, Protocol::TCP), Service::Name("sal")),
        (ServiceQuery(9210, Protocol::UDP), Service::Name("oma-mlp")),
        (ServiceQuery(4080, Protocol::TCP), Service::Name("lorica-in")),
        (ServiceQuery(1849, Protocol::TCP), Service::Name("alpha-sms")),
        (ServiceQuery(8473, Protocol::UDP), Service::Name("vp2p")),
        (ServiceQuery(127, Protocol::TCP), Service::Name("locus-con")),
        (ServiceQuery(9001, Protocol::TCP), Service::Name("tor-orport")),
        (ServiceQuery(1272, Protocol::UDP), Service::Name("cspmlockmgr")),
        (ServiceQuery(1648, Protocol::TCP), Service::Name("concurrent-lm")),
        (ServiceQuery(5303, Protocol::UDP), Service::Name("hacl-probe")),
        (ServiceQuery(3074, Protocol::TCP), Service::Name("xbox")),
        (ServiceQuery(398, Protocol::UDP), Service::Name("kryptolan")),
        (ServiceQuery(3318, Protocol::TCP), Service::Name("ssrip")),
        (ServiceQuery(2955, Protocol::UDP), Service::Name("csnotify")),
        (ServiceQuery(6580, Protocol::UDP), Service::Name("parsec-master")),
        (ServiceQuery(2167, Protocol::UDP), Service::Name("raw-serial")),
        (ServiceQuery(6324, Protocol::TCP), Service::Name("hrd-ncs")),
        (ServiceQuery(5680, Protocol::TCP), Service::Name("canna")),
        (ServiceQuery(1996, Protocol::TCP), Service::Name("tr-rsrb-port")),
        (ServiceQuery(2259, Protocol::UDP), Service::Name("acd-pm")),
        (ServiceQuery(2326, Protocol::TCP), Service::Name("idcp")),
        (ServiceQuery(2477, Protocol::UDP), Service::Name("ssm-cvs")),
        (ServiceQuery(1603, Protocol::TCP), Service::Name("picodbc")),
        (ServiceQuery(8503, Protocol::TCP), Service::Name("lsp-self-ping")),
        (ServiceQuery(80, Protocol::TCP), Service::Name("http")),
        (ServiceQuery(2967, Protocol::TCP), Service::Name("symantec-av")),
        (ServiceQuery(3313, Protocol::UDP), Service::Name("uorb")),
        (ServiceQuery(1484, Protocol::TCP), Service::Name("confluent")),
        (ServiceQuery(510, Protocol::UDP), Service::Name("fcp")),
        (ServiceQuery(5408, Protocol::TCP), Service::Name("foresyte-sec")),
        (ServiceQuery(5675, Protocol::TCP), Service::Name("v5ua")),
        (ServiceQuery(1433, Protocol::TCP), Service::Name("ms-sql-s")),
        (ServiceQuery(2228, Protocol::UDP), Service::Name("ehome-ms")),
        (ServiceQuery(371, Protocol::UDP), Service::Name("clearcase")),
        (ServiceQuery(2158, Protocol::UDP), Service::Name("touchnetplus")),
        (ServiceQuery(1128, Protocol::TCP), Service::Name("saphostctrl")),
        (ServiceQuery(2756, Protocol::UDP), Service::Name("simplement-tie")),
        (ServiceQuery(2158, Protocol::TCP), Service::Name("touchnetplus")),
        (ServiceQuery(33435, Protocol::TCP), Service::Name("mtrace")),
        (ServiceQuery(7312, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(3432, Protocol::UDP), Service::Name("osdcp")),
        (ServiceQuery(153, Protocol::UDP), Service::Name("sgmp")),
        (ServiceQuery(2670, Protocol::TCP), Service::Name("tve-announce")),
        (ServiceQuery(3880, Protocol::TCP), Service::Name("igrs")),
        (ServiceQuery(12013, Protocol::TCP), Service::Name("vipera-ssl")),
        (ServiceQuery(4138, Protocol::UDP), Service::Name("nettest")),
        (ServiceQuery(6551, Protocol::UDP), Service::Name("sum")),
        (ServiceQuery(2893, Protocol::TCP), Service::Name("vseconnector")),
        (ServiceQuery(1488, Protocol::UDP), Service::Name("docstor")),
        (ServiceQuery(8379, Protocol::TCP), Service::Name("cruise-diags")),
        (ServiceQuery(563, Protocol::UDP), Service::Name("snews")),
        (ServiceQuery(3945, Protocol::TCP), Service::Name("emcads")),
        (ServiceQuery(11162, Protocol::TCP), Service::Name("suncacao-jmxmp")),
        (ServiceQuery(7900, Protocol::UDP), Service::Name("mevent")),
        (ServiceQuery(1669, Protocol::TCP), Service::Name("netview-aix-9")),
        (ServiceQuery(417, Protocol::UDP), Service::Name("onmux")),
        (ServiceQuery(17234, Protocol::TCP), Service::Name("integrius-stp")),
        (ServiceQuery(36424, Protocol::TCP), Service::Name("nq-ap")),
        (ServiceQuery(193, Protocol::TCP), Service::Name("srmp")),
        (ServiceQuery(2644, Protocol::TCP), Service::Name("travsoft-ipx-t")),
        (ServiceQuery(630, Protocol::UDP), Service::Name("rda")),
        (ServiceQuery(2340, Protocol::UDP), Service::Name("wrs_registry")),
        (ServiceQuery(1909, Protocol::UDP), Service::Name("global-wlink")),
        (ServiceQuery(3627, Protocol::UDP), Service::Name("jamserverport")),
        (ServiceQuery(1258, Protocol::TCP), Service::Name("opennl")),
        (ServiceQuery(2758, Protocol::TCP), Service::Name("apollo-status")),
        (ServiceQuery(322, Protocol::UDP), Service::Name("rtsps")),
        (ServiceQuery(3592, Protocol::TCP), Service::Name("gtrack-ne")),
        (ServiceQuery(1026, Protocol::UDP), Service::Name("win-rpc")),
        (ServiceQuery(1188, Protocol::TCP), Service::Name("hp-webadmin")),
        (ServiceQuery(4244, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(9000, Protocol::TCP), Service::Name("cslistener")),
        (ServiceQuery(2114, Protocol::UDP), Service::Name("newheights")),
        (ServiceQuery(3650, Protocol::TCP), Service::Name("prismiq-plugin")),
        (ServiceQuery(6073, Protocol::UDP), Service::Name("directplay8")),
        (ServiceQuery(7560, Protocol::UDP), Service::Name("sncp")),
        (ServiceQuery(8033, Protocol::UDP), Service::Name("mindprint")),
        (ServiceQuery(999, Protocol::TCP), Service::Name("garcon")),
        (ServiceQuery(7962, Protocol::TCP), Service::Name("generalsync")),
        (ServiceQuery(2100, Protocol::TCP), Service::Name("amiganetfs")),
        (ServiceQuery(4119, Protocol::TCP), Service::Name("assuria-slm")),
        (ServiceQuery(1937, Protocol::TCP), Service::Name("jwserver")),
        (ServiceQuery(5784, Protocol::TCP), Service::Name("ibar")),
        (ServiceQuery(2711, Protocol::TCP), Service::Name("sso-control")),
        (ServiceQuery(3707, Protocol::UDP), Service::Name("rt-event-s")),
        (ServiceQuery(1203, Protocol::TCP), Service::Name("ssslic-mgr")),
        (ServiceQuery(1251, Protocol::TCP), Service::Name("servergraph")),
        (ServiceQuery(2856, Protocol::TCP), Service::Name("cesdinv")),
        (ServiceQuery(730, Protocol::UDP), Service::Name("netviewdm2")),
        (ServiceQuery(5163, Protocol::TCP), Service::Name("sbackup")),
        (ServiceQuery(1961, Protocol::TCP), Service::Name("bts-appserver")),
        (ServiceQuery(437, Protocol::UDP), Service::Name("comscm")),
        (ServiceQuery(8243, Protocol::TCP), Service::Name("synapse-nhttps")),
        (ServiceQuery(1461, Protocol::TCP), Service::Name("ibm_wrless_lan")),
        (ServiceQuery(2366, Protocol::TCP), Service::Name("qip-login")),
        (ServiceQuery(17755, Protocol::UDP), Service::Name("zigbee-ip")),
        (ServiceQuery(1699, Protocol::TCP), Service::Name("rsvp-encap-2")),
        (ServiceQuery(6788, Protocol::UDP), Service::Name("smc-http")),
        (ServiceQuery(162, Protocol::UDP), Service::Name("snmptrap")),
        (ServiceQuery(2661, Protocol::UDP), Service::Name("olhost")),
        (ServiceQuery(8025, Protocol::UDP), Service::Name("ca-audit-da")),
        (ServiceQuery(581, Protocol::TCP), Service::Name("bdp")),
        (ServiceQuery(3851, Protocol::TCP), Service::Name("spectraport")),
        (ServiceQuery(4103, Protocol::TCP), Service::Name("brlp-2")),
        (ServiceQuery(4549, Protocol::TCP), Service::Name("aegate")),
        (ServiceQuery(7347, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(1561, Protocol::UDP), Service::Name("facilityview")),
        (ServiceQuery(4322, Protocol::TCP), Service::Name("trim-event")),
        (ServiceQuery(3799, Protocol::UDP), Service::Name("radius-dynauth")),
        (ServiceQuery(8611, Protocol::UDP), Service::Name("canon-bjnp1")),
        (ServiceQuery(699, Protocol::UDP), Service::Name("accessnetwork")),
        (ServiceQuery(3427, Protocol::UDP), Service::Name("websphere-snmp")),
        (ServiceQuery(3690, Protocol::TCP), Service::Name("svn")),
        (ServiceQuery(1940, Protocol::UDP), Service::Name("jvclient")),
        (ServiceQuery(27004, Protocol::UDP), Service::Name("flex-lm")),
        (ServiceQuery(4287, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1949, Protocol::UDP), Service::Name("ismaeasdaqlive")),
        (ServiceQuery(2031, Protocol::UDP), Service::Name("mobrien-chat")),
        (ServiceQuery(29167, Protocol::TCP), Service::Name("otmp")),
        (ServiceQuery(4102, Protocol::UDP), Service::Name("brlp-1")),
        (ServiceQuery(2603, Protocol::UDP), Service::Name("servicemeter")),
        (ServiceQuery(5236, Protocol::UDP), Service::Name("padl2sim")),
        (ServiceQuery(1295, Protocol::UDP), Service::Name("ehtp")),
        (ServiceQuery(1801, Protocol::UDP), Service::Name("msmq")),
        (ServiceQuery(2068, Protocol::UDP), Service::Name("avauthsrvprtcl")),
        (ServiceQuery(6407, Protocol::TCP), Service::Name("boe-resssvr1")),
        (ServiceQuery(4111, Protocol::UDP), Service::Name("xgrid")),
        (ServiceQuery(2634, Protocol::TCP), Service::Name("pk-electronics")),
        (ServiceQuery(3806, Protocol::UDP), Service::Name("wsmlb")),
        (ServiceQuery(4547, Protocol::TCP), Service::Name("lanner-lm")),
        (ServiceQuery(4222, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1921, Protocol::TCP), Service::Name("noadmin")),
        (ServiceQuery(1912, Protocol::TCP), Service::Name("rhp-iibp")),
        (ServiceQuery(810, Protocol::UDP), Service::Name("fcp-udp")),
        (ServiceQuery(534, Protocol::TCP), Service::Name("mm-admin")),
        (ServiceQuery(2258, Protocol::TCP), Service::Name("rcts")),
        (ServiceQuery(2381, Protocol::UDP), Service::Name("compaq-https")),
        (ServiceQuery(4304, Protocol::TCP), Service::Name("owserver")),
        (ServiceQuery(3146, Protocol::UDP), Service::Name("bears-02")),
        (ServiceQuery(2869, Protocol::UDP), Service::Name("icslap")),
        (ServiceQuery(3036, Protocol::UDP), Service::Name("hagel-dump")),
        (ServiceQuery(1222, Protocol::TCP), Service::Name("nerv")),
        (ServiceQuery(2313, Protocol::TCP), Service::Name("iapp")),
        (ServiceQuery(1416, Protocol::UDP), Service::Name("novell-lu6.2")),
        (ServiceQuery(1415, Protocol::UDP), Service::Name("dbstar")),
        (ServiceQuery(32787, Protocol::UDP), Service::Name("sometimes-rpc28")),
        (ServiceQuery(1344, Protocol::TCP), Service::Name("icap")),
        (ServiceQuery(30002, Protocol::TCP), Service::Name("pago-services2")),
        (ServiceQuery(3216, Protocol::TCP), Service::Name("ferrari-foam")),
        (ServiceQuery(1684, Protocol::UDP), Service::Name("snaresecure")),
        (ServiceQuery(4284, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(287, Protocol::UDP), Service::Name("k-block")),
        (ServiceQuery(1372, Protocol::TCP), Service::Name("fc-ser")),
        (ServiceQuery(3373, Protocol::TCP), Service::Name("lavenir-lm")),
        (ServiceQuery(2507, Protocol::UDP), Service::Name("spock")),
        (ServiceQuery(1331, Protocol::UDP), Service::Name("intersan")),
        (ServiceQuery(3067, Protocol::UDP), Service::Name("fjhpjp")),
        (ServiceQuery(2220, Protocol::TCP), Service::Name("netiq")),
        (ServiceQuery(23, Protocol::UDP), Service::Name("telnet")),
        (ServiceQuery(2352, Protocol::UDP), Service::Name("pslserver")),
        (ServiceQuery(1825, Protocol::UDP), Service::Name("direcpc-video")),
        (ServiceQuery(4741, Protocol::UDP), Service::Name("lumimgrd")),
        (ServiceQuery(1645, Protocol::TCP), Service::Name("sightline")),
        (ServiceQuery(610, Protocol::TCP), Service::Name("npmp-local")),
        (ServiceQuery(6347, Protocol::TCP), Service::Name("gnutella2")),
        (ServiceQuery(2879, Protocol::UDP), Service::Name("ucentric-ds")),
        (ServiceQuery(9614, Protocol::TCP), Service::Name("iadt-tls")),
        (ServiceQuery(3781, Protocol::UDP), Service::Name("abcvoice-port")),
        (ServiceQuery(536, Protocol::TCP), Service::Name("opalis-rdv")),
        (ServiceQuery(2589, Protocol::UDP), Service::Name("quartus-tcl")),
        (ServiceQuery(1829, Protocol::UDP), Service::Name("optika-emedia")),
        (ServiceQuery(12006, Protocol::TCP), Service::Name("dbisamserver2")),
        (ServiceQuery(4052, Protocol::TCP), Service::Name("interact")),
        (ServiceQuery(3572, Protocol::UDP), Service::Name("megaregsvrport")),
        (ServiceQuery(3282, Protocol::TCP), Service::Name("datusorb")),
        (ServiceQuery(6148, Protocol::TCP), Service::Name("ricardo-lm")),
        (ServiceQuery(2742, Protocol::TCP), Service::Name("tsb2")),
        (ServiceQuery(2568, Protocol::UDP), Service::Name("spamtrap")),
        (ServiceQuery(20000, Protocol::UDP), Service::Name("dnp")),
        (ServiceQuery(4545, Protocol::UDP), Service::Name("worldscores")),
        (ServiceQuery(1187, Protocol::UDP), Service::Name("alias")),
        (ServiceQuery(2087, Protocol::TCP), Service::Name("eli")),
        (ServiceQuery(6579, Protocol::UDP), Service::Name("affiliate")),
        (ServiceQuery(5793, Protocol::UDP), Service::Name("xtreamx")),
        (ServiceQuery(633, Protocol::UDP), Service::Name("servstat")),
        (ServiceQuery(8450, Protocol::TCP), Service::Name("npmp")),
        (ServiceQuery(11165, Protocol::UDP), Service::Name("suncacao-websvc")),
        (ServiceQuery(2790, Protocol::TCP), Service::Name("plgproxy")),
        (ServiceQuery(2314, Protocol::UDP), Service::Name("cr-websystems")),
        (ServiceQuery(5364, Protocol::TCP), Service::Name("kdnet")),
        (ServiceQuery(7982, Protocol::TCP), Service::Name("sossd-agent")),
        (ServiceQuery(3371, Protocol::TCP), Service::Name("satvid-datalnk")),
        (ServiceQuery(2606, Protocol::TCP), Service::Name("netmon")),
        (ServiceQuery(1526, Protocol::UDP), Service::Name("pdap-np")),
        (ServiceQuery(4139, Protocol::UDP), Service::Name("thrtx")),
        (ServiceQuery(1879, Protocol::TCP), Service::Name("nettgain-nms")),
        (ServiceQuery(3828, Protocol::TCP), Service::Name("neteh")),
        (ServiceQuery(535, Protocol::TCP), Service::Name("iiop")),
        (ServiceQuery(8058, Protocol::TCP), Service::Name("senomix07")),
        (ServiceQuery(1291, Protocol::UDP), Service::Name("seagulllms")),
        (ServiceQuery(32769, Protocol::TCP), Service::Name("filenet-rpc")),
        (ServiceQuery(4035, Protocol::UDP), Service::Name("wap-push-http")),
        (ServiceQuery(2133, Protocol::UDP), Service::Name("zymed-zpp")),
        (ServiceQuery(3413, Protocol::UDP), Service::Name("svnet")),
        (ServiceQuery(35, Protocol::TCP), Service::Name("priv-print")),
        (ServiceQuery(1686, Protocol::TCP), Service::Name("cvmon")),
        (ServiceQuery(5556, Protocol::TCP), Service::Name("freeciv")),
        (ServiceQuery(6690, Protocol::TCP), Service::Name("cleverdetect")),
        (ServiceQuery(27003, Protocol::TCP), Service::Name("flexlm3")),
        (ServiceQuery(9119, Protocol::UDP), Service::Name("mxit")),
        (ServiceQuery(6407, Protocol::UDP), Service::Name("boe-resssvr1")),
        (ServiceQuery(9207, Protocol::UDP), Service::Name("wap-vcal-s")),
        (ServiceQuery(2988, Protocol::UDP), Service::Name("hippad")),
        (ServiceQuery(752, Protocol::TCP), Service::Name("qrh")),
        (ServiceQuery(1463, Protocol::UDP), Service::Name("nucleus")),
        (ServiceQuery(3702, Protocol::TCP), Service::Name("ws-discovery")),
        (ServiceQuery(6076, Protocol::TCP), Service::Name("msft-dpm-cert")),
        (ServiceQuery(349, Protocol::UDP), Service::Name("mftp")),
        (ServiceQuery(3011, Protocol::TCP), Service::Name("trusted-web")),
        (ServiceQuery(5556, Protocol::UDP), Service::Name("freeciv")),
        (ServiceQuery(6443, Protocol::UDP), Service::Name("sun-sr-https")),
        (ServiceQuery(248, Protocol::UDP), Service::Name("bhfhs")),
        (ServiceQuery(18104, Protocol::TCP), Service::Name("radpdf")),
        (ServiceQuery(1773, Protocol::TCP), Service::Name("kmscontrol")),
        (ServiceQuery(3784, Protocol::UDP), Service::Name("bfd-control")),
        (ServiceQuery(2142, Protocol::UDP), Service::Name("tdmoip")),
        (ServiceQuery(4749, Protocol::UDP), Service::Name("profilemac")),
        (ServiceQuery(6059, Protocol::UDP), Service::Name("X11:59")),
        (ServiceQuery(557, Protocol::UDP), Service::Name("openvms-sysipc")),
        (ServiceQuery(2317, Protocol::UDP), Service::Name("attachmate-g32")),
        (ServiceQuery(3806, Protocol::TCP), Service::Name("wsmlb")),
        (ServiceQuery(7014, Protocol::TCP), Service::Name("microtalon-com")),
        (ServiceQuery(10113, Protocol::TCP), Service::Name("netiq-endpoint")),
        (ServiceQuery(82, Protocol::TCP), Service::Name("xfer")),
        (ServiceQuery(6010, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(1301, Protocol::TCP), Service::Name("ci3-software-1")),
        (ServiceQuery(3340, Protocol::UDP), Service::Name("anet-m")),
        (ServiceQuery(7234, Protocol::TCP), Service::Name("asa-gateways")),
        (ServiceQuery(3893, Protocol::TCP), Service::Name("cgi-starapi")),
        (ServiceQuery(9085, Protocol::TCP), Service::Name("ibm-rsyscon")),
        (ServiceQuery(1702, Protocol::TCP), Service::Name("deskshare")),
        (ServiceQuery(5068, Protocol::TCP), Service::Name("bitforestsrv")),
        (ServiceQuery(2651, Protocol::TCP), Service::Name("ebinsite")),
        (ServiceQuery(1681, Protocol::UDP), Service::Name("sd-elmd")),
        (ServiceQuery(1094, Protocol::UDP), Service::Name("rootd")),
        (ServiceQuery(4215, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(8351, Protocol::TCP), Service::Name("server-find")),
        (ServiceQuery(670, Protocol::TCP), Service::Name("vacdsm-sws")),
        (ServiceQuery(5252, Protocol::TCP), Service::Name("movaz-ssc")),
        (ServiceQuery(219, Protocol::UDP), Service::Name("uarps")),
        (ServiceQuery(1062, Protocol::UDP), Service::Name("veracity")),
        (ServiceQuery(1841, Protocol::UDP), Service::Name("netopia-vo3")),
        (ServiceQuery(3058, Protocol::UDP), Service::Name("videobeans")),
        (ServiceQuery(1608, Protocol::UDP), Service::Name("smart-lm")),
        (ServiceQuery(12345, Protocol::UDP), Service::Name("italk")),
        (ServiceQuery(2760, Protocol::TCP), Service::Name("sabams")),
        (ServiceQuery(5230, Protocol::TCP), Service::Name("jaxflow-data")),
        (ServiceQuery(4658, Protocol::TCP), Service::Name("playsta2-app")),
        (ServiceQuery(3683, Protocol::TCP), Service::Name("bmc-ea")),
        (ServiceQuery(556, Protocol::UDP), Service::Name("remotefs")),
        (ServiceQuery(1437, Protocol::TCP), Service::Name("tabula")),
        (ServiceQuery(2467, Protocol::UDP), Service::Name("high-criteria")),
        (ServiceQuery(208, Protocol::UDP), Service::Name("at-8")),
        (ServiceQuery(5411, Protocol::TCP), Service::Name("actnet")),
        (ServiceQuery(1082, Protocol::TCP), Service::Name("amt-esd-prot")),
        (ServiceQuery(7279, Protocol::UDP), Service::Name("ctxlic")),
        (ServiceQuery(5681, Protocol::UDP), Service::Name("ncxcp")),
        (ServiceQuery(7597, Protocol::TCP), Service::Name("qaz")),
        (ServiceQuery(1042, Protocol::TCP), Service::Name("afrog")),
        (ServiceQuery(1075, Protocol::TCP), Service::Name("rdrmshc")),
        (ServiceQuery(2814, Protocol::TCP), Service::Name("llm-csv")),
        (ServiceQuery(450, Protocol::UDP), Service::Name("tserver")),
        (ServiceQuery(32635, Protocol::UDP), Service::Name("sec-ntb-clnt")),
        (ServiceQuery(14141, Protocol::UDP), Service::Name("vcs-app")),
        (ServiceQuery(10009, Protocol::UDP), Service::Name("swdtp-sv")),
        (ServiceQuery(5042, Protocol::TCP), Service::Name("asnaacceler8db")),
        (ServiceQuery(1410, Protocol::UDP), Service::Name("hiq")),
        (ServiceQuery(1254, Protocol::TCP), Service::Name("de-noc")),
        (ServiceQuery(2894, Protocol::UDP), Service::Name("abacus-remote")),
        (ServiceQuery(5582, Protocol::TCP), Service::Name("fac-restore")),
        (ServiceQuery(26000, Protocol::TCP), Service::Name("quake")),
        (ServiceQuery(6068, Protocol::TCP), Service::Name("gsmp")),
        (ServiceQuery(5998, Protocol::TCP), Service::Name("ncd-diag")),
        (ServiceQuery(1562, Protocol::TCP), Service::Name("pconnectmgr")),
        (ServiceQuery(3727, Protocol::TCP), Service::Name("e-mdu")),
        (ServiceQuery(9294, Protocol::UDP), Service::Name("armcenterhttp")),
        (ServiceQuery(1186, Protocol::TCP), Service::Name("mysql-cluster")),
        (ServiceQuery(4370, Protocol::TCP), Service::Name("elpro_tunnel")),
        (ServiceQuery(3620, Protocol::TCP), Service::Name("ep-pcp")),
        (ServiceQuery(2320, Protocol::UDP), Service::Name("siebel-ns")),
        (ServiceQuery(10500, Protocol::TCP), Service::Name("hip-nat-t")),
        (ServiceQuery(2708, Protocol::UDP), Service::Name("banyan-net")),
        (ServiceQuery(4333, Protocol::TCP), Service::Name("msql")),
        (ServiceQuery(8003, Protocol::UDP), Service::Name("mcreport")),
        (ServiceQuery(9397, Protocol::TCP), Service::Name("mpidcagt")),
        (ServiceQuery(8097, Protocol::UDP), Service::Name("sac")),
        (ServiceQuery(2450, Protocol::UDP), Service::Name("netadmin")),
        (ServiceQuery(6082, Protocol::TCP), Service::Name("p25cai")),
        (ServiceQuery(3055, Protocol::TCP), Service::Name("policyserver")),
        (ServiceQuery(1733, Protocol::UDP), Service::Name("siipat")),
        (ServiceQuery(14149, Protocol::TCP), Service::Name("vrts-tdd")),
        (ServiceQuery(1980, Protocol::TCP), Service::Name("pearldoc-xact")),
        (ServiceQuery(5813, Protocol::TCP), Service::Name("icmpd")),
        (ServiceQuery(780, Protocol::UDP), Service::Name("wpgs")),
        (ServiceQuery(1509, Protocol::TCP), Service::Name("robcad-lm")),
        (ServiceQuery(13217, Protocol::UDP), Service::Name("rs-pias")),
        (ServiceQuery(2412, Protocol::UDP), Service::Name("cdn")),
        (ServiceQuery(2136, Protocol::TCP), Service::Name("appworxsrv")),
        (ServiceQuery(5794, Protocol::TCP), Service::Name("spdp")),
        (ServiceQuery(1835, Protocol::UDP), Service::Name("ardusmul")),
        (ServiceQuery(3958, Protocol::TCP), Service::Name("mqe-agent")),
        (ServiceQuery(1586, Protocol::UDP), Service::Name("ibm-abtact")),
        (ServiceQuery(4189, Protocol::TCP), Service::Name("pcep")),
        (ServiceQuery(1953, Protocol::UDP), Service::Name("rapidbase")),
        (ServiceQuery(5631, Protocol::UDP), Service::Name("pcanywheredata")),
        (ServiceQuery(5133, Protocol::TCP), Service::Name("nbt-pc")),
        (ServiceQuery(2707, Protocol::TCP), Service::Name("emcsymapiport")),
        (ServiceQuery(9090, Protocol::TCP), Service::Name("zeus-admin")),
        (ServiceQuery(3127, Protocol::UDP), Service::Name("ctx-bridge")),
        (ServiceQuery(1023, Protocol::TCP), Service::Name("netvenuechat")),
        (ServiceQuery(2141, Protocol::TCP), Service::Name("ias-admind")),
        (ServiceQuery(3812, Protocol::TCP), Service::Name("neto-wol-server")),
        (ServiceQuery(1770, Protocol::TCP), Service::Name("bmc-net-svc")),
        (ServiceQuery(5416, Protocol::TCP), Service::Name("sns-gateway")),
        (ServiceQuery(5027, Protocol::UDP), Service::Name("strexec-s")),
        (ServiceQuery(4999, Protocol::UDP), Service::Name("hfcs-manager")),
        (ServiceQuery(27019, Protocol::TCP), Service::Name("mongod")),
        (ServiceQuery(1073, Protocol::UDP), Service::Name("bridgecontrol")),
        (ServiceQuery(48000, Protocol::TCP), Service::Name("nimcontroller")),
        (ServiceQuery(4087, Protocol::TCP), Service::Name("applusservice")),
        (ServiceQuery(1672, Protocol::UDP), Service::Name("netview-aix-12")),
        (ServiceQuery(1057, Protocol::UDP), Service::Name("startron")),
        (ServiceQuery(16367, Protocol::UDP), Service::Name("netserialext3")),
        (ServiceQuery(6480, Protocol::TCP), Service::Name("sun-sr-http")),
        (ServiceQuery(3166, Protocol::TCP), Service::Name("sossecollector")),
        (ServiceQuery(134, Protocol::UDP), Service::Name("ingres-net")),
        (ServiceQuery(3090, Protocol::UDP), Service::Name("stss")),
        (ServiceQuery(6485, Protocol::TCP), Service::Name("sun-sr-iiop")),
        (ServiceQuery(491, Protocol::TCP), Service::Name("go-login")),
        (ServiceQuery(3141, Protocol::UDP), Service::Name("vmodem")),
        (ServiceQuery(85, Protocol::UDP), Service::Name("mit-ml-dev")),
        (ServiceQuery(3155, Protocol::TCP), Service::Name("jpegmpeg")),
        (ServiceQuery(8057, Protocol::UDP), Service::Name("senomix06")),
        (ServiceQuery(3698, Protocol::TCP), Service::Name("sagectlpanel")),
        (ServiceQuery(26000, Protocol::UDP), Service::Name("quake")),
        (ServiceQuery(642, Protocol::UDP), Service::Name("esro-emsdp")),
        (ServiceQuery(12000, Protocol::TCP), Service::Name("cce4x")),
        (ServiceQuery(4725, Protocol::TCP), Service::Name("truckstar")),
        (ServiceQuery(2782, Protocol::UDP), Service::Name("everydayrc")),
        (ServiceQuery(6801, Protocol::TCP), Service::Name("acnet")),
        (ServiceQuery(522, Protocol::TCP), Service::Name("ulp")),
        (ServiceQuery(261, Protocol::UDP), Service::Name("nsiiops")),
        (ServiceQuery(6623, Protocol::UDP), Service::Name("ktelnet")),
        (ServiceQuery(2095, Protocol::UDP), Service::Name("nbx-ser")),
        (ServiceQuery(1373, Protocol::UDP), Service::Name("chromagrafx")),
        (ServiceQuery(3851, Protocol::UDP), Service::Name("spectraport")),
        (ServiceQuery(5746, Protocol::UDP), Service::Name("fcopys-server")),
        (ServiceQuery(2542, Protocol::UDP), Service::Name("udrawgraph")),
        (ServiceQuery(2622, Protocol::TCP), Service::Name("metricadbc")),
        (ServiceQuery(470, Protocol::TCP), Service::Name("scx-proxy")),
        (ServiceQuery(7324, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(475, Protocol::UDP), Service::Name("tcpnethaspsrv")),
        (ServiceQuery(5989, Protocol::UDP), Service::Name("wbem-https")),
        (ServiceQuery(3244, Protocol::UDP), Service::Name("onesaf")),
        (ServiceQuery(2643, Protocol::UDP), Service::Name("gte-samp")),
        (ServiceQuery(12300, Protocol::TCP), Service::Name("linogridengine")),
        (ServiceQuery(6580, Protocol::TCP), Service::Name("parsec-master")),
        (ServiceQuery(3376, Protocol::UDP), Service::Name("cdbroker")),
        (ServiceQuery(4231, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2498, Protocol::UDP), Service::Name("odn-castraq")),
        (ServiceQuery(42508, Protocol::TCP), Service::Name("candp")),
        (ServiceQuery(7690, Protocol::TCP), Service::Name("sovd")),
        (ServiceQuery(2974, Protocol::UDP), Service::Name("signal")),
        (ServiceQuery(595, Protocol::TCP), Service::Name("cab-protocol")),
        (ServiceQuery(16360, Protocol::TCP), Service::Name("netserialext1")),
        (ServiceQuery(4600, Protocol::TCP), Service::Name("piranha1")),
        (ServiceQuery(632, Protocol::UDP), Service::Name("bmpp")),
        (ServiceQuery(3922, Protocol::TCP), Service::Name("sor-update")),
        (ServiceQuery(49150, Protocol::TCP), Service::Name("inspider")),
        (ServiceQuery(1025, Protocol::UDP), Service::Name("blackjack")),
        (ServiceQuery(1636, Protocol::UDP), Service::Name("isdc")),
        (ServiceQuery(5087, Protocol::TCP), Service::Name("biotic")),
        (ServiceQuery(13782, Protocol::UDP), Service::Name("bpcd")),
        (ServiceQuery(3437, Protocol::UDP), Service::Name("autocueds")),
        (ServiceQuery(1643, Protocol::UDP), Service::Name("isis-ambc")),
        (ServiceQuery(5243, Protocol::TCP), Service::Name("xycstatus")),
        (ServiceQuery(6382, Protocol::UDP), Service::Name("metatude-mds")),
        (ServiceQuery(6964, Protocol::UDP), Service::Name("swismgr2")),
        (ServiceQuery(3281, Protocol::UDP), Service::Name("sysopt")),
        (ServiceQuery(3155, Protocol::UDP), Service::Name("jpegmpeg")),
        (ServiceQuery(1192, Protocol::UDP), Service::Name("caids-sensor")),
        (ServiceQuery(6543, Protocol::UDP), Service::Name("lds-distrib")),
        (ServiceQuery(7491, Protocol::TCP), Service::Name("telops-lmd")),
        (ServiceQuery(4344, Protocol::TCP), Service::Name("vinainstall")),
        (ServiceQuery(775, Protocol::TCP), Service::Name("entomb")),
        (ServiceQuery(1732, Protocol::TCP), Service::Name("proxim")),
        (ServiceQuery(2672, Protocol::TCP), Service::Name("nhserver")),
        (ServiceQuery(44443, Protocol::TCP), Service::Name("coldfusion-auth")),
        (ServiceQuery(1519, Protocol::TCP), Service::Name("vpvc")),
        (ServiceQuery(2249, Protocol::UDP), Service::Name("rfmp")),
        (ServiceQuery(46999, Protocol::UDP), Service::Name("mediabox")),
        (ServiceQuery(2035, Protocol::TCP), Service::Name("imsldoc")),
        (ServiceQuery(1841, Protocol::TCP), Service::Name("netopia-vo3")),
        (ServiceQuery(3406, Protocol::TCP), Service::Name("nokia-ann-ch2")),
        (ServiceQuery(3910, Protocol::UDP), Service::Name("prnrequest")),
        (ServiceQuery(4017, Protocol::UDP), Service::Name("talarian-mcast3")),
        (ServiceQuery(6145, Protocol::TCP), Service::Name("statsci2-lm")),
        (ServiceQuery(2698, Protocol::TCP), Service::Name("mck-ivpip")),
        (ServiceQuery(7348, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(23400, Protocol::UDP), Service::Name("novar-dbase")),
        (ServiceQuery(41, Protocol::TCP), Service::Name("graphics")),
        (ServiceQuery(7172, Protocol::TCP), Service::Name("metalbend")),
        (ServiceQuery(2394, Protocol::TCP), Service::Name("ms-olap2")),
        (ServiceQuery(4827, Protocol::TCP), Service::Name("htcp")),
        (ServiceQuery(1938, Protocol::TCP), Service::Name("jwclient")),
        (ServiceQuery(2495, Protocol::UDP), Service::Name("fast-rem-serv")),
        (ServiceQuery(1902, Protocol::TCP), Service::Name("fjicl-tep-b")),
        (ServiceQuery(1312, Protocol::TCP), Service::Name("sti-envision")),
        (ServiceQuery(3232, Protocol::UDP), Service::Name("mdtp")),
        (ServiceQuery(5282, Protocol::TCP), Service::Name("transmit-port")),
        (ServiceQuery(3728, Protocol::TCP), Service::Name("e-woa")),
        (ServiceQuery(3021, Protocol::UDP), Service::Name("agriserver")),
        (ServiceQuery(23003, Protocol::UDP), Service::Name("inovaport4")),
        (ServiceQuery(5272, Protocol::TCP), Service::Name("pk")),
        (ServiceQuery(2840, Protocol::UDP), Service::Name("l3-exprt")),
        (ServiceQuery(3131, Protocol::TCP), Service::Name("netbookmark")),
        (ServiceQuery(2584, Protocol::UDP), Service::Name("cyaserv")),
        (ServiceQuery(8910, Protocol::UDP), Service::Name("manyone-http")),
        (ServiceQuery(3633, Protocol::TCP), Service::Name("wacp")),
        (ServiceQuery(5713, Protocol::TCP), Service::Name("proshareaudio")),
        (ServiceQuery(8911, Protocol::UDP), Service::Name("manyone-xml")),
        (ServiceQuery(2416, Protocol::TCP), Service::Name("rmtserver")),
        (ServiceQuery(3858, Protocol::TCP), Service::Name("trap-port-mom")),
        (ServiceQuery(2979, Protocol::UDP), Service::Name("h263-video")),
        (ServiceQuery(2767, Protocol::TCP), Service::Name("uadtc")),
        (ServiceQuery(1361, Protocol::UDP), Service::Name("linx")),
        (ServiceQuery(6421, Protocol::TCP), Service::Name("nim-wan")),
        (ServiceQuery(7937, Protocol::TCP), Service::Name("nsrexecd")),
        (ServiceQuery(13218, Protocol::TCP), Service::Name("emc-vcas-tcp")),
        (ServiceQuery(3911, Protocol::UDP), Service::Name("prnstatus")),
        (ServiceQuery(14034, Protocol::TCP), Service::Name("sage-best-com2")),
        (ServiceQuery(1740, Protocol::TCP), Service::Name("encore")),
        (ServiceQuery(3524, Protocol::TCP), Service::Name("ecmport")),
        (ServiceQuery(3438, Protocol::UDP), Service::Name("spiral-admin")),
        (ServiceQuery(4178, Protocol::TCP), Service::Name("storman")),
        (ServiceQuery(1886, Protocol::TCP), Service::Name("leoip")),
        (ServiceQuery(2277, Protocol::UDP), Service::Name("bluectrlproxy")),
        (ServiceQuery(10050, Protocol::TCP), Service::Name("zabbix-agent")),
        (ServiceQuery(4389, Protocol::TCP), Service::Name("xandros-cms")),
        (ServiceQuery(674, Protocol::UDP), Service::Name("acap")),
        (ServiceQuery(2813, Protocol::TCP), Service::Name("llm-pass")),
        (ServiceQuery(4115, Protocol::TCP), Service::Name("cds")),
        (ServiceQuery(4402, Protocol::UDP), Service::Name("ds-clnt")),
        (ServiceQuery(1297, Protocol::TCP), Service::Name("sdproxy")),
        (ServiceQuery(1628, Protocol::TCP), Service::Name("lontalk-norm")),
        (ServiceQuery(4868, Protocol::UDP), Service::Name("phrelay")),
        (ServiceQuery(7357, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(4266, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(44553, Protocol::TCP), Service::Name("rbr-debug")),
        (ServiceQuery(8612, Protocol::TCP), Service::Name("canon-bjnp2")),
        (ServiceQuery(676, Protocol::TCP), Service::Name("vpps-via")),
        (ServiceQuery(31620, Protocol::TCP), Service::Name("lm-mon")),
        (ServiceQuery(2050, Protocol::TCP), Service::Name("av-emb-config")),
        (ServiceQuery(4132, Protocol::UDP), Service::Name("nuts_dem")),
        (ServiceQuery(2637, Protocol::TCP), Service::Name("imdocsvc")),
        (ServiceQuery(2493, Protocol::TCP), Service::Name("talarian-mqs")),
        (ServiceQuery(5729, Protocol::TCP), Service::Name("openmail")),
        (ServiceQuery(2198, Protocol::TCP), Service::Name("onehome-remote")),
        (ServiceQuery(9877, Protocol::TCP), Service::Name("x510")),
        (ServiceQuery(2302, Protocol::TCP), Service::Name("binderysupport")),
        (ServiceQuery(32775, Protocol::UDP), Service::Name("sometimes-rpc14")),
        (ServiceQuery(3601, Protocol::UDP), Service::Name("visinet-gui")),
        (ServiceQuery(128, Protocol::UDP), Service::Name("gss-xlicen")),
        (ServiceQuery(1764, Protocol::UDP), Service::Name("cft-3")),
        (ServiceQuery(22350, Protocol::TCP), Service::Name("CodeMeter")),
        (ServiceQuery(4354, Protocol::TCP), Service::Name("qsnet-trans")),
        (ServiceQuery(4500, Protocol::TCP), Service::Name("sae-urn")),
        (ServiceQuery(22763, Protocol::TCP), Service::Name("talikaserver")),
        (ServiceQuery(1513, Protocol::UDP), Service::Name("fujitsu-dtc")),
        (ServiceQuery(2876, Protocol::UDP), Service::Name("sps-tunnel")),
        (ServiceQuery(5730, Protocol::UDP), Service::Name("unieng")),
        (ServiceQuery(6066, Protocol::UDP), Service::Name("ewctsp")),
        (ServiceQuery(7789, Protocol::TCP), Service::Name("office-tools")),
        (ServiceQuery(4991, Protocol::UDP), Service::Name("vrt")),
        (ServiceQuery(13, Protocol::TCP), Service::Name("daytime")),
        (ServiceQuery(1626, Protocol::TCP), Service::Name("shockwave")),
        (ServiceQuery(25006, Protocol::TCP), Service::Name("icl-twobase7")),
        (ServiceQuery(3190, Protocol::TCP), Service::Name("csvr-proxy")),
        (ServiceQuery(3466, Protocol::UDP), Service::Name("workflow")),
        (ServiceQuery(5802, Protocol::TCP), Service::Name("vnc-http-2")),
        (ServiceQuery(40853, Protocol::TCP), Service::Name("ortec-disc")),
        (ServiceQuery(6768, Protocol::UDP), Service::Name("bmc-perf-mgrd")),
        (ServiceQuery(492, Protocol::UDP), Service::Name("ticf-1")),
        (ServiceQuery(4072, Protocol::UDP), Service::Name("zieto-sock")),
        (ServiceQuery(4950, Protocol::TCP), Service::Name("sybasesrvmon")),
        (ServiceQuery(380, Protocol::TCP), Service::Name("is99s")),
        (ServiceQuery(8423, Protocol::TCP), Service::Name("aritts")),
        (ServiceQuery(3347, Protocol::UDP), Service::Name("phoenix-rpc")),
        (ServiceQuery(4135, Protocol::TCP), Service::Name("cl-db-attach")),
        (ServiceQuery(3602, Protocol::TCP), Service::Name("infiniswitchcl")),
        (ServiceQuery(23401, Protocol::TCP), Service::Name("novar-alarm")),
        (ServiceQuery(2431, Protocol::UDP), Service::Name("venus-se")),
        (ServiceQuery(1171, Protocol::TCP), Service::Name("atc-appserver")),
        (ServiceQuery(2513, Protocol::TCP), Service::Name("citrixadmin")),
        (ServiceQuery(2479, Protocol::UDP), Service::Name("ssm-els")),
        (ServiceQuery(5299, Protocol::UDP), Service::Name("nlg-data")),
        (ServiceQuery(3835, Protocol::TCP), Service::Name("spectardb")),
        (ServiceQuery(530, Protocol::TCP), Service::Name("courier")),
        (ServiceQuery(1633, Protocol::UDP), Service::Name("pammrpc")),
        (ServiceQuery(5320, Protocol::TCP), Service::Name("bsfserver-zn")),
        (ServiceQuery(4160, Protocol::UDP), Service::Name("jini-discovery")),
        (ServiceQuery(3385, Protocol::TCP), Service::Name("qnxnetman")),
        (ServiceQuery(2500, Protocol::TCP), Service::Name("rtsserv")),
        (ServiceQuery(3507, Protocol::UDP), Service::Name("nesh-broker")),
        (ServiceQuery(4429, Protocol::TCP), Service::Name("omviagent")),
        (ServiceQuery(2829, Protocol::TCP), Service::Name("silkp1")),
        (ServiceQuery(689, Protocol::UDP), Service::Name("nmap")),
        (ServiceQuery(2583, Protocol::UDP), Service::Name("mon")),
        (ServiceQuery(6421, Protocol::UDP), Service::Name("nim-wan")),
        (ServiceQuery(6049, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(2882, Protocol::TCP), Service::Name("ndtp")),
        (ServiceQuery(421, Protocol::TCP), Service::Name("ariel2")),
        (ServiceQuery(6322, Protocol::TCP), Service::Name("emp-server2")),
        (ServiceQuery(2742, Protocol::UDP), Service::Name("tsb2")),
        (ServiceQuery(3099, Protocol::TCP), Service::Name("chmd")),
        (ServiceQuery(1772, Protocol::TCP), Service::Name("essweb-gw")),
        (ServiceQuery(434, Protocol::TCP), Service::Name("mobileip-agent")),
        (ServiceQuery(6486, Protocol::TCP), Service::Name("sun-sr-iiops")),
        (ServiceQuery(3374, Protocol::UDP), Service::Name("cluster-disc")),
        (ServiceQuery(5560, Protocol::TCP), Service::Name("isqlplus")),
        (ServiceQuery(5, Protocol::TCP), Service::Name("rje")),
        (ServiceQuery(651, Protocol::UDP), Service::Name("ieee-mms")),
        (ServiceQuery(4703, Protocol::TCP), Service::Name("npqes-test")),
        (ServiceQuery(8891, Protocol::TCP), Service::Name("ddi-tcp-4")),
        (ServiceQuery(45185, Protocol::TCP), Service::Name("witsnet")),
        (ServiceQuery(2329, Protocol::TCP), Service::Name("nvd")),
        (ServiceQuery(22273, Protocol::UDP), Service::Name("wnn6")),
        (ServiceQuery(2828, Protocol::UDP), Service::Name("itm-lm")),
        (ServiceQuery(5553, Protocol::UDP), Service::Name("sgi-eventmond")),
        (ServiceQuery(1309, Protocol::TCP), Service::Name("jtag-server")),
        (ServiceQuery(6655, Protocol::TCP), Service::Name("pcs-sf-ui-man")),
        (ServiceQuery(3243, Protocol::UDP), Service::Name("timelot")),
        (ServiceQuery(3312, Protocol::TCP), Service::Name("appman-server")),
        (ServiceQuery(3567, Protocol::UDP), Service::Name("oap")),
        (ServiceQuery(7006, Protocol::TCP), Service::Name("afs3-errors")),
        (ServiceQuery(3017, Protocol::TCP), Service::Name("event_listener")),
        (ServiceQuery(5362, Protocol::TCP), Service::Name("serverwsd2")),
        (ServiceQuery(4216, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(5350, Protocol::UDP), Service::Name("nat-pmp-status")),
        (ServiceQuery(631, Protocol::TCP), Service::Name("ipp")),
        (ServiceQuery(3392, Protocol::UDP), Service::Name("efi-lm")),
        (ServiceQuery(4949, Protocol::TCP), Service::Name("munin")),
        (ServiceQuery(36001, Protocol::UDP), Service::Name("allpeers")),
        (ServiceQuery(13223, Protocol::UDP), Service::Name("powwow-client")),
        (ServiceQuery(1974, Protocol::TCP), Service::Name("drp")),
        (ServiceQuery(1968, Protocol::UDP), Service::Name("lipsinc")),
        (ServiceQuery(715, Protocol::UDP), Service::Name("iris-lwz")),
        (ServiceQuery(6363, Protocol::TCP), Service::Name("ndn")),
        (ServiceQuery(9105, Protocol::UDP), Service::Name("xadmin")),
        (ServiceQuery(5520, Protocol::TCP), Service::Name("sdlog")),
        (ServiceQuery(2069, Protocol::TCP), Service::Name("event-port")),
        (ServiceQuery(2319, Protocol::TCP), Service::Name("infolibria")),
        (ServiceQuery(7040, Protocol::TCP), Service::Name("quest-disc")),
        (ServiceQuery(1843, Protocol::TCP), Service::Name("netopia-vo5")),
        (ServiceQuery(1297, Protocol::UDP), Service::Name("sdproxy")),
        (ServiceQuery(3135, Protocol::TCP), Service::Name("peerbook-port")),
        (ServiceQuery(5360, Protocol::UDP), Service::Name("ms-sideshow")),
        (ServiceQuery(777, Protocol::TCP), Service::Name("multiling-http")),
        (ServiceQuery(2601, Protocol::TCP), Service::Name("zebra")),
        (ServiceQuery(5004, Protocol::TCP), Service::Name("avt-profile-1")),
        (ServiceQuery(318, Protocol::TCP), Service::Name("pkix-timestamp")),
        (ServiceQuery(4253, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(3393, Protocol::UDP), Service::Name("d2k-tapestry1")),
        (ServiceQuery(4664, Protocol::TCP), Service::Name("ems")),
        (ServiceQuery(7236, Protocol::TCP), Service::Name("display")),
        (ServiceQuery(44444, Protocol::TCP), Service::Name("cognex-dataman")),
        (ServiceQuery(1936, Protocol::UDP), Service::Name("jetcmeserver")),
        (ServiceQuery(10113, Protocol::UDP), Service::Name("netiq-endpoint")),
        (ServiceQuery(42510, Protocol::UDP), Service::Name("caerpc")),
        (ServiceQuery(5859, Protocol::UDP), Service::Name("wherehoo")),
        (ServiceQuery(5250, Protocol::TCP), Service::Name("soagateway")),
        (ServiceQuery(3422, Protocol::TCP), Service::Name("rusb-sys-port")),
        (ServiceQuery(5684, Protocol::TCP), Service::Name("coaps")),
        (ServiceQuery(4037, Protocol::UDP), Service::Name("ravehd")),
        (ServiceQuery(2638, Protocol::UDP), Service::Name("sybaseanywhere")),
        (ServiceQuery(5903, Protocol::TCP), Service::Name("vnc-3")),
        (ServiceQuery(2260, Protocol::TCP), Service::Name("apc-2260")),
        (ServiceQuery(511, Protocol::UDP), Service::Name("passgo")),
        (ServiceQuery(81, Protocol::UDP), Service::Name("hosts2-ns")),
        (ServiceQuery(8954, Protocol::TCP), Service::Name("cumulus-admin")),
        (ServiceQuery(9292, Protocol::UDP), Service::Name("armtechdaemon")),
        (ServiceQuery(2361, Protocol::TCP), Service::Name("tl1")),
        (ServiceQuery(46999, Protocol::TCP), Service::Name("mediabox")),
        (ServiceQuery(1658, Protocol::TCP), Service::Name("sixnetudr")),
        (ServiceQuery(2323, Protocol::TCP), Service::Name("3d-nfsd")),
        (ServiceQuery(3232, Protocol::TCP), Service::Name("mdtp")),
        (ServiceQuery(515, Protocol::TCP), Service::Name("printer")),
        (ServiceQuery(3485, Protocol::UDP), Service::Name("celatalk")),
        (ServiceQuery(3095, Protocol::UDP), Service::Name("panasas")),
        (ServiceQuery(4446, Protocol::TCP), Service::Name("n1-fwp")),
        (ServiceQuery(1363, Protocol::UDP), Service::Name("ndm-requester")),
        (ServiceQuery(27007, Protocol::UDP), Service::Name("flex-lm")),
        (ServiceQuery(4711, Protocol::TCP), Service::Name("trinity-dist")),
        (ServiceQuery(847, Protocol::TCP), Service::Name("dhcp-failover2")),
        (ServiceQuery(1245, Protocol::TCP), Service::Name("isbconference2")),
        (ServiceQuery(619, Protocol::UDP), Service::Name("compaq-evm")),
        (ServiceQuery(7775, Protocol::TCP), Service::Name("inetfs")),
        (ServiceQuery(7624, Protocol::UDP), Service::Name("indi")),
        (ServiceQuery(2588, Protocol::TCP), Service::Name("privilege")),
        (ServiceQuery(6101, Protocol::TCP), Service::Name("backupexec")),
        (ServiceQuery(2793, Protocol::UDP), Service::Name("initlsmsad")),
        (ServiceQuery(1107, Protocol::UDP), Service::Name("isoipsigport-2")),
        (ServiceQuery(2641, Protocol::UDP), Service::Name("hdl-srv")),
        (ServiceQuery(10160, Protocol::TCP), Service::Name("qb-db-server")),
        (ServiceQuery(638, Protocol::TCP), Service::Name("mcns-sec")),
        (ServiceQuery(4594, Protocol::TCP), Service::Name("ias-session")),
        (ServiceQuery(2420, Protocol::TCP), Service::Name("dslremote-mgmt")),
        (ServiceQuery(1782, Protocol::TCP), Service::Name("hp-hcip")),
        (ServiceQuery(9084, Protocol::UDP), Service::Name("aurora")),
        (ServiceQuery(8380, Protocol::UDP), Service::Name("cruise-update")),
        (ServiceQuery(4018, Protocol::TCP), Service::Name("talarian-mcast4")),
        (ServiceQuery(1592, Protocol::TCP), Service::Name("commonspace")),
        (ServiceQuery(3625, Protocol::TCP), Service::Name("volley")),
        (ServiceQuery(669, Protocol::TCP), Service::Name("meregister")),
        (ServiceQuery(4785, Protocol::TCP), Service::Name("cncp")),
        (ServiceQuery(888, Protocol::TCP), Service::Name("accessbuilder")),
        (ServiceQuery(347, Protocol::TCP), Service::Name("fatserv")),
        (ServiceQuery(2726, Protocol::TCP), Service::Name("tams")),
        (ServiceQuery(9279, Protocol::UDP), Service::Name("pegasus-ctl")),
        (ServiceQuery(1713, Protocol::UDP), Service::Name("conferencetalk")),
        (ServiceQuery(7789, Protocol::UDP), Service::Name("office-tools")),
        (ServiceQuery(157, Protocol::TCP), Service::Name("knet-cmp")),
        (ServiceQuery(4744, Protocol::UDP), Service::Name("ifsp")),
        (ServiceQuery(3708, Protocol::TCP), Service::Name("sun-as-iiops")),
        (ServiceQuery(3770, Protocol::TCP), Service::Name("cindycollab")),
        (ServiceQuery(8880, Protocol::UDP), Service::Name("cddbp-alt")),
        (ServiceQuery(6962, Protocol::UDP), Service::Name("jmevt2")),
        (ServiceQuery(3293, Protocol::TCP), Service::Name("fg-fps")),
        (ServiceQuery(3998, Protocol::UDP), Service::Name("remoteanything")),
        (ServiceQuery(5603, Protocol::TCP), Service::Name("a1-bs")),
        (ServiceQuery(2576, Protocol::UDP), Service::Name("tclprodebugger")),
        (ServiceQuery(2461, Protocol::UDP), Service::Name("qadmifoper")),
        (ServiceQuery(1832, Protocol::UDP), Service::Name("tht-treasure")),
        (ServiceQuery(2538, Protocol::UDP), Service::Name("vnwk-prapi")),
        (ServiceQuery(31016, Protocol::TCP), Service::Name("ka-sddp")),
        (ServiceQuery(1544, Protocol::TCP), Service::Name("aspeclmd")),
        (ServiceQuery(7542, Protocol::UDP), Service::Name("saratoga")),
        (ServiceQuery(6062, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(21846, Protocol::UDP), Service::Name("netspeak-is")),
        (ServiceQuery(6080, Protocol::TCP), Service::Name("gue")),
        (ServiceQuery(86, Protocol::TCP), Service::Name("mfcobol")),
        (ServiceQuery(1580, Protocol::TCP), Service::Name("tn-tl-r1")),
        (ServiceQuery(4426, Protocol::UDP), Service::Name("beacon-port-2")),
        (ServiceQuery(2670, Protocol::UDP), Service::Name("tve-announce")),
        (ServiceQuery(9996, Protocol::TCP), Service::Name("palace-5")),
        (ServiceQuery(1811, Protocol::UDP), Service::Name("scientia-sdb")),
        (ServiceQuery(4545, Protocol::TCP), Service::Name("worldscores")),
        (ServiceQuery(2465, Protocol::UDP), Service::Name("lbm")),
        (ServiceQuery(2435, Protocol::UDP), Service::Name("optilogic")),
        (ServiceQuery(3669, Protocol::TCP), Service::Name("casanswmgmt")),
        (ServiceQuery(10880, Protocol::TCP), Service::Name("bveapi")),
        (ServiceQuery(2065, Protocol::TCP), Service::Name("dlsrpn")),
        (ServiceQuery(2390, Protocol::UDP), Service::Name("rsmtp")),
        (ServiceQuery(1667, Protocol::TCP), Service::Name("netview-aix-7")),
        (ServiceQuery(1842, Protocol::TCP), Service::Name("netopia-vo4")),
        (ServiceQuery(2114, Protocol::TCP), Service::Name("newheights")),
        (ServiceQuery(2408, Protocol::UDP), Service::Name("optimanet")),
        (ServiceQuery(2880, Protocol::TCP), Service::Name("synapse")),
        (ServiceQuery(8080, Protocol::TCP), Service::Name("http-proxy")),
        (ServiceQuery(3553, Protocol::TCP), Service::Name("rbr-discovery")),
        (ServiceQuery(8377, Protocol::TCP), Service::Name("cruise-swroute")),
        (ServiceQuery(3915, Protocol::UDP), Service::Name("agcat")),
        (ServiceQuery(2353, Protocol::TCP), Service::Name("pspserver")),
        (ServiceQuery(4029, Protocol::UDP), Service::Name("ip-qsig")),
        (ServiceQuery(3819, Protocol::UDP), Service::Name("epl-slp")),
        (ServiceQuery(3182, Protocol::UDP), Service::Name("bmcpatrolrnvu")),
        (ServiceQuery(3903, Protocol::TCP), Service::Name("charsetmgr")),
        (ServiceQuery(607, Protocol::UDP), Service::Name("nqs")),
        (ServiceQuery(4294, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2848, Protocol::TCP), Service::Name("amt-blc-port")),
        (ServiceQuery(4444, Protocol::UDP), Service::Name("krb524")),
        (ServiceQuery(3800, Protocol::UDP), Service::Name("pwgpsi")),
        (ServiceQuery(5540, Protocol::UDP), Service::Name("sdxauthd")),
        (ServiceQuery(2921, Protocol::UDP), Service::Name("cesdcdman")),
        (ServiceQuery(3761, Protocol::TCP), Service::Name("gsakmp")),
        (ServiceQuery(2564, Protocol::TCP), Service::Name("hp-3000-telnet")),
        (ServiceQuery(2291, Protocol::UDP), Service::Name("eapsp")),
        (ServiceQuery(19541, Protocol::TCP), Service::Name("jcp")),
        (ServiceQuery(2986, Protocol::UDP), Service::Name("stonefalls")),
        (ServiceQuery(7393, Protocol::TCP), Service::Name("nfoldman")),
        (ServiceQuery(2876, Protocol::TCP), Service::Name("sps-tunnel")),
        (ServiceQuery(6029, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(5229, Protocol::TCP), Service::Name("jaxflow")),
        (ServiceQuery(3612, Protocol::TCP), Service::Name("hp-dataprotect")),
        (ServiceQuery(17220, Protocol::TCP), Service::Name("avtp")),
        (ServiceQuery(4095, Protocol::TCP), Service::Name("xtgui")),
        (ServiceQuery(3279, Protocol::TCP), Service::Name("admind")),
        (ServiceQuery(44, Protocol::TCP), Service::Name("mpm-flags")),
        (ServiceQuery(1883, Protocol::TCP), Service::Name("mqtt")),
        (ServiceQuery(4739, Protocol::UDP), Service::Name("ipfix")),
        (ServiceQuery(5728, Protocol::UDP), Service::Name("io-dist-group")),
        (ServiceQuery(4026, Protocol::TCP), Service::Name("as-debug")),
        (ServiceQuery(24004, Protocol::UDP), Service::Name("med-ovw")),
        (ServiceQuery(2428, Protocol::UDP), Service::Name("ott")),
        (ServiceQuery(2503, Protocol::UDP), Service::Name("nms-dpnss")),
        (ServiceQuery(4014, Protocol::UDP), Service::Name("taiclock")),
        (ServiceQuery(1679, Protocol::TCP), Service::Name("darcorp-lm")),
        (ServiceQuery(8732, Protocol::UDP), Service::Name("dtp-net")),
        (ServiceQuery(2177, Protocol::TCP), Service::Name("qwave")),
        (ServiceQuery(3620, Protocol::UDP), Service::Name("ep-pcp")),
        (ServiceQuery(4355, Protocol::UDP), Service::Name("qsnet-workst")),
        (ServiceQuery(45, Protocol::UDP), Service::Name("mpm")),
        (ServiceQuery(9212, Protocol::TCP), Service::Name("serverviewdbms")),
        (ServiceQuery(3674, Protocol::TCP), Service::Name("wininstall-ipc")),
        (ServiceQuery(3386, Protocol::UDP), Service::Name("gprs-sig")),
        (ServiceQuery(348, Protocol::TCP), Service::Name("csi-sgwp")),
        (ServiceQuery(1400, Protocol::TCP), Service::Name("cadkey-tablet")),
        (ServiceQuery(2094, Protocol::UDP), Service::Name("nbx-au")),
        (ServiceQuery(1149, Protocol::UDP), Service::Name("bvtsonar")),
        (ServiceQuery(7101, Protocol::UDP), Service::Name("elcn")),
        (ServiceQuery(3285, Protocol::UDP), Service::Name("plato")),
        (ServiceQuery(3449, Protocol::UDP), Service::Name("hotu-chat")),
        (ServiceQuery(3688, Protocol::TCP), Service::Name("simple-push-s")),
        (ServiceQuery(9005, Protocol::TCP), Service::Name("golem")),
        (ServiceQuery(271, Protocol::TCP), Service::Name("pt-tls")),
        (ServiceQuery(1389, Protocol::TCP), Service::Name("iclpv-dm")),
        (ServiceQuery(8022, Protocol::TCP), Service::Name("oa-system")),
        (ServiceQuery(16020, Protocol::TCP), Service::Name("jwpc")),
        (ServiceQuery(5683, Protocol::UDP), Service::Name("coap")),
        (ServiceQuery(741, Protocol::TCP), Service::Name("netgw")),
        (ServiceQuery(2546, Protocol::UDP), Service::Name("vytalvaultbrtp")),
        (ServiceQuery(2923, Protocol::UDP), Service::Name("wta-wsp-wtp-s")),
        (ServiceQuery(484, Protocol::TCP), Service::Name("integra-sme")),
        (ServiceQuery(4701, Protocol::UDP), Service::Name("netxms-mgmt")),
        (ServiceQuery(3424, Protocol::TCP), Service::Name("xtrms")),
        (ServiceQuery(1556, Protocol::TCP), Service::Name("veritas_pbx")),
        (ServiceQuery(1886, Protocol::UDP), Service::Name("leoip")),
        (ServiceQuery(9102, Protocol::TCP), Service::Name("jetdirect")),
        (ServiceQuery(40843, Protocol::TCP), Service::Name("csccfirewall")),
        (ServiceQuery(8769, Protocol::TCP), Service::Name("oktaauthenticat")),
        (ServiceQuery(2372, Protocol::TCP), Service::Name("lanmessenger")),
        (ServiceQuery(4447, Protocol::UDP), Service::Name("n1-rmgmt")),
        (ServiceQuery(207, Protocol::TCP), Service::Name("at-7")),
        (ServiceQuery(5598, Protocol::TCP), Service::Name("mctfeed")),
        (ServiceQuery(4484, Protocol::TCP), Service::Name("hpssmgmt")),
        (ServiceQuery(8091, Protocol::TCP), Service::Name("jamlink")),
        (ServiceQuery(2740, Protocol::UDP), Service::Name("alarm")),
        (ServiceQuery(1633, Protocol::TCP), Service::Name("pammrpc")),
        (ServiceQuery(2846, Protocol::UDP), Service::Name("aimpp-hello")),
        (ServiceQuery(8201, Protocol::TCP), Service::Name("trivnet2")),
        (ServiceQuery(4258, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(9953, Protocol::TCP), Service::Name("acis")),
        (ServiceQuery(2901, Protocol::UDP), Service::Name("allstorcns")),
        (ServiceQuery(2989, Protocol::TCP), Service::Name("zarkov")),
        (ServiceQuery(5135, Protocol::TCP), Service::Name("erp-scale")),
        (ServiceQuery(2545, Protocol::TCP), Service::Name("sis-emt")),
        (ServiceQuery(6039, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(126, Protocol::TCP), Service::Name("unitary")),
        (ServiceQuery(1033, Protocol::TCP), Service::Name("netinfo")),
        (ServiceQuery(3229, Protocol::UDP), Service::Name("global-cd-port")),
        (ServiceQuery(3664, Protocol::UDP), Service::Name("ups-engine")),
        (ServiceQuery(424, Protocol::TCP), Service::Name("opc-job-track")),
        (ServiceQuery(10252, Protocol::TCP), Service::Name("apollo-relay")),
        (ServiceQuery(4050, Protocol::TCP), Service::Name("cisco-wafs")),
        (ServiceQuery(2841, Protocol::TCP), Service::Name("l3-ranger")),
        (ServiceQuery(3907, Protocol::TCP), Service::Name("imoguia-port")),
        (ServiceQuery(2072, Protocol::TCP), Service::Name("msync")),
        (ServiceQuery(2112, Protocol::UDP), Service::Name("idonix-metanet")),
        (ServiceQuery(3701, Protocol::TCP), Service::Name("netcelera")),
        (ServiceQuery(24249, Protocol::TCP), Service::Name("vista-4gl")),
        (ServiceQuery(5045, Protocol::TCP), Service::Name("osp")),
        (ServiceQuery(3948, Protocol::UDP), Service::Name("apdap")),
        (ServiceQuery(30832, Protocol::TCP), Service::Name("samsung-disc")),
        (ServiceQuery(6315, Protocol::UDP), Service::Name("scup-disc")),
        (ServiceQuery(4031, Protocol::UDP), Service::Name("suucp")),
        (ServiceQuery(3827, Protocol::TCP), Service::Name("netmpi")),
        (ServiceQuery(2261, Protocol::TCP), Service::Name("comotionmaster")),
        (ServiceQuery(5253, Protocol::TCP), Service::Name("kpdp")),
        (ServiceQuery(197, Protocol::UDP), Service::Name("dls")),
        (ServiceQuery(2305, Protocol::TCP), Service::Name("mt-scaleserver")),
        (ServiceQuery(4319, Protocol::TCP), Service::Name("fox-skytale")),
        (ServiceQuery(25903, Protocol::UDP), Service::Name("niprobe")),
        (ServiceQuery(8195, Protocol::UDP), Service::Name("blp2")),
        (ServiceQuery(4888, Protocol::TCP), Service::Name("xcap-portal")),
        (ServiceQuery(26263, Protocol::UDP), Service::Name("k3software-cli")),
        (ServiceQuery(3460, Protocol::UDP), Service::Name("edm-manager")),
        (ServiceQuery(3889, Protocol::TCP), Service::Name("dandv-tester")),
        (ServiceQuery(3391, Protocol::UDP), Service::Name("savant")),
        (ServiceQuery(2647, Protocol::UDP), Service::Name("syncserver")),
        (ServiceQuery(448, Protocol::UDP), Service::Name("ddm-ssl")),
        (ServiceQuery(6107, Protocol::UDP), Service::Name("etc-control")),
        (ServiceQuery(1686, Protocol::UDP), Service::Name("cvmon")),
        (ServiceQuery(2705, Protocol::TCP), Service::Name("sds-admin")),
        (ServiceQuery(6446, Protocol::TCP), Service::Name("mysql-proxy")),
        (ServiceQuery(1228, Protocol::TCP), Service::Name("florence")),
        (ServiceQuery(40023, Protocol::TCP), Service::Name("k-patentssensor")),
        (ServiceQuery(1196, Protocol::TCP), Service::Name("netmagic")),
        (ServiceQuery(6010, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(1340, Protocol::UDP), Service::Name("naap")),
        (ServiceQuery(1200, Protocol::UDP), Service::Name("scol")),
        (ServiceQuery(1506, Protocol::TCP), Service::Name("utcd")),
        (ServiceQuery(2085, Protocol::TCP), Service::Name("ada-cip")),
        (ServiceQuery(2221, Protocol::UDP), Service::Name("rockwell-csp1")),
        (ServiceQuery(9280, Protocol::TCP), Service::Name("pgps")),
        (ServiceQuery(3670, Protocol::TCP), Service::Name("smile")),
        (ServiceQuery(4259, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(7430, Protocol::TCP), Service::Name("xmpv7")),
        (ServiceQuery(53, Protocol::UDP), Service::Name("domain")),
        (ServiceQuery(4213, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(4453, Protocol::TCP), Service::Name("nssalertmgr")),
        (ServiceQuery(5304, Protocol::UDP), Service::Name("hacl-local")),
        (ServiceQuery(8442, Protocol::UDP), Service::Name("cybro-a-bus")),
        (ServiceQuery(6403, Protocol::UDP), Service::Name("boe-cachesvr")),
        (ServiceQuery(2056, Protocol::TCP), Service::Name("omnisky")),
        (ServiceQuery(1834, Protocol::TCP), Service::Name("ardusuni")),
        (ServiceQuery(1581, Protocol::TCP), Service::Name("mil-2045-47001")),
        (ServiceQuery(6444, Protocol::TCP), Service::Name("sge_qmaster")),
        (ServiceQuery(19541, Protocol::UDP), Service::Name("jcp")),
        (ServiceQuery(12121, Protocol::TCP), Service::Name("nupaper-ss")),
        (ServiceQuery(1833, Protocol::UDP), Service::Name("udpradio")),
        (ServiceQuery(2076, Protocol::UDP), Service::Name("newlixconfig")),
        (ServiceQuery(2437, Protocol::UDP), Service::Name("unicontrol")),
        (ServiceQuery(19411, Protocol::TCP), Service::Name("hp-sca")),
        (ServiceQuery(3383, Protocol::TCP), Service::Name("esp-lm")),
        (ServiceQuery(378, Protocol::TCP), Service::Name("dsETOS")),
        (ServiceQuery(2757, Protocol::TCP), Service::Name("cnrp")),
        (ServiceQuery(538, Protocol::UDP), Service::Name("gdomap")),
        (ServiceQuery(5431, Protocol::UDP), Service::Name("park-agent")),
        (ServiceQuery(2537, Protocol::TCP), Service::Name("upgrade")),
        (ServiceQuery(1401, Protocol::UDP), Service::Name("goldleaf-licman")),
        (ServiceQuery(2283, Protocol::UDP), Service::Name("lnvstatus")),
        (ServiceQuery(1724, Protocol::TCP), Service::Name("csbphone")),
        (ServiceQuery(10253, Protocol::TCP), Service::Name("eapol-relay")),
        (ServiceQuery(3716, Protocol::UDP), Service::Name("wv-csp-sms-cir")),
        (ServiceQuery(1912, Protocol::UDP), Service::Name("rhp-iibp")),
        (ServiceQuery(382, Protocol::TCP), Service::Name("hp-managed-node")),
        (ServiceQuery(5402, Protocol::UDP), Service::Name("mftp")),
        (ServiceQuery(1075, Protocol::UDP), Service::Name("rdrmshc")),
        (ServiceQuery(499, Protocol::UDP), Service::Name("iso-ill")),
        (ServiceQuery(1202, Protocol::UDP), Service::Name("caiccipc")),
        (ServiceQuery(4677, Protocol::UDP), Service::Name("bcs")),
        (ServiceQuery(7308, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(3451, Protocol::TCP), Service::Name("asam")),
        (ServiceQuery(3255, Protocol::TCP), Service::Name("semaphore")),
        (ServiceQuery(2021, Protocol::TCP), Service::Name("servexec")),
        (ServiceQuery(16021, Protocol::TCP), Service::Name("jwpc-bin")),
        (ServiceQuery(3904, Protocol::UDP), Service::Name("omnilink-port")),
        (ServiceQuery(2126, Protocol::TCP), Service::Name("pktcable-cops")),
        (ServiceQuery(5782, Protocol::UDP), Service::Name("3par-mgmt")),
        (ServiceQuery(3719, Protocol::UDP), Service::Name("itelserverport")),
        (ServiceQuery(4454, Protocol::UDP), Service::Name("nssagentmgr")),
        (ServiceQuery(2214, Protocol::UDP), Service::Name("rpi")),
        (ServiceQuery(2573, Protocol::UDP), Service::Name("trustestablish")),
        (ServiceQuery(7310, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(3179, Protocol::UDP), Service::Name("h2gf-w-2m")),
        (ServiceQuery(1959, Protocol::TCP), Service::Name("simp-all")),
        (ServiceQuery(5432, Protocol::UDP), Service::Name("postgresql")),
        (ServiceQuery(318, Protocol::UDP), Service::Name("pkix-timestamp")),
        (ServiceQuery(3104, Protocol::TCP), Service::Name("autocuelog")),
        (ServiceQuery(7024, Protocol::TCP), Service::Name("vmsvc")),
        (ServiceQuery(1163, Protocol::TCP), Service::Name("sddp")),
        (ServiceQuery(3101, Protocol::TCP), Service::Name("hp-pxpib")),
        (ServiceQuery(1967, Protocol::UDP), Service::Name("sns-quote")),
        (ServiceQuery(9023, Protocol::UDP), Service::Name("swa-1")),
        (ServiceQuery(3059, Protocol::UDP), Service::Name("qsoft")),
        (ServiceQuery(6035, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(4232, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2867, Protocol::UDP), Service::Name("esps-portal")),
        (ServiceQuery(1758, Protocol::UDP), Service::Name("tftp-mcast")),
        (ServiceQuery(4362, Protocol::TCP), Service::Name("afore-vdp-disc")),
        (ServiceQuery(3307, Protocol::UDP), Service::Name("opsession-prxy")),
        (ServiceQuery(2579, Protocol::TCP), Service::Name("mpfoncl")),
        (ServiceQuery(6699, Protocol::TCP), Service::Name("napster")),
        (ServiceQuery(8980, Protocol::TCP), Service::Name("nod-provider")),
        (ServiceQuery(3043, Protocol::TCP), Service::Name("brp")),
        (ServiceQuery(3330, Protocol::TCP), Service::Name("mcs-calypsoicf")),
        (ServiceQuery(12302, Protocol::TCP), Service::Name("rads")),
        (ServiceQuery(8122, Protocol::UDP), Service::Name("apollo-admin")),
        (ServiceQuery(7575, Protocol::TCP), Service::Name("wtmi-panel")),
        (ServiceQuery(3050, Protocol::UDP), Service::Name("gds_db")),
        (ServiceQuery(6148, Protocol::UDP), Service::Name("ricardo-lm")),
        (ServiceQuery(67, Protocol::UDP), Service::Name("dhcps")),
        (ServiceQuery(2389, Protocol::TCP), Service::Name("ovsessionmgr")),
        (ServiceQuery(1840, Protocol::UDP), Service::Name("netopia-vo2")),
        (ServiceQuery(761, Protocol::UDP), Service::Name("rxe")),
        (ServiceQuery(2642, Protocol::TCP), Service::Name("tragic")),
        (ServiceQuery(3016, Protocol::UDP), Service::Name("notify_srvr")),
        (ServiceQuery(1968, Protocol::TCP), Service::Name("lipsinc")),
        (ServiceQuery(134, Protocol::TCP), Service::Name("ingres-net")),
        (ServiceQuery(6946, Protocol::UDP), Service::Name("bioserver")),
        (ServiceQuery(3502, Protocol::UDP), Service::Name("avinstalldisc")),
        (ServiceQuery(1145, Protocol::TCP), Service::Name("x9-icue")),
        (ServiceQuery(30001, Protocol::UDP), Service::Name("pago-services1")),
        (ServiceQuery(3333, Protocol::TCP), Service::Name("dec-notes")),
        (ServiceQuery(1589, Protocol::UDP), Service::Name("vqp")),
        (ServiceQuery(1835, Protocol::TCP), Service::Name("ardusmul")),
        (ServiceQuery(2227, Protocol::TCP), Service::Name("di-msg")),
        (ServiceQuery(3598, Protocol::UDP), Service::Name("a15")),
        (ServiceQuery(1428, Protocol::UDP), Service::Name("informatik-lm")),
        (ServiceQuery(5443, Protocol::TCP), Service::Name("spss")),
        (ServiceQuery(1340, Protocol::TCP), Service::Name("naap")),
        (ServiceQuery(2387, Protocol::UDP), Service::Name("vsamredirector")),
        (ServiceQuery(505, Protocol::TCP), Service::Name("mailbox-lm")),
        (ServiceQuery(3481, Protocol::UDP), Service::Name("cleanerliverc")),
        (ServiceQuery(2799, Protocol::UDP), Service::Name("icon-discover")),
        (ServiceQuery(13717, Protocol::TCP), Service::Name("netbackup")),
        (ServiceQuery(3412, Protocol::TCP), Service::Name("xmlblaster")),
        (ServiceQuery(3624, Protocol::TCP), Service::Name("dist-upgrade")),
        (ServiceQuery(4846, Protocol::TCP), Service::Name("contamac_icm")),
        (ServiceQuery(2482, Protocol::TCP), Service::Name("giop-ssl")),
        (ServiceQuery(4408, Protocol::TCP), Service::Name("slscc")),
        (ServiceQuery(64738, Protocol::UDP), Service::Name("murmur")),
        (ServiceQuery(24323, Protocol::TCP), Service::Name("vrmg-ip")),
        (ServiceQuery(3569, Protocol::UDP), Service::Name("mbg-ctrl")),
        (ServiceQuery(1366, Protocol::TCP), Service::Name("netware-csp")),
        (ServiceQuery(2415, Protocol::TCP), Service::Name("codima-rtp")),
        (ServiceQuery(5225, Protocol::TCP), Service::Name("hp-server")),
        (ServiceQuery(5073, Protocol::TCP), Service::Name("tag-pm")),
        (ServiceQuery(573, Protocol::UDP), Service::Name("banyan-vip")),
        (ServiceQuery(4272, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(28240, Protocol::UDP), Service::Name("siemensgsm")),
        (ServiceQuery(2760, Protocol::UDP), Service::Name("sabams")),
        (ServiceQuery(3413, Protocol::TCP), Service::Name("svnet")),
        (ServiceQuery(1804, Protocol::UDP), Service::Name("enl")),
        (ServiceQuery(4070, Protocol::UDP), Service::Name("tripe")),
        (ServiceQuery(2733, Protocol::TCP), Service::Name("signet-ctf")),
        (ServiceQuery(5062, Protocol::TCP), Service::Name("na-localise")),
        (ServiceQuery(47000, Protocol::TCP), Service::Name("mbus")),
        (ServiceQuery(2197, Protocol::UDP), Service::Name("mnp-exchange")),
        (ServiceQuery(2064, Protocol::TCP), Service::Name("dnet-keyproxy")),
        (ServiceQuery(5443, Protocol::UDP), Service::Name("spss")),
        (ServiceQuery(692, Protocol::UDP), Service::Name("hyperwave-isp")),
        (ServiceQuery(2791, Protocol::UDP), Service::Name("mtport-regist")),
        (ServiceQuery(180, Protocol::UDP), Service::Name("ris")),
        (ServiceQuery(346, Protocol::UDP), Service::Name("zserv")),
        (ServiceQuery(9555, Protocol::UDP), Service::Name("trispen-sra")),
        (ServiceQuery(44, Protocol::UDP), Service::Name("mpm-flags")),
        (ServiceQuery(2944, Protocol::TCP), Service::Name("megaco-h248")),
        (ServiceQuery(1760, Protocol::TCP), Service::Name("www-ldap-gw")),
        (ServiceQuery(2307, Protocol::UDP), Service::Name("pehelp")),
        (ServiceQuery(18000, Protocol::TCP), Service::Name("biimenu")),
        (ServiceQuery(5305, Protocol::TCP), Service::Name("hacl-test")),
        (ServiceQuery(5405, Protocol::TCP), Service::Name("pcduo")),
        (ServiceQuery(1110, Protocol::UDP), Service::Name("nfsd-keepalive")),
        (ServiceQuery(588, Protocol::UDP), Service::Name("cal")),
        (ServiceQuery(13782, Protocol::TCP), Service::Name("netbackup")),
        (ServiceQuery(8770, Protocol::TCP), Service::Name("apple-iphoto")),
        (ServiceQuery(21848, Protocol::UDP), Service::Name("netspeak-acd")),
        (ServiceQuery(2548, Protocol::TCP), Service::Name("vytalvaultpipe")),
        (ServiceQuery(6487, Protocol::UDP), Service::Name("sun-sr-iiop-aut")),
        (ServiceQuery(22004, Protocol::UDP), Service::Name("optohost004")),
        (ServiceQuery(4552, Protocol::TCP), Service::Name("menandmice-mon")),
        (ServiceQuery(8122, Protocol::TCP), Service::Name("apollo-admin")),
        (ServiceQuery(346, Protocol::TCP), Service::Name("zserv")),
        (ServiceQuery(7319, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(3585, Protocol::TCP), Service::Name("emprise-lls")),
        (ServiceQuery(7358, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(1121, Protocol::UDP), Service::Name("rmpp")),
        (ServiceQuery(23001, Protocol::UDP), Service::Name("inovaport2")),
        (ServiceQuery(2945, Protocol::TCP), Service::Name("h248-binary")),
        (ServiceQuery(704, Protocol::TCP), Service::Name("elcsd")),
        (ServiceQuery(5065, Protocol::UDP), Service::Name("ca-2")),
        (ServiceQuery(4052, Protocol::UDP), Service::Name("interact")),
        (ServiceQuery(1929, Protocol::TCP), Service::Name("bandwiz-system")),
        (ServiceQuery(5107, Protocol::TCP), Service::Name("actifioreplic")),
        (ServiceQuery(27010, Protocol::TCP), Service::Name("flexlm10")),
        (ServiceQuery(4023, Protocol::TCP), Service::Name("esnm-zoning")),
        (ServiceQuery(1304, Protocol::TCP), Service::Name("boomerang")),
        (ServiceQuery(1122, Protocol::UDP), Service::Name("availant-mgr")),
        (ServiceQuery(4336, Protocol::TCP), Service::Name("restconf-ch-tls")),
        (ServiceQuery(3870, Protocol::UDP), Service::Name("ovsam-d-agent")),
        (ServiceQuery(2833, Protocol::TCP), Service::Name("glishd")),
        (ServiceQuery(1418, Protocol::TCP), Service::Name("timbuktu-srv2")),
        (ServiceQuery(501, Protocol::TCP), Service::Name("stmf")),
        (ServiceQuery(4261, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(5196, Protocol::TCP), Service::Name("ampl-tableproxy")),
        (ServiceQuery(2139, Protocol::TCP), Service::Name("ias-auth")),
        (ServiceQuery(3944, Protocol::UDP), Service::Name("sops")),
        (ServiceQuery(4188, Protocol::UDP), Service::Name("vatata")),
        (ServiceQuery(37475, Protocol::UDP), Service::Name("neckar")),
        (ServiceQuery(1483, Protocol::UDP), Service::Name("afs")),
        (ServiceQuery(7878, Protocol::TCP), Service::Name("owms")),
        (ServiceQuery(3278, Protocol::TCP), Service::Name("lkcmserver")),
        (ServiceQuery(7171, Protocol::UDP), Service::Name("drm-production")),
        (ServiceQuery(8405, Protocol::TCP), Service::Name("svbackup")),
        (ServiceQuery(5422, Protocol::UDP), Service::Name("salient-mux")),
        (ServiceQuery(2228, Protocol::TCP), Service::Name("ehome-ms")),
        (ServiceQuery(1805, Protocol::TCP), Service::Name("enl-name")),
        (ServiceQuery(3433, Protocol::UDP), Service::Name("alta-smp")),
        (ServiceQuery(601, Protocol::UDP), Service::Name("syslog-conn")),
        (ServiceQuery(2817, Protocol::TCP), Service::Name("nmsigport")),
        (ServiceQuery(5718, Protocol::TCP), Service::Name("dpm")),
        (ServiceQuery(9999, Protocol::UDP), Service::Name("distinct")),
        (ServiceQuery(7545, Protocol::TCP), Service::Name("nta-us")),
        (ServiceQuery(3804, Protocol::TCP), Service::Name("iqnet-port")),
        (ServiceQuery(1717, Protocol::UDP), Service::Name("fj-hdnet")),
        (ServiceQuery(3597, Protocol::UDP), Service::Name("a14")),
        (ServiceQuery(6544, Protocol::UDP), Service::Name("lds-dump")),
        (ServiceQuery(18243, Protocol::TCP), Service::Name("clusterxl")),
        (ServiceQuery(3104, Protocol::UDP), Service::Name("autocuetime")),
        (ServiceQuery(11367, Protocol::UDP), Service::Name("atm-uhas")),
        (ServiceQuery(6069, Protocol::UDP), Service::Name("trip")),
        (ServiceQuery(1723, Protocol::TCP), Service::Name("pptp")),
        (ServiceQuery(2242, Protocol::TCP), Service::Name("foliocorp")),
        (ServiceQuery(2098, Protocol::TCP), Service::Name("dialog-port")),
        (ServiceQuery(2322, Protocol::UDP), Service::Name("ofsd")),
        (ServiceQuery(2898, Protocol::TCP), Service::Name("appliance-cfg")),
        (ServiceQuery(2061, Protocol::UDP), Service::Name("netmount")),
        (ServiceQuery(1630, Protocol::UDP), Service::Name("oraclenet8cman")),
        (ServiceQuery(38201, Protocol::UDP), Service::Name("galaxy7-data")),
        (ServiceQuery(3801, Protocol::UDP), Service::Name("ibm-mgr")),
        (ServiceQuery(16003, Protocol::TCP), Service::Name("alfin")),
        (ServiceQuery(2611, Protocol::UDP), Service::Name("lionhead")),
        (ServiceQuery(8053, Protocol::TCP), Service::Name("senomix02")),
        (ServiceQuery(14142, Protocol::TCP), Service::Name("icpp")),
        (ServiceQuery(4990, Protocol::UDP), Service::Name("busycal")),
        (ServiceQuery(5742, Protocol::UDP), Service::Name("ida-discover2")),
        (ServiceQuery(860, Protocol::UDP), Service::Name("iscsi")),
        (ServiceQuery(6790, Protocol::UDP), Service::Name("hnmp")),
        (ServiceQuery(48002, Protocol::UDP), Service::Name("nimhub")),
        (ServiceQuery(1588, Protocol::UDP), Service::Name("triquest-lm")),
        (ServiceQuery(608, Protocol::TCP), Service::Name("sift-uft")),
        (ServiceQuery(3670, Protocol::UDP), Service::Name("smile")),
        (ServiceQuery(5245, Protocol::UDP), Service::Name("downtools-disc")),
        (ServiceQuery(7570, Protocol::TCP), Service::Name("aries-kfinder")),
        (ServiceQuery(3499, Protocol::TCP), Service::Name("sccip-media")),
        (ServiceQuery(7980, Protocol::TCP), Service::Name("quest-vista")),
        (ServiceQuery(6673, Protocol::UDP), Service::Name("vision_elmd")),
        (ServiceQuery(6103, Protocol::TCP), Service::Name("RETS-or-BackupExec")),
        (ServiceQuery(4321, Protocol::UDP), Service::Name("rwhois")),
        (ServiceQuery(1753, Protocol::TCP), Service::Name("predatar-comms")),
        (ServiceQuery(9093, Protocol::TCP), Service::Name("copycat")),
        (ServiceQuery(2213, Protocol::TCP), Service::Name("kali")),
        (ServiceQuery(3106, Protocol::TCP), Service::Name("cardbox-http")),
        (ServiceQuery(1810, Protocol::TCP), Service::Name("jerand-lm")),
        (ServiceQuery(1175, Protocol::TCP), Service::Name("dossier")),
        (ServiceQuery(3887, Protocol::UDP), Service::Name("ciphire-data")),
        (ServiceQuery(27000, Protocol::TCP), Service::Name("flexlm0")),
        (ServiceQuery(38422, Protocol::TCP), Service::Name("xn-control")),
        (ServiceQuery(7786, Protocol::TCP), Service::Name("minivend")),
        (ServiceQuery(7302, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(389, Protocol::UDP), Service::Name("ldap")),
        (ServiceQuery(3314, Protocol::UDP), Service::Name("uohost")),
        (ServiceQuery(25793, Protocol::UDP), Service::Name("vocaltec-hos")),
        (ServiceQuery(9204, Protocol::UDP), Service::Name("wap-vcard")),
        (ServiceQuery(8890, Protocol::UDP), Service::Name("ddi-udp-3")),
        (ServiceQuery(3114, Protocol::TCP), Service::Name("ccmad")),
        (ServiceQuery(3247, Protocol::TCP), Service::Name("dvt-data")),
        (ServiceQuery(3633, Protocol::UDP), Service::Name("wacp")),
        (ServiceQuery(573, Protocol::TCP), Service::Name("banyan-vip")),
        (ServiceQuery(5409, Protocol::UDP), Service::Name("salient-dtasrv")),
        (ServiceQuery(1625, Protocol::TCP), Service::Name("svs-omagent")),
        (ServiceQuery(1570, Protocol::TCP), Service::Name("orbixd")),
        (ServiceQuery(146, Protocol::TCP), Service::Name("iso-tp0")),
        (ServiceQuery(13818, Protocol::UDP), Service::Name("dsmcc-config")),
        (ServiceQuery(571, Protocol::UDP), Service::Name("umeter")),
        (ServiceQuery(17754, Protocol::TCP), Service::Name("zep")),
        (ServiceQuery(3342, Protocol::TCP), Service::Name("webtie")),
        (ServiceQuery(2389, Protocol::UDP), Service::Name("ovsessionmgr")),
        (ServiceQuery(3203, Protocol::TCP), Service::Name("netwatcher-mon")),
        (ServiceQuery(3660, Protocol::TCP), Service::Name("can-nds-ssl")),
        (ServiceQuery(24000, Protocol::TCP), Service::Name("med-ltp")),
        (ServiceQuery(5111, Protocol::TCP), Service::Name("taep-as-svc")),
        (ServiceQuery(1322, Protocol::TCP), Service::Name("novation")),
        (ServiceQuery(183, Protocol::TCP), Service::Name("ocbinder")),
        (ServiceQuery(4984, Protocol::TCP), Service::Name("webyast")),
        (ServiceQuery(4605, Protocol::TCP), Service::Name("sixchat")),
        (ServiceQuery(3359, Protocol::UDP), Service::Name("wg-netforce")),
        (ServiceQuery(1292, Protocol::TCP), Service::Name("dsdn")),
        (ServiceQuery(3352, Protocol::TCP), Service::Name("ssql")),
        (ServiceQuery(19999, Protocol::TCP), Service::Name("dnp-sec")),
        (ServiceQuery(45824, Protocol::TCP), Service::Name("dai-shell")),
        (ServiceQuery(4329, Protocol::TCP), Service::Name("publiqare-sync")),
        (ServiceQuery(4097, Protocol::TCP), Service::Name("patrolview")),
        (ServiceQuery(7278, Protocol::TCP), Service::Name("oma-dcdocbs")),
        (ServiceQuery(5781, Protocol::UDP), Service::Name("3par-evts")),
        (ServiceQuery(7, Protocol::UDP), Service::Name("echo")),
        (ServiceQuery(596, Protocol::TCP), Service::Name("smsd")),
        (ServiceQuery(16991, Protocol::UDP), Service::Name("intel-rci-mp")),
        (ServiceQuery(3397, Protocol::TCP), Service::Name("saposs")),
        (ServiceQuery(2770, Protocol::UDP), Service::Name("veronica")),
        (ServiceQuery(1458, Protocol::TCP), Service::Name("nrcabq-lm")),
        (ServiceQuery(4206, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(8040, Protocol::TCP), Service::Name("ampify")),
        (ServiceQuery(5056, Protocol::UDP), Service::Name("intecom-ps1")),
        (ServiceQuery(2411, Protocol::UDP), Service::Name("netwave-ap-mgmt")),
        (ServiceQuery(624, Protocol::TCP), Service::Name("cryptoadmin")),
        (ServiceQuery(3607, Protocol::UDP), Service::Name("precise-i3")),
        (ServiceQuery(3193, Protocol::UDP), Service::Name("spandataport")),
        (ServiceQuery(4688, Protocol::UDP), Service::Name("mobile-p2p")),
        (ServiceQuery(3614, Protocol::TCP), Service::Name("sigma-port")),
        (ServiceQuery(7173, Protocol::TCP), Service::Name("zsecure")),
        (ServiceQuery(437, Protocol::TCP), Service::Name("comscm")),
        (ServiceQuery(2659, Protocol::UDP), Service::Name("sns-query")),
        (ServiceQuery(2903, Protocol::TCP), Service::Name("extensisportfolio")),
        (ServiceQuery(3990, Protocol::UDP), Service::Name("bv-is")),
        (ServiceQuery(643, Protocol::TCP), Service::Name("sanity")),
        (ServiceQuery(3777, Protocol::UDP), Service::Name("jibe-eb")),
        (ServiceQuery(5082, Protocol::TCP), Service::Name("qcp")),
        (ServiceQuery(3523, Protocol::TCP), Service::Name("odeumservlink")),
        (ServiceQuery(4725, Protocol::UDP), Service::Name("truckstar")),
        (ServiceQuery(4732, Protocol::TCP), Service::Name("ohmtrigger")),
        (ServiceQuery(590, Protocol::UDP), Service::Name("tns-cml")),
        (ServiceQuery(6582, Protocol::UDP), Service::Name("parsec-game")),
        (ServiceQuery(1283, Protocol::TCP), Service::Name("productinfo")),
        (ServiceQuery(4205, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(5352, Protocol::UDP), Service::Name("dns-llq")),
        (ServiceQuery(2506, Protocol::TCP), Service::Name("jbroker")),
        (ServiceQuery(5767, Protocol::UDP), Service::Name("s-openmail")),
        (ServiceQuery(10160, Protocol::UDP), Service::Name("qb-db-server")),
        (ServiceQuery(1830, Protocol::UDP), Service::Name("net8-cman")),
        (ServiceQuery(5157, Protocol::TCP), Service::Name("mediat")),
        (ServiceQuery(6162, Protocol::TCP), Service::Name("patrol-coll")),
        (ServiceQuery(3920, Protocol::UDP), Service::Name("exasoftport1")),
        (ServiceQuery(1616, Protocol::UDP), Service::Name("netbill-prod")),
        (ServiceQuery(6408, Protocol::UDP), Service::Name("boe-resssvr2")),
        (ServiceQuery(6700, Protocol::TCP), Service::Name("carracho")),
        (ServiceQuery(1032, Protocol::TCP), Service::Name("iad3")),
        (ServiceQuery(7741, Protocol::UDP), Service::Name("scriptview")),
        (ServiceQuery(3721, Protocol::TCP), Service::Name("xsync")),
        (ServiceQuery(2300, Protocol::TCP), Service::Name("cvmmon")),
        (ServiceQuery(3198, Protocol::TCP), Service::Name("embrace-dp-c")),
        (ServiceQuery(1098, Protocol::UDP), Service::Name("rmiactivation")),
        (ServiceQuery(1786, Protocol::UDP), Service::Name("funk-logger")),
        (ServiceQuery(2961, Protocol::UDP), Service::Name("boldsoft-lm")),
        (ServiceQuery(1216, Protocol::UDP), Service::Name("etebac5")),
        (ServiceQuery(2164, Protocol::UDP), Service::Name("ddns-v3")),
        (ServiceQuery(1486, Protocol::UDP), Service::Name("nms_topo_serv")),
        (ServiceQuery(3161, Protocol::UDP), Service::Name("doc1lm")),
        (ServiceQuery(9901, Protocol::UDP), Service::Name("enrp")),
        (ServiceQuery(2054, Protocol::TCP), Service::Name("weblogin")),
        (ServiceQuery(6049, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(2924, Protocol::TCP), Service::Name("precise-vip")),
        (ServiceQuery(22800, Protocol::TCP), Service::Name("aws-brf")),
        (ServiceQuery(1045, Protocol::UDP), Service::Name("fpitp")),
        (ServiceQuery(31948, Protocol::TCP), Service::Name("iceedcp_tx")),
        (ServiceQuery(7099, Protocol::UDP), Service::Name("lazy-ptop")),
        (ServiceQuery(1519, Protocol::UDP), Service::Name("vpvc")),
        (ServiceQuery(2384, Protocol::UDP), Service::Name("sd-capacity")),
        (ServiceQuery(1938, Protocol::UDP), Service::Name("jwclient")),
        (ServiceQuery(2965, Protocol::TCP), Service::Name("bullant-rap")),
        (ServiceQuery(6005, Protocol::TCP), Service::Name("X11:5")),
        (ServiceQuery(5398, Protocol::UDP), Service::Name("elektron-admin")),
        (ServiceQuery(2674, Protocol::UDP), Service::Name("ewnn")),
        (ServiceQuery(3219, Protocol::TCP), Service::Name("wms-messenger")),
        (ServiceQuery(1461, Protocol::UDP), Service::Name("ibm_wrless_lan")),
        (ServiceQuery(2534, Protocol::UDP), Service::Name("combox-web-acc")),
        (ServiceQuery(2407, Protocol::UDP), Service::Name("orion")),
        (ServiceQuery(4405, Protocol::TCP), Service::Name("ds-mail")),
        (ServiceQuery(1701, Protocol::TCP), Service::Name("l2f")),
        (ServiceQuery(3837, Protocol::UDP), Service::Name("mkm-discovery")),
        (ServiceQuery(44322, Protocol::TCP), Service::Name("pmcdproxy")),
        (ServiceQuery(1906, Protocol::UDP), Service::Name("tpmd")),
        (ServiceQuery(6868, Protocol::UDP), Service::Name("acctopus-st")),
        (ServiceQuery(3981, Protocol::UDP), Service::Name("starfish")),
        (ServiceQuery(8204, Protocol::UDP), Service::Name("lm-perfworks")),
        (ServiceQuery(354, Protocol::UDP), Service::Name("bh611")),
        (ServiceQuery(3741, Protocol::UDP), Service::Name("wysdma")),
        (ServiceQuery(1128, Protocol::UDP), Service::Name("saphostctrl")),
        (ServiceQuery(2446, Protocol::UDP), Service::Name("bues_service")),
        (ServiceQuery(5414, Protocol::UDP), Service::Name("statusd")),
        (ServiceQuery(61440, Protocol::TCP), Service::Name("netprowler-manager2")),
        (ServiceQuery(2484, Protocol::UDP), Service::Name("ttc-ssl")),
        (ServiceQuery(6106, Protocol::TCP), Service::Name("isdninfo")),
        (ServiceQuery(282, Protocol::TCP), Service::Name("cableport-ax")),
        (ServiceQuery(4116, Protocol::TCP), Service::Name("smartcard-tls")),
        (ServiceQuery(1857, Protocol::UDP), Service::Name("datacaptor")),
        (ServiceQuery(5794, Protocol::UDP), Service::Name("spdp")),
        (ServiceQuery(7318, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(19410, Protocol::UDP), Service::Name("hp-sco")),
        (ServiceQuery(4341, Protocol::UDP), Service::Name("lisp-data")),
        (ServiceQuery(1669, Protocol::UDP), Service::Name("netview-aix-9")),
        (ServiceQuery(1965, Protocol::UDP), Service::Name("tivoli-npm")),
        (ServiceQuery(4055, Protocol::UDP), Service::Name("ccu-comm-3")),
        (ServiceQuery(3089, Protocol::UDP), Service::Name("ptk-alink")),
        (ServiceQuery(1942, Protocol::TCP), Service::Name("res")),
        (ServiceQuery(11171, Protocol::TCP), Service::Name("snss")),
        (ServiceQuery(266, Protocol::UDP), Service::Name("sst")),
        (ServiceQuery(1538, Protocol::TCP), Service::Name("3ds-lm")),
        (ServiceQuery(3867, Protocol::TCP), Service::Name("dzoglserver")),
        (ServiceQuery(7162, Protocol::TCP), Service::Name("caistoragemgr")),
        (ServiceQuery(2459, Protocol::UDP), Service::Name("community")),
        (ServiceQuery(10544, Protocol::TCP), Service::Name("MOS-soap-opt")),
        (ServiceQuery(7353, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(4345, Protocol::TCP), Service::Name("m4-network-as")),
        (ServiceQuery(2612, Protocol::UDP), Service::Name("qpasa-agent")),
        (ServiceQuery(1545, Protocol::UDP), Service::Name("vistium-share")),
        (ServiceQuery(8322, Protocol::TCP), Service::Name("garmin-marine")),
        (ServiceQuery(1273, Protocol::UDP), Service::Name("emc-gateway")),
        (ServiceQuery(2574, Protocol::TCP), Service::Name("blockade-bpsp")),
        (ServiceQuery(2378, Protocol::TCP), Service::Name("dali")),
        (ServiceQuery(751, Protocol::TCP), Service::Name("kadmin")),
        (ServiceQuery(4970, Protocol::UDP), Service::Name("ccss-qsm")),
        (ServiceQuery(6767, Protocol::TCP), Service::Name("bmc-perf-agent")),
        (ServiceQuery(2789, Protocol::TCP), Service::Name("media-agent")),
        (ServiceQuery(309, Protocol::UDP), Service::Name("entrusttime")),
        (ServiceQuery(10000, Protocol::TCP), Service::Name("snet-sensor-mgmt")),
        (ServiceQuery(3872, Protocol::TCP), Service::Name("oem-agent")),
        (ServiceQuery(5602, Protocol::UDP), Service::Name("a1-msc")),
        (ServiceQuery(585, Protocol::TCP), Service::Name("imap4-ssl")),
        (ServiceQuery(9021, Protocol::TCP), Service::Name("panagolin-ident")),
        (ServiceQuery(1310, Protocol::TCP), Service::Name("husky")),
        (ServiceQuery(3736, Protocol::UDP), Service::Name("rs-rmi")),
        (ServiceQuery(3173, Protocol::UDP), Service::Name("serverview-icc")),
        (ServiceQuery(2990, Protocol::UDP), Service::Name("boscap")),
        (ServiceQuery(4441, Protocol::TCP), Service::Name("netblox")),
        (ServiceQuery(1054, Protocol::UDP), Service::Name("brvread")),
        (ServiceQuery(5002, Protocol::TCP), Service::Name("rfe")),
        (ServiceQuery(16311, Protocol::TCP), Service::Name("pdefmns")),
        (ServiceQuery(2783, Protocol::UDP), Service::Name("aises")),
        (ServiceQuery(1039, Protocol::UDP), Service::Name("sbl")),
        (ServiceQuery(4214, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(5356, Protocol::UDP), Service::Name("ms-smlbiz")),
        (ServiceQuery(44818, Protocol::UDP), Service::Name("EtherNetIP-2")),
        (ServiceQuery(6019, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(31029, Protocol::UDP), Service::Name("yawn")),
        (ServiceQuery(18183, Protocol::UDP), Service::Name("opsec-sam")),
        (ServiceQuery(5505, Protocol::UDP), Service::Name("checkoutdb")),
        (ServiceQuery(364, Protocol::TCP), Service::Name("aurora-cmgr")),
        (ServiceQuery(7501, Protocol::UDP), Service::Name("ovbus")),
        (ServiceQuery(20003, Protocol::UDP), Service::Name("commtact-https")),
        (ServiceQuery(3525, Protocol::TCP), Service::Name("eisport")),
        (ServiceQuery(3405, Protocol::TCP), Service::Name("nokia-ann-ch1")),
        (ServiceQuery(1858, Protocol::TCP), Service::Name("privateark")),
        (ServiceQuery(2029, Protocol::UDP), Service::Name("hsrpv6")),
        (ServiceQuery(1866, Protocol::TCP), Service::Name("swrmi")),
        (ServiceQuery(2080, Protocol::TCP), Service::Name("autodesk-nlm")),
        (ServiceQuery(4039, Protocol::TCP), Service::Name("fazzt-admin")),
        (ServiceQuery(3464, Protocol::UDP), Service::Name("edm-mgr-sync")),
        (ServiceQuery(6052, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(2403, Protocol::TCP), Service::Name("taskmaster2000")),
        (ServiceQuery(2296, Protocol::TCP), Service::Name("theta-lm")),
        (ServiceQuery(2789, Protocol::UDP), Service::Name("media-agent")),
        (ServiceQuery(9909, Protocol::TCP), Service::Name("domaintime")),
        (ServiceQuery(1151, Protocol::UDP), Service::Name("unizensus")),
        (ServiceQuery(4593, Protocol::UDP), Service::Name("ipt-anri-anri")),
        (ServiceQuery(1768, Protocol::TCP), Service::Name("cft-7")),
        (ServiceQuery(465, Protocol::TCP), Service::Name("smtps")),
        (ServiceQuery(4372, Protocol::UDP), Service::Name("l2c-data")),
        (ServiceQuery(2625, Protocol::UDP), Service::Name("blwnkl-port")),
        (ServiceQuery(6222, Protocol::TCP), Service::Name("radmind")),
        (ServiceQuery(1344, Protocol::UDP), Service::Name("icap")),
        (ServiceQuery(7566, Protocol::TCP), Service::Name("vsi-omega")),
        (ServiceQuery(34964, Protocol::TCP), Service::Name("profinet-cm")),
        (ServiceQuery(8081, Protocol::UDP), Service::Name("sunproxyadmin")),
        (ServiceQuery(5310, Protocol::TCP), Service::Name("outlaws")),
        (ServiceQuery(2219, Protocol::UDP), Service::Name("netiq-ncap")),
        (ServiceQuery(43000, Protocol::TCP), Service::Name("recvr-rc")),
        (ServiceQuery(6300, Protocol::TCP), Service::Name("bmc-grx")),
        (ServiceQuery(442, Protocol::UDP), Service::Name("cvc_hostd")),
        (ServiceQuery(2341, Protocol::UDP), Service::Name("xiostatus")),
        (ServiceQuery(3665, Protocol::UDP), Service::Name("ent-engine")),
        (ServiceQuery(1220, Protocol::TCP), Service::Name("quicktime")),
        (ServiceQuery(4410, Protocol::TCP), Service::Name("itwo-server")),
        (ServiceQuery(5245, Protocol::TCP), Service::Name("downtools")),
        (ServiceQuery(3595, Protocol::TCP), Service::Name("shareapp")),
        (ServiceQuery(1223, Protocol::UDP), Service::Name("tgp")),
        (ServiceQuery(1616, Protocol::TCP), Service::Name("netbill-prod")),
        (ServiceQuery(1555, Protocol::UDP), Service::Name("livelan")),
        (ServiceQuery(1168, Protocol::UDP), Service::Name("vchat")),
        (ServiceQuery(5075, Protocol::TCP), Service::Name("pvaccess")),
        (ServiceQuery(1059, Protocol::UDP), Service::Name("nimreg")),
        (ServiceQuery(4848, Protocol::TCP), Service::Name("appserv-http")),
        (ServiceQuery(27009, Protocol::TCP), Service::Name("flexlm9")),
        (ServiceQuery(1172, Protocol::TCP), Service::Name("dnap")),
        (ServiceQuery(8033, Protocol::TCP), Service::Name("mindprint")),
        (ServiceQuery(5353, Protocol::TCP), Service::Name("mdns")),
        (ServiceQuery(2878, Protocol::UDP), Service::Name("aap")),
        (ServiceQuery(3733, Protocol::UDP), Service::Name("multip-msg")),
        (ServiceQuery(694, Protocol::TCP), Service::Name("ha-cluster")),
        (ServiceQuery(1338, Protocol::TCP), Service::Name("wmc-log-svc")),
        (ServiceQuery(7903, Protocol::UDP), Service::Name("tnos-dps")),
        (ServiceQuery(2404, Protocol::UDP), Service::Name("iec-104")),
        (ServiceQuery(1296, Protocol::TCP), Service::Name("dproxy")),
        (ServiceQuery(3379, Protocol::UDP), Service::Name("socorfs")),
        (ServiceQuery(3713, Protocol::TCP), Service::Name("tftps")),
        (ServiceQuery(5084, Protocol::TCP), Service::Name("llrp")),
        (ServiceQuery(3208, Protocol::TCP), Service::Name("pfu-prcallback")),
        (ServiceQuery(2661, Protocol::TCP), Service::Name("olhost")),
        (ServiceQuery(1664, Protocol::UDP), Service::Name("netview-aix-4")),
        (ServiceQuery(7351, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(7648, Protocol::TCP), Service::Name("cuseeme")),
        (ServiceQuery(24006, Protocol::UDP), Service::Name("med-net-svc")),
        (ServiceQuery(3167, Protocol::TCP), Service::Name("nowcontact")),
        (ServiceQuery(5085, Protocol::UDP), Service::Name("encrypted-llrp")),
        (ServiceQuery(3117, Protocol::TCP), Service::Name("mctet-jserv")),
        (ServiceQuery(8282, Protocol::TCP), Service::Name("libelle")),
        (ServiceQuery(316, Protocol::UDP), Service::Name("decauth")),
        (ServiceQuery(2033, Protocol::TCP), Service::Name("glogger")),
        (ServiceQuery(3867, Protocol::UDP), Service::Name("dzoglserver")),
        (ServiceQuery(4488, Protocol::TCP), Service::Name("awacs-ice")),
        (ServiceQuery(3979, Protocol::UDP), Service::Name("smwan")),
        (ServiceQuery(47001, Protocol::TCP), Service::Name("winrm")),
        (ServiceQuery(3798, Protocol::TCP), Service::Name("minilock")),
        (ServiceQuery(3773, Protocol::UDP), Service::Name("ctdhercules")),
        (ServiceQuery(18183, Protocol::TCP), Service::Name("opsec-sam")),
        (ServiceQuery(3853, Protocol::TCP), Service::Name("sscan")),
        (ServiceQuery(5771, Protocol::UDP), Service::Name("netagent")),
        (ServiceQuery(1988, Protocol::TCP), Service::Name("tr-rsrb-p2")),
        (ServiceQuery(6034, Protocol::UDP), Service::Name("x11")),
        (ServiceQuery(2142, Protocol::TCP), Service::Name("tdmoip")),
        (ServiceQuery(2512, Protocol::TCP), Service::Name("citrixima")),
        (ServiceQuery(3242, Protocol::UDP), Service::Name("sdp-id-port")),
        (ServiceQuery(4327, Protocol::UDP), Service::Name("jaxer-web")),
        (ServiceQuery(3710, Protocol::UDP), Service::Name("portgate-auth")),
        (ServiceQuery(4671, Protocol::TCP), Service::Name("acter")),
        (ServiceQuery(31620, Protocol::UDP), Service::Name("lm-mon")),
        (ServiceQuery(1520, Protocol::UDP), Service::Name("atm-zip-office")),
        (ServiceQuery(1874, Protocol::UDP), Service::Name("fjswapsnp")),
        (ServiceQuery(7026, Protocol::TCP), Service::Name("loreji-panel")),
        (ServiceQuery(2102, Protocol::TCP), Service::Name("zephyr-srv")),
        (ServiceQuery(1874, Protocol::TCP), Service::Name("fjswapsnp")),
        (ServiceQuery(91, Protocol::UDP), Service::Name("mit-dov")),
        (ServiceQuery(6269, Protocol::TCP), Service::Name("grid-alt")),
        (ServiceQuery(2482, Protocol::UDP), Service::Name("giop-ssl")),
        (ServiceQuery(1292, Protocol::UDP), Service::Name("dsdn")),
        (ServiceQuery(3137, Protocol::TCP), Service::Name("rtnt-1")),
        (ServiceQuery(3898, Protocol::UDP), Service::Name("senip")),
        (ServiceQuery(1473, Protocol::TCP), Service::Name("openmath")),
        (ServiceQuery(2561, Protocol::UDP), Service::Name("mosaixcc")),
        (ServiceQuery(458, Protocol::TCP), Service::Name("appleqtc")),
        (ServiceQuery(9202, Protocol::UDP), Service::Name("wap-wsp-s")),
        (ServiceQuery(5565, Protocol::TCP), Service::Name("hpe-dp-bura")),
        (ServiceQuery(130, Protocol::UDP), Service::Name("cisco-fna")),
        (ServiceQuery(3048, Protocol::TCP), Service::Name("pctrader")),
        (ServiceQuery(991, Protocol::UDP), Service::Name("nas")),
        (ServiceQuery(2830, Protocol::UDP), Service::Name("silkp2")),
        (ServiceQuery(5049, Protocol::TCP), Service::Name("ivocalize")),
        (ServiceQuery(1485, Protocol::TCP), Service::Name("lansource")),
        (ServiceQuery(649, Protocol::UDP), Service::Name("cadview-3d")),
        (ServiceQuery(186, Protocol::UDP), Service::Name("kis")),
        (ServiceQuery(1827, Protocol::UDP), Service::Name("asi")),
        (ServiceQuery(2958, Protocol::UDP), Service::Name("jmact6")),
        (ServiceQuery(3747, Protocol::UDP), Service::Name("linktest-s")),
        (ServiceQuery(1748, Protocol::TCP), Service::Name("oracle-em1")),
        (ServiceQuery(168, Protocol::TCP), Service::Name("rsvd")),
        (ServiceQuery(4256, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(22, Protocol::TCP), Service::Name("ssh")),
        (ServiceQuery(6936, Protocol::UDP), Service::Name("xsmsvc")),
        (ServiceQuery(7001, Protocol::TCP), Service::Name("afs3-callback")),
        (ServiceQuery(4044, Protocol::TCP), Service::Name("ltp")),
        (ServiceQuery(8044, Protocol::TCP), Service::Name("fs-mgmt")),
        (ServiceQuery(401, Protocol::UDP), Service::Name("ups")),
        (ServiceQuery(5152, Protocol::UDP), Service::Name("sde-discovery")),
        (ServiceQuery(2054, Protocol::UDP), Service::Name("weblogin")),
        (ServiceQuery(1225, Protocol::TCP), Service::Name("slinkysearch")),
        (ServiceQuery(1930, Protocol::UDP), Service::Name("driveappserver")),
        (ServiceQuery(407, Protocol::TCP), Service::Name("timbuktu")),
        (ServiceQuery(4286, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2633, Protocol::TCP), Service::Name("interintelli")),
        (ServiceQuery(2273, Protocol::TCP), Service::Name("mysql-im")),
        (ServiceQuery(4222, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(5079, Protocol::TCP), Service::Name("cp-spxrpts")),
        (ServiceQuery(5748, Protocol::TCP), Service::Name("tunalyzer")),
        (ServiceQuery(2009, Protocol::TCP), Service::Name("news")),
        (ServiceQuery(130, Protocol::TCP), Service::Name("cisco-fna")),
        (ServiceQuery(5008, Protocol::UDP), Service::Name("synapsis-edge")),
        (ServiceQuery(1621, Protocol::TCP), Service::Name("softdataphone")),
        (ServiceQuery(2227, Protocol::UDP), Service::Name("di-msg")),
        (ServiceQuery(614, Protocol::TCP), Service::Name("sshell")),
        (ServiceQuery(8182, Protocol::UDP), Service::Name("vmware-fdm")),
        (ServiceQuery(7320, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(2073, Protocol::TCP), Service::Name("gxs-data-port")),
        (ServiceQuery(4099, Protocol::TCP), Service::Name("dpcp")),
        (ServiceQuery(2888, Protocol::TCP), Service::Name("spcsdlobby")),
        (ServiceQuery(6147, Protocol::TCP), Service::Name("montage-lm")),
        (ServiceQuery(3255, Protocol::UDP), Service::Name("semaphore")),
        (ServiceQuery(1411, Protocol::UDP), Service::Name("af")),
        (ServiceQuery(7009, Protocol::UDP), Service::Name("afs3-rmtsys")),
        (ServiceQuery(7546, Protocol::TCP), Service::Name("cfs")),
        (ServiceQuery(1487, Protocol::TCP), Service::Name("localinfosrvr")),
        (ServiceQuery(2604, Protocol::TCP), Service::Name("ospfd")),
        (ServiceQuery(3936, Protocol::UDP), Service::Name("mailprox")),
        (ServiceQuery(7902, Protocol::UDP), Service::Name("tnos-dp")),
        (ServiceQuery(5192, Protocol::TCP), Service::Name("aol-2")),
        (ServiceQuery(4031, Protocol::TCP), Service::Name("suucp")),
        (ServiceQuery(2781, Protocol::TCP), Service::Name("whosells")),
        (ServiceQuery(5988, Protocol::UDP), Service::Name("wbem-http")),
        (ServiceQuery(9555, Protocol::TCP), Service::Name("trispen-sra")),
        (ServiceQuery(2414, Protocol::UDP), Service::Name("beeyond")),
        (ServiceQuery(7797, Protocol::TCP), Service::Name("pnet-conn")),
        (ServiceQuery(639, Protocol::UDP), Service::Name("msdp")),
        (ServiceQuery(2539, Protocol::UDP), Service::Name("vsiadmin")),
        (ServiceQuery(11434, Protocol::TCP), Service::Name("ollama")),
        (ServiceQuery(1540, Protocol::TCP), Service::Name("rds")),
        (ServiceQuery(24677, Protocol::UDP), Service::Name("flashfiler")),
        (ServiceQuery(3199, Protocol::UDP), Service::Name("dmod-workspace")),
        (ServiceQuery(4233, Protocol::TCP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1933, Protocol::TCP), Service::Name("xmapi")),
        (ServiceQuery(2890, Protocol::TCP), Service::Name("cspclmulti")),
        (ServiceQuery(1413, Protocol::UDP), Service::Name("innosys-acl")),
        (ServiceQuery(352, Protocol::TCP), Service::Name("dtag-ste-sb")),
        (ServiceQuery(5680, Protocol::UDP), Service::Name("auriga-router")),
        (ServiceQuery(324, Protocol::TCP), Service::Name("rpki-rtr-tls")),
        (ServiceQuery(4691, Protocol::UDP), Service::Name("mtn")),
        (ServiceQuery(8997, Protocol::TCP), Service::Name("oracle-ms-ens")),
        (ServiceQuery(1544, Protocol::UDP), Service::Name("aspeclmd")),
        (ServiceQuery(14937, Protocol::TCP), Service::Name("hde-lcesrvr-2")),
        (ServiceQuery(2135, Protocol::TCP), Service::Name("gris")),
        (ServiceQuery(3966, Protocol::UDP), Service::Name("bflckmgr")),
        (ServiceQuery(2369, Protocol::TCP), Service::Name("bif-p2p")),
        (ServiceQuery(8192, Protocol::TCP), Service::Name("sophos")),
        (ServiceQuery(58, Protocol::TCP), Service::Name("xns-mail")),
        (ServiceQuery(1532, Protocol::UDP), Service::Name("miroconnect")),
        (ServiceQuery(1199, Protocol::UDP), Service::Name("dmidi")),
        (ServiceQuery(2350, Protocol::TCP), Service::Name("psbserver")),
        (ServiceQuery(3073, Protocol::TCP), Service::Name("vcrp")),
        (ServiceQuery(4119, Protocol::UDP), Service::Name("assuria-slm")),
        (ServiceQuery(3984, Protocol::TCP), Service::Name("mapper-nodemgr")),
        (ServiceQuery(3046, Protocol::UDP), Service::Name("di-ase")),
        (ServiceQuery(6031, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(2306, Protocol::UDP), Service::Name("tappi-boxnet")),
        (ServiceQuery(3707, Protocol::TCP), Service::Name("rt-event-s")),
        (ServiceQuery(467, Protocol::TCP), Service::Name("mylex-mapd")),
        (ServiceQuery(45678, Protocol::UDP), Service::Name("eba")),
        (ServiceQuery(4549, Protocol::UDP), Service::Name("aegate")),
        (ServiceQuery(3701, Protocol::UDP), Service::Name("netcelera")),
        (ServiceQuery(6268, Protocol::UDP), Service::Name("grid")),
        (ServiceQuery(1658, Protocol::UDP), Service::Name("sixnetudr")),
        (ServiceQuery(9102, Protocol::UDP), Service::Name("bacula-fd")),
        (ServiceQuery(2761, Protocol::UDP), Service::Name("dicom-iscl")),
        (ServiceQuery(2486, Protocol::TCP), Service::Name("netobjects2")),
        (ServiceQuery(3627, Protocol::TCP), Service::Name("jamserverport")),
        (ServiceQuery(3015, Protocol::TCP), Service::Name("nati-dstp")),
        (ServiceQuery(2753, Protocol::UDP), Service::Name("de-spot")),
        (ServiceQuery(4837, Protocol::UDP), Service::Name("varadero-0")),
        (ServiceQuery(3402, Protocol::UDP), Service::Name("fxaengine-net")),
        (ServiceQuery(2607, Protocol::TCP), Service::Name("connection")),
        (ServiceQuery(24001, Protocol::UDP), Service::Name("med-fsp-rx")),
        (ServiceQuery(2229, Protocol::TCP), Service::Name("datalens")),
        (ServiceQuery(3176, Protocol::UDP), Service::Name("ars-master")),
        (ServiceQuery(3924, Protocol::TCP), Service::Name("mpl-gprs-port")),
        (ServiceQuery(9092, Protocol::TCP), Service::Name("XmlIpcRegSvc")),
        (ServiceQuery(7846, Protocol::TCP), Service::Name("apc-7846")),
        (ServiceQuery(10201, Protocol::UDP), Service::Name("rscs")),
        (ServiceQuery(4285, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(2371, Protocol::UDP), Service::Name("worldwire")),
        (ServiceQuery(8937, Protocol::TCP), Service::Name("twds")),
        (ServiceQuery(688, Protocol::UDP), Service::Name("realm-rusd")),
        (ServiceQuery(1412, Protocol::TCP), Service::Name("innosys")),
        (ServiceQuery(47806, Protocol::TCP), Service::Name("ap")),
        (ServiceQuery(4049, Protocol::TCP), Service::Name("wafs")),
        (ServiceQuery(4556, Protocol::TCP), Service::Name("dtn-bundle-tcp")),
        (ServiceQuery(4043, Protocol::TCP), Service::Name("nirp")),
        (ServiceQuery(2068, Protocol::TCP), Service::Name("avocentkvm")),
        (ServiceQuery(27345, Protocol::UDP), Service::Name("imagepump")),
        (ServiceQuery(1459, Protocol::TCP), Service::Name("proshare1")),
        (ServiceQuery(2308, Protocol::TCP), Service::Name("sdhelp")),
        (ServiceQuery(7725, Protocol::TCP), Service::Name("nitrogen")),
        (ServiceQuery(1652, Protocol::UDP), Service::Name("xnmp")),
        (ServiceQuery(7675, Protocol::UDP), Service::Name("imqtunnel")),
        (ServiceQuery(3185, Protocol::TCP), Service::Name("smpppd")),
        (ServiceQuery(48048, Protocol::TCP), Service::Name("juka")),
        (ServiceQuery(396, Protocol::TCP), Service::Name("netware-ip")),
        (ServiceQuery(9106, Protocol::UDP), Service::Name("astergate-disc")),
        (ServiceQuery(3199, Protocol::TCP), Service::Name("dmod-workspace")),
        (ServiceQuery(2330, Protocol::UDP), Service::Name("tscchat")),
        (ServiceQuery(1365, Protocol::UDP), Service::Name("adapt-sna")),
        (ServiceQuery(3266, Protocol::UDP), Service::Name("ns-cfg-server")),
        (ServiceQuery(6109, Protocol::TCP), Service::Name("globecast-id")),
        (ServiceQuery(265, Protocol::TCP), Service::Name("maybe-fw1")),
        (ServiceQuery(9201, Protocol::TCP), Service::Name("wap-wsp-wtp")),
        (ServiceQuery(1747, Protocol::UDP), Service::Name("ftrapid-2")),
        (ServiceQuery(8614, Protocol::TCP), Service::Name("canon-bjnp4")),
        (ServiceQuery(4113, Protocol::UDP), Service::Name("aipn-reg")),
        (ServiceQuery(3862, Protocol::TCP), Service::Name("giga-pocket")),
        (ServiceQuery(373, Protocol::UDP), Service::Name("legent-1")),
        (ServiceQuery(27442, Protocol::TCP), Service::Name("jesmsjc")),
        (ServiceQuery(5155, Protocol::UDP), Service::Name("asctrl-agent")),
        (ServiceQuery(1080, Protocol::TCP), Service::Name("socks")),
        (ServiceQuery(100, Protocol::TCP), Service::Name("newacct")),
        (ServiceQuery(1071, Protocol::UDP), Service::Name("bsquare-voip")),
        (ServiceQuery(3897, Protocol::UDP), Service::Name("sdo-ssh")),
        (ServiceQuery(3532, Protocol::TCP), Service::Name("raven-rmp")),
        (ServiceQuery(8764, Protocol::UDP), Service::Name("openqueue")),
        (ServiceQuery(9092, Protocol::UDP), Service::Name("XmlIpcRegSvc")),
        (ServiceQuery(3261, Protocol::UDP), Service::Name("winshadow")),
        (ServiceQuery(443, Protocol::TCP), Service::Name("https")),
        (ServiceQuery(2599, Protocol::UDP), Service::Name("snapd")),
        (ServiceQuery(7511, Protocol::TCP), Service::Name("pafec-lm")),
        (ServiceQuery(2889, Protocol::UDP), Service::Name("rsom")),
        (ServiceQuery(2526, Protocol::TCP), Service::Name("ema-sent-lm")),
        (ServiceQuery(5051, Protocol::TCP), Service::Name("ida-agent")),
        (ServiceQuery(4667, Protocol::TCP), Service::Name("mmacomm")),
        (ServiceQuery(5907, Protocol::TCP), Service::Name("dsd")),
        (ServiceQuery(6842, Protocol::UDP), Service::Name("netmo-http")),
        (ServiceQuery(8162, Protocol::TCP), Service::Name("lpar2rrd")),
        (ServiceQuery(5264, Protocol::TCP), Service::Name("3com-njack-1")),
        (ServiceQuery(24386, Protocol::UDP), Service::Name("intel_rci")),
        (ServiceQuery(3451, Protocol::UDP), Service::Name("asam")),
        (ServiceQuery(3595, Protocol::UDP), Service::Name("shareapp")),
        (ServiceQuery(2020, Protocol::TCP), Service::Name("xinupageserver")),
        (ServiceQuery(5400, Protocol::UDP), Service::Name("excerpt")),
        (ServiceQuery(2688, Protocol::TCP), Service::Name("md-cg-http")),
        (ServiceQuery(5305, Protocol::UDP), Service::Name("hacl-test")),
        (ServiceQuery(440, Protocol::TCP), Service::Name("sgcp")),
        (ServiceQuery(6443, Protocol::TCP), Service::Name("sun-sr-https")),
        (ServiceQuery(1390, Protocol::UDP), Service::Name("iclpv-sc")),
        (ServiceQuery(2139, Protocol::UDP), Service::Name("ias-auth")),
        (ServiceQuery(1232, Protocol::TCP), Service::Name("first-defense")),
        (ServiceQuery(4453, Protocol::UDP), Service::Name("nssalertmgr")),
        (ServiceQuery(683, Protocol::UDP), Service::Name("corba-iiop")),
        (ServiceQuery(104, Protocol::UDP), Service::Name("acr-nema")),
        (ServiceQuery(1313, Protocol::UDP), Service::Name("bmc_patroldb")),
        (ServiceQuery(45000, Protocol::TCP), Service::Name("asmp")),
        (ServiceQuery(2134, Protocol::TCP), Service::Name("avenue")),
        (ServiceQuery(8054, Protocol::UDP), Service::Name("senomix03")),
        (ServiceQuery(4803, Protocol::UDP), Service::Name("notateit-disc")),
        (ServiceQuery(645, Protocol::TCP), Service::Name("pssc")),
        (ServiceQuery(993, Protocol::TCP), Service::Name("imaps")),
        (ServiceQuery(3434, Protocol::UDP), Service::Name("opencm")),
        (ServiceQuery(3096, Protocol::UDP), Service::Name("ndl-aps")),
        (ServiceQuery(42509, Protocol::UDP), Service::Name("candrp")),
        (ServiceQuery(322, Protocol::TCP), Service::Name("rtsps")),
        (ServiceQuery(402, Protocol::UDP), Service::Name("genie")),
        (ServiceQuery(146, Protocol::UDP), Service::Name("iso-tp0")),
        (ServiceQuery(7313, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(3410, Protocol::UDP), Service::Name("networklenss")),
        (ServiceQuery(8205, Protocol::TCP), Service::Name("lm-instmgr")),
        (ServiceQuery(7322, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(1099, Protocol::UDP), Service::Name("rmiregistry")),
        (ServiceQuery(9087, Protocol::TCP), Service::Name("classic")),
        (ServiceQuery(2950, Protocol::TCP), Service::Name("esip")),
        (ServiceQuery(4249, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(400, Protocol::TCP), Service::Name("work-sol")),
        (ServiceQuery(10200, Protocol::UDP), Service::Name("trisoap")),
        (ServiceQuery(3150, Protocol::UDP), Service::Name("nm-asses-admin")),
        (ServiceQuery(2651, Protocol::UDP), Service::Name("ebinsite")),
        (ServiceQuery(245, Protocol::UDP), Service::Name("link")),
        (ServiceQuery(2751, Protocol::TCP), Service::Name("fjippol-port2")),
        (ServiceQuery(3112, Protocol::TCP), Service::Name("ksysguard")),
        (ServiceQuery(2750, Protocol::UDP), Service::Name("fjippol-port1")),
        (ServiceQuery(2752, Protocol::TCP), Service::Name("rsisysaccess")),
        (ServiceQuery(9536, Protocol::UDP), Service::Name("laes-bf")),
        (ServiceQuery(6123, Protocol::UDP), Service::Name("backup-express")),
        (ServiceQuery(2719, Protocol::TCP), Service::Name("scan-change")),
        (ServiceQuery(13930, Protocol::TCP), Service::Name("medevolve")),
        (ServiceQuery(2110, Protocol::UDP), Service::Name("umsp")),
        (ServiceQuery(2745, Protocol::TCP), Service::Name("urbisnet")),
        (ServiceQuery(35100, Protocol::TCP), Service::Name("axio-disc")),
        (ServiceQuery(2257, Protocol::UDP), Service::Name("simple-tx-rx")),
        (ServiceQuery(648, Protocol::UDP), Service::Name("rrp")),
        (ServiceQuery(4445, Protocol::UDP), Service::Name("upnotifyp")),
        (ServiceQuery(41795, Protocol::TCP), Service::Name("crestron-ctp")),
        (ServiceQuery(6481, Protocol::TCP), Service::Name("servicetags")),
        (ServiceQuery(6316, Protocol::UDP), Service::Name("abb-escp")),
        (ServiceQuery(7345, Protocol::TCP), Service::Name("swx")),
        (ServiceQuery(3175, Protocol::UDP), Service::Name("t1-e1-over-ip")),
        (ServiceQuery(2609, Protocol::UDP), Service::Name("system-monitor")),
        (ServiceQuery(4057, Protocol::TCP), Service::Name("wfm")),
        (ServiceQuery(716, Protocol::TCP), Service::Name("pana")),
        (ServiceQuery(7509, Protocol::TCP), Service::Name("acplt")),
        (ServiceQuery(4161, Protocol::TCP), Service::Name("omscontact")),
        (ServiceQuery(1822, Protocol::TCP), Service::Name("es-elmd")),
        (ServiceQuery(568, Protocol::UDP), Service::Name("ms-shuttle")),
        (ServiceQuery(2980, Protocol::UDP), Service::Name("wimd")),
        (ServiceQuery(2785, Protocol::UDP), Service::Name("aic-np")),
        (ServiceQuery(5991, Protocol::UDP), Service::Name("nuxsl")),
        (ServiceQuery(8041, Protocol::TCP), Service::Name("enguity-xccetp")),
        (ServiceQuery(5300, Protocol::UDP), Service::Name("hacl-hb")),
        (ServiceQuery(2637, Protocol::UDP), Service::Name("imdocsvc")),
        (ServiceQuery(8891, Protocol::UDP), Service::Name("ddi-udp-4")),
        (ServiceQuery(1836, Protocol::UDP), Service::Name("ste-smsc")),
        (ServiceQuery(1444, Protocol::UDP), Service::Name("marcam-lm")),
        (ServiceQuery(2045, Protocol::UDP), Service::Name("cdfunc")),
        (ServiceQuery(1919, Protocol::TCP), Service::Name("can-dch")),
        (ServiceQuery(1867, Protocol::UDP), Service::Name("udrive")),
        (ServiceQuery(459, Protocol::TCP), Service::Name("ampr-rcmd")),
        (ServiceQuery(9007, Protocol::TCP), Service::Name("ogs-client")),
        (ServiceQuery(2370, Protocol::UDP), Service::Name("l3-hbmon")),
        (ServiceQuery(3582, Protocol::TCP), Service::Name("press")),
        (ServiceQuery(3130, Protocol::TCP), Service::Name("icpv2")),
        (ServiceQuery(1718, Protocol::UDP), Service::Name("h225gatedisc")),
        (ServiceQuery(1515, Protocol::TCP), Service::Name("ifor-protocol")),
        (ServiceQuery(3997, Protocol::UDP), Service::Name("remoteanything")),
        (ServiceQuery(4915, Protocol::TCP), Service::Name("frcs")),
        (ServiceQuery(4142, Protocol::UDP), Service::Name("oidocsvc")),
        (ServiceQuery(3483, Protocol::UDP), Service::Name("slim-devices")),
        (ServiceQuery(8023, Protocol::TCP), Service::Name("arca-api")),
        (ServiceQuery(3858, Protocol::UDP), Service::Name("trap-port-mom")),
        (ServiceQuery(3144, Protocol::TCP), Service::Name("tarantella")),
        (ServiceQuery(2002, Protocol::TCP), Service::Name("globe")),
        (ServiceQuery(2308, Protocol::UDP), Service::Name("sdhelp")),
        (ServiceQuery(2592, Protocol::TCP), Service::Name("netrek")),
        (ServiceQuery(3158, Protocol::UDP), Service::Name("stvp")),
        (ServiceQuery(5247, Protocol::UDP), Service::Name("capwap-data")),
        (ServiceQuery(2655, Protocol::UDP), Service::Name("unglue")),
        (ServiceQuery(2301, Protocol::TCP), Service::Name("compaqdiag")),
        (ServiceQuery(5683, Protocol::TCP), Service::Name("coap")),
        (ServiceQuery(5430, Protocol::TCP), Service::Name("radec-corp")),
        (ServiceQuery(3019, Protocol::UDP), Service::Name("resource_mgr")),
        (ServiceQuery(3013, Protocol::TCP), Service::Name("gilatskysurfer")),
        (ServiceQuery(16367, Protocol::TCP), Service::Name("netserialext3")),
        (ServiceQuery(5506, Protocol::UDP), Service::Name("amc")),
        (ServiceQuery(3722, Protocol::TCP), Service::Name("xserveraid")),
        (ServiceQuery(1643, Protocol::TCP), Service::Name("isis-ambc")),
        (ServiceQuery(1863, Protocol::UDP), Service::Name("msnp")),
        (ServiceQuery(1561, Protocol::TCP), Service::Name("facilityview")),
        (ServiceQuery(2805, Protocol::UDP), Service::Name("wta-wsp-s")),
        (ServiceQuery(3070, Protocol::TCP), Service::Name("mgxswitch")),
        (ServiceQuery(4166, Protocol::UDP), Service::Name("joost")),
        (ServiceQuery(1702, Protocol::UDP), Service::Name("deskshare")),
        (ServiceQuery(1212, Protocol::UDP), Service::Name("lupa")),
        (ServiceQuery(2332, Protocol::UDP), Service::Name("rcc-host")),
        (ServiceQuery(6966, Protocol::UDP), Service::Name("swispol")),
        (ServiceQuery(425, Protocol::UDP), Service::Name("icad-el")),
        (ServiceQuery(5798, Protocol::TCP), Service::Name("enlabel-dpl")),
        (ServiceQuery(4163, Protocol::TCP), Service::Name("silverpeakpeer")),
        (ServiceQuery(2710, Protocol::TCP), Service::Name("sso-service")),
        (ServiceQuery(1842, Protocol::UDP), Service::Name("netopia-vo4")),
        (ServiceQuery(1209, Protocol::TCP), Service::Name("ipcd3")),
        (ServiceQuery(3950, Protocol::TCP), Service::Name("namemunge")),
        (ServiceQuery(6112, Protocol::TCP), Service::Name("dtspc")),
        (ServiceQuery(2079, Protocol::TCP), Service::Name("idware-router")),
        (ServiceQuery(3492, Protocol::UDP), Service::Name("tvdumtray-port")),
        (ServiceQuery(1150, Protocol::TCP), Service::Name("blaze")),
        (ServiceQuery(2272, Protocol::TCP), Service::Name("mmcal")),
        (ServiceQuery(2724, Protocol::TCP), Service::Name("qotps")),
        (ServiceQuery(3803, Protocol::UDP), Service::Name("soniqsync")),
        (ServiceQuery(1276, Protocol::UDP), Service::Name("ivmanager")),
        (ServiceQuery(25565, Protocol::TCP), Service::Name("minecraft")),
        (ServiceQuery(6420, Protocol::UDP), Service::Name("nim-vdrshell")),
        (ServiceQuery(1602, Protocol::TCP), Service::Name("inspect")),
        (ServiceQuery(2259, Protocol::TCP), Service::Name("acd-pm")),
        (ServiceQuery(1705, Protocol::UDP), Service::Name("slingshot")),
        (ServiceQuery(317, Protocol::UDP), Service::Name("zannet")),
        (ServiceQuery(1528, Protocol::UDP), Service::Name("mciautoreg")),
        (ServiceQuery(12007, Protocol::TCP), Service::Name("accuracer")),
        (ServiceQuery(1918, Protocol::UDP), Service::Name("can-nds")),
        (ServiceQuery(50002, Protocol::TCP), Service::Name("iiimsf")),
        (ServiceQuery(1690, Protocol::TCP), Service::Name("ng-umds")),
        (ServiceQuery(7070, Protocol::TCP), Service::Name("realserver")),
        (ServiceQuery(4361, Protocol::UDP), Service::Name("nacnl")),
        (ServiceQuery(2156, Protocol::UDP), Service::Name("trp")),
        (ServiceQuery(2217, Protocol::TCP), Service::Name("gotodevice")),
        (ServiceQuery(1523, Protocol::TCP), Service::Name("cichild-lm")),
        (ServiceQuery(3971, Protocol::UDP), Service::Name("lanrevserver")),
        (ServiceQuery(7903, Protocol::TCP), Service::Name("tnos-dps")),
        (ServiceQuery(5689, Protocol::TCP), Service::Name("qmvideo")),
        (ServiceQuery(5717, Protocol::TCP), Service::Name("prosharenotify")),
        (ServiceQuery(1190, Protocol::UDP), Service::Name("commlinx-avl")),
        (ServiceQuery(23333, Protocol::TCP), Service::Name("elxmgmt")),
        (ServiceQuery(1982, Protocol::UDP), Service::Name("estamp")),
        (ServiceQuery(5777, Protocol::UDP), Service::Name("dali-port")),
        (ServiceQuery(2562, Protocol::TCP), Service::Name("delibo")),
        (ServiceQuery(27004, Protocol::TCP), Service::Name("flexlm4")),
        (ServiceQuery(2938, Protocol::TCP), Service::Name("sm-pas-1")),
        (ServiceQuery(2382, Protocol::TCP), Service::Name("ms-olap3")),
        (ServiceQuery(1678, Protocol::UDP), Service::Name("prolink")),
        (ServiceQuery(11319, Protocol::TCP), Service::Name("imip")),
        (ServiceQuery(1105, Protocol::UDP), Service::Name("ftranhc")),
        (ServiceQuery(3345, Protocol::UDP), Service::Name("influence")),
        (ServiceQuery(1308, Protocol::UDP), Service::Name("odsi")),
        (ServiceQuery(8192, Protocol::UDP), Service::Name("sophos")),
        (ServiceQuery(5674, Protocol::TCP), Service::Name("hyperscsi-port")),
        (ServiceQuery(1820, Protocol::UDP), Service::Name("mcagent")),
        (ServiceQuery(2271, Protocol::TCP), Service::Name("mmcals")),
        (ServiceQuery(3794, Protocol::UDP), Service::Name("jaus")),
        (ServiceQuery(2842, Protocol::TCP), Service::Name("l3-hawk")),
        (ServiceQuery(1342, Protocol::TCP), Service::Name("esbroker")),
        (ServiceQuery(3963, Protocol::TCP), Service::Name("thrp")),
        (ServiceQuery(4738, Protocol::UDP), Service::Name("solera-lpn")),
        (ServiceQuery(2792, Protocol::TCP), Service::Name("f5-globalsite")),
        (ServiceQuery(35000, Protocol::TCP), Service::Name("heathview")),
        (ServiceQuery(664, Protocol::UDP), Service::Name("secure-aux-bus")),
        (ServiceQuery(5580, Protocol::TCP), Service::Name("tmosms0")),
        (ServiceQuery(494, Protocol::UDP), Service::Name("pov-ray")),
        (ServiceQuery(2532, Protocol::TCP), Service::Name("ovtopmd")),
        (ServiceQuery(773, Protocol::TCP), Service::Name("submit")),
        (ServiceQuery(3854, Protocol::UDP), Service::Name("stryker-com")),
        (ServiceQuery(2770, Protocol::TCP), Service::Name("veronica")),
        (ServiceQuery(10544, Protocol::UDP), Service::Name("MOS-soap-opt")),
        (ServiceQuery(3491, Protocol::TCP), Service::Name("swr-port")),
        (ServiceQuery(569, Protocol::TCP), Service::Name("ms-rome")),
        (ServiceQuery(3498, Protocol::TCP), Service::Name("dashpas-port")),
        (ServiceQuery(5237, Protocol::TCP), Service::Name("mnet-discovery")),
        (ServiceQuery(1514, Protocol::UDP), Service::Name("fujitsu-dtcns")),
        (ServiceQuery(6072, Protocol::UDP), Service::Name("diagnose-proc")),
        (ServiceQuery(1068, Protocol::TCP), Service::Name("instl_bootc")),
        (ServiceQuery(3749, Protocol::TCP), Service::Name("cimtrak")),
        (ServiceQuery(781, Protocol::UDP), Service::Name("hp-collector")),
        (ServiceQuery(19315, Protocol::TCP), Service::Name("keyshadow")),
        (ServiceQuery(4185, Protocol::TCP), Service::Name("wcpp")),
        (ServiceQuery(2853, Protocol::TCP), Service::Name("ispipes")),
        (ServiceQuery(1462, Protocol::TCP), Service::Name("world-lm")),
        (ServiceQuery(30004, Protocol::TCP), Service::Name("amicon-fpsu-s")),
        (ServiceQuery(3152, Protocol::TCP), Service::Name("feitianrockey")),
        (ServiceQuery(1481, Protocol::TCP), Service::Name("airs")),
        (ServiceQuery(2904, Protocol::TCP), Service::Name("m2ua")),
        (ServiceQuery(3520, Protocol::UDP), Service::Name("galileolog")),
        (ServiceQuery(4151, Protocol::TCP), Service::Name("menandmice_noh")),
        (ServiceQuery(1920, Protocol::UDP), Service::Name("can-ferret")),
        (ServiceQuery(5510, Protocol::TCP), Service::Name("secureidprop")),
        (ServiceQuery(36865, Protocol::UDP), Service::Name("kastenxpipe")),
        (ServiceQuery(3733, Protocol::TCP), Service::Name("multip-msg")),
        (ServiceQuery(557, Protocol::TCP), Service::Name("openvms-sysipc")),
        (ServiceQuery(2898, Protocol::UDP), Service::Name("appliance-cfg")),
        (ServiceQuery(3230, Protocol::TCP), Service::Name("sftdst-port")),
        (ServiceQuery(3645, Protocol::TCP), Service::Name("cyc")),
        (ServiceQuery(3763, Protocol::TCP), Service::Name("xo-wave")),
        (ServiceQuery(3399, Protocol::TCP), Service::Name("sapeps")),
        (ServiceQuery(12001, Protocol::UDP), Service::Name("entextnetwk")),
        (ServiceQuery(2673, Protocol::UDP), Service::Name("firstcall42")),
        (ServiceQuery(2927, Protocol::UDP), Service::Name("unimobilectrl")),
        (ServiceQuery(10000, Protocol::UDP), Service::Name("ndmp")),
        (ServiceQuery(1377, Protocol::UDP), Service::Name("cichlid")),
        (ServiceQuery(3615, Protocol::TCP), Service::Name("start-network")),
        (ServiceQuery(1458, Protocol::UDP), Service::Name("nrcabq-lm")),
        (ServiceQuery(9011, Protocol::TCP), Service::Name("d-star")),
        (ServiceQuery(1958, Protocol::UDP), Service::Name("dxadmind")),
        (ServiceQuery(565, Protocol::UDP), Service::Name("whoami")),
        (ServiceQuery(2828, Protocol::TCP), Service::Name("itm-lm")),
        (ServiceQuery(3868, Protocol::TCP), Service::Name("diameter")),
        (ServiceQuery(6320, Protocol::TCP), Service::Name("repsvc")),
        (ServiceQuery(6084, Protocol::TCP), Service::Name("p2p-sip")),
        (ServiceQuery(17185, Protocol::UDP), Service::Name("wdbrpc")),
        (ServiceQuery(771, Protocol::TCP), Service::Name("rtip")),
        (ServiceQuery(1649, Protocol::TCP), Service::Name("kermit")),
        (ServiceQuery(6402, Protocol::TCP), Service::Name("boe-eventsrv")),
        (ServiceQuery(44442, Protocol::TCP), Service::Name("coldfusion-auth")),
        (ServiceQuery(1924, Protocol::TCP), Service::Name("xiip")),
        (ServiceQuery(2077, Protocol::UDP), Service::Name("tsrmagt")),
        (ServiceQuery(4257, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(1707, Protocol::TCP), Service::Name("vdmplay")),
        (ServiceQuery(36524, Protocol::TCP), Service::Name("febooti-aw")),
        (ServiceQuery(1274, Protocol::TCP), Service::Name("t1distproc")),
        (ServiceQuery(6030, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(5303, Protocol::TCP), Service::Name("hacl-probe")),
        (ServiceQuery(17777, Protocol::TCP), Service::Name("sw-orion")),
        (ServiceQuery(20002, Protocol::UDP), Service::Name("commtact-http")),
        (ServiceQuery(4729, Protocol::TCP), Service::Name("gsmtap")),
        (ServiceQuery(214, Protocol::TCP), Service::Name("vmpwscs")),
        (ServiceQuery(2777, Protocol::UDP), Service::Name("ridgeway2")),
        (ServiceQuery(3205, Protocol::UDP), Service::Name("isns")),
        (ServiceQuery(38412, Protocol::TCP), Service::Name("ng-control")),
        (ServiceQuery(2624, Protocol::TCP), Service::Name("aria")),
        (ServiceQuery(606, Protocol::TCP), Service::Name("urm")),
        (ServiceQuery(3098, Protocol::UDP), Service::Name("umm-port")),
        (ServiceQuery(6017, Protocol::TCP), Service::Name("xmail-ctrl")),
        (ServiceQuery(13, Protocol::UDP), Service::Name("daytime")),
        (ServiceQuery(5905, Protocol::TCP), Service::Name("asmgcs")),
        (ServiceQuery(17225, Protocol::TCP), Service::Name("trdp-md")),
        (ServiceQuery(5349, Protocol::UDP), Service::Name("stuns")),
        (ServiceQuery(975, Protocol::TCP), Service::Name("securenetpro-sensor")),
        (ServiceQuery(1640, Protocol::UDP), Service::Name("cert-responder")),
        (ServiceQuery(633, Protocol::TCP), Service::Name("servstat")),
        (ServiceQuery(4028, Protocol::TCP), Service::Name("dtserver-port")),
        (ServiceQuery(2986, Protocol::TCP), Service::Name("stonefalls")),
        (ServiceQuery(5161, Protocol::TCP), Service::Name("snmpssh")),
        (ServiceQuery(169, Protocol::TCP), Service::Name("send")),
        (ServiceQuery(2404, Protocol::TCP), Service::Name("iec-104")),
        (ServiceQuery(3406, Protocol::UDP), Service::Name("nokia-ann-ch2")),
        (ServiceQuery(3147, Protocol::TCP), Service::Name("rfio")),
        (ServiceQuery(2187, Protocol::UDP), Service::Name("ssmd")),
        (ServiceQuery(38865, Protocol::TCP), Service::Name("secrmmsafecopya")),
        (ServiceQuery(1975, Protocol::TCP), Service::Name("tcoflashagent")),
        (ServiceQuery(1855, Protocol::UDP), Service::Name("fiorano-rtrsvc")),
        (ServiceQuery(96, Protocol::UDP), Service::Name("dixie")),
        (ServiceQuery(3917, Protocol::TCP), Service::Name("aftmux")),
        (ServiceQuery(7391, Protocol::TCP), Service::Name("mindfilesys")),
        (ServiceQuery(7720, Protocol::UDP), Service::Name("medimageportal")),
        (ServiceQuery(4393, Protocol::TCP), Service::Name("apwi-rxspooler")),
        (ServiceQuery(9002, Protocol::UDP), Service::Name("dynamid")),
        (ServiceQuery(2128, Protocol::TCP), Service::Name("net-steward")),
        (ServiceQuery(13894, Protocol::TCP), Service::Name("ucontrol")),
        (ServiceQuery(3324, Protocol::UDP), Service::Name("active-net")),
        (ServiceQuery(7168, Protocol::TCP), Service::Name("cnckadserver")),
        (ServiceQuery(3196, Protocol::UDP), Service::Name("ncu-2")),
        (ServiceQuery(3514, Protocol::UDP), Service::Name("must-p2p")),
        (ServiceQuery(1421, Protocol::UDP), Service::Name("gandalf-lm")),
        (ServiceQuery(1817, Protocol::TCP), Service::Name("rkb-oscs")),
        (ServiceQuery(2101, Protocol::TCP), Service::Name("rtcm-sc104")),
        (ServiceQuery(1678, Protocol::TCP), Service::Name("prolink")),
        (ServiceQuery(8750, Protocol::TCP), Service::Name("dey-keyneg")),
        (ServiceQuery(748, Protocol::TCP), Service::Name("ris-cm")),
        (ServiceQuery(48619, Protocol::UDP), Service::Name("iqobject")),
        (ServiceQuery(12001, Protocol::TCP), Service::Name("entextnetwk")),
        (ServiceQuery(6670, Protocol::UDP), Service::Name("vocaltec-gold")),
        (ServiceQuery(3209, Protocol::TCP), Service::Name("netwkpathengine")),
        (ServiceQuery(1371, Protocol::UDP), Service::Name("fc-cli")),
        (ServiceQuery(9208, Protocol::UDP), Service::Name("rjcdb-vcards")),
        (ServiceQuery(2922, Protocol::UDP), Service::Name("cesdcdtrn")),
        (ServiceQuery(9618, Protocol::TCP), Service::Name("condor")),
        (ServiceQuery(3941, Protocol::UDP), Service::Name("homeportal-web")),
        (ServiceQuery(2311, Protocol::UDP), Service::Name("messageservice")),
        (ServiceQuery(1124, Protocol::UDP), Service::Name("hpvmmcontrol")),
        (ServiceQuery(3128, Protocol::UDP), Service::Name("ndl-aas")),
        (ServiceQuery(1911, Protocol::UDP), Service::Name("mtp")),
        (ServiceQuery(1204, Protocol::TCP), Service::Name("ssslog-mgr")),
        (ServiceQuery(2215, Protocol::UDP), Service::Name("ipcore")),
        (ServiceQuery(1266, Protocol::TCP), Service::Name("dellpwrappks")),
        (ServiceQuery(4379, Protocol::TCP), Service::Name("ctdb")),
        (ServiceQuery(10115, Protocol::UDP), Service::Name("netiq-endpt")),
        (ServiceQuery(3561, Protocol::TCP), Service::Name("bmc-onekey")),
        (ServiceQuery(1036, Protocol::UDP), Service::Name("nsstp")),
        (ServiceQuery(1884, Protocol::TCP), Service::Name("idmaps")),
        (ServiceQuery(7165, Protocol::TCP), Service::Name("doc-server")),
        (ServiceQuery(2962, Protocol::UDP), Service::Name("iph-policy-cli")),
        (ServiceQuery(16993, Protocol::UDP), Service::Name("amt-soap-https")),
        (ServiceQuery(586, Protocol::TCP), Service::Name("password-chg")),
        (ServiceQuery(3260, Protocol::TCP), Service::Name("iscsi")),
        (ServiceQuery(3764, Protocol::TCP), Service::Name("mni-prot-rout")),
        (ServiceQuery(8400, Protocol::TCP), Service::Name("cvd")),
        (ServiceQuery(3788, Protocol::TCP), Service::Name("isrp-port")),
        (ServiceQuery(6868, Protocol::TCP), Service::Name("acctopus-cc")),
        (ServiceQuery(2577, Protocol::UDP), Service::Name("scipticslsrvr")),
        (ServiceQuery(7701, Protocol::TCP), Service::Name("nfapi")),
        (ServiceQuery(2057, Protocol::UDP), Service::Name("rich-cp")),
        (ServiceQuery(5504, Protocol::UDP), Service::Name("fcp-cics-gw1")),
        (ServiceQuery(7697, Protocol::UDP), Service::Name("klio")),
        (ServiceQuery(5048, Protocol::TCP), Service::Name("texai")),
        (ServiceQuery(1995, Protocol::TCP), Service::Name("perf-port")),
        (ServiceQuery(2814, Protocol::UDP), Service::Name("llm-csv")),
        (ServiceQuery(1180, Protocol::UDP), Service::Name("mc-client")),
        (ServiceQuery(4130, Protocol::TCP), Service::Name("fronet")),
        (ServiceQuery(2614, Protocol::UDP), Service::Name("neveroffline")),
        (ServiceQuery(7787, Protocol::UDP), Service::Name("popup-reminders")),
        (ServiceQuery(2123, Protocol::UDP), Service::Name("gtp-control")),
        (ServiceQuery(1864, Protocol::UDP), Service::Name("paradym-31")),
        (ServiceQuery(6108, Protocol::UDP), Service::Name("sercomm-scadmin")),
        (ServiceQuery(2231, Protocol::UDP), Service::Name("wimaxasncp")),
        (ServiceQuery(1807, Protocol::UDP), Service::Name("fhsp")),
        (ServiceQuery(3631, Protocol::UDP), Service::Name("cs-services")),
        (ServiceQuery(2602, Protocol::UDP), Service::Name("discp-server")),
        (ServiceQuery(3846, Protocol::TCP), Service::Name("an-pcp")),
        (ServiceQuery(6556, Protocol::TCP), Service::Name("checkmk-agent")),
        (ServiceQuery(4556, Protocol::UDP), Service::Name("dtn-bundle-udp")),
        (ServiceQuery(1660, Protocol::UDP), Service::Name("skip-mc-gikreq")),
        (ServiceQuery(7201, Protocol::UDP), Service::Name("dlip")),
        (ServiceQuery(3299, Protocol::UDP), Service::Name("pdrncs")),
        (ServiceQuery(6100, Protocol::UDP), Service::Name("synchronet-db")),
        (ServiceQuery(149, Protocol::TCP), Service::Name("aed-512")),
        (ServiceQuery(4674, Protocol::UDP), Service::Name("appiq-mgmt")),
        (ServiceQuery(4334, Protocol::TCP), Service::Name("netconf-ch-ssh")),
        (ServiceQuery(2708, Protocol::TCP), Service::Name("banyan-net")),
        (ServiceQuery(223, Protocol::TCP), Service::Name("cdc")),
        (ServiceQuery(3327, Protocol::UDP), Service::Name("bbars")),
        (ServiceQuery(1680, Protocol::UDP), Service::Name("microcom-sbp")),
        (ServiceQuery(7163, Protocol::TCP), Service::Name("cacsambroker")),
        (ServiceQuery(5000, Protocol::TCP), Service::Name("upnp")),
        (ServiceQuery(18, Protocol::TCP), Service::Name("msp")),
        (ServiceQuery(2165, Protocol::UDP), Service::Name("x-bone-api")),
        (ServiceQuery(128, Protocol::TCP), Service::Name("gss-xlicen")),
        (ServiceQuery(27345, Protocol::TCP), Service::Name("imagepump")),
        (ServiceQuery(560, Protocol::UDP), Service::Name("rmonitor")),
        (ServiceQuery(2205, Protocol::TCP), Service::Name("jps")),
        (ServiceQuery(4294, Protocol::UDP), Service::Name("vrml-multi-use")),
        (ServiceQuery(5310, Protocol::UDP), Service::Name("outlaws")),
        (ServiceQuery(537, Protocol::UDP), Service::Name("nmsp")),
        (ServiceQuery(1603, Protocol::UDP), Service::Name("picodbc")),
        (ServiceQuery(88, Protocol::UDP), Service::Name("kerberos-sec")),
        (ServiceQuery(3250, Protocol::UDP), Service::Name("hicp")),
        (ServiceQuery(28240, Protocol::TCP), Service::Name("siemensgsm")),
        (ServiceQuery(164, Protocol::TCP), Service::Name("cmip-agent")),
        (ServiceQuery(7340, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(2963, Protocol::TCP), Service::Name("iph-policy-adm")),
        (ServiceQuery(6018, Protocol::TCP), Service::Name("x11")),
        (ServiceQuery(34962, Protocol::TCP), Service::Name("profinet-rt")),
        (ServiceQuery(7725, Protocol::UDP), Service::Name("nitrogen")),
        (ServiceQuery(2616, Protocol::TCP), Service::Name("appswitch-emp")),
        (ServiceQuery(10500, Protocol::UDP), Service::Name("hip-nat-t")),
        (ServiceQuery(4602, Protocol::TCP), Service::Name("mtsserver")),
        (ServiceQuery(1917, Protocol::UDP), Service::Name("noagent")),
        (ServiceQuery(9443, Protocol::TCP), Service::Name("tungsten-https")),
        (ServiceQuery(9213, Protocol::UDP), Service::Name("serverstart")),
        (ServiceQuery(716, Protocol::UDP), Service::Name("pana")),
        (ServiceQuery(1808, Protocol::UDP), Service::Name("oracle-vp2")),
        (ServiceQuery(1362, Protocol::UDP), Service::Name("timeflies")),
        (ServiceQuery(3697, Protocol::TCP), Service::Name("nw-license")),
        (ServiceQuery(9992, Protocol::UDP), Service::Name("palace-1")),
        (ServiceQuery(3703, Protocol::TCP), Service::Name("adobeserver-3")),
        (ServiceQuery(8912, Protocol::TCP), Service::Name("wcbackup")),
        (ServiceQuery(3310, Protocol::TCP), Service::Name("dyna-access")),
        (ServiceQuery(17755, Protocol::TCP), Service::Name("zigbee-ip")),
        (ServiceQuery(5312, Protocol::TCP), Service::Name("permabit-cs")),
        (ServiceQuery(4117, Protocol::UDP), Service::Name("hillrserv")),
        (ServiceQuery(10252, Protocol::UDP), Service::Name("apollo-relay")),
        (ServiceQuery(7354, Protocol::UDP), Service::Name("swx")),
        (ServiceQuery(4342, Protocol::TCP), Service::Name("lisp-cons")),
        (ServiceQuery(4005, Protocol::UDP), Service::Name("pxc-pin")),
        (ServiceQuery(3895, Protocol::TCP), Service::Name("syam-smc")),
        (ServiceQuery(3559, Protocol::UDP), Service::Name("cctv-port")),
        (ServiceQuery(194, Protocol::UDP), Service::Name("irc")),
        (ServiceQuery(3307, Protocol::TCP), Service::Name("opsession-prxy")),
        (ServiceQuery(6124, Protocol::TCP), Service::Name("pnbs")),
        (ServiceQuery(2455, Protocol::TCP), Service::Name("wago-io-system")),
        (ServiceQuery(1958, Protocol::TCP), Service::Name("dxadmind")),
        (ServiceQuery(6104, Protocol::TCP), Service::Name("dbdb")),
        (ServiceQuery(5410, Protocol::UDP), Service::Name("salient-usrmgr")),
        (ServiceQuery(3233, Protocol::TCP), Service::Name("whisker")),
        (ServiceQuery(4998, Protocol::TCP), Service::Name("maybe-veritas")),
        (ServiceQuery(1337, Protocol::TCP), Service::Name("waste")),
        (ServiceQuery(4448, Protocol::TCP), Service::Name("asc-slmd")),
        (ServiceQuery(7443, Protocol::TCP), Service::Name("oracleas-https")),
        (ServiceQuery(1685, Protocol::TCP), Service::Name("n2nremote")),
        (ServiceQuery(3767, Protocol::UDP), Service::Name("listmgr-port")),
        (ServiceQuery(10810, Protocol::UDP), Service::Name("nmc-disc")),
        (ServiceQuery(9997, Protocol::UDP), Service::Name("palace-6")),
        (ServiceQuery(2373, Protocol::TCP), Service::Name("remographlm")),
        (ServiceQuery(3290, Protocol::TCP), Service::Name("caps-lm")),
        (ServiceQuery(7872, Protocol::TCP), Service::Name("mipv6tls")),
        (ServiceQuery(7689, Protocol::UDP), Service::Name("collaber")),
        (ServiceQuery(7548, Protocol::UDP), Service::Name("tidp")),
        (ServiceQuery(1793, Protocol::UDP), Service::Name("rsc-robot")),
        (ServiceQuery(33000, Protocol::TCP), Service::Name("wg-endpt-comms")),
        (ServiceQuery(2294, Protocol::TCP), Service::Name("konshus-lm")),
        (ServiceQuery(13819, Protocol::UDP), Service::Name("dsmcc-session")),
        (ServiceQuery(4136, Protocol::TCP), Service::Name("cl-db-request")),
        (ServiceQuery(7397, Protocol::TCP), Service::Name("hexarc")),
        (ServiceQuery(8555, Protocol::TCP), Service::Name("d-fence")),
    ],
};
